@@ -1,4 +1,4 @@
-/* Copyright (c) 2021, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2021, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,10 +33,16 @@ namespace keyring_common::config {
 Config_reader::Config_reader(std::string config_file_path)
     : config_file_path_(std::move(config_file_path)), valid_(false) {
   std::ifstream file_stream(config_file_path_);
-  if (!file_stream.is_open()) return;
+  if (!file_stream.is_open()) {
+    err_ = "cannot read config file " + config_file_path_;
+    return;
+  }
   rapidjson::IStreamWrapper json_fstream_reader(file_stream);
-  valid_ = !data_.ParseStream(json_fstream_reader).HasParseError();
-  file_stream.close();
+  if (data_.ParseStream(json_fstream_reader).HasParseError()) {
+    err_ = "config file " + config_file_path_ + " has not valid format";
+    return;
+  }
+  valid_ = true;
 }
 
 }  // namespace keyring_common::config
