@@ -1,4 +1,4 @@
-/* Copyright (c) 2021, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2021, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -29,7 +29,9 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <components/keyrings/common/component_helpers/include/service_requirements.h>
 #include <components/keyrings/common/operations/operations.h>
 #include <mysql/components/services/log_builtins.h> /* LogComponentErr */
-#include "mysqld_error.h"                           /* Errors */
+#include "mysql/components/component_implementation.h"
+#include "mysql/components/services/registry.h"
+#include "mysqld_error.h" /* Errors */
 
 #include "backend/backend.h"
 #include "config/config.h"
@@ -55,3 +57,6 @@ bool init_or_reinit_keyring(std::string &err);
 bool set_paths(const char *component_path, const char *instance_path);
 
 }  // namespace keyring_file
+
+extern REQUIRES_SERVICE_PLACEHOLDER(registry);
+extern REQUIRES_SERVICE_PLACEHOLDER(registry_registration);
\ No newline at end of file
