@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -21,16 +21,10 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+#include <mysql/components/services/dynamic_loader_service_notification.h>
 #include "component_common.h"
-#include "mysql_service_implementation.h"
 #include "registry_imp.h"
 
-#include <mysql/components/my_service.h>
-#include <mysql/components/service_implementation.h>
-#include <mysql/components/services/registry.h>
-#include <map>
-#include <memory>
-
 // pfs instrumentation headers
 #include <mysql/components/services/mysql_rwlock.h>
 
@@ -46,15 +40,36 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
   For each Service a default Service Implementation is distinguished, the first
   one to be added, but it is possible to change it at will at any point.
 
+  @section registry_no_lock_deprecated Registry _no_lock deprecation note
+
+  Dynamic Loader unload operation order is altered and does not require _no_lock
+  variant of registry API. For more information on dynamic loader unload
+  operation see @ref dynamic_loader_unload_order.
+
+  The following registry API functions are depreceated:
+
+    - mysql_registry_no_lock_imp::acquire()
+    - mysql_registry_no_lock_imp::acquire_related()
+    - mysql_registry_no_lock_imp::release()
+    - mysql_registry_no_lock_imp::register_service()
+    - mysql_registry_no_lock_imp::unregister()
+    - mysql_registry_no_lock_imp::set_default()
+
+  It is recommended to use synchronized variant of registry API instead:
+
+    - mysql_registry_imp::acquire()
+    - mysql_registry_imp::acquire_related()
+    - mysql_registry_imp::release()
+    - mysql_registry_imp::register_service()
+    - mysql_registry_imp::unregister()
+    - mysql_registry_imp::set_default()
+
   A convenient RAII-style wrapper on Service acquisition and automatic release
   is provided, the my_service<T> class.
 */
 
 static PSI_rwlock_key key_rwlock_LOCK_registry;
 
-typedef std::map<const char *, mysql_service_implementation *, c_string_less>
-    my_service_registry;
-
 struct my_h_service_iterator_imp {
   my_service_registry::const_iterator m_it;
   minimal_chassis::rwlock_scoped_lock m_lock;
@@ -69,7 +84,7 @@ static PSI_rwlock_info all_registry_rwlocks[] = {
     {&::key_rwlock_LOCK_registry, "LOCK_registry", PSI_FLAG_SINGLETON, 0,
      PSI_DOCUMENT_ME}};
 
-static void init_registry_psi_keys(void) {
+static void init_registry_psi_keys() {
   const char *category = "components";
   int count;
 
@@ -86,14 +101,13 @@ void mysql_registry_imp::init() {
   mysql_rwlock_init(key_rwlock_LOCK_registry,
                     &mysql_registry_imp::LOCK_registry);
 }
+
 /**
   De-initializes registry. De-initializes RW lock, all other structures
   are cleaned up.
 */
 void mysql_registry_imp::deinit() {
-  mysql_registry_imp::service_registry.clear();
-  mysql_registry_imp::interface_mapping.clear();
-
+  mysql_registry_no_lock_imp::deinit();
   mysql_rwlock_destroy(&mysql_registry_imp::LOCK_registry);
 }
 
@@ -111,240 +125,7 @@ void mysql_registry_imp::rw_lock_deinit() {
 */
 minimal_chassis::rwlock_scoped_lock
 mysql_registry_imp::lock_registry_for_write() {
-  return minimal_chassis::rwlock_scoped_lock(&LOCK_registry, true, __FILE__,
-                                             __LINE__);
-}
-
-/**
-  Finds a Service Implementation data structure based on the pointer to
-  interface struct supplied. Assumes caller has at least a read lock on the
-  Registry.
-
-  @param interface A pointer to the interface structure of the Service
-    Implementation to look for.
-  @return A pointer to respective Service Implementation data structure, or
-    NULL if no such interface pointer is registered within the Registry.
-*/
-mysql_service_implementation *
-mysql_registry_imp::get_service_implementation_by_interface(
-    my_h_service interface) {
-  my_interface_mapping::const_iterator iter =
-      mysql_registry_imp::interface_mapping.find(interface);
-  if (iter == mysql_registry_imp::interface_mapping.cend()) {
-    return nullptr;
-  }
-
-  return iter->second;
-}
-
-/**
-  Gets current reference count for a Service Implementation related to the
-  specified pointer to the interface structure. Assumes caller has at least
-  a read lock on the Registry.
-
-  @param interface A pointer to the interface structure of the Service
-    Implementation to get reference count of.
-  @return A current reference count for specified Service Implementation.
-    Returns 0 in case there is no such interface or it is not referenced.
-*/
-uint64_t mysql_registry_imp::get_service_implementation_reference_count(
-    my_h_service interface) {
-  my_interface_mapping::const_iterator iter =
-      mysql_registry_imp::interface_mapping.find(interface);
-  if (iter == mysql_registry_imp::interface_mapping.cend()) {
-    return -1;
-  }
-
-  return iter->second->get_reference_count();
-}
-
-/**
-  Finds and acquires a Service by name. A name of the Service or the Service
-  Implementation can be specified. In case of the Service name, the default
-  Service Implementation for Service specified will be returned. Assumes
-  caller has at least a read lock on the Registry.
-
-  @param service_name Name of Service or Service Implementation to acquire.
-  @param [out] out_service Pointer to Service handle to set acquired Service.
-  @return Status of performed operation
-  @retval false success
-  @retval true failure
-*/
-bool mysql_registry_imp::acquire_nolock(const char *service_name,
-                                        my_h_service *out_service) {
-  try {
-    if (out_service == nullptr) {
-      return true;
-    }
-    my_service_registry::const_iterator iter;
-
-    iter = mysql_registry_imp::service_registry.find(service_name);
-
-    if (iter == mysql_registry_imp::service_registry.cend()) return true;
-
-    mysql_service_implementation *imp = iter->second;
-    imp->add_reference();
-    *out_service = imp->interface();
-    return false;
-  } catch (...) {
-  }
-  return true;
-}
-
-/**
-  Releases the Service Implementation previously acquired. After the call to
-  this method the usage of the Service Implementation handle will lead to
-  unpredicted results. Assumes caller has at least a read lock on the
-  Registry.
-
-  @param service Service Implementation handle of already acquired Service.
-  @return Status of performed operation
-  @retval false success
-  @retval true failure
-*/
-bool mysql_registry_imp::release_nolock(my_h_service service) {
-  try {
-    if (service == nullptr) {
-      return true;
-    }
-
-    mysql_service_implementation *service_implementation =
-        mysql_registry_imp::get_service_implementation_by_interface(service);
-    if (service_implementation == nullptr) {
-      return true;
-    }
-    return service_implementation->release_reference();
-  } catch (...) {
-  }
-  return true;
-}
-
-/**
-  Registers a new Service Implementation. If it is the first Service
-  Implementation for the specified Service then it is made a default one.
-  Assumes caller has a write lock on the Registry.
-
-  @param service_implementation_name Name of the Service Implementation to
-    register.
-  @param ptr Pointer to the Service Implementation structure.
-  @return Status of performed operation
-  @retval false success
-  @retval true failure
-*/
-bool mysql_registry_imp::register_service_nolock(
-    const char *service_implementation_name, my_h_service ptr) {
-  try {
-    std::unique_ptr<mysql_service_implementation> imp =
-        std::unique_ptr<mysql_service_implementation>(
-            new mysql_service_implementation(ptr, service_implementation_name));
-
-    if (imp->interface() == nullptr) {
-      return true;
-    }
-
-    /* Register the implementation name. */
-    std::pair<my_service_registry::iterator, bool> addition_result =
-        mysql_registry_imp::service_registry.emplace(imp->name_c_str(),
-                                                     imp.get());
-
-    /* Fail if it was present already. */
-    if (!addition_result.second) {
-      return true;
-    } else {
-      try {
-        /* Register interface in mapping */
-        mysql_registry_imp::interface_mapping.emplace(imp->interface(),
-                                                      imp.get());
-
-        /* Register the Service Implementation as default for Service name in
-          case none were registered before. */
-        mysql_registry_imp::service_registry.emplace_hint(
-            addition_result.first, imp->service_name_c_str(), imp.get());
-      } catch (...) {
-        mysql_registry_imp::service_registry.erase(addition_result.first);
-        /* unique_ptr still has ownership over implementation object, we
-          don't have to delete it explicitly. */
-        return true;
-      }
-    }
-
-    /* Pointer is stored in registry, thous we release ownership. */
-    imp.release();
-
-    return false;
-  } catch (...) {
-  }
-  return true;
-}
-
-/**
-  Removes previously registered Service Implementation from registry. If it is
-  the default one for specified Service then any one still registered is made
-  default. If there is no other, the default entry is removed from the
-  Registry too. Assumes caller has a write lock on the Registry.
-
-  @param service_implementation_name Name of the Service Implementation to
-    unregister.
-  @return Status of performed operation
-  @retval false success
-  @retval true Failure. May happen when Service is still being referenced.
-*/
-bool mysql_registry_imp::unregister_nolock(
-    const char *service_implementation_name) {
-  try {
-    std::unique_ptr<mysql_service_implementation> imp;
-
-    {
-      /* Find the implementation and check if it is not being referenced. */
-      my_service_registry::iterator imp_iter =
-          mysql_registry_imp::service_registry.find(
-              service_implementation_name);
-      if (imp_iter == mysql_registry_imp::service_registry.end() ||
-          imp_iter->second->get_reference_count() > 0) {
-        return true;
-      }
-
-      /* First remove specified implementation, to not include it in search
-        for new default one. Take ownership on implementation object. */
-      imp.reset(imp_iter->second);
-      mysql_registry_imp::service_registry.erase(imp_iter);
-      /* After deletion, implementation iterator is not valid, we go out of
-        scope to prevent it from being reused. */
-    }
-
-    /* Remove interface mapping. */
-    mysql_registry_imp::interface_mapping.erase(
-        mysql_registry_imp::interface_mapping.find(imp->interface()));
-
-    /* Look if it is the default implementation. */
-    my_service_registry::iterator default_iter =
-        mysql_registry_imp::service_registry.find(imp->service_name_c_str());
-    if (default_iter == mysql_registry_imp::service_registry.end()) {
-      /* A Service Implementation and no default present. The state is not
-        consistent. */
-      return true;
-    }
-
-    if (default_iter->second == imp.get()) {
-      /* Remove the default implementation too. */
-      my_service_registry::iterator new_default_iter =
-          mysql_registry_imp::service_registry.erase(default_iter);
-
-      /* Search for a new default implementation. */
-      if (new_default_iter != mysql_registry_imp::service_registry.end() &&
-          !strcmp(imp->service_name_c_str(),
-                  new_default_iter->second->service_name_c_str())) {
-        /* Set as default implementation. */
-        mysql_service_implementation *new_default = new_default_iter->second;
-        mysql_registry_imp::service_registry.emplace_hint(
-            new_default_iter, new_default->service_name_c_str(), new_default);
-      }
-    }
-
-    return false;
-  } catch (...) {
-  }
-  return true;
+  return {&LOCK_registry, true, __FILE__, __LINE__};
 }
 
 /**
@@ -360,10 +141,10 @@ bool mysql_registry_imp::unregister_nolock(
 */
 DEFINE_BOOL_METHOD(mysql_registry_imp::acquire,
                    (const char *service_name, my_h_service *out_service)) {
-  minimal_chassis::rwlock_scoped_lock lock(&LOCK_registry, false, __FILE__,
-                                           __LINE__);
+  minimal_chassis::rwlock_scoped_lock const lock(&LOCK_registry, false,
+                                                 __FILE__, __LINE__);
 
-  return mysql_registry_imp::acquire_nolock(service_name, out_service);
+  return mysql_registry_no_lock_imp::acquire(service_name, out_service);
 }
 
 /**
@@ -382,37 +163,11 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::acquire,
 DEFINE_BOOL_METHOD(mysql_registry_imp::acquire_related,
                    (const char *service_name, my_h_service service,
                     my_h_service *out_service)) {
-  try {
-    minimal_chassis::rwlock_scoped_lock lock(&mysql_registry_imp::LOCK_registry,
-                                             false, __FILE__, __LINE__);
+  minimal_chassis::rwlock_scoped_lock const lock(
+      &mysql_registry_imp::LOCK_registry, false, __FILE__, __LINE__);
 
-    mysql_service_implementation *service_implementation =
-        mysql_registry_imp::get_service_implementation_by_interface(service);
-    if (service_implementation == nullptr) {
-      return true;
-    }
-    /* Find dot, the component name is right after the dot. */
-    const char *component_part =
-        strchr(service_implementation->name_c_str(), '.');
-    if (component_part == nullptr) {
-      return true;
-    }
-    /* Assure given service_name is not fully qualified. */
-    if (strchr(service_name, '.') != nullptr) {
-      return true;
-    }
-    my_string service_implementation_name =
-        my_string(service_name) + component_part;
-    /* Try to acquire such Service. */
-    if (mysql_registry_imp::acquire_nolock(service_implementation_name.c_str(),
-                                           out_service)) {
-      /* service is not found */
-      return true;
-    }
-    return false;
-  } catch (...) {
-  }
-  return true;
+  return mysql_registry_no_lock_imp::acquire_related(service_name, service,
+                                                     out_service);
 }
 
 /**
@@ -426,51 +181,67 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::acquire_related,
   @retval true failure
 */
 DEFINE_BOOL_METHOD(mysql_registry_imp::release, (my_h_service service)) {
-  minimal_chassis::rwlock_scoped_lock lock(&mysql_registry_imp::LOCK_registry,
-                                           false, __FILE__, __LINE__);
+  minimal_chassis::rwlock_scoped_lock const lock(
+      &mysql_registry_imp::LOCK_registry, false, __FILE__, __LINE__);
 
-  return mysql_registry_imp::release_nolock(service);
+  return mysql_registry_no_lock_imp::release(service);
 }
 
-/**
-  Registers a new Service Implementation. If it is the first Service
-  Implementation for the specified Service then it is made a default one.
+bool mysql_registry_imp::register_service_sans_notify(
+    const char *service_implementation_name, my_h_service ptr) {
+  minimal_chassis::rwlock_scoped_lock const lock(
+      &mysql_registry_imp::LOCK_registry, true, __FILE__, __LINE__);
 
-  @param service_implementation_name Name of the Service Implementation to
-    register.
-  @param ptr Pointer to the Service Implementation structure.
-  @return Status of performed operation
-  @retval false success
-  @retval true failure
-*/
-DEFINE_BOOL_METHOD(mysql_registry_imp::register_service,
-                   (const char *service_implementation_name,
-                    my_h_service ptr)) {
-  minimal_chassis::rwlock_scoped_lock lock(&mysql_registry_imp::LOCK_registry,
-                                           true, __FILE__, __LINE__);
-
-  return mysql_registry_imp::register_service_nolock(
+  return mysql_registry_no_lock_imp::register_service(
       service_implementation_name, ptr);
 }
 
-/**
-  Removes previously registered Service Implementation from registry. If it is
-  the default one for specified Service then any one still registered is made
-  default. If there is no other, the default entry is removed from the
-  Registry too.
+DEFINE_BOOL_METHOD(mysql_registry_imp::register_service,
+                   (const char *service_implementation_name,
+                    my_h_service ptr)) {
+  bool ret = mysql_registry_imp::register_service_sans_notify(
+      service_implementation_name, ptr);
+
+  if (!ret) {
+    /* notify about the new service, if registered */
+    my_h_service hsvc = nullptr;
+    if (0 == mysql_registry_imp::acquire(
+                 "dynamic_loader_services_loaded_notification", &hsvc) &&
+        hsvc != nullptr) {
+      reinterpret_cast<SERVICE_TYPE(
+          dynamic_loader_services_loaded_notification) *>(hsvc)
+          ->notify(&service_implementation_name, 1);
+      mysql_registry_imp::release(hsvc);
+    }
+  }
+  return ret;
+}
+
+bool mysql_registry_imp::unregister_sans_notify(
+    const char *service_implementation_name) {
+  minimal_chassis::rwlock_scoped_lock const lock(
+      &mysql_registry_imp::LOCK_registry, true, __FILE__, __LINE__);
+
+  return mysql_registry_no_lock_imp::unregister(service_implementation_name);
+}
 
-  @param service_implementation_name Name of the Service Implementation to
-    unregister.
-  @return Status of performed operation
-  @retval false success
-  @retval true Failure. May happen when Service is still being referenced.
-*/
 DEFINE_BOOL_METHOD(mysql_registry_imp::unregister,
                    (const char *service_implementation_name)) {
-  minimal_chassis::rwlock_scoped_lock lock(&mysql_registry_imp::LOCK_registry,
-                                           true, __FILE__, __LINE__);
+  /* notify about the service being removed */
+  {
+    my_h_service hsvc = nullptr;
+    if (0 == mysql_registry_imp::acquire(
+                 "dynamic_loader_services_unload_notification", &hsvc) &&
+        hsvc != nullptr) {
+      reinterpret_cast<SERVICE_TYPE(
+          dynamic_loader_services_unload_notification) *>(hsvc)
+          ->notify(&service_implementation_name, 1);
+      mysql_registry_imp::release(hsvc);
+    }
+  }
 
-  return mysql_registry_imp::unregister_nolock(service_implementation_name);
+  return mysql_registry_imp::unregister_sans_notify(
+      service_implementation_name);
 }
 
 /**
@@ -484,31 +255,10 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::unregister,
 */
 DEFINE_BOOL_METHOD(mysql_registry_imp::set_default,
                    (const char *service_implementation_name)) {
-  try {
-    my_service_registry::const_iterator iter;
+  minimal_chassis::rwlock_scoped_lock const lock(
+      &mysql_registry_imp::LOCK_registry, true, __FILE__, __LINE__);
 
-    minimal_chassis::rwlock_scoped_lock lock(&mysql_registry_imp::LOCK_registry,
-                                             true, __FILE__, __LINE__);
-
-    /* register the implementation name */
-    iter =
-        mysql_registry_imp::service_registry.find(service_implementation_name);
-
-    if (iter == mysql_registry_imp::service_registry.cend()) {
-      return true;
-    }
-    mysql_service_implementation *imp = iter->second;
-    /* We have to remove and reinsert value as key, the string pointer will
-      not be valid if we unregister previous default implementation. */
-    iter = mysql_registry_imp::service_registry.erase(
-        mysql_registry_imp::service_registry.find(imp->service_name_c_str()));
-    mysql_registry_imp::service_registry.emplace_hint(
-        iter, imp->service_name_c_str(), imp);
-
-    return false;
-  } catch (...) {
-  }
-  return true;
+  return mysql_registry_no_lock_imp::set_default(service_implementation_name);
 }
 
 /**
@@ -541,14 +291,28 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::iterator_create,
     minimal_chassis::rwlock_scoped_lock lock(&mysql_registry_imp::LOCK_registry,
                                              false, __FILE__, __LINE__);
 
-    my_service_registry::const_iterator r =
-        (!service_name_pattern || !*service_name_pattern)
-            ? mysql_registry_imp::service_registry.cbegin()
-            : mysql_registry_imp::service_registry.find(service_name_pattern);
-    if (r == mysql_registry_imp::service_registry.cend()) {
+    auto r = (!service_name_pattern || !*service_name_pattern)
+                 ? mysql_registry_no_lock_imp::service_registry.cbegin()
+                 : mysql_registry_no_lock_imp::service_registry.find(
+                       service_name_pattern);
+    if (r == mysql_registry_no_lock_imp::service_registry.cend()) {
       return true;
     }
 
+    if (service_name_pattern && *service_name_pattern) {
+      /*
+        If the query was of the format <service_name> or
+        <service_name>.<implementation_name> then make sure that there exists at
+        least one service matching the criteria. If not, there is no point
+        returning the iterator because it would invariably point to a service
+        other than what's caller is looking for.
+      */
+      if (strncmp(r->first, service_name_pattern,
+                  std::min(strlen(r->first), strlen(service_name_pattern))) !=
+          0)
+        return true;
+    }
+
     *out_iterator = new my_h_service_iterator_imp{r, std::move(lock)};
     return false;
   } catch (...) {
@@ -564,8 +328,7 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::iterator_create,
 DEFINE_METHOD(void, mysql_registry_imp::iterator_release,
               (my_h_service_iterator iterator)) {
   try {
-    my_h_service_iterator_imp *iter =
-        reinterpret_cast<my_h_service_iterator_imp *>(iterator);
+    auto *iter = reinterpret_cast<my_h_service_iterator_imp *>(iterator);
 
     if (!iter) return;
 
@@ -592,10 +355,10 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::iterator_get,
 
     if (!iterator) return true;
 
-    my_service_registry::const_iterator &iter =
+    my_service_registry::const_iterator const &iter =
         reinterpret_cast<my_h_service_iterator_imp *>(iterator)->m_it;
 
-    if (iter != mysql_registry_imp::service_registry.cend()) {
+    if (iter != mysql_registry_no_lock_imp::service_registry.cend()) {
       *out_name = iter->second->name_c_str();
 
       return false;
@@ -623,9 +386,9 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::iterator_next,
     my_service_registry::const_iterator &iter =
         reinterpret_cast<my_h_service_iterator_imp *>(iterator)->m_it;
 
-    if (iter != mysql_registry_imp::service_registry.cend()) {
+    if (iter != mysql_registry_no_lock_imp::service_registry.cend()) {
       ++iter;
-      return iter == mysql_registry_imp::service_registry.cend();
+      return iter == mysql_registry_no_lock_imp::service_registry.cend();
     }
   } catch (...) {
   }
@@ -646,10 +409,10 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::iterator_is_valid,
   try {
     if (!iterator) return true;
 
-    my_service_registry::const_iterator &iter =
+    my_service_registry::const_iterator const &iter =
         reinterpret_cast<my_h_service_iterator_imp *>(iterator)->m_it;
 
-    return iter == mysql_registry_imp::service_registry.cend();
+    return iter == mysql_registry_no_lock_imp::service_registry.cend();
   } catch (...) {
   }
   return true;
@@ -661,7 +424,7 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::iterator_is_valid,
   types, but also static members with different name, so usage of templates
   is not enough to reuse that part of code. */
 #define REGISTRY_IMP mysql_registry_imp
-#define REGISTRY mysql_registry_imp::service_registry
+#define REGISTRY mysql_registry_no_lock_imp::service_registry
 #define REGISTRY_TYPE my_service_registry
 #define LOCK mysql_registry_imp::LOCK_registry
 #define ITERATOR_TYPE my_h_service_iterator_imp
@@ -672,6 +435,4 @@ DEFINE_BOOL_METHOD(mysql_registry_imp::iterator_is_valid,
 #include "registry_metadata.cc.inc"
 
 /* static members for mysql_registry_imp */
-my_service_registry mysql_registry_imp::service_registry;
-mysql_registry_imp::my_interface_mapping mysql_registry_imp::interface_mapping;
 mysql_rwlock_t mysql_registry_imp::LOCK_registry;
