@@ -1,4 +1,4 @@
-/* Copyright (c) 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2024, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -22,14 +22,18 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <fcntl.h>
+#include <cstdio>
+#ifndef _WIN32
+#include <unistd.h>
+#endif
+
 #include <mysql/components/component_implementation.h>
 #include <mysql/components/service_implementation.h>
+#include <mysql/components/services/bits/system_variables_bits.h>
 #include <mysql/components/services/component_sys_var_service.h>
 #include <mysql/components/services/mysql_current_thread_reader.h>
 #include <mysql/components/services/mysql_system_variable.h>
-#include <mysql/plugin.h>
 
-#include "my_macros.h"
 #include "nulls.h"
 #include "template_utils.h"
 #include "typelib.h"
