@@ -54,6 +54,7 @@
 
 #include <atomic>  // error_handler_hook
 #include <cstring>
+#include <string>
 
 #include "my_compiler.h"
 #include "my_compress.h"
@@ -64,7 +65,9 @@
 #include "my_psi_config.h" /* IWYU pragma: keep */
 
 #include "my_sharedlib.h"
+// IWYU pragma: begin_exports
 #include "mysql/components/services/bits/my_io_bits.h"
+#include "mysql/components/services/bits/my_syslog_bits.h"
 #include "mysql/components/services/bits/mysql_cond_bits.h"
 #include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/psi_bits.h"
@@ -72,7 +75,8 @@
 #include "mysql/components/services/bits/psi_memory_bits.h"
 #include "mysql/components/services/bits/psi_metric_bits.h"
 #include "mysql/components/services/bits/psi_stage_bits.h"
-#include "sql/stream_cipher.h"
+#include "mysql/components/services/bits/server_telemetry_logs_client_bits.h"
+// IWYU pragma: end_exports
 #include "string_with_len.h"
 
 class MY_CHARSET_LOADER;
@@ -85,6 +89,7 @@ struct PSI_data_lock_bootstrap;
 struct PSI_error_bootstrap;
 struct PSI_file_bootstrap;
 struct PSI_idle_bootstrap;
+struct PSI_logs_client_bootstrap;
 struct PSI_mdl_bootstrap;
 struct PSI_memory_bootstrap;
 struct PSI_metric_bootstrap;
@@ -206,14 +211,8 @@ extern DebugSyncCallbackFp debug_sync_C_callback_ptr;
     if (debug_sync_C_callback_ptr != NULL)                              \
       (*debug_sync_C_callback_ptr)(STRING_WITH_LEN(_sync_point_name_)); \
   } while (0)
-#define DEBUG_SYNC_C_IF_THD(thd, _sync_point_name_)                     \
-  do {                                                                  \
-    if (debug_sync_C_callback_ptr != NULL && thd)                       \
-      (*debug_sync_C_callback_ptr)(STRING_WITH_LEN(_sync_point_name_)); \
-  } while (0)
 #else
 #define DEBUG_SYNC_C(_sync_point_name_)
-#define DEBUG_SYNC_C_IF_THD(thd, _sync_point_name_)
 #endif /* defined(ENABLED_DEBUG_SYNC) */
 
 #ifdef HAVE_LINUX_LARGE_PAGES
@@ -306,14 +305,6 @@ enum flush_type {
   FLUSH_FORCE_WRITE
 };
 
-struct DYNAMIC_ARRAY {
-  uchar *buffer{nullptr};
-  uint elements{0}, max_element{0};
-  uint alloc_increment{0};
-  uint size_of_element{0};
-  PSI_memory_key m_psi_key{PSI_NOT_INSTRUMENTED};
-};
-
 struct MY_TMPDIR {
   char **list{nullptr};
   uint cur{0}, max{0};
@@ -343,6 +334,7 @@ struct IO_CACHE_SHARE {
   int error;           /* Last error. */
 };
 
+class Stream_cipher;
 struct IO_CACHE /* Used when caching files */
 {
   /* Offset in file corresponding to the first byte of uchar* buffer. */
@@ -570,6 +562,10 @@ struct USED_MEM {
 
 /* Prototypes for mysys and my_func functions */
 
+extern size_t escape_string_for_mysql(const CHARSET_INFO *charset_info,
+                                      char *to, size_t to_length,
+                                      const char *from, size_t length);
+
 extern int my_copy(const char *from, const char *to, myf MyFlags);
 extern int my_delete(const char *name, myf MyFlags);
 extern int my_getwd(char *buf, size_t size, myf MyFlags);
@@ -586,6 +582,9 @@ extern int my_close(File fd, myf MyFlags);
 extern int my_mkdir(const char *dir, int Flags, myf MyFlags);
 extern int my_readlink(char *to, const char *filename, myf MyFlags);
 extern int my_is_symlink(const char *filename, ST_FILE_ID *file_id);
+extern bool my_rm_dir_w_symlink(const char *directory_path, bool send_error,
+                                bool send_intermediate_errors,
+                                bool &directory_deletion_failed);
 extern int my_realpath(char *to, const char *filename, myf MyFlags);
 extern int my_is_same_file(File file, const ST_FILE_ID *file_id);
 extern File my_create_with_symlink(const char *linkname, const char *filename,
@@ -618,9 +617,6 @@ extern my_off_t my_ftell(FILE *stream);
 // Maximum size of message  that will be logged.
 #define MAX_SYSLOG_MESSAGE_SIZE 1024
 
-/* Platform-independent SysLog support */
-enum my_syslog_options { MY_SYSLOG_PIDS = 1 };
-
 extern int my_openlog(const char *eventSourceName, int option, int facility);
 extern int my_closelog();
 extern int my_syslog(const CHARSET_INFO *cs, enum loglevel level,
@@ -712,7 +708,9 @@ extern void free_tmpdir(MY_TMPDIR *tmpdir);
 
 extern size_t dirname_part(char *to, const char *name, size_t *to_res_length);
 extern size_t dirname_length(const char *name);
-#define base_name(A) (A + dirname_length(A))
+ALWAYS_INLINE const char *base_name(const char *A) {
+  return A + dirname_length(A);
+}
 extern int test_if_hard_path(const char *dir_name);
 extern bool has_path(const char *name);
 extern char *convert_dirname(char *to, const char *from, const char *from_end);
@@ -789,20 +787,6 @@ const size_t MY_MAX_TEMP_FILENAME_LEN = 19;
 File create_temp_file(char *to, const char *dir, const char *pfx, int mode,
                       UnlinkOrKeepFile unlink_or_keep, myf MyFlags);
 
-// Use Prealloced_array or std::vector or something similar in C++
-extern bool my_init_dynamic_array(DYNAMIC_ARRAY *array, PSI_memory_key key,
-                                  uint element_size, void *init_buffer,
-                                  uint init_alloc, uint alloc_increment);
-/* init_dynamic_array() function is deprecated */
-
-#define dynamic_element(array, array_index, type) \
-  ((type)((array)->buffer) + (array_index))
-
-/* Some functions are still in use in C++, because HASH uses DYNAMIC_ARRAY */
-extern bool insert_dynamic(DYNAMIC_ARRAY *array, const void *element);
-extern void *alloc_dynamic(DYNAMIC_ARRAY *array);
-extern void delete_dynamic(DYNAMIC_ARRAY *array);
-
 extern bool init_dynamic_string(DYNAMIC_STRING *str, const char *init_str,
                                 size_t init_alloc);
 extern bool dynstr_append(DYNAMIC_STRING *str, const char *append);
@@ -887,7 +871,7 @@ extern uint get_collation_number(const char *name);
 extern const char *get_collation_name(uint charset_number);
 
 extern CHARSET_INFO *get_charset(uint cs_number, myf flags);
-extern CHARSET_INFO *get_charset_by_name(const char *cs_name, myf flags);
+extern CHARSET_INFO *get_charset_by_name(const char *collation_name, myf flags);
 extern CHARSET_INFO *my_collation_get_by_name(const char *collation_name,
                                               myf flags, MY_CHARSET_ERRMSG *);
 extern CHARSET_INFO *get_charset_by_csname(const char *cs_name, uint cs_flags,
@@ -901,18 +885,11 @@ extern bool resolve_collation(const char *cl_name,
                               const CHARSET_INFO **cl);
 extern char *get_charsets_dir(char *buf);
 
-extern size_t escape_string_for_mysql(const CHARSET_INFO *charset_info,
-                                      char *to, size_t to_length,
-                                      const char *from, size_t length);
 extern void charset_uninit();
 #ifdef _WIN32
 /* File system character set */
 extern CHARSET_INFO *fs_character_set(void);
-#endif
-extern size_t escape_quotes_for_mysql(CHARSET_INFO *charset_info, char *to,
-                                      size_t to_length, const char *from,
-                                      size_t length, char quote);
-#ifdef _WIN32
+
 extern bool have_tcpip; /* Is set if tcpip is used */
 
 /* implemented in my_windac.c */
@@ -956,6 +933,8 @@ extern void set_psi_memory_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_mutex_bootstrap *psi_mutex_hook;
 extern void set_psi_metric_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_metric_bootstrap *psi_metric_hook;
+extern void set_psi_logs_client_service(void *psi);
+extern MYSQL_PLUGIN_IMPORT PSI_logs_client_bootstrap *psi_logs_client_hook;
 extern void set_psi_mutex_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_rwlock_bootstrap *psi_rwlock_hook;
 extern void set_psi_rwlock_service(void *psi);
@@ -977,12 +956,9 @@ extern MYSQL_PLUGIN_IMPORT PSI_tls_channel_bootstrap *psi_tls_channel_hook;
 extern void set_psi_tls_channel_service(void *psi);
 #endif /* HAVE_PSI_INTERFACE */
 
-/* Total physical memory available */
-[[nodiscard]] extern unsigned long long my_physical_memory();
-
 /* Compares versions and determine if clone is allowed */
-[[nodiscard]] extern bool are_versions_clone_compatible(std::string ver1,
-                                                        std::string ver2);
+[[nodiscard]] extern bool are_versions_clone_compatible(
+    const std::string &ver1, const std::string &ver2);
 
 /**
   @} (end of group MYSYS)
