@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,8 +36,6 @@
 #include <time.h>
 
 #include "keycache.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "my_base.h"
 #include "my_check_opt.h"
 #include "my_checksum.h"  // ha_checksum
@@ -45,6 +43,7 @@
 #include "my_inttypes.h"
 #include "my_io.h"
 #include "my_macros.h"
+#include "mysql/strings/m_ctype.h"
 
 /*
   Limit max keys according to HA_MAX_POSSIBLE_KEY
