@@ -1,4 +1,4 @@
-/* Copyright (c) 2022, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2022, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,6 +31,7 @@
 #include <assert.h>
 #include <mysql/components/service.h>
 #include <stddef.h>
+#include <cstdint>
 #include <cstring>
 #include <functional>
 #include <iomanip>
@@ -60,6 +61,7 @@ struct Bulk_load_error_location_details {
   std::string m_error_mesg{};
   std::string m_table_name{};
   size_t m_bytes;
+  size_t m_column_length;
 
   std::ostream &print(std::ostream &out) const;
 };
@@ -88,6 +90,13 @@ struct Column_text {
   /** Column data length. */
   size_t m_data_len{};
 
+  /** Check if it is DB_ROW_ID column based on the value it contains.
+  @return true if it is DB_ROW_ID column, false otherwise */
+  bool is_row_id() const { return m_row_id != UINT64_MAX; }
+
+  /** The generated DB_ROW_ID value */
+  uint64_t m_row_id{UINT64_MAX};
+
   /** Mark the column to be null, by setting length to a special value. This is
   only used for columns whose state is maintained across chunks
   (aka fragmented columns). */
@@ -134,6 +143,7 @@ struct Column_text {
     m_data_ptr = nullptr;
     m_data_len = 0;
     m_is_ext = false;
+    m_row_id = UINT64_MAX;
   }
 
   /** Print this object into the given output stream.
@@ -190,6 +200,8 @@ inline std::ostream &operator<<(std::ostream &out, const Column_text &obj) {
   return obj.print(out);
 }
 
+struct Row_meta;
+
 struct Column_mysql {
   /** Column Data Type */
   int16_t m_type{};
@@ -200,18 +212,56 @@ struct Column_mysql {
   /** If column is NULL. */
   bool m_is_null{false};
 
-  /** Column data */
-  char *m_data_ptr{};
+  char *get_data() const { return m_is_null ? nullptr : m_data_ptr; }
+
+  void set_data(char *ptr) { m_data_ptr = ptr; }
+
+  /** Save the beginning of the row pointer in this object.  This should be
+  called only when the column is null.
+  @param[in]  row_begin  pointer to beginning of row.*/
+  void row(char *row_begin) {
+    assert(m_is_null);
+    m_data_len = 0;
+    m_data_ptr = row_begin;
+  }
+
+  /** Get the pointer to the beginning of row. This is valid only if the
+  column is null. This should be called on the first column of the row. There
+  is no need to call this on other columns.
+  @param[in]  row_meta meta data information about the row
+  @param[in]  col_index  Index of the first column which is 0.
+  @return pointer to row beginning. */
+  char *get_row_begin(const Row_meta &row_meta,
+                      size_t col_index [[maybe_unused]]) const;
 
   /** Column data in integer format. Used only for specific datatype. */
   uint64_t m_int_data;
 
+  void init() {
+    m_type = 0;
+    m_data_len = 0;
+    m_is_null = false;
+    m_data_ptr = nullptr;
+    m_int_data = 0;
+  }
+
   std::string to_string() const;
+
+ private:
+  /** Column data or row begin.  There is a need to fetch the beginning of
+  the row from the vector of Column_mysql.  But in the case of secondary
+  indexes, all the keys could be null and it becomes impossible to obtain
+  the pointer to beginning of the row.  To solve this problem, I am re-using
+  this pointer to hold the row begin when the column is null.  So it becomes
+  important to make use of m_is_null to check if the column is null. It is NOT
+  correct to check this pointer against nullptr to confirm if column is null.*/
+  char *m_data_ptr{nullptr};
 };
 
 inline std::string Column_mysql::to_string() const {
   std::ostringstream sout;
-  sout << "[Column_mysql: len=" << m_data_len;
+  sout << "[Column_mysql: type=" << m_type << ", len=" << m_data_len
+       << ", m_int_data=" << m_int_data;
   sout << ", val=";
 
   switch (m_type) {
@@ -286,6 +336,12 @@ class Row_bunch {
     return true;
   }
 
+  void reset() {
+    for (auto &col : m_columns) {
+      col.init();
+    }
+  }
+
   /** Get current row offset to access columns.
   @param[in]  row_index  row index
   @return row offset in column vector. */
@@ -396,6 +452,21 @@ struct Column_meta {
     MYSQL
   };
 
+  std::string get_compare_string() const {
+    switch (m_compare) {
+      case Compare::INTEGER_SIGNED:
+        return "INTEGER_SIGNED";
+      case Compare::INTEGER_UNSIGNED:
+        return "INTEGER_UNSIGNED";
+      case Compare::BINARY:
+        return "BINARY";
+      case Compare::MYSQL:
+        return "MYSQL";
+    }
+    assert(0);
+    return "INVALID";
+  }
+
   /** @return true if integer type. */
   bool is_integer() const {
     return (m_compare == Compare::INTEGER_SIGNED ||
@@ -407,36 +478,42 @@ struct Column_meta {
   @return false if the column data cannot be stored externally */
   bool can_be_stored_externally() const;
 
+  /** true if this column is part of secondary index. */
+  bool m_is_part_of_sk{false};
+
   /** Field type. (@ref enum_field_types) */
   enum_field_types m_type;
 
   /** If column could be NULL. */
-  bool m_is_nullable;
+  bool m_is_nullable{false};
 
-  /** If column is part of primary key. */
-  bool m_is_key;
+  /** true if column belongs to primary index (key or non-key) */
+  bool m_is_pk{false};
+
+  /** true if column is a key for primary or secondary index. */
+  bool m_is_key{false};
 
   /** If the key is descending. */
-  bool m_is_desc_key;
+  bool m_is_desc_key{false};
 
   /** If the key is prefix of the column. */
-  bool m_is_prefix_key;
+  bool m_is_prefix_key{false};
 
   /** If it is fixed length type. */
-  bool m_is_fixed_len;
+  bool m_is_fixed_len{false};
 
   /** If it is integer type. */
   Compare m_compare;
 
   /** If it is unsigned integer type. */
-  bool m_is_unsigned;
+  bool m_is_unsigned{false};
 
   /** Check the row header to find out if it is fixed length. For
   character data type the row header indicates fixed length. */
-  bool m_fixed_len_if_set_in_row;
+  bool m_fixed_len_if_set_in_row{false};
 
   /** If character column length can be kept in one byte. */
-  bool m_is_single_byte_len;
+  bool m_is_single_byte_len{false};
 
   /** The length of column data if fixed. */
   uint16_t m_fixed_len;
@@ -447,6 +524,9 @@ struct Column_meta {
   /** Index of column in row. */
   uint16_t m_index;
 
+  /** Position of column in table. Refer to Field::field_index() */
+  uint16_t m_field_index;
+
   /** Byte index in NULL bitmap. */
   uint16_t m_null_byte;
 
@@ -459,18 +539,105 @@ struct Column_meta {
   /** Field name */
   std::string m_field_name;
 
+  /** Get a string representation of Column_meta object. Useful only for
+  debugging purposes.
+  @see Column_meta
+  @return string representation of this object. */
+  std::string to_string() const;
+
   /** Print this object into the given output stream.
   @param[in]  out  output stream into which object will be printed
   @return given output stream. */
   std::ostream &print(std::ostream &out) const;
+
+  /** Get the data type of the column as a string.
+  @return data type of the column as a string. */
+  std::string get_type_string() const;
 };
 
+inline std::string Column_meta::get_type_string() const {
+  switch (m_type) {
+    case MYSQL_TYPE_DECIMAL:
+      return "decimal";
+    case MYSQL_TYPE_TINY:
+      return "tiny";
+    case MYSQL_TYPE_SHORT:
+      return "short";
+    case MYSQL_TYPE_LONG:
+      return "long";
+    case MYSQL_TYPE_FLOAT:
+      return "float";
+    case MYSQL_TYPE_DOUBLE:
+      return "double";
+    case MYSQL_TYPE_NULL:
+      return "null";
+    case MYSQL_TYPE_TIMESTAMP:
+      return "timestamp";
+    case MYSQL_TYPE_LONGLONG:
+      return "longlong";
+    case MYSQL_TYPE_INT24:
+      return "int";
+    case MYSQL_TYPE_DATE:
+      return "date";
+    case MYSQL_TYPE_TIME:
+      return "time";
+    case MYSQL_TYPE_DATETIME:
+      return "datetime";
+    case MYSQL_TYPE_YEAR:
+      return "year";
+    case MYSQL_TYPE_NEWDATE:
+      return "date";
+    case MYSQL_TYPE_VARCHAR:
+      return "varchar";
+    case MYSQL_TYPE_BIT:
+      return "bit";
+    case MYSQL_TYPE_TIMESTAMP2:
+      return "timestamp";
+    case MYSQL_TYPE_DATETIME2:
+      return "datetime";
+    case MYSQL_TYPE_TIME2:
+      return "time";
+    case MYSQL_TYPE_TYPED_ARRAY:
+      return "typed_array";
+    case MYSQL_TYPE_VECTOR:
+      return "vector";
+    case MYSQL_TYPE_INVALID:
+      return "invalid";
+    case MYSQL_TYPE_BOOL:
+      return "bool";
+    case MYSQL_TYPE_JSON:
+      return "json";
+    case MYSQL_TYPE_NEWDECIMAL:
+      return "decimal";
+    case MYSQL_TYPE_ENUM:
+      return "enum";
+    case MYSQL_TYPE_SET:
+      return "set";
+    case MYSQL_TYPE_TINY_BLOB:
+      return "tiny_blob";
+    case MYSQL_TYPE_MEDIUM_BLOB:
+      return "medium_blob";
+    case MYSQL_TYPE_LONG_BLOB:
+      return "long_blob";
+    case MYSQL_TYPE_BLOB:
+      return "blob";
+    case MYSQL_TYPE_VAR_STRING:
+      return "var_string";
+    case MYSQL_TYPE_STRING:
+      return "string";
+    case MYSQL_TYPE_GEOMETRY:
+      return "geometry";
+  }
+  return "invalid";
+}
+
 inline bool Column_meta::can_be_stored_externally() const {
   switch (m_type) {
     case MYSQL_TYPE_JSON:
     case MYSQL_TYPE_GEOMETRY:
     case MYSQL_TYPE_VARCHAR:
     case MYSQL_TYPE_TINY_BLOB:
+    case MYSQL_TYPE_VECTOR:
     case MYSQL_TYPE_BLOB:
     case MYSQL_TYPE_MEDIUM_BLOB:
     case MYSQL_TYPE_LONG_BLOB: {
@@ -482,10 +649,22 @@ inline bool Column_meta::can_be_stored_externally() const {
   return false;
 }
 
-inline std::ostream &Column_meta::print(std::ostream &out) const {
-  out << "[Column_meta: m_is_single_byte_len=" << m_is_single_byte_len
+inline std::string Column_meta::to_string() const {
+  std::ostringstream out;
+  out << "[Column_meta: m_type=" << get_type_string()
+      << ", m_field_name=" << m_field_name << ", m_index=" << m_index
+      << ", m_field_index=" << m_field_index
+      << ", m_is_single_byte_len=" << m_is_single_byte_len
       << ", m_is_fixed_len=" << m_is_fixed_len
-      << ", m_fixed_len=" << m_fixed_len << "]";
+      << ", m_fixed_len=" << m_fixed_len << ", m_null_byte=" << m_null_byte
+      << ", m_null_bit=" << m_null_bit << ", m_compare=" << get_compare_string()
+      << ", m_is_desc_key=" << m_is_desc_key << ", m_is_key=" << m_is_key
+      << ", m_is_prefix_key=" << m_is_prefix_key << "]";
+  return out.str();
+}
+
+inline std::ostream &Column_meta::print(std::ostream &out) const {
+  out << to_string();
   return out;
 }
 
@@ -498,6 +677,21 @@ inline std::ostream &operator<<(std::ostream &out, const Column_meta &obj) {
   return obj.print(out);
 }
 
+/** Table metadata. */
+struct Table_meta {
+  /** Number of keys/indexes the table has. */
+  size_t m_n_keys;
+
+  /** Key number of the primary key. */
+  size_t m_keynr_pk;
+
+  /** True if generated DB_ROW_ID is the pk. */
+  bool dbrowid_is_pk{false};
+
+  /** Table being bulk loaded. */
+  std::string m_table_name;
+};
+
 /** Row metadata */
 struct Row_meta {
   /** Key type for fast comparison. */
@@ -515,6 +709,19 @@ struct Row_meta {
   /** All columns in a row arranged as per col_index. */
   std::vector<const Column_meta *> m_columns_text_order;
 
+  /** Get a string representation of this Row_meta object.
+  @see Row_meta
+  @return string representation of this object. */
+  std::string to_string() const;
+
+  /** Get the metadata of the given column.
+  @param[in] col_index  position of the column in the index.
+  @return metadata of the requested column. */
+  const Column_meta &get_column_meta_index_order(size_t col_index) const {
+    assert(col_index < m_columns.size());
+    return m_columns[col_index];
+  }
+
   /** Get the meta data of the column.
   @param[in]  col_index  the index of the column as it appears in CSV file.
   @return a reference to the column meta data.*/
@@ -556,8 +763,55 @@ struct Row_meta {
 
   /** Number of columns that can be stored externally. */
   size_t m_n_blob_cols{0};
+
+  /** Name of the key */
+  std::string m_name;
+
+  /** true if primary key, false if secondary key. */
+  bool is_pk{false};
+
+  /** true if DB_ROW_ID is the pk, false otherwise. */
+  bool dbrowid_is_pk{false};
 };
 
+inline std::ostream &operator<<(std::ostream &os,
+                                const Row_meta::Key_type &key_type) {
+  switch (key_type) {
+    case Row_meta::Key_type::ANY:
+      os << "ANY";
+      break;
+    case Row_meta::Key_type::INT_SIGNED_ASC:
+      os << "INT_SIGNED_ASC";
+      break;
+    case Row_meta::Key_type::INT:
+      os << "INT";
+      break;
+  }
+  return os;
+}
+
+inline std::string Row_meta::to_string() const {
+  std::ostringstream out;
+  out << "[Row_meta: m_name=" << m_name << ", m_num_columns=" << m_num_columns
+      << ", m_keys=" << m_keys << ", m_non_keys=" << m_non_keys
+      << ", m_key_length=" << m_key_length << ", m_key_type=" << m_key_type
+      << ", m_approx_row_len=" << m_approx_row_len;
+  for (auto &col_meta : m_columns) {
+    out << col_meta.to_string() << ", ";
+  }
+  out << "]";
+  return out.str();
+}
+
+inline char *Column_mysql::get_row_begin(const Row_meta &row_meta,
+                                         size_t col_index
+                                         [[maybe_unused]]) const {
+  assert(m_is_null || col_index == 0);
+  return m_is_null ? m_data_ptr
+                   : (m_data_ptr - row_meta.m_first_key_len -
+                      row_meta.m_header_length);
+}
+
 namespace Bulk_load {
 
 class Json_serialization_error_handler final
@@ -673,15 +927,23 @@ DECLARE_METHOD(int, compare_keys,
                (const Column_mysql &key1, const Column_mysql &key2,
                 const Column_meta &col_meta));
 
-/** Get Table row metadata.
+/** Get row metadata information for all the indexes.
 @param[in,out]  thd       session THD
 @param[in]      table     MySQL TABLE
 @param[in]      have_key  include Primary Key metadata
-@param[out]     metadata  Metadata
+@param[out]     metadata  Metadata for each of the indexes.
 @return true if successful. */
-DECLARE_METHOD(bool, get_row_metadata,
+DECLARE_METHOD(bool, get_row_metadata_all,
                (THD * thd, const TABLE *table, bool have_key,
-                Row_meta &metadata));
+                std::vector<Row_meta> &metadata));
+
+/** Get table metadata information for the table being bulk loaded.
+@param[in,out]  thd       session THD
+@param[in]      table     MySQL TABLE
+@param[out]     metadata  Metadata of the table.
+@return true if successful. */
+DECLARE_METHOD(bool, get_table_metadata,
+               (THD * thd, const TABLE *table, Table_meta &metadata));
 
 END_SERVICE_DEFINITION(bulk_data_convert)
 
@@ -691,13 +953,14 @@ BEGIN_SERVICE_DEFINITION(bulk_data_load)
 /** Begin Loading bulk data to SE.
 @param[in,out]  thd          session THD
 @param[in]      table        MySQL TABLE
+@param[in]      keynr        key number, identifying the index being loaded.
 @param[in]      data_size    total data size to load
 @param[in]      memory       SE memory to be used
 @param[in]      num_threads  Number of concurrent threads
 @return SE bulk load context or nullptr in case of an error. */
 DECLARE_METHOD(void *, begin,
-               (THD * thd, const TABLE *table, size_t data_size, size_t memory,
-                size_t num_threads));
+               (THD * thd, const TABLE *table, size_t keynr, size_t data_size,
+                size_t memory, size_t num_threads));
 
 /** Load a set of rows to SE table by one thread.
 @param[in,out]  thd    session THD
