@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -48,6 +49,37 @@
 #include "sql/log.h"
 #endif
 
+/**
+  typedef for log-processing functions ("buffer this event",
+  "process this event", etc.)
+*/
+typedef bool (*log_line_processor)(log_line *ll);
+
+/**
+  Set the log-event processor.
+
+  When a log-event is submitted, a function is applied to that event.
+  That function usually either buffers the event for later processing,
+  or filters and logs the event.
+
+  That function can be set here.
+
+  @param llp   A log-processor
+*/
+extern void log_line_process_hook_set(log_line_processor llp);
+
+/**
+  Get current log-event processor.
+
+  When a log-event is submitted, a function is applied to that event.
+  That function usually either buffers the event for later processing,
+  or filters and logs the event.
+  log_line_process_hook_get() returns a pointer to that function.
+
+  @retval      a pointer to a log-event processing function
+*/
+log_line_processor log_line_process_hook_get(void);
+
 /**
   Primitives for services to interact with the structured logger:
   functions pertaining to log_line and log_item data
@@ -211,6 +243,18 @@ DECLARE_METHOD(bool, item_set_lexstring,
 */
 DECLARE_METHOD(bool, item_set_cstring, (log_item_data * lid, const char *s));
 
+/**
+  Set/reset one or more log line flags.
+
+  Example to set the flag:
+    log_line_set_flag(ll, LOG_LINE_EMIT_TELEMETRY, LOG_LINE_EMIT_TELEMETRY);
+  to reset the flag:
+    log_line_set_flag(ll, LOG_LINE_EMIT_TELEMETRY, 0);
+*/
+DECLARE_METHOD(void, line_set_flag,
+               (log_line * ll, log_line_flags_mask mask,
+                log_line_flags_mask value));
+
 /**
   Create new log item with key name "key", and allocation flags of
   "alloc" (see enum_log_item_free).
@@ -605,11 +649,38 @@ DECLARE_METHOD(int, dedicated_errstream, (void *my_errstream));
 
   @param       my_stream  a handle describing the log file
 
-  @returns     LOG_SERVICE_SUCCESS          success
-  @returns     otherwise                    failure
+  @returns    LOG_SERVICE_SUCCESS on success
 */
 DECLARE_METHOD(log_service_error, close_errstream, (void **my_errstream));
 
+/**
+  re-open an error log file
+  (primarily to facilitate flush/log-rotation)
+
+  If the new file can be opened, update the my_errstream descriptor to
+  use it and close the old file. Otherwise, keep using the old file.
+
+  @param       name_or_ext   if beginning with '.':
+                               @@global.log_error, except with this extension
+                             otherwise:
+                               use this as file name in the same location as
+                               @@global.log_error
+
+                             Value may not contain folder separators!
+
+                             In the general case, the caller will be a
+                             log-writer, the log-writer will just pass
+                             its preferred file extension, and the resulting
+                             file name and path will therefore be the same
+                             as for the original log file.
+
+  @param[in,out]  my_errstream  an error log handle
+
+  @returns LOG_SERVICE_INVALID_ARGUMENT, or the result of open_errstream()
+*/
+DECLARE_METHOD(log_service_error, reopen_errstream,
+               (const char *file, void **my_errstream));
+
 END_SERVICE_DEFINITION(log_builtins)
 
 /**
@@ -697,6 +768,7 @@ extern SERVICE_TYPE(log_builtins_string) * log_bs;
 #define log_set_float log_bi->item_set_float
 #define log_set_lexstring log_bi->item_set_lexstring
 #define log_set_cstring log_bi->item_set_cstring
+#define log_line_set_flag log_bi->line_set_flag
 #define log_malloc log_bs->malloc
 #define log_free log_bs->free
 #define log_msg log_bs->substitutev
@@ -716,22 +788,6 @@ extern SERVICE_TYPE(log_builtins_string) * log_bs;
 #define log_set_lexstring log_item_set_lexstring
 #define log_set_cstring log_item_set_cstring
 
-/**
-  Very long-running functions during server start-up can use this
-  function to check whether the time-out for buffered logging has
-  been reached. If so and we have urgent information, all buffered
-  log events will be flushed to the log using built-in default-logger
-  for the time being.  The information will be kept until start-up
-  completes in case it later turns out the user configured a loadable
-  logger, in which case we'll also flush the buffered information to
-  that logger later once the logger becomes available.
-
-  This function should only be used during start-up; once external
-  components are loaded by the component framework, this function
-  should no longer be called (as log events are no longer buffered,
-  but logged immediately).
-*/
-void log_sink_buffer_check_timeout(void);
 #endif  // LOG_H
 
 #ifndef DISABLE_ERROR_LOGGING
@@ -749,6 +805,22 @@ void log_sink_buffer_check_timeout(void);
       .function(__FUNCTION__)        \
       .lookup(ecode, ##__VA_ARGS__)
 
+#ifdef HAVE_LOG_DIAGNOSTIC
+#define LogDiag(severity, ecode, ...) \
+  LogEvent()                          \
+      .prio(severity)                 \
+      .errcode(ecode)                 \
+      .subsys(LOG_SUBSYSTEM_TAG)      \
+      .component(LOG_COMPONENT_TAG)   \
+      .source_line(__LINE__)          \
+      .source_file(MY_BASENAME)       \
+      .function(__FUNCTION__)         \
+      .type(LOG_TYPE_DIAG)            \
+      .lookup(ecode, ##__VA_ARGS__)
+#else
+#define LogDiag(...)
+#endif /* HAVE_LOG_DIAGNOSTIC */
+
 #define LogComponentErr(severity, ecode, ...)                           \
   LogEvent()                                                            \
       .prio(severity)                                                   \
@@ -807,12 +879,26 @@ void log_sink_buffer_check_timeout(void);
       .function(__FUNCTION__)        \
       .lookup(ecode, ##__VA_ARGS__)
 
+#ifdef HAVE_LOG_DIAGNOSTIC
+#define LogDiag(severity, ecode, ...) \
+  LogEvent()                          \
+      .prio(severity)                 \
+      .errcode(ecode)                 \
+      .subsys(LOG_SUBSYSTEM_TAG)      \
+      .source_line(__LINE__)          \
+      .source_file(MY_BASENAME)       \
+      .function(__FUNCTION__)         \
+      .type(LOG_TYPE_DIAG)            \
+      .lookup(ecode, ##__VA_ARGS__)
+#else
+#define LogDiag(...)
+#endif /* HAVE_LOG_DIAGNOSTIC */
 #endif
 
 #else
 
-inline void dummy_log_message(longlong severity MY_ATTRIBUTE((unused)),
-                              longlong ecode MY_ATTRIBUTE((unused)), ...) {
+inline void dummy_log_message(longlong severity [[maybe_unused]],
+                              longlong ecode [[maybe_unused]], ...) {
   return;
 }
 
@@ -837,9 +923,10 @@ inline void dummy_log_message(longlong severity MY_ATTRIBUTE((unused)),
 
 class LogEvent {
  private:
-  log_line *ll;
-  char *msg;
+  log_line *ll;  // Temporary allocation to hold a log-event.
+  char *msg;     // Temporary allocation to hold a message.
   const char *msg_tag;
+  bool have_msg;  // Was a message set in `msg` using set_message()?
 
   /**
     Set MySQL error-code if none has been set yet.
@@ -888,8 +975,28 @@ class LogEvent {
     if (ll != nullptr) {
       log_line_submit(this->ll);
       log_line_exit(ll);
-      log_free(msg);
+      /*
+        If a message was set on the LogEvent using set_message,
+        the message became part of a log_item on the log_line.
+        have_msg is true. The log_line's log_items were released
+        in log_line_submit(). Null `msg` here to prevent double-free.
+
+        On the other hand, if set_message() was not used, the
+        convenience buffer was never associated with the log_line,
+        and therefore wasn't freed when the log_line was submitted.
+        In that case, we'll leave the pointer intact for clean-up
+        further down.
+      */
+      if (have_msg) msg = nullptr;
     }
+
+    /*
+      If set_message() attached the buffer `msg` to the log_line,
+      the allocation has either been freed in log_line_submit()
+      above, or it is now owned by someone who stole it using steal().
+      In either case, msg==nullptr and we do nothing here.
+    */
+    if (msg != nullptr) log_free(msg);
   }
 
   /**
@@ -901,6 +1008,7 @@ class LogEvent {
     server, and therefore need to submit them free-form.
   */
   LogEvent() {
+    have_msg = false;
     if ((ll = log_line_init()) != nullptr) {
       if ((msg = (char *)log_malloc(LOG_BUFF_MAX)) == nullptr) {
         log_line_exit(ll);
@@ -912,8 +1020,59 @@ class LogEvent {
   }
 
   /**
+    Save the log-event allocated by LogEvent().
 
-     Set log type.
+    LogEvent() internally represents the log-event in a log_line
+    structure it allocates. steal() saves this pointer to the
+    method's argument. The pointer in the LogEvent() is then
+    cleared to prevent the destructor from freeing the log_line.
+    Freeing allocated memory is now the caller's responsibility:
+
+      log_line *ll;
+      LogEvent().prio(SYSTEM_LEVEL).message("Hi %s!", user).steal(&ll);
+
+      // Do some things with the event ...
+
+      log_line_item_free_all(ll);
+      log_line_exit(ll);
+
+    If a message was set, the message buffer pointed to by `msg`
+    becomes part of the log_line and is released on
+    log_line_item_free_all() (or equivalent).
+
+    If no message was set, the message buffer is released when
+    the LogEvent's destructor is called.
+
+    Note that verbatim() does NOT copy its argument to the
+    LogEvent's internal allocation `msg`.
+    Hence the life-cycle management of verbatim()'s argument
+    would be up to the developer. When using steal(), using it
+    with one of the set_message() wrappers is generally preferable:
+
+      LogEvent().message("%s", my_verbatim_string).steal(&ll);
+
+    @param  save_to  where to save the pointer to the log-event
+  */
+  void steal(log_line **save_to) {
+    *save_to = ll;
+    ll = nullptr;
+    /*
+      If the message was set, the allocation is now part of a log_item
+      on the log_line. Thus, it is now owned by the called and will
+      be released when they release the log_items on the log_line.
+      In that case, we null our pointer to it so we won't double-free
+      the allocation.
+
+      Conversely, if the buffer exists, but hasn't become part of
+      the log_line through use of a set_message() wrapper, we'll hold
+      on to the pointer so the empty buffer is released when we dest
+      the LogEvent().
+    */
+    if (have_msg) msg = nullptr;
+  }
+
+  /**
+    Set log type.
 
     @param  val  the log type (LOG_TYPE_ERROR)
 
@@ -1185,6 +1344,14 @@ class LogEvent {
     say unkind things about you.  Use registered messages and their
     error codes wherever possible!
 
+    Combining verbatim() with steal() is discouraged as it burdens
+    the developer with the life-cycle management of verbatim's
+    argument. This is a result of verbatim() using its argument
+    verbatim, rather than setting it up in the LogEvent's internal
+    allocation `msg` using set_message(). Hence, the LogEvent has
+    no copy of the message, which is an optimization for the
+    common, non-steal() case.
+
     @param  msg_arg the message. % substitution will not happen.
 
     @retval         the LogEvent, for easy fluent-style chaining.
@@ -1369,6 +1536,16 @@ class LogEvent {
         val);
     return *this;
   }
+
+  /**
+    Mark log line to skip being additionally emitted as a telemetry log record.
+
+    @retval      the LogEvent, for easy fluent-style chaining.
+  */
+  LogEvent &no_telemetry() {
+    log_line_set_flag(this->ll, LOG_LINE_EMIT_TELEMETRY, 0);
+    return *this;
+  }
 };
 
 inline void LogEvent::set_message_by_errcode(longlong errcode, va_list ap) {
@@ -1393,8 +1570,11 @@ inline void LogEvent::set_message(const char *fmt, va_list ap) {
       len = LOG_BUFF_MAX - 1;
       strcpy(&msg[LOG_BUFF_MAX - sizeof(ellipsis)], ellipsis);
     }
-    log_set_lexstring(log_line_item_set(this->ll, LOG_ITEM_LOG_MESSAGE), msg,
-                      len);
+    log_item_data *lid;
+    lid = log_line_item_set_with_key(this->ll, LOG_ITEM_LOG_MESSAGE, nullptr,
+                                     LOG_ITEM_FREE_VALUE);
+    log_set_lexstring(lid, msg, len);
+    have_msg = true;
   }
 }
 
@@ -1475,9 +1655,9 @@ inline bool init_logging_service_for_plugin(
   @retval     true   Failed.
 */
 inline bool init_logging_service_for_plugin(
-    SERVICE_TYPE(registry) * *reg_srv MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins) * *log_bi MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins_string) * *log_bs MY_ATTRIBUTE((unused)))
+    SERVICE_TYPE(registry) * *reg_srv [[maybe_unused]],
+    SERVICE_TYPE(log_builtins) * *log_bi [[maybe_unused]],
+    SERVICE_TYPE(log_builtins_string) * *log_bs [[maybe_unused]])
 
 {
   return false;
@@ -1491,9 +1671,9 @@ inline bool init_logging_service_for_plugin(
   param[in,out]  log_bs     String service for error logging.
 */
 inline void deinit_logging_service_for_plugin(
-    SERVICE_TYPE(registry) * *reg_srv MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins) * *log_bi MY_ATTRIBUTE((unused)),
-    SERVICE_TYPE(log_builtins_string) * *log_bs MY_ATTRIBUTE((unused))) {}
+    SERVICE_TYPE(registry) * *reg_srv [[maybe_unused]],
+    SERVICE_TYPE(log_builtins) * *log_bi [[maybe_unused]],
+    SERVICE_TYPE(log_builtins_string) * *log_bs [[maybe_unused]]) {}
 
 #endif  // MYSQL_DYNAMIC_PLUGIN
 
