@@ -1,7 +1,7 @@
 #ifndef SQL_STRING_INCLUDED
 #define SQL_STRING_INCLUDED
 
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -39,8 +39,6 @@
 #include <string_view>
 
 #include "lex_string.h"
-#include "m_ctype.h"   // my_convert
-#include "m_string.h"  // LEX_CSTRING
 #include "memory_debugging.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
@@ -51,6 +49,7 @@
 #include "mysql/mysql_lex_string.h"  // LEX_STRING
 #include "mysql/psi/psi_memory.h"
 #include "mysql/service_mysql_alloc.h"  // my_free
+#include "mysql/strings/m_ctype.h"      // my_convert
 
 struct MEM_ROOT;
 
@@ -159,9 +158,11 @@ size_t bin_to_hex_str(char *to, size_t to_len, const char *from,
 
 /**
   Using this class is fraught with peril, and you need to be very careful
-  when doing so. In particular, copy construction and assignment does not
+  when doing so. In particular, copy construction does not
   do a deep _nor_ a shallow copy; instead, it makes a _reference_ to the
   original string that will be invalid as soon as that string goes out of scope.
+  Copy assignment has been disabled for this reason, use one of the set()
+  functions instead.
   (Move constructiong and assignment is safe, though.) In general, it is
   probably better not to use this class at all if you can avoid it.
 */
@@ -264,6 +265,28 @@ class String {
     if (m_ptr && m_length < m_alloced_length) m_ptr[m_length] = 0;
     return m_ptr;
   }
+
+  /**
+    Returns a pointer to a C-style null-terminated string. The function is
+    "safe" in the sense that the returned string is guaranteed to be
+    null-terminated (as opposed to c_ptr_quick()). However, in terms of memory
+    safety, this function might perform a heap allocation, so using it
+    necessitates manual cleanup.
+
+    @note One potential pitfall when using this function happens when calling
+    c_ptr_safe() on a String object from the parser. In this case the internal
+    m_ptr string is already allocated on a MEM_ROOT and is cleaned up as part of
+    the query lifecycle. This cleanup is performed by simply clearing the
+    MEM_ROOT (freeing its allocated memory) and does not involve destructing the
+    String object. So in the case when the parser works with (ptr, length)-style
+    strings that are placed in String objects and m_length = m_allocated_length
+    we get a heap allocation when calling c_ptr_safe(), even if just to read the
+    contents of the string, and this newly allocated memory is not on the same
+    MEM_ROOT as the original string, and thus we get a memory leak if we do not
+    make sure to free it.
+
+    @return Pointer to null-terminated string.
+  */
   char *c_ptr_safe() {
     if (m_ptr && m_length < m_alloced_length)
       m_ptr[m_length] = 0;
@@ -414,22 +437,7 @@ class String {
     }
   }
   bool is_alloced() const { return m_is_alloced; }
-  String &operator=(const String &s) {
-    if (&s != this) {
-      /*
-        It is forbidden to do assignments like
-        some_string = substring_of_that_string
-       */
-      assert(!s.uses_buffer_owned_by(this));
-      mem_free();
-      m_ptr = s.m_ptr;
-      m_length = s.m_length;
-      m_alloced_length = s.m_alloced_length;
-      m_charset = s.m_charset;
-      m_is_alloced = false;
-    }
-    return *this;
-  }
+  String &operator=(const String &s) = delete;
   String &operator=(String &&s) noexcept {
     if (&s != this) {
       /*
@@ -474,8 +482,10 @@ class String {
 
   bool copy();                 // Alloc string if not allocated
   bool copy(const String &s);  // Allocate new string
-  // Allocate new string
   bool copy(const char *s, size_t arg_length, const CHARSET_INFO *cs);
+  bool copy(const char *s, size_t arg_length, const CHARSET_INFO *from_cs,
+            const CHARSET_INFO *to_cs, uint *errors);
+
   static bool needs_conversion(size_t arg_length, const CHARSET_INFO *cs_from,
                                const CHARSET_INFO *cs_to, size_t *offset);
   bool needs_conversion(const CHARSET_INFO *cs_to) const {
@@ -495,8 +505,6 @@ class String {
                     const CHARSET_INFO *cs);
   bool set_or_copy_aligned(const char *s, size_t arg_length,
                            const CHARSET_INFO *cs);
-  bool copy(const char *s, size_t arg_length, const CHARSET_INFO *csfrom,
-            const CHARSET_INFO *csto, uint *errors);
   bool append(const String &s);
   bool append(std::string_view s) { return append(s.data(), s.size()); }
   bool append(LEX_STRING *ls) { return append(ls->str, ls->length); }
@@ -565,17 +573,17 @@ class String {
   /* Inline (general) functions used by the protocol functions */
 
   char *prep_append(size_t arg_length, size_t step_alloc) {
-    size_t new_length = arg_length + m_length;
+    const size_t new_length = arg_length + m_length;
     if (new_length > m_alloced_length) {
       if (mem_realloc(new_length + step_alloc)) return nullptr;
     }
-    size_t old_length = m_length;
+    const size_t old_length = m_length;
     m_length += arg_length;
     return m_ptr + old_length; /* Area to use */
   }
 
   bool append(const char *s, size_t arg_length, size_t step_alloc) {
-    size_t new_length = arg_length + m_length;
+    const size_t new_length = arg_length + m_length;
     if (new_length > m_alloced_length &&
         mem_realloc_exp(new_length + step_alloc))
       return true;
@@ -617,7 +625,7 @@ class String {
   @param arg_length     Length of string to copy.
   @param from_cs        Character set to copy from
   @param to_cs          Character set to copy to
-  @param *offset	Returns number of unaligned characters.
+  @param[out] offset	Returns number of unaligned characters.
 
   @returns true if conversion is required, false otherwise.
 
@@ -641,8 +649,12 @@ inline bool String::needs_conversion(size_t arg_length,
 
 static inline void swap(String &a, String &b) noexcept { a.swap(b); }
 
-static inline std::string to_string(const String &str) {
-  return std::string(str.ptr(), str.length());
+inline std::string_view to_string_view(const String &str) {
+  return {str.ptr(), str.length()};
+}
+
+inline std::string to_string(const String &str) {
+  return std::string{to_string_view(str)};
 }
 
 /**
