@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,16 +21,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-/// @defgroup GroupLibsMysql MySQL Libraries
-
-/// @defgroup GroupLibsMysqlBinlog MySQL Libraries : Binlog Libraries
-/// @ingroup GroupLibsMysql
-
-/// @defgroup GroupLibsMysqlBinlogEvent MySQL Libraries : Binlog Event
-/// @ingroup GroupLibsMysqlBinlog
-
 /**
-  @file binlog_event.h
+  @file
 
   @brief Contains the classes representing events occurring in the replication
   stream. Each event is represented as a byte sequence with logical divisions
@@ -315,12 +308,13 @@ enum Log_event_type {
 
   TABLE_MAP_EVENT = 19,
 
-  /**
+  /*
     The V1 event numbers are used from 5.1.16 until mysql-5.6.
+    Not generated since 8.2.0, and rejected by the applier since 8.4.0
   */
-  WRITE_ROWS_EVENT_V1 = 23,
-  UPDATE_ROWS_EVENT_V1 = 24,
-  DELETE_ROWS_EVENT_V1 = 25,
+  OBSOLETE_WRITE_ROWS_EVENT_V1 = 23,
+  OBSOLETE_UPDATE_ROWS_EVENT_V1 = 24,
+  OBSOLETE_DELETE_ROWS_EVENT_V1 = 25,
 
   /**
     Something out of the ordinary happened on the master
@@ -399,14 +393,14 @@ struct Log_event_type_helper {
   }
 };
 
-/**
-  Struct to pass basic information about a event: type, query, is it ignorable
-*/
+/// Holds basic information about an event: common-header fields, query, etc.
 struct Log_event_basic_info {
   Log_event_type event_type{UNKNOWN_EVENT};
   const char *query{nullptr};
   size_t query_length{0};
   bool ignorable_event{false};
+  int32_t log_pos{0};
+  int32_t server_id{0};
 };
 
 /**
@@ -708,7 +702,7 @@ class Log_event_header {
 
   /*
     The offset in the log where this event originally appeared (it is
-    preserved in relay logs, making SHOW SLAVE STATUS able to print
+    preserved in relay logs, making SHOW REPLICA STATUS able to print
     coordinates of the event in the master's binlog).
   */
   unsigned long long log_pos;
