@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +22,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /**
-  @file control_events.h
+  @file
 
   @brief Contains the classes representing events operating in the replication
   stream properties. Each event is represented as a byte sequence with logical
@@ -41,9 +42,10 @@
 
 #include "mysql/binlog/event/binlog_event.h"
 #include "mysql/binlog/event/compression/base.h"  // mysql::binlog::event::compression::type
-#include "mysql/binlog/event/compression/buffer/buffer_sequence_view.h"  // Buffer_sequence_view
+#include "mysql/containers/buffers/buffer_sequence_view.h"  // Buffer_sequence_view
 #include "mysql/gtid/gtid_constants.h"
-#include "mysql/gtid/tsid.h"
+#include "mysql/gtid/tsid.h"        // Tsid
+#include "mysql/gtid/tsid_plain.h"  // Tsid_plain
 #include "mysql/gtid/uuid.h"
 #include "mysql/serialization/field_definition_helpers.h"
 #include "mysql/serialization/field_functor.h"
@@ -367,7 +369,7 @@ class Format_description_event : public Binary_log_event {
   A stop event is written to the log files under these circumstances:
   - A master writes the event to the binary log when it shuts down.
   - A slave writes the event to the relay log when it shuts down or
-    when a RESET SLAVE statement is executed.
+    when a RESET REPLICA statement is executed.
 
   @section Stop_event_binary_format Binary Format
 
@@ -387,7 +389,7 @@ class Stop_event : public Binary_log_event {
     A Stop_event is occurs under these circumstances:
     -  A master writes the event to the binary log when it shuts down
     -  A slave writes the event to the relay log when it shuts down or when a
-       RESET SLAVE statement is executed
+       RESET REPLICA statement is executed
     @param buf  Contains the serialized event.
     @param fde  An FDE event (see Rotate_event constructor for more info).
   */
@@ -734,7 +736,7 @@ struct gtid_info {
 class Transaction_payload_event : public Binary_log_event {
  public:
   using Buffer_sequence_view_t =
-      mysql::binlog::event::compression::buffer::Buffer_sequence_view<>;
+      mysql::containers::buffers::Buffer_sequence_view<>;
 
  private:
   Transaction_payload_event &operator=(const Transaction_payload_event &) =
@@ -1120,18 +1122,18 @@ class Gtid_event : public Binary_log_event,
         mysql::serialization::define_field(sequence_number),
         mysql::serialization::define_field(immediate_commit_timestamp),
         mysql::serialization::define_field(
-            original_commit_timestamp, Field_missing_functor([this]() -> auto {
+            original_commit_timestamp, Field_missing_functor([this]() -> auto{
               this->original_commit_timestamp =
                   this->immediate_commit_timestamp;
             })),
         mysql::serialization::define_field(transaction_length),
         mysql::serialization::define_field(immediate_server_version),
         mysql::serialization::define_field(
-            original_server_version, Field_missing_functor([this]() -> auto {
+            original_server_version, Field_missing_functor([this]() -> auto{
               this->original_server_version = this->immediate_server_version;
             })),
         mysql::serialization::define_field(
-            commit_group_ticket, Field_missing_functor([this]() -> auto {
+            commit_group_ticket, Field_missing_functor([this]() -> auto{
               this->commit_group_ticket = Gtid_event::kGroupTicketUnset;
             })));
   }
@@ -1159,7 +1161,7 @@ class Gtid_event : public Binary_log_event,
         mysql::serialization::define_field(immediate_commit_timestamp),
         mysql::serialization::define_field(
             original_commit_timestamp, Field_encode_predicate([this]() -> bool {
-              return this->original_commit_timestamp ==
+              return this->original_commit_timestamp !=
                      this->immediate_commit_timestamp;
             })),
         mysql::serialization::define_field(transaction_length),
@@ -1757,6 +1759,10 @@ class Heartbeat_event : public Binary_log_event {
   // Return the length of file name
   unsigned int get_ident_len() { return ident_len; }
 
+  ~Heartbeat_event() {
+    if (log_ident) bapi_free(const_cast<char *>(log_ident));
+  }
+
  protected:
   const char *log_ident;
   unsigned int ident_len; /** filename length */
