@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,16 +31,20 @@
 */
 
 #include <fcntl.h>
+#include <stdlib.h>
+#include <sys/stat.h>
 #include <sys/types.h>
 
 #include <cassert>
 #include <cstdarg>
 #include <cstdint>
 #include <cstring>
-#include <deque>
+#include <functional>
 #include <memory>
 #include <mutex>
 #include <new>
+#include <string>
+#include <string_view>
 
 #include "my_config.h"
 
@@ -64,6 +68,16 @@
 #include "strmake.h"
 #include "strxmov.h"
 
+#ifdef MYSQL_SERVER
+#include "mysql/components/services/log_builtins.h"
+#include "mysqld_error.h"
+#include "sql/current_thd.h"
+#include "sql/mysqld.h"
+#include "sql/sql_class.h"
+#endif
+
+using myf = int;
+
 /*
   The code below implements this functionality:
 
@@ -74,8 +88,6 @@
     - Setting server default character set
 */
 
-extern CHARSET_INFO my_charset_cp932_japanese_ci;
-
 namespace {
 
 class Mysys_charset_loader : public MY_CHARSET_LOADER {
@@ -132,7 +144,7 @@ void *Mysys_charset_loader::read_file(const char *path, size_t *size) {
     return nullptr;
   }
 
-  size_t len = stat_info.st_size;
+  size_t const len = stat_info.st_size;
   if (len > MY_MAX_ALLOWED_BUF) {
     return nullptr;
   }
@@ -143,12 +155,12 @@ void *Mysys_charset_loader::read_file(const char *path, size_t *size) {
     return nullptr;
   }
 
-  int fd = mysql_file_open(key_file_charset, path, O_RDONLY, 0);
+  int const fd = mysql_file_open(key_file_charset, path, O_RDONLY, 0);
   if (fd < 0) {
     return nullptr;
   }
 
-  size_t tmp_len = mysql_file_read(fd, buf.get(), len, 0);
+  size_t const tmp_len = mysql_file_read(fd, buf.get(), len, 0);
   mysql_file_close(fd, 0);
   if (tmp_len != len) {
     return nullptr;
@@ -198,13 +210,13 @@ static void init_available_charsets() {
 
 uint get_collation_number(const char *collation_name) {
   std::call_once(charsets_initialized, init_available_charsets);
-  mysql::collation::Name name{collation_name};
+  mysql::collation::Name const name{collation_name};
   return entry()->get_collation_id(name);
 }
 
 unsigned get_charset_number(const char *cs_name, uint cs_flags) {
   std::call_once(charsets_initialized, init_available_charsets);
-  mysql::collation::Name name{cs_name};
+  mysql::collation::Name const name{cs_name};
   if ((cs_flags & MY_CS_PRIMARY)) {
     return entry()->get_primary_collation_id(name);
   }
@@ -250,6 +262,75 @@ CHARSET_INFO *get_charset(uint cs_number, myf flags) {
   return cs;
 }
 
+namespace {
+
+bool starts_with_utf8(const char *name) {
+  // See normalization of names in mysql::collation::Name::Name()
+  const uint8_t *map = my_charset_latin1.to_lower;
+  constexpr const char prefix[] = "UTF8_";
+  if (strlen(name) < strlen(prefix)) return false;
+  for (unsigned int ix = 0; ix < strlen(prefix); ++ix) {
+    if (map[static_cast<uint8_t>(name[ix])] !=
+        map[static_cast<uint8_t>(prefix[ix])]) {
+      return false;
+    }
+  }
+  return true;
+}
+
+/**
+  What does "utf8" mean, "utf8mb3" or "utf8mb4"?
+
+  We ask 'current_thd' whether MODE_INTERPRET_UTF8_AS_UTF8MB4 has been
+  set in SQL_MODE.
+
+  We return 3 or 4 to indicate what alias is active.  A special value
+  42 is used during bootstrap: reject command line arguments, and
+  abort. There are no lookups during shutdown.
+
+  For client-side requests (e.g. mysql --default-character-set=utf8)
+  we always return 3.
+
+  @param [in] cname Charset/Collation name, used during bootstrap
+              to give the desired error message.
+
+  @return 3,4,42 to indicate what alias to use, or to abort.
+ */
+int utf8_alias_lookup(const char *cname [[maybe_unused]]) {
+#ifdef MYSQL_SERVER
+  enum_server_operational_state server_state = get_server_state();
+  if (server_state == SERVER_BOOTING) {
+    if (native_strcasecmp(cname, "utf8") == 0) {
+      LogErr(ERROR_LEVEL, ER_INVALID_SERVER_OPTION_CHARSET);
+      return 42;  // abort
+    }
+    if (native_strncasecmp(cname, "utf8_", 5) == 0) {
+      LogErr(ERROR_LEVEL, ER_INVALID_SERVER_OPTION_COLLATION, cname);
+      return 42;  // abort
+    }
+    assert(false);
+    return 3;
+  }
+  if (server_state == SERVER_OPERATING) {
+    assert(current_thd != nullptr);
+    return (current_thd->interpret_utf8_as_utf8mb4() ? 4 : 3);
+  }
+  if (server_state == SERVER_SHUTTING_DOWN) {
+    assert(false);
+    return 42;
+  }
+  assert(false);
+  return 42;
+#else
+  // We do not want to report an error
+  //    Character set 'utf8' is not a compiled character set ...
+  // whenever a client is invoked with --default-character-set=utf8
+  return 3;
+#endif  // MYSQL_SERVER
+}
+
+}  // namespace
+
 /**
   Find collation by name: extended version of get_charset_by_name()
   to return error messages to the caller.
@@ -264,19 +345,56 @@ CHARSET_INFO *my_collation_get_by_name(const char *collation_name, myf flags,
                                        MY_CHARSET_ERRMSG *errmsg) {
   std::call_once(charsets_initialized, init_available_charsets);
 
-  mysql::collation::Name name{collation_name};
+  std::string collation_name_string(collation_name);
+  if (starts_with_utf8(collation_name)) {
+    int retval = utf8_alias_lookup(collation_name);
+    // abort, assume that utf8_alias_lookup has logged something
+    if (retval == 42) return nullptr;
+    assert(retval == 3 || retval == 4);
+
+    // Insert "mb3" to get "utf8mb3_xxxx" or
+    //        "mb4" to get "utf8mb4_xxxx"
+    // These have no utf8mb4 counterparts:
+    //   utf8mb3_general_mysql500_ci
+    //   utf8mb3_tolower_ci
+    // So for retval 4 we return an error.
+    const char *mb3_or_mb4 = nullptr;
+    if (retval == 4) {
+      // See normalization of names in mysql::collation::Name::Name()
+      if (0 == my_strcasecmp(&my_charset_latin1, collation_name,
+                             "utf8_general_mysql500_ci") ||
+          0 == my_strcasecmp(&my_charset_latin1, collation_name,
+                             "utf8_tolower_ci")) {
+        errmsg->errcode = EE_COLLATION_ALIAS_ERROR;
+        collation_name_string.insert(4, "mb3");
+        snprintf(errmsg->errarg, sizeof(errmsg->errarg),
+                 "Collation '%s' must be specified explicitly as '%s'",
+                 collation_name, collation_name_string.c_str());
+        return nullptr;
+      }
+      mb3_or_mb4 = "mb4";
+    } else {
+      mb3_or_mb4 = "mb3";
+    }
+
+    collation_name_string.insert(4, mb3_or_mb4);
+    collation_name = collation_name_string.c_str();
+  }
+
+  mysql::collation::Name const name{collation_name};
   CHARSET_INFO *cs = entry()->find_by_name(name, flags, errmsg);
-  if (!cs && (flags & MY_WME)) {
+  if (cs == nullptr && (flags & MY_WME)) {
     char index_file[FN_REFLEN + sizeof(MY_CHARSET_INDEX)];
     my_stpcpy(get_charsets_dir(index_file), MY_CHARSET_INDEX);
-    my_error(EE_UNKNOWN_COLLATION, MYF(0), name().c_str(), index_file);
+    my_error(EE_UNKNOWN_COLLATION, MYF(0),
+             std::string(name.to_string_view()).c_str(), index_file);
   }
   return cs;
 }
 
-CHARSET_INFO *get_charset_by_name(const char *cs_name, myf flags) {
+CHARSET_INFO *get_charset_by_name(const char *collation_name, myf flags) {
   MY_CHARSET_ERRMSG dummy;
-  return my_collation_get_by_name(cs_name, flags, &dummy);
+  return my_collation_get_by_name(collation_name, flags, &dummy);
 }
 
 /**
@@ -296,21 +414,34 @@ CHARSET_INFO *my_charset_get_by_name(const char *cs_name, uint cs_flags,
 
   std::call_once(charsets_initialized, init_available_charsets);
 
-  mysql::collation::Name name{cs_name};
+  mysql::collation::Name const name{cs_name};
   CHARSET_INFO *cs = nullptr;
   if (cs_flags & MY_CS_PRIMARY) {
     cs = entry()->find_primary(name, flags, errmsg);
-    if (cs == nullptr && name() == "utf8") {
+
+    if (cs == nullptr && name.to_string_view() == "utf8") {
+      int retval = utf8_alias_lookup("utf8");
+      // abort, assume that utf8_alias_lookup has logged something
+      if (retval == 42) return nullptr;
+      assert(retval == 3 || retval == 4);
+
       // The parser does get_charset_by_csname().
       // Also needed for e.g. SET character_set_client= 'utf8'.
-      cs = entry()->find_primary(mysql::collation::Name("utf8mb3"), flags,
-                                 errmsg);
+      // Also needed by the lexer for: "select _utf8 0xD0B0D0B1D0B2;"
+      cs = entry()->find_primary(
+          mysql::collation::Name(retval == 4 ? "utf8mb4" : "utf8mb3"), flags,
+          errmsg);
     }
   } else if (cs_flags & MY_CS_BINSORT) {
     cs = entry()->find_default_binary(name, flags, errmsg);
-    if (cs == nullptr && name() == "utf8") {
-      cs = entry()->find_default_binary(mysql::collation::Name("utf8mb3"),
-                                        flags, errmsg);
+    if (cs == nullptr && name.to_string_view() == "utf8") {
+      int retval = utf8_alias_lookup("utf8");
+      if (retval == 42) return nullptr;
+      assert(retval == 3 || retval == 4);
+
+      cs = entry()->find_default_binary(
+          mysql::collation::Name(retval == 4 ? "utf8mb4" : "utf8mb3"), flags,
+          errmsg);
     }
   }
   if (!cs && (flags & MY_WME)) {
@@ -364,9 +495,9 @@ bool resolve_charset(const char *cs_name, const CHARSET_INFO *default_cs,
   and false is returned. If there is no such collation, "default_cl" is
   assigned to the "cl" and true is returned.
 
-  @param[out] cl        Variable to store collation.
   @param[in] cl_name    Collation name.
   @param[in] default_cl Default collation.
+  @param[out] cl        Variable to store collation.
 
   @return false if collation was resolved successfully; true if there is no
   collation with given name.
@@ -384,123 +515,23 @@ bool resolve_collation(const char *cl_name, const CHARSET_INFO *default_cl,
   return false;
 }
 
-/*
-  Escape string with backslashes (\)
-
-  SYNOPSIS
-    escape_string_for_mysql()
-    charset_info        Charset of the strings
-    to                  Buffer for escaped string
-    to_length           Length of destination buffer, or 0
-    from                The string to escape
-    length              The length of the string to escape
-
-  DESCRIPTION
-    This escapes the contents of a string by adding backslashes before special
-    characters, and turning others into specific escape sequences, such as
-    turning newlines into \n and null bytes into \0.
-
-  NOTE
-    To maintain compatibility with the old C API, to_length may be 0 to mean
-    "big enough"
-
-  RETURN VALUES
-    (size_t) -1 The escaped string did not fit in the to buffer
-    #           The length of the escaped string
-*/
-
-size_t escape_string_for_mysql(const CHARSET_INFO *charset_info, char *to,
-                               size_t to_length, const char *from,
-                               size_t length) {
-  const char *to_start = to;
-  const char *end = nullptr;
-  const char *to_end = to_start + (to_length ? to_length - 1 : 2 * length);
-  bool overflow = false;
-  const bool use_mb_flag = use_mb(charset_info);
-  for (end = from + length; from < end; from++) {
-    char escape = 0;
-    int tmp_length = 0;
-    if (use_mb_flag && (tmp_length = my_ismbchar(charset_info, from, end))) {
-      if (to + tmp_length > to_end) {
-        overflow = true;
-        break;
-      }
-      while (tmp_length--) *to++ = *from++;
-      from--;
-      continue;
-    }
-    /*
-     If the next character appears to begin a multi-byte character, we
-     escape that first byte of that apparent multi-byte character. (The
-     character just looks like a multi-byte character -- if it were actually
-     a multi-byte character, it would have been passed through in the test
-     above.)
-
-     Without this check, we can create a problem by converting an invalid
-     multi-byte character into a valid one. For example, 0xbf27 is not
-     a valid GBK character, but 0xbf5c is. (0x27 = ', 0x5c = \)
-    */
-    tmp_length = use_mb_flag ? my_mbcharlen_ptr(charset_info, from, end) : 0;
-    if (tmp_length > 1)
-      escape = *from;
-    else
-      switch (*from) {
-        case 0: /* Must be escaped for 'mysql' */
-          escape = '0';
-          break;
-        case '\n': /* Must be escaped for logs */
-          escape = 'n';
-          break;
-        case '\r':
-          escape = 'r';
-          break;
-        case '\\':
-          escape = '\\';
-          break;
-        case '\'':
-          escape = '\'';
-          break;
-        case '"': /* Better safe than sorry */
-          escape = '"';
-          break;
-        case '\032': /* This gives problems on Win32 */
-          escape = 'Z';
-          break;
-      }
-    if (escape) {
-      if (to + 2 > to_end) {
-        overflow = true;
-        break;
-      }
-      *to++ = '\\';
-      *to++ = escape;
-    } else {
-      if (to + 1 > to_end) {
-        overflow = true;
-        break;
-      }
-      *to++ = *from;
-    }
-  }
-  *to = 0;
-  return overflow ? (size_t)-1 : (size_t)(to - to_start);
-}
-
 #ifdef _WIN32
+extern CHARSET_INFO my_charset_cp932_japanese_ci;
+
 static CHARSET_INFO *fs_cset_cache = nullptr;
 
 CHARSET_INFO *fs_character_set() {
-  if (!fs_cset_cache) {
+  if (fs_cset_cache == nullptr) {
     char buf[10] = "cp";
     GetLocaleInfo(LOCALE_SYSTEM_DEFAULT, LOCALE_IDEFAULTANSICODEPAGE, buf + 2,
                   sizeof(buf) - 3);
     /*
-      We cannot call get_charset_by_name here
-      because fs_character_set() is executed before
-      LOCK_THD_charset mutex initialization, which
-      is used inside get_charset_by_name.
-      As we're now interested in cp932 only,
-      let's just detect it using strcmp().
+      We cannot call get_charset_by_name here,
+      we will end up in a deadlock (in std::call_once) because of recursion:
+        init-avaliable-charsets ->
+        get_charsets_dir ->
+        convert_dirname ->
+        fs_character_set
     */
     fs_cset_cache =
         !strcmp(buf, "cp932") ? &my_charset_cp932_japanese_ci : &my_charset_bin;
@@ -509,75 +540,6 @@ CHARSET_INFO *fs_character_set() {
 }
 #endif
 
-/*
-  Escape apostrophes by doubling them up
-
-  SYNOPSIS
-    escape_quotes_for_mysql()
-    charset_info        Charset of the strings
-    to                  Buffer for escaped string
-    to_length           Length of destination buffer, or 0
-    from                The string to escape
-    length              The length of the string to escape
-    quote               The quote the buffer will be escaped against
-
-  DESCRIPTION
-    This escapes the contents of a string by doubling up any character
-    specified by the quote parameter. This is used when the
-    NO_BACKSLASH_ESCAPES SQL_MODE is in effect on the server.
-
-  NOTE
-    To be consistent with escape_string_for_mysql(), to_length may be 0 to
-    mean "big enough"
-
-  RETURN VALUES
-    ~0          The escaped string did not fit in the to buffer
-    >=0         The length of the escaped string
-*/
-
-size_t escape_quotes_for_mysql(CHARSET_INFO *charset_info, char *to,
-                               size_t to_length, const char *from,
-                               size_t length, char quote) {
-  const char *to_start = to;
-  const char *end = nullptr;
-  const char *to_end = to_start + (to_length ? to_length - 1 : 2 * length);
-  bool overflow = false;
-  const bool use_mb_flag = use_mb(charset_info);
-  for (end = from + length; from < end; from++) {
-    int tmp_length = 0;
-    if (use_mb_flag && (tmp_length = my_ismbchar(charset_info, from, end))) {
-      if (to + tmp_length > to_end) {
-        overflow = true;
-        break;
-      }
-      while (tmp_length--) *to++ = *from++;
-      from--;
-      continue;
-    }
-    /*
-      We don't have the same issue here with a non-multi-byte character being
-      turned into a multi-byte character by the addition of an escaping
-      character, because we are only escaping the ' character with itself.
-     */
-    if (*from == quote) {
-      if (to + 2 > to_end) {
-        overflow = true;
-        break;
-      }
-      *to++ = quote;
-      *to++ = quote;
-    } else {
-      if (to + 1 > to_end) {
-        overflow = true;
-        break;
-      }
-      *to++ = *from;
-    }
-  }
-  *to = 0;
-  return overflow ? (ulong)~0 : (ulong)(to - to_start);
-}
-
 void charset_uninit() {
   mysql::collation::shutdown();
 
