@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,18 +30,17 @@
   @file mysys/my_fopen.cc
 */
 
-#include <errno.h>
 #include <fcntl.h>
-#include <stdio.h>
-#include <sys/types.h>
+#include <cassert>
+#include <cerrno>
+#include <cstdio>
+#include <string>
 
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
 #include "my_sys.h"
 #include "my_thread_local.h"
-#include "mysql/psi/mysql_mutex.h"
-#include "mysql/service_mysql_alloc.h"
 #include "mysys/mysys_priv.h"
 #include "mysys_err.h"
 
