@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   Without limiting anything contained in the foregoing, this file,
   which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -36,9 +37,10 @@
   my_timer_microseconds     ulonglong "microseconds"
   my_timer_milliseconds     ulonglong milliseconds
   my_timer_ticks            ulonglong ticks
+  my_timer_thread_cpu       ulonglong thread_cpu
   my_timer_init             initialization / test
 
-  We'll call the first 5 functions (the ones that return
+  We'll call the first 6 functions (the ones that return
   a ulonglong) "my_timer_xxx" functions.
   Each my_timer_xxx function returns a 64-bit timing value
   since an arbitrary 'epoch' start. Since the only purpose
@@ -62,18 +64,17 @@
   elapsed_time= (time2 - time1) - overhead
 */
 
-#include <stdio.h>
+#include "my_rdtsc.h"
 
 #include "my_config.h"
 #include "my_inttypes.h"
-#include "my_rdtsc.h"
 #if defined(_WIN32)
 #include "windows.h"
 #endif
 
 #if defined(TIME_WITH_SYS_TIME)
 #include <sys/time.h>
-#include <time.h> /* for clock_gettime */
+#include <ctime> /* for clock_gettime */
 #endif
 
 #if defined(HAVE_SYS_TIMES_H) && defined(HAVE_TIMES)
@@ -84,26 +85,6 @@
 #include <mach/mach_time.h>
 #endif
 
-#if defined(__SUNPRO_CC) && defined(__sparcv9) && defined(_LP64) && \
-    !defined(__SunOS_5_7)
-extern "C" ulonglong my_timer_cycles_il_sparc64();
-#elif defined(__SUNPRO_CC) && defined(_ILP32) && !defined(__SunOS_5_7)
-extern "C" ulonglong my_timer_cycles_il_sparc32();
-#elif defined(__SUNPRO_CC) && defined(__i386) && defined(_ILP32)
-extern "C" ulonglong my_timer_cycles_il_i386();
-#elif defined(__SUNPRO_CC) && defined(__x86_64) && defined(_LP64)
-extern "C" ulonglong my_timer_cycles_il_x86_64();
-#elif defined(__SUNPRO_C) && defined(__sparcv9) && defined(_LP64) && \
-    !defined(__SunOS_5_7)
-ulonglong my_timer_cycles_il_sparc64();
-#elif defined(__SUNPRO_C) && defined(_ILP32) && !defined(__SunOS_5_7)
-ulonglong my_timer_cycles_il_sparc32();
-#elif defined(__SUNPRO_C) && defined(__i386) && defined(_ILP32)
-ulonglong my_timer_cycles_il_i386();
-#elif defined(__SUNPRO_C) && defined(__x86_64) && defined(_LP64)
-ulonglong my_timer_cycles_il_x86_64();
-#endif
-
 /*
   For cycles, we depend on RDTSC for x86 platforms,
   or on time buffer (which is not really a cycle count
@@ -112,14 +93,12 @@ ulonglong my_timer_cycles_il_x86_64();
   gethrtime which is okay for solaris.
 */
 
-ulonglong my_timer_cycles(void) {
+ulonglong my_timer_cycles() {
 #if defined(__GNUC__) && defined(__i386__)
   /* This works much better if compiled with "gcc -O3". */
   ulonglong result;
   __asm__ __volatile__("rdtsc" : "=A"(result));
   return result;
-#elif defined(__SUNPRO_C) && defined(__i386)
-  __asm("rdtsc");
 #elif defined(__GNUC__) && defined(__x86_64__)
   ulonglong result;
   __asm__ __volatile__(
@@ -163,21 +142,8 @@ ulonglong my_timer_cycles(void) {
     result = x1;
     return (result << 32) | x2;
   }
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(__sparcv9) && \
-    defined(_LP64) && !defined(__SunOS_5_7)
-  return (my_timer_cycles_il_sparc64());
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(_ILP32) && \
-    !defined(__SunOS_5_7)
-  return (my_timer_cycles_il_sparc32());
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(__i386) && \
-    defined(_ILP32)
-  /* This is probably redundant for __SUNPRO_C. */
-  return (my_timer_cycles_il_i386());
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(__x86_64) && \
-    defined(_LP64)
-  return (my_timer_cycles_il_x86_64());
 #elif defined(__GNUC__) && (defined(__sparcv9) || defined(__sparc_v9__)) && \
-    defined(_LP64)
+    defined(_LP64) && !defined(__clang__)
   {
     ulonglong result;
     __asm __volatile__("rd %%tick,%0" : "=r"(result));
@@ -200,7 +166,13 @@ ulonglong my_timer_cycles(void) {
 #elif defined(__GNUC__) && defined(__aarch64__)
   {
     ulonglong result;
-    __asm __volatile__("mrs %[rt],cntvct_el0" : [ rt ] "=r"(result));
+    __asm __volatile__("mrs %[rt],cntvct_el0" : [rt] "=r"(result));
+    return result;
+  }
+#elif defined(__GNUC__) && defined(__s390x__)
+  {
+    uint64_t result;
+    __asm __volatile__("stckf %0" : "=Q"(result) : : "cc");
     return result;
   }
 #elif defined(HAVE_SYS_TIMES_H) && defined(HAVE_GETHRTIME)
@@ -217,7 +189,7 @@ ulonglong my_timer_cycles(void) {
   (b) really has nanosecond resolution.
 */
 
-ulonglong my_timer_nanoseconds(void) {
+ulonglong my_timer_nanoseconds() {
 #if defined(HAVE_SYS_TIMES_H) && defined(HAVE_GETHRTIME)
   /* SunOS 5.10+, Solaris, HP-UX: hrtime_t gethrtime(void) */
   return (ulonglong)gethrtime();
@@ -249,7 +221,7 @@ ulonglong my_timer_nanoseconds(void) {
   the frequency is same as the cycle frequency.)
 */
 
-ulonglong my_timer_microseconds(void) {
+ulonglong my_timer_microseconds() {
 #if defined(HAVE_GETTIMEOFDAY)
   {
     struct timeval tv;
@@ -288,7 +260,7 @@ ulonglong my_timer_microseconds(void) {
   GetSystemTimeAsFileTime.
 */
 
-ulonglong my_timer_milliseconds(void) {
+ulonglong my_timer_milliseconds() {
 #if defined(HAVE_GETTIMEOFDAY)
   {
     struct timeval tv;
@@ -325,19 +297,56 @@ ulonglong my_timer_milliseconds(void) {
   bad, sometimes even worse than gettimeofday's overhead.
 */
 
-ulonglong my_timer_ticks(void) {
+ulonglong my_timer_ticks() {
 #if defined(HAVE_SYS_TIMES_H) && defined(HAVE_TIMES)
   {
     struct tms times_buf;
     return (ulonglong)times(&times_buf);
   }
 #elif defined(_WIN32)
-  return (ulonglong)GetTickCount();
+  return GetTickCount64();
 #else
   return 0;
 #endif
 }
 
+/**
+  THREAD_CPU timer.
+  Expressed in nanoseconds.
+*/
+ulonglong my_timer_thread_cpu() {
+#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_THREAD_CPUTIME_ID)
+  {
+    struct timespec tp;
+    clock_gettime(CLOCK_THREAD_CPUTIME_ID, &tp);
+    return (ulonglong)tp.tv_sec * 1000000000 + (ulonglong)tp.tv_nsec;
+  }
+#elif defined(_WIN32)
+  {
+    HANDLE hThread = GetCurrentThread();
+    FILETIME start;
+    FILETIME end;
+    FILETIME system;
+    FILETIME user;
+    ulonglong result;
+    if (GetThreadTimes(hThread, &start, &end, &system, &user) != 0) {
+      /*
+        GetThreadTimes() return a number expressed in 100 nanosecs units.
+      */
+      result = (user.dwHighDateTime + system.dwHighDateTime);
+      result <<= 32;
+      result += (user.dwLowDateTime + system.dwLowDateTime);
+      result *= 100;
+    } else {
+      result = 0;
+    }
+    return result;
+  }
+#else
+#warning "Implement my_timer_thread_cpu() for this platform."
+  return 0;
+#endif
+}
 /*
   The my_timer_init() function and its sub-functions
   have several loops which call timers. If there's
@@ -360,8 +369,8 @@ ulonglong my_timer_ticks(void) {
 */
 
 static void my_timer_init_overhead(ulonglong *overhead,
-                                   ulonglong (*cycle_timer)(void),
-                                   ulonglong (*this_timer)(void),
+                                   ulonglong (*cycle_timer)(),
+                                   ulonglong (*this_timer)(),
                                    ulonglong best_timer_overhead) {
   ulonglong time1, time2;
   int i;
@@ -390,7 +399,7 @@ static void my_timer_init_overhead(ulonglong *overhead,
   Often GetTickCount() has resolution = 15.
   We don't check with ticks because they take too long.
 */
-static ulonglong my_timer_init_resolution(ulonglong (*this_timer)(void),
+static ulonglong my_timer_init_resolution(ulonglong (*this_timer)(),
                                           ulonglong overhead_times_2) {
   ulonglong time1, time2;
   ulonglong best_jump;
@@ -442,6 +451,13 @@ static ulonglong my_timer_init_frequency(MY_TIMER_INFO *mti) {
   time4 = my_timer_cycles() - mti->cycles.overhead;
   time4 -= mti->microseconds.overhead;
 
+  if (time4 <= time1) {
+    /*
+      Seen happening on Sparc S7 when execution during measurement switched to
+      different CPU that has different value of the tick.
+    */
+    return 0;
+  }
   if (time3 <= time2) {
     /*
       Seen happening with ASAN / UBSAN builds.
@@ -464,15 +480,13 @@ static ulonglong my_timer_init_frequency(MY_TIMER_INFO *mti) {
 */
 
 void my_timer_init(MY_TIMER_INFO *mti) {
-  ulonglong (*best_timer)(void);
+  ulonglong (*best_timer)();
   ulonglong best_timer_overhead;
 
   /* cycles */
   mti->cycles.frequency = 1000000000;
 #if defined(__GNUC__) && defined(__i386__)
   mti->cycles.routine = MY_TIMER_ROUTINE_ASM_X86;
-#elif defined(__SUNPRO_C) && defined(__i386)
-  mti->cycles.routine = MY_TIMER_ROUTINE_ASM_X86;
 #elif defined(__GNUC__) && defined(__x86_64__)
   mti->cycles.routine = MY_TIMER_ROUTINE_ASM_X86_64;
 #elif defined(_WIN64) && defined(_M_X64)
@@ -485,25 +499,13 @@ void my_timer_init(MY_TIMER_INFO *mti) {
 #elif defined(__GNUC__) && (defined(__powerpc__) || defined(__POWERPC__)) && \
     (!defined(__64BIT__) && !defined(_ARCH_PPC64))
   mti->cycles.routine = MY_TIMER_ROUTINE_ASM_PPC;
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(__sparcv9) && \
-    defined(_LP64) && !defined(__SunOS_5_7)
-  mti->cycles.routine = MY_TIMER_ROUTINE_ASM_SUNPRO_SPARC64;
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(_ILP32) && \
-    !defined(__SunOS_5_7)
-  mti->cycles.routine = MY_TIMER_ROUTINE_ASM_SUNPRO_SPARC32;
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(__i386) && \
-    defined(_ILP32)
-  mti->cycles.routine = MY_TIMER_ROUTINE_ASM_SUNPRO_I386;
-#elif (defined(__SUNPRO_CC) || defined(__SUNPRO_C)) && defined(__x86_64) && \
-    defined(_LP64)
-  mti->cycles.routine = MY_TIMER_ROUTINE_ASM_SUNPRO_X86_64;
 #elif defined(__GNUC__) && (defined(__sparcv9) || defined(__sparc_v9__)) && \
     defined(_LP64)
   mti->cycles.routine = MY_TIMER_ROUTINE_ASM_GCC_SPARC64;
-#elif defined(__GNUC__) && defined(__sparc__) && !defined(_LP64)
-  mti->cycles.routine = MY_TIMER_ROUTINE_ASM_GCC_SPARC32;
 #elif defined(__GNUC__) && defined(__aarch64__)
   mti->cycles.routine = MY_TIMER_ROUTINE_ASM_AARCH64;
+#elif defined(__GNUC__) && defined(__s390x__)
+  mti->cycles.routine = MY_TIMER_ROUTINE_ASM_S390X;
 #elif defined(HAVE_SYS_TIMES_H) && defined(HAVE_GETHRTIME)
   mti->cycles.routine = MY_TIMER_ROUTINE_GETHRTIME;
 #else
@@ -592,6 +594,22 @@ void my_timer_init(MY_TIMER_INFO *mti) {
     mti->ticks.overhead = 0;
   }
 
+  /* thread_cpu */
+  mti->thread_cpu.frequency = 1000000000; /* initial assumption */
+#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_THREAD_CPUTIME_ID)
+  mti->thread_cpu.routine = MY_TIMER_ROUTINE_CLOCK_GETTIME;
+#elif defined(_WIN32)
+  mti->thread_cpu.routine = MY_TIMER_ROUTINE_GET_THREAD_TIMES;
+#else
+  mti->thread_cpu.routine = 0;
+#endif
+  if (!mti->thread_cpu.routine || !my_timer_thread_cpu()) {
+    mti->thread_cpu.routine = 0;
+    mti->thread_cpu.resolution = 0;
+    mti->thread_cpu.frequency = 0;
+    mti->thread_cpu.overhead = 0;
+  }
+
   /*
     Calculate overhead in terms of the timer that
     gives the best resolution: cycles or nanoseconds.
@@ -629,6 +647,9 @@ void my_timer_init(MY_TIMER_INFO *mti) {
   if (mti->ticks.routine)
     my_timer_init_overhead(&mti->ticks.overhead, best_timer, &my_timer_ticks,
                            best_timer_overhead);
+  if (mti->thread_cpu.routine)
+    my_timer_init_overhead(&mti->thread_cpu.overhead, best_timer,
+                           &my_timer_thread_cpu, best_timer_overhead);
 
   /*
     Calculate resolution for nanoseconds or microseconds
@@ -647,6 +668,9 @@ void my_timer_init(MY_TIMER_INFO *mti) {
     mti->milliseconds.resolution =
         my_timer_init_resolution(&my_timer_milliseconds, 0);
   if (mti->ticks.routine) mti->ticks.resolution = 1;
+  if (mti->thread_cpu.routine)
+    mti->thread_cpu.resolution =
+        my_timer_init_resolution(&my_timer_thread_cpu, 20000);
 
   /*
     Calculate cycles frequency,
@@ -665,17 +689,26 @@ void my_timer_init(MY_TIMER_INFO *mti) {
         mti->microseconds.resolution > 100)
       mti->cycles.frequency = mti->microseconds.frequency;
     else {
-      ulonglong time1, time2, lowest;
-      time1 = my_timer_init_frequency(mti);
-      /* Repeat once in case there was an interruption. */
-      time2 = my_timer_init_frequency(mti);
-
-      lowest = 0;
-      if (time1 != 0) {
-        lowest = time1;
-      }
-      if ((time2 != 0) && (time2 < lowest)) {
-        lowest = time2;
+      ulonglong lowest = 0;
+      int values = 0;
+      /*
+        We measure frequency for 200us, there is some probability we will end up
+        switched off the cpu entirely, or be switched to different CPU that has
+        different values of tick. We measure till we have 3 values that are less
+        than 1THz, as a higher value would yield 0 ticks per picosecond leading
+        to useless calculations later on, and in itself look incorrect. We give
+        up after taking 10 readouts.
+      */
+      for (int i = 10; i-- > 0;) {
+        const auto freq = my_timer_init_frequency(mti);
+        if (freq != 0 && freq <= 1'000'000'000'000) {
+          if (lowest == 0 || freq < lowest) {
+            lowest = freq;
+          }
+          if (++values >= 3) {
+            break;
+          }
+        }
       }
 
       mti->cycles.frequency = lowest;
@@ -727,6 +760,29 @@ void my_timer_init(MY_TIMER_INFO *mti) {
     mti->ticks.frequency =
         (mti->cycles.frequency * (time3 - time2)) / (time4 - time1);
   }
+
+  /*
+    Calculate thread_cpu.frequency =
+    (cycles-frequency/#-of-cycles * #-of-thread_cpu,
+    if we have both a thread_cpu routine and a cycles routine.
+
+    The 'frequency' of the thread cpu timer is ill defined,
+    as this timer is not progressing if the thread is not running.
+  */
+  if (mti->thread_cpu.routine && mti->cycles.routine) {
+    int i;
+    ulonglong time1, time2, time3, time4;
+    time1 = my_timer_cycles();
+    time2 = my_timer_thread_cpu();
+    time3 = time2;
+    for (i = 0; i < MY_TIMER_ITERATIONS * 1000; ++i) {
+      time3 = my_timer_thread_cpu();
+      if (time3 - time2 > 10) break;
+    }
+    time4 = my_timer_cycles();
+    mti->thread_cpu.frequency =
+        (mti->cycles.frequency * (time3 - time2)) / (time4 - time1);
+  }
 }
 
 /*
@@ -873,10 +929,10 @@ void my_timer_init(MY_TIMER_INFO *mti) {
    Any clock-based timer can be affected by NPT (ntpd program),
    which means:
    - full-second correction can occur for leap second
-   - tiny corrections can occcur approimately every 11 minutes
-     (but I think they only affect the RTC which isn't the PIT).
+   - tiny corrections can occur approimately every 11 minutes
+     (but they only affect the RTC which isn't the PIT).
 
-   We define "precision" as "frequency" and "high precision" is
+   We define "precision" as "frequency" and "high precision" as
    "frequency better than 1 microsecond". We define "resolution"
    as a synonym for "granularity". We define "accuracy" as
    "closeness to the truth" as established by some authoritative
