@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2004, 2025, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
@@ -41,23 +41,29 @@
    @page LOW_LEVEL_FORMATS Low-level memory and disk formats
 
    - @subpage datetime_and_date_low_level_rep
-   - @subpage time_low_level_rep
- */
+*/
 
 #include "my_time.h"
 
-#include <assert.h>   // assert
+#include <sys/types.h>
+#include <time.h>
 #include <algorithm>  // std::max
-#include <cctype>     // std::isspace
-#include <climits>    // UINT_MAX
-#include <cstdio>     // std::sprintf
-#include <cstring>    // std::memset
+#include <cassert>
+#include <cctype>   // std::isspace
+#include <climits>  // UINT_MAX
+#include <cstdint>
+#include <cstdio>   // std::sprintf
+#include <cstdlib>  // std::for abs
+#include <cstring>  // std::memset
 
 #include "field_types.h"     // enum_field_types
 #include "integer_digits.h"  // count_digits, write_digits, write_two_digits
 #include "my_byteorder.h"    // int3store
-#include "my_systime.h"      // localtime_r
-#include "myisampack.h"      // mi_int2store
+#include "my_inttypes.h"     // longlong, ulonglong, uchar
+#include "my_systime.h"      // IWYU pragma: keep localtime_r
+#include "my_time_t.h"
+#include "myisampack.h"  // mi_int2store
+#include "mysql_time.h"
 #include "template_utils.h"  // pointer_cast
 
 const ulonglong log_10_int[20] = {1,
@@ -193,10 +199,11 @@ bool check_date(const MYSQL_TIME &my_time, bool not_zero_date,
         (my_time.month == 0 || my_time.day == 0)) {
       *was_cut = MYSQL_TIME_WARN_ZERO_IN_DATE;
       return true;
-    } else if ((!(flags & TIME_INVALID_DATES) && my_time.month &&
-                my_time.day > days_in_month[my_time.month - 1] &&
-                (my_time.month != 2 || calc_days_in_year(my_time.year) != 366 ||
-                 my_time.day != 29))) {
+    }
+    if ((!(flags & TIME_INVALID_DATES) && my_time.month &&
+         my_time.day > days_in_month[my_time.month - 1] &&
+         (my_time.month != 2 || calc_days_in_year(my_time.year) != 366 ||
+          my_time.day != 29))) {
       *was_cut = MYSQL_TIME_WARN_OUT_OF_RANGE;
       return true;
     }
@@ -233,14 +240,13 @@ bool check_time_mmssff_range(const MYSQL_TIME &my_time) {
   @retval        true if value is out of range.
 */
 bool check_time_range_quick(const MYSQL_TIME &my_time) {
-  longlong hour = static_cast<longlong>(my_time.hour) + 24LL * my_time.day;
+  const longlong hour =
+      static_cast<longlong>(my_time.hour) + 24LL * my_time.day;
   /* The input value should not be fatally bad */
   assert(!check_time_mmssff_range(my_time));
-  if (hour <= TIME_MAX_HOUR &&
-      (hour != TIME_MAX_HOUR || my_time.minute != TIME_MAX_MINUTE ||
-       my_time.second != TIME_MAX_SECOND || !my_time.second_part))
-    return false;
-  return true;
+  return !(hour <= TIME_MAX_HOUR &&
+           (hour != TIME_MAX_HOUR || my_time.minute != TIME_MAX_MINUTE ||
+            my_time.second != TIME_MAX_SECOND || !my_time.second_part));
 }
 
 /**
@@ -254,6 +260,10 @@ bool check_datetime_range(const MYSQL_TIME &my_time) {
     In case of MYSQL_TIMESTAMP_TIME hour value can be up to TIME_MAX_HOUR.
     In case of MYSQL_TIMESTAMP_DATETIME it cannot be bigger than 23.
   */
+  assert(my_time.time_type == MYSQL_TIMESTAMP_TIME ||
+         my_time.time_type == MYSQL_TIMESTAMP_DATE ||
+         my_time.time_type == MYSQL_TIMESTAMP_DATETIME ||
+         my_time.time_type == MYSQL_TIMESTAMP_DATETIME_TZ);
   return my_time.year > 9999U || my_time.month > 12U || my_time.day > 31U ||
          my_time.minute > 59U || my_time.second > 59U ||
          my_time.second_part > 999999U ||
@@ -278,18 +288,18 @@ bool time_zone_displacement_to_seconds(const char *str, size_t length,
                                        int *result) {
   if (length < 6) return true;
 
-  int sign = str[0] == '+' ? 1 : (str[0] == '-' ? -1 : 0);
+  const int sign = str[0] == '+' ? 1 : (str[0] == '-' ? -1 : 0);
   if (sign == 0) return true;
 
   if (!(std::isdigit(str[1]) && std::isdigit(str[2]))) return true;
-  int hours = (str[1] - '0') * 10 + str[2] - '0';
+  const int hours = (str[1] - '0') * 10 + str[2] - '0';
 
   if (str[3] != ':') return true;
 
   if (!(std::isdigit(str[4]) && std::isdigit(str[5]))) return true;
-  int minutes = (str[4] - '0') * 10 + str[5] - '0';
+  const int minutes = (str[4] - '0') * 10 + str[5] - '0';
   if (minutes >= MINS_PER_HOUR) return true;
-  int seconds = hours * SECS_PER_HOUR + minutes * SECS_PER_MIN;
+  const int seconds = hours * SECS_PER_HOUR + minutes * SECS_PER_MIN;
 
   if (seconds > MAX_TIME_ZONE_HOURS * SECS_PER_HOUR) return true;
 
@@ -434,7 +444,7 @@ bool str_to_datetime(const char *const str_arg, std::size_t length,
     2003-03-03 20:00:20.44
   */
 
-  ulong allow_space = ((1 << 2) | (1 << 6));
+  const ulong allow_space = ((1 << 2) | (1 << 6));
 
   not_zero_date = 0;
   uint i;
@@ -450,7 +460,7 @@ bool str_to_datetime(const char *const str_arg, std::size_t length,
       zeroes are significant, and where we never process more than six
       digits.
     */
-    bool scan_until_delim = !is_internal_format && (i != 6);
+    const bool scan_until_delim = !is_internal_format && (i != 6);
 
     while (str != end && isdigit_char(str[0]) &&
            (scan_until_delim || --field_length)) {
@@ -687,11 +697,10 @@ bool str_to_datetime(const char *const str_arg, std::size_t length,
       status->warnings = MYSQL_TIME_WARN_TRUNCATED;
       l_time->time_type = MYSQL_TIMESTAMP_NONE;
       return true;
-    } else {
-      l_time->time_type = MYSQL_TIMESTAMP_DATETIME_TZ;
-      l_time->time_zone_displacement = displacement;
-      return false;
     }
+    l_time->time_type = MYSQL_TIMESTAMP_DATETIME_TZ;
+    l_time->time_zone_displacement = displacement;
+    return false;
   }
 
   for (; str != end; str++) {
@@ -837,7 +846,7 @@ bool str_to_time(const char *str, std::size_t length, MYSQL_TIME *l_time,
   if (state != 4) { /* Not HH:MM:SS */
     /* Fix the date to assume that seconds was given */
     if (!found_hours && !found_days) {
-      std::size_t len = sizeof(long) * (state - 1);
+      const std::size_t len = sizeof(long) * (state - 1);
       memmove(pointer_cast<uchar *>(date + 4) - len,
               pointer_cast<uchar *>(date + state) - len, len);
       memset(date, 0, sizeof(long) * (4 - state));
@@ -905,7 +914,7 @@ fractional:
 
   if (check_time_mmssff_range(*l_time)) {
     status->warnings |= MYSQL_TIME_WARN_OUT_OF_RANGE;
-    l_time->time_type = MYSQL_TIMESTAMP_ERROR;
+    set_zero_time(l_time, MYSQL_TIMESTAMP_ERROR);
     return true;
   }
 
@@ -923,10 +932,10 @@ fractional:
           return true;
         }
         break;
-      } else {
-        status->set_deprecation(MYSQL_TIME_STATUS::DEPRECATION::DP_SUPERFLUOUS,
-                                str_arg, end, str);
       }
+      status->set_deprecation(MYSQL_TIME_STATUS::DEPRECATION::DP_SUPERFLUOUS,
+                              str_arg, end, str);
+
     } while (++str != end);
   }
   return false;
@@ -946,14 +955,15 @@ bool number_to_time(longlong nr, MYSQL_TIME *ltime, int *warnings) {
     /* For huge numbers try full DATETIME, like str_to_time does. */
     if (nr >= 10000000000LL) /* '0001-00-00 00-00-00' */
     {
-      int warnings_backup = *warnings;
+      const int warnings_backup = *warnings;
       if (number_to_datetime(nr, ltime, 0, warnings) != -1LL) return false;
       *warnings = warnings_backup;
     }
     set_max_time(ltime, false);
     *warnings |= MYSQL_TIME_WARN_OUT_OF_RANGE;
     return true;
-  } else if (nr < -TIME_MAX_VALUE) {
+  }
+  if (nr < -TIME_MAX_VALUE) {
     set_max_time(ltime, true);
     *warnings |= MYSQL_TIME_WARN_OUT_OF_RANGE;
     return true;
@@ -989,6 +999,11 @@ void adjust_time_range(MYSQL_TIME *my_time, int *warning) {
     set_max_hhmmss(my_time);
     *warning |= MYSQL_TIME_WARN_OUT_OF_RANGE;
   }
+  // -00:00:00 is not valid:
+  if (my_time->neg && my_time->hour == 0 && my_time->minute == 0 &&
+      my_time->second == 0 && my_time->second_part == 0) {
+    my_time->neg = false;
+  }
 }
 
 /**
@@ -1148,7 +1163,8 @@ my_time_t my_system_gmt_sec(const MYSQL_TIME &my_time, my_time_t *my_timezone,
   */
   if (t->year > MYTIME_MAX_YEAR) {
     return 0;  // out of range
-  } else if ((t->year == MYTIME_MAX_YEAR) && (t->month == 1) && (t->day > 4)) {
+  }
+  if ((t->year == MYTIME_MAX_YEAR) && (t->month == 1) && (t->day > 4)) {
     /*
       Below we will pass static_cast<uint>(t->day - shift) to calc_daynr.
       As we don't want to get an overflow here, we will shift
@@ -1162,9 +1178,10 @@ my_time_t my_system_gmt_sec(const MYSQL_TIME &my_time, my_time_t *my_timezone,
       calc_daynr(static_cast<uint>(t->year), static_cast<uint>(t->month),
                  static_cast<uint>(t->day));
   tmp_days = tmp_days - static_cast<my_time_t>(days_at_timestart);
-  my_time_t tmp_seconds = tmp_days * SECONDS_IN_24H +
-                          (static_cast<int64_t>(t->hour) * 3600 +
-                           static_cast<int64_t>(t->minute * 60 + t->second));
+  const my_time_t tmp_seconds =
+      tmp_days * SECONDS_IN_24H +
+      (static_cast<int64_t>(t->hour) * 3600 +
+       static_cast<int64_t>(t->minute * 60 + t->second));
   // This will be a narrowing on 32 bit time platforms, but checked range above
   tmp = static_cast<time_t>(tmp_seconds + my_time_zone - 3600);
 
@@ -1368,7 +1385,7 @@ int my_datetime_to_str(const MYSQL_TIME &my_time, char *to, uint dec) {
   int len = TIME_to_datetime_str(my_time, to);
   if (dec) len += my_useconds_to_str(to + len, my_time.second_part, dec);
   if (my_time.time_type == MYSQL_TIMESTAMP_DATETIME_TZ) {
-    int tzd = my_time.time_zone_displacement;
+    const int tzd = my_time.time_zone_displacement;
     len += sprintf(to + len, "%+02i:%02i", tzd / SECS_PER_HOUR,
                    std::abs(tzd) / SECS_PER_MIN % MINS_PER_HOUR);
   } else
@@ -1659,177 +1676,6 @@ ulonglong TIME_to_ulonglong_time_round(const MYSQL_TIME &my_time) {
   return TIME_to_ulonglong_time(tmp);
 }
 
-/**
-   @page time_low_level_rep TIME
-
-  In-memory format:
-
-| Bits  | Field         | Value range |
-| ----: | :----         | :---- |
-|   1   | sign          |(Used for sign, when on disk) |
-|   1   | unused        |(Reserved for wider hour range, e.g. for intervals) |
-|   10  | hour          |(0-838) |
-|   6   | minute        |(0-59) |
-|   6   | second        |(0-59) |
-|  24   | microseconds  |(0-999999) |
-
- Total: 48 bits = 6 bytes
-
-@verbatim
-Format: Suhhhhhh.hhhhmmmm.mmssssss.ffffffff.ffffffff.ffffffff
-@endverbatim
-*/
-
-/**
-  Convert time value to numeric packed representation.
-
-  @param    my_time The value to convert.
-  @return           Numeric packed representation.
-*/
-longlong TIME_to_longlong_time_packed(const MYSQL_TIME &my_time) {
-  /* If month is 0, we mix day with hours: "1 00:10:10" -> "24:00:10" */
-  long hms = (((my_time.month ? 0 : my_time.day * 24) + my_time.hour) << 12) |
-             (my_time.minute << 6) | my_time.second;
-  longlong tmp = my_packed_time_make(hms, my_time.second_part);
-  return my_time.neg ? -tmp : tmp;
-}
-
-/**
-  Convert time packed numeric representation to time.
-
-  @param [out] ltime  The MYSQL_TIME variable to set.
-  @param      tmp    The packed numeric representation.
-*/
-void TIME_from_longlong_time_packed(MYSQL_TIME *ltime, longlong tmp) {
-  longlong hms;
-  if ((ltime->neg = (tmp < 0))) tmp = -tmp;
-  hms = my_packed_time_get_int_part(tmp);
-  ltime->year = static_cast<uint>(0);
-  ltime->month = static_cast<uint>(0);
-  ltime->day = static_cast<uint>(0);
-  ltime->hour =
-      static_cast<uint>(hms >> 12) % (1 << 10); /* 10 bits starting at 12th */
-  ltime->minute =
-      static_cast<uint>(hms >> 6) % (1 << 6); /* 6 bits starting at 6th   */
-  ltime->second =
-      static_cast<uint>(hms) % (1 << 6); /* 6 bits starting at 0th   */
-  ltime->second_part = my_packed_time_get_frac_part(tmp);
-  ltime->time_type = MYSQL_TIMESTAMP_TIME;
-}
-
-/**
-  On disk we convert from signed representation to unsigned
-  representation using TIMEF_OFS, so all values become binary comparable.
-*/
-#define TIMEF_OFS 0x800000000000LL
-#define TIMEF_INT_OFS 0x800000LL
-
-/**
-  Convert in-memory numeric time representation to on-disk representation
-
-  @param       nr   Value in packed numeric time format.
-  @param [out] ptr  The buffer to put value at.
-  @param       dec  Precision.
-*/
-void my_time_packed_to_binary(longlong nr, uchar *ptr, uint dec) {
-  assert(dec <= DATETIME_MAX_DECIMALS);
-  /* Make sure the stored value was previously properly rounded or truncated */
-  assert((my_packed_time_get_frac_part(nr) %
-          static_cast<int>(log_10_int[DATETIME_MAX_DECIMALS - dec])) == 0);
-
-  switch (dec) {
-    case 0:
-    default:
-      mi_int3store(ptr, TIMEF_INT_OFS + my_packed_time_get_int_part(nr));
-      break;
-
-    case 1:
-    case 2:
-      mi_int3store(ptr, TIMEF_INT_OFS + my_packed_time_get_int_part(nr));
-      ptr[3] = static_cast<unsigned char>(
-          static_cast<char>(my_packed_time_get_frac_part(nr) / 10000));
-      break;
-
-    case 4:
-    case 3:
-      mi_int3store(ptr, TIMEF_INT_OFS + my_packed_time_get_int_part(nr));
-      mi_int2store(ptr + 3, my_packed_time_get_frac_part(nr) / 100);
-      break;
-
-    case 5:
-    case 6:
-      mi_int6store(ptr, nr + TIMEF_OFS);
-      break;
-  }
-}
-
-/**
-  Convert on-disk time representation to in-memory packed numeric
-  representation.
-
-  @param   ptr  The pointer to read the value at.
-  @param   dec  Precision.
-  @return       Packed numeric time representation.
-*/
-longlong my_time_packed_from_binary(const uchar *ptr, uint dec) {
-  assert(dec <= DATETIME_MAX_DECIMALS);
-
-  switch (dec) {
-    case 0:
-    default: {
-      longlong intpart = mi_uint3korr(ptr) - TIMEF_INT_OFS;
-      return my_packed_time_make_int(intpart);
-    }
-    case 1:
-    case 2: {
-      longlong intpart = mi_uint3korr(ptr) - TIMEF_INT_OFS;
-      int frac = static_cast<uint>(ptr[3]);
-      if (intpart < 0 && frac) {
-        /*
-           Negative values are stored with
-           reverse fractional part order,
-           for binary sort compatibility.
-
-            Disk value  intpart frac   Time value   Memory value
-            800000.00    0      0      00:00:00.00  0000000000.000000
-            7FFFFF.FF   -1      255   -00:00:00.01  FFFFFFFFFF.FFD8F0
-            7FFFFF.9D   -1      99    -00:00:00.99  FFFFFFFFFF.F0E4D0
-            7FFFFF.00   -1      0     -00:00:01.00  FFFFFFFFFF.000000
-            7FFFFE.FF   -1      255   -00:00:01.01  FFFFFFFFFE.FFD8F0
-            7FFFFE.F6   -2      246   -00:00:01.10  FFFFFFFFFE.FE7960
-
-            Formula to convert fractional part from disk format
-            (now stored in "frac" variable) to absolute value: "0x100 - frac".
-            To reconstruct in-memory value, we shift
-            to the next integer value and then substruct fractional part.
-        */
-        intpart++;     /* Shift to the next integer value */
-        frac -= 0x100; /* -(0x100 - frac) */
-      }
-      return my_packed_time_make(intpart, frac * 10000);
-    }
-
-    case 3:
-    case 4: {
-      longlong intpart = mi_uint3korr(ptr) - TIMEF_INT_OFS;
-      int frac = mi_uint2korr(ptr + 3);
-      if (intpart < 0 && frac) {
-        /*
-          Fix reverse fractional part order: "0x10000 - frac".
-          See comments for FSP=1 and FSP=2 above.
-        */
-        intpart++;       /* Shift to the next integer value */
-        frac -= 0x10000; /* -(0x10000-frac) */
-      }
-      return my_packed_time_make(intpart, frac * 100);
-    }
-
-    case 5:
-    case 6:
-      return (static_cast<longlong>(mi_uint6korr(ptr))) - TIMEF_OFS;
-  }
-}
-
 /**
    @page datetime_and_date_low_level_rep DATETIME and DATE
 
@@ -1858,9 +1704,11 @@ Format: SYYYYYYY.YYYYYYYY.YYdddddh.hhhhmmmm.mmssssss.ffffffff.ffffffff.ffffffff
   @return       Packed numeric representation of my_time.
 */
 longlong TIME_to_longlong_datetime_packed(const MYSQL_TIME &my_time) {
-  longlong ymd = ((my_time.year * 13 + my_time.month) << 5) | my_time.day;
-  longlong hms = (my_time.hour << 12) | (my_time.minute << 6) | my_time.second;
-  longlong tmp = my_packed_time_make(((ymd << 17) | hms), my_time.second_part);
+  const longlong ymd = ((my_time.year * 13 + my_time.month) << 5) | my_time.day;
+  const longlong hms =
+      (my_time.hour << 12) | (my_time.minute << 6) | my_time.second;
+  const longlong tmp =
+      my_packed_time_make(((ymd << 17) | hms), my_time.second_part);
   assert(!check_datetime_range(my_time)); /* Make sure no overflow */
   return my_time.neg ? -tmp : tmp;
 }
@@ -1874,7 +1722,7 @@ longlong TIME_to_longlong_datetime_packed(const MYSQL_TIME &my_time) {
   @return      Packed numeric representation of ltime.
 */
 longlong TIME_to_longlong_date_packed(const MYSQL_TIME &my_time) {
-  longlong ymd = ((my_time.year * 13 + my_time.month) << 5) | my_time.day;
+  const longlong ymd = ((my_time.year * 13 + my_time.month) << 5) | my_time.day;
   return my_packed_time_make_int(ymd << 17);
 }
 
@@ -1885,7 +1733,7 @@ longlong TIME_to_longlong_date_packed(const MYSQL_TIME &my_time) {
   @return packed value for date YYYY-00-00.
 */
 longlong year_to_longlong_datetime_packed(long year) {
-  longlong ymd = ((year * 13) << 5);
+  const longlong ymd = ((year * 13) << 5);
   return my_packed_time_make_int(ymd << 17);
 }
 
@@ -1948,7 +1796,7 @@ void TIME_from_longlong_date_packed(MYSQL_TIME *ltime, longlong tmp) {
   @return      In-memory packed numeric datetime representation.
 */
 longlong my_datetime_packed_from_binary(const uchar *ptr, uint dec) {
-  longlong intpart = mi_uint5korr(ptr) - DATETIMEF_INT_OFS;
+  const longlong intpart = mi_uint5korr(ptr) - DATETIMEF_INT_OFS;
   int frac;
   assert(dec <= DATETIME_MAX_DECIMALS);
   switch (dec) {
@@ -2074,7 +1922,7 @@ void my_timestamp_to_binary(const my_timeval *tm, uchar *ptr, uint dec) {
   @param [out]  ptr   The pointer to store the value to.
 */
 void my_date_to_binary(const MYSQL_TIME *ltime, uchar *ptr) {
-  long tmp = ltime->day + ltime->month * 32 + ltime->year * 16 * 32;
+  const long tmp = ltime->day + ltime->month * 32 + ltime->year * 16 * 32;
   int3store(ptr, tmp);
 }
 
@@ -2086,6 +1934,8 @@ void my_date_to_binary(const MYSQL_TIME *ltime, uchar *ptr) {
   @return  Packed numeric time/date/datetime representation.
 */
 longlong TIME_to_longlong_packed(const MYSQL_TIME &my_time) {
+  assert(my_time.time_type != MYSQL_TIMESTAMP_TIME);
+
   switch (my_time.time_type) {
     case MYSQL_TIMESTAMP_DATE:
       return TIME_to_longlong_date_packed(my_time);
@@ -2093,11 +1943,12 @@ longlong TIME_to_longlong_packed(const MYSQL_TIME &my_time) {
       assert(false);  // Should not be this type at this point.
     case MYSQL_TIMESTAMP_DATETIME:
       return TIME_to_longlong_datetime_packed(my_time);
-    case MYSQL_TIMESTAMP_TIME:
-      return TIME_to_longlong_time_packed(my_time);
     case MYSQL_TIMESTAMP_NONE:
     case MYSQL_TIMESTAMP_ERROR:
       return 0;
+    default:
+      assert(false);
+      return 0;
   }
   assert(false);
   return 0;
@@ -2197,11 +2048,11 @@ int calc_weekday(long daynr, bool sunday_first_day_of_week) {
 */
 uint calc_week(const MYSQL_TIME &my_time, uint week_behaviour, uint *year) {
   uint days;
-  ulong daynr = calc_daynr(my_time.year, my_time.month, my_time.day);
+  const ulong daynr = calc_daynr(my_time.year, my_time.month, my_time.day);
   ulong first_daynr = calc_daynr(my_time.year, 1, 1);
-  bool monday_first = (week_behaviour & WEEK_MONDAY_FIRST);
+  const bool monday_first = (week_behaviour & WEEK_MONDAY_FIRST);
   bool week_year = (week_behaviour & WEEK_YEAR);
-  bool first_weekday = (week_behaviour & WEEK_FIRST_WEEKDAY);
+  const bool first_weekday = (week_behaviour & WEEK_FIRST_WEEKDAY);
 
   uint weekday = calc_weekday(first_daynr, !monday_first);
   *year = my_time.year;
@@ -2282,7 +2133,7 @@ bool date_add_interval(MYSQL_TIME *ltime, interval_type int_type,
                        Interval interval, int *warnings) {
   ltime->neg = false;
 
-  long long sign = (interval.neg ? -1 : 1);
+  const long long sign = (interval.neg ? -1 : 1);
 
   switch (int_type) {
     case INTERVAL_SECOND:
@@ -2533,8 +2384,7 @@ bool time_add_nanoseconds_adjust_frac(MYSQL_TIME *ltime, uint nanoseconds,
                                       int *warnings, bool truncate) {
   if (truncate)
     return time_add_nanoseconds_with_truncate(ltime, nanoseconds, warnings);
-  else
-    return time_add_nanoseconds_with_round(ltime, nanoseconds, warnings);
+  return time_add_nanoseconds_with_round(ltime, nanoseconds, warnings);
 }
 
 /**
@@ -2551,8 +2401,7 @@ bool datetime_add_nanoseconds_adjust_frac(MYSQL_TIME *ltime, uint nanoseconds,
                                           int *warnings, bool truncate) {
   if (truncate)
     return datetime_add_nanoseconds_with_truncate(ltime, nanoseconds);
-  else
-    return datetime_add_nanoseconds_with_round(ltime, nanoseconds, warnings);
+  return datetime_add_nanoseconds_with_round(ltime, nanoseconds, warnings);
 }
 
 /** Rounding functions */
@@ -2572,8 +2421,8 @@ bool my_time_adjust_frac(MYSQL_TIME *ltime, uint dec, bool truncate) {
   int warnings = 0;
   assert(dec <= DATETIME_MAX_DECIMALS);
   /* Add half away from zero */
-  bool rc = time_add_nanoseconds_adjust_frac(ltime, msec_round_add[dec],
-                                             &warnings, truncate);
+  const bool rc = time_add_nanoseconds_adjust_frac(ltime, msec_round_add[dec],
+                                                   &warnings, truncate);
 
   /* Truncate non-significant digits */
   my_time_trunc(ltime, dec);
@@ -2594,8 +2443,8 @@ bool my_datetime_adjust_frac(MYSQL_TIME *ltime, uint dec, int *warnings,
                              bool truncate) {
   assert(dec <= DATETIME_MAX_DECIMALS);
   /* Add half away from zero */
-  bool rc = datetime_add_nanoseconds_adjust_frac(ltime, msec_round_add[dec],
-                                                 warnings, truncate);
+  const bool rc = datetime_add_nanoseconds_adjust_frac(
+      ltime, msec_round_add[dec], warnings, truncate);
   /* Truncate non-significant digits */
   my_time_trunc(ltime, dec);
   return rc;
@@ -2610,7 +2459,7 @@ bool my_datetime_adjust_frac(MYSQL_TIME *ltime, uint dec, int *warnings,
 */
 bool my_timeval_round(struct my_timeval *tv, uint decimals) {
   assert(decimals <= DATETIME_MAX_DECIMALS);
-  uint nanoseconds = msec_round_add[decimals];
+  const uint nanoseconds = msec_round_add[decimals];
   tv->m_tv_usec += (nanoseconds + 500) / 1000;
   if (tv->m_tv_usec < 1000000) goto ret;
 
@@ -2649,7 +2498,7 @@ void mix_date_and_time(MYSQL_TIME *ldate, const MYSQL_TIME &my_time) {
     /* Complex case: TIME is negative or outside of 24 hours internal. */
     longlong seconds;
     long days, useconds;
-    int sign = my_time.neg ? 1 : -1;
+    const int sign = my_time.neg ? 1 : -1;
     ldate->neg = calc_time_diff(*ldate, my_time, sign, &seconds, &useconds);
     assert(!ldate->neg);
 
@@ -2780,8 +2629,8 @@ bool calc_time_diff(const MYSQL_TIME &my_time1, const MYSQL_TIME &my_time2,
    @retval 1 if b comes before a
  */
 int my_time_compare(const MYSQL_TIME &my_time_a, const MYSQL_TIME &my_time_b) {
-  ulonglong a_t = TIME_to_ulonglong_datetime(my_time_a);
-  ulonglong b_t = TIME_to_ulonglong_datetime(my_time_b);
+  const ulonglong a_t = TIME_to_ulonglong_datetime(my_time_a);
+  const ulonglong b_t = TIME_to_ulonglong_datetime(my_time_b);
 
   if (a_t < b_t) return -1;
   if (a_t > b_t) return 1;
@@ -2802,9 +2651,9 @@ int my_time_compare(const MYSQL_TIME &my_time_a, const MYSQL_TIME &my_time_b) {
 */
 longlong TIME_to_longlong_packed(const MYSQL_TIME &my_time,
                                  enum enum_field_types type) {
+  assert(type != MYSQL_TYPE_TIME);
+
   switch (type) {
-    case MYSQL_TYPE_TIME:
-      return TIME_to_longlong_time_packed(my_time);
     case MYSQL_TYPE_DATETIME:
     case MYSQL_TYPE_TIMESTAMP:
       return TIME_to_longlong_datetime_packed(my_time);
@@ -2825,10 +2674,9 @@ longlong TIME_to_longlong_packed(const MYSQL_TIME &my_time,
 */
 void TIME_from_longlong_packed(MYSQL_TIME *ltime, enum enum_field_types type,
                                longlong packed_value) {
+  assert(type != MYSQL_TYPE_TIME);
+
   switch (type) {
-    case MYSQL_TYPE_TIME:
-      TIME_from_longlong_time_packed(ltime, packed_value);
-      break;
     case MYSQL_TYPE_DATE:
       TIME_from_longlong_date_packed(ltime, packed_value);
       break;
@@ -2855,10 +2703,10 @@ void TIME_from_longlong_packed(MYSQL_TIME *ltime, enum enum_field_types type,
 longlong longlong_from_datetime_packed(enum enum_field_types type,
                                        longlong packed_value) {
   MYSQL_TIME ltime;
+
+  assert(type != MYSQL_TYPE_TIME);
+
   switch (type) {
-    case MYSQL_TYPE_TIME:
-      TIME_from_longlong_time_packed(&ltime, packed_value);
-      return TIME_to_ulonglong_time(ltime);
     case MYSQL_TYPE_DATE:
       TIME_from_longlong_date_packed(&ltime, packed_value);
       return TIME_to_ulonglong_date(ltime);
@@ -2884,7 +2732,7 @@ longlong longlong_from_datetime_packed(enum enum_field_types type,
 */
 double double_from_datetime_packed(enum enum_field_types type,
                                    longlong packed_value) {
-  longlong result = longlong_from_datetime_packed(type, packed_value);
+  const longlong result = longlong_from_datetime_packed(type, packed_value);
   return result +
          (static_cast<double>(my_packed_time_get_frac_part(packed_value))) /
              1000000;
