@@ -1,4 +1,4 @@
-/*  Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/*  Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License, version 2.0,
@@ -23,9 +23,9 @@
 
 #include <mysql/client_plugin.h>
 #include <mysql/plugin_auth.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
 #include "my_compiler.h"
 
@@ -56,7 +56,7 @@ static int qa_auth_interface(MYSQL_PLUGIN_VIO *vio,
   info->password_used = PASSWORD_USED_YES;
 
   /* fail if the password is wrong */
-  if (strcmp((const char *)pkt, info->auth_string)) return CR_ERROR;
+  if (strcmp((const char *)pkt, info->auth_string) != 0) return CR_ERROR;
 
   /* Test of default_auth */
   if (strcmp(info->user_name, "qa_test_11_user") == 0) {
