@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,9 +45,11 @@
 #include "my_sys.h"
 #include "plugin/group_replication/include/gcs_plugin_messages.h"
 #include "plugin/group_replication/include/member_version.h"
+#include "plugin/group_replication/include/plugin_constants.h"
 #include "plugin/group_replication/include/plugin_psi.h"
 #include "plugin/group_replication/include/services/notification/notification.h"
 #include "plugin/group_replication/libmysqlgcs/include/mysql/gcs/gcs_member_identifier.h"
+#include "sql/malloc_allocator.h"
 
 /*
   Encoding of the group_replication_enforce_update_everywhere_checks
@@ -141,8 +144,26 @@ class Group_member_info : public Plugin_gcs_message {
     // Length of the payload item: variable
     PIT_RECOVERY_ENDPOINTS = 20,
 
+    // Length of the payload item: variable
+    PIT_VIEW_CHANGE_UUID = 21,
+
+    // Length of the paylod item: 1 byte
+    PIT_ALLOW_SINGLE_LEADER = 22,
+
+    // Length of the paylod item: variable
+    PIT_GROUP_ACTION_RUNNING_NAME = 23,
+
+    // Length of the paylod item: variable
+    PIT_GROUP_ACTION_RUNNING_DESCRIPTION = 24,
+
+    // Length of the paylod item: 1 byte
+    PIT_PREEMPTIVE_GARBAGE_COLLECTION = 25,
+
+    // Length of the paylod item: 1 byte
+    PIT_COMPONENT_PRIMARY_ELECTION_ENABLED = 26,
+
     // No valid type codes can appear after this one.
-    PIT_MAX = 21
+    PIT_MAX = 27
   };
 
   /*
@@ -171,6 +192,83 @@ class Group_member_info : public Plugin_gcs_message {
     MEMBER_ROLE_END
   } Group_member_role;
 
+  /*
+    Values of the old transaction_write_set_extraction sysvar.
+    This enum is only used on Group Replication plugin for
+    compatibility purposes.
+  */
+  enum enum_transaction_write_set_hashing_algorithm_compatibility {
+    HASH_ALGORITHM_OFF = 0,
+    HASH_ALGORITHM_MURMUR32 = 1,
+    HASH_ALGORITHM_XXHASH64 = 2
+  };
+
+  /*
+    Allocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] size    memory size to be allocated
+    @param[in] nothrow When the nothrow constant is passed as second parameter
+                       to operator new, operator new returns a null-pointer on
+                       failure instead of throwing a bad_alloc exception.
+
+    @return pointer to the allocated memory, or NULL if memory could not
+            be allocated.
+  */
+  void *operator new(size_t size, const std::nothrow_t &) noexcept {
+    /*
+      Call my_malloc() with the MY_WME flag to make sure that it will
+      write an error message if the memory could not be allocated.
+    */
+    return my_malloc(key_group_member_info, size, MYF(MY_WME));
+  }
+
+  /*
+    Deallocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] ptr     pointer to the allocated memory
+    @param[in] nothrow When the nothrow constant is passed as second parameter
+                       to operator new, operator new returns a null-pointer on
+                       failure instead of throwing a bad_alloc exception.
+  */
+  void operator delete(void *ptr, const std::nothrow_t &) noexcept {
+    my_free(ptr);
+  }
+
+  /**
+    Allocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] size    memory size to be allocated
+
+    @return pointer to the allocated memory, or NULL if memory could not
+            be allocated.
+  */
+  void *operator new(size_t size) noexcept {
+    /*
+      Call my_malloc() with the MY_WME flag to make sure that it will
+      write an error message if the memory could not be allocated.
+    */
+    return my_malloc(key_group_member_info, size, MYF(MY_WME));
+  }
+
+  /**
+    Deallocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] ptr     pointer to the allocated memory
+  */
+  void operator delete(void *ptr) noexcept { my_free(ptr); }
+
+  /**
+    Group_member_info empty constructor
+
+    @param[in] psi_mutex_key_arg                      mutex key
+   */
+  Group_member_info(PSI_mutex_key psi_mutex_key_arg =
+                        key_GR_LOCK_group_member_info_update_lock);
+
   /**
     Group_member_info constructor
 
@@ -192,10 +290,17 @@ class Group_member_info : public Plugin_gcs_message {
     check
     @param[in] member_weight_arg                      member_weight
     @param[in] lower_case_table_names_arg             lower case table names
-    @param[in] psi_mutex_key_arg                      mutex key
     @param[in] default_table_encryption_arg           default_table_encryption
     @param[in] recovery_endpoints_arg                 recovery endpoints
+    @param[in] view_change_uuid_arg                   view change uuid
     advertised
+    @param[in] allow_single_leader                    flag indicating whether or
+    @param[in] preemptive_garbage_collection          member's
+    group_replication_preemptive_garbage_collection value
+    not to use single-leader behavior
+    @param[in] component_primary_election_enabled      member's has enabled
+    primary election component
+    @param[in] psi_mutex_key_arg                      mutex key
    */
   Group_member_info(const char *hostname_arg, uint port_arg,
                     const char *uuid_arg, int write_set_extraction_algorithm,
@@ -209,6 +314,9 @@ class Group_member_info : public Plugin_gcs_message {
                     uint member_weight_arg, uint lower_case_table_names_arg,
                     bool default_table_encryption_arg,
                     const char *recovery_endpoints_arg,
+                    const char *view_change_uuid_arg, bool allow_single_leader,
+                    bool preemptive_garbage_collection,
+                    bool component_primary_election_enabled,
                     PSI_mutex_key psi_mutex_key_arg =
                         key_GR_LOCK_group_member_info_update_lock);
 
@@ -259,6 +367,13 @@ class Group_member_info : public Plugin_gcs_message {
     @param[in] default_table_encryption_arg           default table encryption
     @param[in] recovery_endpoints_arg                 recovery endpoints
     advertised
+    @param[in] view_change_uuid_arg                   view change uuid
+    @param[in] allow_single_leader                    flag indicating whether or
+    not to use single-leader behavior
+    @param[in] preemptive_garbage_collection          member's
+    group_replication_preemptive_garbage_collection value
+    @param[in] component_primary_election_enabled      member's has enabled
+    primary election component
    */
   void update(const char *hostname_arg, uint port_arg, const char *uuid_arg,
               int write_set_extraction_algorithm,
@@ -271,7 +386,10 @@ class Group_member_info : public Plugin_gcs_message {
               bool has_enforces_update_everywhere_checks,
               uint member_weight_arg, uint lower_case_table_names_arg,
               bool default_table_encryption_arg,
-              const char *recovery_endpoints_arg);
+              const char *recovery_endpoints_arg,
+              const char *view_change_uuid_arg, bool allow_single_leader,
+              bool preemptive_garbage_collection,
+              bool component_primary_election_enabled);
 
   /**
     Update Group_member_info.
@@ -340,6 +458,11 @@ class Group_member_info : public Plugin_gcs_message {
   */
   uint get_write_set_extraction_algorithm();
 
+  /**
+    @return the member algorithm name for extracting write sets
+  */
+  const char *get_write_set_extraction_algorithm_name();
+
   /**
     @return the member gtid assignment block size
   */
@@ -543,6 +666,70 @@ class Group_member_info : public Plugin_gcs_message {
    */
   void set_recovery_endpoints(const char *endpoints);
 
+  /**
+    Get UID used when logging view change events
+    @return view change uuid or "AUTOMATIC"
+   */
+  std::string get_view_change_uuid();
+
+  bool get_allow_single_leader();
+
+  /**
+    Get group action name if running on the member.
+    Refer group_action_running to check if any action is running.
+    @return group action name if running on the member
+   */
+  const std::string &get_group_action_running_name();
+
+  /**
+    Set group action name if running on the member.
+    Refer group_action_running to check if any action is running.
+    @param group_action_running_name set group action name
+   */
+  void set_group_action_running_name(
+      const std::string &group_action_running_name);
+
+  /**
+    Get group action description if running on the member.
+    Refer group_action_running to check if any action is running.
+    @return group action description if running on the member
+   */
+  const std::string &get_group_action_running_description();
+
+  /**
+    Set group action description if running on the member.
+    Refer group_action_running to check if any action is running.
+    @param group_action_running_description set group action description
+   */
+  void set_group_action_running_description(
+      const std::string &group_action_running_description);
+
+  /**
+    Save member view change uuid
+    @param view_change_cnf uuid to be used on change views or "AUTOMATIC"
+   */
+  void set_view_change_uuid(const char *view_change_cnf);
+
+  /**
+    Get the value of 'group_replication_preemptive_garbage_collection'
+    option on this member.
+
+    @return true  enabled
+            false disabled
+  */
+  bool get_preemptive_garbage_collection();
+
+  /**
+    Get value of variable enabled from component
+    'group_replication_primary_election' on this member.
+
+    @return true  enabled
+            false disabled
+  */
+  bool get_component_primary_election_enabled();
+
+  void set_component_primary_election_enabled(bool enabled);
+
  protected:
   void encode_payload(std::vector<unsigned char> *buffer) const override;
   void decode_payload(const unsigned char *buffer,
@@ -568,8 +755,8 @@ class Group_member_info : public Plugin_gcs_message {
   uint port;
   std::string uuid;
   Group_member_status status;
-  Gcs_member_identifier *gcs_member_id;
-  Member_version *member_version;
+  Gcs_member_identifier *gcs_member_id{nullptr};
+  Member_version *member_version{nullptr};
   std::string executed_gtid_set;
   std::string purged_gtid_set;
   std::string retrieved_gtid_set;
@@ -585,14 +772,31 @@ class Group_member_info : public Plugin_gcs_message {
   bool group_action_running;
   bool primary_election_running;
   std::string recovery_endpoints;
+  std::string m_view_change_uuid;
+  bool m_allow_single_leader;
+  std::string m_group_action_running_name;
+  std::string m_group_action_running_description;
+  bool m_preemptive_garbage_collection{PREEMPTIVE_GARBAGE_COLLECTION_DEFAULT};
+  bool m_component_primary_election_enabled{false};
 #ifndef NDEBUG
  public:
   bool skip_encode_default_table_encryption;
+  bool m_skip_encode_view_change_uuid;
 #endif
   // Allow use copy constructor on unit tests.
   PSI_mutex_key psi_mutex_key;
 };
 
+typedef std::vector<Group_member_info *, Malloc_allocator<Group_member_info *>>
+    Group_member_info_list;
+typedef Group_member_info_list::iterator Group_member_info_list_iterator;
+
+typedef std::map<
+    std::string, Group_member_info *, std::less<std::string>,
+    Malloc_allocator<std::pair<const std::string, Group_member_info *>>>
+    Group_member_info_map;
+typedef Group_member_info_map::iterator Group_member_info_map_iterator;
+
 /*
   @interface Group_member_info_manager_interface
 
@@ -602,7 +806,7 @@ class Group_member_info : public Plugin_gcs_message {
  */
 class Group_member_info_manager_interface {
  public:
-  virtual ~Group_member_info_manager_interface() {}
+  virtual ~Group_member_info_manager_interface() = default;
 
   /**
     Number of members in the group.
@@ -630,21 +834,32 @@ class Group_member_info_manager_interface {
   /**
     Retrieves a registered Group member by its uuid
 
-    @param[in] uuid uuid to retrieve
-    @return reference to a copy of Group_member_info. NULL if not managed.
-            The return value must deallocated by the caller.
+    @param[in]  uuid             uuid to retrieve
+    @param[out] member_info_arg  a member info reference local to the
+                                 method caller that is updated when the
+                                 member is found.
+
+    @return true  if the member is not found.
+            false if the member is found.
    */
-  virtual Group_member_info *get_group_member_info(const std::string &uuid) = 0;
+  [[nodiscard]] virtual bool get_group_member_info(
+      const std::string &uuid, Group_member_info &member_info_arg) = 0;
 
   /**
     Retrieves a registered Group member by an index function.
     One is free to determine the index function. Nevertheless, it should have
     the same result regardless of the member of the group where it is called
 
-    @param[in] idx the index
-    @return reference to a Group_member_info. NULL if not managed
+    @param[in]  idx              the index
+    @param[out] member_info_arg  a member info reference local to the
+                                 method caller that is updated when the
+                                 member is found.
+
+    @return true  if the member is not found.
+            false if the member is found.
    */
-  virtual Group_member_info *get_group_member_info_by_index(int idx) = 0;
+  [[nodiscard]] virtual bool get_group_member_info_by_index(
+      int idx, Group_member_info &member_info_arg) = 0;
 
   /**
     Return lowest member version.
@@ -657,11 +872,25 @@ class Group_member_info_manager_interface {
   /**
     Retrieves a registered Group member by its backbone GCS identifier.
 
-    @param[in] id the GCS identifier
-    @return reference to a copy of Group_member_info. NULL if not managed.
-            The return value must be deallocated by the caller.
+    @param[in]  id               the GCS identifier
+    @param[out] member_info_arg  a member info reference local to the
+                                 method caller that is updated when the
+                                 member is found.
+
+    @return true  if the member is not found.
+            false if the member is found.
    */
-  virtual Group_member_info *get_group_member_info_by_member_id(
+  [[nodiscard]] virtual bool get_group_member_info_by_member_id(
+      const Gcs_member_identifier &id, Group_member_info &member_info_arg) = 0;
+
+  /**
+    Returns the member-uuid of the given GCS identifier.
+
+    @param[in] id the GCS identifier
+    @return false if success and the member-uuid
+            true if fails and the member-uuid is empty.
+   */
+  virtual std::pair<bool, std::string> get_group_member_uuid_from_member_id(
       const Gcs_member_identifier &id) = 0;
 
   /**
@@ -679,7 +908,7 @@ class Group_member_info_manager_interface {
 
     @return a vector with copies to all managed Group_member_info
    */
-  virtual std::vector<Group_member_info *> *get_all_members() = 0;
+  virtual Group_member_info_list *get_all_members() = 0;
 
   /**
     Retrieves all ONLINE Group members managed by this site, or
@@ -715,7 +944,7 @@ class Group_member_info_manager_interface {
 
     @param[in] new_members new Group members
    */
-  virtual void update(std::vector<Group_member_info *> *new_members) = 0;
+  virtual void update(Group_member_info_list *new_members) = 0;
 
   /**
     Updates the status of a single member
@@ -813,8 +1042,8 @@ class Group_member_info_manager_interface {
     @param[in] length    raw encoded data length
     @return a vector of Group_member_info references
    */
-  virtual std::vector<Group_member_info *> *decode(const uchar *to_decode,
-                                                   size_t length) = 0;
+  virtual Group_member_info_list *decode(const uchar *to_decode,
+                                         size_t length) = 0;
 
   /**
     Check if some member of the group has the conflict detection enable
@@ -839,11 +1068,15 @@ class Group_member_info_manager_interface {
   /**
     Return the group member info for the current group primary
 
-    @note the returned reference must be deallocated by the caller.
+    @param[out] member_info_arg  a member info reference local to the
+                                 method caller that is updated when the
+                                 member is found.
 
-    @return reference to a Group_member_info. NULL if not managed
+    @return true  if the member is not found.
+            false if the member is found.
   */
-  virtual Group_member_info *get_primary_member_info() = 0;
+  [[nodiscard]] virtual bool get_primary_member_info(
+      Group_member_info &member_info_arg) = 0;
 
   /**
     Check if majority of the group is unreachable
@@ -859,7 +1092,7 @@ class Group_member_info_manager_interface {
     Check if an unreachable member exists
 
     This approach is optimistic, right after return a member can be marked as
-    rechable/unreachable
+    reachable/unreachable
 
     @return true if an unreachable member exists
   */
@@ -888,6 +1121,31 @@ class Group_member_info_manager_interface {
     @return update_lock reference
   */
   virtual mysql_mutex_t *get_update_lock() = 0;
+
+  /**
+  Updates the component primary failover
+
+  @param[in] uuid        member uuid
+  @param[in] enabled     uptodate election enabled
+*/
+  virtual void update_component_primary_election_enabled(
+      const std::string &uuid, bool enabled) = 0;
+
+  /**
+  All members have component primary election enabled.
+
+  @return true       all members have component enabled
+          false      otherwise
+*/
+  virtual bool is_group_replication_elect_prefers_most_updated_enabled() = 0;
+
+  /**
+  Return last timestamp since epoch when a view change updated the group
+  members info.
+
+  @return timestamp of last view change
+*/
+  virtual uint64_t get_timestamp_last_view_change() = 0;
 };
 
 /**
@@ -904,25 +1162,89 @@ class Group_member_info_manager : public Group_member_info_manager_interface {
 
   ~Group_member_info_manager() override;
 
+  /*
+    Allocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] size    memory size to be allocated
+    @param[in] nothrow When the nothrow constant is passed as second parameter
+                       to operator new, operator new returns a null-pointer on
+                       failure instead of throwing a bad_alloc exception.
+
+    @return pointer to the allocated memory, or NULL if memory could not
+            be allocated.
+  */
+  void *operator new(size_t size, const std::nothrow_t &) noexcept {
+    /*
+      Call my_malloc() with the MY_WME flag to make sure that it will
+      write an error message if the memory could not be allocated.
+    */
+    return my_malloc(key_group_member_info, size, MYF(MY_WME));
+  }
+
+  /*
+    Deallocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] ptr     pointer to the allocated memory
+    @param[in] nothrow When the nothrow constant is passed as second parameter
+                       to operator new, operator new returns a null-pointer on
+                       failure instead of throwing a bad_alloc exception.
+  */
+  void operator delete(void *ptr, const std::nothrow_t &) noexcept {
+    my_free(ptr);
+  }
+
+  /**
+    Allocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] size    memory size to be allocated
+
+    @return pointer to the allocated memory, or NULL if memory could not
+            be allocated.
+  */
+  void *operator new(size_t size) noexcept {
+    /*
+      Call my_malloc() with the MY_WME flag to make sure that it will
+      write an error message if the memory could not be allocated.
+    */
+    return my_malloc(key_group_member_info, size, MYF(MY_WME));
+  }
+
+  /**
+    Deallocate memory on the heap with instrumented memory allocation, so
+    that memory consumption can be tracked.
+
+    @param[in] ptr     pointer to the allocated memory
+  */
+  void operator delete(void *ptr) noexcept { my_free(ptr); }
+
   size_t get_number_of_members() override;
 
   size_t get_number_of_members_online() override;
 
   bool is_member_info_present(const std::string &uuid) override;
 
-  Group_member_info *get_group_member_info(const std::string &uuid) override;
+  [[nodiscard]] bool get_group_member_info(
+      const std::string &uuid, Group_member_info &member_info_arg) override;
 
-  Group_member_info *get_group_member_info_by_index(int idx) override;
+  [[nodiscard]] bool get_group_member_info_by_index(
+      int idx, Group_member_info &member_info_arg) override;
 
   Member_version get_group_lowest_online_version() override;
 
-  Group_member_info *get_group_member_info_by_member_id(
+  [[nodiscard]] bool get_group_member_info_by_member_id(
+      const Gcs_member_identifier &id,
+      Group_member_info &member_info_arg) override;
+
+  std::pair<bool, std::string> get_group_member_uuid_from_member_id(
       const Gcs_member_identifier &id) override;
 
   Group_member_info::Group_member_status get_group_member_status_by_member_id(
       const Gcs_member_identifier &id) override;
 
-  std::vector<Group_member_info *> *get_all_members() override;
+  Group_member_info_list *get_all_members() override;
 
   std::list<Gcs_member_identifier> *get_online_members_with_guarantees(
       const Gcs_member_identifier &exclude_member) override;
@@ -931,7 +1253,7 @@ class Group_member_info_manager : public Group_member_info_manager_interface {
 
   void update(Group_member_info *update_local_member) override;
 
-  void update(std::vector<Group_member_info *> *new_members) override;
+  void update(Group_member_info_list *new_members) override;
 
   void update_member_status(const std::string &uuid,
                             Group_member_info::Group_member_status new_status,
@@ -961,14 +1283,15 @@ class Group_member_info_manager : public Group_member_info_manager_interface {
 
   void encode(std::vector<uchar> *to_encode) override;
 
-  std::vector<Group_member_info *> *decode(const uchar *to_decode,
-                                           size_t length) override;
+  Group_member_info_list *decode(const uchar *to_decode,
+                                 size_t length) override;
 
   bool is_conflict_detection_enabled() override;
 
   bool get_primary_member_uuid(std::string &primary_member_uuid) override;
 
-  Group_member_info *get_primary_member_info() override;
+  [[nodiscard]] bool get_primary_member_info(
+      Group_member_info &member_info_arg) override;
 
   bool is_majority_unreachable() override;
 
@@ -980,16 +1303,26 @@ class Group_member_info_manager : public Group_member_info_manager_interface {
 
   mysql_mutex_t *get_update_lock() override { return &update_lock; }
 
+  void update_component_primary_election_enabled(const std::string &uuid,
+                                                 bool enabled) override;
+
+  [[nodiscard]] bool is_group_replication_elect_prefers_most_updated_enabled()
+      override;
+
+  uint64_t get_timestamp_last_view_change() override;
+
  private:
   void clear_members();
 
   Group_member_info *get_group_member_info_by_member_id_internal(
       const Gcs_member_identifier &id);
 
-  std::map<std::string, Group_member_info *> *members;
+  Group_member_info_map *members;
   Group_member_info *local_member_info;
 
   mysql_mutex_t update_lock;
+
+  uint64_t view_change_timestamp{0};
 };
 
 /**
@@ -1015,9 +1348,21 @@ class Group_member_info_manager : public Group_member_info_manager_interface {
   | payload_item_len  |   8 bytes | size of CT_MEMBER_INFO_MESSAGE data  |
   | payload_item      |   X bytes | CT_MEMBER_INFO_MESSAGE data          |
   +-------------------+-----------+--------------------------------------+
+  | payload_item_type |   2 bytes | PIT_MEMBER_ACTIONS                   |
+  | payload_item_len  |   8 bytes | size of PIT_MEMBER_ACTIONS data      |
+  | payload_item      |   X bytes | PIT_MEMBER_ACTIONS data              |
+  +-------------------+-----------+--------------------------------------+
+  | payload_item_type |   2 bytes | PIT_RPL_FAILOVER_CONFIGURATION       |
+  | payload_item_len  |   8 bytes | size of                              |
+  |                   |           | PIT_RPL_FAILOVER_CONFIGURATION data  |
+  | payload_item      |   X bytes | PIT_RPL_FAILOVER_CONFIGURATION data  |
+  +-------------------+-----------+--------------------------------------+
 
- The last tree lines occur the number of times specified on
+ The PIT_MEMBER_DATA lines occur the number of times specified on
  PIT_MEMBERS_NUMBER.
+ The PIT_MEMBER_ACTIONS and PIT_RPL_FAILOVER_CONFIGURATION lines will
+ exist if the member that sent the Group_member_info_manager message
+ is not joining.
 */
 class Group_member_info_manager_message : public Plugin_gcs_message {
  public:
@@ -1031,8 +1376,14 @@ class Group_member_info_manager_message : public Plugin_gcs_message {
     // Length of the payload item: variable
     PIT_MEMBER_DATA = 2,
 
+    // Length of the payload item: variable
+    PIT_MEMBER_ACTIONS = 3,
+
+    // Length of the payload item: variable
+    PIT_RPL_FAILOVER_CONFIGURATION = 4,
+
     // No valid type codes can appear after this one.
-    PIT_MAX = 3
+    PIT_MAX = 5
   };
 
   /**
@@ -1064,7 +1415,51 @@ class Group_member_info_manager_message : public Plugin_gcs_message {
 
     @return a vector with copies to all members.
    */
-  std::vector<Group_member_info *> *get_all_members();
+  Group_member_info_list *get_all_members();
+
+  /**
+    Adds a already serialized member actions configuration
+    to the Group_member_info_manager_message content.
+
+    @param[in] buffer  message buffer
+    @param[in] member_actions_serialized_configuration
+                       serialized member actions configuration
+   */
+  void add_member_actions_serialized_configuration(
+      std::vector<unsigned char> *buffer,
+      const std::string &member_actions_serialized_configuration) const;
+
+  /**
+    Gets the data that belongs to payload_item_type pit.
+
+    @param[in]  pit     the payload_item_type to which the data belongs
+    @param[in]  buffer  message buffer
+    @param[in]  length  message buffer length
+    @param[out] pit_data
+                       the data from payload_item_type pit
+    @param[out] pit_length
+                       the length of the data from payload_item_type pit
+
+    @return the operation status
+      @retval false  OK
+      @retval true   member actions do not exist on the message
+   */
+  bool get_pit_data(const enum_payload_item_type pit,
+                    const unsigned char *buffer, size_t length,
+                    const unsigned char **pit_data, size_t *pit_length);
+
+  /**
+    Adds a already serialized replication failover channels
+    configuration to the Group_member_info_manager_message content.
+
+    @param[in] buffer  message buffer
+    @param[in] replication_failover_channels_serialized_configuration
+                       serialized failover channels configuration
+   */
+  void add_replication_failover_channels_serialized_configuration(
+      std::vector<unsigned char> *buffer,
+      const std::string &replication_failover_channels_serialized_configuration)
+      const;
 
  protected:
   void encode_payload(std::vector<unsigned char> *buffer) const override;
@@ -1077,7 +1472,7 @@ class Group_member_info_manager_message : public Plugin_gcs_message {
   */
   void clear_members();
 
-  std::vector<Group_member_info *> *members;
+  Group_member_info_list *members;
 };
 
 #endif /* MEMBER_INFO_INCLUDE */
