@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -21,17 +21,17 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
-#include <ctype.h>
-#include <errno.h>
-#include <limits.h>
-#include <math.h>
+#include <cassert>
+#include <cctype>
+#include <cerrno>
+#include <climits>
+#include <cmath>
 #ifdef _MSC_VER
 #include <stdint.h>
 #endif
 #include <rpc/rpc.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 
 #include "xcom/xcom_profile.h"
 #ifndef XCOM_STANDALONE
@@ -153,7 +153,7 @@ result set_nodelay(int fd) {
     ret.val =
         setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (xcom_buf *)&n, sizeof n);
     ret.funerr = to_errno(GET_OS_ERR);
-    IFDBG(D_NONE, FN; NDBG(from_errno(ret.funerr), d));
+    XCOM_IFDBG(D_NONE, FN; NDBG(from_errno(ret.funerr), d));
   } while (ret.val < 0 && can_retry(ret.funerr));
   return ret;
 }
@@ -225,7 +225,7 @@ int flush_srv_buf(server *s, int64_t *ret) {
   reset_srv_buf(&s->out_buf);
   if (s->con->fd >= 0) {
     if (ep->buflen) {
-      IFDBG(D_TRANSPORT, FN; PTREXP(stack); NDBG(ep->buflen, u));
+      XCOM_IFDBG(D_TRANSPORT, FN; PTREXP(stack); NDBG(ep->buflen, u));
       /* LOCK_FD(s->con.fd, 'w'); */
       TASK_CALL(task_write(s->con, s->out_buf.buf, ep->buflen, &sent));
       /* UNLOCK_FD(s->fd, 'w'); */
@@ -317,11 +317,12 @@ static int _send_msg(server *s, pax_msg *p, node_no to, int64_t *ret) {
   int64_t sent{0};
   TASK_BEGIN
   p->to = to;
-  IFDBG(D_NONE, FN; PTREXP(stack); PTREXP(s); PTREXP(p); NDBG(s->con.fd, d));
-  IFDBG(D_NONE, FN; STREXP(s->srv); NDBG(s->port, d); NDBG(task_now(), f);
-        COPY_AND_FREE_GOUT(dbg_pax_msg(p)););
+  XCOM_IFDBG(D_NONE, FN; PTREXP(stack); PTREXP(s); PTREXP(p);
+             NDBG(s->con->fd, d));
+  XCOM_IFDBG(D_NONE, FN; STREXP(s->srv); NDBG(s->port, d); NDBG(task_now(), f);
+             COPY_AND_FREE_GOUT(dbg_pax_msg(p)););
   if (to == p->from) {
-    IFDBG(D_NONE, FN; COPY_AND_FREE_GOUT(dbg_pax_msg(p)););
+    XCOM_IFDBG(D_NONE, FN; COPY_AND_FREE_GOUT(dbg_pax_msg(p)););
     dispatch_op(find_site_def(p->synode), p, nullptr);
     TASK_RETURN(sizeof(*p));
   } else {
@@ -329,7 +330,7 @@ static int _send_msg(server *s, pax_msg *p, node_no to, int64_t *ret) {
     if (s->con->fd >= 0) {
       /* LOCK_FD(s->con.fd, 'w'); */
       serialize_msg(p, s->con->x_proto, &ep->buflen, &ep->buf);
-      IFDBG(D_TRANSPORT, FN; NDBG(ep->buflen, u));
+      XCOM_IFDBG(D_TRANSPORT, FN; NDBG(ep->buflen, u));
       if (ep->buflen) {
         /* Not enough space? Flush the buffer */
         if (ep->buflen > srv_buf_free_space(&s->out_buf)) {
@@ -339,7 +340,8 @@ static int _send_msg(server *s, pax_msg *p, node_no to, int64_t *ret) {
           }
           /* Still not enough? Message must be huge, send without buffering */
           if (ep->buflen > srv_buf_free_space(&s->out_buf)) {
-            IFDBG(D_TRANSPORT, FN; STRLIT("task_write "); NDBG(ep->buflen, u));
+            XCOM_IFDBG(D_TRANSPORT, FN; STRLIT("task_write ");
+                       NDBG(ep->buflen, u));
             TASK_CALL(task_write(s->con, ep->buf, ep->buflen, &sent));
             if (s->con->fd < 0) {
               TASK_FAIL;
@@ -355,7 +357,8 @@ static int _send_msg(server *s, pax_msg *p, node_no to, int64_t *ret) {
         send_count[p->op]++;
         send_bytes[p->op] += ep->buflen;
         alive(s); /* Note activity */
-        /* IFDBG(D_NONE, STRLIT("sent message "); STRLIT(pax_op_to_str(p->op));
+        /* XCOM_IFDBG(D_NONE, STRLIT("sent message ");
+         * STRLIT(pax_op_to_str(p->op));
          */
         /* NDBG(p->from,d); NDBG(p->to,d); */
         /* SYCEXP(p->synode);  */
@@ -384,8 +387,9 @@ xcom_proto read_protoversion(unsigned char *p) { return (xcom_proto)get_32(p); }
 
 int check_protoversion(xcom_proto x_proto, xcom_proto negotiated) {
   if (x_proto != negotiated) {
-    IFDBG(D_NONE, FN; STRLIT(" found XCOM protocol version "); NDBG(x_proto, d);
-          STRLIT(" need version "); NDBG(negotiated, d););
+    XCOM_IFDBG(D_NONE, FN; STRLIT(" found XCOM protocol version ");
+               NDBG(x_proto, d); STRLIT(" need version ");
+               NDBG(negotiated, d););
 
     return 0;
   }
@@ -492,7 +496,8 @@ static int serialize(void *p, xcom_proto x_proto, uint32_t *out_len,
   if (!msg_buflen) return 0;
   assert(msg_buflen);
   tot_buflen = SERIALIZED_BUFLEN(msg_buflen);
-  IFDBG(D_TRANSPORT, FN; PTREXP(p); NDBG64(msg_buflen); NDBG64(tot_buflen));
+  XCOM_IFDBG(D_TRANSPORT, FN; PTREXP(p); NDBG64(msg_buflen);
+             NDBG64(tot_buflen));
   /*
     Paxos message size is limited in UINT32 range. It will return an
     error if the serialized message is bigger than UINT32_MAX bytes.
@@ -522,8 +527,8 @@ static int serialize(void *p, xcom_proto x_proto, uint32_t *out_len,
     *out_len = (uint32_t)tot_buflen;
     *out_buf = (char *)buf;
   }
-  IFDBG(D_TRANSPORT, FN; NDBG(retval, d); NDBG(*out_len, u); PTREXP(*out_buf);
-        dump_header(*out_buf));
+  XCOM_IFDBG(D_TRANSPORT, FN; NDBG(retval, d); NDBG(*out_len, u);
+             PTREXP(*out_buf); dump_header(*out_buf));
   return retval;
 }
 
@@ -598,9 +603,9 @@ static server *mksrv(char *srv, xcom_port port) {
 
   s = (server *)xcom_calloc((size_t)1, sizeof(*s));
 
-  IFDBG(D_NONE, FN; PTREXP(s); STREXP(srv));
+  XCOM_IFDBG(D_NONE, FN; PTREXP(s); STREXP(srv));
   if (s == nullptr) {
-    g_critical("out of memory");
+    G_FATAL("out of memory");
     abort();
   }
   s->garbage = 0;
@@ -618,18 +623,18 @@ static server *mksrv(char *srv, xcom_port port) {
 #endif
 
   channel_init(&s->outgoing, TYPE_HASH("msg_link"));
-  IFDBG(D_NONE, FN; STREXP(srv); NDBG(port, d));
+  XCOM_IFDBG(D_NONE, FN; STREXP(srv); NDBG(port, d));
 
   if (xcom_mynode_match(srv, port)) { /* Short-circuit local messages */
-    IFDBG(D_NONE, FN; STRLIT("creating local sender"); STREXP(srv);
-          NDBG(port, d));
+    XCOM_IFDBG(D_NONE, FN; STRLIT("creating local sender"); STREXP(srv);
+               NDBG(port, d));
     s->sender = task_new(local_sender_task, void_arg(s), "local_sender_task",
                          XCOM_THREAD_DEBUG);
   } else {
     s->sender =
         task_new(sender_task, void_arg(s), "sender_task", XCOM_THREAD_DEBUG);
-    IFDBG(D_NONE, FN; STRLIT("creating sender and reply_handler"); STREXP(srv);
-          NDBG(port, d));
+    XCOM_IFDBG(D_NONE, FN; STRLIT("creating sender and reply_handler");
+               STREXP(srv); NDBG(port, d));
     s->reply_handler = task_new(reply_handler_task, void_arg(s),
                                 "reply_handler_task", XCOM_THREAD_DEBUG);
   }
@@ -648,9 +653,9 @@ static server *addsrv(char *srv, xcom_port port) {
    reply_handler_task begin.
   */
   srv_ref(s);
-  IFDBG(D_NONE, FN; PTREXP(all_servers[maxservers]);
-        STREXP(all_servers[maxservers]->srv);
-        NDBG(all_servers[maxservers]->port, d); NDBG(maxservers, d));
+  XCOM_IFDBG(D_NONE, FN; PTREXP(all_servers[maxservers]);
+             STREXP(all_servers[maxservers]->srv);
+             NDBG(all_servers[maxservers]->port, d); NDBG(maxservers, d));
   maxservers++;
   return s;
 }
@@ -659,8 +664,8 @@ static void rmsrv(int i) {
   assert(all_servers[i]);
   assert(maxservers > 0);
   assert(i < maxservers);
-  IFDBG(D_NONE, FN; PTREXP(all_servers[i]); STREXP(all_servers[i]->srv);
-        NDBG(all_servers[i]->port, d); NDBG(i, d));
+  XCOM_IFDBG(D_NONE, FN; PTREXP(all_servers[i]); STREXP(all_servers[i]->srv);
+             NDBG(all_servers[i]->port, d); NDBG(i, d));
   maxservers--;
 
   /* Allow the server to be freed. This unref pairs with the ref from addsrv. */
@@ -710,7 +715,7 @@ static void sweep() {
     server *s = all_servers[i];
     assert(s);
     if (s->garbage) {
-      IFDBG(D_NONE, FN; STREXP(s->srv));
+      XCOM_IFDBG(D_NONE, FN; STREXP(s->srv));
       shut_srv(s);
       rmsrv(i);
     } else {
@@ -720,11 +725,11 @@ static void sweep() {
 }
 
 void garbage_collect_servers() {
-  IFDBG(D_NONE, FN);
+  XCOM_IFDBG(D_NONE, FN);
   init_collect();
   mark();
   sweep();
-  IFDBG(D_NONE, FN);
+  XCOM_IFDBG(D_NONE, FN);
 }
 
 /* Free a server */
@@ -744,7 +749,7 @@ double server_active(site_def const *s, node_no i) {
 /* Shutdown server */
 static void shut_srv(server *s) {
   if (!s) return;
-  IFDBG(D_NONE, FN; PTREXP(s); STREXP(s->srv));
+  XCOM_IFDBG(D_NONE, FN; PTREXP(s); STREXP(s->srv));
 
   shutdown_connection(s->con);
 
@@ -797,7 +802,7 @@ int incoming_connection_task(task_arg arg [[maybe_unused]]) {
   }
   free(clean_conn);
 
-  IFDBG(D_BUG, FN; STRLIT(" shutdown "));
+  XCOM_IFDBG(D_BUG, FN; STRLIT(" shutdown "));
   TASK_END;
 }
 
@@ -813,8 +818,8 @@ static int dial(server *s,
   END_ENV;
 
   TASK_BEGIN
-  IFDBG(D_BUG, FN; STRLIT(" dial "); NPUT(get_nodeno(get_site_def()), u);
-        STRLIT(s->srv); NDBG(s->port, u));
+  XCOM_IFDBG(D_BUG, FN; STRLIT(" dial "); NPUT(get_nodeno(get_site_def()), u);
+             STRLIT(s->srv); NDBG(s->port, u));
 
   // Delete old connection
   reset_connection(s->con);
@@ -827,17 +832,17 @@ static int dial(server *s,
   }
 
   if (s->con->fd < 0) {
-    IFDBG(D_NONE, FN; STRLIT("could not dial "); STRLIT(s->srv);
-          NDBG(s->port, u););
+    XCOM_IFDBG(D_NONE, FN; STRLIT("could not dial "); STRLIT(s->srv);
+               NDBG(s->port, u););
   } else {
     if (NAGLE == 0) {
       set_nodelay(s->con->fd);
     }
 
     unblock_fd(s->con->fd);
-    IFDBG(D_BUG, FN; STRLIT(" dial connected ");
-          NPUT(get_nodeno(get_site_def()), u); STRLIT(s->srv);
-          NDBG(s->port, u));
+    XCOM_IFDBG(D_BUG, FN; STRLIT(" dial connected ");
+               NPUT(get_nodeno(get_site_def()), u); STRLIT(s->srv);
+               NDBG(s->port, u));
     set_connected(s->con, CON_FD);
     alive(s);
     update_detected(get_site_def_rw());
@@ -853,14 +858,14 @@ int send_msg(server *s, node_no from, node_no to, uint32_t group_id,
   assert(s);
   {
     msg_link *link = msg_link_new(p, to);
-    IFDBG(D_NONE, FN; PTREXP(&s->outgoing);
-          COPY_AND_FREE_GOUT(dbg_msg_link(link)););
+    XCOM_IFDBG(D_NONE, FN; PTREXP(&s->outgoing);
+               COPY_AND_FREE_GOUT(dbg_msg_link(link)););
     p->from = from;
     p->group_id = group_id;
     p->max_synode = get_max_synode();
     p->delivered_msg = get_delivered_msg();
-    IFDBG(D_NONE, FN; PTREXP(p); STREXP(s->srv); NDBG(p->from, d);
-          NDBG(p->to, d); NDBG(p->group_id, u));
+    XCOM_IFDBG(D_NONE, FN; PTREXP(p); STREXP(s->srv); NDBG(p->from, d);
+               NDBG(p->to, d); NDBG(p->group_id, u));
     channel_put(&s->outgoing, &link->l);
   }
   return 0;
@@ -904,8 +909,8 @@ static inline int send_to_node_set(site_def const *s, node_no max, pax_msg *p,
     node_no i = 0;
     for (i = 0; i < max; i++) {
       if (test_func(s, i)) {
-        IFDBG(D_NONE, FN; STRLIT(dbg); STRLIT(" "); NDBG(i, u); NDBG(max, u);
-              PTREXP(p));
+        XCOM_IFDBG(D_NONE, FN; STRLIT(dbg); STRLIT(" "); NDBG(i, u);
+                   NDBG(max, u); PTREXP(p));
         retval = _send_server_msg(s, i, p);
       }
     }
@@ -947,8 +952,8 @@ static inline int send_other_loop(site_def const *s, pax_msg *p,
 #endif
   for (i = 0; i < max; i++) {
     if (i != s->nodeno) {
-      IFDBG(D_NONE, FN; STRLIT(dbg); STRLIT(" "); NDBG(i, u); NDBG(max, u);
-            PTREXP(p));
+      XCOM_IFDBG(D_NONE, FN; STRLIT(dbg); STRLIT(" "); NDBG(i, u); NDBG(max, u);
+                 PTREXP(p));
       retval = _send_server_msg(s, i, p);
     }
   }
@@ -975,14 +980,14 @@ int send_to_someone(site_def const *s, pax_msg *p,
   assert(s);
   max = get_maxnodes(s);
 #endif
-  /* IFDBG(D_NONE, FN; NDBG(max,u); NDBG(s->maxnodes,u)); */
+  /* XCOM_IFDBG(D_NONE, FN; NDBG(max,u); NDBG(s->maxnodes,u)); */
   assert(max > 0);
   prev = i % max;
   i = (i + 1) % max;
   while (i != prev) {
-    /* IFDBG(D_NONE, FN; NDBG(i,u); NDBG(prev,u)); */
+    /* XCOM_IFDBG(D_NONE, FN; NDBG(i,u); NDBG(prev,u)); */
     if (i != s->nodeno && !may_be_dead(s->detected, i, task_now())) {
-      IFDBG(D_NONE, FN; STRLIT(dbg); NDBG(i, u); NDBG(max, u); PTREXP(p));
+      XCOM_IFDBG(D_NONE, FN; STRLIT(dbg); NDBG(i, u); NDBG(max, u); PTREXP(p));
       retval = _send_server_msg(s, i, p);
       break;
     }
@@ -1033,7 +1038,7 @@ static int read_bytes(connection_descriptor const *rfd, char *p, uint32_t n,
 
   int64_t nread = 0;
 
-  IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); PTREXP(s); NDBG(n, u));
+  XCOM_IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); PTREXP(s); NDBG(n, u));
   TASK_BEGIN
 
       (void)
@@ -1043,11 +1048,12 @@ static int read_bytes(connection_descriptor const *rfd, char *p, uint32_t n,
   while (ep->left > 0) {
     TASK_CALL(task_read(rfd, ep->bytes,
                         ep->left >= INT_MAX ? INT_MAX : (int)ep->left, &nread));
-    IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(nread); NDBG(ep->left, u));
+    XCOM_IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(nread);
+               NDBG(ep->left, u));
     if (nread == 0) {
       TASK_RETURN(0);
     } else if (nread < 0) {
-      IFDBG(D_NONE, FN; NDBG64(nread));
+      XCOM_IFDBG(D_NONE, FN; NDBG64(nread));
       TASK_FAIL;
     } else {
       ep->bytes += nread;
@@ -1085,7 +1091,7 @@ static int buffered_read_bytes(connection_descriptor const *rfd, srv_buf *buf,
   END_ENV;
   uint32_t nget = 0;
 
-  IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); PTREXP(s); NDBG(n, u));
+  XCOM_IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); PTREXP(s); NDBG(n, u));
 
   int64_t nread{0};
   TASK_BEGIN(void) s;
@@ -1099,7 +1105,7 @@ static int buffered_read_bytes(connection_descriptor const *rfd, srv_buf *buf,
 
   if (ep->left >= srv_buf_capacity(buf)) {
     /* Too big, do direct read of rest */
-    IFDBG(D_TRANSPORT, FN; STRLIT("Too big, do direct read of rest"));
+    XCOM_IFDBG(D_TRANSPORT, FN; STRLIT("Too big, do direct read of rest"));
     TASK_CALL(read_bytes(rfd, ep->bytes, ep->left, s, ret));
     if (*ret <= 0) {
       TASK_FAIL;
@@ -1113,11 +1119,11 @@ static int buffered_read_bytes(connection_descriptor const *rfd, srv_buf *buf,
 
       TASK_CALL(task_read(rfd, srv_buf_insert_ptr(buf),
                           (int)srv_buf_free_space(buf), &nread));
-      IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(nread););
+      XCOM_IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(nread););
       if (nread == 0) {
         TASK_RETURN(0);
       } else if (nread < 0) {
-        IFDBG(D_NONE, FN; NDBG64(nread));
+        XCOM_IFDBG(D_NONE, FN; NDBG64(nread));
         TASK_FAIL;
       } else {
         /* Update buffer to reflect number of bytes read */
@@ -1139,12 +1145,13 @@ void get_header_1_0(unsigned char header_buf[], uint32_t *msgsize,
   *msgsize = get_32(LENGTH_PTR(header_buf));
   *x_type = (x_msg_type)header_buf[X_TYPE];
   *tag = get_16(X_TAG_PTR(header_buf));
-  IFDBG(D_TRANSPORT, FN; NDBG(*msgsize, u); NDBG(*x_type, d); NDBG(*tag, u));
+  XCOM_IFDBG(D_TRANSPORT, FN; NDBG(*msgsize, u); NDBG(*x_type, d);
+             NDBG(*tag, u));
 }
 
 void put_header_1_0(unsigned char header_buf[], uint32_t msgsize,
                     x_msg_type x_type, unsigned int tag) {
-  IFDBG(D_TRANSPORT, FN; NDBG(msgsize, u); NDBG(x_type, d); NDBG(tag, u));
+  XCOM_IFDBG(D_TRANSPORT, FN; NDBG(msgsize, u); NDBG(x_type, d); NDBG(tag, u));
   put_32(LENGTH_PTR(header_buf), msgsize);
   header_buf[X_TYPE] = (unsigned char)x_type;
   put_16(X_TAG_PTR(header_buf), tag);
@@ -1171,13 +1178,13 @@ int read_msg(connection_descriptor *rfd, pax_msg *p, server *s, int64_t *ret) {
     ep->bytes = nullptr;
     /* Read length field, protocol version, and checksum */
     ep->n = 0;
-    IFDBG(D_TRANSPORT, FN; STRLIT("reading header"));
+    XCOM_IFDBG(D_TRANSPORT, FN; STRLIT("reading header"));
     TASK_CALL(read_bytes(rfd, (char *)ep->header_buf, MSG_HDR_SIZE, s, &ep->n));
 
     if (ep->n != MSG_HDR_SIZE) {
       G_INFO("Failure reading from fd=%d n=%" PRIu64 " from %s:%d", rfd->fd,
              ep->n, s->srv, s->port);
-      IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(ep->n));
+      XCOM_IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(ep->n));
       TASK_FAIL;
     }
 
@@ -1187,10 +1194,10 @@ int read_msg(connection_descriptor *rfd, pax_msg *p, server *s, int64_t *ret) {
     if (ep->x_type == x_version_req) {
       /* Negotiation request. See what we can offer */
       rfd->x_proto = negotiate_protocol(ep->x_version);
-      IFDBG(D_TRANSPORT,
-            STRLIT("incoming connection will use protcol version ");
-            NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto));
-            NDBG(rfd->fd, d));
+      XCOM_IFDBG(D_TRANSPORT,
+                 STRLIT("incoming connection will use protcol version ");
+                 NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto));
+                 NDBG(rfd->fd, d));
       ADD_DBG(
           D_TRANSPORT,
           add_event(EVENT_DUMP_PAD,
@@ -1212,9 +1219,10 @@ int read_msg(connection_descriptor *rfd, pax_msg *p, server *s, int64_t *ret) {
       /* Mark connection with negotiated protocol version */
       if (rfd->snd_tag == ep->tag) {
         rfd->x_proto = ep->x_version;
-        IFDBG(D_TRANSPORT, STRLIT("peer connection will use protcol version ");
-              NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto));
-              NDBG(rfd->fd, d));
+        XCOM_IFDBG(
+            D_TRANSPORT, STRLIT("peer connection will use protcol version ");
+            NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto));
+            NDBG(rfd->fd, d));
 
         ADD_DBG(D_TRANSPORT,
                 add_event(
@@ -1246,18 +1254,19 @@ int read_msg(connection_descriptor *rfd, pax_msg *p, server *s, int64_t *ret) {
 
   /* Read message */
   ep->n = 0;
-  IFDBG(D_TRANSPORT, FN; STRLIT("reading message"));
+  XCOM_IFDBG(D_TRANSPORT, FN; STRLIT("reading message"));
   TASK_CALL(read_bytes(rfd, ep->bytes, ep->msgsize, s, &ep->n));
 
   if (ep->n > 0) {
     /* Deserialize message */
     deserialize_ok = deserialize_msg(p, rfd->x_proto, ep->bytes, ep->msgsize);
-    IFDBG(D_NONE, FN; STRLIT(" deserialized message"));
+    XCOM_IFDBG(D_NONE, FN; STRLIT(" deserialized message"));
   }
   /* Deallocate buffer */
   X_FREE(ep->bytes);
   if (ep->n <= 0 || !deserialize_ok) {
-    IFDBG(D_NONE, FN; NDBG(rfd->fd, d); NDBG64(ep->n); NDBG(deserialize_ok, d));
+    XCOM_IFDBG(D_NONE, FN; NDBG(rfd->fd, d); NDBG64(ep->n);
+               NDBG(deserialize_ok, d));
     TASK_FAIL;
   }
   TASK_RETURN(ep->n);
@@ -1289,12 +1298,12 @@ int buffered_read_msg(connection_descriptor *rfd, srv_buf *buf, pax_msg *p,
     ep->bytes = nullptr;
     /* Read length field, protocol version, and checksum */
     ep->n = 0;
-    IFDBG(D_TRANSPORT, FN; STRLIT("reading header"));
+    XCOM_IFDBG(D_TRANSPORT, FN; STRLIT("reading header"));
     TASK_CALL(buffered_read_bytes(rfd, buf, (char *)ep->header_buf,
                                   MSG_HDR_SIZE, s, &ep->n));
 
     if (ep->n != MSG_HDR_SIZE) {
-      IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(ep->n));
+      XCOM_IFDBG(D_TRANSPORT, FN; NDBG(rfd->fd, d); NDBG64(ep->n));
       TASK_FAIL;
     }
 
@@ -1304,9 +1313,9 @@ int buffered_read_msg(connection_descriptor *rfd, srv_buf *buf, pax_msg *p,
     if (ep->x_type == x_version_req) {
       /* Negotiation request. See what we can offer */
       rfd->x_proto = negotiate_protocol(ep->x_version);
-      IFDBG(D_TRANSPORT,
-            STRLIT("incoming connection will use protcol version ");
-            NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto)));
+      XCOM_IFDBG(
+          D_TRANSPORT, STRLIT("incoming connection will use protcol version ");
+          NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto)));
       ADD_DBG(
           D_TRANSPORT,
           add_event(EVENT_DUMP_PAD,
@@ -1329,8 +1338,9 @@ int buffered_read_msg(connection_descriptor *rfd, srv_buf *buf, pax_msg *p,
       /* Mark connection with negotiated protocol version */
       if (rfd->snd_tag == ep->tag) {
         rfd->x_proto = ep->x_version;
-        IFDBG(D_TRANSPORT, STRLIT("peer connection will use protcol version ");
-              NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto)));
+        XCOM_IFDBG(
+            D_TRANSPORT, STRLIT("peer connection will use protcol version ");
+            NDBG(rfd->x_proto, u); STRLIT(xcom_proto_to_str(rfd->x_proto)));
         ADD_DBG(D_TRANSPORT,
                 add_event(
                     0, string_arg("peer connection will use protcol version"));
@@ -1360,18 +1370,19 @@ int buffered_read_msg(connection_descriptor *rfd, srv_buf *buf, pax_msg *p,
   }
   /* Read message */
   ep->n = 0;
-  IFDBG(D_TRANSPORT, FN; STRLIT("reading message"));
+  XCOM_IFDBG(D_TRANSPORT, FN; STRLIT("reading message"));
   TASK_CALL(buffered_read_bytes(rfd, buf, ep->bytes, ep->msgsize, s, &ep->n));
 
   if (ep->n > 0) {
     /* Deserialize message */
     deserialize_ok = deserialize_msg(p, rfd->x_proto, ep->bytes, ep->msgsize);
-    IFDBG(D_NONE, FN; STRLIT(" deserialized message"));
+    XCOM_IFDBG(D_NONE, FN; STRLIT(" deserialized message"));
   }
   /* Deallocate buffer */
   X_FREE(ep->bytes);
   if (ep->n <= 0 || !deserialize_ok) {
-    IFDBG(D_NONE, FN; NDBG(rfd->fd, d); NDBG64(ep->n); NDBG(deserialize_ok, d));
+    XCOM_IFDBG(D_NONE, FN; NDBG(rfd->fd, d); NDBG64(ep->n);
+               NDBG(deserialize_ok, d));
     TASK_FAIL;
   }
   TASK_RETURN(ep->n);
@@ -1392,11 +1403,11 @@ int recv_proto(connection_descriptor const *rfd, xcom_proto *x_proto,
 
   /* Read length field, protocol version, and checksum */
   ep->n = 0;
-  IFDBG(D_TRANSPORT, FN; STRLIT("reading header"));
+  XCOM_IFDBG(D_TRANSPORT, FN; STRLIT("reading header"));
   TASK_CALL(read_bytes(rfd, (char *)ep->header_buf, MSG_HDR_SIZE, 0, &ep->n));
 
   if (ep->n != MSG_HDR_SIZE) {
-    IFDBG(D_NONE, FN; NDBG64(ep->n));
+    XCOM_IFDBG(D_NONE, FN; NDBG64(ep->n));
     TASK_FAIL;
   }
 
@@ -1417,7 +1428,8 @@ static inline unsigned int incr_tag(unsigned int tag) {
 
 static void start_protocol_negotiation(channel *outgoing) {
   msg_link *link = msg_link_new(nullptr, VOID_NODE_NO);
-  IFDBG(D_NONE, FN; PTREXP(outgoing); COPY_AND_FREE_GOUT(dbg_msg_link(link)););
+  XCOM_IFDBG(D_NONE, FN; PTREXP(outgoing);
+             COPY_AND_FREE_GOUT(dbg_msg_link(link)););
   channel_put_front(outgoing, &link->l);
 }
 
@@ -1470,6 +1482,9 @@ int sender_task(task_arg arg) {
 #if defined(_WIN32)
   bool was_connected;
 #endif
+  bool attempt_connect;
+  double attempt_connect_wait_total_time_not_in_current_view;
+  bool server_in_current_view;
   network_provider_dynamic_log_level dial_call_log_level;
   ENV_INIT
   END_ENV_INIT
@@ -1486,6 +1501,9 @@ int sender_task(task_arg arg) {
 #if defined(_WIN32)
   ep->was_connected = false;
 #endif
+  ep->attempt_connect = true;
+  ep->attempt_connect_wait_total_time_not_in_current_view = 0.0;
+  ep->server_in_current_view = true;
   ep->s = (server *)get_void_arg(arg);
   ep->link = nullptr;
   ep->tag = TAG_START;
@@ -1493,29 +1511,76 @@ int sender_task(task_arg arg) {
 
   while (!xcom_shutdown) {
     /* Loop until connected */
-    G_MESSAGE("Connecting to %s:%d", ep->s->srv, ep->s->port);
+    G_INFO("Connecting to %s:%d", ep->s->srv, ep->s->port);
     for (;;) {
+      if (ep->attempt_connect) {
 #if defined(_WIN32)
-      if (!ep->was_connected) {
+        if (!ep->was_connected) {
 #endif
-        TASK_CALL(dial(ep->s, ep->dial_call_log_level));
+          TASK_CALL(dial(ep->s, ep->dial_call_log_level));
 #if defined(_WIN32)
-      } else {
-        ep->s->reconnect = true;
+        } else {
+          ep->s->reconnect = true;
+        }
+#endif
       }
-#endif
       if (is_connected(ep->s->con)) break;
 
       // Check, for each reconnect loop, if we belong to this configuration
-      ep->dial_call_log_level =
-          is_server_in_current_view(ep->s)
-              ? network_provider_dynamic_log_level::PROVIDED
-              : network_provider_dynamic_log_level::DEBUG;
+      ep->server_in_current_view = is_server_in_current_view(ep->s);
 
-      if (ep->dtime < MAX_CONNECT_WAIT) {
-        if (ep->dial_call_log_level !=
-            network_provider_dynamic_log_level::DEBUG) {
-          G_MESSAGE("Connection to %s:%d failed", ep->s->srv, ep->s->port);
+      if (ep->attempt_connect) {
+        ep->dial_call_log_level =
+            ep->server_in_current_view
+                ? network_provider_dynamic_log_level::PROVIDED
+                : network_provider_dynamic_log_level::DEBUG;
+
+        if (ep->dtime < MAX_CONNECT_WAIT) {
+          if (ep->dial_call_log_level !=
+              network_provider_dynamic_log_level::DEBUG) {
+            G_INFO("Connection to %s:%d failed", ep->s->srv, ep->s->port);
+          }
+        }
+
+        if (!ep->server_in_current_view) {
+          ep->attempt_connect_wait_total_time_not_in_current_view += ep->dtime;
+
+          /*
+            Stop connection attempts if we reached MAX_CONNECT_WAIT_TOTAL_TIME
+            and the server we want to connect to is not in the current view.
+            We do not fail this task to avoid consider this a critical
+            error that would cause the local server to leave the group.
+          */
+          if (ep->attempt_connect_wait_total_time_not_in_current_view >=
+              MAX_CONNECT_WAIT_TOTAL_TIME) {
+            ep->attempt_connect = false;
+#if defined(_WIN32)
+            ep->s->reconnect = false;
+#endif
+
+            G_INFO(
+                "Connection attempts to %s:%d stopped because this server "
+                "belongs to a previous connfiguration",
+                ep->s->srv, ep->s->port);
+          }
+        }
+      } else {
+        if (ep->server_in_current_view) {
+          /*
+            Server rejoined, resume connection attempts on next loop
+            iteration.
+          */
+          ep->dtime = INITIAL_CONNECT_WAIT;
+          ep->attempt_connect = true;
+          ep->attempt_connect_wait_total_time_not_in_current_view = 0.0;
+#if defined(_WIN32)
+          ep->s->reconnect = true;
+#endif
+
+          G_INFO(
+              "Connection attempts to %s:%d resumed because this server "
+              "belongs to the current connfiguration",
+              ep->s->srv, ep->s->port);
         }
       }
 
@@ -1533,12 +1598,14 @@ int sender_task(task_arg arg) {
       }
     }
 
-    G_MESSAGE("Connected to %s:%d", ep->s->srv, ep->s->port);
+    G_INFO("Connected to %s:%d", ep->s->srv, ep->s->port);
     ep->dtime = INITIAL_CONNECT_WAIT;
 #if defined(_WIN32)
     ep->was_connected = true;
     ep->s->reconnect = false;
 #endif
+    ep->attempt_connect = true;
+    ep->attempt_connect_wait_total_time_not_in_current_view = 0.0;
     reset_srv_buf(&ep->s->out_buf);
 
     /* We are ready to start sending messages.
@@ -1551,22 +1618,22 @@ int sender_task(task_arg arg) {
       if (false && link_empty(&ep->s->outgoing.data)) {
         TASK_DELAY(0.1 * xcom_drand48());
       }
-      /* FWD_ITER(&ep->s->outgoing.data, msg_link, IFDBG(D_NONE, FN;
+      /* FWD_ITER(&ep->s->outgoing.data, msg_link, XCOM_IFDBG(D_NONE, FN;
        * PTREXP(link_iter));); */
       if (link_empty(&ep->s->outgoing.data)) {
         TASK_CALL(flush_srv_buf(ep->s, &ret));
       }
       CHANNEL_GET(&ep->s->outgoing, &ep->link, msg_link);
       {
-        /* IFDBG(D_NONE, FN; PTREXP(stack); PTREXP(ep->link));
-        IFDBG(D_NONE, FN; PTREXP(&ep->s->outgoing);
+        /* XCOM_IFDBG(D_NONE, FN; PTREXP(stack); PTREXP(ep->link));
+        XCOM_IFDBG(D_NONE, FN; PTREXP(&ep->s->outgoing);
                COPY_AND_FREE_GOUT(dbg_msg_link(ep->link)););
-        IFDBG(D_NONE, FN; STRLIT(" extracted ");
+        XCOM_IFDBG(D_NONE, FN; STRLIT(" extracted ");
                         COPY_AND_FREE_GOUT(dbg_linkage(&ep->link->l));
             ); */
 
         /* If ep->link->p is 0, it is a protocol (re)negotiation request */
-        /* IFDBG(D_NONE, FN;
+        /* XCOM_IFDBG(D_NONE, FN;
                 NDBG(ep->s->con.x_proto,u);
            STRLIT(xcom_proto_to_str(ep->s->con.x_proto));
                 NDBG(get_latest_common_proto(),u);
@@ -1623,21 +1690,21 @@ int sender_task(task_arg arg) {
               D_TRANSPORT,
               add_event(EVENT_DUMP_PAD, string_arg("will use protocol"));
               add_event(EVENT_DUMP_PAD,
-                        string_arg(xcom_proto_to_str(ep->s->con.x_proto))););
+                        string_arg(xcom_proto_to_str(ep->s->con->x_proto))););
         }
       }
     next:
       msg_link_delete(&ep->link);
       /* TASK_YIELD; */
     }
-    G_MESSAGE("Sender task disconnected from %s:%d", ep->s->srv, ep->s->port);
+    G_INFO("Sender task disconnected from %s:%d", ep->s->srv, ep->s->port);
   }
   FINALLY
   empty_msg_channel(&ep->s->outgoing);
   ep->s->sender = nullptr;
   srv_unref(ep->s);
   if (ep->link) msg_link_delete(&ep->link);
-  IFDBG(D_BUG, FN; STRLIT(" shutdown "));
+  XCOM_IFDBG(D_BUG, FN; STRLIT(" shutdown "));
   TASK_END;
 }
 
@@ -1682,7 +1749,7 @@ int tcp_reconnection_task(task_arg arg [[maybe_unused]]) {
     TASK_DELAY(2.0);
   }
   FINALLY
-  IFDBG(D_BUG, FN; STRLIT(" shutdown "));
+  XCOM_IFDBG(D_BUG, FN; STRLIT(" shutdown "));
   TASK_END;
 }
 #endif
@@ -1710,11 +1777,11 @@ int local_sender_task(task_arg arg) {
     assert(!ep->link);
     CHANNEL_GET(&ep->s->outgoing, &ep->link, msg_link);
     {
-      /* IFDBG(D_NONE, FN; PTREXP(stack); PTREXP(ep->link)); */
-      IFDBG(D_NONE, FN; PTREXP(&ep->s->outgoing);
-            COPY_AND_FREE_GOUT(dbg_msg_link(ep->link)););
-      IFDBG(D_NONE, FN; STRLIT(" extracted ");
-            COPY_AND_FREE_GOUT(dbg_linkage(&ep->link->l)););
+      /* XCOM_IFDBG(D_NONE, FN; PTREXP(stack); PTREXP(ep->link)); */
+      XCOM_IFDBG(D_NONE, FN; PTREXP(&ep->s->outgoing);
+                 COPY_AND_FREE_GOUT(dbg_msg_link(ep->link)););
+      XCOM_IFDBG(D_NONE, FN; STRLIT(" extracted ");
+                 COPY_AND_FREE_GOUT(dbg_linkage(&ep->link->l)););
       assert(ep->link->p);
       ep->link->p->to = ep->link->p->from;
       dispatch_op(find_site_def(ep->link->p->synode), ep->link->p, nullptr);
@@ -1726,7 +1793,7 @@ int local_sender_task(task_arg arg) {
   ep->s->sender = nullptr;
   srv_unref(ep->s);
   if (ep->link) msg_link_delete(&ep->link);
-  IFDBG(D_BUG, FN; STRLIT(" shutdown "));
+  XCOM_IFDBG(D_BUG, FN; STRLIT(" shutdown "));
   TASK_END;
 }
 
@@ -1737,7 +1804,7 @@ void update_servers(site_def *s, cargo_type operation) {
     u_int i = 0;
     n = s->nodes.node_list_len;
 
-    IFDBG(D_NONE, FN; NDBG(get_maxnodes(s), u); NDBG(n, d); PTREXP(s));
+    XCOM_IFDBG(D_NONE, FN; NDBG(get_maxnodes(s), u); NDBG(n, d); PTREXP(s));
 
     G_INFO("Updating physical connections to other servers");
 
@@ -1784,7 +1851,7 @@ void update_servers(site_def *s, cargo_type operation) {
             /* purecov: end */
           }
         }
-        IFDBG(D_BUG, FN; PTREXP(s->servers[i]));
+        XCOM_IFDBG(D_BUG, FN; PTREXP(s->servers[i]));
       }
     }
     /* Zero the rest */
@@ -1857,7 +1924,7 @@ int tcp_reaper_task(task_arg arg [[maybe_unused]]) {
     TASK_DELAY(1.0);
   }
   FINALLY
-  IFDBG(D_BUG, FN; STRLIT(" shutdown "));
+  XCOM_IFDBG(D_BUG, FN; STRLIT(" shutdown "));
   TASK_END;
 }
 
