@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,6 +33,7 @@
 #include "mysql/gtid/tsid.h"
 #include "plugin/group_replication/include/certifier.h"
 #include "plugin/group_replication/include/observer_trans.h"
+#include "plugin/group_replication/include/opt_tracker.h"
 #include "plugin/group_replication/include/plugin.h"
 #include "plugin/group_replication/include/plugin_handlers/metrics_handler.h"
 #include "plugin/group_replication/include/plugin_messages/recovery_metadata_message_compressed_parts.h"
@@ -91,8 +92,11 @@ int Certifier_broadcast_thread::initialize() {
   if ((mysql_thread_create(key_GR_THD_cert_broadcast, &broadcast_pthd,
                            get_connection_attrib(), launch_broadcast_thread,
                            (void *)this))) {
-    mysql_mutex_unlock(&broadcast_run_lock); /* purecov: inspected */
-    return 1;                                /* purecov: inspected */
+    /* purecov: begin inspected */
+    mysql_mutex_unlock(&broadcast_run_lock);
+    LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CERT_BROADCAST_THREAD_CREATE_FAILED);
+    return 1;
+    /* purecov: end */
   }
   broadcast_thd_state.set_created();
 
@@ -107,13 +111,13 @@ int Certifier_broadcast_thread::initialize() {
   return 0;
 }
 
-int Certifier_broadcast_thread::terminate() {
+void Certifier_broadcast_thread::terminate() {
   DBUG_TRACE;
 
   mysql_mutex_lock(&broadcast_run_lock);
   if (broadcast_thd_state.is_thread_dead()) {
     mysql_mutex_unlock(&broadcast_run_lock);
-    return 0;
+    return;
   }
 
   aborted = true;
@@ -134,8 +138,6 @@ int Certifier_broadcast_thread::terminate() {
     mysql_cond_timedwait(&broadcast_run_cond, &broadcast_run_lock, &abstime);
   }
   mysql_mutex_unlock(&broadcast_run_lock);
-
-  return 0;
 }
 
 void Certifier_broadcast_thread::dispatcher() {
@@ -155,7 +157,16 @@ void Certifier_broadcast_thread::dispatcher() {
   mysql_cond_broadcast(&broadcast_run_cond);
   mysql_mutex_unlock(&broadcast_run_lock);
 
+  LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_CERT_BROADCAST_THREAD_STARTED);
+
   while (!aborted) {
+    // Increase Group Replication feature usage every 10 minutes.
+    if (broadcast_counter % 600 == 0 ||
+        DBUG_EVALUATE_IF("rpl_opt_tracker_small_tracking_period", true,
+                         false)) {
+      ++opt_option_tracker_usage_group_replication_plugin;
+    }
+
     // Broadcast Transaction identifiers every 30 seconds
     if (broadcast_counter % 30 == 0) {
       applier_module->get_pipeline_stats_member_collector()
@@ -208,6 +219,8 @@ void Certifier_broadcast_thread::dispatcher() {
   mysql_cond_broadcast(&broadcast_run_cond);
   mysql_mutex_unlock(&broadcast_run_lock);
 
+  LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_CERT_BROADCAST_THREAD_STOPPED);
+
   my_thread_exit(nullptr);
 }
 
@@ -316,6 +329,10 @@ Certifier::Certifier()
 Certifier::~Certifier() {
   mysql_mutex_lock(&LOCK_certification_info);
   initialized = false;
+
+  broadcast_thread->terminate();
+  delete broadcast_thread;
+
   clear_certification_info();
   delete certification_info_tsid_map;
 
@@ -326,7 +343,6 @@ Certifier::~Certifier() {
   delete group_gtid_extracted;
   delete group_gtid_tsid_map;
   mysql_mutex_unlock(&LOCK_certification_info);
-  delete broadcast_thread;
 
   mysql_mutex_lock(&LOCK_members);
   clear_members();
@@ -499,7 +515,15 @@ void Certifier::clear_certification_info() {
   for (Certification_info::iterator it = certification_info.begin();
        it != certification_info.end(); ++it) {
     // We can only delete the last reference.
-    if (it->second->unlink() == 0) delete it->second;
+    if (it->second->unlink() == 0) {
+      /*
+        Claim Gtid_set_ref used memory to
+        `thread/group_rpl/THD_certifier_broadcast` thread, since this is thread
+        that does release the memory.
+      */
+      it->second->claim_memory_ownership(true);
+      delete it->second;
+    }
   }
 
   certification_info.clear();
@@ -550,15 +574,6 @@ int Certifier::initialize(ulonglong gtid_assignment_block_size) {
   return error;
 }
 
-int Certifier::terminate() {
-  DBUG_TRACE;
-  int error = 0;
-
-  if (is_initialized()) error = broadcast_thread->terminate();
-
-  return error;
-}
-
 void Certifier::update_parallel_applier_indexes(
     bool update_parallel_applier_last_committed_global,
     bool increment_parallel_applier_sequence_number) {
@@ -719,6 +734,15 @@ Certification_result Certifier::add_writeset_to_certification_info(
         item_previous_sequence_number != parallel_applier_sequence_number)
       transaction_last_committed = item_previous_sequence_number;
   }
+  /*
+    The memory used by Gtid_set_ref is allocated by
+    `thread/group_rpl/THD_applier_module_receiver`, though it will be released
+    by `thread/group_rpl/THD_certifier_broadcast` thread.  To avoid untracked
+    memory release on `thread/group_rpl/THD_applier_module_receiver` we do
+    dissociate this used memory from this thread.
+  */
+  snapshot_version_value->claim_memory_ownership(false);
+
   return Certification_result::positive;
 }
 
@@ -748,7 +772,7 @@ namespace {
                            added to group_gtid_executed as part of initial
                            try(step 2).
 */
-[[NODISCARD]] Certification_result check_gtid_collision(
+[[nodiscard]] Certification_result check_gtid_collision(
     rpl_sidno gtid_group_sidno, rpl_sidno gtid_global_sidno, rpl_gno gno,
     Gtid_set &group_gtid_executed, const std::string &sid_str) {
   if (group_gtid_executed.contains_gtid(gtid_group_sidno, gno)) {
@@ -769,6 +793,7 @@ void Certifier::update_transaction_dependency_timestamps(
     Gtid_log_event &gle, bool has_write_set, bool has_write_set_large_size,
     int64 transaction_last_committed) {
   bool update_parallel_applier_last_committed_global = false;
+  bool is_empty_transaction = false;
 
   /*
     'CREATE TABLE ... AS SELECT' is considered a DML, though in reality it
@@ -781,8 +806,19 @@ void Certifier::update_transaction_dependency_timestamps(
     update_parallel_applier_last_committed_global = true;
   }
 
-  if (!has_write_set || has_write_set_large_size ||
-      update_parallel_applier_last_committed_global) {
+  /*
+    Empty transactions, despite not having write-set, can be
+    applied in parallel with any other transaction.
+    Empty transactions are assigned `last_committed = -1` by GR
+    before send.
+  */
+  else if (!has_write_set && -1 == gle.last_committed) {
+    is_empty_transaction = true;
+  }
+
+  if (!is_empty_transaction &&
+      (!has_write_set || has_write_set_large_size ||
+       update_parallel_applier_last_committed_global)) {
     /*
       DDL does not have write-set, so we need to ensure that it
       is applied without any other transaction in parallel.
@@ -797,8 +833,9 @@ void Certifier::update_transaction_dependency_timestamps(
   assert(gle.last_committed < gle.sequence_number);
 
   update_parallel_applier_indexes(
-      !has_write_set || has_write_set_large_size ||
-          update_parallel_applier_last_committed_global,
+      (!is_empty_transaction &&
+       (!has_write_set || has_write_set_large_size ||
+        update_parallel_applier_last_committed_global)),
       true);
 
   /*
@@ -855,7 +892,7 @@ Certified_gtid Certifier::certify(Gtid_set *snapshot_version,
     &is_gtid_specified, &gtid_global_sidno, &gtid_group_sidno, &gtid_gno,
     local_transaction,
     this
-  ](Certification_result result) -> auto {
+  ](Certification_result result) -> auto{
     update_certified_transaction_count(result == Certification_result::positive,
                                        local_transaction);
     return end_certification_result(gtid_global_sidno, gtid_group_sidno,
@@ -1043,7 +1080,15 @@ bool Certifier::add_item(const char *item, Gtid_set_ref *snapshot_version,
     *item_previous_sequence_number =
         it->second->get_parallel_applier_sequence_number();
 
-    if (it->second->unlink() == 0) delete it->second;
+    if (it->second->unlink() == 0) {
+      /*
+        Claim Gtid_set_ref used memory to
+        `thread/group_rpl/THD_certifier_broadcast` thread, since this is thread
+        that does release the memory.
+      */
+      it->second->claim_memory_ownership(true);
+      delete it->second;
+    }
 
     it->second = snapshot_version;
     error = false;
@@ -1230,12 +1275,41 @@ void Certifier::garbage_collect_internal(Gtid_set *executed_gtid_set,
       Certification_info::iterator it = certification_info.begin();
       stable_gtid_set_lock->wrlock();
 
+      uint64 garbage_collector_counter =
+          metrics_handler->get_certification_garbage_collector_count();
+
+      DBUG_EXECUTE_IF("group_replication_garbage_collect_counter_overflow", {
+        DBUG_SET("-d,group_replication_garbage_collect_counter_overflow");
+        garbage_collector_counter = 0;
+      });
+
       while (it != certification_info.end()) {
-        if (it->second->is_subset_not_equals(stable_gtid_set)) {
-          if (it->second->unlink() == 0) delete it->second;
+        uint64 write_set_counter = it->second->get_garbage_collect_counter();
+
+        /*
+           we need to clear gtid_set_ref if marked with UINT64_MAX or
+           subset_not_equals of stable_gtid_set
+        */
+        if (write_set_counter == UINT64_MAX ||
+            (write_set_counter < garbage_collector_counter &&
+             it->second->is_subset_not_equals(stable_gtid_set))) {
+          it->second->set_garbage_collect_counter(UINT64_MAX);
+          if (it->second->unlink() == 0) {
+            /*
+              Claim Gtid_set_ref used memory to
+              `thread/group_rpl/THD_certifier_broadcast` thread, since this is
+              thread that does release the memory.
+            */
+            it->second->claim_memory_ownership(true);
+            delete it->second;
+          }
           certification_info.erase(it++);
-        } else
+        } else {
+          DBUG_EXECUTE_IF("group_replication_ci_rows_counter_high",
+                          { assert(write_set_counter > 0); });
+          it->second->set_garbage_collect_counter(garbage_collector_counter);
           ++it;
+        }
       }
       stable_gtid_set_lock->unlock();
     }
@@ -1750,6 +1824,15 @@ bool Certifier::set_certification_info_part(
       value->link();
       certification_info.insert(
           std::pair<std::string, Gtid_set_ref *>(key, value));
+      /*
+        The memory used by Gtid_set_ref is allocated by
+        `thread/group_rpl/THD_applier_module_receiver`, though it will be
+        released by `thread/group_rpl/THD_certifier_broadcast` thread.  To avoid
+        untracked memory release on
+        `thread/group_rpl/THD_applier_module_receiver` we do dissociate this
+        used memory from this thread.
+      */
+      value->claim_memory_ownership(false);
     }
 
     return false;
@@ -1999,6 +2082,14 @@ int Certifier::set_certification_info(
     value->link();
     certification_info.insert(
         std::pair<std::string, Gtid_set_ref *>(key, value));
+    /*
+      The memory used by Gtid_set_ref is allocated by
+      `thread/group_rpl/THD_applier_module_receiver`, though it will be released
+      by `thread/group_rpl/THD_certifier_broadcast` thread.  To avoid untracked
+      memory release on `thread/group_rpl/THD_applier_module_receiver` we do
+      dissociate this used memory from this thread.
+    */
+    value->claim_memory_ownership(false);
   }
 
   if (initialize_server_gtid_set()) {
