@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,8 +26,8 @@
 #include <fcntl.h>
 #include <mysql/plugin.h>
 #include <mysql/service_srv_session_info.h>
-#include <stdlib.h>
 #include <sys/types.h>
+#include <cstdlib>
 
 #include <mysql/components/my_service.h>
 #include <mysql/components/services/log_builtins.h>
@@ -41,6 +41,7 @@
 #include "sql_string.h" /* STRING_PSI_MEMORY_KEY */
 #include "template_utils.h"
 #include "thr_cond.h"
+#include "thr_mutex.h"
 
 static constexpr int STRING_BUFFER = 1024 * 4;
 
@@ -280,7 +281,7 @@ static void sql_handle_ok(void *ctx, uint server_status,
                           ulonglong last_insert_id, const char *const message) {
   DBUG_TRACE;
 
-  Callback_data *cbd = (Callback_data *)ctx;
+  auto *cbd = (Callback_data *)ctx;
 
   cbd->server_status = server_status;
   cbd->warn_count = statement_warn_count;
@@ -293,7 +294,7 @@ static void sql_handle_error(void *ctx, uint sql_errno,
                              const char *const err_msg,
                              const char *const sqlstate) {
   DBUG_TRACE;
-  Callback_data *cbd = (Callback_data *)ctx;
+  auto *cbd = (Callback_data *)ctx;
   WRITE_VAL("ERROR %i %s\n", sql_errno, err_msg);
   cbd->error_called = true;
   cbd->err = sql_errno;
@@ -303,7 +304,7 @@ static void sql_handle_error(void *ctx, uint sql_errno,
 
 static void sql_shutdown(void *ctx, int shutdown_server) {
   DBUG_TRACE;
-  Callback_data *cbd = (Callback_data *)ctx;
+  auto *cbd = (Callback_data *)ctx;
 
   cbd->shutdown = shutdown_server;
   cbd->shutdown_called = true;
