@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,9 +25,9 @@
 
 #include <fcntl.h>
 #include <mysql/plugin.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <sys/types.h>
+#include <cstdio>
+#include <cstdlib>
 
 #include <mysql/components/my_service.h>
 #include <mysql/components/services/log_builtins.h>
@@ -217,7 +218,7 @@ struct st_plugin_ctx {
 
 static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
                                      const CHARSET_INFO *resultcs) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_start_result_metadata\n");
   DBUG_TRACE;
   DBUG_PRINT("info", ("resultcs->number: %d", resultcs->number));
@@ -232,7 +233,7 @@ static int sql_start_result_metadata(void *ctx, uint num_cols, uint,
 
 static int sql_field_metadata(void *ctx, struct st_send_field *field,
                               const CHARSET_INFO *) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   st_send_field_n *cfield = &pctx->sql_field[pctx->current_col];
   //  WRITE_STR("sql_field_metadata\n");
   DBUG_TRACE;
@@ -264,7 +265,7 @@ static int sql_field_metadata(void *ctx, struct st_send_field *field,
 
 static int sql_end_result_metadata(void *ctx, uint server_status,
                                    uint warn_count) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_end_result_metadata\n");
   DBUG_TRACE;
   pctx->meta_server_status = server_status;
@@ -274,7 +275,7 @@ static int sql_end_result_metadata(void *ctx, uint server_status,
 }
 
 static int sql_start_row(void *ctx) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_start_row\n");
   DBUG_TRACE;
   pctx->current_col = 0;
@@ -282,7 +283,7 @@ static int sql_start_row(void *ctx) {
 }
 
 static int sql_end_row(void *ctx) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_end_row\n");
   DBUG_TRACE;
   pctx->num_rows++;
@@ -297,7 +298,7 @@ static ulong sql_get_client_capabilities(void *) {
 }
 
 static int sql_get_null(void *ctx) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_null\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -311,7 +312,7 @@ static int sql_get_null(void *ctx) {
 }
 
 static int sql_get_integer(void *ctx, longlong value) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_integer\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -328,7 +329,7 @@ static int sql_get_integer(void *ctx, longlong value) {
 }
 
 static int sql_get_longlong(void *ctx, longlong value, uint is_unsigned) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_longlong\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -352,7 +353,7 @@ static const char *test_decimal_as_string(char *buff, const decimal_t *val,
 }
 
 static int sql_get_decimal(void *ctx, const decimal_t *value) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_decimal\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -367,7 +368,7 @@ static int sql_get_decimal(void *ctx, const decimal_t *value) {
 }
 
 static int sql_get_double(void *ctx, double value, uint32) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_double\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -384,7 +385,7 @@ static int sql_get_double(void *ctx, double value, uint32) {
 }
 
 static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_date\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -401,7 +402,7 @@ static int sql_get_date(void *ctx, const MYSQL_TIME *value) {
 }
 
 static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_time\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -419,7 +420,7 @@ static int sql_get_time(void *ctx, const MYSQL_TIME *value, uint) {
 }
 
 static int sql_get_datetime(void *ctx, const MYSQL_TIME *value, uint) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_datetime\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -438,7 +439,7 @@ static int sql_get_datetime(void *ctx, const MYSQL_TIME *value, uint) {
 
 static int sql_get_string(void *ctx, const char *const value, size_t length,
                           const CHARSET_INFO *const) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_get_string\n");
   DBUG_TRACE;
   const uint row = pctx->num_rows;
@@ -454,7 +455,7 @@ static int sql_get_string(void *ctx, const char *const value, size_t length,
 static void sql_handle_ok(void *ctx, uint server_status,
                           uint statement_warn_count, ulonglong affected_rows,
                           ulonglong last_insert_id, const char *const message) {
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_handle_ok\n");
   DBUG_TRACE;
   /* This could be an EOF */
@@ -471,7 +472,7 @@ static void sql_handle_error(void *ctx, uint sql_errno,
                              const char *const err_msg,
                              const char *const sqlstate) {
   char buffer[1024];
-  struct st_plugin_ctx *pctx = (struct st_plugin_ctx *)ctx;
+  auto *pctx = (struct st_plugin_ctx *)ctx;
   //  WRITE_STR("sql_handle_error\n");
   DBUG_TRACE;
   WRITE_VAL2("[%u][%s]", sql_errno, sqlstate);
@@ -598,7 +599,6 @@ static char *fieldflags2str(uint f) {
   ff2s_check_flag(NO_DEFAULT_VALUE);
   ff2s_check_flag(NUM);
   ff2s_check_flag(PART_KEY);
-  ff2s_check_flag(GROUP);
   ff2s_check_flag(UNIQUE);
   ff2s_check_flag(BINCMP);
   ff2s_check_flag(ON_UPDATE_NOW);
@@ -822,20 +822,17 @@ static void test_selects(MYSQL_SESSION session, void *p) {
   DBUG_TRACE;
   char buffer[STRING_BUFFER_SIZE];
 
-  struct st_plugin_ctx *plugin_ctx = new st_plugin_ctx();
+  auto *plugin_ctx = new st_plugin_ctx();
 
   const char *last_db = nullptr;
-  const size_t stmt_count =
-      sizeof(test_query_plan) / sizeof(test_query_plan[0]);
-  for (size_t i = 0; i < stmt_count; i++) {
+  for (auto &i : test_query_plan) {
     /* Change current DB if needed */
-    if (last_db != test_query_plan[i].db) {
-      last_db = test_query_plan[i].db;
+    if (last_db != i.db) {
+      last_db = i.db;
 
       change_current_db(session, last_db ? last_db : "", plugin_ctx, p);
     }
-    run_statement(session, test_query_plan[i].query, plugin_ctx,
-                  test_query_plan[i].generates_result_set, p);
+    run_statement(session, i.query, plugin_ctx, i.generates_result_set, p);
 
     last_db = nullptr;
   }
@@ -850,7 +847,7 @@ static void test_sql(void *p) {
   DBUG_TRACE;
 
   WRITE_STR("[srv_session_open]\n");
-  MYSQL_SESSION session = srv_session_open(NULL, NULL);
+  MYSQL_SESSION session = srv_session_open(nullptr, nullptr);
   if (!session) {
     LogPluginErr(ERROR_LEVEL, ER_LOG_PRINTF_MSG, "srv_session_open failed.");
     return;
@@ -875,7 +872,7 @@ struct test_thread_context {
 
 static void *test_sql_threaded_wrapper(void *param) {
   char buffer[STRING_BUFFER_SIZE];
-  struct test_thread_context *context = (struct test_thread_context *)param;
+  auto *context = (struct test_thread_context *)param;
 
   WRITE_SEP();
   WRITE_STR("init thread\n");
