@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2019, 2025, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -53,7 +53,7 @@ namespace xpl {
 
 std::string generate_hash(const std::string &input) {
   std::string hash(2 * SHA1_HASH_SIZE + 2, '\0');
-  ::scrambled_input(&hash[0], input.c_str(), input.length());
+  ::scrambled_input(hash.data(), input.c_str(), input.length());
   hash.resize(2 * SHA1_HASH_SIZE);  // strip the \0
   return hash;
 }
