@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2021, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2021, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -128,7 +128,7 @@ static void init(mysql_harness::PluginFuncEnv *env) {
               std::chrono::seconds{config.idle_timeout}));
     }
 
-    init_guard.commit();
+    init_guard.release();
   } catch (const std::invalid_argument &exc) {
     set_error(env, mysql_harness::kConfigInvalidArgument, "%s", exc.what());
 
