@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -95,7 +95,7 @@ namespace logging {
 
 // throws std::logic_error
 void Registry::create_logger(const std::string &name, LogLevel level) {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::unique_lock lock(mtx_);
   auto result = loggers_.emplace(name, Logger(*this, level));
   if (result.second == false)
     throw std::logic_error("Duplicate logger '" + name + "'");
@@ -103,14 +103,14 @@ void Registry::create_logger(const std::string &name, LogLevel level) {
 
 // throws std::logic_error
 void Registry::remove_logger(const std::string &name) {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::unique_lock lock(mtx_);
   if (loggers_.erase(name) == 0)
     throw std::logic_error("Removing non-existant logger '" + name + "'");
 }
 
 // throws std::logic_error
 Logger Registry::get_logger(const std::string &name) const {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::shared_lock lock(mtx_);
 
   auto it = loggers_.find(name);
   if (it == loggers_.end())
@@ -121,7 +121,7 @@ Logger Registry::get_logger(const std::string &name) const {
 
 Logger Registry::get_logger_or_default(const std::string &name,
                                        const std::string &default_name) const {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::shared_lock lock(mtx_);
 
   auto it = loggers_.find(name);
   if (it != loggers_.end()) return it->second;
@@ -137,7 +137,7 @@ void Registry::update_logger(const std::string &name, const Logger &logger) {
   // this internally locks mtx_, so we call it before we lock it for good here
   const std::set<std::string> handlers_in_registry = get_handler_names();
 
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::unique_lock lock(mtx_);
 
   // verify logger exists
   auto it = loggers_.find(name);
@@ -155,14 +155,17 @@ void Registry::update_logger(const std::string &name, const Logger &logger) {
 }
 
 std::set<std::string> Registry::get_logger_names() const {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::shared_lock lock(mtx_);
+
   std::set<std::string> result;
   for (const auto &pair : loggers_) result.emplace(pair.first);
+
   return result;
 }
 
-void Registry::flush_all_loggers(const std::string dst) {
-  std::lock_guard<std::mutex> lock(mtx_);
+void Registry::flush_all_loggers(const std::string &dst) {
+  std::unique_lock lock(mtx_);
+
   for (const auto &handler : handlers_) {
     handler.second->reopen(dst);
   }
@@ -176,7 +179,7 @@ void Registry::flush_all_loggers(const std::string dst) {
 
 // throws std::logic_error
 void Registry::add_handler(std::string name, std::shared_ptr<Handler> handler) {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::unique_lock lock(mtx_);
 
   auto result = handlers_.emplace(name, handler);
   if (!result.second)
@@ -185,7 +188,7 @@ void Registry::add_handler(std::string name, std::shared_ptr<Handler> handler) {
 
 // throws std::logic_error
 void Registry::remove_handler(std::string name) {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::unique_lock lock(mtx_);
 
   auto it = handlers_.find(name);
   if (it == handlers_.end())
@@ -199,7 +202,7 @@ void Registry::remove_handler(std::string name) {
 
 // throws std::logic_error
 std::shared_ptr<Handler> Registry::get_handler(const std::string &name) const {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::shared_lock lock(mtx_);
 
   auto it = handlers_.find(name);
   if (it == handlers_.end())
@@ -209,14 +212,16 @@ std::shared_ptr<Handler> Registry::get_handler(const std::string &name) const {
 }
 
 std::set<std::string> Registry::get_handler_names() const {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::shared_lock lock(mtx_);
+
   std::set<std::string> result;
   for (const auto &pair : handlers_) result.emplace(pair.first);
+
   return result;
 }
 
 bool Registry::is_handled(LogLevel level) const {
-  std::lock_guard<std::mutex> lock(mtx_);
+  std::shared_lock lock(mtx_);
 
   for (const auto &handler_pair : handlers_) {
     if (level <= handler_pair.second->get_level()) return true;
@@ -233,6 +238,10 @@ bool Registry::is_handled(LogLevel level) const {
 
 static std::string g_main_app_log_domain;
 
+Logger Registry::get_logger_or_default(const std::string &name) const {
+  return get_logger_or_default(name, g_main_app_log_domain);
+}
+
 void attach_handler_to_all_loggers(Registry &registry,
                                    std::string handler_name) {
   for (const std::string &logger_name : registry.get_logger_names()) {
@@ -251,6 +260,15 @@ void set_log_level_for_all_loggers(Registry &registry, LogLevel level) {
   }
 }
 
+void set_log_level_for_handler(std::string name, LogLevel level) {
+  auto &registry = DIM::instance().get_LoggingRegistry();
+  try {
+    auto handler = registry.get_handler(name);
+    handler->set_level(level);
+  } catch (std::logic_error &) {
+  }
+}
+
 void set_log_level_for_all_handlers(const Registry &registry, LogLevel level) {
   for (const std::string &handler_name : registry.get_handler_names()) {
     std::shared_ptr<Handler> handler = registry.get_handler(handler_name);
@@ -490,6 +508,12 @@ void unregister_handler(std::string name) {
   registry.remove_handler(name);
 }
 
+bool handler_registered(std::string name) {
+  mysql_harness::logging::Registry &registry =
+      mysql_harness::DIM::instance().get_LoggingRegistry();
+  return registry.get_handler_names().count(name) > 0;
+}
+
 void set_log_level_for_all_loggers(LogLevel level) {
   mysql_harness::logging::Registry &registry =
       mysql_harness::DIM::instance().get_LoggingRegistry();
