@@ -25,15 +25,11 @@
 
 #include "mysql/harness/plugin_config.h"
 
-#include <stdexcept>
-
 #ifndef _WIN32
 #include <sys/un.h>
 #include <unistd.h>
 #endif
 
-#include "tcp_address.h"
-
 namespace mysql_harness {
 
 /* static */
@@ -81,12 +77,11 @@ std::optional<std::string> BasePluginConfig::get_option_string_(
     }
 
     return value;
-  } else {
-    try {
-      return value = section->get(option);
-    } catch (const mysql_harness::bad_option &) {
-      return {};
-    }
+  }
+  try {
+    return value = section->get(option);
+  } catch (const mysql_harness::bad_option &) {
+    return {};
   }
 }
 
