@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -41,7 +41,7 @@
 #include <condition_variable>
 #include <cstdarg>
 #include <cstdlib>
-#include <cstring>
+#include <cstring>  // strcmp
 #include <mutex>
 #include <stdexcept>
 #include <thread>
