@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -35,10 +35,10 @@
 #include <sys/types.h>
 #include <cerrno>
 
-#include "digest.h"
 #include "http_auth_error.h"
 #include "kdf_pbkdf2.h"
 #include "kdf_sha_crypt.h"
+#include "mysqlrouter/digest.h"
 
 std::pair<std::error_code, struct stat> FileMeta::stat(
     const std::string &filename) {
