@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2022, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2022, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -28,7 +28,7 @@
 
 #include <array>
 
-static constexpr std::array rest_plugin_supported_options = {
+static constexpr std::array plugin_supported_options = {
     "require_realm",
 };
 
