@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -61,9 +61,9 @@ std::string require_realm_metadata_cache;
 
 using StringOption = mysql_harness::StringOption;
 
-#define GET_OPTION_CHECKED(option, section, name, value)                      \
-  static_assert(                                                              \
-      mysql_harness::str_in_collection(rest_plugin_supported_options, name)); \
+#define GET_OPTION_CHECKED(option, section, name, value)                 \
+  static_assert(                                                         \
+      mysql_harness::str_in_collection(plugin_supported_options, name)); \
   option = get_option(section, name, value);
 
 class RestMetadataCachePluginConfig : public mysql_harness::BasePluginConfig {
@@ -373,6 +373,54 @@ static void spec_adder(RestApiComponent::JsonDocument &spec_doc) {
                                               JsonValue(rapidjson::kObjectType)
                                                   .AddMember("type", "integer",
                                                              allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "X_port",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "integer",
+                                                             allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "UUID",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "string",
+                                                             allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "Cluster_name",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "string",
+                                                             allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "member_role",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "string",
+                                                             allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "ClusterSet_name",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "string",
+                                                             allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "Cluster_role",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "string",
+                                                             allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "label",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "string",
+                                                             allocator),
+                                              allocator)
+                                          .AddMember(
+                                              "tags",
+                                              JsonValue(rapidjson::kObjectType)
+                                                  .AddMember("type", "string",
+                                                             allocator),
                                               allocator),
                                       allocator),
                               allocator),
@@ -950,8 +998,8 @@ mysql_harness::Plugin DLLEXPORT harness_plugin_rest_metadata_cache = {
     start,    // start
     nullptr,  // stop
     true,     // declares_readiness
-    rest_plugin_supported_options.size(),
-    rest_plugin_supported_options.data(),
+    plugin_supported_options.size(),
+    plugin_supported_options.data(),
     expose_configuration,
 };
 }
