@@ -35,15 +35,18 @@
  *
  */
 
+#include "keyring_handler.h"
 #include "mysql/harness/arg_handler.h"
 #include "mysql/harness/loader.h"
 #include "mysql/harness/signal_handler.h"
 #include "mysqlrouter/keyring_info.h"
 #include "mysqlrouter/sys_user_operations.h"
+#include "router_conf.h"
 
 #include <cstdint>
 #include <iostream>  // cerr, cout
 #include <stdexcept>
+#include <string>
 #include <vector>
 
 static const size_t kHelpScreenWidth = 72;
@@ -96,7 +99,8 @@ class MySQLRouter {
         ,
         sys_user_operations_(mysqlrouter::SysUserOperations::instance())
 #endif
-  {
+        ,
+        bootstrapper_(keyring_.get_ki()) {
   }
 
   /** @brief Constructor with command line arguments as vector
@@ -375,23 +379,6 @@ class MySQLRouter {
    */
   void show_help();
 
-  /** @brief Saves the selected command line option in the internal options
-   * array after verifying it's value not empty and the router is doing
-   * bootstrap.
-   *
-   *  Throws: std::runtime_error
-   */
-  void save_bootstrap_option_not_empty(const std::string &option_name,
-                                       const std::string &save_name,
-                                       const std::string &option_value);
-
-  /**
-   * @brief verify that bootstrap option (--bootstrap or -B) was given by user.
-   *
-   * @throw std::runtime_error if called in non-bootstrap mode.
-   */
-  void assert_bootstrap_mode(const std::string &option_name) const;
-
   /**
    * @brief verify that option given by user is not used with bootstrap option
    * (--bootstrap or -B).
@@ -400,16 +387,6 @@ class MySQLRouter {
    */
   void assert_not_bootstrap_mode(const std::string &option_name) const;
 
-  /**
-   * @brief verify that option given by user is an integer value in the given
-   * range.
-   *
-   * @throw std::out_of_range - option not in [min, max] range
-   * @throw std::invalid_argument - not a valid integer
-   */
-  void assert_option_value_in_range(const std::string &option_value,
-                                    const int min, const int max) const;
-
   /** @brief Shows command line usage and option description
    *
    * Shows command line usage and all available options together with their
@@ -453,10 +430,6 @@ class MySQLRouter {
    */
   void set_default_config_files(const char *locations) noexcept;
 
-  void bootstrap(const std::string &program_name,
-                 const std::string &metadata_server_uri,
-                 const std::string &plugin_folder = "");
-
   /*
    * @brief returns id of the router.
    *
@@ -467,28 +440,6 @@ class MySQLRouter {
   void init_keyring(mysql_harness::Config &config);
 
   void init_dynamic_state(mysql_harness::Config &config);
-
-  /**
-   * @brief Initializes keyring using master-key-reader and master-key-writer.
-   *
-   * @throw MasterKeyReadError
-   */
-  void init_keyring_using_external_facility(mysql_harness::Config &config);
-
-  /**
-   * @brief Initializes keyring using master key file.
-   *
-   * @throw std::runtime_error
-   */
-  void init_keyring_using_master_key_file();
-
-  /**
-   * @brief Initializes keyring using password read from STDIN.
-   *
-   * @throw std::runtime_error
-   */
-  void init_keyring_using_prompted_password();
-
   // throws std::runtime_error
   void init_loader(mysql_harness::LoaderConfig &config);
 
@@ -497,8 +448,6 @@ class MySQLRouter {
       const std::map<std::string, std::string> params,
       const std::vector<std::string> &config_files);
 
-  std::map<std::string, std::string> get_default_paths() const;
-
   /** @brief Tuple describing the MySQL Router version, with major, minor and
    * patch level **/
   std::tuple<const uint8_t, const uint8_t, const uint8_t> version_;
@@ -510,6 +459,7 @@ class MySQLRouter {
   /** @brief Vector with configuration files passed through command line
    * arguments **/
   std::vector<std::string> config_files_;
+
   /** @brief PID file location **/
   std::string pid_file_path_;
   bool pid_file_created_{false};
@@ -523,25 +473,6 @@ class MySQLRouter {
   /** @brief Whether we are showing information on command line, for example,
    * using --help or --version **/
   bool showing_info_;
-  /**
-   * @brief Value of the argument passed to the -B or --bootstrap
-   *        command line option for bootstrapping.
-   */
-  std::string bootstrap_uri_;
-  /**
-   * @brief Valueof the argument passed to the --directory command line option
-   */
-  std::string bootstrap_directory_;
-  /**
-   * @brief key/value map of additional configuration options for bootstrap
-   */
-  std::map<std::string, std::string> bootstrap_options_;
-
-  /**
-   * @brief key/list-of-values map of additional configuration options for
-   * bootstrap
-   */
-  std::map<std::string, std::vector<std::string>> bootstrap_multivalue_options_;
 
   /**
    * Path to origin of executable.
@@ -551,7 +482,7 @@ class MySQLRouter {
    */
   mysql_harness::Path origin_;
 
-  KeyringInfo keyring_info_;
+  KeyringHandler keyring_;
 
   std::ostream &out_stream_;
   std::ostream &err_stream_;
@@ -572,22 +503,18 @@ class MySQLRouter {
 
   /** @brief Pointer to the object to be used to perform system specific
    * user-related operations **/
-  mysqlrouter::SysUserOperationsBase *sys_user_operations_;
+  mysqlrouter::SysUserOperationsBase *sys_user_operations_ = nullptr;
 #endif
 
   mysql_harness::SignalHandler signal_handler_;
 
+  MySQLRouterConf bootstrapper_;
 #ifdef FRIEND_TEST
   FRIEND_TEST(Bug24909259, PasswordPrompt_plain);
   FRIEND_TEST(Bug24909259, PasswordPrompt_keyed);
 #endif
 };
 
-class silent_exception : public std::exception {
- public:
-  silent_exception() : std::exception() {}
-};
-
 /** @brief Expose the configured application-level options (the ones
  * from [DEFAULT] section that are not specific to any particular plugin and are
  * relevant from the Cluster metadata perspective) **/
