@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2022, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2022, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -33,6 +33,7 @@
 #include <ratio>
 #include <sstream>
 
+#include "classic_auth_caching_sha2.h"
 #include "classic_change_user_sender.h"
 #include "classic_connect.h"
 #include "classic_connection_base.h"
@@ -40,7 +41,6 @@
 #include "classic_greeting_forwarder.h"  // ServerGreetor
 #include "classic_init_schema_sender.h"
 #include "classic_query_sender.h"
-#include "classic_quit_sender.h"
 #include "classic_reset_connection_sender.h"
 #include "classic_set_option_sender.h"
 #include "mysql/harness/logging/logging.h"
@@ -48,6 +48,7 @@
 #include "mysql_com.h"
 #include "mysqlrouter/classic_protocol_message.h"
 #include "mysqlrouter/connection_pool_component.h"
+#include "mysqlrouter/datatypes.h"
 #include "mysqlrouter/utils.h"  // to_string
 #include "router_require.h"
 #include "sql_value.h"  // sql_value_to_string
@@ -78,9 +79,10 @@ class FailedQueryHandler : public QuerySender::Handler {
 
 class IsTrueHandler : public QuerySender::Handler {
  public:
-  IsTrueHandler(LazyConnector &processor,
+  IsTrueHandler(LazyConnector &processor, std::string stmt,
                 classic_protocol::message::server::Error on_cond_fail_error)
       : processor_(processor),
+        stmt_(std::move(stmt)),
         on_condition_fail_error_(std::move(on_cond_fail_error)) {}
 
   void on_column_count(uint64_t count) override {
@@ -123,7 +125,7 @@ class IsTrueHandler : public QuerySender::Handler {
   }
 
   void on_error(const classic_protocol::message::server::Error &err) override {
-    log_warning("%s", err.message().c_str());
+    log_warning("%s failed: %s", stmt_.c_str(), err.message().c_str());
 
     processor_.failed(err);
   }
@@ -132,6 +134,8 @@ class IsTrueHandler : public QuerySender::Handler {
   LazyConnector &processor_;
   uint64_t row_count_{};
 
+  std::string stmt_;
+
   classic_protocol::message::server::Error on_condition_fail_error_;
 };
 
@@ -229,6 +233,8 @@ class SelectSessionVariablesHandler : public QuerySender::Handler {
 
 stdx::expected<Processor::Result, std::error_code> LazyConnector::process() {
   switch (stage()) {
+    case Stage::Init:
+      return init();
     case Stage::FromStash:
       return from_stash();
     case Stage::Connect:
@@ -293,6 +299,13 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::process() {
   harness_assert_this_should_not_execute();
 }
 
+stdx::expected<Processor::Result, std::error_code> LazyConnector::init() {
+  connection()->current_server_mode(connection()->expected_server_mode());
+
+  stage(Stage::FromStash);
+  return Result::Again;
+}
+
 stdx::expected<Processor::Result, std::error_code> LazyConnector::from_stash() {
   connection()->has_transient_error_at_connect(false);
 
@@ -315,13 +328,14 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::from_stash() {
           trace_event_from_stash = trace_span(ev, "mysql/from_stash");
         }
 
-        if (auto pop_res =
-                pool->unstash_mine(mysqlrouter::to_string(*ep), connection())) {
+        if (auto pop_res = pool->unstash_mine(ep->str(), connection())) {
           connection()->server_conn() = std::move(*pop_res);
 
           // reset the seq-id of the server side as this is a new command.
           connection()->server_protocol().seq_id(0xff);
 
+          connection()->server_address(connection()->server_conn().endpoint());
+
           if (auto &tr = tracer()) {
             tr.trace(Tracer::Event().stage(
                 "connect::from_stash::unstashed::mine: fd=" +
@@ -448,7 +462,8 @@ stdx::expected<Processor::Result, std::error_code> LazyConnector::connected() {
         connection(), in_handshake_,
         [this](const classic_protocol::message::server::Error &err) {
           if (connect_error_is_transient(err) &&
-              (connection()->client_protocol().password().has_value() ||
+              (connection()->client_protocol().credentials().get(
+                   AuthCachingSha2Password::kName) ||
                !connection()
                     ->server_protocol()
                     .server_greeting()
@@ -805,7 +820,7 @@ LazyConnector::wait_gtid_executed() {
                                         // didn't wait.
 
   if (connection()->wait_for_my_writes() &&
-      (connection()->expected_server_mode() ==
+      (connection()->current_server_mode() ==
        mysqlrouter::ServerMode::ReadOnly)) {
     auto gtid_executed = connection()->gtid_at_least_executed();
     if (!gtid_executed.empty()) {
@@ -823,19 +838,22 @@ LazyConnector::wait_gtid_executed() {
 
       std::ostringstream oss;
       if (max_replication_lag.count() == 0) {
-        oss << "SELECT GTID_SUBSET(" << std::quoted(gtid_executed)
+        // use ' to quote to make it ANSI_QUOTES safe.
+        oss << "SELECT GTID_SUBSET(" << std::quoted(gtid_executed, '\'')
             << ", @@GLOBAL.gtid_executed)";
       } else {
+        // use ' to quote to make it ANSI_QUOTES safe.
         oss << "SELECT NOT WAIT_FOR_EXECUTED_GTID_SET("
-            << std::quoted(gtid_executed) << ", "
+            << std::quoted(gtid_executed, '\'') << ", "
             << std::to_string(max_replication_lag.count()) << ")";
       }
 
       connection()->push_processor(std::make_unique<QuerySender>(
           connection(), oss.str(),
           std::make_unique<IsTrueHandler>(
-              *this, classic_protocol::message::server::Error{
-                         0, "wait_for_my_writes timed out", "HY000"})));
+              *this, oss.str(),
+              classic_protocol::message::server::Error{
+                  0, "wait_for_my_writes timed out", "HY000"})));
     }
   }
 
@@ -869,38 +887,20 @@ stdx::expected<Processor::Result, std::error_code>
 LazyConnector::pool_or_close() {
   stage(Stage::FallbackToWrite);
 
-#if 1
   connection()->stash_server_conn();
-#else
-  const auto pool_res = pool_server_connection();
-  if (!pool_res) return stdx::unexpected(pool_res.error());
-
-  const auto still_open = *pool_res;
-  if (still_open) {
-    if (auto &tr = tracer()) {
-      tr.trace(Tracer::Event().stage("connect::pooled"));
-    }
-
-  } else {
-    // connection wasn't pooled as the pool was full. close it.
-    if (auto &tr = tracer()) {
-      tr.trace(Tracer::Event().stage("connect::pool_full"));
-    }
-
-    connection()->push_processor(std::make_unique<QuitSender>(connection()));
-  }
-#endif
 
   return Result::Again;
 }
 
 stdx::expected<Processor::Result, std::error_code>
 LazyConnector::fallback_to_write() {
-  if (already_fallback_ || connection()->expected_server_mode() ==
-                               mysqlrouter::ServerMode::ReadWrite) {
+  if (already_fallback_ ||
+      (connection()->expected_server_mode() ==
+       mysqlrouter::ServerMode::ReadWrite) ||
+      (connection()->current_server_mode() ==
+       mysqlrouter::ServerMode::ReadWrite)) {
     // only fallback to the primary once and if the client is asking for
     // "read-only" nodes
-    //
 
     // failed() is already set.
 
@@ -912,7 +912,8 @@ LazyConnector::fallback_to_write() {
     tr.trace(Tracer::Event().stage("connect::fallback_to_write"));
   }
 
-  connection()->expected_server_mode(mysqlrouter::ServerMode::ReadWrite);
+  // connect to the read-write node in read-only mode.
+  connection()->current_server_mode(mysqlrouter::ServerMode::ReadWrite);
   already_fallback_ = true;
 
   // reset the failed state
