@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2021, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2021, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -33,9 +33,11 @@
 #include <mysqlx.pb.h>
 #include <mysqlx_connection.pb.h>
 
+#include "basic_protocol_splicer.h"
 #include "connection.h"  // MySQLRoutingConnectionBase
+#include "mysql/harness/destination.h"
 
-class XProtocolState : public ProtocolStateBase {
+class XProtocolState {
  public:
   struct FrameInfo {
     size_t frame_size_;            //!< size of the whole frame
@@ -61,26 +63,31 @@ class MysqlRoutingXConnection
     : public MySQLRoutingConnectionBase,
       public std::enable_shared_from_this<MysqlRoutingXConnection> {
  private:
+  using ClientSideConnection = TlsSwitchableClientConnection<XProtocolState>;
+  using ServerSideConnection = TlsSwitchableConnection<XProtocolState>;
+
   // constructor
   //
   // use ::create() instead.
   MysqlRoutingXConnection(
-      MySQLRoutingContext &context, RouteDestination *route_destination,
+      MySQLRoutingContext &context, DestinationManager *destination_manager,
       std::unique_ptr<ConnectionBase> client_connection,
       std::unique_ptr<RoutingConnectionBase> client_routing_connection,
       std::function<void(MySQLRoutingConnectionBase *)> remove_callback)
       : MySQLRoutingConnectionBase{context, std::move(remove_callback)},
-        route_destination_{route_destination},
-        destinations_{route_destination_->destinations()},
-        connector_{client_connection->io_ctx(), route_destination,
-                   destinations_},
-        socket_splicer_{std::make_unique<ProtocolSplicerBase>(
-            TlsSwitchableConnection{std::move(client_connection),
-                                    std::move(client_routing_connection),
-                                    context.source_ssl_mode(),
-                                    std::make_unique<XProtocolState>()},
-            TlsSwitchableConnection{nullptr, nullptr, context.dest_ssl_mode(),
-                                    std::make_unique<XProtocolState>()})} {}
+        connector_{client_connection->io_ctx(), context, destination_manager},
+        client_conn_{std::move(client_connection),
+                     std::move(client_routing_connection),
+                     context.source_ssl_mode(),
+                     ClientSideConnection::protocol_state_type{}},
+        server_conn_{nullptr, context.dest_ssl_mode(),
+                     ServerSideConnection::protocol_state_type{}} {
+    client_address(client_conn_.connection()->endpoint());
+
+    if (destination_manager->routing_guidelines_session_rand_used()) {
+      set_routing_guidelines_session_rand();
+    }
+  }
 
  public:
   using connector_type = Connector<std::unique_ptr<ConnectionBase>>;
@@ -88,10 +95,7 @@ class MysqlRoutingXConnection
   // create a shared_ptr<ThisClass>
   template <typename... Args>
   [[nodiscard]] static std::shared_ptr<MysqlRoutingXConnection> create(
-      // clang-format off
-      Args &&... args) {
-    // clang-format on
-
+      Args &&...args) {
     // can't use make_unique<> here as the constructor is private.
     return std::shared_ptr<MysqlRoutingXConnection>(
         new MysqlRoutingXConnection(std::forward<Args>(args)...));
@@ -107,20 +111,12 @@ class MysqlRoutingXConnection
       const std::string &msg, const std::string &sql_state = "HY000",
       Mysqlx::Error::Severity severity = Mysqlx::Error::ERROR);
 
-  SslMode source_ssl_mode() const {
-    return this->socket_splicer()->source_ssl_mode();
-  }
+  SslMode source_ssl_mode() const { return client_conn().ssl_mode(); }
 
-  SslMode dest_ssl_mode() const {
-    return this->socket_splicer()->dest_ssl_mode();
-  }
+  SslMode dest_ssl_mode() const { return server_conn().ssl_mode(); }
 
-  std::string get_client_address() const override {
-    return socket_splicer()->client_conn().endpoint();
-  }
-
-  std::string get_server_address() const override {
-    return socket_splicer()->server_conn().endpoint();
+  net::impl::socket::native_handle_type get_client_fd() const override {
+    return client_conn().native_handle();
   }
 
   void disconnect() override;
@@ -557,8 +553,8 @@ class MysqlRoutingXConnection
 
   void done();
 
-  stdx::expected<void, std::error_code> forward_tls(Channel *src_channel,
-                                                    Channel *dst_channel);
+  stdx::expected<void, std::error_code> forward_tls(Channel &src_channel,
+                                                    Channel &dst_channel);
 
   void forward_tls_client_to_server();
 
@@ -721,26 +717,59 @@ class MysqlRoutingXConnection
   void server_recv_session_reset_response_forward();
   void server_recv_session_reset_response_forward_last();
 
-  XProtocolState *client_protocol() {
-    return dynamic_cast<XProtocolState *>(
-        socket_splicer()->client_conn().protocol());
+  ClientSideConnection::protocol_state_type &client_protocol() {
+    return client_conn().protocol();
+  }
+  const ClientSideConnection::protocol_state_type &client_protocol() const {
+    return client_conn().protocol();
   }
 
-  XProtocolState *server_protocol() {
-    return dynamic_cast<XProtocolState *>(
-        socket_splicer()->server_conn().protocol());
+  ServerSideConnection::protocol_state_type &server_protocol() {
+    return server_conn().protocol();
+  }
+  const ServerSideConnection::protocol_state_type &server_protocol() const {
+    return server_conn().protocol();
   }
 
-  const ProtocolSplicerBase *socket_splicer() const {
-    return socket_splicer_.get();
-  }
+  ClientSideConnection &client_conn() { return client_conn_; }
+  const ClientSideConnection &client_conn() const { return client_conn_; }
 
-  ProtocolSplicerBase *socket_splicer() { return socket_splicer_.get(); }
+  ServerSideConnection &server_conn() { return server_conn_; }
+  const ServerSideConnection &server_conn() const { return server_conn_; }
 
-  std::string get_destination_id() const override {
+  std::optional<mysql_harness::Destination> get_destination_id()
+      const override {
     return connector().destination_id();
   }
 
+  std::optional<mysql_harness::DestinationEndpoint> destination_endpoint()
+      const override {
+    return std::nullopt;
+  }
+
+  std::string get_routing_source() const override {
+    return connector().routing_source();
+  }
+
+  void set_routing_source(std::string name) override {
+    return connector().set_routing_source(std::move(name));
+  }
+
+  void wait_until_completed() override {
+    is_completed_.wait([](auto ready) { return ready == true; });
+  }
+
+  void completed() override {
+    is_completed_.serialize_with_cv([](auto &ready, auto &cv) {
+      ready = true;
+      cv.notify_all();
+    });
+  }
+
+  routing_guidelines::Server_info get_server_info() const override {
+    return connector().server_info();
+  }
+
  private:
   int active_work_{0};
 
@@ -754,11 +783,12 @@ class MysqlRoutingXConnection
   connector_type &connector() { return connector_; }
   const connector_type &connector() const { return connector_; }
 
-  RouteDestination *route_destination_;
-  Destinations destinations_;
   connector_type connector_;
 
-  std::unique_ptr<ProtocolSplicerBase> socket_splicer_;
+  ClientSideConnection client_conn_;
+  ServerSideConnection server_conn_;
+
+  WaitableMonitor<bool> is_completed_{false};
 };
 
 #endif
