@@ -28,6 +28,8 @@ var common_responses = common_stmts.prepare_statement_responses(
       "router_select_cluster_type_v2",
       "router_select_view_id_v2_ar",
       "select_port",
+      "get_guidelines_router_info",
+      "get_routing_guidelines",
     ],
     options);
 
