@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+  Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,10 +31,10 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
-#include "cluster_metadata.h"
 #include "config_builder.h"
 #include "mysql/harness/filesystem.h"
 #include "mysql/harness/string_utils.h"  // join
+#include "mysqlrouter/cluster_metadata.h"
 #include "router_component_test.h"
 #include "router_test_helpers.h"  // init_windows_socket()
 
@@ -46,11 +47,16 @@ struct BootstrapTlsEndpointFailParams {
   const std::vector<std::string> cmdline_args;
 
   stdx::expected<void, std::string> expected_result;
+  bool new_executable{false};
 };
 
 class BootstrapTlsEndpointFail
     : public RouterComponentBootstrapTest,
-      public ::testing::WithParamInterface<BootstrapTlsEndpointFailParams> {};
+      public ::testing::WithParamInterface<BootstrapTlsEndpointFailParams> {
+ public:
+  BootstrapTlsEndpointFail()
+      : RouterComponentBootstrapTest(GetParam().new_executable) {}
+};
 
 TEST_P(BootstrapTlsEndpointFail, check) {
   // launch the router in bootstrap mode
@@ -68,7 +74,7 @@ TEST_P(BootstrapTlsEndpointFail, check) {
   EXPECT_NO_THROW(router.wait_for_exit());
 
   if (!GetParam().expected_result) {
-    // check if the bootstraping was successful
+    // check if the bootstrapping was successful
     EXPECT_THAT(router.get_full_output(),
                 ::testing::HasSubstr(GetParam().expected_result.error()));
   }
@@ -78,141 +84,133 @@ const BootstrapTlsEndpointFailParams bootstrap_tls_endpoint_fail_params[] = {
     // client-ssl-mode
     {"client_ssl_mode_invalid",  // BS_ARGS_BAD_01
      {"--client-ssl-mode", "foo"},
-     stdx::make_unexpected(
-         "value 'foo' provided to --client-ssl-mode is not one "
-         "of DISABLED,PREFERRED,REQUIRED,PASSTHROUGH"s)},
+     stdx::unexpected("value 'foo' provided to --client-ssl-mode is not one "
+                      "of DISABLED,PREFERRED,REQUIRED,PASSTHROUGH"s)},
     {"client_ssl_mode_empty",  // BS_ARGS_BAD_02
      {"--client-ssl-mode", ""},
-     stdx::make_unexpected("value '' provided to --client-ssl-mode is not one "
-                           "of DISABLED,PREFERRED,REQUIRED,PASSTHROUGH"s)},
+     stdx::unexpected("value '' provided to --client-ssl-mode is not one "
+                      "of DISABLED,PREFERRED,REQUIRED,PASSTHROUGH"s)},
     {"client_ssl_mode_space",  // BS_ARGS_BAD_03
      {"--client-ssl-mode", " "},
-     stdx::make_unexpected("value ' ' provided to --client-ssl-mode is not one "
-                           "of DISABLED,PREFERRED,REQUIRED,PASSTHROUGH"s)},
+     stdx::unexpected("value ' ' provided to --client-ssl-mode is not one "
+                      "of DISABLED,PREFERRED,REQUIRED,PASSTHROUGH"s)},
     {"client_ssl_mode_no_value",  // BS_ARGS_BAD_04
      {"--client-ssl-mode", "--foo"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: option '--client-ssl-mode' expects a value, got nothing"s)},
     {"client_ssl_mode_as_last_arg",  // BS_ARGS_BAD_05
      {"--client-ssl-mode"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: option '--client-ssl-mode' expects a value, got nothing"s)},
 
     // server-ssl-mode
     {"server_ssl_mode_invalid",  // BS_ARGS_BAD_06
      {"--server-ssl-mode", "foo"},
-     stdx::make_unexpected(
-         "value 'foo' provided to --server-ssl-mode is not one "
-         "of DISABLED,PREFERRED,REQUIRED,AS_CLIENT"s)},
+     stdx::unexpected("value 'foo' provided to --server-ssl-mode is not one "
+                      "of DISABLED,PREFERRED,REQUIRED,AS_CLIENT"s)},
     {"server_ssl_mode_empty",  // BS_ARGS_BAD_07
      {"--server-ssl-mode", ""},
-     stdx::make_unexpected("value '' provided to --server-ssl-mode is not one "
-                           "of DISABLED,PREFERRED,REQUIRED,AS_CLIENT"s)},
+     stdx::unexpected("value '' provided to --server-ssl-mode is not one "
+                      "of DISABLED,PREFERRED,REQUIRED,AS_CLIENT"s)},
     {"server_ssl_mode_space",  // BS_ARGS_BAD_08
      {"--server-ssl-mode", " "},
-     stdx::make_unexpected("value ' ' provided to --server-ssl-mode is not one "
-                           "of DISABLED,PREFERRED,REQUIRED,AS_CLIENT"s)},
+     stdx::unexpected("value ' ' provided to --server-ssl-mode is not one "
+                      "of DISABLED,PREFERRED,REQUIRED,AS_CLIENT"s)},
     {"server_ssl_mode_no_value",  // BS_ARGS_BAD_09
      {"--server-ssl-mode", "--foo"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: option '--server-ssl-mode' expects a value, got nothing"s)},
     {"server_ssl_mode_as_last_arg",  // BS_ARGS_BAD_10
      {"--server-ssl-mode"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: option '--server-ssl-mode' expects a value, got nothing"s)},
 
     // server-ssl-verify
     {"server_ssl_verify_invalid",  // BS_ARGS_BAD_11
      {"--server-ssl-verify", "foo"},
-     stdx::make_unexpected(
-         "value 'foo' provided to --server-ssl-verify is not one "
-         "of DISABLED,VERIFY_CA,VERIFY_IDENTITY"s)},
+     stdx::unexpected("value 'foo' provided to --server-ssl-verify is not one "
+                      "of DISABLED,VERIFY_CA,VERIFY_IDENTITY"s)},
     {"server_ssl_verify_empty",  // BS_ARGS_BAD_12
      {"--server-ssl-verify", ""},
-     stdx::make_unexpected(
-         "value '' provided to --server-ssl-verify is not one "
-         "of DISABLED,VERIFY_CA,VERIFY_IDENTITY"s)},
+     stdx::unexpected("value '' provided to --server-ssl-verify is not one "
+                      "of DISABLED,VERIFY_CA,VERIFY_IDENTITY"s)},
     {"server_ssl_verify_no_value",  // BS_ARGS_BAD_13
      {"--server-ssl-verify", "--foo"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: option '--server-ssl-verify' expects a value, got nothing"s)},
     {"server_ssl_verify_as_last_arg",  // BS_ARGS_BAD_14
      {"--server-ssl-verify"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: option '--server-ssl-verify' expects a value, got nothing"s)},
 
     // client-ssl-cipher
     {"client_ssl_cipher_empty",  // BS_ARGS_EMPTY_Q_01
      {"--client-ssl-cipher", ""},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Value for option '--client-ssl-cipher' can't be empty"s)},
 
     // client-ssl-curves
     {"client_ssl_curves_empty",  // BS_ARGS_EMPTY_Q_02
      {"--client-ssl-curves", ""},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Value for option '--client-ssl-curves' can't be empty"s)},
 
     // client-ssl-cert
     {"client_ssl_cert_empty",  // BS_ARGS_EMPTY_Q_03
      {"--client-ssl-cert", ""},
-     stdx::make_unexpected(
-         "Value for option '--client-ssl-cert' can't be empty"s)},
+     stdx::unexpected("Value for option '--client-ssl-cert' can't be empty"s)},
     {"client_ssl_cert_without_key",
      {"--client-ssl-cert", "foo"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "If --client-ssl-cert is set, --client-ssl-key can't be empty"s)},
 
     // client-ssl-key
     {"client_ssl_key_empty",  // BS_ARGS_EMPTY_Q_04
      {"--client-ssl-key", ""},
-     stdx::make_unexpected(
-         "Value for option '--client-ssl-key' can't be empty"s)},
+     stdx::unexpected("Value for option '--client-ssl-key' can't be empty"s)},
     {"client_ssl_key_without_cert",
      {"--client-ssl-key", "foo"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "If --client-ssl-key is set, --client-ssl-cert can't be empty"s)},
 
     // client-ssl-dh-params
     {"client_ssl_dh_params_empty",  // BS_ARGS_EMPTY_Q_05
      {"--client-ssl-dh-params", ""},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Value for option '--client-ssl-dh-params' can't be empty"s)},
 
     // server-ssl-cipher
     {"server_ssl_cipher_empty",  // BS_ARGS_EMPTY_Q_06
      {"--server-ssl-cipher", ""},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Value for option '--server-ssl-cipher' can't be empty"s)},
 
     // server-ssl-curves
     {"server_ssl_curves_empty",  // BS_ARGS_EMPTY_Q_07
      {"--server-ssl-curves", ""},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Value for option '--server-ssl-curves' can't be empty"s)},
 
     // server-ssl-ca
     {"server_ssl_ca_empty",  // BS_ARGS_EMPTY_Q_08
      {"--server-ssl-ca", ""},
-     stdx::make_unexpected(
-         "Value for option '--server-ssl-ca' can't be empty"s)},
+     stdx::unexpected("Value for option '--server-ssl-ca' can't be empty"s)},
 
     // server-ssl-capath
     {"server_ssl_capath_empty",  // BS_ARGS_EMPTY_Q_09
      {"--server-ssl-capath", ""},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Value for option '--server-ssl-capath' can't be empty"s)},
 
     // server-ssl-crl
     {"server_ssl_crl_empty",  // BS_ARGS_EMPTY_Q_10
      {"--server-ssl-crl", ""},
-     stdx::make_unexpected(
-         "Value for option '--server-ssl-crl' can't be empty"s)},
+     stdx::unexpected("Value for option '--server-ssl-crl' can't be empty"s)},
 
     // server-ssl-crlpath
     {"server_ssl_crlpath_empty",  // BS_ARGS_EMPTY_Q_11
      {"--server-ssl-crlpath", ""},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Value for option '--server-ssl-crlpath' can't be empty"s)},
 
 };
@@ -226,7 +224,11 @@ INSTANTIATE_TEST_SUITE_P(
 
 class BootstrapTlsEndpointWithoutBootstrapFail
     : public RouterComponentBootstrapTest,
-      public ::testing::WithParamInterface<BootstrapTlsEndpointFailParams> {};
+      public ::testing::WithParamInterface<BootstrapTlsEndpointFailParams> {
+ public:
+  BootstrapTlsEndpointWithoutBootstrapFail()
+      : RouterComponentBootstrapTest(false) {}
+};
 
 TEST_P(BootstrapTlsEndpointWithoutBootstrapFail, check) {
   // don't set the --bootstrap option.
@@ -244,7 +246,7 @@ TEST_P(BootstrapTlsEndpointWithoutBootstrapFail, check) {
   EXPECT_NO_THROW(router.wait_for_exit());
 
   if (!GetParam().expected_result) {
-    // check if the bootstraping was successful
+    // check if the bootstrapping was successful
     EXPECT_THAT(router.get_full_output(),
                 ::testing::HasSubstr(GetParam().expected_result.error()));
   }
@@ -253,59 +255,59 @@ TEST_P(BootstrapTlsEndpointWithoutBootstrapFail, check) {
 const BootstrapTlsEndpointFailParams bootstrap_tls_endpoint_without_bootstrap_fail_params[] = {
     {"client_ssl_mode",  // BS_NOBS_C_01
      {"--client-ssl-mode", "disabled"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --client-ssl-mode can only be used together with -B/--bootstrap"s)},
     {"client_ssl_cipher",  // BS_NOBS_C_02
      {"--client-ssl-cipher", "some-valid-cipher"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --client-ssl-cipher can only be used together with -B/--bootstrap"s)},
     {"client_ssl_curves",  // BS_NOBS_C_03
      {"--client-ssl-curves", "some-valid-curves"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --client-ssl-curves can only be used together with -B/--bootstrap"s)},
     {"client_ssl_cert_and_key",  // BS_NOBS_C_04
      {"--client-ssl-cert", "some-cert", "--client-ssl-key", "some-key"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --client-ssl-cert can only be used together with -B/--bootstrap"s)},
     {"client_ssl_key_and_cert",  // BS_NOBS_C_05
      {"--client-ssl-key", "some-key", "--client-ssl-cert", "some-cert"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --client-ssl-key can only be used together with -B/--bootstrap"s)},
     {"client_ssl_dh_params",  // BS_NOBS_C_06
      {"--client-ssl-dh-params", "some-valid-dh-params"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --client-ssl-dh-params can only be used together with -B/--bootstrap"s)},
     {"server_ssl_mode",  // BS_NOBS_S_01
      {"--server-ssl-mode", "disabled"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-mode can only be used together with -B/--bootstrap"s)},
     {"server_ssl_verify",  // BS_NOBS_S_02
      {"--server-ssl-verify", "disabled"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-verify can only be used together with -B/--bootstrap"s)},
     {"server_ssl_cipher",  // BS_NOBS_S_03
      {"--server-ssl-cipher", "some-valid-ciphers"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-cipher can only be used together with -B/--bootstrap"s)},
     {"server_ssl_curves",  // BS_NOBS_S_04
      {"--server-ssl-curves", "some-valid-curves"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-curves can only be used together with -B/--bootstrap"s)},
     {"server_ssl_ca",  // BS_NOBS_S_05
      {"--server-ssl-ca", "some-valid-ca-file.pem"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-ca can only be used together with -B/--bootstrap"s)},
     {"server_ssl_capath",  // BS_NOBS_S_05
      {"--server-ssl-capath", "some-valid-capath"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-capath can only be used together with -B/--bootstrap"s)},
     {"server_ssl_crl",  // BS_NOBS_S_05
      {"--server-ssl-crl", "some-valid-crl-file.pem"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-crl can only be used together with -B/--bootstrap"s)},
     {"server_ssl_crlpath",  // BS_NOBS_S_05
      {"--server-ssl-crlpath", "some-valid-crlpath"},
-     stdx::make_unexpected(
+     stdx::unexpected(
          "Error: Option --server-ssl-crlpath can only be used together with -B/--bootstrap"s)},
 };
 
@@ -326,7 +328,11 @@ struct BootstrapTlsEndpointParams {
 
 class BootstrapTlsEndpointWithoutCertGeneration
     : public RouterComponentBootstrapTest,
-      public ::testing::WithParamInterface<BootstrapTlsEndpointParams> {};
+      public ::testing::WithParamInterface<BootstrapTlsEndpointParams> {
+ public:
+  BootstrapTlsEndpointWithoutCertGeneration()
+      : RouterComponentBootstrapTest(false) {}
+};
 
 TEST_P(BootstrapTlsEndpointWithoutCertGeneration, succeeds) {
   auto cmdline_args = GetParam().cmdline_args;
@@ -345,13 +351,14 @@ TEST_P(BootstrapTlsEndpointWithoutCertGeneration, succeeds) {
   cmdline_args.emplace_back(server_key_pem);
 
   ASSERT_NO_FATAL_FAILURE(bootstrap_failover(
-      {
-          Config{"127.0.0.1", port_pool_.get_next_available(),
-                 port_pool_.get_next_available(),
-                 get_data_dir().join("bootstrap_gr.js").str()},
-      },
+      {Config{
+          "127.0.0.1",
+          port_pool_.get_next_available(),
+          port_pool_.get_next_available(),
+          "bootstrap_gr.js",
+      }},
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_SUCCESS,
-      {"# MySQL Router configured"}, 1000ms, {2, 0, 3}, cmdline_args));
+      {"# MySQL Router configured"}, 5s, {2, 0, 3}, cmdline_args));
 
   ASSERT_NE(config_file.size(), 0);
 
@@ -376,7 +383,7 @@ const BootstrapTlsEndpointParams
                config_file_lines,
                ::testing::IsSupersetOf({
                    "client_ssl_mode=PREFERRED",
-                   "server_ssl_mode=AS_CLIENT",
+                   "server_ssl_mode=PREFERRED",
                    "client_ssl_cert=" SSL_TEST_DATA_DIR "server-cert.pem",
                    "client_ssl_key=" SSL_TEST_DATA_DIR "server-key.pem",
                    "server_ssl_verify=DISABLED",
@@ -401,7 +408,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({"client_ssl_mode=DISABLED",
-                                                "server_ssl_mode=AS_CLIENT"}));
+                                                "server_ssl_mode=PREFERRED"}));
          }},
 
         {"client_ssl_mode_disabled_disabled",  // BS_MODES_07
@@ -438,7 +445,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({"client_ssl_mode=PREFERRED",
-                                                "server_ssl_mode=AS_CLIENT"}));
+                                                "server_ssl_mode=PREFERRED"}));
          }},
         {"client_ssl_mode_preferred_disabled",  // BS_MODES_12
          {"--client-ssl-mode", "Preferred", "--server-ssl-mode", "Disabled"},
@@ -473,7 +480,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({"client_ssl_mode=REQUIRED",
-                                                "server_ssl_mode=AS_CLIENT"}));
+                                                "server_ssl_mode=PREFERRED"}));
          }},
         {"client_ssl_mode_required_disabled",  // BS_MODES_17
          {"--client-ssl-mode", "ReQuired", "--server-ssl-mode", "Disabled"},
@@ -552,26 +559,28 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=AS_CLIENT",
+                                              "server_ssl_mode=PREFERRED",
                                               "server_ssl_verify=DISABLED",
                                           }));
          }},
         {"server_ssl_verify_verify_identity",  // BS_ARGCASE_SV_02
-         {"--server-ssl-verify", "verify_identITY"},
+         {"--server-ssl-verify", "verify_identITY", "--server-ssl-ca",
+          "some-valid-ca-file.pem"},
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "server_ssl_verify=VERIFY_IDENTITY",
                        }));
          }},
         {"server_ssl_verify_verify_ca",  // BS_ARGCASE_SV_03
-         {"--server-ssl-verify", "verify_CA"},
+         {"--server-ssl-verify", "verify_CA", "--server-ssl-ca",
+          "some-valid-ca-file.pem"},
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=AS_CLIENT",
+                                              "server_ssl_mode=PREFERRED",
                                               "server_ssl_verify=VERIFY_CA",
                                           }));
          }},
@@ -581,7 +590,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "client_ssl_cipher=some-cipher-string",
                        }));
          }},
@@ -591,7 +600,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "client_ssl_curves=some-curves-string",
                        }));
          }},
@@ -601,7 +610,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "client_ssl_dh_params=some-dh-param-string",
                        }));
          }},
@@ -611,7 +620,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "server_ssl_cipher=some-cipher-string",
                        }));
          }},
@@ -621,7 +630,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "server_ssl_curves=some-curves-string",
                        }));
          }},
@@ -630,7 +639,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=AS_CLIENT",
+                                              "server_ssl_mode=PREFERRED",
                                               "server_ssl_ca=some-ca-string",
                                           }));
          }},
@@ -640,7 +649,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "server_ssl_capath=some-capath-string",
                        }));
          }},
@@ -649,7 +658,7 @@ const BootstrapTlsEndpointParams
          [](const std::vector<std::string> &config_file_lines) {
            ASSERT_THAT(config_file_lines, ::testing::IsSupersetOf({
                                               "client_ssl_mode=PREFERRED",
-                                              "server_ssl_mode=AS_CLIENT",
+                                              "server_ssl_mode=PREFERRED",
                                               "server_ssl_crl=some-crl-string",
                                           }));
          }},
@@ -659,7 +668,7 @@ const BootstrapTlsEndpointParams
            ASSERT_THAT(config_file_lines,
                        ::testing::IsSupersetOf({
                            "client_ssl_mode=PREFERRED",
-                           "server_ssl_mode=AS_CLIENT",
+                           "server_ssl_mode=PREFERRED",
                            "server_ssl_crlpath=some-crlpath-string",
                        }));
          }},
@@ -676,20 +685,26 @@ INSTANTIATE_TEST_SUITE_P(
 
 class BootstrapTlsEndpoint
     : public RouterComponentBootstrapTest,
-      public ::testing::WithParamInterface<BootstrapTlsEndpointParams> {};
+      public ::testing::WithParamInterface<BootstrapTlsEndpointParams> {
+ public:
+  BootstrapTlsEndpoint() : RouterComponentBootstrapTest(false) {}
+};
 
 TEST_P(BootstrapTlsEndpoint, succeeds) {
   auto cmdline_args = GetParam().cmdline_args;
 
   ASSERT_NO_FATAL_FAILURE(bootstrap_failover(
       {
-          Config{"127.0.0.1", port_pool_.get_next_available(),
-                 port_pool_.get_next_available(),
-                 get_data_dir().join("bootstrap_gr.js").str()},
+          Config{
+              "127.0.0.1",
+              port_pool_.get_next_available(),
+              port_pool_.get_next_available(),
+              "bootstrap_gr.js",
+          },
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_SUCCESS,
       {"# MySQL Router configured"},
-      10s,  // 10 seconds as cert-generation may take a while on slow machines
+      20s,  // 20 seconds as cert-generation may take a while on slow machines
       {2, 0, 3}, cmdline_args));
 
   ASSERT_NE(config_file.size(), 0);
@@ -720,19 +735,22 @@ TEST_P(BootstrapTlsEndpoint, existing_config) {
               {"destinations", "127.0.0.1:" + std::to_string(server_port)},
               {"routing_strategy", "round-robin"},
           })},
-      "\n");
+      "");
   SCOPED_TRACE("starting router with config:\n" + config);
   auto conf_file = create_config_file(bootstrap_dir.name(), config);
 
   ASSERT_NO_FATAL_FAILURE(bootstrap_failover(
       {
-          Config{"127.0.0.1", port_pool_.get_next_available(),
-                 port_pool_.get_next_available(),
-                 get_data_dir().join("bootstrap_gr.js").str()},
+          Config{
+              "127.0.0.1",
+              port_pool_.get_next_available(),
+              port_pool_.get_next_available(),
+              "bootstrap_gr.js",
+          },
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_SUCCESS,
       {"# MySQL Router configured"},
-      10s,  // 10 seconds as cert-generation may take a while on slow machines
+      20s,  // 20 seconds as cert-generation may take a while on slow machines
       {2, 0, 3}, cmdline_args));
 
   ASSERT_NE(config_file.size(), 0);
@@ -763,7 +781,7 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_cert) {
               {"destinations", "127.0.0.1:" + std::to_string(server_port)},
               {"routing_strategy", "round-robin"},
           })},
-      "\n");
+      "");
   SCOPED_TRACE("starting router with config:\n" + config);
   auto conf_file = create_config_file(
       bootstrap_dir.name(), config, nullptr, "mysqlrouter.conf",
@@ -772,17 +790,20 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_cert) {
               mysql_harness::ConfigBuilder::build_pair(
                   std::make_pair("client_ssl_cert", "foo")),
           },
-          "\n"));
+          ""));
 
   ASSERT_NO_FATAL_FAILURE(bootstrap_failover(
       {
-          Config{"127.0.0.1", port_pool_.get_next_available(),
-                 port_pool_.get_next_available(),
-                 get_data_dir().join("bootstrap_gr.js").str()},
+          Config{
+              "127.0.0.1",
+              port_pool_.get_next_available(),
+              port_pool_.get_next_available(),
+              "bootstrap_gr.js",
+          },
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_SUCCESS,
       {"# MySQL Router configured"},
-      10s,  // 10 seconds as cert-generation may take a while on slow machines
+      20s,  // 20 seconds as cert-generation may take a while on slow machines
       {2, 0, 3}, cmdline_args));
 
   ASSERT_NE(config_file.size(), 0);
@@ -826,13 +847,16 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_key) {
 
   ASSERT_NO_FATAL_FAILURE(bootstrap_failover(
       {
-          Config{"127.0.0.1", port_pool_.get_next_available(),
-                 port_pool_.get_next_available(),
-                 get_data_dir().join("bootstrap_gr.js").str()},
+          Config{
+              "127.0.0.1",
+              port_pool_.get_next_available(),
+              port_pool_.get_next_available(),
+              "bootstrap_gr.js",
+          },
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_SUCCESS,
       {"# MySQL Router configured"},
-      10s,  // 10 seconds as cert-generation may take a while on slow machines
+      20s,  // 20 seconds as cert-generation may take a while on slow machines
       {2, 0, 3}, cmdline_args));
 
   ASSERT_NE(config_file.size(), 0);
@@ -863,7 +887,7 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_cert_and_key) {
               {"destinations", "127.0.0.1:" + std::to_string(server_port)},
               {"routing_strategy", "round-robin"},
           })},
-      "\n");
+      "");
   SCOPED_TRACE("starting router with config:\n" + config);
   auto conf_file = create_config_file(
       bootstrap_dir.name(), config, nullptr, "mysqlrouter.conf",
@@ -878,13 +902,16 @@ TEST_P(BootstrapTlsEndpoint, existing_config_with_client_ssl_cert_and_key) {
 
   ASSERT_NO_FATAL_FAILURE(bootstrap_failover(
       {
-          Config{"127.0.0.1", port_pool_.get_next_available(),
-                 port_pool_.get_next_available(),
-                 get_data_dir().join("bootstrap_gr.js").str()},
+          Config{
+              "127.0.0.1",
+              port_pool_.get_next_available(),
+              port_pool_.get_next_available(),
+              "bootstrap_gr.js",
+          },
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_SUCCESS,
       {"# MySQL Router configured"},
-      10s,  // 10 seconds as cert-generation may take a while on slow machines
+      20s,  // 20 seconds as cert-generation may take a while on slow machines
       {2, 0, 3}, cmdline_args));
 
   ASSERT_NE(config_file.size(), 0);
@@ -1055,12 +1082,12 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                       // BS_CERT_KEY_CONFIG_PERSISTS_10
      {},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_generated(config_file_lines, "PREFERRED", "AS_CLIENT");
+       check_cert_generated(config_file_lines, "PREFERRED", "PREFERRED");
      }},
     {"client_ssl_mode_preferred_cert_gen",  // BS_MODES_02, BS_CERT_KEY_MODE_02
      {"--client-ssl-mode", "PREFERRED"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_generated(config_file_lines, "PREFERRED", "AS_CLIENT");
+       check_cert_generated(config_file_lines, "PREFERRED", "PREFERRED");
      }},
     {"client_ssl_mode_required_cert_gen",  // BS_MODES_03, BS_CERT_KEY_MODE_03,
                                            // BS_CERT_KEY_CONFIG_PERSISTS_02
@@ -1069,7 +1096,7 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                                            // BS_CERT_KEY_CONFIG_PERSISTS_11
      {"--client-ssl-mode", "REQUIRED"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_generated(config_file_lines, "REQUIRED", "AS_CLIENT");
+       check_cert_generated(config_file_lines, "REQUIRED", "PREFERRED");
      }},
     {"client_ssl_mode_passthrough_no_cert_gen",  // BS_MODES_04,
                                                  // BS_CERT_KEY_MODE_04
@@ -1077,7 +1104,7 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                                                  // BS_CERT_KEY_CONFIG_PERSISTS_06
                                                  // BS_CERT_KEY_CONFIG_PERSISTS_09
                                                  // BS_CERT_KEY_CONFIG_PERSISTS_12
-     {"--client-ssl-mode", "PASSTHROUGH"},
+     {"--client-ssl-mode", "PASSTHROUGH", "--disable-rw-split"},
      [](const std::vector<std::string> &config_file_lines) {
        check_no_cert_generated(config_file_lines, "PASSTHROUGH", "AS_CLIENT");
 
@@ -1089,9 +1116,9 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
      }},
     {"client_ssl_mode_disabled_no_cert_gen",  // BS_MODES_05,
                                               // BS_CERT_KEY_MODE_05
-     {"--client-ssl-mode", "DISABLED"},
+     {"--client-ssl-mode", "DISABLED", "--disable-rw-split"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_no_cert_generated(config_file_lines, "DISABLED", "AS_CLIENT");
+       check_no_cert_generated(config_file_lines, "DISABLED", "PREFERRED");
 
        // not specified at command-line, no set in config
        ASSERT_THAT(config_file_lines,
@@ -1111,28 +1138,28 @@ const BootstrapTlsEndpointParams bootstrap_tls_endpoint_params[] = {
                                  // BS_CERT_KEY_ARGS_01
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "PREFERRED", "AS_CLIENT",
+       check_cert_specified(config_file_lines, "PREFERRED", "PREFERRED",
                             "some-ssl-cert", "some-ssl-key");
      }},
     {"client_ssl_cert_key_and_mode_disabled",  // BS_CERT_KEY_ARGS_??
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key",
       "--client-ssl-mode", "disabled"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "DISABLED", "AS_CLIENT",
+       check_cert_specified(config_file_lines, "DISABLED", "PREFERRED",
                             "some-ssl-cert", "some-ssl-key");
      }},
     {"client_ssl_cert_key_and_mode_preferred",  // BS_CERT_KEY_ARGS_??
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key",
       "--client-ssl-mode", "preferred"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "PREFERRED", "AS_CLIENT",
+       check_cert_specified(config_file_lines, "PREFERRED", "PREFERRED",
                             "some-ssl-cert", "some-ssl-key");
      }},
     {"client_ssl_cert_key_and_mode_required",  // BS_CERT_KEY_ARGS_??
      {"--client-ssl-cert", "some-ssl-cert", "--client-ssl-key", "some-ssl-key",
       "--client-ssl-mode", "REQUIRED"},
      [](const std::vector<std::string> &config_file_lines) {
-       check_cert_specified(config_file_lines, "REQUIRED", "AS_CLIENT",
+       check_cert_specified(config_file_lines, "REQUIRED", "PREFERRED",
                             "some-ssl-cert", "some-ssl-key");
      }},
 
@@ -1155,18 +1182,22 @@ struct BootstrapTlsEndpointFailMockParams {
 class BootstrapTlsEndpointFailMock
     : public RouterComponentBootstrapTest,
       public ::testing::WithParamInterface<BootstrapTlsEndpointFailMockParams> {
+ public:
+  BootstrapTlsEndpointFailMock() : RouterComponentBootstrapTest(false) {}
 };
 
 TEST_P(BootstrapTlsEndpointFailMock, fails) {
   bootstrap_failover(
       {
-          Config{"127.0.0.1", port_pool_.get_next_available(),
-                 port_pool_.get_next_available(),
-                 get_data_dir().join("bootstrap_gr.js").str()},
+          Config{
+              "127.0.0.1",
+              port_pool_.get_next_available(),
+              port_pool_.get_next_available(),
+              "bootstrap_gr.js",
+          },
       },
       mysqlrouter::ClusterType::GR_V2, {}, EXIT_FAILURE,
-      GetParam().expected_output_lines, 1000ms, {2, 0, 3},
-      GetParam().cmdline_args);
+      GetParam().expected_output_lines, 1s, {2, 0, 3}, GetParam().cmdline_args);
 }
 
 const BootstrapTlsEndpointFailMockParams
