@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -103,23 +103,19 @@ class MetadataHttpAuthTest : public RouterComponentTest {
   }
 
   std::string get_rest_section() const {
-    const std::string result =
-        "[http_server]\n"
-        "port=" +
-        std::to_string(http_server_port) +
-        "\n"
-        "[rest_router]\n"
-        "require_realm = somerealm\n"
-        "[rest_api]\n"
-        "[http_auth_realm:somerealm]\n"
-        "backend = somebackend\n"
-        "method = basic\n"
-        "name = test\n" +
-        auth_backend_settings() +
-        "[rest_routing]\n"
-        "require_realm = somerealm\n";
-
-    return result;
+    return mysql_harness::ConfigBuilder::build_section("rest_api", {}) +
+           mysql_harness::ConfigBuilder::build_section(
+               "rest_router", {{"require_realm", "somerealm"}}) +
+           mysql_harness::ConfigBuilder::build_section(
+               "rest_routing", {{"require_realm", "somerealm"}}) +
+           mysql_harness::ConfigBuilder::build_section(
+               "http_auth_realm:somerealm", {{"backend", "somebackend"},
+                                             {"method", "basic"},
+                                             {"name", "test"}}) +
+           mysql_harness::ConfigBuilder::build_section(
+               "http_server", {{"port", std::to_string(http_server_port)},
+                               {"bind_address", "127.0.0.1"}}) +
+           auth_backend_settings();
   }
 
   std::string create_state_file_content(const std::string &cluster_id,
@@ -183,58 +179,57 @@ class MetadataHttpAuthTest : public RouterComponentTest {
         /*with_sudo=*/false, wait_for_notify_ready);
   }
 
-  void set_mock_metadata(
-      const std::vector<Auth_data> &auth_data_collection,
-      const uint16_t http_port, const std::string &gr_id,
-      const uint16_t cluster_node_port, const bool error_on_md_query = false,
-      const unsigned primary_id = 0, const uint64_t view_id = 0,
-      const mysqlrouter::MetadataSchemaVersion md_version = {2, 0, 3}) const {
-    auto json_doc = mock_GR_metadata_as_json(gr_id, {cluster_node_port}, 0,
-                                             {cluster_node_port}, primary_id,
-                                             view_id, error_on_md_query);
+  void set_mock_metadata(const std::vector<Auth_data> &auth_data_collection,
+                         const uint16_t http_port, const std::string &gr_id,
+                         const uint16_t cluster_node_port,
+                         const bool error_on_md_query = false,
+                         const uint64_t view_id = 0,
+                         const mysqlrouter::MetadataSchemaVersion md_version = {
+                             2, 0, 3}) const {
+    auto json_doc = mock_GR_metadata_as_json(
+        gr_id, classic_ports_to_gr_nodes({cluster_node_port}), 0,
+        classic_ports_to_cluster_nodes({cluster_node_port}), view_id,
+        error_on_md_query);
 
     JsonAllocator allocator;
     JsonValue nodes(rapidjson::kArrayType);
     for (const auto &auth_data : auth_data_collection) {
       JsonValue node(rapidjson::kArrayType);
-      node.PushBack(JsonValue(auth_data.credentials.username.c_str(),
-                              auth_data.credentials.username.size(), allocator),
-                    allocator);
-
-      node.PushBack(
-          JsonValue(auth_data.credentials.password_hash.c_str(),
-                    auth_data.credentials.password_hash.size(), allocator),
-          allocator);
-
-      node.PushBack(JsonValue(auth_data.privileges.c_str(),
-                              auth_data.privileges.size(), allocator),
-                    allocator);
-
-      node.PushBack(JsonValue(auth_data.auth_method.c_str(),
-                              auth_data.auth_method.size(), allocator),
-                    allocator);
 
+      node.PushBack(JsonValue(auth_data.credentials.username, allocator),
+                    allocator)
+          .PushBack(JsonValue(auth_data.credentials.password_hash, allocator),
+                    allocator)
+          .PushBack(JsonValue(auth_data.privileges, allocator), allocator)
+          .PushBack(JsonValue(auth_data.auth_method, allocator), allocator);
       nodes.PushBack(node, allocator);
     }
 
     json_doc.AddMember("rest_user_credentials", nodes, allocator);
 
     JsonValue metadata_version_node(rapidjson::kArrayType);
-    metadata_version_node.PushBack(md_version.major, allocator);
-    metadata_version_node.PushBack(md_version.minor, allocator);
-    metadata_version_node.PushBack(md_version.patch, allocator);
-    json_doc.AddMember("metadata_version", metadata_version_node, allocator);
+    metadata_version_node.PushBack(md_version.major, allocator)
+        .PushBack(md_version.minor, allocator)
+        .PushBack(md_version.patch, allocator);
+    json_doc.AddMember("metadata_schema_version", metadata_version_node,
+                       allocator);
 
     const auto json_str = json_to_string(json_doc);
 
     EXPECT_NO_THROW(MockServerRestClient(http_port).set_globals(json_str));
   }
 
-  int get_rest_auth_queries_count(const std::string &json_string) const {
+  [[nodiscard]] int get_rest_auth_queries_count(
+      const std::string &json_string) const {
     rapidjson::Document json_doc;
-    json_doc.Parse(json_string.c_str());
+    json_doc.Parse(json_string);
     if (json_doc.HasMember("rest_auth_query_count")) {
-      EXPECT_TRUE(json_doc["rest_auth_query_count"].IsInt());
+      if (json_doc["rest_auth_query_count"].IsNull()) {
+        // no rest-auth-query yet.
+        return 0;
+      }
+
+      EXPECT_TRUE(json_doc["rest_auth_query_count"].IsInt()) << json_string;
       return json_doc["rest_auth_query_count"].GetInt();
     }
     return 0;
@@ -263,11 +258,12 @@ class MetadataHttpAuthTest : public RouterComponentTest {
     http_server_port = port_pool_.get_next_available();
 
     SCOPED_TRACE("// Launch a server mock that will act as our cluster member");
-    const auto trace_file =
-        get_data_dir().join("metadata_http_auth_backend.js").str();
 
-    cluster_node = &ProcessManager::launch_mysql_server_mock(
-        trace_file, cluster_node_port, EXIT_SUCCESS, false, cluster_http_port);
+    cluster_node = &mock_server_spawner().spawn(
+        mock_server_cmdline("metadata_http_auth_backend.js")
+            .port(cluster_node_port)
+            .http_port(cluster_http_port)
+            .args());
 
     router_port = port_pool_.get_next_available();
 
@@ -344,13 +340,13 @@ class BasicMetadataHttpAuthTest
 
 TEST_F(BasicMetadataHttpAuthTest, MetadataHttpAuthDefaultConfig) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, 0, view_id);
+                    cluster_node_port, false, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   launch_router(kMetadataCacheSectionBase);
 
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
-  EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
+  ASSERT_GE(wait_for_rest_auth_query(2, cluster_http_port), 2);
 
   IOContext io_ctx;
   RestClient rest_client(io_ctx, "127.0.0.1", http_server_port, "foobar",
@@ -362,9 +358,9 @@ TEST_F(BasicMetadataHttpAuthTest, MetadataHttpAuthDefaultConfig) {
                                        kContentTypeJson));
 }
 
-TEST_F(BasicMetadataHttpAuthTest, UnsupportedMetadataSchemaVersion) {
+TEST_F(BasicMetadataHttpAuthTest, DISABLED_UnsupportedMetadataSchemaVersion) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, 0, view_id, {1, 0, 0});
+                    cluster_node_port, false, view_id, {2, 0, 0});
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   launch_router(kMetadataCacheSectionBase);
@@ -385,7 +381,7 @@ TEST_P(BasicMetadataHttpAuthTest, BasicMetadataHttpAuth) {
   set_mock_metadata(
       {{GetParam().cached_info.credentials, GetParam().cached_info.privileges,
         GetParam().cached_info.auth_method}},
-      cluster_http_port, cluster_id, cluster_node_port, false, 0, view_id);
+      cluster_http_port, cluster_id, cluster_node_port, false, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   const std::string metadata_cache_section =
@@ -393,7 +389,7 @@ TEST_P(BasicMetadataHttpAuthTest, BasicMetadataHttpAuth) {
   launch_router(metadata_cache_section);
 
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
-  EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
+  ASSERT_GE(wait_for_rest_auth_query(2, cluster_http_port), 2);
 
   IOContext io_ctx;
   RestClient rest_client(io_ctx, "127.0.0.1", http_server_port,
@@ -497,7 +493,7 @@ TEST_F(FileAuthBackendWithMetadataAuthSettings, MixedBackendSettings) {
   EXPECT_EQ(cmd.wait_for_exit(), 0) << cmd.get_full_output();
 
   set_mock_metadata({}, cluster_http_port, cluster_id, cluster_node_port, false,
-                    0, view_id);
+                    view_id);
 
   const std::string metadata_cache_section =
       get_metadata_cache_section(kTTL, kAuthCacheTTL, kAuthCacheRefreshRate);
@@ -513,7 +509,7 @@ class InvalidMetadataHttpAuthTimersTest
 
 TEST_P(InvalidMetadataHttpAuthTimersTest, InvalidMetadataHttpAuthTimers) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, 0, view_id);
+                    cluster_node_port, false, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   auto &router =
@@ -542,7 +538,7 @@ class ValidMetadataHttpAuthTimersTest
 
 TEST_P(ValidMetadataHttpAuthTimersTest, ValidMetadataHttpAuthTimers) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, 0, view_id);
+                    cluster_node_port, false, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   launch_router(kMetadataCacheSectionBase + "ttl=0.001\n" + GetParam());
@@ -566,14 +562,14 @@ class MetadataHttpAuthTestCustomTimers
 
 TEST_P(MetadataHttpAuthTestCustomTimers, MetadataHttpAuthCustomTimers) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, 0, view_id);
+                    cluster_node_port, false, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   launch_router(kMetadataCacheSectionBase + GetParam());
 
   ASSERT_TRUE(wait_for_port_ready(router_port));
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
-  EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
+  ASSERT_GE(wait_for_rest_auth_query(2, cluster_http_port), 2);
 
   IOContext io_ctx;
   RestClient rest_client(io_ctx, "127.0.0.1", http_server_port, "foobar",
@@ -596,7 +592,7 @@ INSTANTIATE_TEST_SUITE_P(
 
 TEST_F(MetadataHttpAuthTest, ExpiredAuthCacheTTL) {
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, false, 0, view_id);
+                    cluster_node_port, false, view_id);
 
   std::chrono::milliseconds cache_ttl = kAuthCacheRefreshRate * 4;
   SCOPED_TRACE("// Launch the router with the initial state file");
@@ -605,7 +601,7 @@ TEST_F(MetadataHttpAuthTest, ExpiredAuthCacheTTL) {
   launch_router(metadata_cache_section);
 
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
-  EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
+  ASSERT_GE(wait_for_rest_auth_query(2, cluster_http_port), 2);
 
   IOContext io_ctx;
   RestClient rest_client(io_ctx, "127.0.0.1", http_server_port, "foobar",
@@ -619,7 +615,7 @@ TEST_F(MetadataHttpAuthTest, ExpiredAuthCacheTTL) {
   const bool fail_on_md_query = true;
   // Start to fail metadata cache updates
   set_mock_metadata({{kTestUser1, ""}}, cluster_http_port, cluster_id,
-                    cluster_node_port, fail_on_md_query, 0, view_id);
+                    cluster_node_port, fail_on_md_query, view_id);
 
   // wait long enough for the auth cache to expire
   std::this_thread::sleep_for(cache_ttl);
@@ -630,6 +626,8 @@ TEST_F(MetadataHttpAuthTest, ExpiredAuthCacheTTL) {
 }
 
 struct MetadataAuthCacheUpdateParams {
+  std::string test_name;
+
   std::vector<Auth_data> first_auth_cache_data_set;
   Http_response_details first_http_response;
   std::vector<Auth_data> second_auth_cache_data_set;
@@ -642,7 +640,7 @@ class MetadataAuthCacheUpdate
 
 TEST_P(MetadataAuthCacheUpdate, AuthCacheUpdate) {
   set_mock_metadata(GetParam().first_auth_cache_data_set, cluster_http_port,
-                    cluster_id, cluster_node_port, false, 0, view_id);
+                    cluster_id, cluster_node_port, false, view_id);
 
   SCOPED_TRACE("// Launch the router with the initial state file");
   const std::string metadata_cache_section =
@@ -650,24 +648,29 @@ TEST_P(MetadataAuthCacheUpdate, AuthCacheUpdate) {
   launch_router(metadata_cache_section);
 
   ASSERT_TRUE(wait_for_rest_endpoint_ready(uri, http_server_port));
-  EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
+  ASSERT_GE(wait_for_rest_auth_query(1, cluster_http_port), 1);
 
   IOContext io_ctx;
+
   RestClient rest_client(io_ctx, "127.0.0.1", http_server_port, "foobar",
                          "password");
 
   JsonDocument json_doc;
+
+  SCOPED_TRACE("// authenticate via HTTP with the right password");
   ASSERT_NO_FATAL_FAILURE(request_json(
       rest_client, uri, HttpMethod::Get, GetParam().first_http_response.code,
       json_doc, GetParam().first_http_response.type));
 
-  // Update authentication metadata
+  SCOPED_TRACE("// Update authentication metadata");
   set_mock_metadata(GetParam().second_auth_cache_data_set, cluster_http_port,
-                    cluster_id, cluster_node_port, false, 0, view_id);
+                    cluster_id, cluster_node_port, false, view_id);
 
-  // auth_cache is updated
-  EXPECT_GT(wait_for_rest_auth_query(2, cluster_http_port), 0);
+  SCOPED_TRACE("// wait for the auth_cache to update");
+  // wait for 3 as the 2nd refresh may happen while we set the mock-metadata.
+  ASSERT_GE(wait_for_rest_auth_query(3, cluster_http_port), 3);
 
+  SCOPED_TRACE("// authenticate via HTTP a 2nd time with the same credentials");
   ASSERT_NO_FATAL_FAILURE(request_json(
       rest_client, uri, HttpMethod::Get, GetParam().second_http_response.code,
       json_doc, GetParam().second_http_response.type));
@@ -676,27 +679,46 @@ TEST_P(MetadataAuthCacheUpdate, AuthCacheUpdate) {
 INSTANTIATE_TEST_SUITE_P(
     AuthCacheUpdate, MetadataAuthCacheUpdate,
     ::testing::Values(
-        // add user
-        MetadataAuthCacheUpdateParams{{{kTestUser2, ""}},
+        MetadataAuthCacheUpdateParams{"add_user",
+                                      {
+                                          {kTestUser2, ""},
+                                      },
                                       ResponseUnauthorized,
-                                      {{kTestUser1, ""}, {kTestUser2, ""}},
+                                      {
+                                          {kTestUser1, ""},
+                                          {kTestUser2, ""},
+                                      },
                                       ResponseOk},
-        // add user privileges
-        MetadataAuthCacheUpdateParams{{{kTestUser1, ""}},
+        MetadataAuthCacheUpdateParams{"add_user_privileges",
+                                      {
+                                          {kTestUser1, ""},
+                                      },
                                       ResponseOk,
-                                      {{kTestUser1, "{\"foo\": \"bar\"}"}},
+                                      {
+                                          {kTestUser1, "{\"foo\": \"bar\"}"},
+                                      },
                                       ResponseForbidden},
-        // change password
         MetadataAuthCacheUpdateParams{
-            {{kTestUser1, ""}},
+            "change_password",
+            {
+                {kTestUser1, ""},
+            },
             ResponseOk,
-            {{{kTestUser1.username, kTestUser2.password_hash}, ""}},
+            {
+                {{kTestUser1.username, kTestUser2.password_hash}, ""},
+            },
             ResponseUnauthorized},
-        // rm user
-        MetadataAuthCacheUpdateParams{{{kTestUser1, ""}, {kTestUser2, ""}},
+        MetadataAuthCacheUpdateParams{"remove_user",
+                                      {
+                                          {kTestUser1, ""},
+                                          {kTestUser2, ""},
+                                      },
                                       ResponseOk,
-                                      {{kTestUser2, ""}},
-                                      ResponseUnauthorized}));
+                                      {
+                                          {kTestUser2, ""},
+                                      },
+                                      ResponseUnauthorized}),
+    [](const auto &info) { return info.param.test_name; });
 
 int main(int argc, char *argv[]) {
   init_windows_sockets();
