@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -253,16 +253,14 @@ class ConfigGenerator {
 class RouterRoutingConnectionCommonTest : public RouterComponentTest {
  public:
   void SetUp() override {
+    static mysql_harness::RandomGenerator static_rg;
+
     RouterComponentTest::SetUp();
 
     mysql_harness::DIM &dim = mysql_harness::DIM::instance();
     // RandomGenerator
-    dim.set_RandomGenerator(
-        []() {
-          static mysql_harness::RandomGenerator rg;
-          return &rg;
-        },
-        [](mysql_harness::RandomGeneratorInterface *) {});
+    dim.set_static_RandomGenerator(&static_rg);
+
 #if 1
     {
       ProcessWrapper::OutputResponder responder{
@@ -316,9 +314,11 @@ class RouterRoutingConnectionCommonTest : public RouterComponentTest {
 
   auto &launch_server(uint16_t cluster_port, const std::string &json_file,
                       uint16_t http_port, size_t number_of_servers = 5) {
-    auto &cluster_node = ProcessManager::launch_mysql_server_mock(
-        get_data_dir().join(json_file).str(), cluster_port, EXIT_SUCCESS, false,
-        http_port);
+    auto &cluster_node =
+        mock_server_spawner().spawn(mock_server_cmdline(json_file)
+                                        .port(cluster_port)
+                                        .http_port(http_port)
+                                        .args());
 
     std::vector<uint16_t> nodes_ports;
     nodes_ports.resize(number_of_servers);
