@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -77,6 +77,9 @@ constexpr auto kTTL = 100ms;
 }  // namespace
 
 class StateFileTest : public RouterComponentBootstrapTest {
+ public:
+  StateFileTest() : RouterComponentBootstrapTest(false) {}
+
  protected:
   void SetUp() override {
     RouterComponentTest::SetUp();
@@ -91,7 +94,7 @@ class StateFileTest : public RouterComponentBootstrapTest {
         {"cluster_type", (cluster_type == ClusterType::RS_V2) ? "rs" : "gr"},
         {"router_id", "1"},
         {"user", "mysql_router1_user"},
-        {"metadata_cluster", "mycluster"},
+        {"metadata_cluster", "my-cluster"},
         {"connect_timeout", "1"},
         {"ttl", std::to_string(std::chrono::duration<double>(ttl).count())},
     };
@@ -249,16 +252,19 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
     cluster_http_ports.push_back(port_pool_.get_next_available());
   }
 
-  const std::string node_host = param.ipv6 ? "[::1]" : "127.0.0.1";
+  const std::string node_host = param.ipv6 ? "::1" : "127.0.0.1";
+  const std::string node_host_pretty = param.ipv6 ? "[::1]" : "127.0.0.1";
   const std::string bind_address = param.ipv6 ? "::" : "127.0.0.1";
 
   SCOPED_TRACE(
       "// Launch 3 server mocks that will act as our metadata servers");
-  const auto trace_file = get_data_dir().join(param.trace_file).str();
   for (unsigned i = 0; i < CLUSTER_NODES; ++i) {
-    cluster_nodes.push_back(&ProcessManager::launch_mysql_server_mock(
-        trace_file, cluster_nodes_ports[i], EXIT_SUCCESS, false,
-        cluster_http_ports[i], 0, "", bind_address));
+    cluster_nodes.push_back(
+        &mock_server_spawner().spawn(mock_server_cmdline(param.trace_file)
+                                         .port(cluster_nodes_ports[i])
+                                         .http_port(cluster_http_ports[i])
+                                         .bind_address(bind_address)
+                                         .args()));
     try {
       ASSERT_NO_FATAL_FAILURE(check_port_ready(
           *cluster_nodes[i], cluster_nodes_ports[i], kDefaultPortReadyTimeout,
@@ -276,21 +282,26 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
       std::this_thread::sleep_for(100ms);
       return;
     }
-    ASSERT_TRUE(MockServerRestClient(cluster_http_ports[i])
+    ASSERT_TRUE(MockServerRestClient(cluster_http_ports[i], node_host)
                     .wait_for_rest_endpoint_ready());
 
     SCOPED_TRACE(
         "// Make our metadata server to return single node as a cluster "
         "member (meaning single metadata server)");
-    set_mock_metadata(cluster_http_ports[i], kGroupId,
-                      classic_ports_to_gr_nodes({cluster_nodes_ports[i]}), i,
-                      {cluster_nodes_ports[i]}, 0, false, node_host);
+    MockGrMetadata()
+        .gr_id(kGroupId)
+        .gr_nodes(classic_ports_to_gr_nodes({cluster_nodes_ports[i]}))
+        .gr_node_host(node_host_pretty)
+        .cluster_nodes({cluster_nodes_ports[i]})
+        .gr_pos(i)
+        .send(cluster_http_ports[i], node_host);
   }
 
   SCOPED_TRACE("// Create a router state file with a single metadata server");
   const std::string state_file = create_state_file(
       temp_test_dir.name(),
-      create_state_file_content(kGroupId, {cluster_nodes_ports[0]}, node_host));
+      create_state_file_content(kGroupId, {cluster_nodes_ports[0]},
+                                node_host_pretty));
 
   SCOPED_TRACE(
       "// Create a configuration file sections with low ttl so that any "
@@ -303,24 +314,27 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
       router_port, "PRIMARY", "first-available");
 
   SCOPED_TRACE("// Launch ther router with the initial state file");
-  launch_router(temp_test_dir.name(), metadata_cache_section, routing_section,
-                state_file);
+  auto &router = launch_router(temp_test_dir.name(), metadata_cache_section,
+                               routing_section, state_file);
 
   SCOPED_TRACE(
       "// Check our state file content, it should not change yet, there is "
       "single metadata server reported as initially");
 
   check_state_file(state_file, param.cluster_type, kGroupId,
-                   {cluster_nodes_ports[0]}, 0, node_host);
+                   {cluster_nodes_ports[0]}, 0, node_host_pretty);
 
   SCOPED_TRACE(
       "// Now change the response from the metadata server to return 3 gr "
       "nodes (metadata servers)");
   for (unsigned i = 0; i < CLUSTER_NODES; ++i) {
-    set_mock_metadata(cluster_http_ports[i], kGroupId,
-                      classic_ports_to_gr_nodes(cluster_nodes_ports), i,
-                      classic_ports_to_cluster_nodes(cluster_nodes_ports), 0,
-                      false, node_host);
+    MockGrMetadata()
+        .gr_id(kGroupId)
+        .gr_nodes(classic_ports_to_gr_nodes({cluster_nodes_ports}))
+        .gr_node_host(node_host_pretty)
+        .cluster_nodes(classic_ports_to_cluster_nodes(cluster_nodes_ports))
+        .gr_pos(i)
+        .send(cluster_http_ports[i], node_host);
   }
 
   SCOPED_TRACE(
@@ -330,7 +344,7 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
   check_state_file(
       state_file, param.cluster_type, kGroupId,
       {cluster_nodes_ports[0], cluster_nodes_ports[1], cluster_nodes_ports[2]},
-      0, node_host);
+      0, node_host_pretty);
 
   ///////////////////////////////////////////////////
 
@@ -349,16 +363,16 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
   SCOPED_TRACE(
       "// Instrument the second and third metadata servers to return 2 "
       "servers: second and third");
-  set_mock_metadata(cluster_http_ports[1], kGroupId,
-                    classic_ports_to_gr_nodes(
-                        {cluster_nodes_ports[1], cluster_nodes_ports[2]}),
-                    1, {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0,
-                    false, node_host);
-  set_mock_metadata(cluster_http_ports[2], kGroupId,
-                    classic_ports_to_gr_nodes(
-                        {cluster_nodes_ports[1], cluster_nodes_ports[2]}),
-                    2, {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0,
-                    false, node_host);
+  for (unsigned i = 1; i <= 2; i++) {
+    MockGrMetadata()
+        .gr_id(kGroupId)
+        .gr_nodes(classic_ports_to_gr_nodes(
+            {cluster_nodes_ports[1], cluster_nodes_ports[2]}))
+        .gr_node_host(node_host_pretty)
+        .cluster_nodes({cluster_nodes_ports[1], cluster_nodes_ports[2]})
+        .gr_pos(i)
+        .send(cluster_http_ports[i], node_host);
+  }
 
   SCOPED_TRACE("// Kill first metada server");
   kill_server(cluster_nodes[0]);
@@ -368,7 +382,17 @@ TEST_P(StateFileMetadataServersChangedInRuntimeTest,
       "servers reported by the second metadata server");
   check_state_file(state_file, param.cluster_type, kGroupId,
                    {cluster_nodes_ports[1], cluster_nodes_ports[2]}, 0,
-                   node_host, 10000ms);
+                   node_host_pretty, 10000ms);
+
+  router.kill();
+  check_exit_code(router, EXIT_SUCCESS, 5s);
+
+  // check that the state.json.tmp was not left behind
+  Path tmp_file(temp_test_dir.file("state.json.tmp"));
+  EXPECT_FALSE(tmp_file.exists());
+
+  Path dynamic_state_file(temp_test_dir.file("state.json"));
+  EXPECT_TRUE(dynamic_state_file.exists());
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -404,9 +428,12 @@ TEST_P(StateFileMetadataServersInaccessibleTest, MetadataServersInaccessible) {
 
   SCOPED_TRACE(
       "// Launch single server mock that will act as our metadata server");
-  const auto trace_file = get_data_dir().join(param.trace_file).str();
-  auto &cluster_node(ProcessManager::launch_mysql_server_mock(
-      trace_file, cluster_node_port, EXIT_SUCCESS, false, cluster_http_port));
+  auto &cluster_node =
+      mock_server_spawner().spawn(mock_server_cmdline(param.trace_file)
+                                      .port(cluster_node_port)
+                                      .http_port(cluster_http_port)
+                                      .args());
+
   ASSERT_NO_FATAL_FAILURE(check_port_ready(cluster_node, cluster_node_port));
   ASSERT_TRUE(
       MockServerRestClient(cluster_http_port).wait_for_rest_endpoint_ready());
@@ -485,9 +512,12 @@ TEST_P(StateFileGroupReplicationIdDiffersTest, GroupReplicationIdDiffers) {
   auto cluster_http_port = port_pool_.get_next_available();
 
   SCOPED_TRACE("// Launch server mock that will act as our metadata server");
-  const auto trace_file = get_data_dir().join(param.trace_file).str();
-  /*auto &cluster_node =*/ProcessManager::launch_mysql_server_mock(
-      trace_file, cluster_node_port, EXIT_SUCCESS, false, cluster_http_port);
+
+  /*auto &cluster_node =*/mock_server_spawner().spawn(
+      mock_server_cmdline(param.trace_file)
+          .port(cluster_node_port)
+          .http_port(cluster_http_port)
+          .args());
 
   SCOPED_TRACE(
       "// Make our metadata server to return single node as a cluster "
@@ -515,7 +545,7 @@ TEST_P(StateFileGroupReplicationIdDiffersTest, GroupReplicationIdDiffers) {
   const std::string routing_section = get_metadata_cache_routing_section(
       router_port, "PRIMARY", "first-available");
 
-  SCOPED_TRACE("// Launch ther router with the initial state file");
+  SCOPED_TRACE("// Launch the router with the initial state file");
   auto &router = launch_router(temp_test_dir.name(), metadata_cache_section,
                                routing_section, state_file, EXIT_SUCCESS, -1s);
 
@@ -576,12 +606,15 @@ TEST_P(StateFileSplitBrainScenarioTest, SplitBrainScenario) {
   }
 
   SCOPED_TRACE("// Launch  server mocks that play as our split brain cluster");
-  const auto trace_file = get_data_dir().join(param.trace_file).str();
   for (unsigned i = 0; i < kNodesNum; i++) {
-    const auto port_connect = cluster_node_ports[i].first;
-    const auto port_http = cluster_node_ports[i].second;
-    cluster_nodes.push_back(&ProcessManager::launch_mysql_server_mock(
-        trace_file, port_connect, EXIT_SUCCESS, false, port_http));
+    const auto [port_connect, port_http] = cluster_node_ports[i];
+
+    cluster_nodes.push_back(
+        &mock_server_spawner().spawn(mock_server_cmdline(param.trace_file)
+                                         .port(port_connect)
+                                         .http_port(port_http)
+                                         .args()));
+
     ASSERT_NO_FATAL_FAILURE(check_port_ready(*cluster_nodes[i], port_connect));
     ASSERT_TRUE(MockServerRestClient(port_http).wait_for_rest_endpoint_ready());
   }
@@ -947,14 +980,18 @@ INSTANTIATE_TEST_SUITE_P(
 #ifndef _WIN32
 
 struct StateFileAccessRightsTestParams {
-  bool read_access;
-  bool write_access;
+  bool read_access;             // read access to state file
+  bool write_access_tmp_file;   // write access to tmp state file
+  bool write_access_directory;  // write access to the state file dir
   std::string expected_error;
 
-  StateFileAccessRightsTestParams(bool read_access_, bool write_access_,
+  StateFileAccessRightsTestParams(bool read_access_,
+                                  bool write_access_tmp_file_,
+                                  bool write_access_directory_,
                                   const std::string &expected_error_)
       : read_access(read_access_),
-        write_access(write_access_),
+        write_access_tmp_file(write_access_tmp_file_),
+        write_access_directory(write_access_directory_),
         expected_error(expected_error_) {}
 };
 
@@ -974,6 +1011,7 @@ TEST_P(StateFileAccessRightsTest, ParametrizedStateFileSchemaTest) {
   auto test_params = GetParam();
 
   TempDirectory temp_test_dir;
+  TempDirectory temp_test_dir_data;
 
   const uint16_t router_port = port_pool_.get_next_available();
 
@@ -981,12 +1019,26 @@ TEST_P(StateFileAccessRightsTest, ParametrizedStateFileSchemaTest) {
   // dynamic state file configured via test parameter
 
   const std::string state_file = create_state_file(
-      temp_test_dir.name(), create_state_file_content("000-000", {10000}));
+      temp_test_dir_data.name(), create_state_file_content("000-000", {10000}));
   mode_t file_mode = 0;
   if (test_params.read_access) file_mode |= S_IRUSR;
-  if (test_params.write_access) file_mode |= S_IWUSR;
   chmod(state_file.c_str(), file_mode);
 
+  // create empty temp state file with no write access
+  if (!test_params.write_access_tmp_file) {
+    const std::string tmp_state_file = [&]() {
+      Path file_path = Path(temp_test_dir_data.name()).join("state.json.tmp");
+      std::ofstream ofs_config(file_path.str());
+
+      return file_path.str();
+    }();
+    chmod(tmp_state_file.c_str(), S_IRUSR);
+  }
+
+  if (!test_params.write_access_directory) {
+    chmod(temp_test_dir_data.name().c_str(), S_IRUSR | S_IXUSR);
+  }
+
   auto writer = config_writer(temp_test_dir.name())
                     .section(metadata_cache_section())
                     .section(metadata_cache_routing_section(
@@ -1014,12 +1066,17 @@ TEST_P(StateFileAccessRightsTest, ParametrizedStateFileSchemaTest) {
 INSTANTIATE_TEST_SUITE_P(
     StateFileTests, StateFileAccessRightsTest,
     ::testing::Values(
-        // no read, nor write access
-        StateFileAccessRightsTestParams(false, false,
+        // no read for state file
+        StateFileAccessRightsTestParams(false, true, true,
                                         "Could not open dynamic state file"),
-        // read access, no write access
-        StateFileAccessRightsTestParams(true, false,
-                                        "Could not open dynamic state file")));
+
+        // read access, no write access to temporary file
+        StateFileAccessRightsTestParams(
+            true, false, true, "Could not open temporary dynamic state file"),
+
+        // read access, no write access to directory
+        StateFileAccessRightsTestParams(
+            true, true, false, "Could not open temporary dynamic state file")));
 
 #endif
 
@@ -1039,13 +1096,17 @@ class StateFileDirectoryBootstrapTest : public StateFileTest {
 TEST_F(StateFileDirectoryBootstrapTest, DirectoryBootstrapTest) {
   TempDirectory temp_test_dir;
 
+  prepare_config_dir_with_default_certs(temp_test_dir.name());
+
   SCOPED_TRACE("// Launch our metadata server we bootstrap against");
 
-  const auto trace_file = get_data_dir().join("bootstrap_gr.js").str();
   const auto metadata_server_port = port_pool_.get_next_available();
   const auto http_port = port_pool_.get_next_available();
-  ProcessManager::launch_mysql_server_mock(trace_file, metadata_server_port,
-                                           EXIT_SUCCESS, false, http_port);
+
+  mock_server_spawner().spawn(mock_server_cmdline("bootstrap_gr.js")
+                                  .port(metadata_server_port)
+                                  .http_port(http_port)
+                                  .args());
   set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
                     classic_ports_to_gr_nodes({metadata_server_port}), 0,
                     {metadata_server_port});
@@ -1053,7 +1114,7 @@ TEST_F(StateFileDirectoryBootstrapTest, DirectoryBootstrapTest) {
   SCOPED_TRACE("// Bootstrap against our metadata server");
   std::vector<std::string> router_cmdline{
       "--bootstrap=localhost:" + std::to_string(metadata_server_port), "-d",
-      temp_test_dir.name()};
+      temp_test_dir.name(), "--logger.level=debug"};
   auto &router = launch_router_for_bootstrap(router_cmdline, EXIT_SUCCESS);
 
   ASSERT_NO_FATAL_FAILURE(check_exit_code(router, EXIT_SUCCESS));
@@ -1103,16 +1164,20 @@ class StateFileSystemBootstrapTest : public StateFileTest,
 TEST_F(StateFileSystemBootstrapTest, SystemBootstrapTest) {
   SCOPED_TRACE("// Launch our metadata server we bootstrap against");
 
-  const auto trace_file = get_data_dir().join("bootstrap_gr.js").str();
   const auto metadata_server_port = port_pool_.get_next_available();
   const auto http_port = port_pool_.get_next_available();
-  ProcessManager::launch_mysql_server_mock(trace_file, metadata_server_port,
-                                           EXIT_SUCCESS, false, http_port);
+
+  mock_server_spawner().spawn(mock_server_cmdline("bootstrap_gr.js")
+                                  .port(metadata_server_port)
+                                  .http_port(http_port)
+                                  .args());
+
   set_mock_metadata(http_port, "00000000-0000-0000-0000-0000000000g1",
                     classic_ports_to_gr_nodes({metadata_server_port}), 0,
                     {metadata_server_port});
 
   SCOPED_TRACE("// Bootstrap against our metadata server");
+
   std::vector<std::string> router_cmdline{"--bootstrap=localhost:" +
                                           std::to_string(metadata_server_port)};
   auto &router = launch_router_for_bootstrap(router_cmdline, EXIT_SUCCESS);
