@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2019, 2024, Oracle and/or its affiliates.
+  Copyright (c) 2019, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -65,7 +65,9 @@
 #include "mysqlrouter/rest_client.h"
 #include "mysqlrouter/utils.h"
 #include "process_launcher.h"
+#include "process_wrapper.h"
 #include "random_generator.h"
+#include "stdx_expected_no_error.h"
 
 #ifdef USE_STD_REGEX
 #include <regex>
@@ -75,18 +77,13 @@
 
 #include <gtest/gtest.h>  // FAIL
 
-#define EXPECT_NO_ERROR(x) \
-  EXPECT_THAT((x), ::testing::Truly([](auto const &v) { return bool(v); }))
-
-#define ASSERT_NO_ERROR(x) \
-  ASSERT_THAT((x), ::testing::Truly([](auto const &v) { return bool(v); }))
-
 using mysql_harness::Path;
 using mysql_harness::ProcessLauncher;
 Path ProcessManager::origin_dir_;
 Path ProcessManager::data_dir_;
 Path ProcessManager::plugin_dir_;
 Path ProcessManager::mysqlrouter_exec_;
+Path ProcessManager::mysqlrouter_bootstrap_exec_;
 Path ProcessManager::mysqlserver_mock_exec_;
 
 using namespace std::chrono_literals;
@@ -106,7 +103,7 @@ stdx::expected<ProcessManager::notify_socket_t, std::error_code> accept_until(
                                               std::system_category()};
 
       if (ec != ec_pipe_listening) {
-        return accept_res.get_unexpected();
+        return stdx::unexpected(accept_res.error());
       }
 
       // nothing is connected yet, sleep a bit an retry.
@@ -117,7 +114,7 @@ stdx::expected<ProcessManager::notify_socket_t, std::error_code> accept_until(
     }
   } while (clock_type::now() < end_time);
 
-  return stdx::make_unexpected(make_error_code(std::errc::timed_out));
+  return stdx::unexpected(make_error_code(std::errc::timed_out));
 }
 
 stdx::expected<void, std::error_code>
@@ -132,7 +129,7 @@ ProcessManager::Spawner::wait_for_notified(
   do {
     auto accept_res = accept_until<clock_type>(sock, end_time);
     if (!accept_res) {
-      return accept_res.get_unexpected();
+      return stdx::unexpected(accept_res.error());
     }
 
     auto accepted = std::move(accept_res.value());
@@ -140,7 +137,7 @@ ProcessManager::Spawner::wait_for_notified(
     // make the read non-blocking.
     const auto non_block_res = accepted.native_non_blocking(true);
     if (!non_block_res) {
-      return non_block_res.get_unexpected();
+      return stdx::unexpected(non_block_res.error());
     }
 
     const size_t BUFF_SIZE = 512;
@@ -151,7 +148,7 @@ ProcessManager::Spawner::wait_for_notified(
     if (!read_res) {
       if (read_res.error() !=
           std::error_code{ERROR_NO_DATA, std::system_category()}) {
-        return read_res.get_unexpected();
+        return stdx::unexpected(read_res.error());
       }
 
       // there was no data. Wait a bit and try again.
@@ -173,7 +170,7 @@ ProcessManager::Spawner::wait_for_notified(
     // either not matched, or no data yet.
   } while (clock_type::now() < end_time);
 
-  return stdx::make_unexpected(make_error_code(std::errc::timed_out));
+  return stdx::unexpected(make_error_code(std::errc::timed_out));
 }
 
 #else
@@ -193,13 +190,13 @@ ProcessManager::Spawner::wait_for_notified(
     const auto poll_res =
         net::impl::poll::poll(fds.data(), fds.size(), timeout);
     if (!poll_res) {
-      return poll_res.get_unexpected();
+      return stdx::unexpected(poll_res.error());
     }
 
     const auto read_res =
         net::read(sock, net::buffer(buff), net::transfer_at_least(1));
     if (!read_res) {
-      return read_res.get_unexpected();
+      return stdx::unexpected(read_res.error());
     } else {
       const auto bytes_read = read_res.value();
 
@@ -255,6 +252,35 @@ ProcessWrapper &ProcessManager::Spawner::launch_command(
 
   processes_.emplace_back(std::move(pw), expected_exit_status_);
 
+#ifdef _WIN32
+  // add the process to the job-object.
+
+  if (!job_object_.is_open()) {
+    auto create_res = mysql_harness::win32::JobObject::create();
+    if (!create_res) {
+      throw std::system_error(create_res.error());
+    }
+    job_object_ = std::move(*create_res);
+
+    JOBOBJECT_EXTENDED_LIMIT_INFORMATION job_limit{};
+    job_limit.BasicLimitInformation.LimitFlags =
+        JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE;
+
+    auto set_info_res = job_object_.set_information(
+        JobObjectExtendedLimitInformation, &job_limit, sizeof(job_limit));
+    if (!set_info_res) {
+      throw std::system_error(set_info_res.error());
+    }
+  }
+
+  auto process_handle = std::get<0>(processes_.back())->process_handle();
+
+  auto assign_res = job_object_.assign_process(process_handle);
+  if (!assign_res) {
+    throw std::system_error(assign_res.error());
+  }
+#endif
+
   return *std::get<0>(processes_.back()).get();
 }
 
@@ -361,8 +387,16 @@ ProcessWrapper &ProcessManager::Spawner::spawn(
 
 ProcessManager::Spawner ProcessManager::spawner(std::string executable,
                                                 std::string logging_file) {
-  return {executable, logging_dir_.name(), logging_file,
-          generate_notify_socket_path(get_test_temp_dir_name()), processes_};
+  return {
+      std::move(executable),
+      logging_dir_.name(),
+      std::move(logging_file),
+      generate_notify_socket_path(get_test_temp_dir_name()),
+      processes_,
+#ifdef _WIN32
+      job_object_,
+#endif
+  };
 }
 
 stdx::expected<void, std::error_code> ProcessManager::wait_for_notified(
@@ -435,84 +469,35 @@ ProcessWrapper &ProcessManager::launch_router(
       .spawn(params);
 }
 
+ProcessWrapper &ProcessManager::launch_router_bootstrap(
+    const std::vector<std::string> &params, int expected_exit_code /*= 0*/,
+    bool catch_stderr /*= true*/, bool with_sudo /*= false*/,
+    std::chrono::milliseconds wait_for_notify_ready /*= 30s*/,
+    OutputResponder output_resp) {
+  return router_bootstrap_spawner()
+      .with_sudo(with_sudo)
+      .catch_stderr(catch_stderr)
+      .expected_exit_code(expected_exit_code)
+      .wait_for_notify_ready(wait_for_notify_ready)
+      .output_responder(std::move(output_resp))
+      .spawn(params);
+}
+
 std::vector<std::string> ProcessManager::mysql_server_mock_cmdline_args(
     const std::string &json_file, uint16_t port, uint16_t http_port,
     uint16_t x_port, const std::string &module_prefix /* = "" */,
-    const std::string &bind_address /*= "0.0.0.0"*/,
-    bool enable_ssl /* = false */) {
-  std::vector<std::string> server_params{
-      "--filename",       json_file,             //
-      "--port",           std::to_string(port),  //
-      "--bind-address",   bind_address,
-      "--logging-folder", get_test_temp_dir_name(),
-  };
-
-  server_params.emplace_back("--module-prefix");
-  if (module_prefix.empty()) {
-    server_params.emplace_back(get_data_dir().str());
-  } else {
-    server_params.emplace_back(module_prefix);
-  }
-
-  if (http_port > 0) {
-    server_params.emplace_back("--http-port");
-    server_params.emplace_back(std::to_string(http_port));
-  }
-
-  if (x_port > 0) {
-    server_params.emplace_back("--xport");
-    server_params.emplace_back(std::to_string(x_port));
-  }
-
-  if (enable_ssl) {
-    server_params.emplace_back("--ssl-mode");
-    server_params.emplace_back("PREFERRED");
-    server_params.emplace_back("--ssl-key");
-    server_params.emplace_back(SSL_TEST_DATA_DIR "server-key.pem");
-    server_params.emplace_back("--ssl-cert");
-    server_params.emplace_back(SSL_TEST_DATA_DIR "server-cert.pem");
-  }
-
-  return server_params;
-}
-
-ProcessWrapper &ProcessManager::launch_mysql_server_mock(
-    const std::vector<std::string> &server_params, unsigned port,
-    int expected_exit_code,
-    std::chrono::milliseconds wait_for_notify_ready /*= 30s*/) {
-  auto &result = spawner(mysqlserver_mock_exec_.str())
-                     .expected_exit_code(expected_exit_code)
-                     .wait_for_notify_ready(wait_for_notify_ready)
-                     .catch_stderr(true)
-                     .with_core_dump(true)
-                     .spawn(server_params);
-
-  result.set_logging_path(get_test_temp_dir_name(),
-                          "mock_server_" + std::to_string(port) + ".log");
-
-  return result;
-}
-
-ProcessWrapper &ProcessManager::launch_mysql_server_mock(
-    const std::string &json_file, unsigned port, int expected_exit_code,
-    bool debug_mode, uint16_t http_port, uint16_t x_port,
-    const std::string &module_prefix /* = "" */,
     const std::string &bind_address /*= "127.0.0.1"*/,
-    std::chrono::milliseconds wait_for_notify_ready /*= 30s*/,
     bool enable_ssl /* = false */) {
-  if (mysqlserver_mock_exec_.str().empty())
-    throw std::logic_error("path to mysql-server-mock must not be empty");
-
-  auto server_params =
-      mysql_server_mock_cmdline_args(json_file, port, http_port, x_port,
-                                     module_prefix, bind_address, enable_ssl);
-
-  if (debug_mode) {
-    server_params.emplace_back("--verbose");
-  }
-
-  return launch_mysql_server_mock(server_params, port, expected_exit_code,
-                                  wait_for_notify_ready);
+  return MockServerCmdline()
+      .logging_folder(get_test_temp_dir_name())
+      .absolute_filename(json_file)
+      .port(port)
+      .http_port(http_port)
+      .x_port(x_port)
+      .module_prefix(module_prefix)
+      .bind_address(bind_address)
+      .enable_ssl(enable_ssl)
+      .args();
 }
 
 std::map<std::string, std::string> ProcessManager::get_DEFAULT_defaults()
@@ -605,8 +590,8 @@ std::string ProcessManager::create_config_file(
   return file_path.str();
 }
 
-std::string ProcessManager::create_state_file(const std::string &dir_name,
-                                              const std::string &content) {
+/* static */ std::string ProcessManager::create_state_file(
+    const std::string &dir_name, const std::string &content) {
   Path file_path = Path(dir_name).join("state.json");
   std::ofstream ofs_config(file_path.str());
 
@@ -649,22 +634,31 @@ void ProcessManager::terminate_all_still_alive() {
   }
 }
 
+std::string ProcessManager::dump(ProcessWrapper &proc) {
+  std::stringstream ss;
+
+  ss << "# Process: (pid=" << proc.get_pid() << ")\n"
+     << proc.get_command_line() << "\n\n";
+
+  auto output = proc.get_current_output();
+  if (!output.empty()) {
+    ss << "## Console output:\n\n" << output << "\n";
+  }
+
+  auto log_content = proc.get_logfile_content("", "", 500);
+  if (!log_content.empty()) {
+    ss << "## Log content:\n\n" << log_content << "\n";
+  }
+
+  return ss.str();
+}
+
 void ProcessManager::dump_all() {
   std::stringstream ss;
   for (const auto &proc_and_exit_code : processes_) {
     const auto &proc = std::get<0>(proc_and_exit_code);
-    ss << "# Process: (pid=" << proc->get_pid() << ")\n"
-       << proc->get_command_line() << "\n\n";
 
-    auto output = proc->get_current_output();
-    if (!output.empty()) {
-      ss << "## Console output:\n\n" << output << "\n";
-    }
-
-    auto log_content = proc->get_logfile_content("", "", 500);
-    if (!log_content.empty()) {
-      ss << "## Log content:\n\n" << log_content << "\n";
-    }
+    ss << dump(*proc);
   }
 
   FAIL() << ss.str();
@@ -707,7 +701,7 @@ stdx::expected<void, std::error_code> ProcessManager::wait_for_exit(
     try {
       proc->native_wait_for_exit(timeout);
     } catch (const std::system_error &e) {
-      res = stdx::make_unexpected(e.code());
+      res = stdx::unexpected(e.code());
     }
   }
 
@@ -737,7 +731,9 @@ void ProcessManager::check_exit_code(ProcessWrapper &process,
     if (dump_res) std::cerr << *dump_res;
 
     ASSERT_EQ(expected_exit_status, result)
-        << "Process " << process.get_pid() << " terminated with " << result;
+        << "Process " << process.get_pid() << " terminated with " << result
+        << "\n"
+        << dump(process);
   } else if (auto sig = result.stopped()) {
     ASSERT_EQ(expected_exit_status, result)
         << "Process " << process.get_pid() << " stopped with " << result;
@@ -811,6 +807,7 @@ void ProcessManager::set_origin(const Path &dir) {
   };
 
   mysqlrouter_exec_ = get_exe_path("mysqlrouter");
+  mysqlrouter_bootstrap_exec_ = get_exe_path("mysqlrouter_bootstrap");
   mysqlserver_mock_exec_ = get_exe_path("mysql_server_mock");
 
   data_dir_ = COMPONENT_TEST_DATA_DIR;
