@@ -1,4 +1,4 @@
-/* Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -64,12 +64,12 @@ struct Password_lock {
   /**
      read from the user config. The number of days to keep the account locked
   */
-  int password_lock_time_days;
+  int password_lock_time_days{};
   /**
     read from the user config. The number of failed login attempts before the
     account is locked
   */
-  uint failed_login_attempts;
+  uint failed_login_attempts{};
 
   Password_lock();
 
