@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -443,6 +443,50 @@ Query_block *build_show_triggers_query(const POS &pos, THD *thd, String *wild,
 Query_block *build_show_procedures_query(const POS &pos, THD *thd, String *wild,
                                          Item *where_cond);
 
+/**
+  Build a substitute query for SHOW LIBRARY STATUS
+
+  For command like,
+  @code
+    SHOW LIBRARY STATUS
+        [LIKE 'pattern' | WHERE expr]
+  @endcode
+
+  We build following,
+  @code
+    SELECT
+      Db,
+      Name,
+      Language,
+      Modified,
+      Created,
+      Comment
+    FROM
+      (SELECT
+         ROUTINE_SCHEMA AS `Db`,
+         ROUTINE_NAME AS `Name`,
+         EXTERNAL_LANGUAGE AS `Language`,
+         LAST_ALTERED AS `Modified`,
+         CREATED AS `Created`,
+         ROUTINE_COMMENT AS `Comment`,
+        FROM information_schema.libraries) libraries
+    WHERE Db == '<value>'           <-- Default DB or IN clause
+          AND
+          [ Name LIKE "<value>" | @<where_clause@> ]
+    ORDER BY `Db`, `Name`;
+
+  @endcode
+
+  @param pos  - POS position of parsing context.
+  @param thd  - Current thread.
+  @param wild - The value of LIKE clause.
+  @param where_cond - @<where_clause@> clause provided by user.
+
+  @returns pointer to Query_block on success, NULL otherwise.
+*/
+Query_block *build_show_library_query(const POS &pos, THD *thd, String *wild,
+                                      Item *where_cond);
+
 /**
   Build a substitute query for SHOW EVENTS
 
