@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,12 +29,12 @@
 #include <vector>
 
 #include "lex_string.h"
-#include "m_ctype.h"  // my_casedn_str
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
 #include "my_sys.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/strings/m_ctype.h"  // my_casedn_str
 #include "mysqld_error.h"
 #include "sql/auth/auth_common.h"
 #include "sql/dd/cache/dictionary_client.h"  // dd::cache::Dictionary_client
@@ -534,6 +534,10 @@ static bool open_views_and_update_metadata(
       }
       continue;
     }
+    // Content tree will not be needed here, so we can destroy it right away.
+    if (view->is_json_duality_view() && view->jdv_content_tree != nullptr)
+      jdv::destroy_content_tree(view->jdv_content_tree);
+
     thd->stmt_arena->mem_root = saved_arena_mem_root;
     if (view->is_view() == false) {
       // In between listing views and locking(opening), if view is dropped and
@@ -586,7 +590,7 @@ static bool open_views_and_update_metadata(
         view->algorithm != VIEW_ALGORITHM_TEMPTABLE) {
       for (ORDER *order = thd->lex->query_block->order_list.first; order;
            order = order->next)
-        order->used_alias = false;  /// @see Item::print_for_order()
+        order->used_alias = nullptr;  /// @see Item::print_for_order()
     }
     Sql_mode_parse_guard parse_guard(thd);
     thd->lex->unit->print(
@@ -787,7 +791,7 @@ bool update_referencing_views_metadata(
   DBUG_TRACE;
   assert(table != nullptr);
 
-  bool error = update_referencing_views_metadata(
+  const bool error = update_referencing_views_metadata(
       thd, table->get_db_name(), table->get_table_name(), new_db,
       new_table_name, commit_dd_changes, uncommitted_tables);
   return error;
@@ -806,7 +810,7 @@ bool update_referencing_views_metadata(
   DBUG_TRACE;
   assert(db_name && table_name);
 
-  bool error = update_referencing_views_metadata(
+  const bool error = update_referencing_views_metadata(
       thd, db_name, table_name, nullptr, nullptr, commit_dd_changes,
       uncommitted_tables);
   return error;
