@@ -1,17 +1,18 @@
 #ifndef DD_TABLE_SHARE_INCLUDED
 #define DD_TABLE_SHARE_INCLUDED
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,6 +38,7 @@ struct TABLE_SHARE;
 
 namespace dd {
 class Table;
+class Abstract_table;
 
 enum class enum_column_types;
 }  // namespace dd
@@ -103,4 +105,26 @@ static inline CHARSET_INFO *dd_get_mysql_charset(dd::Object_id dd_cs_id) {
 */
 bool is_suitable_for_primary_key(KEY_PART_INFO *key_part, Field *table_field);
 
+/**
+  Read the view definition from the data-dictionary.
+
+  @param thd        Thread handler
+  @param share      Fill this with table definition
+  @param abstract_table  A data-dictionary Table-object describing
+                    the view to be used for opening, instead of reading
+                    information from DD.
+
+  @note
+    This function is called when the view definition is not cached in
+    table_def_cache.
+    The data is returned in 'share', which is allocated by
+    alloc_table_share().. The code assumes that share is initialized.
+
+  @returns
+   false   OK
+   true    Error
+*/
+bool open_view_def(THD *thd, TABLE_SHARE *share,
+                   const dd::Abstract_table *abstract_table);
+
 #endif  // DD_TABLE_SHARE_INCLUDED
