@@ -1,7 +1,7 @@
 #ifndef DEBUG_SYNC_INCLUDED
 #define DEBUG_SYNC_INCLUDED
 
-/* Copyright (c) 2009, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -67,7 +67,8 @@ extern bool debug_sync_set_action(THD *thd, const char *action_str, size_t len);
 extern bool debug_sync_update(THD *thd, char *val_str);
 extern uchar *debug_sync_value_ptr(THD *thd);
 extern void conditional_sync_point_for_timestamp(std::string name);
-extern void conditional_sync_point(std::string name);
+extern void conditional_sync_point(std::string name,
+                                   unsigned short timeout = 0);
 
 /**
   This macro simplifies when a DBUG_EXECUTE_IF will generate a given
@@ -95,6 +96,25 @@ extern void conditional_sync_point(std::string name);
   */
 #define CONDITIONAL_SYNC_POINT(NAME) conditional_sync_point(NAME)
 
+/**
+  Set a sync point that is activated by setting
+  @@debug='d,syncpoint_NAME', and which will emit the signal
+  "reached_NAME" and wait for the signal "continue_NAME"
+
+  @param[in] NAME The name of the debug symbol. This should indicate
+    the logical point in time in the code flow where it
+    appears. Usually it should have the form "before_EVENT" or
+    "after_EVENT", where EVENT identifies something that the code
+    does. EVENT might for instance be the name of a function in the
+    source code.  The sync point might be reused by multiple tests, so
+    the name should relate to what the server does and not the test
+    scenario.
+
+  @param[in] TIMEOUT Timeout to use during conditional wait_for.
+*/
+#define CONDITIONAL_SYNC_POINT_TIMEOUT(NAME, TIMEOUT) \
+  conditional_sync_point(NAME, TIMEOUT)
+
 /**
    Set a sync point that is activated by setting
    @@debug='d,syncpoint_NAME_TIMESTAMP', where NAME is given as an
@@ -127,6 +147,7 @@ extern void conditional_sync_point(std::string name);
   do {                                   \
   } while (0)
 #define CONDITIONAL_SYNC_POINT(NAME)
+#define CONDITIONAL_SYNC_POINT_TIMEOUT(NAME, TIMEOUT)
 #define CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP(NAME)
 
 #endif /* defined(ENABLED_DEBUG_SYNC) */
