@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,10 +36,10 @@
 #include "my_inttypes.h"
 #include "mysql/udf_registration_types.h"
 #include "sql-common/json_dom.h"  // Json_wrapper
-#include "sql/current_thd.h"      // current_thd
-#include "sql/item.h"             // Cached_item, ...
-#include "sql/my_decimal.h"
-#include "sql/sql_class.h"  // THD
+#include "sql-common/my_decimal.h"
+#include "sql/current_thd.h"  // current_thd
+#include "sql/item.h"         // Cached_item, ...
+#include "sql/sql_class.h"    // THD
 #include "sql/system_variables.h"
 #include "sql/thr_malloc.h"
 #include "sql_string.h"
@@ -55,11 +55,18 @@ using std::min;
 Cached_item *new_Cached_item(THD *thd, Item *item) {
   switch (item->result_type()) {
     case STRING_RESULT:
-      if (item->is_temporal())
-        return new (thd->mem_root) Cached_item_temporal(item);
-      if (item->data_type() == MYSQL_TYPE_JSON)
-        return new (thd->mem_root) Cached_item_json(item);
-      return new (thd->mem_root) Cached_item_str(item);
+      switch (item->data_type()) {
+        case MYSQL_TYPE_TIME:
+          return new (thd->mem_root) Cached_item_time(item);
+        case MYSQL_TYPE_DATE:
+        case MYSQL_TYPE_DATETIME:
+        case MYSQL_TYPE_TIMESTAMP:
+          return new (thd->mem_root) Cached_item_temporal(item);
+        case MYSQL_TYPE_JSON:
+          return new (thd->mem_root) Cached_item_json(item);
+        default:
+          return new (thd->mem_root) Cached_item_str(item);
+      }
     case INT_RESULT:
       return new (thd->mem_root) Cached_item_int(item);
     case REAL_RESULT:
@@ -94,9 +101,11 @@ bool Cached_item_str::cmp() {
 }
 
 Cached_item_json::Cached_item_json(Item *item_arg)
-    : Cached_item(item_arg), m_value(new (*THR_MALLOC) Json_wrapper()) {}
+    : Cached_item(item_arg), m_value(new(*THR_MALLOC) Json_wrapper()) {}
 
-Cached_item_json::~Cached_item_json() { destroy(m_value); }
+Cached_item_json::~Cached_item_json() {
+  if (m_value != nullptr) ::destroy_at(m_value);
+}
 
 /**
   Compare the new JSON value in member 'item' with the previous value.
@@ -134,7 +143,7 @@ bool Cached_item_json::cmp() {
 
 bool Cached_item_real::cmp() {
   DBUG_TRACE;
-  double nr = item->val_real();
+  const double nr = item->val_real();
   DBUG_PRINT("info", ("old: %f, new: %f", value, nr));
   if (item->null_value) {
     if (null_value) return false;
@@ -150,7 +159,7 @@ bool Cached_item_real::cmp() {
 
 bool Cached_item_int::cmp() {
   DBUG_TRACE;
-  longlong nr = item->val_int();
+  const longlong nr = item->val_int();
   DBUG_PRINT("info", ("old: 0x%.16llx, new: 0x%.16llx", (ulonglong)value,
                       (ulonglong)nr));
   if (item->null_value) {
@@ -165,9 +174,37 @@ bool Cached_item_int::cmp() {
   return false;
 }
 
+bool Cached_item_time::cmp() {
+  DBUG_TRACE;
+  Time_val time;
+  if (item->val_time(&time)) return true;
+
+  if (item->null_value) {
+    if (null_value) return false;
+    null_value = true;
+    return true;
+  } else if (null_value || m_time.compare(time) != 0) {
+    null_value = false;
+    m_time = time;
+    return true;
+  }
+  return false;
+}
+
 bool Cached_item_temporal::cmp() {
   DBUG_TRACE;
-  longlong nr = item->val_temporal_by_field_type();
+  assert(item->data_type() != MYSQL_TYPE_TIME);
+  longlong nr = 0;
+  switch (item->data_type()) {
+    case MYSQL_TYPE_DATE:
+    case MYSQL_TYPE_DATETIME:
+    case MYSQL_TYPE_TIMESTAMP:
+      nr = item->val_date_temporal();
+      break;
+    default:
+      assert(false);
+      break;
+  }
   DBUG_PRINT("info", ("old: %lld, new: %lld", value, nr));
   if (item->null_value) {
     if (null_value) return false;
