@@ -1,7 +1,7 @@
 #ifndef ITEM_JSON_FUNC_INCLUDED
 #define ITEM_JSON_FUNC_INCLUDED
 
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,12 +44,14 @@
 #include "sql-common/json_error_handler.h"
 #include "sql-common/json_path.h"    // Json_path
 #include "sql-common/json_schema.h"  //Json_schema_validator_holder
+#include "sql/current_thd.h"         // current_thd
 #include "sql/enum_query_type.h"
 #include "sql/field.h"
 #include "sql/item.h"
 #include "sql/item_cmpfunc.h"
 #include "sql/item_func.h"
-#include "sql/item_strfunc.h"    // Item_str_func
+#include "sql/item_strfunc.h"  // Item_str_func
+#include "sql/json_duality_view/content_tree.h"
 #include "sql/mem_root_array.h"  // Mem_root_array
 #include "sql/parse_location.h"  // POS
 #include "sql/psi_memory_key.h"  // key_memory_JSON
@@ -63,6 +65,7 @@ class Json_scalar_holder;
 class Json_schema_validator;
 class Json_wrapper;
 class PT_item_list;
+class PT_jdv_name_value_list;
 class THD;
 class my_decimal;
 enum Cast_target : unsigned char;
@@ -183,7 +186,7 @@ class Item_json_func : public Item_func {
     @param parent_args  arguments to forward to Item_func's constructor
   */
   template <typename... Args>
-  Item_json_func(THD *thd, Args &&... parent_args)
+  Item_json_func(THD *thd, Args &&...parent_args)
       : Item_func(std::forward<Args>(parent_args)...),
         m_path_cache(thd, arg_count) {
     set_data_type_json();
@@ -196,8 +199,9 @@ class Item_json_func : public Item_func {
   }
   enum Item_result result_type() const override { return STRING_RESULT; }
   String *val_str(String *arg) override;
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override;
-  bool get_time(MYSQL_TIME *ltime) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_time(Time_val *time) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   longlong val_int() override;
   double val_real() override;
   my_decimal *val_decimal(my_decimal *decimal_value) override;
@@ -346,6 +350,7 @@ class Item_func_json_schema_valid final : public Item_bool_func {
   ~Item_func_json_schema_valid() override;
 
   const char *func_name() const override { return "json_schema_valid"; }
+  enum Functype functype() const override { return JSON_SCHEMA_VALID_FUNC; }
 
   bool val_bool() override;
 
@@ -373,6 +378,10 @@ class Item_func_json_schema_validation_report final : public Item_json_func {
     return "json_schema_validation_report";
   }
 
+  enum Functype functype() const override {
+    return JSON_SCHEMA_VALIDATION_REPORT_FUNC;
+  }
+
   bool val_json(Json_wrapper *wr) override;
 
   bool resolve_type(THD *thd) override {
@@ -611,7 +620,7 @@ class Item_func_json_extract final : public Item_json_func {
 class Item_func_modify_json_in_path : public Item_json_func {
  protected:
   template <typename... Args>
-  explicit Item_func_modify_json_in_path(Args &&... parent_args)
+  explicit Item_func_modify_json_in_path(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...) {
     // The function does not necessarily return NULL when an argument is NULL.
     // It returns NULL only if the first argument is NULL, or if one of the JSON
@@ -682,7 +691,7 @@ class Item_func_json_set_replace : public Item_func_modify_json_in_path {
 
  protected:
   template <typename... Args>
-  explicit Item_func_json_set_replace(bool json_set, Args &&... parent_args)
+  explicit Item_func_json_set_replace(bool json_set, Args &&...parent_args)
       : Item_func_modify_json_in_path(std::forward<Args>(parent_args)...),
         m_json_set(json_set),
         m_path(key_memory_JSON) {}
@@ -697,7 +706,7 @@ class Item_func_json_set_replace : public Item_func_modify_json_in_path {
 class Item_func_json_set final : public Item_func_json_set_replace {
  public:
   template <typename... Args>
-  explicit Item_func_json_set(Args &&... parent_args)
+  explicit Item_func_json_set(Args &&...parent_args)
       : Item_func_json_set_replace(true, std::forward<Args>(parent_args)...) {}
 
   const char *func_name() const override { return "json_set"; }
@@ -709,7 +718,7 @@ class Item_func_json_set final : public Item_func_json_set_replace {
 class Item_func_json_replace final : public Item_func_json_set_replace {
  public:
   template <typename... Args>
-  explicit Item_func_json_replace(Args &&... parent_args)
+  explicit Item_func_json_replace(Args &&...parent_args)
       : Item_func_json_set_replace(false, std::forward<Args>(parent_args)...) {}
 
   const char *func_name() const override { return "json_replace"; }
@@ -721,7 +730,7 @@ class Item_func_json_replace final : public Item_func_json_set_replace {
 class Item_func_json_array final : public Item_json_func {
  public:
   template <typename... Args>
-  explicit Item_func_json_array(Args &&... parent_args)
+  explicit Item_func_json_array(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...) {
     // Does not return NULL on NULL input. A NULL argument is interpreted as the
     // JSON null literal.
@@ -743,7 +752,7 @@ class Item_func_json_array final : public Item_json_func {
 /**
   Represents the JSON function JSON_OBJECT()
 */
-class Item_func_json_row_object final : public Item_json_func {
+class Item_func_json_row_object : public Item_json_func {
   String tmp_key_value;
 
  public:
@@ -755,6 +764,14 @@ class Item_func_json_row_object final : public Item_json_func {
     null_on_null = false;
   }
 
+  Item_func_json_row_object(THD *thd, mem_root_deque<Item *> *list)
+      : Item_json_func(thd, list) {
+    // Does not return NULL on NULL input. If a key argument is NULL, an error
+    // is raised. If a value argument is NULL, it is interpreted as the JSON
+    // null literal.
+    null_on_null = false;
+  }
+
   const char *func_name() const override { return "json_object"; }
   enum Functype functype() const override { return JSON_OBJECT_FUNC; }
 
@@ -767,6 +784,42 @@ class Item_func_json_row_object final : public Item_json_func {
   bool val_json(Json_wrapper *wr) override;
 };
 
+/**
+ * @brief Represents the JSON function JSON_DUALITY_OBJECT()
+ */
+class Item_func_json_duality_object final : public Item_func_json_row_object {
+  typedef Item_func_json_row_object super;
+
+  jdv::Duality_view_tags m_table_tags{0};
+  PT_jdv_name_value_list *m_jdv_name_value_list{nullptr};
+  bool m_inject_object_hash{false};
+  std::unordered_set<std::string> m_json_arrayagg_keys;
+  bool get_inject_object_hash() const { return m_inject_object_hash; }
+
+ public:
+  Item_func_json_duality_object(THD *thd, const POS &pos, int table_tags,
+                                PT_jdv_name_value_list *jdv_name_value_list);
+
+  const char *func_name() const override { return "json_duality_object"; }
+  enum Functype functype() const override { return JSON_DUALITY_OBJECT_FUNC; }
+
+  bool resolve_type(THD *thd) override;
+  bool val_json(Json_wrapper *wr) override;
+
+  void print(const THD *thd, String *str,
+             enum_query_type query_type) const override;
+
+  bool set_json_arrayagg_keys(THD *thd);
+
+  jdv::Duality_view_tags table_tags() const { return m_table_tags; }
+  Mem_root_array<LEX_STRING> *name_list();
+  Mem_root_array<uint> *col_tags_list();
+
+  std::unordered_set<std::string> get_json_arrayagg_keys() {
+    return m_json_arrayagg_keys;
+  }
+};
+
 /**
   Represents the JSON function JSON_SEARCH()
 */
@@ -785,7 +838,7 @@ class Item_func_json_search : public Item_json_func {
     @param parent_args arguments to pass to Item_json_func's constructor
   */
   template <typename... Args>
-  Item_func_json_search(Args &&... parent_args)
+  Item_func_json_search(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...),
         m_cached_ooa(ooa_uninitialized) {}
 
@@ -819,7 +872,7 @@ class Item_func_json_remove : public Item_json_func {
 
  public:
   template <typename... Args>
-  Item_func_json_remove(Args &&... parent_args)
+  Item_func_json_remove(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...) {}
 
   const char *func_name() const override { return "json_remove"; }
@@ -1065,11 +1118,15 @@ class Item_func_array_cast final : public Item_func {
     assert(false);
     return 0;
   }
-  bool get_date(MYSQL_TIME *, my_time_flags_t) override {
+  bool val_date(Date_val *, my_time_flags_t) override {
     assert(false);
     return true;
   }
-  bool get_time(MYSQL_TIME *) override {
+  bool val_time(Time_val *) override {
+    assert(false);
+    return true;
+  }
+  bool val_datetime(Datetime_val *, my_time_flags_t) override {
     assert(false);
     return true;
   }
@@ -1144,8 +1201,9 @@ class Item_func_json_value final : public Item_func {
   double val_real() override;
   longlong val_int() override;
   my_decimal *val_decimal(my_decimal *value) override;
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t flags) override;
-  bool get_time(MYSQL_TIME *ltime) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_time(Time_val *time) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   Json_on_response_type on_empty_response_type() const;
   Json_on_response_type on_error_response_type() const;
 
@@ -1195,12 +1253,12 @@ class Item_func_json_value final : public Item_func {
   int64_t extract_integer_value();
   /// Implements val_int() for RETURNING YEAR
   int64_t extract_year_value();
-  /// Implements get_date() for RETURNING DATE.
-  bool extract_date_value(MYSQL_TIME *ltime);
-  /// Implements get_time() for RETURNING TIME.
-  bool extract_time_value(MYSQL_TIME *ltime);
-  /// Implements get_date() for RETURNING DATETIME.
-  bool extract_datetime_value(MYSQL_TIME *ltime);
+  /// Implements val_date() for RETURNING DATE.
+  bool extract_date_value(Date_val *date);
+  /// Implements val_time() for RETURNING TIME.
+  bool extract_time_value(Time_val *time);
+  /// Implements val_datetime() for RETURNING DATETIME.
+  bool extract_datetime_value(Datetime_val *dt);
   /// Implements val_decimal() for RETURNING DECIMAL.
   my_decimal *extract_decimal_value(my_decimal *value);
   /// Implements val_str() for RETURNING CHAR and RETURNING BINARY.
