@@ -1,7 +1,7 @@
 #ifndef ITEM_TIMEFUNC_INCLUDED
 #define ITEM_TIMEFUNC_INCLUDED
 
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -157,10 +157,13 @@ class Item_func_month final : public Item_func {
     str->set(nr, collation.collation);
     return str;
   }
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
-    return get_date_from_int(ltime, fuzzydate);
+  bool val_date(Date_val *date, my_time_flags_t flags) override {
+    return get_date_from_int(date, flags);
+  }
+  bool val_time(Time_val *time) override { return get_time_from_int(time); }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override {
+    return get_datetime_from_int(dt, flags);
   }
-  bool get_time(MYSQL_TIME *ltime) override { return get_time_from_int(ltime); }
   const char *func_name() const override { return "month"; }
   enum Functype functype() const override { return MONTH_FUNC; }
   enum Item_result result_type() const override { return INT_RESULT; }
@@ -330,10 +333,13 @@ class Item_func_weekday : public Item_func {
     str->set(val_int(), &my_charset_bin);
     return null_value ? nullptr : str;
   }
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
-    return get_date_from_int(ltime, fuzzydate);
+  bool val_date(Date_val *date, my_time_flags_t flags) override {
+    return get_date_from_int(date, flags);
+  }
+  bool val_time(Time_val *time) override { return get_time_from_int(time); }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override {
+    return get_datetime_from_int(dt, flags);
   }
-  bool get_time(MYSQL_TIME *ltime) override { return get_time_from_int(ltime); }
   const char *func_name() const override {
     return (odbc_type ? "dayofweek" : "weekday");
   }
@@ -360,11 +366,12 @@ class Item_func_dayname final : public Item_func_weekday {
   const char *func_name() const override { return "dayname"; }
   enum Functype functype() const override { return DAYNAME_FUNC; }
   String *val_str(String *str) override;
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
-    return get_date_from_string(ltime, fuzzydate);
+  bool val_date(Date_val *date, my_time_flags_t flags) override {
+    return get_date_from_string(date, flags);
   }
-  bool get_time(MYSQL_TIME *ltime) override {
-    return get_time_from_string(ltime);
+  bool val_time(Time_val *time) override { return get_time_from_string(time); }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override {
+    return get_datetime_from_string(dt, flags);
   }
   enum Item_result result_type() const override { return STRING_RESULT; }
   bool resolve_type(THD *thd) override;
@@ -391,11 +398,12 @@ class Item_timeval_func : public Item_func {
   double val_real() override;
   String *val_str(String *str) override;
   my_decimal *val_decimal(my_decimal *decimal_value) override;
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override {
-    return get_date_from_numeric(ltime, fuzzydate);
+  bool val_date(Date_val *date, my_time_flags_t flags) override {
+    return get_date_from_numeric(date, flags);
   }
-  bool get_time(MYSQL_TIME *ltime) override {
-    return get_time_from_numeric(ltime);
+  bool val_time(Time_val *time) override { return get_time_from_numeric(time); }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override {
+    return get_datetime_from_numeric(dt, flags);
   }
   enum Item_result result_type() const override {
     return decimals ? DECIMAL_RESULT : INT_RESULT;
@@ -534,13 +542,30 @@ class Item_temporal_hybrid_func : public Item_str_func {
   sql_mode_t sql_mode;  // sql_mode value is cached here in resolve_type()
   String ascii_buf;     // Conversion buffer
   /**
-    Get "native" temporal value as MYSQL_TIME
-    @param[out] ltime       The value is stored here.
-    @param[in]  fuzzy_date  Date flags.
-    @retval     false       On success.
-    @retval     true        On error.
+    Evaluate temporal expression as datetime value
+    @param[out] dt     The value is stored here.
+    @param      flags  Date flags.
+
+    @returns false on success, true on error or if NULL value
   */
-  virtual bool val_datetime(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) = 0;
+  virtual bool eval_datetime(Datetime_val *dt, my_time_flags_t flags) = 0;
+  /**
+    Evaluate temporal expression as date value
+    @param[out] date   The value is stored here.
+    @param      flags  Date flags.
+
+    @returns false on success, true on error or if NULL value
+  */
+  bool eval_date(Date_val *date, my_time_flags_t flags) {
+    return eval_datetime(date, flags);
+  }
+  /**
+    Evaluate temporal expression as time value
+    @param[out] time  The value is stored here.
+
+    @returns false on success, true on error or if NULL value
+  */
+  virtual bool eval_time(Time_val *time) = 0;
   type_conversion_status save_in_field_inner(Field *field,
                                              bool no_conversions) override;
   void add_json_info(Json_object *obj) override {
@@ -581,8 +606,9 @@ class Item_temporal_hybrid_func : public Item_str_func {
   String *val_str(String *str) override {
     return val_str_from_val_str_ascii(str, &ascii_buf);
   }
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) override;
-  bool get_time(MYSQL_TIME *ltime) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_time(Time_val *time) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
 };
 
 /*
@@ -611,9 +637,7 @@ class Item_date_func : public Item_temporal_func {
       : Item_temporal_func(pos, a, b) {
     set_data_type_date();
   }
-  bool get_time(MYSQL_TIME *ltime) override {
-    return get_time_from_date(ltime);
-  }
+  bool val_time(Time_val *time) override { return get_time_from_date(time); }
   String *val_str(String *str) override { return val_string_from_date(str); }
   longlong val_int() override { return val_int_from_date(); }
   longlong val_date_temporal() override;
@@ -625,10 +649,10 @@ class Item_date_func : public Item_temporal_func {
     assert(fixed);
     return val_decimal_from_date(decimal_value);
   }
-  // All date functions must implement get_date()
-  // to avoid use of generic Item::get_date()
+  // All date functions must implement val_date()
+  // to avoid use of generic Item::val_date()
   // which converts to string and then parses the string as DATE.
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override = 0;
+  bool val_date(Date_val *date, my_time_flags_t flags) override = 0;
 };
 
 /**
@@ -683,13 +707,16 @@ class Item_datetime_func : public Item_temporal_func {
     assert(fixed);
     return val_decimal_from_date(decimal_value);
   }
-  bool get_time(MYSQL_TIME *ltime) override {
-    return get_time_from_datetime(ltime);
+  bool val_time(Time_val *time) override {
+    return get_time_from_datetime(time);
   }
-  // All datetime functions must implement get_date()
-  // to avoid use of generic Item::get_date()
+  // All datetime functions must implement val_datetime()
+  // to avoid use of generic Item::val_datetime()
   // which converts to string and then parses the string as DATETIME.
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override = 0;
+  bool val_date(Date_val *date, my_time_flags_t flags) override {
+    return val_datetime(date, flags);
+  }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override = 0;
 };
 
 /**
@@ -726,15 +753,17 @@ class Item_time_func : public Item_temporal_func {
     return val_decimal_from_time(decimal_value);
   }
   longlong val_int() override { return val_int_from_time(); }
-  longlong val_time_temporal() override;
-  bool get_date(MYSQL_TIME *res, my_time_flags_t) override {
-    return get_date_from_time(res);
+  bool val_date(Date_val *date, my_time_flags_t) override {
+    return get_date_from_time(date);
+  }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t) override {
+    return get_datetime_from_time(dt);
   }
   String *val_str(String *str) override { return val_string_from_time(str); }
-  // All time functions must implement get_time()
-  // to avoid use of generic Item::get_time()
+  // All time functions must implement val_time()
+  // to avoid use of generic Item::val_time()
   // which converts to string and then parses the string as TIME.
-  bool get_time(MYSQL_TIME *res) override = 0;
+  bool val_time(Time_val *time) override = 0;
 };
 
 /**
@@ -768,10 +797,6 @@ class MYSQL_TIME_cache {
     Set time and time_packed from a DATE value.
   */
   void set_date(MYSQL_TIME *ltime);
-  /**
-    Set time and time_packed from a TIME value.
-  */
-  void set_time(MYSQL_TIME *ltime, uint8 dec_arg);
   /**
     Set time and time_packed from a DATETIME value.
   */
@@ -782,11 +807,6 @@ class MYSQL_TIME_cache {
     in "struct timeval" representation and its time zone.
   */
   void set_date(my_timeval tv, Time_zone *tz);
-  /**
-    Set time and time_packed according to TIME value
-    in "struct timeval" representation and its time zone.
-  */
-  void set_time(my_timeval tv, uint8 dec_arg, Time_zone *tz);
   /**
     Set time and time_packed according to DATETIME value
     in "struct timeval" representation and its time zone.
@@ -818,14 +838,12 @@ class MYSQL_TIME_cache {
     Store MYSQL_TIME representation into the given date/datetime variable
     checking date flags.
   */
-  bool get_date(MYSQL_TIME *ltime, uint fuzzyflags) const;
-  /**
-    Store MYSQL_TIME representation into the given time variable.
-  */
-  bool get_time(MYSQL_TIME *ltime) const {
+  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzydate) const;
+  bool get_date(MYSQL_TIME *ltime) const {
     get_TIME(ltime);
     return false;
   }
+
   /**
     Return pointer to MYSQL_TIME representation.
   */
@@ -833,6 +851,7 @@ class MYSQL_TIME_cache {
     assert(time.time_type != MYSQL_TIMESTAMP_NONE);
     return &time;
   }
+
   /**
     Store string representation into String.
   */
@@ -862,13 +881,18 @@ class Item_date_literal final : public Item_date_func {
   const char *func_name() const override { return "date_literal"; }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
+  uint64_t hash() override;
   longlong val_date_temporal() override {
     assert(fixed);
     return cached_time.val_packed();
   }
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override {
+  bool val_date(Date_val *date, my_time_flags_t flags) override {
     assert(fixed);
-    return cached_time.get_date(ltime, fuzzy_date);
+    return cached_time.get_date(date, flags);
+  }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override {
+    assert(fixed);
+    return cached_time.get_date(dt, flags);
   }
   String *val_str(String *str) override {
     assert(fixed);
@@ -887,33 +911,25 @@ class Item_date_literal final : public Item_date_func {
   TIME'10:10:10'
 */
 class Item_time_literal final : public Item_time_func {
-  MYSQL_TIME_cache cached_time;
-
  public:
   /**
     Constructor for Item_time_literal.
-    @param ltime    TIME value.
-    @param dec_arg  number of fractional digits in ltime.
+
+    @param time     TIME value, must have fraction according to dec_arg.
+    @param dec_arg  number of fractional digits in time.
   */
-  Item_time_literal(MYSQL_TIME *ltime, uint dec_arg) {
+  Item_time_literal(Time_val *time, uint dec_arg) : m_time(*time) {
     set_data_type_time(std::min(dec_arg, uint(DATETIME_MAX_DECIMALS)));
-    cached_time.set_time(ltime, decimals);
+    assert(m_time.is_adjusted(decimals));
     fixed = true;
   }
   const char *func_name() const override { return "time_literal"; }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
-  longlong val_time_temporal() override {
+  bool val_time(Time_val *time) override {
     assert(fixed);
-    return cached_time.val_packed();
-  }
-  bool get_time(MYSQL_TIME *ltime) override {
-    assert(fixed);
-    return cached_time.get_time(ltime);
-  }
-  String *val_str(String *str) override {
-    assert(fixed);
-    return cached_time.val_str(str);
+    *time = m_time;
+    return false;
   }
   bool resolve_type(THD *) override { return false; }
   bool check_partition_func_processor(uchar *) override { return false; }
@@ -922,6 +938,11 @@ class Item_time_literal final : public Item_time_func {
   table_map not_null_tables() const override { return used_tables(); }
   void cleanup() override { assert(marker == MARKER_NONE); }
   bool eq_specific(const Item *item) const override;
+
+ private:
+  Time_val m_time;
+  uint64_t m_hash{0};
+  uint64_t hash() override;
 };
 
 /**
@@ -933,14 +954,14 @@ class Item_datetime_literal final : public Item_datetime_func {
  public:
   /**
     Constructor for Item_datetime_literal.
-    @param ltime   DATETIME value.
+    @param dt      DATETIME value.
     @param dec_arg Number of fractional digits in ltime.
     @param tz      The current time zone, used for converting literals with
                    time zone upon storage.
   */
-  Item_datetime_literal(MYSQL_TIME *ltime, uint dec_arg, const Time_zone *tz) {
+  Item_datetime_literal(Datetime_val *dt, uint dec_arg, const Time_zone *tz) {
     set_data_type_datetime(std::min(dec_arg, uint{DATETIME_MAX_DECIMALS}));
-    cached_time.set_datetime(ltime, decimals, tz);
+    cached_time.set_datetime(dt, decimals, tz);
     fixed = true;
   }
   const char *func_name() const override { return "datetime_literal"; }
@@ -951,9 +972,13 @@ class Item_datetime_literal final : public Item_datetime_func {
     assert(fixed);
     return cached_time.val_packed();
   }
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override {
+  bool val_date(Date_val *date, my_time_flags_t flags) override {
     assert(fixed);
-    return cached_time.get_date(ltime, fuzzy_date);
+    return cached_time.get_date(date, flags);
+  }
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override {
+    assert(fixed);
+    return cached_time.get_date(dt, flags);
   }
   String *val_str(String *str) override {
     assert(fixed);
@@ -966,6 +991,7 @@ class Item_datetime_literal final : public Item_datetime_func {
   table_map not_null_tables() const override { return used_tables(); }
   void cleanup() override { assert(marker == MARKER_NONE); }
   bool eq_specific(const Item *item) const override;
+  uint64_t hash() override;
 };
 
 /**
@@ -997,9 +1023,13 @@ class Item_func_at_time_zone final : public Item_datetime_func {
 
   const char *func_name() const override { return "time_zone"; }
 
+  void print(const THD *thd, String *str,
+             enum_query_type query_type) const override;
+
   bool set_time_zone(THD *thd);
 
-  bool get_date(MYSQL_TIME *res, my_time_flags_t) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
 
   const char *specifier_string() const { return m_specifier_string; }
 
@@ -1050,9 +1080,7 @@ class Item_func_curtime : public Item_time_func {
   }
 
   bool resolve_type(THD *thd) override;
-  longlong val_time_temporal() override;
-  bool get_time(MYSQL_TIME *ltime) override;
-  String *val_str(String *) override;
+  bool val_time(Time_val *time) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
         pointer_cast<Check_function_as_value_generator_parameters *>(
@@ -1106,7 +1134,8 @@ class Item_func_curdate : public Item_date_func {
 
   bool resolve_type(THD *) override;
   longlong val_date_temporal() override;
-  bool get_date(MYSQL_TIME *res, my_time_flags_t) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   String *val_str(String *) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
@@ -1169,7 +1198,8 @@ class Item_func_now : public Item_datetime_func {
 
   bool resolve_type(THD *) override;
   longlong val_date_temporal() override;
-  bool get_date(MYSQL_TIME *res, my_time_flags_t) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   String *val_str(String *) override;
   bool check_function_as_value_generator(uchar *checker_args) override {
     Check_function_as_value_generator_parameters *func_arg =
@@ -1229,7 +1259,8 @@ class Item_func_sysdate_local final : public Item_datetime_func {
   const char *func_name() const override { return "sysdate"; }
   bool resolve_type(THD *) override;
   enum Functype functype() const override { return SYSDATE_FUNC; }
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   /**
     This function is non-deterministic and hence depends on the 'RAND'
     pseudo-table.
@@ -1245,7 +1276,8 @@ class Item_func_from_days final : public Item_date_func {
  public:
   Item_func_from_days(const POS &pos, Item *a) : Item_date_func(pos, a) {}
   const char *func_name() const override { return "from_days"; }
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   bool check_partition_func_processor(uchar *) override { return false; }
   enum Functype functype() const override { return FROM_DAYS_FUNC; }
   bool check_valid_arguments_processor(uchar *) override {
@@ -1283,7 +1315,8 @@ class Item_func_from_unixtime final : public Item_datetime_func {
   const char *func_name() const override { return "from_unixtime"; }
   enum Functype functype() const override { return FROM_UNIXTIME_FUNC; }
   bool resolve_type(THD *thd) override;
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
 };
 
 /*
@@ -1312,7 +1345,8 @@ class Item_func_convert_tz final : public Item_datetime_func {
   const char *func_name() const override { return "convert_tz"; }
   enum Functype functype() const override { return CONVERT_TZ_FUNC; }
   bool resolve_type(THD *) override;
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   void cleanup() override;
 };
 
@@ -1330,7 +1364,7 @@ class Item_func_sec_to_time final : public Item_time_func {
   }
   const char *func_name() const override { return "sec_to_time"; }
   enum Functype functype() const override { return SEC_TO_TIME_FUNC; }
-  bool get_time(MYSQL_TIME *ltime) override;
+  bool val_time(Time_val *time) override;
 };
 
 extern const char *interval_names[];
@@ -1369,9 +1403,9 @@ class Item_date_add_interval final : public Item_temporal_hybrid_func {
   void add_json_info(Json_object *obj) override;
 
  private:
-  bool val_datetime(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
-  bool get_date_internal(MYSQL_TIME *res, my_time_flags_t fuzzy_date);
-  bool get_time_internal(MYSQL_TIME *res);
+  bool eval_datetime(Datetime_val *dt, my_time_flags_t flags) override;
+  bool eval_time(Time_val *time) override;
+  bool get_datetime_internal(Datetime_val *dt, my_time_flags_t flags);
 
   /// The type of the interval argument
   const interval_type m_interval_type;
@@ -1454,7 +1488,8 @@ class Item_typecast_date final : public Item_date_func {
   const char *func_name() const override { return "cast_as_date"; }
   enum Functype functype() const override { return TYPECAST_FUNC; }
   bool is_explicit_cast() const { return m_explicit_cast; }
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   const char *cast_type() const { return "date"; }
 };
 
@@ -1483,7 +1518,7 @@ class Item_typecast_time final : public Item_time_func {
   const char *func_name() const override { return "cast_as_time"; }
   enum Functype functype() const override { return TYPECAST_FUNC; }
   bool is_explicit_cast() const { return m_explicit_cast; }
-  bool get_time(MYSQL_TIME *ltime) override;
+  bool val_time(Time_val *time) override;
   const char *cast_type() const { return "time"; }
   bool resolve_type(THD *thd) override {
     if (reject_vector_args()) return true;
@@ -1532,7 +1567,8 @@ class Item_typecast_datetime final : public Item_datetime_func {
     set_nullable(true);
     return false;
   }
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
 };
 
 class Item_func_makedate final : public Item_date_func {
@@ -1542,7 +1578,8 @@ class Item_func_makedate final : public Item_date_func {
     set_nullable(true);
   }
   const char *func_name() const override { return "makedate"; }
-  bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   enum Functype functype() const override { return MAKEDATE_FUNC; }
   bool resolve_type(THD *thd) override {
     if (reject_vector_args()) return true;
@@ -1559,20 +1596,21 @@ class Item_func_makedate final : public Item_date_func {
 */
 class Item_func_add_time final : public Item_temporal_hybrid_func {
   const bool m_datetime;  ///< True if first argument expected to be datetime
-  int m_sign;             ///< +1 for ADD, -1 for SUBTRACT
+  bool m_subtract;        ///< FALSE for ADD, TRUE for SUBTRACT
 
-  bool val_datetime(MYSQL_TIME *time, my_time_flags_t fuzzy_date) override;
+  bool eval_datetime(Datetime_val *dt, my_time_flags_t flags) override;
+  bool eval_time(Time_val *time) override;
 
  public:
-  Item_func_add_time(Item *a, Item *b, bool datetime, bool negate)
+  Item_func_add_time(Item *a, Item *b, bool datetime, bool subtract)
       : Item_temporal_hybrid_func(a, b),
         m_datetime(datetime),
-        m_sign(negate ? -1 : 1) {}
+        m_subtract(subtract) {}
   Item_func_add_time(const POS &pos, Item *a, Item *b, bool datetime,
-                     bool negate)
+                     bool subtract)
       : Item_temporal_hybrid_func(pos, a, b),
         m_datetime(datetime),
-        m_sign(negate ? -1 : 1) {}
+        m_subtract(subtract) {}
 
   Item_func_add_time(const POS &pos, Item *a, Item *b)
       : Item_func_add_time(pos, a, b, false, false) {}
@@ -1582,7 +1620,7 @@ class Item_func_add_time final : public Item_temporal_hybrid_func {
              enum_query_type query_type) const override;
   const char *func_name() const override { return "add_time"; }
   enum Functype functype() const override { return ADDTIME_FUNC; }
-  int sign() const { return m_sign; }
+  bool is_subtract() const { return m_subtract; }
 };
 
 class Item_func_timediff final : public Item_time_func {
@@ -1606,7 +1644,7 @@ class Item_func_timediff final : public Item_time_func {
     set_nullable(true);
     return false;
   }
-  bool get_time(MYSQL_TIME *ltime) override;
+  bool val_time(Time_val *time) override;
 };
 
 class Item_func_maketime final : public Item_time_func {
@@ -1625,7 +1663,7 @@ class Item_func_maketime final : public Item_time_func {
   }
   const char *func_name() const override { return "maketime"; }
   enum Functype functype() const override { return MAKETIME_FUNC; }
-  bool get_time(MYSQL_TIME *ltime) override;
+  bool val_time(Time_val *time) override;
 };
 
 class Item_func_microsecond final : public Item_int_func {
@@ -1697,7 +1735,16 @@ class Item_func_str_to_date final : public Item_temporal_hybrid_func {
   void fix_from_format(const char *format, size_t length);
 
  protected:
-  bool val_datetime(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
+  bool eval_datetime(Datetime_val *dt, my_time_flags_t flags) override;
+  bool eval_time(Time_val *time) override {
+    Datetime_val dt;
+    if (eval_datetime(&dt, TIME_FUZZY_DATE)) return true;
+    if (dt.time_type != MYSQL_TIMESTAMP_TIME) {
+      datetime_to_time(&dt);
+    }
+    *time = Time_val(dt);
+    return false;
+  }
 
  public:
   Item_func_str_to_date(const POS &pos, Item *a, Item *b)
@@ -1713,7 +1760,8 @@ class Item_func_last_day final : public Item_date_func {
   }
   const char *func_name() const override { return "last_day"; }
   enum Functype functype() const override { return LAST_DAY_FUNC; }
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
   bool resolve_type(THD *thd) override {
     if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
@@ -1728,7 +1776,8 @@ class Item_func_internal_update_time final : public Item_datetime_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   const char *func_name() const override { return "internal_update_time"; }
   bool resolve_type(THD *thd) override;
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
 };
 
 class Item_func_internal_check_time final : public Item_datetime_func {
@@ -1738,7 +1787,8 @@ class Item_func_internal_check_time final : public Item_datetime_func {
   enum Functype functype() const override { return DD_INTERNAL_FUNC; }
   const char *func_name() const override { return "internal_check_time"; }
   bool resolve_type(THD *thd) override;
-  bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
+  bool val_date(Date_val *date, my_time_flags_t flags) override;
+  bool val_datetime(Datetime_val *dt, my_time_flags_t flags) override;
 };
 
 /* Function prototypes */
