@@ -1,4 +1,4 @@
-/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -26,7 +26,9 @@
 
 #include <assert.h>
 #include <stdint.h>
+#include <cmath>
 #include <cstddef>
+#include <span>
 #include <type_traits>
 #include <utility>
 #include <vector>
@@ -92,12 +94,12 @@ struct JoinPredicate {
   // or {a, c} → b.
   //
   // Used in the processing of interesting orders.
-  FunctionalDependencySet functional_dependencies;
+  FunctionalDependencySet functional_dependencies{};
 
   // A less compact form of functional_dependencies, used during building
   // (FunctionalDependencySet bitmaps are only available after all functional
   // indexes have been collected and Build() has been called).
-  Mem_root_array<int> functional_dependencies_idx;
+  Mem_root_array<int> functional_dependencies_idx{};
 
   // A semijoin on the following format:
   //
@@ -120,8 +122,7 @@ struct JoinPredicate {
   // Same as ordering_idx_needed_for_semijoin_rewrite, but given to the
   // RemoveDuplicatesIterator for doing the actual grouping. Allocated
   // on the MEM_ROOT. Can be empty, in which case a LIMIT 1 would do.
-  Item **semijoin_group = nullptr;
-  int semijoin_group_size = 0;
+  std::span<Item *> semijoin_group{};
 };
 
 /**
@@ -157,6 +158,17 @@ struct Predicate {
   // sargable predicate.
   bool was_join_condition = false;
 
+  // Whether this predicate references tables that could be NULL-complemented
+  // later by an outer join. This could for example be true for degenerate outer
+  // join conditions that are pushed down as a table filter on one of the inner
+  // tables, or for join conditions in inner joins that are on the inner side of
+  // an outer join.
+  //
+  // We keep track of this here in order to prevent collection of functional
+  // dependencies from such predicates if the functional dependencies are not
+  // valid after the outer join.
+  bool possibly_null_complemented_later = false;
+
   // If this is a join condition that came from a multiple equality,
   // and we have decided to create a mesh from that multiple equality,
   // returns the index of it into the “multiple_equalities” array
@@ -181,12 +193,25 @@ struct AppendPathParameters {
 };
 
 /// To indicate that a row estimate is not yet made.
-constexpr double kUnknownRowCount = -1.0;
+inline constexpr double kUnknownRowCount = -1.0;
 
 /// To indicate that a cost estimate is not yet made. We use a large negative
 /// value to avoid getting a positive result if we by mistake add this to
 /// a real (positive) cost.
-constexpr double kUnknownCost = -1e12;
+inline constexpr double kUnknownCost = -1e12;
+
+/// Calculate the cost of reading the first row from an access path, given
+/// estimates for init cost, total cost and the number of rows returned.
+inline double FirstRowCost(double init_cost, double total_cost,
+                           double output_rows) {
+  assert(init_cost >= 0.0);
+  assert(total_cost >= init_cost);
+  assert(output_rows >= 0.0);
+  if (output_rows <= 1.0) {
+    return total_cost;
+  }
+  return init_cost + (total_cost - init_cost) / output_rows;
+}
 
 /**
   Access paths are a query planning structure that correspond 1:1 to iterators,
@@ -211,21 +236,6 @@ constexpr double kUnknownCost = -1e12;
   planning structure.
  */
 struct AccessPath {
-  // Most of the members are declared with initializers, but bit fields cannot
-  // be declared with initializers until C++20, so we need to define a
-  // constructor to initialize those fields.
-  // TODO(khatlen): When we move to C++20, add initializers to the bit fields
-  // too, and use the default constructor generated by the compiler.
-  AccessPath()
-      : count_examined_rows(false),
-        has_group_skip_scan(false)
-#ifndef NDEBUG
-        ,
-        forced_by_dbug(false)
-#endif
-  {
-  }
-
   enum Type : uint8_t {
     // Basic access paths (those with no children, at least nominally).
     // NOTE: When adding more paths to this section, also update GetBasicTable()
@@ -318,15 +328,15 @@ struct AccessPath {
   /// and thus should be counted towards examined_rows. It can sometimes
   /// seem a bit arbitrary which iterators count towards examined_rows
   /// and which ones do not, so the only canonical reference is the tests.
-  bool count_examined_rows : 1;
+  bool count_examined_rows : 1 {false};
 
   /// Whether this access path contains a GROUP_INDEX_SKIP_SCAN
-  bool has_group_skip_scan : 1;
+  bool has_group_skip_scan : 1 {false};
 
 #ifndef NDEBUG
   /// Whether this access path is forced preferred over all others by means
   /// of a SET DEBUG force_subplan_0x... statement.
-  bool forced_by_dbug : 1;
+  bool forced_by_dbug : 1 {false};
 #endif
 
   /// For UPDATE and DELETE statements: The node index of a table which can be
@@ -388,26 +398,35 @@ struct AccessPath {
 
   double init_cost() const { return m_init_cost; }
 
+  /// The cost of reading the first row.
+  double first_row_cost() const {
+    return FirstRowCost(m_init_cost, m_cost, m_num_output_rows);
+  }
+
   double init_once_cost() const { return m_init_once_cost; }
 
   double cost_before_filter() const { return m_cost_before_filter; }
 
   void set_cost(double val) {
+    assert(std::isfinite(val));
     assert(val >= 0.0 || val == kUnknownCost);
     m_cost = val;
   }
 
   void set_init_cost(double val) {
+    assert(std::isfinite(val));
     assert(val >= 0.0 || val == kUnknownCost);
     m_init_cost = val;
   }
 
   void set_init_once_cost(double val) {
+    assert(std::isfinite(val));
     assert(val >= 0.0);
     m_init_once_cost = val;
   }
 
   void set_cost_before_filter(double val) {
+    assert(std::isfinite(val));
     assert(val >= 0.0 || val == kUnknownCost);
     m_cost_before_filter = val;
   }
@@ -420,6 +439,9 @@ struct AccessPath {
   /// in the code.
   double rescan_cost() const { return cost() - init_once_cost(); }
 
+  /// Return true if costs and row counts are consistent.
+  bool HasConsistentCostsAndRows(const JoinHypergraph &graph) const;
+
   /// If no filter, identical to num_output_rows.
   double num_output_rows_before_filter{kUnknownRowCount};
 
@@ -518,6 +540,10 @@ struct AccessPath {
   /// destroyed.
   void *secondary_engine_data{nullptr};
 
+  /// Signature used to uniquely identify the access path.
+  /// 0 meaning non-initialized.
+  size_t signature{0};
+
   // Accessors for the union below.
   auto &table_scan() {
     assert(type == TABLE_SCAN);
@@ -890,7 +916,11 @@ struct AccessPath {
 
   double num_output_rows() const { return m_num_output_rows; }
 
-  void set_num_output_rows(double val) { m_num_output_rows = val; }
+  void set_num_output_rows(double val) {
+    assert(std::isfinite(val));
+    assert(val == kUnknownRowCount || val >= 0.0);
+    m_num_output_rows = val;
+  }
 
  private:
   /// Expected number of output rows.
@@ -1242,6 +1272,9 @@ struct AccessPath {
       MaterializePathParameters *param;
       /** The total cost of executing the queries that we materialize.*/
       double subquery_cost;
+      /// The number of materialized rows (as opposed to the number of rows
+      /// fetched by table_path). Needed for 'explain'.
+      double subquery_rows;
     } materialize;
     struct {
       AccessPath *table_path;
@@ -1265,9 +1298,25 @@ struct AccessPath {
       table_map tables_to_get_rowid_for;
     } weedout;
     struct {
+      using ItemSpan = std::span<Item *>;
       AccessPath *child;
-      Item **group_items;
-      int group_items_size;
+
+      /// @cond IGNORE
+      // These functions somehow triggers a doxygen warning. (Presumably
+      // a doxygen bug.)
+      ItemSpan &group_items() {
+        return reinterpret_cast<ItemSpan &>(m_group_items);
+      }
+
+      const ItemSpan &group_items() const {
+        return reinterpret_cast<const ItemSpan &>(m_group_items);
+      }
+      /// @endcond
+
+     private:
+      // gcc 11 does not support a span as a union member. Replace this
+      // with "std::span<Item *> group_items" when we move to newer gcc version.
+      alignas(alignof(ItemSpan)) std::byte m_group_items[sizeof(ItemSpan)];
     } remove_duplicates;
     struct {
       AccessPath *child;
@@ -1303,10 +1352,10 @@ static_assert(std::is_trivially_destructible<AccessPath>::value,
               "on the MEM_ROOT and not wrapped in unique_ptr_destroy_only"
               "(because multiple candidates during planning could point to "
               "the same access paths, and refcounting would be expensive)");
-static_assert(sizeof(AccessPath) <= 144,
+static_assert(sizeof(AccessPath) <= 152,
               "We are creating a lot of access paths in the join "
               "optimizer, so be sure not to bloat it without noticing. "
-              "(96 bytes for the base, 48 bytes for the variant.)");
+              "(104 bytes for the base, 52 bytes for the variant.)");
 
 inline void CopyBasicProperties(const AccessPath &from, AccessPath *to) {
   to->set_num_output_rows(from.num_output_rows());
@@ -1317,8 +1366,12 @@ inline void CopyBasicProperties(const AccessPath &from, AccessPath *to) {
   to->safe_for_rowid = from.safe_for_rowid;
   to->ordering_state = from.ordering_state;
   to->has_group_skip_scan = from.has_group_skip_scan;
+  to->signature = from.signature;
 }
 
+/// Return the name of an AccessPath::Type enumerator.
+std::string_view AccessPathTypeName(AccessPath::Type type);
+
 // Trivial factory functions for all of the types of access paths above.
 
 inline AccessPath *NewTableScanAccessPath(THD *thd, TABLE *table,
@@ -1497,6 +1550,8 @@ inline AccessPath *NewNestedLoopSemiJoinWithDuplicateRemovalAccessPath(
   path->nested_loop_semijoin_with_duplicate_removal().table = table;
   path->nested_loop_semijoin_with_duplicate_removal().key = key;
   path->nested_loop_semijoin_with_duplicate_removal().key_len = key_len;
+  path->has_group_skip_scan =
+      outer->has_group_skip_scan || inner->has_group_skip_scan;
   return path;
 }
 
@@ -1556,8 +1611,7 @@ inline AccessPath *NewLimitOffsetAccessPath(THD *thd, AccessPath *child,
   path->limit_offset().count_all_rows = count_all_rows;
   path->limit_offset().reject_multiple_rows = reject_multiple_rows;
   path->limit_offset().send_records_override = send_records_override;
-  path->ordering_state = child->ordering_state;
-  path->has_group_skip_scan = child->has_group_skip_scan;
+  CopyBasicProperties(*child, path);
   EstimateLimitOffsetCost(path);
   return path;
 }
@@ -1680,6 +1734,7 @@ inline AccessPath *NewMaterializeAccessPath(
   path->materialize().table_path = table_path;
   path->materialize().param = param;
   path->materialize().subquery_cost = kUnknownCost;
+  path->materialize().subquery_rows = kUnknownRowCount;
   if (rematerialize) {
     path->safe_for_rowid = AccessPath::SAFE_IF_SCANNED_ONCE;
   } else {
@@ -1711,6 +1766,18 @@ inline double AddCost(double c1, double c2) {
   }
 }
 
+/// Add row counts c1 and c2, but handle kUnknownRowCount correctly.
+inline double AddRowCount(double c1, double c2) {
+  // If one is undefined, use the other, as we have nothing else.
+  if (c1 == kUnknownRowCount) {
+    return c2;
+  } else if (c2 == kUnknownRowCount) {
+    return c1;
+  } else {
+    return c1 + c2;
+  }
+}
+
 // The Mem_root_array must be allocated on a MEM_ROOT that lives at least for as
 // long as the access path.
 inline AccessPath *NewAppendAccessPath(
@@ -1718,13 +1785,14 @@ inline AccessPath *NewAppendAccessPath(
   AccessPath *path = new (thd->mem_root) AccessPath;
   path->type = AccessPath::APPEND;
   path->append().children = children;
-  double num_output_rows = 0.0;
+  double num_output_rows = kUnknownRowCount;
   for (const AppendPathParameters &child : *children) {
     path->set_cost(AddCost(path->cost(), child.path->cost()));
     path->set_init_cost(AddCost(path->init_cost(), child.path->init_cost()));
     path->set_init_once_cost(path->init_once_cost() +
                              child.path->init_once_cost());
-    num_output_rows += child.path->num_output_rows();
+    num_output_rows =
+        AddRowCount(num_output_rows, child.path->num_output_rows());
   }
   path->set_num_output_rows(num_output_rows);
   return path;
@@ -1757,14 +1825,12 @@ inline AccessPath *NewWeedoutAccessPath(THD *thd, AccessPath *child,
   return path;
 }
 
-inline AccessPath *NewRemoveDuplicatesAccessPath(THD *thd, AccessPath *child,
-                                                 Item **group_items,
-                                                 int group_items_size) {
+inline AccessPath *NewRemoveDuplicatesAccessPath(
+    THD *thd, AccessPath *child, std::span<Item *> group_items) {
   AccessPath *path = new (thd->mem_root) AccessPath;
   path->type = AccessPath::REMOVE_DUPLICATES;
   path->remove_duplicates().child = child;
-  path->remove_duplicates().group_items = group_items;
-  path->remove_duplicates().group_items_size = group_items_size;
+  path->remove_duplicates().group_items() = group_items;
   path->has_group_skip_scan = child->has_group_skip_scan;
   return path;
 }
@@ -1807,7 +1873,7 @@ AccessPath *NewDeleteRowsAccessPath(THD *thd, AccessPath *child,
                                     table_map immediate_tables);
 
 AccessPath *NewUpdateRowsAccessPath(THD *thd, AccessPath *child,
-                                    table_map delete_tables,
+                                    table_map update_tables,
                                     table_map immediate_tables);
 
 /**
@@ -1857,6 +1923,42 @@ void SetCostOnTableAccessPath(const Cost_model_server &cost_model,
 */
 TABLE *GetBasicTable(const AccessPath *path);
 
+/**
+  Applies the secondary storage engine nrows modification function, if any.
+
+  @param params input params for the callback function.
+   Refer to typedef for the actual input parameters explainations.
+
+  @return true if secondary engine has proposed modification to ap's nrows.
+*/
+bool ApplySecondaryEngineNrowsHook(
+    const SecondaryEngineNrowsParameters &params);
+
+/**
+  Returns whether SecondaryNrows is applicable given the parameters.
+
+  @param path access path to be verified.
+  @param thd current query thd.
+  @param graph current query block hypergraph.
+
+  @return true if nrows hook is applicable.
+*/
+bool IsSecondaryEngineNrowsHookApplicable(AccessPath *path, THD *thd,
+                                          const JoinHypergraph *graph);
+
+/**
+  Returns whether SecondaryNrows hook enabled and is applicable given the
+  parameters.
+
+  @param path access path to be verified.
+  @param thd current query thd.
+  @param graph current query block hypergraph.
+
+  @return true if nrows hook enabled and is applicable.
+*/
+bool IsSecondaryNrowsHookEnabledAndApplicable(AccessPath *path, THD *thd,
+                                              const JoinHypergraph *graph);
+
 /**
   Returns a map of all tables read when `path` or any of its children are
   executed. Only iterators that are part of the same query block as `path`
@@ -1892,9 +1994,8 @@ Mem_root_array<TABLE *> CollectTables(THD *thd, AccessPath *root_path);
 
   “join” is the join that “path” is part of.
  */
-void ExpandFilterAccessPaths(THD *thd, AccessPath *path, const JOIN *join,
-                             const Mem_root_array<Predicate> &predicates,
-                             unsigned num_where_predicates);
+void ExpandFilterAccessPaths(THD *thd, const JoinHypergraph &graph,
+                             AccessPath *path, const JOIN *join);
 
 /**
   Extracts the Item expression from the given “filter_predicates” corresponding
@@ -1906,9 +2007,29 @@ Item *ConditionFromFilterPredicates(const Mem_root_array<Predicate> &predicates,
 
 /// Like ExpandFilterAccessPaths(), but expands only the single access path
 /// at “path”.
-void ExpandSingleFilterAccessPath(THD *thd, AccessPath *path, const JOIN *join,
-                                  const Mem_root_array<Predicate> &predicates,
-                                  unsigned num_where_predicates);
+void ExpandSingleFilterAccessPath(THD *thd, const JoinHypergraph &graph,
+                                  AccessPath *path, const JOIN *join);
+
+/**
+  Clear all the bits representing filter predicates in a bitset, and keep only
+  the bits representing applied sargable join conditions.
+
+  See AccessPath::filter_predicates and
+  AccessPath::applied_sargable_join_predicates() for details about how filter
+  predicates and applied sargable join predicates are stored in different
+  partitions of the same bitset.
+
+  @param predicates A bitset representing both filter predicates and applied
+  sargable join predicates.
+  @param num_where_predicates The number of filter predicates.
+  @param mem_root The root on which to allocate memory, if needed.
+
+  @return A copy of "predicates" with only the bits for applied sargable join
+  predicates set.
+ */
+MutableOverflowBitset ClearFilterPredicates(OverflowBitset predicates,
+                                            int num_where_predicates,
+                                            MEM_ROOT *mem_root);
 
 /// Returns the tables that are part of a hash join.
 table_map GetHashJoinTables(AccessPath *path);
