@@ -1,4 +1,4 @@
-/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,6 +29,8 @@
 #include <algorithm>
 #include <array>
 #include <cstddef>
+#include <optional>
+#include <span>
 #include <string>
 #include <unordered_map>
 #include <utility>
@@ -44,7 +46,6 @@
 #include "sql/mem_root_array.h"
 #include "sql/sql_const.h"
 
-class CompanionSet;
 class Field;
 class JOIN;
 class Query_block;
@@ -75,6 +76,15 @@ struct SargablePredicate {
   bool can_evaluate;
 };
 
+/// Information about a join condition that can potentially be pushed down as a
+/// sargable predicate for a Node.
+struct PushableJoinCondition {
+  /// The condition that may be pushed as a sargable predicate.
+  Item *cond;
+  /// The relational expression from which this condition is pushable.
+  const RelationalExpression *from;
+};
+
 /**
   A struct containing a join hypergraph of a single query block, encapsulating
   the constraints given by the relational expressions (e.g. inner joins are
@@ -110,9 +120,8 @@ struct JoinHypergraph {
 
   class Node final {
    public:
-    Node(MEM_ROOT *mem_root, TABLE *table, const CompanionSet *companion_set)
+    Node(MEM_ROOT *mem_root, TABLE *table)
         : m_table{table},
-          m_companion_set{companion_set},
           m_sargable_predicates{mem_root},
           m_pushable_conditions(mem_root) {
       assert(mem_root != nullptr);
@@ -120,7 +129,6 @@ struct JoinHypergraph {
     }
 
     TABLE *table() const { return m_table; }
-    const CompanionSet *companion_set() const { return m_companion_set; }
 
     void AddSargable(const SargablePredicate &predicate) {
       assert(predicate.predicate_index >= 0);
@@ -133,18 +141,27 @@ struct JoinHypergraph {
       return m_sargable_predicates;
     }
 
-    void AddPushable(Item *cond) {
+    /**
+      Add a join condition that is potentially pushable as a sargable predicate
+      for this node.
+
+      @param cond   The condition to be considered for pushdown.
+      @param from   The relational expression from which this condition
+                    originates.
+    */
+    void AddPushable(Item *cond, const RelationalExpression *from) {
       // Don't add duplicate conditions, as this causes their selectivity to
       // be applied multiple times, giving poor row estimates (cf.
       // bug#36135001).
-      if (std::none_of(
-              m_pushable_conditions.cbegin(), m_pushable_conditions.cend(),
-              [&](const Item *other) { return ItemsAreEqual(cond, other); })) {
-        m_pushable_conditions.push_back(cond);
+      if (std::ranges::none_of(m_pushable_conditions,
+                               [&](const PushableJoinCondition &other) {
+                                 return ItemsAreEqual(cond, other.cond);
+                               })) {
+        m_pushable_conditions.push_back({.cond = cond, .from = from});
       }
     }
 
-    const Mem_root_array<Item *> &pushable_conditions() const {
+    const Mem_root_array<PushableJoinCondition> &pushable_conditions() const {
       return m_pushable_conditions;
     }
 
@@ -156,9 +173,11 @@ struct JoinHypergraph {
       m_lateral_dependencies = dependencies;
     }
 
+    /* Estimated rows cardinality after table filters. */
+    std::optional<double> cardinality;
+
    private:
     TABLE *m_table;
-    const CompanionSet *m_companion_set;
     // List of all sargable predicates (see SargablePredicate) where
     // the field is part of this table. When we see the node for
     // the first time, we will evaluate all of these and consider
@@ -175,7 +194,7 @@ struct JoinHypergraph {
     // ourselves when determining sargable conditions, but there would be
     // a fair amount of duplicated code in determining pushability,
     // which is why regular join pushdown does the computation.)
-    Mem_root_array<Item *> m_pushable_conditions;
+    Mem_root_array<PushableJoinCondition> m_pushable_conditions;
 
     // The lateral dependencies of this table. That is, the set of tables that
     // must be available on the outer side of a nested loop join in which this
@@ -189,15 +208,58 @@ struct JoinHypergraph {
   // for more information), so edges[i] corresponds to graph.edges[i*2].
   Mem_root_array<JoinPredicate> edges;
 
-  // The first <num_where_predicates> are WHERE predicates;
-  // the rest are sargable join predicates. The latter are in the array
+  // The first <num_where_predicates> are filter predicates. These are the
+  // predicates that may be added as filters on nodes in the join tree by
+  // setting the corresponding bit in AccessPath::filter_predicates, which at
+  // the end of join optimization gets expanded to proper FILTER access paths by
+  // ExpandFilterAccessPaths(). Despite the name "num_where_predicates", they
+  // are not necessarily WHERE predicates. They include:
+  //
+  // - Actual WHERE predicates that could not be pushed down into one of the
+  //   join conditions.
+  //
+  // - Predicates that could be pushed all the way down and become a table
+  //   filter. These could be WHERE predicates, but they could also be
+  //   predicates that are possible to push all the way down, but not possible
+  //   to pull all the way up. Take for example "SELECT 1 FROM t1 LEFT JOIN t2
+  //   ON t1.a=t2.a AND t2.b=1". The t2.b=1 predicate can be pushed down as a
+  //   table filter, but it cannot be used as a WHERE predicate, as it would
+  //   incorrectly filter out the NULL-complemented rows. Still, such table
+  //   filters are also counted in "num_where_predicates".
+  //
+  // - Predicates that are join conditions in some inner join that is involved
+  //   in a cycle in the join hypergraph. These are applied as filters in the
+  //   join tree if the tables are joined via another edge in the cycle. Such
+  //   predicates are also not necessarily possible to pull up to the WHERE
+  //   clause. If they for example came from an inner join on the inner side of
+  //   some outer join, they cannot be applied as WHERE predicates. Even so,
+  //   they are still counted in "num_where_predicates".
+  //
+  // The rest are sargable join predicates. The latter are in the array
   // solely so they can be part of the regular “applied_filters” bitmap
   // if they are pushed down into an index, so that we know that we
   // don't need to apply them as join conditions later.
+  //
+  // If a sargable join predicate comes from a join that is part of a cycle in
+  // the hypergraph, it could be present in both partitions of the array.
   Mem_root_array<Predicate> predicates;
 
+  // How many of the predicates in "predicates" are filter predicates. The rest
+  // of them are sargable join predicates.
   unsigned num_where_predicates = 0;
 
+  /// Returns an immutable view of the filter predicates portion of the
+  /// predicates array.
+  std::span<const Predicate> filter_predicates() const {
+    return {predicates.data(), num_where_predicates};
+  }
+
+  /// Returns a mutable view of the filter predicates portion of the predicates
+  /// array.
+  std::span<Predicate> filter_predicates() {
+    return {predicates.data(), num_where_predicates};
+  }
+
   // A bitmap over predicates that are, or contain, at least one
   // materializable subquery.
   OverflowBitset materializable_predicates{0};
@@ -221,10 +283,23 @@ struct JoinHypergraph {
   /// the root cause.
   bool has_reordered_left_joins = false;
 
-  /// The set of tables that are on the inner side of some outer join or
-  /// antijoin. If a table is not part of this set, and it is found to be empty,
-  /// we can assume that the result of the top-level join will also be empty.
-  table_map tables_inner_to_outer_or_anti = 0;
+  // True if estimates for one or more Nodes in the graph have been provided
+  // by the secondary engine (i.e. at least one Node has the field `cardinality`
+  // set).
+  bool has_estimates_from_secondary_engine = false;
+
+  /// The set of nodes that are on the inner side of some outer join.
+  hypergraph::NodeMap nodes_inner_to_outer_join = 0;
+
+  /// The set of nodes that are on the inner side of some semijoin.
+  hypergraph::NodeMap nodes_inner_to_semijoin = 0;
+
+  /// The set of nodes that are on the inner side of some antijoin.
+  hypergraph::NodeMap nodes_inner_to_antijoin = 0;
+
+  /// The set of nodes that are represented by table_function. This will be set
+  /// only when optimizing for secondary engine.
+  hypergraph::NodeMap nodes_for_table_function = 0;
 
   int FindSargableJoinPredicate(const Item *predicate) const {
     const auto iter = m_sargable_join_predicates.find(predicate);
