@@ -1,7 +1,7 @@
 #ifndef LEX_INCLUDED
 #define LEX_INCLUDED
 
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -73,6 +73,7 @@ static const SYMBOL symbols[] = {
     {SYM("<<", SHIFT_LEFT)},
     {SYM(">>", SHIFT_RIGHT)},
     {SYM("<=>", EQUAL_SYM)},
+    {SYM("ABSENT", ABSENT_SYM)},
     {SYM("ACCESSIBLE", ACCESSIBLE_SYM)},
     {SYM("ACCOUNT", ACCOUNT_SYM)},
     {SYM("ACTION", ACTION)},
@@ -83,6 +84,7 @@ static const SYMBOL symbols[] = {
     {SYM("AGAINST", AGAINST)},
     {SYM("AGGREGATE", AGGREGATE_SYM)},
     {SYM("ALL", ALL)},
+    {SYM("ALLOW_MISSING_FILES", ALLOW_MISSING_FILES_SYM)},
     {SYM("ALGORITHM", ALGORITHM_SYM)},
     {SYM("ALTER", ALTER)},
     {SYM("ALWAYS", ALWAYS_SYM)},
@@ -99,6 +101,8 @@ static const SYMBOL symbols[] = {
     {SYM("AUTHENTICATION", AUTHENTICATION_SYM)},
     {SYM("AUTO", AUTO_SYM)},
     {SYM("AUTO_INCREMENT", AUTO_INC)},
+    {SYM("AUTO_REFRESH", AUTO_REFRESH_SYM)},
+    {SYM("AUTO_REFRESH_SOURCE", AUTO_REFRESH_SOURCE_SYM)},
     {SYM("AUTOEXTEND_SIZE", AUTOEXTEND_SIZE_SYM)},
     {SYM("AVG", AVG_SYM)},
     {SYM("AVG_ROW_LENGTH", AVG_ROW_LENGTH)},
@@ -221,6 +225,7 @@ static const SYMBOL symbols[] = {
     {SYM("DOUBLE", DOUBLE_SYM)},
     {SYM("DROP", DROP)},
     {SYM("DUAL", DUAL_SYM)},
+    {SYM("DUALITY", DUALITY_SYM)},
     {SYM("DUMPFILE", DUMPFILE)},
     {SYM("DUPLICATE", DUPLICATE_SYM)},
     {SYM("DYNAMIC", DYNAMIC_SYM)},
@@ -256,6 +261,8 @@ static const SYMBOL symbols[] = {
     {SYM("EXPLAIN", DESCRIBE)},
     {SYM("EXTENDED", EXTENDED_SYM)},
     {SYM("EXTENT_SIZE", EXTENT_SIZE_SYM)},
+    {SYM("EXTERNAL", EXTERNAL_SYM)},
+    {SYM("EXTERNAL_FORMAT", EXTERNAL_FORMAT_SYM)},
     {SYM("FACTOR", FACTOR_SYM)},
     {SYM("FAILED_LOGIN_ATTEMPTS", FAILED_LOGIN_ATTEMPTS_SYM)},
     {SYM("FALSE", FALSE_SYM)},
@@ -264,7 +271,12 @@ static const SYMBOL symbols[] = {
     {SYM("FETCH", FETCH_SYM)},
     {SYM("FIELDS", COLUMNS)},
     {SYM("FILE", FILE_SYM)},
+    {SYM("FILES", FILES_SYM)},
     {SYM("FILE_BLOCK_SIZE", FILE_BLOCK_SIZE_SYM)},
+    {SYM("FILE_FORMAT", FILE_FORMAT_SYM)},
+    {SYM("FILE_NAME", FILE_NAME_SYM)},
+    {SYM("FILE_PATTERN", FILE_PATTERN_SYM)},
+    {SYM("FILE_PREFIX", FILE_PREFIX_SYM)},
     {SYM("FILTER", FILTER_SYM)},
     {SYM("FINISH", FINISH_SYM)},
     {SYM("FIRST", FIRST_SYM)},
@@ -303,9 +315,11 @@ static const SYMBOL symbols[] = {
     {SYM("GROUPS", GROUPS_SYM)},
     {SYM("GTIDS", GTIDS_SYM)},
     {SYM("GTID_ONLY", GTID_ONLY_SYM)},
+    {SYM("GUIDED", GUIDED_SYM)},
     {SYM("HANDLER", HANDLER_SYM)},
     {SYM("HASH", HASH_SYM)},
     {SYM("HAVING", HAVING)},
+    {SYM("HEADER", HEADER_SYM)},
     {SYM("HELP", HELP_SYM)},
     {SYM("HIGH_PRIORITY", HIGH_PRIORITY)},
     {SYM("HISTOGRAM", HISTOGRAM_SYM)},
@@ -377,6 +391,7 @@ static const SYMBOL symbols[] = {
     {SYM("LEFT", LEFT)},
     {SYM("LESS", LESS_SYM)},
     {SYM("LEVEL", LEVEL_SYM)},
+    {SYM("LIBRARY", LIBRARY_SYM)},
     {SYM("LIKE", LIKE)},
     {SYM("LIMIT", LIMIT)},
     {SYM("LINEAR", LINEAR_SYM)},
@@ -401,6 +416,7 @@ static const SYMBOL symbols[] = {
     {SYM("MANUAL", MANUAL_SYM)},
     {SYM("MASTER", MASTER_SYM)},
     {SYM("MATCH", MATCH)},
+    {SYM("MATERIALIZED", MATERIALIZED_SYM)},
     {SYM("MAX_CONNECTIONS_PER_HOUR", MAX_CONNECTIONS_PER_HOUR)},
     {SYM("MAX_QUERIES_PER_HOUR", MAX_QUERIES_PER_HOUR)},
     {SYM("MAX_ROWS", MAX_ROWS)},
@@ -486,6 +502,7 @@ static const SYMBOL symbols[] = {
     {SYM("OWNER", OWNER_SYM)},
     {SYM("PACK_KEYS", PACK_KEYS_SYM)},
     {SYM("PATH", PATH_SYM)},
+    {SYM("PARAMETERS", PARAMETERS_SYM)},
     {SYM("PARSE_TREE", PARSE_TREE_SYM)},
     {SYM("PARSER", PARSER_SYM)},
     {SYM("PAGE", PAGE_SYM)},
@@ -543,6 +560,7 @@ static const SYMBOL symbols[] = {
     {SYM("REFERENCES", REFERENCES)},
     {SYM("REGEXP", REGEXP)},
     {SYM("REGISTRATION", REGISTRATION_SYM)},
+    {SYM("RELATIONAL", RELATIONAL_SYM)},
     {SYM("RELAY", RELAY)},
     {SYM("RELAYLOG", RELAYLOG_SYM)},
     {SYM("RELAY_LOG_FILE", RELAY_LOG_FILE_SYM)},
@@ -704,6 +722,7 @@ static const SYMBOL symbols[] = {
     {SYM("STORED", STORED_SYM)},
     {SYM("STRAIGHT_JOIN", STRAIGHT_JOIN)},
     {SYM("STREAM", STREAM_SYM)},
+    {SYM("STRICT_LOAD", STRICT_LOAD_SYM)},
     {SYM("STRING", STRING_SYM)},
     {SYM("SUBCLASS_ORIGIN", SUBCLASS_ORIGIN_SYM)},
     {SYM("SUBJECT", SUBJECT_SYM)},
@@ -761,6 +780,7 @@ static const SYMBOL symbols[] = {
     {SYM("UNSIGNED", UNSIGNED_SYM)},
     {SYM("UNTIL", UNTIL_SYM)},
     {SYM("UPGRADE", UPGRADE_SYM)},
+    {SYM("URI", URI_SYM)},
     {SYM("URL", URL_SYM)},
     {SYM("USAGE", USAGE)},
     {SYM("USE", USE_SYM)},
@@ -780,6 +800,7 @@ static const SYMBOL symbols[] = {
     {SYM("VARIABLES", VARIABLES)},
     {SYM("VARYING", VARYING)},
     {SYM("VECTOR", VECTOR_SYM)},
+    {SYM("VERIFY_KEY_CONSTRAINTS", VERIFY_KEY_CONSTRAINTS_SYM)},
     {SYM("WAIT", WAIT_SYM)},
     {SYM("WARNINGS", WARNINGS)},
     {SYM("WEEK", WEEK_SYM)},
@@ -831,6 +852,7 @@ static const SYMBOL symbols[] = {
     {SYM_FN("DATE_SUB", DATE_SUB_INTERVAL)},
     {SYM_FN("EXTRACT", EXTRACT_SYM)},
     {SYM_FN("GROUP_CONCAT", GROUP_CONCAT_SYM)},
+    {SYM_FN("JSON_DUALITY_OBJECT", JSON_DUALITY_OBJECT_SYM)},
     {SYM_FN("JSON_OBJECTAGG", JSON_OBJECTAGG)},
     {SYM_FN("JSON_ARRAYAGG", JSON_ARRAYAGG)},
     {SYM_FN("MAX", MAX_SYM)},
