@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -96,6 +97,7 @@
 #include "sql/auth/auth_common.h"  // SUPER_ACL
 #include "sql/dd/types/event.h"
 #include "sql/dd/types/function.h"
+#include "sql/dd/types/library.h"
 #include "sql/dd/types/procedure.h"
 #include "sql/dd/types/resource_group.h"
 #include "sql/debug_sync.h"
@@ -866,6 +868,9 @@ bool lock_object_name(THD *thd, MDL_key::enum_mdl_namespace mdl_type,
     case MDL_key::PROCEDURE:
       dd::Procedure::create_mdl_key(db, name, &mdl_key);
       break;
+    case MDL_key::LIBRARY:
+      dd::Library::create_mdl_key(db, name, &mdl_key);
+      break;
     case MDL_key::EVENT:
       dd::Event::create_mdl_key(db, name, &mdl_key);
       break;
