@@ -1,6 +1,6 @@
 #ifndef MDL_H
 #define MDL_H
-/* Copyright (c) 2009, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,17 +27,18 @@
 #include <string.h>
 #include <sys/types.h>
 #include <algorithm>
+#include <functional>
 #include <new>
 #include <unordered_map>
 
-#include "m_string.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
+#include "strmake.h"
 
 #include "my_inttypes.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
-#include "my_systime.h"  // Timout_type
+#include "my_systime.h"  // Timeout_type
 #include "mysql/components/services/bits/mysql_cond_bits.h"
 #include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/mysql_rwlock_bits.h"
@@ -383,6 +384,7 @@ struct MDL_key {
      - TABLE is for tables and views.
      - FUNCTION is for stored functions.
      - PROCEDURE is for stored procedures.
+     - LIBRARY is for libraries.
      - TRIGGER is for triggers.
      - EVENT is for event scheduler events.
      - COMMIT is for enabling the global read lock to block commits.
@@ -416,6 +418,7 @@ struct MDL_key {
     RESOURCE_GROUPS,
     FOREIGN_KEY,
     CHECK_CONSTRAINT,
+    LIBRARY,
     /* This should be the last ! */
     NAMESPACE_END
   };
@@ -536,10 +539,10 @@ struct MDL_key {
     end = strmake(start, name, NAME_LEN);
     m_object_name_length = static_cast<uint16>(end - start);
 
-    size_t col_len = strlen(column_name);
+    const size_t col_len = strlen(column_name);
     assert(col_len <= NAME_LEN);
     start = end + 1;
-    size_t remaining =
+    const size_t remaining =
         MAX_MDLKEY_LENGTH - m_db_name_length - m_object_name_length - 3;
     uint16 extra_length = 0;
 
@@ -711,9 +714,10 @@ struct MDL_key {
     m_object_name_length = m_length - m_db_name_length - 3;
   }
   void mdl_key_init(const MDL_key *rhs) {
-    uint16 copy_length = rhs->use_normalized_object_name()
-                             ? rhs->m_length + rhs->m_object_name_length + 1
-                             : rhs->m_length;
+    const uint16 copy_length =
+        rhs->use_normalized_object_name()
+            ? rhs->m_length + rhs->m_object_name_length + 1
+            : rhs->m_length;
     memcpy(m_ptr, rhs->m_ptr, copy_length);
     m_length = rhs->m_length;
     m_db_name_length = rhs->m_db_name_length;
@@ -776,8 +780,8 @@ struct MDL_key {
   */
   bool use_normalized_object_name() const {
     return (mdl_namespace() == FUNCTION || mdl_namespace() == PROCEDURE ||
-            mdl_namespace() == EVENT || mdl_namespace() == RESOURCE_GROUPS ||
-            mdl_namespace() == TRIGGER);
+            mdl_namespace() == LIBRARY || mdl_namespace() == EVENT ||
+            mdl_namespace() == RESOURCE_GROUPS || mdl_namespace() == TRIGGER);
   }
 
  private:
