@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -519,6 +519,16 @@ bool Persisted_variables_cache::set_variable(THD *thd, set_var *setvar) {
             plugin_variables.erase(tmp_var);
             plugin_variables.insert(tmp_var);
           }
+          if (system_var->is_sensitive()) {
+            auto it = std::find_if(m_persisted_dynamic_variables.begin(),
+                                   m_persisted_dynamic_variables.end(),
+                                   [name](st_persist_var const &s) {
+                                     return !strcmp(s.key.c_str(), name);
+                                   });
+            if (it != m_persisted_dynamic_variables.end()) {
+              m_persisted_dynamic_variables.erase(it);
+            }
+          }
         }
       }
 
@@ -815,17 +825,21 @@ bool Persisted_variables_cache::flush_to_file() {
     /* write to file */
     if (mysql_file_fputs(dest.c_ptr(), m_fd) < 0) {
       ret = true;
+    } else {
+      DBUG_EXECUTE_IF("crash_after_write_persist_file", DBUG_SUICIDE(););
+      /* flush contents to disk immediately */
+      if ((mysql_file_fflush(m_fd) != 0) ||
+          (my_sync(my_fileno(m_fd->m_file), MYF(MY_WME)) == -1))
+        ret = true;
     }
-    DBUG_EXECUTE_IF("crash_after_write_persist_file", DBUG_SUICIDE(););
-    /* flush contents to disk immediately */
-    if (mysql_file_fflush(m_fd) != 0) ret = true;
-    if (my_sync(my_fileno(m_fd->m_file), MYF(MY_WME)) == -1) ret = true;
+    close_persist_file();
   }
-  close_persist_file();
   if (!ret) {
     DBUG_EXECUTE_IF("crash_after_close_persist_file", DBUG_SUICIDE(););
-    my_rename(m_persist_backup_filename.c_str(), m_persist_filename.c_str(),
-              MYF(MY_WME));
+    if (my_rename(m_persist_backup_filename.c_str(), m_persist_filename.c_str(),
+                  MYF(MY_WME)))
+      ret = true;
+    DBUG_EXECUTE_IF("crash_after_rename_persist_file", DBUG_SUICIDE(););
   }
   if (ret == false && do_cleanup == true) clear_sensitive_blob_and_iv();
   mysql_mutex_unlock(&m_LOCK_persist_file);
@@ -946,7 +960,7 @@ bool Persisted_variables_cache::set_persisted_options(
   const std::vector<std::string> priv_list = {
       "ENCRYPTION_KEY_ADMIN", "ROLE_ADMIN",          "SYSTEM_VARIABLES_ADMIN",
       "AUDIT_ADMIN",          "TELEMETRY_LOG_ADMIN", "CONNECTION_ADMIN"};
-  const ulong static_priv_list = (SUPER_ACL | FILE_ACL);
+  const Access_bitmask static_priv_list = (SUPER_ACL | FILE_ACL);
   Sctx_ptr<Security_context> ctx;
   /*
     if persisted_globals_load is set to false or --no-defaults is set
@@ -2073,8 +2087,11 @@ bool Persisted_variables_cache::append_read_only_variables(
 
   auto result = decrypt_sensitive_variables();
   if (result == return_status::ERROR) {
-    LogErr(ERROR_LEVEL, ER_CANNOT_INTERPRET_PERSISTED_SENSITIVE_VARIABLES);
-    return true;
+    if (!opt_persist_sensitive_variables_in_plaintext) {
+      LogErr(ERROR_LEVEL, ER_CANNOT_INTERPRET_PERSISTED_SENSITIVE_VARIABLES);
+      return true;
+    }
+    LogErr(WARNING_LEVEL, ER_CANNOT_INTERPRET_PERSISTED_SENSITIVE_VARIABLES);
   }
 
   /* create a set of values sorted by timestamp */
