@@ -78,6 +78,7 @@ PSI_memory_key key_memory_applier;
 PSI_memory_key key_memory_recovery;
 PSI_memory_key key_memory_show_binlog_events;
 PSI_memory_key key_memory_relaylog_recovery;
+PSI_memory_key key_memory_applier_metric_service;
 PSI_memory_key key_memory_REPLICA_INFO;
 PSI_memory_key key_memory_ST_SCHEMA_TABLE;
 PSI_memory_key key_memory_Slave_applier_json_diff_vector;
@@ -399,7 +400,9 @@ static PSI_memory_info all_server_memory[] = {
     {&key_memory_show_binlog_events, "show_binlog_events", 0, 0,
      PSI_DOCUMENT_ME},
     {&key_memory_relaylog_recovery, "Relay_log_sanitizer::analyze_logs", 0, 0,
-     PSI_DOCUMENT_ME}};
+     PSI_DOCUMENT_ME},
+    {&key_memory_applier_metric_service, "Applier_metrics_service_handler", 0,
+     0, PSI_DOCUMENT_ME}};
 
 void register_server_memory_keys() {
   const char *category = "sql";
