@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -37,6 +37,7 @@
 
 #include "lex_string.h"
 #include "my_dbug.h"
+#include "my_sys.h"
 #include "my_thread_local.h"
 #include "mysql/psi/mysql_file.h"
 #include "mysql/strings/m_ctype.h"
@@ -54,6 +55,7 @@
 #include "sql/sql_const.h"
 #include "sql/sql_error.h"
 #include "sql/sql_exchange.h"
+#include "sql/sql_outfile_defaults.h"
 #include "sql/system_variables.h"
 #include "sql/visible_fields.h"
 #include "sql_string.h"
@@ -120,27 +122,318 @@ bool Query_result_send::send_eof(THD *thd) {
   return false;
 }
 
-static const String default_line_term("\n", default_charset_info);
-static const String default_escaped("\\", default_charset_info);
-static const String default_field_term("\t", default_charset_info);
-static const String default_xml_row_term("<row>", default_charset_info);
-static const String my_empty_string("", default_charset_info);
-
-sql_exchange::sql_exchange(const char *name, bool flag,
-                           enum enum_filetype filetype_arg)
-    : file_name(name), dumpfile(flag), skip_lines(0) {
-  field.opt_enclosed = false;
-  filetype = filetype_arg;
-  field.field_term = &default_field_term;
-  field.enclosed = line.line_start = &my_empty_string;
-  line.line_term =
-      filetype == FILETYPE_CSV ? &default_line_term : &default_xml_row_term;
-  field.escaped = &default_escaped;
-  cs = nullptr;
+bool File_information::do_contextualize() {
+  if (filetype_str != nullptr) {
+    if (my_strcasecmp(system_charset_info, filetype_str, "csv") == 0) {
+      filetype = FILETYPE_CSV;
+    } else if (my_strcasecmp(system_charset_info, filetype_str, "text") == 0) {
+      filetype = FILETYPE_TEXT;
+    } else if (my_strcasecmp(system_charset_info, filetype_str, "json") == 0) {
+      filetype = FILETYPE_JSON;
+    } else if (my_strcasecmp(system_charset_info, filetype_str, "parquet") ==
+               0) {
+      filetype = FILETYPE_PARQUET;
+    } else if (my_strcasecmp(system_charset_info, filetype_str, "xml") == 0) {
+      filetype = FILETYPE_XML;
+    } else {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), filetype_str,
+               "exporting query result to object storage");
+      return true;
+    }
+  }
+  return false;
 }
 
+void File_information::assign_default_values() {
+  if (filetype == FILETYPE_PARQUET && compression == nullptr) {
+    compression = &default_compression;
+  }
+}
+
+void Line_separators::assign_default_values(enum_destination dumpfile,
+                                            enum_filetype filetype_arg) {
+  if (dumpfile != OBJECT_STORE_DEST && line_start == nullptr) {
+    line_start = &my_empty_string;
+  }
+  if (line_term == nullptr) {
+    if (filetype_arg == FILETYPE_TEXT || filetype_arg == FILETYPE_CSV) {
+      line_term = &default_line_term_text_csv;
+    } else if (filetype_arg == FILETYPE_XML) {
+      line_term = &default_xml_row_term;
+    }
+  }
+}
+
+void Field_separators::assign_default_values(enum_filetype filetype_arg) {
+  if (escaped == nullptr &&
+      (filetype_arg == FILETYPE_TEXT || filetype_arg == FILETYPE_CSV ||
+       filetype_arg == FILETYPE_XML)) {
+    escaped = &default_escaped;
+  }
+  if (enclosed == nullptr) {
+    if (filetype_arg == FILETYPE_TEXT || filetype_arg == FILETYPE_CSV ||
+        filetype_arg == FILETYPE_XML) {
+      enclosed = &my_empty_string;
+    } else if (filetype_arg == FILETYPE_CSV) {
+      enclosed = &default_enclosed_csv;
+    }
+  }
+  if (field_term == nullptr) {
+    if (filetype_arg == FILETYPE_TEXT || filetype_arg == FILETYPE_XML) {
+      field_term = &default_field_term_text;
+    } else if (filetype_arg == FILETYPE_CSV) {
+      field_term = &default_field_term_csv;
+    }
+  }
+}
+
+sql_exchange::sql_exchange(const char *name,
+                           enum enum_destination dumpfile_flag)
+    : field(),
+      line(),
+      uri_info(),
+      file_info(dumpfile_flag),
+      file_name(name),
+      dumpfile(dumpfile_flag) {}
+
+sql_exchange::sql_exchange(const char *name,
+                           enum enum_destination dumpfile_flag,
+                           enum_filetype filetype)
+    : field(),
+      line(),
+      uri_info(),
+      file_info(filetype),
+      file_name(name),
+      dumpfile(dumpfile_flag) {}
+
+sql_exchange::sql_exchange(enum enum_destination dumpfile_flag)
+    : field(),
+      line(),
+      uri_info(),
+      file_info(dumpfile_flag),
+      dumpfile(dumpfile_flag) {}
+
 bool sql_exchange::escaped_given(void) {
-  return field.escaped != &default_escaped;
+  return (field.escaped != &default_escaped);
+}
+
+void sql_exchange::assign_default_values() {
+  file_info.assign_default_values();
+  field.assign_default_values(file_info.filetype);
+  line.assign_default_values(dumpfile, file_info.filetype);
+}
+
+bool sql_exchange::do_contextualize(Parse_context *pc) {
+  if (file_info.do_contextualize()) {
+    return true;
+  }
+
+  if (file_info.filetype == FILETYPE_PARQUET ||
+      file_info.filetype == FILETYPE_JSON) {
+    std::string file_format_msg(file_info.filetype_str);
+    file_format_msg += " format";
+    if (field.field_term != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0),
+               "COLUMNS or FIELDS TERMINATED BY", file_format_msg.c_str());
+      return true;
+    }
+    if (field.enclosed != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0),
+               "COLUMNS or FIELDS ENCLOSED BY", file_format_msg.c_str());
+      return true;
+    }
+    if (field.escaped != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0),
+               "COLUMNS or FIELDS ESCAPED BY", file_format_msg.c_str());
+      return true;
+    }
+    if (field.date_format != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "DATE FORMAT",
+               file_format_msg.c_str());
+      return true;
+    }
+    if (field.time_format != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "TIME FORMAT",
+               file_format_msg.c_str());
+      return true;
+    }
+    if (field.trim_spaces != enum_trim_spaces::DEFAULT_TRIM_SPACES) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "TRIM SPACES",
+               file_format_msg.c_str());
+      return true;
+    }
+    if (file_info.with_header != enum_with_header::DEFAULT_HEADER) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "HEADER",
+               file_format_msg.c_str());
+      return true;
+    }
+    if (field.null_value != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "NULL AS",
+               file_format_msg.c_str());
+      return true;
+    }
+    if (field.empty_value != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "EMPTY VALUE",
+               file_format_msg.c_str());
+      return true;
+    }
+    if (field.not_enclosed) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0),
+               "COLUMNS or FIELDS NOT ENCLOSED", file_format_msg.c_str());
+      return true;
+    }
+  }
+  if (file_info.filetype == FILETYPE_PARQUET) {
+    if (line.line_term != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "LINES TERMINATED BY",
+               "PARQUET format");
+      return true;
+    }
+    if (file_info.cs != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "CHARACTER SET",
+               "PARQUET format");
+      return true;
+    }
+  }
+  auto *thd = pc->thd;
+  if (dumpfile == OBJECT_STORE_DEST) {
+    if (file_info.filetype == FILETYPE_CSV ||
+        file_info.filetype == FILETYPE_TEXT ||
+        file_info.filetype == FILETYPE_JSON) {
+      if (file_info.compression != nullptr) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "COMPRESSION",
+                            "OUTFILE");
+      }
+      if (file_info.cs != nullptr) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "CHARACTER SET",
+                            "OUTFILE");
+      }
+    }
+    if (file_info.filetype == FILETYPE_CSV ||
+        file_info.filetype == FILETYPE_TEXT) {
+      if (field.empty_value != nullptr) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "EMPTY VALUE",
+                            "OUTFILE");
+      }
+      if (field.not_enclosed) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "NOT ENCLOSED BY",
+                            "OUTFILE");
+      }
+      if (field.enclosed != nullptr) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED),
+                            "ENCLOSED BY or OPTIONALLY ENCLOSED BY", "OUTFILE");
+      }
+      if (field.escaped != nullptr) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "ESCAPED BY",
+                            "OUTFILE");
+      }
+      if (field.trim_spaces != enum_trim_spaces::DEFAULT_TRIM_SPACES) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "TRIM SPACES",
+                            "OUTFILE");
+      }
+      if (field.date_format != nullptr) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "DATE FORMAT",
+                            "OUTFILE");
+      }
+
+      if (field.time_format != nullptr) {
+        push_warning_printf(thd, Sql_condition::SL_WARNING, ER_CLAUSE_IGNORED,
+                            ER_THD(thd, ER_CLAUSE_IGNORED), "TIME FORMAT",
+                            "OUTFILE");
+      }
+    }
+    if (line.line_start != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "LINES STARTING BY",
+               "exporting query result to object storage");
+      return true;
+    }
+
+    if (file_info.filetype == FILETYPE_XML) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "XML",
+               "exporting query result to object storage");
+      return true;
+    }
+  } else {
+    if (field.not_enclosed) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "NOT ENCLOSED",
+               "exporting query result to disk");
+      return true;
+    }
+
+    if (field.date_format != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "DATE FORMAT",
+               "exporting query result to disk");
+      return true;
+    }
+
+    if (field.time_format != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "TIME FORMAT",
+               "exporting query result to disk");
+      return true;
+    }
+    if (field.trim_spaces != enum_trim_spaces::DEFAULT_TRIM_SPACES) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "TRIM SPACES",
+               "exporting query result to disk");
+      return true;
+    }
+    if (file_info.with_header != enum_with_header::DEFAULT_HEADER) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "HEADER",
+               "exporting query result to disk");
+      return true;
+    }
+    if (field.null_value != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "NULL AS",
+               "exporting query result to disk");
+      return true;
+    }
+
+    if (field.empty_value != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "EMPTY VALUE",
+               "exporting query result to disk");
+      return true;
+    }
+    if (file_info.compression != nullptr) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "COMPRESSION",
+               "exporting query result to disk");
+      return true;
+    }
+    if (file_info.filetype == FILETYPE_CSV) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "CSV FORMAT",
+               "exporting query result to disk");
+      return true;
+    }
+    if (file_info.filetype == FILETYPE_PARQUET) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "PARQUET",
+               "exporting query result to disk");
+      return true;
+    }
+    if (file_info.filetype == FILETYPE_JSON) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "JSON format",
+               "exporting query result to disk");
+      return true;
+    }
+    if (file_info.filetype == FILETYPE_XML) {
+      my_error(ER_INVALID_CLAUSE_IN_CONTEXT, MYF(0), "XML format",
+               "exporting query result to disk");
+      return true;
+    }
+  }
+
+  if (field.not_enclosed && field.enclosed != nullptr) {
+    my_error(ER_INVALID_CLAUSE_COMBINATION, MYF(0), "NOT ENCLOSED",
+             "OPTIONALLY ENCLOSED BY");
+    return true;
+  }
+
+  return false;
 }
 
 /************************************************************************
@@ -178,6 +471,22 @@ void Query_result_to_file::cleanup() {
   row_count = 0;
 }
 
+/**
+ * \brief Returns true incase of an error.
+ */
+bool Query_result_to_object_store::send_eof(THD *thd) {
+  if (thd->is_error()) return true;
+  ::my_ok(thd, thd->current_found_rows);
+  return false;
+}
+
+/**
+ * \brief Reset the number of affected rows
+ */
+void Query_result_to_object_store::cleanup() {
+  current_thd->set_sent_row_count(0);
+}
+
 /***************************************************************************
 ** Export of select to textfile
 ***************************************************************************/
@@ -260,7 +569,8 @@ bool Query_result_export::prepare(THD *thd, const mem_root_deque<Item *> &list,
   if (strlen(exchange->file_name) + NAME_LEN >= FN_REFLEN)
     strmake(path, exchange->file_name, FN_REFLEN - 1);
 
-  write_cs = exchange->cs ? exchange->cs : &my_charset_bin;
+  write_cs = (exchange->file_info.cs != nullptr) ? exchange->file_info.cs
+                                                 : &my_charset_bin;
 
   /* Check if there is any blobs in data */
   for (Item *item : VisibleFields(list)) {
@@ -724,7 +1034,8 @@ bool Query_dumpvar::send_data(THD *thd, const mem_root_deque<Item *> &items) {
   while ((mv = var_li++) && it != VisibleFields(items).end()) {
     Item *item = *it++;
     if (mv->is_local()) {
-      if (thd->sp_runtime_ctx->set_variable(thd, mv->get_offset(), &item))
+      if (thd->sp_runtime_ctx->set_variable(thd, false, mv->get_offset(),
+                                            &item))
         return true;
     } else {
       Item_func_set_user_var *suv = new Item_func_set_user_var(mv->name, item);
