@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -44,6 +44,7 @@
 #include "sql/item_sum.h"
 #include "sql/join_optimizer/access_path.h"
 #include "sql/join_optimizer/bit_utils.h"
+#include "sql/join_optimizer/cost_model.h"
 #include "sql/key.h"
 #include "sql/key_spec.h"
 #include "sql/opt_costmodel.h"
@@ -166,7 +167,14 @@ AccessPath *make_group_skip_scan_path(
     A) Table T has at least one compound index I of the form:
        I = <A_1, ...,A_k, [B_1,..., B_m], C, [D_1,...,D_n]>
     B) Query conditions:
-    B0. Q is over a single table T.
+    B0. Q is over a single table T. For a single table query which is internally
+        transformed into a multi-table query E.g. due to semijoin
+        transformations, group skip scan can still be used for the original
+        table specified in the query for duplicate removal. E.g. SELECT DISTINCT
+        f1 FROM t1 IN (SELECT f1 FROM t2); In this case, with a semi-join
+        transformation, query would look like SELECT DISTINCT f1 FROM t1
+        semi-join t2 ON t1.f1=t2.f1; Group skip scan can still be used for table
+        "t1" for duplicate removal even though the query has a JOIN now.
     B1. The attributes referenced by Q are a subset of the attributes of I.
     B2. All attributes QA in Q can be divided into 3 overlapping groups:
         - SA = {S_1, ..., S_l, [C]} - from the SELECT clause, where C is
@@ -369,9 +377,15 @@ void collect_group_skip_scans(
   /* Perform few 'cheap' tests whether this access method is applicable. */
   if (!join)
     cause = "no_join";
-  else if (param->query_block->leaf_table_count !=
-           1) /* Query must reference one table. */
+  else if (param->query_block->original_tables_map != 1)
+    /* Check (B0) Query must reference only one table. */
     cause = "not_single_table";
+  else if (table->pos_in_table_list->map() != 1)
+    /*
+      Check (B0) - Group skip scan is allowed only on the table that was used
+      originally in the query i.e before the query transformations.
+    */
+    cause = "not_original_query_table";
   else if (join->query_block->olap == ROLLUP_TYPE) /* Check (B3) for ROLLUP */
     cause = "rollup";
   else if (table->s->keys == 0) /* There are no indexes to use. */
@@ -393,6 +407,22 @@ void collect_group_skip_scans(
         .add_alnum("cause", "not_group_by_or_distinct");
     return;
   }
+
+  /*
+    Additional checking for (B0) - Group skip scan is allowed only for single
+    table queries. The exception being, queries that were transformed from a
+    single table query to multi-table queries because of semijoin
+    transformations. For such a case, do not allow group skip scan if
+    aggregation functions are present. Only duplicate removal could happen
+    before joins not aggregation.
+  */
+  if (join->sum_funcs[0] != nullptr &&
+      param->query_block->leaf_table_count != 1) {
+    trace_group.add("chosen", false)
+        .add_alnum("cause", "Multi_table_with_aggregate");
+    return;
+  }
+
   /* Analyze the query in more detail. */
 
   if (join->sum_funcs[0]) {
@@ -932,6 +962,12 @@ Mem_root_array<AccessPath *> get_all_group_skip_scans(
       // Adjust num_output_rows for hypergraph to match aggregate path rowcounts
       cur_path->set_num_output_rows(rows > 1 ? rows - 1 : rows);
       cur_path->num_output_rows_before_filter = cur_path->num_output_rows();
+
+      // Calculate cost for hypergraph
+      const uint cur_index = param->real_keynr[gskip_scan->param_idx];
+      cur_path->set_cost(
+          EstimateGroupSkipScanCost(param->table, cur_index, rows, have_max));
+
       group_skip_scan_paths.push_back(cur_path);
     }
   }
@@ -1675,7 +1711,7 @@ static inline uint get_field_keypart(KEY *index, const Field *field) {
      - When both min and max are present, LIS will make two reads per group
        instead of one. Similarly when min and max functions are not present,
        rows retrieved are different. Cost model should reflect what happens
-       in GroupIndexSkipScanIterator::Read()
+       in GroupIndexSkipScanIterator::DoRead()
 
   RETURN
     None
