@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -60,18 +60,18 @@ IndexRangeScanIterator::IndexRangeScanIterator(
     MEM_ROOT *return_mem_root, uint mrr_flags, uint mrr_buf_size,
     Bounds_checked_array<QUICK_RANGE *> ranges_arg)
     : RowIDCapableRowIterator(thd, table_arg),
-      ranges(ranges_arg),
-      free_file(false),
-      cur_range(nullptr),
-      last_range(nullptr),
-      mrr_flags(mrr_flags),
-      mrr_buf_size(mrr_buf_size),
-      mrr_buf_desc(nullptr),
-      need_rows_in_rowid_order(need_rows_in_rowid_order),
-      reuse_handler(reuse_handler),
-      mem_root(return_mem_root),
-      m_expected_rows(expected_rows),
-      m_examined_rows(examined_rows) {
+      ranges{ranges_arg},
+      free_file{false},
+      cur_range{nullptr},
+      last_range{nullptr},
+      mrr_flags{mrr_flags},
+      mrr_buf_size{mrr_buf_size},
+      mrr_buf_desc{nullptr},
+      need_rows_in_rowid_order{need_rows_in_rowid_order},
+      reuse_handler{reuse_handler},
+      mem_root{return_mem_root},
+      m_expected_rows{expected_rows},
+      m_examined_rows{examined_rows} {
   DBUG_TRACE;
 
   in_ror_merged_scan = false;
@@ -170,10 +170,9 @@ uint quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range) {
   start_key->key = cur->min_key;
   start_key->length = cur->min_length;
   start_key->keypart_map = cur->min_keypart_map;
-  start_key->flag =
-      ((cur->flag & NEAR_MIN)
-           ? HA_READ_AFTER_KEY
-           : (cur->flag & EQ_RANGE) ? HA_READ_KEY_EXACT : HA_READ_KEY_OR_NEXT);
+  start_key->flag = ((cur->flag & NEAR_MIN)   ? HA_READ_AFTER_KEY
+                     : (cur->flag & EQ_RANGE) ? HA_READ_KEY_EXACT
+                                              : HA_READ_KEY_OR_NEXT);
   end_key->key = cur->max_key;
   end_key->length = cur->max_length;
   end_key->keypart_map = cur->max_keypart_map;
@@ -202,7 +201,7 @@ static bool has_blob_primary_key(const TABLE *table) {
                      });
 }
 
-bool IndexRangeScanIterator::Init() {
+bool IndexRangeScanIterator::DoInit() {
   empty_record(table());
 
   /*
@@ -234,11 +233,19 @@ bool IndexRangeScanIterator::Init() {
     // outside of the record. So don't request a buffer in this case, even
     // though the current read_set gives the impression that using a
     // record buffer would be fine.
-    const bool skip_record_buffer =
-        need_rows_in_rowid_order &&
-        Overlaps(table()->file->ha_table_flags(),
-                 HA_PRIMARY_KEY_REQUIRED_FOR_POSITION) &&
-        has_blob_primary_key(table());
+    bool skip_record_buffer = need_rows_in_rowid_order &&
+                              Overlaps(table()->file->ha_table_flags(),
+                                       HA_PRIMARY_KEY_REQUIRED_FOR_POSITION) &&
+                              has_blob_primary_key(table());
+    // Skip the record buffer for covering multi-valued index range scans.
+    // The current implementation of Field_typed_array::key_cmp() needs the
+    // value of the generated column for the indexed expression, and this
+    // column is not available in the multi-valued index, so the storage
+    // engine cannot safely evaluate the end range condition when filling the
+    // record buffer when it's a covering scan.
+    skip_record_buffer |=
+        Overlaps(table()->key_info[index].flags, HA_MULTI_VALUED_KEY) &&
+        table()->key_read;
     if (!skip_record_buffer) {
       if (set_record_buffer(table(), m_expected_rows)) {
         return true; /* purecov: inspected */
@@ -350,7 +357,7 @@ bool IndexRangeScanIterator::shared_reset() {
   return false;
 }
 
-int IndexRangeScanIterator::Read() {
+int IndexRangeScanIterator::DoRead() {
   MY_BITMAP *const save_read_set = table()->read_set;
   MY_BITMAP *const save_write_set = table()->write_set;
   DBUG_TRACE;
@@ -431,7 +438,8 @@ int IndexRangeScanIterator::cmp_next(QUICK_RANGE *range_arg) {
   int cmp;
   if (range_arg->flag & NO_MAX_RANGE) return 0; /* key can't be to large */
 
-  cmp = key_cmp(key_part_info, range_arg->max_key, range_arg->max_length);
+  cmp = key_cmp(key_part_info, range_arg->max_key, range_arg->max_length,
+                /*is_reverse_multi_valued_index_scan=*/false);
   if (cmp < 0 || (cmp == 0 && !(range_arg->flag & NEAR_MAX))) return 0;
   return 1;  // outside of range
 }
@@ -444,7 +452,8 @@ int IndexRangeScanIterator::cmp_prev(QUICK_RANGE *range_arg) {
   int cmp;
   if (range_arg->flag & NO_MIN_RANGE) return 0; /* key can't be to small */
 
-  cmp = key_cmp(key_part_info, range_arg->min_key, range_arg->min_length);
+  cmp = key_cmp(key_part_info, range_arg->min_key, range_arg->min_length,
+                /*is_reverse_multi_valued_index_scan=*/false);
   if (cmp > 0 || (cmp == 0 && !(range_arg->flag & NEAR_MIN))) return 0;
   return 1;  // outside of range
 }
