@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,18 +28,20 @@
 #include <math.h>
 #include <string.h>
 #include <algorithm>
+#include <memory>
 
-#include "m_ctype.h"
 #include "my_bitmap.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_table_map.h"
+#include "mysql/strings/m_ctype.h"
 #include "sql/current_thd.h"
 #include "sql/field.h"
 #include "sql/handler.h"
 #include "sql/item.h"
 #include "sql/item_sum.h"
 #include "sql/join_optimizer/access_path.h"
+#include "sql/join_optimizer/cost_model.h"
 #include "sql/key.h"
 #include "sql/key_spec.h"
 #include "sql/opt_costmodel.h"
@@ -109,10 +111,49 @@ void trace_basic_info_index_skip_scan(THD *thd, const AccessPath *path,
   }
 }
 
-static void cost_skip_scan(TABLE *table, uint key, uint distinct_key_parts,
-                           ha_rows quick_prefix_records,
-                           Cost_estimate *cost_est, ha_rows *records,
-                           Item *where_cond, Opt_trace_object *trace_idx);
+static bool find_skip_scans(
+    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
+    enum_order order_direction, bool skip_records_in_range,
+    bool find_all_skip_scans,
+    Mem_root_array<IndexSkipScanParameters *> *skip_scan_info_list);
+
+static AccessPath *make_skip_scan_path(RANGE_OPT_PARAM *param,
+                                       bool force_skip_scan,
+                                       IndexSkipScanParameters *skip_scan_info);
+
+/**
+  Test if skip scan is applicable and if so, construct a new AccessPath for each
+  candidate index skip scan.
+
+  @param  thd       Thread info
+  @param  param     Parameter from test_quick_select
+  @param  tree      Range tree generated by get_mm_tree
+  @param  order_direction The sort order the range access method must be able
+                    to provide. Three-value logic: asc/desc/don't care
+  @param  skip_records_in_range Same value as JOIN_TAB::skip_records_in_range()
+  @param  force_skip_scan  TRUE if skip scan is forced by optimizer hint
+
+  @retval Mem_root_array of candidate INDEX_SKIP_SCAN AccessPaths.
+*/
+Mem_root_array<AccessPath *> get_all_skip_scans(THD *thd,
+                                                RANGE_OPT_PARAM *param,
+                                                SEL_TREE *tree,
+                                                enum_order order_direction,
+                                                bool skip_records_in_range,
+                                                bool force_skip_scan) {
+  Mem_root_array<AccessPath *> skip_scan_paths(param->temp_mem_root);
+  Mem_root_array<IndexSkipScanParameters *> skip_scan_info_list(
+      param->temp_mem_root);
+  AccessPath *cur_path = nullptr;
+  if (!find_skip_scans(thd, param, tree, order_direction, skip_records_in_range,
+                       /*get_all_skip_scans*/ true, &skip_scan_info_list)) {
+    for (IndexSkipScanParameters *iskip_scan : skip_scan_info_list) {
+      cur_path = make_skip_scan_path(param, force_skip_scan, iskip_scan);
+      skip_scan_paths.push_back(cur_path);
+    }
+  }
+  return skip_scan_paths;
+}
 
 /**
   Test if skip scan is applicable and if so, construct a new AccessPath.
@@ -152,6 +193,7 @@ static void cost_skip_scan(TABLE *table, uint key, uint distinct_key_parts,
     AccessPath object, that is later used to construct a new
     IndexSkipScanIterator.
 
+  @param  thd       Thread info
   @param  param     Parameter from test_quick_select
   @param  tree      Range tree generated by get_mm_tree
   @param  order_direction The sort order the range access method must be able
@@ -160,13 +202,124 @@ static void cost_skip_scan(TABLE *table, uint key, uint distinct_key_parts,
   @param  force_skip_scan  TRUE if skip scan is forced by optimizer hint
 
   @retval NULL, if skip index scan not applicable,
-          otherwise skip index scan table read plan.
+          otherwise index skip scan access path.
 */
-
 AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
                                enum_order order_direction,
                                bool skip_records_in_range,
                                bool force_skip_scan) {
+  Mem_root_array<IndexSkipScanParameters *> skip_scan_info_list(
+      param->temp_mem_root);
+  AccessPath *skip_scan_path = nullptr;
+  if (!find_skip_scans(thd, param, tree, order_direction, skip_records_in_range,
+                       /*get_all_skip_scans*/ false, &skip_scan_info_list)) {
+    assert(skip_scan_info_list.size() <= 1);
+    for (const IndexSkipScanParameters *iskip_scan : skip_scan_info_list) {
+      IndexSkipScanParameters *skip_scan_info =
+          const_cast<IndexSkipScanParameters *>(iskip_scan);
+      skip_scan_path =
+          make_skip_scan_path(param, force_skip_scan, skip_scan_info);
+    }
+  }
+  return skip_scan_path;
+}
+
+/**
+  Set up ranges for index skip scan
+  - Set flags in skip scan info for null conditions and equality ranges based
+    on the range condition
+  - Alloc storage and set values of min/max range keys
+
+  @param  range_sel_arg  Range condition info
+  @param  param          Parameter from test_quick_select
+  @param  skip_scan_info Info about a single index skip scan
+
+  @retval true on error, otherwise false
+*/
+static bool setup_range_for_skip_scan(SEL_ARG *range_sel_arg,
+                                      RANGE_OPT_PARAM *param,
+                                      IndexSkipScanParameters *skip_scan_info) {
+  skip_scan_info->range_cond_flag = 0;
+  // Setup fields that hold the range condition on key part C.
+  // This is only the suffix of the whole key, that we use
+  // to append to the prefix to get the full key later on.
+  if (!(range_sel_arg->min_flag & NO_MIN_RANGE) &&
+      !(range_sel_arg->max_flag & NO_MAX_RANGE)) {
+    // IS NULL condition
+    if (range_sel_arg->maybe_null() && range_sel_arg->min_value[0] &&
+        range_sel_arg->max_value[0])
+      skip_scan_info->range_cond_flag |= NULL_RANGE;
+    // equality condition
+    else if (memcmp(range_sel_arg->min_value, range_sel_arg->max_value,
+                    skip_scan_info->range_key_part->store_length) == 0)
+      skip_scan_info->range_cond_flag |= EQ_RANGE;
+  }
+
+  uchar *min_value, *max_value;
+  if (range_sel_arg->is_ascending) {
+    skip_scan_info->range_cond_flag |=
+        range_sel_arg->min_flag | range_sel_arg->max_flag;
+    min_value = range_sel_arg->min_value;
+    max_value = range_sel_arg->max_value;
+  } else {
+    skip_scan_info->range_cond_flag |=
+        invert_min_flag(range_sel_arg->min_flag) |
+        invert_max_flag(range_sel_arg->max_flag);
+    min_value = range_sel_arg->max_value;
+    max_value = range_sel_arg->min_value;
+  }
+
+  unsigned range_key_len = skip_scan_info->range_key_part->store_length;
+
+  // Allocate storage for min/max key if they exist.
+  unsigned max_used_key_length = 0;
+  for (uint i = 0; i < skip_scan_info->used_key_parts; ++i) {
+    max_used_key_length += skip_scan_info->index_info->key_part[i].store_length;
+  }
+
+  MEM_ROOT *const mem_root = param->return_mem_root;
+  if (!(skip_scan_info->range_cond_flag & NO_MIN_RANGE)) {
+    skip_scan_info->min_range_key = mem_root->ArrayAlloc<uchar>(range_key_len);
+    if (!skip_scan_info->min_range_key) return true;
+    skip_scan_info->min_search_key =
+        mem_root->ArrayAlloc<uchar>(max_used_key_length);
+    if (!skip_scan_info->min_search_key) return true;
+
+    memcpy(skip_scan_info->min_range_key, min_value, range_key_len);
+  }
+  if (!(skip_scan_info->range_cond_flag & NO_MAX_RANGE)) {
+    skip_scan_info->max_range_key = mem_root->ArrayAlloc<uchar>(range_key_len);
+    if (!skip_scan_info->max_range_key) return true;
+    skip_scan_info->max_search_key =
+        mem_root->ArrayAlloc<uchar>(max_used_key_length);
+    if (!skip_scan_info->max_search_key) return true;
+
+    memcpy(skip_scan_info->max_range_key, max_value, range_key_len);
+  }
+  return false;
+}
+
+/**
+  Collect all possible index skip scans and store them in the
+  skip_scan_info_list.
+
+  @param  thd       Thread info
+  @param  param     Parameter from test_quick_select
+  @param  tree      Range tree generated by get_mm_tree
+  @param  order_direction The sort order the range access method must be able
+                    to provide. Three-value logic: asc/desc/don't care
+  @param  skip_records_in_range Same value as JOIN_TAB::skip_records_in_range()
+  @param  find_all_skip_scans TRUE if every possible skip scan should be added
+  to skip scan list, FALSE if the best skip scan should be added
+  @param[out]  skip_scan_info_list List of index skip scans
+
+  @retval true on error, otherwise false.
+*/
+bool find_skip_scans(
+    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
+    enum_order order_direction, bool skip_records_in_range,
+    bool find_all_skip_scans,
+    Mem_root_array<IndexSkipScanParameters *> *skip_scan_info_list) {
   JOIN *join = param->query_block->join;
   TABLE *table = param->table;
   const char *cause = nullptr;
@@ -181,7 +334,8 @@ AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
 
   if (!join)
     cause = "no_join";
-  else if (join->primary_tables != 1) /* Query must reference one table. */
+  else if (param->query_block->leaf_table_count !=
+           1) /* Query must reference one table. */
     cause = "not_single_table";
   else if (table->s->keys == 0) /* There are no indexes to use. */
     cause = "no_index";
@@ -196,17 +350,17 @@ AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
 
   if (cause != nullptr) {
     trace_group.add("chosen", false).add_alnum("cause", cause);
-    return nullptr;
+    return true;
   }
 
-  KEY *index_info = nullptr; /* The index chosen for data access. */
-  uint index = 0;
-  KEY_PART_INFO *range_key_part = nullptr;
-  SEL_ROOT *index_range_tree = nullptr;
-  SEL_ARG *range_sel_arg = nullptr;
-  uint eq_prefix_len = 0;
-  uint eq_prefix_key_parts = 0;
-  uint used_key_parts = 0;
+  KEY *best_index_info = nullptr; /* The index chosen for data access. */
+  uint best_index = 0;
+  KEY_PART_INFO *best_range_key_part = nullptr;
+  SEL_ROOT *best_index_range_tree = nullptr;
+  SEL_ARG *best_range_sel_arg = nullptr;
+  uint best_eq_prefix_len = 0;
+  uint best_eq_prefix_key_parts = 0;
+  uint best_used_key_parts = 0;
   Cost_estimate min_diff_cost;
   Opt_trace_array trace_indices(trace, "potential_skip_scan_indexes");
 
@@ -223,7 +377,7 @@ AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
           min_max_item->sum_func() == Item_sum::AVG_DISTINCT_FUNC) {
         trace_group.add("chosen", false)
             .add_alnum("cause", "has_aggregate_distinct");
-        return nullptr;
+        return true;
       }
     }
     has_aggregate_function = true;
@@ -238,7 +392,7 @@ AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
     KEY_PART_INFO *cur_part;
     KEY_PART_INFO *end_part;
     Cost_estimate cur_read_cost;
-    ha_rows cur_records;
+    ha_rows cur_records{0};
     SEL_ARG *cur_range_sel_arg = nullptr;
     SEL_ROOT *cur_index_range_tree = nullptr;
     uint cur_eq_prefix_len = 0;
@@ -357,27 +511,60 @@ AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
           order_direction, skip_records_in_range, &mrr_flags, &mrr_bufsize,
           &dummy_cost, &is_ror_scan, &is_imerge_scan);
     }
-    cost_skip_scan(table, cur_index, cur_used_key_parts - 1,
-                   quick_prefix_records, &cur_read_cost, &cur_records,
-                   join->where_cond, &trace_idx);
+    if (!find_all_skip_scans) {
+      IndexSkipScanCost skip_scan_cost(table, cur_index, cur_used_key_parts - 1,
+                                       quick_prefix_records, join->where_cond,
+                                       &trace_idx);
+      cur_read_cost = skip_scan_cost.GetCost();
+      cur_records = skip_scan_cost.GetNumRecords();
 
-    trace_idx.add("rows", cur_records).add("cost", cur_read_cost);
+      trace_idx.add("rows", cur_records).add("cost", cur_read_cost);
 
-    min_diff_cost = cur_read_cost;
-    min_diff_cost.multiply(DBL_EPSILON);
-    if (cur_read_cost < (best_read_cost - min_diff_cost)) {
-      index_info = cur_index_info;
-      index = cur_index;
+      min_diff_cost = cur_read_cost;
+      min_diff_cost.multiply(DBL_EPSILON);
+    }
+    if (find_all_skip_scans) {  // called by hypergraph optimizer
+      IndexSkipScanParameters *cur_skip_scan_info =
+          new (param->return_mem_root) IndexSkipScanParameters;
+      if (cur_skip_scan_info == nullptr) return true;
+
+      IndexSkipScanCost skip_scan_cost(table, cur_index, cur_used_key_parts - 1,
+                                       quick_prefix_records, join->where_cond,
+                                       &trace_idx);
+      cur_records = skip_scan_cost.GetNumRecords();
+      cur_skip_scan_info->read_cost = skip_scan_cost.GetCostForHypergraph();
+
+      cur_skip_scan_info->index = cur_index;
+      cur_skip_scan_info->index_info = cur_index_info;
+      cur_skip_scan_info->eq_prefix_len = cur_eq_prefix_len;
+      cur_skip_scan_info->eq_prefix_key_parts = cur_eq_prefix_key_parts;
+      cur_skip_scan_info->range_key_part = cur_range_key_part;
+      cur_skip_scan_info->used_key_parts = cur_used_key_parts;
+      cur_skip_scan_info->num_output_rows = cur_records;
+      cur_skip_scan_info->range_key_len = cur_range_key_part->store_length;
+      cur_skip_scan_info->range_part_tracing_only = cur_range_sel_arg->first();
+      cur_skip_scan_info->index_range_tree = cur_index_range_tree;
+      cur_skip_scan_info->has_aggregate_function = has_aggregate_function;
+
+      if (!setup_range_for_skip_scan(cur_range_sel_arg, param,
+                                     cur_skip_scan_info)) {
+        skip_scan_info_list->push_back(cur_skip_scan_info);
+      } else {
+        ::destroy_at(cur_skip_scan_info);
+      }
+    } else if (cur_read_cost < (best_read_cost - min_diff_cost)) {
+      best_index = cur_index;
+      best_index_info = cur_index_info;
       best_read_cost = cur_read_cost;
       best_records = cur_records;
 
-      eq_prefix_len = cur_eq_prefix_len;
-      eq_prefix_key_parts = cur_eq_prefix_key_parts;
-      index_range_tree = cur_index_range_tree;
+      best_eq_prefix_len = cur_eq_prefix_len;
+      best_eq_prefix_key_parts = cur_eq_prefix_key_parts;
+      best_index_range_tree = cur_index_range_tree;
 
-      range_sel_arg = cur_range_sel_arg;
-      range_key_part = cur_range_key_part;
-      used_key_parts = cur_used_key_parts;
+      best_range_sel_arg = cur_range_sel_arg;
+      best_range_key_part = cur_range_key_part;
+      best_used_key_parts = cur_used_key_parts;
     }
   next_index:
     if (cause) {
@@ -385,101 +572,79 @@ AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
       cause = nullptr;
     }
   }
+  if (!find_all_skip_scans && best_index_info) {  // save only the best scan
+    IndexSkipScanParameters *best_skip_scan_info =
+        new (param->return_mem_root) IndexSkipScanParameters;
+    if (best_skip_scan_info == nullptr) return true;
+
+    best_skip_scan_info->index = best_index;
+    best_skip_scan_info->index_info = best_index_info;
+    best_skip_scan_info->eq_prefix_len = best_eq_prefix_len;
+    best_skip_scan_info->eq_prefix_key_parts = best_eq_prefix_key_parts;
+    best_skip_scan_info->range_key_part = best_range_key_part;
+    best_skip_scan_info->used_key_parts = best_used_key_parts;
+    best_skip_scan_info->num_output_rows = best_records;
+    best_skip_scan_info->range_key_len = best_range_key_part->store_length;
+    best_skip_scan_info->range_part_tracing_only = best_range_sel_arg->first();
+    best_skip_scan_info->index_range_tree = best_index_range_tree;
+    best_skip_scan_info->has_aggregate_function = has_aggregate_function;
+    best_skip_scan_info->read_cost = best_read_cost.total_cost();
+
+    if (!setup_range_for_skip_scan(best_range_sel_arg, param,
+                                   best_skip_scan_info)) {
+      skip_scan_info_list->push_back(best_skip_scan_info);
+    } else {
+      ::destroy_at(best_skip_scan_info);
+    }
+  }
   trace_indices.end();
+  trace_group.end();
+  return false;
+}
 
-  if (!index_info) /* No usable index found. */
-    return nullptr;
+/**
+  Construct a new INDEX_SKIP_SCAN AccessPath from the IndexSkipScanParameters
 
-  // Setup fields that hold the range condition on key part C.
-  //
-  // This is only the suffix of the whole key, that we use
-  // to append to the prefix to get the full key later on.
-  uint range_cond_flag = 0;
-  if (!(range_sel_arg->min_flag & NO_MIN_RANGE) &&
-      !(range_sel_arg->max_flag & NO_MAX_RANGE)) {
-    // IS NULL condition
-    if (range_sel_arg->maybe_null() && range_sel_arg->min_value[0] &&
-        range_sel_arg->max_value[0])
-      range_cond_flag |= NULL_RANGE;
-    // equality condition
-    else if (memcmp(range_sel_arg->min_value, range_sel_arg->max_value,
-                    range_key_part->store_length) == 0)
-      range_cond_flag |= EQ_RANGE;
-  }
-
-  uchar *min_value, *max_value;
-  if (range_sel_arg->is_ascending) {
-    range_cond_flag |= range_sel_arg->min_flag | range_sel_arg->max_flag;
-    min_value = range_sel_arg->min_value;
-    max_value = range_sel_arg->max_value;
-  } else {
-    range_cond_flag |= invert_min_flag(range_sel_arg->min_flag) |
-                       invert_max_flag(range_sel_arg->max_flag);
-    min_value = range_sel_arg->max_value;
-    max_value = range_sel_arg->min_value;
-  }
-
-  unsigned range_key_len = range_key_part->store_length;
-
-  // Allocate storage for min/max key if they exist.
-  unsigned max_used_key_length = 0;
-  for (uint i = 0; i < used_key_parts; ++i) {
-    max_used_key_length += index_info->key_part[i].store_length;
-  }
-
-  MEM_ROOT *const mem_root = param->return_mem_root;
-  uchar *min_range_key = nullptr;
-  uchar *min_search_key = nullptr;
-  if (!(range_cond_flag & NO_MIN_RANGE)) {
-    if (!(min_range_key = mem_root->ArrayAlloc<uchar>(range_key_len)))
-      return nullptr;
-    if (!(min_search_key = mem_root->ArrayAlloc<uchar>(max_used_key_length)))
-      return nullptr;
-
-    memcpy(min_range_key, min_value, range_key_len);
-  }
-  uchar *max_range_key = nullptr;
-  uchar *max_search_key = nullptr;
-  if (!(range_cond_flag & NO_MAX_RANGE)) {
-    if (!(max_range_key = mem_root->ArrayAlloc<uchar>(range_key_len)))
-      return nullptr;
-    if (!(max_search_key = mem_root->ArrayAlloc<uchar>(max_used_key_length)))
-      return nullptr;
-
-    memcpy(max_range_key, max_value, range_key_len);
-  }
-
-  /* The query passes all tests, so construct a new AccessPath. */
+  @param  param     Parameter from test_quick_select
+  @param  force_skip_scan  TRUE if skip scan is forced by optimizer hint
+  @param  skip_scan_info Parameters of index skip scan
+  @retval INDEX_SKIP_SCAN AccessPath
+*/
+AccessPath *make_skip_scan_path(RANGE_OPT_PARAM *param, bool force_skip_scan,
+                                IndexSkipScanParameters *skip_scan_info) {
+  // The query passes all tests, so construct a new AccessPath.
   EQPrefix *eq_prefixes = nullptr;
-  if (eq_prefix_key_parts > 0) {
-    eq_prefixes = mem_root->ArrayAlloc<EQPrefix>(eq_prefix_key_parts);
+  if (skip_scan_info->eq_prefix_key_parts > 0) {
+    eq_prefixes = param->return_mem_root->ArrayAlloc<EQPrefix>(
+        skip_scan_info->eq_prefix_key_parts);
     if (eq_prefixes == nullptr) {
       return nullptr;
     }
 
+    SEL_ROOT *index_range_tree = skip_scan_info->index_range_tree;
     const SEL_ARG *cur_range = index_range_tree->root->first();
     const SEL_ARG *first_range = nullptr;
     const SEL_ROOT *cur_root = index_range_tree;
-    for (uint i = 0; i < eq_prefix_key_parts;
+    for (uint i = 0; i < skip_scan_info->eq_prefix_key_parts;
          i++, cur_range = cur_range->next_key_part->root) {
       eq_prefixes[i].cur_eq_prefix = 0;
       unsigned num_elements = cur_root->elements;
       cur_root = cur_range->next_key_part;
       assert(num_elements > 0);
-      eq_prefixes[i].eq_key_prefixes =
-          Bounds_checked_array<uchar *>::Alloc(mem_root, num_elements);
+      eq_prefixes[i].eq_key_prefixes = Bounds_checked_array<uchar *>::Alloc(
+          param->return_mem_root, num_elements);
 
       uint j = 0;
       first_range = cur_range->first();
       for (cur_range = first_range; cur_range;
            j++, cur_range = cur_range->next) {
-        KEY_PART_INFO *keypart = index_info->key_part + i;
+        KEY_PART_INFO *keypart = skip_scan_info->index_info->key_part + i;
         size_t field_length = keypart->store_length;
         //  Store ranges in the reverse order if key part is descending.
         uint pos = cur_range->is_ascending ? j : num_elements - j - 1;
 
         if (!(eq_prefixes[i].eq_key_prefixes[pos] =
-                  mem_root->ArrayAlloc<uchar>(field_length)))
+                  param->return_mem_root->ArrayAlloc<uchar>(field_length)))
           return nullptr;
 
         if (cur_range->maybe_null() && cur_range->min_value[0] &&
@@ -500,99 +665,47 @@ AccessPath *get_best_skip_scan(THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
 
   AccessPath *path = new (param->return_mem_root) AccessPath;
   path->type = AccessPath::INDEX_SKIP_SCAN;
-  path->cost = best_read_cost.total_cost();
-  path->set_num_output_rows(best_records);
+  path->set_cost(skip_scan_info->read_cost);
+  path->set_init_cost(0.0);
+  path->set_num_output_rows(skip_scan_info->num_output_rows);
 
-  IndexSkipScanParameters *ext =
-      new (param->return_mem_root) IndexSkipScanParameters;
-  ext->index_info = index_info;
-  ext->eq_prefix_len = eq_prefix_len;
-  ext->eq_prefix_key_parts = eq_prefix_key_parts;
-  ext->eq_prefixes = eq_prefixes;
-  ext->range_key_part = range_key_part;
-  ext->min_range_key = min_range_key;
-  ext->max_range_key = max_range_key;
-  ext->min_search_key = min_search_key;
-  ext->max_search_key = max_search_key;
-  ext->range_cond_flag = range_cond_flag;
-  ext->range_key_len = range_key_len;
-  ext->range_part_tracing_only = range_sel_arg->first();
-  ext->index_range_tree = index_range_tree;
-  ext->has_aggregate_function = has_aggregate_function;
-
-  path->index_skip_scan().table = table;
-  path->index_skip_scan().index = index;
-  path->index_skip_scan().num_used_key_parts = used_key_parts;
+  path->index_skip_scan().table = param->table;
+  path->index_skip_scan().index = skip_scan_info->index;
+  path->index_skip_scan().num_used_key_parts = skip_scan_info->used_key_parts;
   path->index_skip_scan().forced_by_hint = force_skip_scan;
-  path->index_skip_scan().param = ext;
+
+  skip_scan_info->eq_prefixes = eq_prefixes;
+  path->index_skip_scan().param =
+      const_cast<IndexSkipScanParameters *>(skip_scan_info);
   return path;
 }
 
-/**
-  Compute the cost of a IndexSkipScanIterator for a particular index.
-
-  SYNOPSIS
-    cost_skip_scan()
-    table                [in] The table being accessed
-    key                  [in] The index used to access the table
-    distinct_key_parts   [in] Number of key_parts used to get distinct prefix
-    quick_prefix_records [in] Number of records processed by prefix ranges
-    cost_est             [out] The cost to retrieve rows via this quick select
-    records              [out] The number of rows retrieved
-    where_cond           [in] WHERE condition
-    trace_idx            [in] optimizer_trace object
-
-  DESCRIPTION
-    This method computes the access cost of an INDEX_SKIP_SCAN access path
-    and the number of rows returned.
-
-  NOTES
-
-    To estimate the size of the groups to read, index statistics
-    from rec_per_key is used. Each equality range decreases
-    number of the groups to read. The total number of processed
-    records from all the groups will be quick_prefix_records if
-    there are equality ranges else it will be the entire table.
-    Number of distinct group is calculated by dividing the
-    number of processed record by the number keys in a group.
-
-    Number of processed records is calculated using following formula:
-
-    records = number_of_distinct_groups * records_per_group * filtering_effect
-
-    where filtering_effect is filtering effect of the range condition.
-
-  RETURN
-    None
-*/
-
-void cost_skip_scan(TABLE *table, uint key, uint distinct_key_parts,
-                    ha_rows quick_prefix_records, Cost_estimate *cost_est,
-                    ha_rows *records, Item *where_cond,
-                    Opt_trace_object *trace_idx) {
+void IndexSkipScanCost::CalcCardinality() {
   ha_rows table_records, skip_scan_records;
-  uint num_groups;
   rec_per_key_t keys_per_group;
-  const KEY *const index_info = &table->key_info[key];
+  const KEY *const index_info = &m_table->key_info[m_key];
   DBUG_TRACE;
 
-  table_records = table->file->stats.records;
-  if (quick_prefix_records == HA_POS_ERROR)
+  table_records = m_table->file->stats.records;
+  if (m_quick_prefix_records == HA_POS_ERROR) {
     skip_scan_records = table_records;
-  else
-    skip_scan_records = quick_prefix_records;
+  } else {
+    skip_scan_records = m_quick_prefix_records;
+  }
 
   /* Compute the number of keys in a group. */
-  if (index_info->has_records_per_key(distinct_key_parts - 1)) {
+  if (index_info->has_records_per_key(m_distinct_key_parts - 1)) {
     // Use index statistics
-    keys_per_group = index_info->records_per_key(distinct_key_parts - 1);
+    keys_per_group = index_info->records_per_key(m_distinct_key_parts - 1);
     assert(keys_per_group >= 0);
   } else
     /* If there is no statistics try to guess */
-    keys_per_group = guess_rec_per_key(table, index_info, distinct_key_parts);
+    keys_per_group =
+        guess_rec_per_key(m_table, index_info, m_distinct_key_parts);
 
-  num_groups = (uint)(skip_scan_records / keys_per_group) + 1;
-  num_groups = std::max(num_groups, 1U);
+  m_cardinality.num_groups =
+      static_cast<uint>(skip_scan_records / keys_per_group) + 1;
+  m_cardinality.num_groups = std::max(m_cardinality.num_groups, 1U);
 
   /* Calculate filtering effect for the range condition */
   {
@@ -602,18 +715,19 @@ void cost_skip_scan(TABLE *table, uint key, uint distinct_key_parts,
     my_bitmap_map *bitbuf =
         static_cast<my_bitmap_map *>(static_cast<void *>(&buf));
     MY_BITMAP ignored_fields;
-    bitmap_init(&ignored_fields, bitbuf, table->s->fields);
+    bitmap_init(&ignored_fields, bitbuf, m_table->s->fields);
     bitmap_set_all(&ignored_fields);
     bitmap_clear_bit(
         &ignored_fields,
-        index_info->key_part[distinct_key_parts].field->field_index());
+        index_info->key_part[m_distinct_key_parts].field->field_index());
 
     /* Compute the number of records per group for the range. */
-    if (index_info->has_records_per_key(distinct_key_parts))
-      keys_per_range = index_info->records_per_key(distinct_key_parts);
-    else
+    if (index_info->has_records_per_key(m_distinct_key_parts)) {
+      keys_per_range = index_info->records_per_key(m_distinct_key_parts);
+    } else {
       keys_per_range =
-          guess_rec_per_key(table, index_info, distinct_key_parts + 1);
+          guess_rec_per_key(m_table, index_info, m_distinct_key_parts + 1);
+    }
     /*
       Calculation of the filtering effect is based on
       Item_field::get_cond_filter_default_probability() where
@@ -622,35 +736,54 @@ void cost_skip_scan(TABLE *table, uint key, uint distinct_key_parts,
     */
     double max_distinct_values = max(
         1.0, static_cast<double>(uint(keys_per_group) / uint(keys_per_range)));
-    float filtering_effect = where_cond->get_filtering_effect(
-        current_thd, table->pos_in_table_list->map(), used_tables,
+    float filtering_effect = m_where_cond->get_filtering_effect(
+        current_thd, m_table->pos_in_table_list->map(), used_tables,
         &ignored_fields, max_distinct_values);
-    *records = max(ha_rows(1), ha_rows(skip_scan_records * filtering_effect));
+    m_cardinality.records =
+        max(ha_rows(1), ha_rows(skip_scan_records * filtering_effect));
   }
 
+  DBUG_PRINT("info", ("table rows: %lu keys/group: %u result rows: %lu",
+                      static_cast<ulong>(table_records),
+                      static_cast<uint>(keys_per_group),
+                      static_cast<ulong>(m_cardinality.records)));
+}
+
+Cost_estimate IndexSkipScanCost::GetCost() const {
+  DBUG_TRACE;
+
   /* Estimate IO cost. */
-  const Cost_model_table *const cost_model = table->cost_model();
-  Cost_estimate cost_skip_scan =
-      table->file->index_scan_cost(key, num_groups, *records);
+  const Cost_model_table *const cost_model = m_table->cost_model();
+  Cost_estimate cost_skip_scan = m_table->file->index_scan_cost(
+      m_key, m_cardinality.num_groups, m_cardinality.records);
 
   /* CPU cost*/
+  ha_rows table_records = m_table->file->stats.records;
   const double tree_height =
       table_records == 0 ? 1.0 : ceil(log2(double(table_records)));
   const double tree_traversal_cost = cost_model->key_compare_cost(tree_height);
   /* Number of re-positions happens twice per group. */
-  trace_idx->add("tree_travel_cost", tree_traversal_cost)
-      .add("num_groups", num_groups);
+  m_trace->add("tree_travel_cost", tree_traversal_cost)
+      .add("num_groups", m_cardinality.num_groups);
   const double cpu_cost =
-      tree_traversal_cost * num_groups * 2 +
-      cost_model->row_evaluate_cost(static_cast<double>(*records)) +
-      cost_model->key_compare_cost(static_cast<double>(*records));
+      tree_traversal_cost * m_cardinality.num_groups * 2 +
+      cost_model->row_evaluate_cost(
+          static_cast<double>(m_cardinality.records)) +
+      cost_model->key_compare_cost(static_cast<double>(m_cardinality.records));
   cost_skip_scan.add_cpu(cpu_cost);
+  m_trace->add("index skip scan cost", cost_skip_scan.total_cost());
 
-  *cost_est = cost_skip_scan;
+  return cost_skip_scan;
+}
 
-  DBUG_PRINT("info",
-             ("table rows: %lu keys/group: %u result rows: %lu",
-              (ulong)table_records, (uint)keys_per_group, (ulong)*records));
+double IndexSkipScanCost::GetCostForHypergraph() const {
+  DBUG_TRACE;
+
+  double cost = EstimateSkipScanCost(m_table, m_key, m_cardinality.num_groups,
+                                     m_cardinality.records);
+  m_trace->add("index skip scan cost in hypergraph", cost);
+
+  return cost;
 }
 
 #ifndef NDEBUG
