@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,7 +24,8 @@
 #ifndef REPLICATION_H
 #define REPLICATION_H
 
-#include "my_thread_local.h"         // my_thread_id
+#include "my_thread_local.h"  // my_thread_id
+#include "mysql/gtid/tag_plain.h"
 #include "mysql/psi/mysql_thread.h"  // mysql_mutex_t
 #include "rpl_context.h"
 #include "sql/handler.h"  // enum_tx_isolation
@@ -102,12 +104,6 @@ typedef struct Trans_context_info {
   ulong binlog_checksum_options;  // enum values in enum
                                   // enum_binlog_checksum_alg
   ulong binlog_format;            // enum values in enum enum_binlog_format
-  // enum values in enum_transaction_write_set_hashing_algorithm
-  ulong transaction_write_set_extraction;
-  ulong mi_repository_type;   // enum values in enum_info_repository
-  ulong rli_repository_type;  // enum values in enum_info_repository
-  // enum values in enum_mts_parallel_type
-  ulong parallel_applier_type;
   ulong parallel_applier_workers;
   bool parallel_applier_preserve_commit_order;
   enum_tx_isolation tx_isolation;  // enum values in enum_tx_isolation
@@ -119,10 +115,16 @@ typedef struct Trans_context_info {
   This represents the GTID context of the transaction.
  */
 typedef struct Trans_gtid_info {
-  rpl_sid sid;        // transaction sid
-  ulong type;         // enum values in enum_gtid_type
-  int sidno;          // transaction sidno
-  long long int gno;  // transaction gno
+  /// transaction specified TSID, filled in the after-commit hook
+  mysql::gtid::Tsid_plain tsid;
+  /// enum values in enum_gtid_type
+  ulong type;
+  /// transaction sidno
+  int sidno;
+  /// transaction gno
+  long long int gno;
+  /// defined tag for automatic GTIDs, propagated in the before-commit hook
+  mysql::gtid::Tag_plain automatic_tag;
 } Trans_gtid_info;
 
 class Binlog_cache_storage;
@@ -194,6 +196,18 @@ typedef struct Trans_param {
 
   /*
     Flag to identify a 'CREATE TABLE ... AS SELECT'.
+
+    'CREATE TABLE ... AS SELECT' is binlogged as
+
+      'CREATE TABLE ... START TRANSACTION'
+      'TABLE MAP'
+      'ROW EVENT'
+      'ROW EVENT'
+      ...
+      'XID / COMMIT'
+
+    At replica, flag is used to identify the 'CREATE TABLE ... START
+    TRANSACTION' block of 'CREATE TABLE ... AS SELECT' event.
   */
   bool is_create_table_as_query_block;
 
@@ -399,6 +413,7 @@ typedef struct Server_state_observer {
   after_recovery_t after_recovery;
   before_server_shutdown_t before_server_shutdown;
   after_server_shutdown_t after_server_shutdown;
+  /* TODO To be removed in WL#16215 */
   after_dd_upgrade_t after_dd_upgrade_from_57;
 } Server_state_observer;
 
@@ -692,7 +707,7 @@ typedef int (*after_queue_event_t)(Binlog_relay_IO_param *param,
                                    unsigned long event_len, uint32 flags);
 
 /**
-  This callback is called after reset slave relay log IO status
+  This callback is called after reset replica relay log IO status
 
   @param param Observer common parameter
 
