@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +25,7 @@
 #define RPL_ASYNC_CONN_FAILOVER_TABLE_OPERATIONS_H
 
 #include <functional>
+#include <string_view>
 #include <tuple>
 #include "sql/field.h"
 #include "sql/table.h"
@@ -407,10 +409,10 @@ class Rpl_async_conn_failover_table_operations {
   static void get_data(Rpl_sys_table_access &table_op, TUP &rows);
 
   /* Configuration column primary key name */
-  static const MYSQL_LEX_CSTRING Primary_weight_key;
+  static constexpr std::string_view Primary_weight_key{"Primary_weight"};
 
   /* Configuration column secondary key name */
-  static const MYSQL_LEX_CSTRING Secondary_weight_key;
+  static constexpr std::string_view Secondary_weight_key{"Secondary_weight"};
 
  private:
   enum thr_lock_type m_lock_type;   // table lock type
