@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -70,6 +70,7 @@
 #include "sql/rpl_trx_boundary_parser.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
+#include "sql/transaction.h"  // trans_begin
 
 /**
   Auxiliary function to stop all the running channel threads according to the
@@ -121,18 +122,7 @@ static void set_mi_settings(Master_info *mi,
           ? opt_mts_replica_parallel_workers
           : channel_info->channel_mts_parallel_workers;
 
-  if (channel_info->channel_mts_parallel_type == RPL_SERVICE_SERVER_DEFAULT) {
-    if (mts_parallel_option == MTS_PARALLEL_TYPE_DB_NAME)
-      mi->rli->channel_mts_submode = MTS_PARALLEL_TYPE_DB_NAME;
-    else
-      mi->rli->channel_mts_submode = MTS_PARALLEL_TYPE_LOGICAL_CLOCK;
-  } else {
-    if (channel_info->channel_mts_parallel_type ==
-        CHANNEL_MTS_PARALLEL_TYPE_DB_NAME)
-      mi->rli->channel_mts_submode = MTS_PARALLEL_TYPE_DB_NAME;
-    else
-      mi->rli->channel_mts_submode = MTS_PARALLEL_TYPE_LOGICAL_CLOCK;
-  }
+  mi->rli->channel_mts_submode = MTS_PARALLEL_TYPE_LOGICAL_CLOCK;
 
   mi->rli->checkpoint_group =
       (channel_info->channel_mta_checkpoint_group == RPL_SERVICE_SERVER_DEFAULT)
@@ -1414,6 +1404,8 @@ bool start_failover_channels() {
 bool channel_change_source_connection_auto_failover(const char *channel,
                                                     bool status) {
   bool error = false;
+  bool thd_created = false;
+  THD *thd = current_thd;
   channel_map.assert_some_wrlock();
 
   Master_info *mi = channel_map.get_mi(channel);
@@ -1423,9 +1415,30 @@ bool channel_change_source_connection_auto_failover(const char *channel,
     return true;
   }
 
+  if (!thd) {
+    thd_created = true;
+    thd = create_surrogate_thread();
+    thd->set_skip_readonly_check();
+  }
+
   mi->channel_wrlock();
   lock_slave_threads(mi);
 
+  /*
+    When autocommit= 0, we force a new transaction to prevent
+    table access deadlocks when a master and slave info repositories
+    transaction is executed after changing SOURCE_CONNECTION_AUTO_FAILOVER.
+  */
+  if (thd->variables.option_bits & OPTION_NOT_AUTOCOMMIT) {
+    assert(!(thd->server_status & SERVER_STATUS_IN_TRANS));
+    if (trans_begin(thd)) {
+      unlock_slave_threads(mi);
+      mi->channel_unlock();
+      if (thd_created) delete_surrogate_thread(thd);
+      return true;
+    }
+  }
+
   if (status && !mi->is_source_connection_auto_failover()) {
     mi->set_source_connection_auto_failover();
     error |= (flush_master_info(mi, true, true, false) != 0);
@@ -1436,9 +1449,25 @@ bool channel_change_source_connection_auto_failover(const char *channel,
     error |= (flush_master_info(mi, true, true, false) != 0);
   }
 
+  /*
+    When autocommit= 0, we force a new transaction to prevent
+    table access deadlocks when a master and slave info repositories
+    transaction is executed after changing SOURCE_CONNECTION_AUTO_FAILOVER.
+  */
+  if (thd->variables.option_bits & OPTION_NOT_AUTOCOMMIT) {
+    if (trans_commit(thd)) {
+      unlock_slave_threads(mi);
+      mi->channel_unlock();
+      if (thd_created) delete_surrogate_thread(thd);
+      return true;
+    }
+  }
+
   unlock_slave_threads(mi);
   mi->channel_unlock();
 
+  if (thd_created) delete_surrogate_thread(thd);
+
   return error;
 }
 
