@@ -1,15 +1,16 @@
-/* Copyright (c) 2005, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,7 +50,9 @@
 #include "mysql/thread_type.h"
 #include "prealloced_array.h"  // Prealloced_array
 #include "sql/binlog.h"        // MYSQL_BIN_LOG
-#include "sql/log_event.h"     //Gtid_log_event
+#include "sql/changestreams/apply/metrics/applier_metrics.h"
+#include "sql/changestreams/apply/metrics/applier_metrics_stub.h"
+#include "sql/log_event.h"  //Gtid_log_event
 #include "sql/psi_memory_key.h"
 #include "sql/query_options.h"
 #include "sql/rpl_gtid.h"         // Gtid_set
@@ -71,8 +74,7 @@ class Rpl_info_handler;
 class Slave_committed_queue;
 class Slave_worker;
 class String;
-struct LEX_MASTER_INFO;
-struct db_worker_hash_entry;
+struct LEX_SOURCE_INFO;
 
 extern uint sql_replica_skip_counter;
 
@@ -80,14 +82,15 @@ typedef Prealloced_array<Slave_worker *, 4> Slave_worker_array;
 
 typedef struct slave_job_item {
   Log_event *data;
-  uint relay_number;
   my_off_t relay_pos;
+  char event_relay_log_name[FN_REFLEN + 1];
+  bool m_is_after_metrics_breakpoint;
 } Slave_job_item;
 
 /**
   This class is used to store the type and value for
-  Assign_gtids_to_anonymous_transactions parameter of Change master command on
-  slave.
+  Assign_gtids_to_anonymous_transactions parameter of Change replication source
+  command on slave.
 */
 class Assign_gtids_to_anonymous_transactions_info {
  public:
@@ -99,7 +102,7 @@ class Assign_gtids_to_anonymous_transactions_info {
     of replica which is the server where this transformation of anonymous to
     gtid event happens. UUID: Anonymous gtid events will be converted to Gtid
     event & the UUID used while create GTIDs will be the one specified via
-    Change master command to the parameter
+    Change replication source command to the parameter
     ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS
   */
   enum class enum_type { AGAT_OFF = 1, AGAT_LOCAL, AGAT_UUID };
@@ -142,7 +145,7 @@ Relay_log_info is initialized from a repository, i.e. table or file, if there is
 one. Otherwise, data members are initialized with defaults by calling
 init_relay_log_info().
 
-The relay.info table/file shall be updated whenever: (i) the relay log file
+The applier metadata shall be updated whenever: (i) the relay log file
 is rotated, (ii) SQL Thread is stopped, (iii) while processing a Xid_log_event,
 (iv) after a Query_log_event (i.e. commit or rollback) and (v) after processing
 any statement written to the binary log without a transaction context.
@@ -284,7 +287,7 @@ class Relay_log_info : public Rpl_info {
 
   /**
     Stores the information related to the ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS
-    parameter of CHANGE MASTER
+    parameter of CHANGE REPLICATION SOURCE
   */
   Assign_gtids_to_anonymous_transactions_info
       m_assign_gtids_to_anonymous_transactions_info;
@@ -339,7 +342,7 @@ class Relay_log_info : public Rpl_info {
   /* The following variables are safe to read any time */
 
   /*
-    When we restart slave thread we need to have access to the previously
+    When we restart replica thread we need to have access to the previously
     created temporary tables. Modified only on init/end and by the SQL
     thread, read only by SQL thread.
   */
@@ -698,8 +701,6 @@ class Relay_log_info : public Rpl_info {
   char group_relay_log_name[FN_REFLEN];
   ulonglong group_relay_log_pos;
   char event_relay_log_name[FN_REFLEN];
-  /* The suffix number of relay log name */
-  uint event_relay_log_number;
   ulonglong event_relay_log_pos;
   ulonglong future_event_relay_log_pos;
 
@@ -763,6 +764,9 @@ class Relay_log_info : public Rpl_info {
   /* Flag that ensures the retrieved GTID set is initialized only once. */
   bool gtid_retrieved_initialized;
 
+  /// Flag that ensures the relay log is sanitized only once.
+  bool relay_log_sanitized = false;
+
   /**
     Stores information on the last processed transaction or the transaction
     that is currently being processed.
@@ -835,13 +839,13 @@ class Relay_log_info : public Rpl_info {
  public:
   bool is_relay_log_truncated() { return m_relay_log_truncated; }
 
-  Sid_map *get_sid_map() { return gtid_set->get_sid_map(); }
+  Tsid_map *get_tsid_map() { return gtid_set->get_tsid_map(); }
 
-  Checkable_rwlock *get_sid_lock() { return get_sid_map()->get_sid_lock(); }
+  Checkable_rwlock *get_tsid_lock() { return get_tsid_map()->get_tsid_lock(); }
 
   void add_logged_gtid(rpl_sidno sidno, rpl_gno gno) {
-    get_sid_lock()->assert_some_lock();
-    assert(sidno <= get_sid_map()->get_max_sidno());
+    get_tsid_lock()->assert_some_lock();
+    assert(sidno <= get_tsid_map()->get_max_sidno());
     gtid_set->ensure_sidno(sidno);
     gtid_set->_add_gtid(sidno, gno);
   }
@@ -889,7 +893,7 @@ class Relay_log_info : public Rpl_info {
 
   /**
     Flag that the group_master_log_pos is invalid. This may occur
-    (for example) after CHANGE MASTER TO RELAY_LOG_POS.  This will
+    (for example) after CHANGE REPLICATION SOURCE TO RELAY_LOG_POS.  This will
     be unset after the first event has been executed and the
     group_master_log_pos is valid again.
 
@@ -899,25 +903,28 @@ class Relay_log_info : public Rpl_info {
 
   /*
     Handling of the relay_log_space_limit optional constraint.
-    ignore_log_space_limit is used to resolve a deadlock between I/O and SQL
-    threads, the SQL thread sets it to unblock the I/O thread and make it
-    temporarily forget about the constraint.
   */
-  ulonglong log_space_limit, log_space_total;
-  std::atomic<bool> ignore_log_space_limit;
+  std::atomic<ulonglong> log_space_limit, log_space_total;
 
-  /*
-    Used by the SQL thread to instructs the IO thread to rotate
-    the logs when the SQL thread needs to purge to release some
-    disk space.
-   */
-  std::atomic<bool> sql_force_rotate_relay;
+  // This flag is used by a coordinator to check if the receiver waits for
+  // a relay log space. If yes, it will enable aggressive relay log
+  // purge.
+  std::atomic_bool is_receiver_waiting_for_rl_space;
+
+  // This is file to which coordinator moved after enforced purge
+  // It is used by the receiver to check if all possible files were purged
+  // before making a decision on whether transaction may fit into the
+  // relay_log_space_limit. It is used to avoid possibly infinite waiting in
+  // case a transaction and required relay log metadata is bigger than
+  // 'relay_log_space_limit'. This filename is protected with the
+  // log_space_lock
+  std::string coordinator_log_after_purge{""};
 
   time_t last_master_timestamp;
 
   /**
     Reset the delay.
-    This is used by RESET SLAVE to clear the delay.
+    This is used by RESET REPLICA to clear the delay.
   */
   void clear_sql_delay() { sql_delay = 0; }
 
@@ -927,12 +934,13 @@ class Relay_log_info : public Rpl_info {
     errors, and have been manually applied by DBA already.
   */
   std::atomic<uint32> slave_skip_counter;
-  std::atomic<ulong> abort_pos_wait; /* Incremented on change master */
+  std::atomic<ulong>
+      abort_pos_wait; /* Incremented on change replication source */
   mysql_mutex_t log_space_lock;
   mysql_cond_t log_space_cond;
 
   /*
-     Condition and its parameters from START SLAVE UNTIL clause.
+     Condition and its parameters from START REPLICA UNTIL clause.
 
      UNTIL condition is tested with is_until_satisfied() method that is
      called by exec_relay_log_event(). is_until_satisfied() caches the result
@@ -1149,15 +1157,6 @@ class Relay_log_info : public Rpl_info {
   // number's is determined by global replica_parallel_workers
   Slave_worker_array workers;
 
-  // To map a database to a worker
-  malloc_unordered_map<std::string,
-                       unique_ptr_with_deleter<db_worker_hash_entry>>
-      mapping_db_to_worker{key_memory_db_worker_hash_entry};
-  bool inited_hash_workers;  //  flag to check if mapping_db_to_worker is inited
-
-  mysql_mutex_t slave_worker_hash_lock;  // for mapping_db_to_worker
-  mysql_cond_t slave_worker_hash_cond;   // for mapping_db_to_worker
-
   /*
     For the purpose of reporting the worker status in performance schema table,
     we need to preserve the workers array after worker thread was killed. So, we
@@ -1200,16 +1199,12 @@ class Relay_log_info : public Rpl_info {
   /*
     Container for references of involved partitions for the current event group
   */
-  // CGAP dynarray holds id:s of partitions of the Current being executed Group
-  Prealloced_array<db_worker_hash_entry *, 4> curr_group_assigned_parts;
   // deferred array to hold partition-info-free events
   Prealloced_array<Slave_job_item, 8> curr_group_da;
 
   bool curr_group_seen_gtid;   // current group started with Gtid-event or not
   bool curr_group_seen_begin;  // current group started with B-event or not
   bool curr_group_isolated;    // current group requires execution in isolation
-  bool mts_end_group_sets_max_dbs;  // flag indicates if partitioning info is
-                                    // discovered
   volatile ulong
       mts_wq_underrun_w_id;  // Id of a Worker whose queue is getting empty
   /*
@@ -1223,6 +1218,7 @@ class Relay_log_info : public Rpl_info {
      Coordinator in order to avoid reaching WQ limits.
   */
   std::atomic<long> mts_wq_excess_cnt;
+  ulonglong mts_groups_assigned;    // number of groups (transactions) scheduled
   long mts_worker_underrun_level;   // % of WQ size at which W is considered
                                     // hungry
   ulong mts_coordinator_basic_nap;  // C sleeps to avoid WQs overrun
@@ -1269,42 +1265,22 @@ class Relay_log_info : public Rpl_info {
     MTS_KILLED_GROUP /* Coordinator gave up to reach MTS_END_GROUP */
   } mts_group_status;
 
-  /*
-    MTS statistics:
-  */
-  ulonglong mts_events_assigned;  // number of events (statements) scheduled
-  ulonglong mts_groups_assigned;  // number of groups (transactions) scheduled
-  std::atomic<ulong>
-      mts_wq_overrun_cnt;   // counter of all mts_wq_excess_cnt increments
-  ulong wq_size_waits_cnt;  // number of times C slept due to WQ:s oversize
-  /*
-    Counter of how many times Coordinator saw Workers are filled up
-    "enough" with assignments. The enough definition depends on
-    the scheduler type.
-  */
-  ulong mts_wq_no_underrun_cnt;
-  std::atomic<longlong>
-      mts_total_wait_overlap;  // Waiting time corresponding to above
-  /*
-    Stats to compute Coordinator waiting time for any Worker available,
-    applies solely to the Commit-clock scheduler.
-  */
-  ulonglong mts_total_wait_worker_avail;
-  ulong mts_wq_overfill_cnt;  // counter of C waited due to a WQ queue was full
-  /*
-    Statistics (todo: replace with WL5631) applies to either Coordinator and
-    Worker. The exec time in the Coordinator case means scheduling. The read
-    time in the Worker case means getting an event out of Worker queue
-  */
-  ulonglong stats_exec_time;
-  ulonglong stats_read_time;
-  struct timespec ts_exec[2];   // per event pre- and post- exec timestamp
-  struct timespec stats_begin;  // applier's bootstrap time
+ private:
+  /// @brief The applier metrics aggregator
+  cs::apply::instruments::Applier_metrics m_coordinator_metrics;
+  /// @brief Empty metric aggregator when metric collection is not active
+  cs::apply::instruments::Applier_metrics_stub m_disabled_metric_aggregator;
+
+ public:
+  /// @brief Returns the replication applier metrics aggregator
+  /// @return the MTA metrics aggregator
+  cs::apply::instruments::Applier_metrics_interface &get_applier_metrics();
+
+  /// Number of times queue memory is exceeded
+  ulong worker_queue_mem_exceeded_count{0};
 
-  time_t mts_last_online_stat;
   /// Last moment in time the MTA printed a coordinator waited stats
   time_t mta_coordinator_has_waited_stat;
-  /* end of MTS statistics */
 
   /**
     Storage for holding newly computed values for the last executed
@@ -1542,8 +1518,8 @@ class Relay_log_info : public Rpl_info {
     (if necessary), calculating the received GTID set for the channel and
     storing the updated rli object configuration into the repository.
 
-    When this function is called in a change master process and the change
-    master procedure will purge all the relay log files later, there is no
+    When this function is called in a change replication source process and the
+    change procedure will purge all the relay log files later, there is no
     reason to try to calculate the received GTID set of the channel based on
     existing relay log files (they will be purged). Allowing reads to existing
     relay log files at this point may lead to put the server in a state where
@@ -1551,13 +1527,13 @@ class Relay_log_info : public Rpl_info {
     replication log files was ON and the keyring plugin is not available
     anymore.
 
-    @param skip_received_gtid_set_recovery When true, skips the received GTID
-                                           set recovery.
+    @param skip_received_gtid_set_and_relaylog_recovery When true, skips the
+    received GTID set and relay log recovery.
 
     @retval 0 Success.
     @retval 1 Error.
   */
-  int rli_init_info(bool skip_received_gtid_set_recovery = false);
+  int rli_init_info(bool skip_received_gtid_set_and_relaylog_recovery = false);
   void end_info();
 
   /** No flush options given to relay log flush */
@@ -1640,26 +1616,9 @@ class Relay_log_info : public Rpl_info {
   inline void set_event_relay_log_name(const char *log_file_name) {
     strmake(event_relay_log_name, log_file_name,
             sizeof(event_relay_log_name) - 1);
-    set_event_relay_log_number(relay_log_name_to_number(log_file_name));
     notify_relay_log_change();
   }
 
-  uint get_event_relay_log_number() { return event_relay_log_number; }
-  void set_event_relay_log_number(uint number) {
-    event_relay_log_number = number;
-  }
-
-  /**
-    Given the extension number of the relay log, gets the full
-    relay log path. Currently used in Slave_worker::retry_transaction()
-
-    @param [in]   number      extension number of relay log
-    @param[in, out] name      The full path of the relay log (per-channel)
-                              to be read by the slave worker.
-  */
-  void relay_log_number_to_name(uint number, char name[FN_REFLEN + 1]);
-  uint relay_log_name_to_number(const char *name);
-
   void set_event_start_pos(my_off_t pos) { event_start_pos = pos; }
   my_off_t get_event_start_pos() { return event_start_pos; }
 
@@ -1686,7 +1645,7 @@ class Relay_log_info : public Rpl_info {
 
     This does not actually sleep; it only sets the state of this
     Relay_log_info object to delaying so that the correct state can be
-    reported by SHOW SLAVE STATUS and SHOW PROCESSLIST.
+    reported by SHOW REPLICA STATUS and SHOW PROCESSLIST.
 
     Requires rli->data_lock.
 
@@ -1847,12 +1806,12 @@ class Relay_log_info : public Rpl_info {
     commit time. Exceptionally, if a server in the replication chain does not
     support the commit timestamps in Gtid_log_event, the delay is applied per
     event and is based on the event timestamp.
-    This is set with CHANGE MASTER TO MASTER_DELAY=X.
+    This is set with CHANGE REPLICATION SOURCE TO SOURCE_DELAY=X.
 
     Guarded by data_lock.  Initialized by the client thread executing
-    START SLAVE.  Written by client threads executing CHANGE MASTER TO
-    MASTER_DELAY=X.  Read by SQL thread and by client threads
-    executing SHOW SLAVE STATUS.  Note: must not be written while the
+    START REPLICA.  Written by client threads executing CHANGE REPLICATION
+    SOURCE TO SOURCE_DELAY=X.  Read by SQL thread and by client threads
+    executing SHOW REPLICA STATUS.  Note: must not be written while the
     slave SQL thread is running, since the SQL thread reads it without
     a lock when executing flush_info().
   */
@@ -1861,43 +1820,47 @@ class Relay_log_info : public Rpl_info {
   /**
     During a delay, specifies the point in time when the delay ends.
 
-    This is used for the SQL_Remaining_Delay column in SHOW SLAVE STATUS.
+    This is used for the SQL_Remaining_Delay column in SHOW REPLICA STATUS.
 
     Guarded by data_lock. Written by the sql thread.  Read by client
-    threads executing SHOW SLAVE STATUS.
+    threads executing SHOW REPLICA STATUS.
   */
   time_t sql_delay_end;
 
   uint32 m_flags;
 
   /*
-    Before the MASTER_DELAY parameter was added (WL#344), relay_log.info
+    Historically, the number of entires in applier metadata was the number
+    of lines in applier metadata file. Since WL#13959, applier metadata can
+    be stored only in table, but the notion of number of line is still
+    preserved.
+    Before the SOURCE_DELAY parameter was added (WL#344), applier metadata
     had 4 lines. Now it has 5 lines.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_DELAY = 5;
+  static const int APPLIER_METADATA_LINES_WITH_DELAY = 5;
 
   /*
-    Before the WL#5599, relay_log.info had 5 lines. Now it has 6 lines.
+    Before the WL#5599, applier metadata had 5 lines. Now it has 6 lines.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_WORKERS = 6;
+  static const int APPLIER_METADATA_LINES_WITH_WORKERS = 6;
 
   /*
-    Before the Id was added (BUG#2334346), relay_log.info
+    Before the Id was added (BUG#2334346), applier metadata
     had 6 lines. Now it has 7 lines.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_ID = 7;
+  static const int APPLIER_METADATA_LINES_WITH_ID = 7;
 
   /*
-    Add a channel in the slave relay log info
+    Add a channel in the applier metadata
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_CHANNEL = 8;
+  static const int APPLIER_METADATA_LINES_WITH_CHANNEL = 8;
 
   /*
-    Represents line number in relay_log.info to save PRIVILEGE_CHECKS_USERNAME.
-    It is username part of PRIVILEGES_CHECKS_USER column in
-    performance_schema.replication_applier_configuration.
+    Represents entry id in applier metadata to save
+    PRIVILEGE_CHECKS_USERNAME. It is username part of PRIVILEGES_CHECKS_USER
+    column in performance_schema.replication_applier_configuration.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_PRIV_CHECKS_USERNAME = 9;
+  static const int APPLIER_METADATA_LINES_WITH_PRIV_CHECKS_USERNAME = 9;
 
   /*
     Maximum length of PRIVILEGE_CHECKS_USERNAME.
@@ -1905,11 +1868,11 @@ class Relay_log_info : public Rpl_info {
   static const int PRIV_CHECKS_USERNAME_LENGTH = 32;
 
   /*
-    Represents line number in relay_log.info to save PRIVILEGE_CHECKS_HOSTNAME.
-    It is hostname part of PRIVILEGES_CHECKS_USER column in
-    performance_schema.replication_applier_configuration.
+    Represents entry id in applier metadata to save
+    PRIVILEGE_CHECKS_HOSTNAME. It is hostname part of PRIVILEGES_CHECKS_USER
+    column in performance_schema.replication_applier_configuration.
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_PRIV_CHECKS_HOSTNAME = 10;
+  static const int APPLIER_METADATA_LINES_WITH_PRIV_CHECKS_HOSTNAME = 10;
 
   /*
     Maximum length of PRIVILEGE_CHECKS_USERNAME.
@@ -1917,38 +1880,42 @@ class Relay_log_info : public Rpl_info {
   static const int PRIV_CHECKS_HOSTNAME_LENGTH = 255;
 
   /*
-    Represents line number in relay_log.info to save REQUIRE_ROW_FORMAT
+    Represents entry id in applier metadata to save REQUIRE_ROW_FORMAT
   */
-  static const int LINES_IN_RELAY_LOG_INFO_WITH_REQUIRE_ROW_FORMAT = 11;
+  static const int APPLIER_METADATA_LINES_WITH_REQUIRE_ROW_FORMAT = 11;
 
   /*
-    Represents line number in relay_log.info to save
+    Represents entry id in applier metadata to save
     REQUIRE_TABLE_PRIMARY_KEY_CHECK
   */
-  static const int
-      LINES_IN_RELAY_LOG_INFO_WITH_REQUIRE_TABLE_PRIMARY_KEY_CHECK = 12;
+  static const int APPLIER_METADATA_LINES_WITH_REQUIRE_TABLE_PRIMARY_KEY_CHECK =
+      12;
 
   /*
-    Represent line number in relay_log.info to save
+    Represent entry id in applier metadata to save
     ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_TYPE.
   */
   static const int
-      LINES_IN_RELAY_LOG_INFO_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_TYPE =
+      APPLIER_METADATA_LINES_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_TYPE =
           13;
 
   /*
-    Represent line number in relay_log.info to save
+    Represent entry id in applier metadata to save
     ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE.
   */
   static const int
-      LINES_IN_RELAY_LOG_INFO_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE =
+      APPLIER_METADATA_LINES_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE =
           14;
   /*
-    Total lines in relay_log.info.
+    Total lines in applier metadata.
     This has to be updated every time a member is added or removed.
+    Historically, the number of entires in applier metadata was the number
+    of lines in applier metadata file. Since WL#13959, applier metadata can
+    be stored only in table, but the notion of number of line is still
+    preserved.
   */
-  static const int MAXIMUM_LINES_IN_RELAY_LOG_INFO_FILE =
-      LINES_IN_RELAY_LOG_INFO_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE;
+  static const int MAXIMUM_APPLIER_METADATA_LINES =
+      APPLIER_METADATA_LINES_WITH_ASSIGN_GTIDS_TO_ANONYMOUS_TRANSACTIONS_VALUE;
 
   bool read_info(Rpl_info_handler *from) override;
   bool write_info(Rpl_info_handler *to) override;
@@ -2005,7 +1972,7 @@ class Relay_log_info : public Rpl_info {
   */
   bool m_allow_drop_write_set;
 
-  /* The object stores and handles START SLAVE UNTIL option */
+  /* The object stores and handles START REPLICA UNTIL option */
   Until_option *until_option;
 
  public:
@@ -2077,13 +2044,13 @@ class Relay_log_info : public Rpl_info {
    Initialize until option object when starting slave.
 
    @param[in] thd The thread object of current session.
-   @param[in] master_param the parameters of START SLAVE.
+   @param[in] master_param the parameters of START REPLICA.
 
    @return int
      @retval 0      Succeeds to initialize until option object.
      @retval <> 0   A defined error number is return if any error happens.
    */
-  int init_until_option(THD *thd, const LEX_MASTER_INFO *master_param);
+  int init_until_option(THD *thd, const LEX_SOURCE_INFO *master_param);
 
   /**
     Detaches the engine ha_data from THD. The fact
@@ -2180,11 +2147,6 @@ inline bool is_mts_worker(const THD *thd) {
   return thd->system_thread == SYSTEM_THREAD_SLAVE_WORKER;
 }
 
-/**
- Auxiliary function to check if we have a db partitioned MTS
- */
-bool is_mts_db_partitioned(Relay_log_info *rli);
-
 /**
   Checks whether the supplied event encodes a (2pc-aware) DDL
   that has been already committed.
@@ -2437,7 +2399,7 @@ class Applier_security_context_guard {
             false, otherwise.
    */
   bool has_access(
-      std::vector<std::tuple<ulong, TABLE const *, Rows_log_event *>>
+      std::vector<std::tuple<Access_bitmask, TABLE const *, Rows_log_event *>>
           &extra_privileges) const;
   /**
     Checks if the `PRIVILEGE_CHECKS_USER` user has access to the privilieges
@@ -2466,7 +2428,7 @@ class Applier_security_context_guard {
     @return true if the privileges are included in the security context and
             false, otherwise.
    */
-  bool has_access(std::initializer_list<ulong> extra_privileges) const;
+  bool has_access(std::initializer_list<Access_bitmask> extra_privileges) const;
 
   /**
     Returns the username for the user for which the security context was
