@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -78,6 +78,7 @@ Alter_info::Alter_info(const Alter_info &rhs, MEM_ROOT *mem_root)
       requested_algorithm(rhs.requested_algorithm),
       requested_lock(rhs.requested_lock),
       with_validation(rhs.with_validation),
+      guided_load(rhs.guided_load),
       new_db_name(rhs.new_db_name),
       new_table_name(rhs.new_table_name) {
   /*
@@ -356,6 +357,18 @@ bool Sql_cmd_alter_table::execute(THD *thd) {
   return result;
 }
 
+bool Sql_cmd_alter_table::reprepare_on_execute_required() const {
+  // Expressions in key and partition clauses end up with being allocated on
+  // differing (incompatible) MEM_ROOTs and thus need to be reprepared. The
+  // incompatibility arises in the case of prepared statements as a parse tree
+  // MEM_ROOT whose lifetime is associated with the lifetime of the prepared
+  // statement ends up containing pointers to parse tree objects that have been
+  // allocated from a MEM_ROOT with a lifetime of the prepared statement's
+  // execution. It's benign (though wasteful) to reprepare other alter table
+  // statements as well.
+  return true;
+}
+
 bool Sql_cmd_discard_import_tablespace::execute(THD *thd) {
   /* Verify that exactly one of the DISCARD and IMPORT flags are set. */
   assert((m_alter_info->flags & Alter_info::ALTER_DISCARD_TABLESPACE) ^
