@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -54,6 +54,7 @@ class Item;
 class Key_spec;
 class String;
 class THD;
+struct CHARSET_INFO;
 class Table_ref;
 
 /**
@@ -342,7 +343,10 @@ class Alter_info {
     ANY_ENGINE_ATTRIBUTE = 1ULL << 39,
 
     /// Set for column visibility attribute alter.
-    ALTER_COLUMN_VISIBILITY = 1ULL << 40
+    ALTER_COLUMN_VISIBILITY = 1ULL << 40,
+
+    /// Must be last, not a real type.
+    ALTER_TYPE_END = 1ULL << 41
   };
 
   enum enum_enable_or_disable { LEAVE_AS_IS, ENABLE, DISABLE };
@@ -442,6 +446,9 @@ class Alter_info {
   */
   enum_with_validation with_validation;
 
+  ///  Whether SECONDARY_LOAD should do guided load.
+  enum_with_validation guided_load;
+
   /// "new_db" (if any) or "db" (if any) or default database from
   /// ALTER TABLE [db.]table [ RENAME [TO|AS|=] [new_db.]new_table ]
   LEX_CSTRING new_db_name;
@@ -467,6 +474,7 @@ class Alter_info {
         requested_algorithm(ALTER_TABLE_ALGORITHM_DEFAULT),
         requested_lock(ALTER_TABLE_LOCK_DEFAULT),
         with_validation(ALTER_VALIDATION_DEFAULT),
+        guided_load(ALTER_VALIDATION_DEFAULT),
         new_db_name(LEX_CSTRING{nullptr, 0}),
         new_table_name(LEX_CSTRING{nullptr, 0}) {}
 
@@ -618,6 +626,7 @@ class Sql_cmd_alter_table : public Sql_cmd_common_alter_table {
   using Sql_cmd_common_alter_table::Sql_cmd_common_alter_table;
 
   bool execute(THD *thd) override;
+  bool reprepare_on_execute_required() const override;
 };
 
 /**
