@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,6 +36,7 @@
 #include "my_inttypes.h"
 #include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "prealloced_array.h"        // Prealloced_array
+#include "sql/auth/auth_acls.h"      // Access_bitmask
 #include "sql/locked_tables_list.h"  // enum_locked_tables_mode
 #include "sql/mdl.h"                 // MDL_savepoint
 #include "sql/sql_array.h"           // Bounds_checked_array
@@ -88,13 +90,14 @@ class Table;
 */
 #define SKIP_NEW_HANDLER 32768
 
-enum find_item_error_report_type {
-  REPORT_ALL_ERRORS,
-  REPORT_EXCEPT_NOT_FOUND,
-  IGNORE_ERRORS,
-  REPORT_EXCEPT_NON_UNIQUE,
-  IGNORE_EXCEPT_NON_UNIQUE
-};
+constexpr int REPORT_NO_ERRORS = 0x00;
+constexpr int REPORT_NON_UNIQUE = 0x01;
+constexpr int REPORT_UNKNOWN_TABLE = 0x02;
+constexpr int REPORT_BAD_FIELD = 0x04;
+constexpr int REPORT_ALL_ERRORS =
+    REPORT_NON_UNIQUE | REPORT_UNKNOWN_TABLE | REPORT_BAD_FIELD;
+
+enum Find_field_result { FIELD_NOT_FOUND, BASE_FIELD_FOUND, VIEW_FIELD_FOUND };
 
 enum enum_tdc_remove_table_type {
   TDC_RT_REMOVE_ALL,
@@ -215,7 +218,7 @@ bool resolve_var_assignments(THD *thd, LEX *lex);
 bool insert_fields(THD *thd, Query_block *query_block, const char *db_name,
                    const char *table_name, mem_root_deque<Item *> *fields,
                    mem_root_deque<Item *>::iterator *it, bool any_privileges);
-bool setup_fields(THD *thd, ulong want_privilege, bool allow_sum_func,
+bool setup_fields(THD *thd, Access_bitmask want_privilege, bool allow_sum_func,
                   bool split_sum_funcs, bool column_update,
                   const mem_root_deque<Item *> *typed_items,
                   mem_root_deque<Item *> *fields,
@@ -242,21 +245,20 @@ bool check_record(THD *thd, Field **ptr);
 */
 bool invoke_table_check_constraints(THD *thd, const TABLE *table);
 
-Field *find_field_in_tables(THD *thd, Item_ident *item, Table_ref *first_table,
-                            Table_ref *last_table, Item **ref,
-                            find_item_error_report_type report_error,
-                            ulong want_privilege, bool register_tree_change);
-Field *find_field_in_table_ref(THD *thd, Table_ref *table_list,
-                               const char *name, size_t length,
-                               const char *item_name, const char *db_name,
-                               const char *table_name, Item **ref,
-                               ulong want_privilege, bool allow_rowid,
-                               uint *cached_field_index_ptr,
-                               bool register_tree_change,
-                               Table_ref **actual_table);
-Field *find_field_in_table(TABLE *table, const char *name, bool allow_rowid,
-                           uint *cached_field_index_ptr);
-Field *find_field_in_table_sef(TABLE *table, const char *name);
+bool find_field_in_tables(THD *thd, Item_ident *item, Table_ref *first_table,
+                          Table_ref *last_table, int report_error,
+                          Access_bitmask want_privilege,
+                          Find_field_result *result, Field **base_field,
+                          Item_ident **ref_field);
+bool find_field_in_table_ref(THD *thd, Table_ref *tr, const char *field_name,
+                             size_t length, const char *alias,
+                             const char *db_name, const char *table_name,
+                             Access_bitmask want_privilege, bool allow_rowid,
+                             Find_field_result *result, Field **base_field,
+                             Item_ident **ref_field);
+Field *find_field_in_table(TABLE *table, const char *field_name,
+                           bool allow_rowid, uint *field_index);
+Field *find_field_in_table_sef(TABLE *table, const char *field_name);
 bool find_item_in_list(THD *thd, Item *item, mem_root_deque<Item *> *items,
                        Item ***found, uint *counter,
                        enum_resolution_type *resolution);
