@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -55,6 +55,7 @@
 #include "sql/join_optimizer/access_path.h"
 #include "sql/join_optimizer/bit_utils.h"
 #include "sql/join_optimizer/walk_access_paths.h"
+#include "sql/json_duality_view/dml.h"  // jdv_delete
 #include "sql/key_spec.h"
 #include "sql/mem_root_array.h"
 #include "sql/mysqld.h"       // stage_...
@@ -245,6 +246,8 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
       table->triggers->has_triggers(TRG_EVENT_DELETE, TRG_ACTION_AFTER);
   unit->set_limit(thd, query_block);
 
+  ulonglong jdv_affected_rows = 0;
+
   AccessPath *range_scan = nullptr;
   join_type type = JT_UNKNOWN;
 
@@ -297,7 +300,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
     IGNORE keyword within federated storage engine. If federated engine is
     removed in the future, use of HA_EXTRA_IGNORE_DUP_KEY and
     HA_EXTRA_NO_IGNORE_DUP_KEY flag should be removed from
-    delete_from_single_table(), DeleteRowsIterator::Init() and
+    delete_from_single_table(), DeleteRowsIterator::DoInit() and
     handler::ha_reset().
   */
   if (lex->is_ignore()) table->file->ha_extra(HA_EXTRA_IGNORE_DUP_KEY);
@@ -608,13 +611,34 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
 
       assert(!thd->is_error());
 
-      if (DeleteCurrentRowAndProcessTriggers(thd, table, has_before_triggers,
-                                             has_after_triggers,
-                                             &deleted_rows)) {
-        error = 1;
-        break;
-      }
+      if (table_list->is_json_duality_view()) {
+        if (deleted_rows != 0 ||
+            table_list->field_translation->item == nullptr) {
+          my_error(ER_JDV_OPERATION_NOT_SUPPORTED, MYF(0),
+                   "Multiple object delete");
+          error = 1;
+          break;
+        }
 
+        DBUG_LOG("jdv_dml", "DML-DELETE: "
+                                << " table_list->field_translation->name: "
+                                << table_list->field_translation->name
+                                << " ->type():"
+                                << table_list->field_translation->item->type());
+        if (jdv::jdv_delete(thd, table_list, &jdv_affected_rows)) {
+          assert(thd->is_error());
+          error = 1;
+          break;
+        }
+        deleted_rows++;
+      } else {
+        if (DeleteCurrentRowAndProcessTriggers(thd, table, has_before_triggers,
+                                               has_after_triggers,
+                                               &deleted_rows)) {
+          error = 1;
+          break;
+        }
+      }
       if (!--limit && using_limit) {
         error = -1;
         break;
@@ -647,6 +671,14 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
 cleanup:
   assert(!lex->is_explain());
 
+  if (table_list->is_json_duality_view()) {
+    if (error < 0) {
+      my_ok(thd, jdv_affected_rows);
+    }
+    // For JSON duality view, event is already written to binlog.
+    return error > 0;
+  }
+
   if (!transactional_table && deleted_rows > 0)
     thd->get_transaction()->mark_modified_non_trans_table(
         Transaction_ctx::STMT);
@@ -721,6 +753,11 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
     if (select->check_view_privileges(thd, DELETE_ACL, SELECT_ACL)) return true;
   }
 
+  if (table_list->is_json_duality_view() &&
+      jdv::jdv_prepare_delete(thd, table_list, is_single_table_plan())) {
+    return true;
+  }
+
   /*
     Deletability test is spread across several places:
     - Target table or view must be updatable (checked below)
@@ -778,6 +815,12 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
   // flattening even if features specific to single-table DELETE (that is, ORDER
   // BY and LIMIT) are used.
   if (lex->using_hypergraph_optimizer()) {
+    if (!multitable) {
+      Table_ref *const delete_table_ref = table_list->updatable_base_table();
+      TABLE *const table = delete_table_ref->table;
+      if (select->active_options() & OPTION_QUICK)
+        (void)table->file->ha_extra(HA_EXTRA_QUICK);
+    }
     multitable = true;
   }
 
@@ -873,6 +916,7 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
 
   opt_trace_print_expanded_query(thd, select, &trace_wrapper);
 
+  select->original_tables_map = select->all_tables_map();
   if (select->has_sj_candidates() && select->flatten_subqueries(thd))
     return true;
 
@@ -903,8 +947,10 @@ bool Sql_cmd_delete::execute_inner(THD *thd) {
     my_ok(thd);
     return false;
   }
-  return multitable ? Sql_cmd_dml::execute_inner(thd)
-                    : delete_from_single_table(thd);
+  return (multitable &&
+          !lex->query_block->get_table_list()->is_json_duality_view())
+             ? Sql_cmd_dml::execute_inner(thd)
+             : delete_from_single_table(thd);
 }
 
 /***************************************************************************
@@ -999,7 +1045,7 @@ bool CheckSqlSafeUpdate(THD *thd, const JOIN *join) {
   return false;
 }
 
-bool DeleteRowsIterator::Init() {
+bool DeleteRowsIterator::DoInit() {
   if (CheckSqlSafeUpdate(thd(), m_join)) {
     return true;
   }
@@ -1187,7 +1233,7 @@ bool DeleteRowsIterator::DoDelayedDeletesFromTable(TABLE *table) {
   return local_error;
 }
 
-int DeleteRowsIterator::Read() {
+int DeleteRowsIterator::DoRead() {
   bool local_error = false;
 
   // First process all the rows returned by the join. Delete immediately from
