@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -87,7 +88,8 @@ bool Sql_cmd_alter_table_exchange_partition::execute(THD *thd) {
   */
   const HA_CREATE_INFO create_info(*lex->create_info);
   Alter_info alter_info(*m_alter_info, thd->mem_root);
-  const ulong priv_needed = ALTER_ACL | DROP_ACL | INSERT_ACL | CREATE_ACL;
+  const Access_bitmask priv_needed =
+      ALTER_ACL | DROP_ACL | INSERT_ACL | CREATE_ACL;
 
   DBUG_TRACE;
 
@@ -718,7 +720,7 @@ bool Sql_cmd_alter_table_truncate_partition::execute(THD *thd) {
         When db_stat is 0, we can pass nullptr as dd::Table since it
         won't be used.
       */
-      destroy(&table);
+      ::destroy_at(&table);
       error = open_table_from_share(thd, &share, "", 0, (uint)READ_ALL, 0,
                                     &table, true, nullptr);
 
