@@ -1,6 +1,6 @@
 #ifndef SQL_PREPARE_H
 #define SQL_PREPARE_H
-/* Copyright (c) 2009, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -202,6 +202,8 @@ class Prepared_statement final {
 
   bool m_with_log{false};
 
+  bool m_first_execution{true};
+
   /// Name of the prepared statement.
   LEX_CSTRING m_name{NULL_CSTR};
   /**
@@ -227,8 +229,8 @@ class Prepared_statement final {
   bool prepare_query(THD *thd);
 
  public:
-  Prepared_statement(THD *thd_arg);
-  virtual ~Prepared_statement();
+  explicit Prepared_statement(THD *thd_arg);
+  ~Prepared_statement();
 
   bool set_name(const LEX_CSTRING &name);
   const LEX_CSTRING &name() const { return m_name; }
