@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116943_1720812561_91_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116943_1720812561_91_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21179566_1759338175_92_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21179566_1759338175_92_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef MY_SQL_PARSER_DEBUG
 # if defined YYDEBUG
@@ -860,11 +860,33 @@ extern int my_sql_parser_debug;
     BERNOULLI_SYM = 1213,
     TABLESAMPLE_SYM = 1214,
     VECTOR_SYM = 1215,
-    KEYWORD_USED_AS_IDENT = 1217,
-    KEYWORD_USED_AS_KEYWORD = 1218,
-    CONDITIONLESS_JOIN = 1219,
-    PREFER_PARENTHESES = 1220,
-    EMPTY_FROM_CLAUSE = 1221
+    PARAMETERS_SYM = 1216,
+    HEADER_SYM = 1217,
+    LIBRARY_SYM = 1218,
+    URI_SYM = 1219,
+    DUALITY_SYM = 1220,
+    RELATIONAL_SYM = 1221,
+    JSON_DUALITY_OBJECT_SYM = 1222,
+    ABSENT_SYM = 1223,
+    FILE_FORMAT_SYM = 1224,
+    FILES_SYM = 1225,
+    FILE_NAME_SYM = 1226,
+    FILE_PATTERN_SYM = 1227,
+    FILE_PREFIX_SYM = 1228,
+    ALLOW_MISSING_FILES_SYM = 1229,
+    AUTO_REFRESH_SYM = 1230,
+    AUTO_REFRESH_SOURCE_SYM = 1231,
+    VERIFY_KEY_CONSTRAINTS_SYM = 1232,
+    STRICT_LOAD_SYM = 1233,
+    EXTERNAL_FORMAT_SYM = 1234,
+    EXTERNAL_SYM = 1235,
+    MATERIALIZED_SYM = 1236,
+    GUIDED_SYM = 1237,
+    KEYWORD_USED_AS_IDENT = 1239,
+    KEYWORD_USED_AS_KEYWORD = 1240,
+    CONDITIONLESS_JOIN = 1241,
+    PREFER_PARENTHESES = 1242,
+    EMPTY_FROM_CLAUSE = 1243
   };
 #endif
 /* Tokens.  */
@@ -1675,11 +1697,33 @@ extern int my_sql_parser_debug;
 #define BERNOULLI_SYM 1213
 #define TABLESAMPLE_SYM 1214
 #define VECTOR_SYM 1215
-#define KEYWORD_USED_AS_IDENT 1217
-#define KEYWORD_USED_AS_KEYWORD 1218
-#define CONDITIONLESS_JOIN 1219
-#define PREFER_PARENTHESES 1220
-#define EMPTY_FROM_CLAUSE 1221
+#define PARAMETERS_SYM 1216
+#define HEADER_SYM 1217
+#define LIBRARY_SYM 1218
+#define URI_SYM 1219
+#define DUALITY_SYM 1220
+#define RELATIONAL_SYM 1221
+#define JSON_DUALITY_OBJECT_SYM 1222
+#define ABSENT_SYM 1223
+#define FILE_FORMAT_SYM 1224
+#define FILES_SYM 1225
+#define FILE_NAME_SYM 1226
+#define FILE_PATTERN_SYM 1227
+#define FILE_PREFIX_SYM 1228
+#define ALLOW_MISSING_FILES_SYM 1229
+#define AUTO_REFRESH_SYM 1230
+#define AUTO_REFRESH_SOURCE_SYM 1231
+#define VERIFY_KEY_CONSTRAINTS_SYM 1232
+#define STRICT_LOAD_SYM 1233
+#define EXTERNAL_FORMAT_SYM 1234
+#define EXTERNAL_SYM 1235
+#define MATERIALIZED_SYM 1236
+#define GUIDED_SYM 1237
+#define KEYWORD_USED_AS_IDENT 1239
+#define KEYWORD_USED_AS_KEYWORD 1240
+#define CONDITIONLESS_JOIN 1241
+#define PREFER_PARENTHESES 1242
+#define EMPTY_FROM_CLAUSE 1243
 
 /* Value type.  */
 
@@ -1701,4 +1745,4 @@ struct MY_SQL_PARSER_LTYPE
 
 int my_sql_parser_parse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116943_1720812561_91_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_21179566_1759338175_92_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
