@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,6 +22,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include <algorithm>
+#include <utility>
 
 #ifdef _WIN32
 /* In OpenSSL before 1.1.0, we need this first. */
@@ -74,7 +76,7 @@ static bool verify_individual_certificate(const char *ssl_cert,
   using raii_store_ctx =
       std::unique_ptr<X509_STORE_CTX, decltype(&X509_STORE_CTX_free)>;
   auto deleter = [&](FILE *ptr) { my_fclose(ptr, MYF(0)); };
-  std::unique_ptr<FILE, decltype(deleter)> fp(
+  std::unique_ptr<FILE, decltype(deleter)> const fp(
       my_fopen(ssl_cert, O_RDONLY | MY_FOPEN_BINARY, MYF(MY_WME)), deleter);
 
   if (!fp) {
@@ -82,7 +84,7 @@ static bool verify_individual_certificate(const char *ssl_cert,
     return true;
   }
 
-  raii_bio bio(BIO_new(BIO_s_file()), &BIO_free);
+  raii_bio const bio(BIO_new(BIO_s_file()), &BIO_free);
   if (!bio) {
     /* purecov: begin inspected */
     LogErr(ERROR_LEVEL, ER_TLS_LIBRARY_ERROR_INTERNAL);
@@ -92,14 +94,14 @@ static bool verify_individual_certificate(const char *ssl_cert,
   }
 
   BIO_set_fp(bio.get(), fp.get(), BIO_NOCLOSE);
-  raii_server_cert server_cert(
+  raii_server_cert const server_cert(
       PEM_read_bio_X509(bio.get(), nullptr, nullptr, nullptr), &X509_free);
   if (!server_cert) {
     /* We are not interested in anything other than X509 certificates */
     return false;
   }
 
-  raii_store store(X509_STORE_new(), &X509_STORE_free);
+  raii_store const store(X509_STORE_new(), &X509_STORE_free);
   if (!store) {
     /* purecov: begin inspected */
     LogErr(ERROR_LEVEL, ER_TLS_LIBRARY_ERROR_INTERNAL);
@@ -126,7 +128,7 @@ static bool verify_individual_certificate(const char *ssl_cert,
     }
   }
 
-  raii_store_ctx store_ctx(X509_STORE_CTX_new(), &X509_STORE_CTX_free);
+  raii_store_ctx const store_ctx(X509_STORE_CTX_new(), &X509_STORE_CTX_free);
   if (!store_ctx) {
     /* purecov: begin inspected */
     LogErr(ERROR_LEVEL, ER_TLS_LIBRARY_ERROR_INTERNAL);
@@ -280,9 +282,11 @@ Ssl_acceptor_context_property_type &operator++(
 }
 
 Ssl_acceptor_context_data::Ssl_acceptor_context_data(
-    std::string channel, bool use_ssl_arg, Ssl_init_callback *callbacks,
+    std::string channel, Ssl_init_callback *callbacks,
     bool report_ssl_error /* = true */, enum enum_ssl_init_error *out_error)
-    : channel_(channel), ssl_acceptor_fd_(nullptr), acceptor_(nullptr) {
+    : channel_(std::move(channel)),
+      ssl_acceptor_fd_(nullptr),
+      acceptor_(nullptr) {
   enum enum_ssl_init_error error_num = SSL_INITERR_NOERROR;
   {
     callbacks->read_parameters(
@@ -292,58 +296,55 @@ Ssl_acceptor_context_data::Ssl_acceptor_context_data(
         &current_tls_session_cache_timeout_);
   }
 
-  if (use_ssl_arg) {
-    /* Verify server certificate */
-    if (verify_individual_certificate(
-            current_cert_.c_str(), current_ca_.c_str(), current_capath_.c_str(),
-            current_crl_.c_str(), current_crlpath_.c_str())) {
-      LogErr(WARNING_LEVEL, ER_SERVER_CERT_VERIFY_FAILED,
-             current_cert_.c_str());
-      /* Verify possible issues in CA certificates*/
-      if (verify_ca_certificates(current_ca_.c_str(), current_capath_.c_str(),
-                                 current_crl_.c_str(),
-                                 current_crlpath_.c_str())) {
-        LogErr(WARNING_LEVEL, ER_WARN_CA_CERT_VERIFY_FAILED);
-      }
-      error_num = SSL_INITERR_INVALID_CERTIFICATES;
-      if (opt_tls_certificates_enforced_validation) {
-        if (out_error) *out_error = error_num;
-        assert(ssl_acceptor_fd_ == nullptr);
-        return;
-      }
+  /* Verify server certificate */
+  if (verify_individual_certificate(
+          current_cert_.c_str(), current_ca_.c_str(), current_capath_.c_str(),
+          current_crl_.c_str(), current_crlpath_.c_str())) {
+    LogErr(WARNING_LEVEL, ER_SERVER_CERT_VERIFY_FAILED, current_cert_.c_str());
+    /* Verify possible issues in CA certificates*/
+    if (verify_ca_certificates(current_ca_.c_str(), current_capath_.c_str(),
+                               current_crl_.c_str(),
+                               current_crlpath_.c_str())) {
+      LogErr(WARNING_LEVEL, ER_WARN_CA_CERT_VERIFY_FAILED);
     }
-    long ssl_flags = process_tls_version(current_version_.c_str());
-
-    /* Turn off server's ticket sending for TLS 1.2 if requested */
-    if (!current_tls_session_cache_mode_) ssl_flags |= SSL_OP_NO_TICKET;
-
-    ssl_acceptor_fd_ = new_VioSSLAcceptorFd(
-        current_key_.c_str(), current_cert_.c_str(), current_ca_.c_str(),
-        current_capath_.c_str(), current_cipher_.c_str(),
-        current_ciphersuites_.c_str(), &error_num, current_crl_.c_str(),
-        current_crlpath_.c_str(), ssl_flags);
-
-    if (!ssl_acceptor_fd_ && report_ssl_error) {
-      LogErr(WARNING_LEVEL, ER_WARN_TLS_CHANNEL_INITIALIZATION_ERROR,
-             channel_.c_str());
-      LogErr(WARNING_LEVEL, ER_SSL_LIBRARY_ERROR, sslGetErrString(error_num));
+    error_num = SSL_INITERR_INVALID_CERTIFICATES;
+    if (opt_tls_certificates_enforced_validation) {
+      if (out_error) *out_error = error_num;
+      assert(ssl_acceptor_fd_ == nullptr);
+      return;
     }
+  }
+  long ssl_flags = process_tls_version(current_version_.c_str());
 
-    if (ssl_acceptor_fd_) acceptor_ = SSL_new(ssl_acceptor_fd_->ssl_context);
+  /* Turn off server's ticket sending for TLS 1.2 if requested */
+  if (!current_tls_session_cache_mode_) ssl_flags |= SSL_OP_NO_TICKET;
 
-    if (ssl_acceptor_fd_ && acceptor_) {
-      SSL_CTX_set_session_cache_mode(ssl_acceptor_fd_->ssl_context,
-                                     current_tls_session_cache_mode_
-                                         ? SSL_SESS_CACHE_SERVER
-                                         : SSL_SESS_CACHE_OFF);
-      SSL_CTX_set_timeout(ssl_acceptor_fd_->ssl_context,
-                          current_tls_session_cache_timeout_);
+  ssl_acceptor_fd_ = new_VioSSLAcceptorFd(
+      current_key_.c_str(), current_cert_.c_str(), current_ca_.c_str(),
+      current_capath_.c_str(), current_cipher_.c_str(),
+      current_ciphersuites_.c_str(), &error_num, current_crl_.c_str(),
+      current_crlpath_.c_str(), ssl_flags);
+
+  if (!ssl_acceptor_fd_ && report_ssl_error) {
+    LogErr(WARNING_LEVEL, ER_WARN_TLS_CHANNEL_INITIALIZATION_ERROR,
+           channel_.c_str());
+    LogErr(WARNING_LEVEL, ER_SSL_LIBRARY_ERROR, sslGetErrString(error_num));
+  }
+
+  if (ssl_acceptor_fd_) acceptor_ = SSL_new(ssl_acceptor_fd_->ssl_context);
+
+  if (ssl_acceptor_fd_ && acceptor_) {
+    SSL_CTX_set_session_cache_mode(ssl_acceptor_fd_->ssl_context,
+                                   current_tls_session_cache_mode_
+                                       ? SSL_SESS_CACHE_SERVER
+                                       : SSL_SESS_CACHE_OFF);
+    SSL_CTX_set_timeout(ssl_acceptor_fd_->ssl_context,
+                        current_tls_session_cache_timeout_);
 #ifdef HAVE_TLSv13
-      /* Turn off server's ticket sending for TLS 1.3 if requested */
-      if (!current_tls_session_cache_mode_ && !(ssl_flags & SSL_OP_NO_TLSv1_3))
-        SSL_CTX_set_num_tickets(ssl_acceptor_fd_->ssl_context, 0);
+    /* Turn off server's ticket sending for TLS 1.3 if requested */
+    if (!current_tls_session_cache_mode_ && !(ssl_flags & SSL_OP_NO_TLSv1_3))
+      SSL_CTX_set_num_tickets(ssl_acceptor_fd_->ssl_context, 0);
 #endif
-    }
   }
   if (out_error) *out_error = error_num;
 }
@@ -355,9 +356,9 @@ Ssl_acceptor_context_data::~Ssl_acceptor_context_data() {
 
 std::string Ssl_acceptor_context_data::show_property(
     Ssl_acceptor_context_property_type property_type) const {
-  auto c =
+  auto *c =
       (ssl_acceptor_fd_ == nullptr) ? nullptr : ssl_acceptor_fd_->ssl_context;
-  auto s = acceptor_;
+  auto *s = acceptor_;
   std::string output;
   switch (property_type) {
     case Ssl_acceptor_context_property_type::accept_renegotiates: {
