@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,20 +29,21 @@
 #include <utility>
 
 #include "field_types.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "my_sys.h"
 #include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
+#include "sql-common/json_dom.h"
+#include "sql-common/json_error_handler.h"
+#include "sql-common/json_path.h"
+#include "sql/current_thd.h"
 #include "sql/error_handler.h"
 #include "sql/field.h"
 #include "sql/handler.h"
 #include "sql/item.h"
 #include "sql/item_json_func.h"
-#include "sql/json_dom.h"
-#include "sql/json_path.h"
 #include "sql/psi_memory_key.h"
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_exception_handler.h"
@@ -52,6 +54,7 @@
 #include "sql/table.h"
 #include "sql/thd_raii.h"
 #include "sql_string.h"
+#include "string_with_len.h"
 
 /******************************************************************************
   Implementation of Table_function
@@ -71,7 +74,9 @@ bool Table_function::write_row() {
 
   if ((error = table->file->ha_write_row(table->record[0]))) {
     if (!table->file->is_ignorable_error(error) &&
-        create_ondisk_from_heap(current_thd, table, error, true, nullptr))
+        create_ondisk_from_heap(
+            current_thd, table, error, /*insert_last_record=*/true,
+            /*ignore_last_dup=*/true, /*is_duplicate=*/nullptr))
       return true;  // Not a table_is_full error
   }
   return false;
@@ -90,6 +95,12 @@ bool Table_function::init_args() {
   return false;
 }
 
+void Table_function::fix_after_pullout(Query_block *parent_query_block,
+                                       Query_block *removed_query_block) {
+  do_fix_after_pullout(parent_query_block, removed_query_block);
+  table->pos_in_table_list->dep_tables = used_tables();
+}
+
 /******************************************************************************
   Implementation of JSON_TABLE function
 ******************************************************************************/
@@ -124,7 +135,7 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
     This need to be set up once per statement, as it doesn't change between
     EXECUTE calls.
   */
-  Prepared_stmt_arena_holder ps_arena_holder(current_thd);
+  const Prepared_stmt_arena_holder ps_arena_holder(current_thd);
 
   while ((col = li++)) {
     String buffer;
@@ -133,7 +144,7 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
     if (col->m_jtc_type != enum_jt_column::JTC_NESTED_PATH) {
       col->m_field_idx = m_vt_list.elements;
       m_vt_list.push_back(col);
-      if (check_column_name(col->field_name)) {
+      if (check_column_name(to_lex_cstring(col->field_name))) {
         my_error(ER_WRONG_COLUMN_NAME, MYF(0), col->field_name);
         return true;
       }
@@ -158,9 +169,9 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
               col->m_default_empty_string->val_str(&buffer);
           assert(default_string != nullptr);
           Json_dom_ptr dom;  //@< we'll receive a DOM here
-          bool parse_error;
-          if (parse_json(*default_string, 0, "JSON_TABLE", &dom, true,
-                         &parse_error) ||
+          JsonParseDefaultErrorHandler parse_handler("JSON_TABLE", 0);
+          if (parse_json(*default_string, &dom, true, parse_handler,
+                         JsonDepthErrorHandler) ||
               (col->sql_type != MYSQL_TYPE_JSON && !dom->is_scalar())) {
             my_error(ER_INVALID_DEFAULT, MYF(0), col->field_name);
             return true;
@@ -172,9 +183,9 @@ bool Table_function_json::init_json_table_col_lists(uint *nest_idx,
               col->m_default_error_string->val_str(&buffer);
           assert(default_string != nullptr);
           Json_dom_ptr dom;  //@< we'll receive a DOM here
-          bool parse_error;
-          if (parse_json(*default_string, 0, "JSON_TABLE", &dom, true,
-                         &parse_error) ||
+          JsonParseDefaultErrorHandler parse_handler("JSON_TABLE", 0);
+          if (parse_json(*default_string, &dom, true, parse_handler,
+                         JsonDepthErrorHandler) ||
               (col->sql_type != MYSQL_TYPE_JSON && !dom->is_scalar())) {
             my_error(ER_INVALID_DEFAULT, MYF(0), col->field_name);
             return true;
@@ -243,6 +254,11 @@ bool Table_function_json::do_init_args() {
     my_error(ER_WRONG_ARGUMENTS, MYF(0), "JSON_TABLE");
     return true;
   }
+
+  if (source->check_cols(1)) {
+    return true;
+  }
+
   try {
     /*
       Check whether given JSON source is a const and it's valid, see also
@@ -287,23 +303,14 @@ bool Table_function_json::init() {
   if (m_vt_list.elements == 0) {
     uint nest_idx = 0;
     if (init_json_table_col_lists(&nest_idx, &top)) return true;
-    List_iterator<Json_table_column> li(m_vt_list);
 
-    /*
-      Check for duplicate names.
-      Two iterators over vt_list are used. First is used to get a field,
-      second - to compare the field with fields in the rest of the list.
-      For each iteration of the first list, we skip fields prior to the
-      first iterator's field.
-    */
-    Json_table_column *first;
-    while ((first = li++)) {
-      Json_table_column *col;
-      List_iterator<Json_table_column> li2(m_vt_list);
-      // Compare 'first' with all columns prior to it
-      while ((col = li2++) && col != first) {
-        if (!strncmp(first->field_name, col->field_name, NAME_CHAR_LEN)) {
-          my_error(ER_DUP_FIELDNAME, MYF(0), first->field_name);
+    // Check for duplicate field names.
+    for (Create_field &outer : m_vt_list) {
+      Name_string outer_name(to_lex_cstring(outer.field_name));
+      for (Create_field &inner : m_vt_list) {
+        if (&outer == &inner) break;
+        if (outer_name.eq(inner.field_name)) {
+          my_error(ER_DUP_FIELDNAME, MYF(0), inner.field_name);
           return true;
         }
       }
@@ -360,7 +367,7 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
             Json_array *a = new (std::nothrow) Json_array();
             if (!a) return true;
             for (Json_wrapper &w : data_v) {
-              if (a->append_alias(w.clone_dom(thd))) {
+              if (a->append_alias(w.clone_dom())) {
                 delete a; /* purecov: inspected */
                 return true;
               }
@@ -435,6 +442,7 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
         fld->store(1, true);
       else
         fld->store(0, true);
+      if (current_thd->is_error()) return true;
       fld->set_notnull();
       break;
     }
@@ -484,7 +492,7 @@ bool Json_table_column::fill_column(Table_function_json *table_function,
 
 Json_table_column::~Json_table_column() {
   // Reset paths and wrappers to free allocated memory.
-  m_path_json = Json_path();
+  m_path_json = Json_path(key_memory_JSON);
   if (m_on_empty == Json_on_response_type::DEFAULT)
     m_default_empty_json = Json_wrapper();
   if (m_on_error == Json_on_response_type::DEFAULT)
@@ -618,11 +626,15 @@ bool Table_function_json::fill_result_table() {
       3. Non-const, e.g. a table field: source will be parsed here EVERY TIME
          fill_result_table() is called
     */
-    if (((!source->const_item() || !is_source_parsed) &&
-         get_json_wrapper(args, 0, &buf, func_name(), &m_jds[0].jdata)) ||
-        args[0]->null_value)
+    if (!source->const_item() || !is_source_parsed) {
+      if (get_json_wrapper(args, 0, &buf, func_name(), &m_jds[0].jdata)) {
+        return true;
+      }
+    }
+    if (args[0]->null_value) {
       // No need to set null_value as it's not used by table functions
       return false;
+    }
     is_source_parsed = true;
     return fill_json_table();
   } catch (...) {
@@ -660,6 +672,26 @@ void print_on_empty_or_error(const THD *thd, String *str,
     str->append(STRING_WITH_LEN(" on error"));
 }
 
+/// Prints the type of a column in a JSON_TABLE expression.
+static bool print_json_table_column_type(const Field *field, String *str) {
+  StringBuffer<STRING_BUFFER_USUAL_SIZE> type;
+  field->sql_type(type);
+  if (str->append(type)) return true;
+  if (field->has_charset()) {
+    // Append the character set.
+    if (str->append(STRING_WITH_LEN(" character set ")) ||
+        str->append(field->charset()->csname))
+      return true;
+    // Append the collation, if it is not the primary collation of the
+    // character set.
+    if ((field->charset()->state & MY_CS_PRIMARY) == 0 &&
+        (str->append(STRING_WITH_LEN(" collate ")) ||
+         str->append(field->charset()->m_coll_name)))
+      return true;
+  }
+  return false;
+}
+
 /**
   Helper function to print a single NESTED PATH column.
 
@@ -691,21 +723,13 @@ static bool print_nested_path(const THD *thd, const TABLE *table,
       case enum_jt_column::JTC_PATH: {
         append_identifier(thd, str, jtc.field_name, strlen(jtc.field_name));
         if (str->append(' ')) return true;
-        const Field *field = table->field[jtc.m_field_idx];
-        StringBuffer<STRING_BUFFER_USUAL_SIZE> type;
-        field->sql_type(type);
-        if (str->append(type)) return true;
-        if (field->has_charset()) {
-          // Append the character set.
-          if (str->append(STRING_WITH_LEN(" character set ")) ||
-              str->append(field->charset()->csname))
-            return true;
-          // Append the collation, if it is not the primary collation of the
-          // character set.
-          if ((field->charset()->state & MY_CS_PRIMARY) == 0 &&
-              (str->append(STRING_WITH_LEN(" collate ")) ||
-               str->append(field->charset()->name)))
+        if (table == nullptr) {
+          if (str->append(STRING_WITH_LEN("<column type not resolved yet>"))) {
             return true;
+          }
+        } else if (print_json_table_column_type(table->field[jtc.m_field_idx],
+                                                str)) {
+          return true;
         }
         if (jtc.m_jtc_type == enum_jt_column::JTC_EXISTS) {
           if (str->append(STRING_WITH_LEN(" exists"))) return true;
@@ -741,7 +765,9 @@ bool Table_function_json::print(const THD *thd, String *str,
           str->append(')'));
 }
 
-table_map Table_function_json::used_tables() { return source->used_tables(); }
+table_map Table_function_json::used_tables() const {
+  return source->used_tables();
+}
 
 void Table_function_json::do_cleanup() {
   source->cleanup();
@@ -750,6 +776,11 @@ void Table_function_json::do_cleanup() {
   for (uint i = 0; i < m_all_columns.size(); i++) m_all_columns[i]->cleanup();
 }
 
+void Table_function_json::do_fix_after_pullout(
+    Query_block *parent_query_block, Query_block *removed_query_block) {
+  source->fix_after_pullout(parent_query_block, removed_query_block);
+}
+
 void JT_data_source::cleanup() {
   v.clear();
   producing_records = false;
