@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2004, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2004, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,11 +41,9 @@
 #include <sys/types.h>
 #include <time.h>
 
-#include "guard.h"
 #include "lex_string.h"
-#include "m_ctype.h"
-#include "m_string.h"  // strmake
 #include "map_helpers.h"
+#include "mutex_lock.h"  // MUTEX_LOCK
 #include "my_alloc.h"
 #include "my_base.h"
 #include "my_compiler.h"
@@ -52,21 +51,22 @@
 #include "my_dir.h"
 #include "my_inttypes.h"
 #include "my_io.h"
-#include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_pointer_arithmetic.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
-#include "my_time.h"  // MY_TIME_T_MIN
+#include "my_time.h"
+#include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/components/services/bits/psi_memory_bits.h"
+#include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
-#include "mysql/components/services/mysql_mutex_bits.h"
-#include "mysql/components/services/psi_memory_bits.h"
-#include "mysql/components/services/psi_mutex_bits.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "sql/dd/types/event.h"
 #include "sql/field.h"
@@ -78,6 +78,7 @@
 #include "sql/thr_malloc.h"
 #include "sql/time_zone_common.h"  // ABBR_ARE_USED
 #include "sql/tzfile.h"            // TZ_MAX_REV_RANGES
+#include "string_with_len.h"
 #include "template_utils.h"
 #include "thr_lock.h"
 #include "thr_mutex.h"
@@ -88,11 +89,13 @@
 #include "sql/sql_base.h"   // close_trans_system_tables
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_time.h"   // localtime_to_TIME
-#include "sql/table.h"      // TABLE_LIST
+#include "sql/table.h"      // Table_ref
 #include "sql_string.h"     // String
+#include "strmake.h"
 
 #include <algorithm>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 
@@ -186,11 +189,11 @@ void sec_to_TIME(MYSQL_TIME *tmp, my_time_t t, int64 offset) {
 }
 
 /*
-  Find time range wich contains given my_time_t value
+  Find time range which contains given time value
 
   SYNOPSIS
     find_time_range()
-      t                - my_time_t value for which we looking for range
+      t                - time value for which we looking for containing range
       range_boundaries - sorted array of range starts.
       higher_bound     - number of ranges
 
@@ -198,7 +201,7 @@ void sec_to_TIME(MYSQL_TIME *tmp, my_time_t t, int64 offset) {
     Performs binary search for range which contains given my_time_t value.
     It has sense if number of ranges is greater than zero and my_time_t value
     is greater or equal than beginning of first range. It also assumes that
-    t belongs to some range specified or end of last is MY_TIME_T_MAX.
+    t belongs to some range specified or end of last is MYTIME_MAX_VALUE.
 
     With this localtime_r on real data may takes less time than with linear
     search (I've seen 30% speed up).
@@ -268,7 +271,7 @@ static const TRAN_TYPE_INFO *find_transition_type(my_time_t t,
 
   SYNOPSIS
     gmt_sec_to_TIME()
-      tmp          - pointer to structure for broken down represenatation
+      tmp          - pointer to structure for broken down representation
       sec_in_utc   - my_time_t value to be converted
       sp           - pointer to struct with time zone description
 
@@ -279,7 +282,7 @@ static const TRAN_TYPE_INFO *find_transition_type(my_time_t t,
     (60th and 61st second, look how we calculate them as "hit" in this
     function).
     Under realistic assumptions about frequency of transitions the same array
-    can be used fot MYSQL_TIME -> my_time_t conversion. For this we need to
+    can be used for MYSQL_TIME -> my_time_t conversion. For this we need to
     implement tweaked binary search which will take into account that some
     MYSQL_TIME has two matching my_time_t ranges and some of them have none.
 */
@@ -331,34 +334,9 @@ static void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t sec_in_utc,
   tmp->second += hit;
 }
 
-/**
-  Converts time from a MYSQL_TIME struct to a unix timestamp-like 64 bit
-  integer. The function is guaranteed to use 64 bits on any platform.
-
-  @todo Make sec_since_epoch() call this function instead of duplicating the
-  code.
-
-  @param mt The time to convert.
-  @return A value compatible with a 64 bit Unix timestamp.
-*/
-static int64_t sec_since_epoch64(const MYSQL_TIME &mt) {
-  assert(mt.month > 0 && mt.month < 13);
-  // The year can be negative wrt to the epoch, hence the cast to signed.
-  auto year = static_cast<int64_t>(mt.year);
-  int64_t days = year * DAYS_PER_NYEAR - EPOCH_YEAR * DAYS_PER_NYEAR +
-                 LEAPS_THRU_END_OF(year - 1) -
-                 LEAPS_THRU_END_OF(EPOCH_YEAR - 1);
-  days += mon_starts[isleap(year)][mt.month - 1];
-  days += mt.day - 1;
-
-  return ((days * HOURS_PER_DAY + mt.hour) * MINS_PER_HOUR + mt.minute) *
-             SECS_PER_MIN +
-         mt.second;
-}
-
 /*
   Converts local time in broken down representation to local
-  time zone analog of my_time_t represenation.
+  time zone analog of my_time_t representation.
 
   SYNOPSIS
     sec_since_epoch()
@@ -366,29 +344,45 @@ static int64_t sec_since_epoch64(const MYSQL_TIME &mt) {
 
   DESCRIPTION
     Converts time in broken down representation to my_time_t representation
-    ignoring time zone. Note that we cannot convert back some valid _local_
-    times near ends of my_time_t range because of my_time_t  overflow. But we
-    ignore this fact now since MySQL will never pass such argument.
+    ignoring time zone.
 
   RETURN VALUE
     Seconds since epoch time representation.
 */
 static my_time_t sec_since_epoch(int year, int mon, int mday, int hour, int min,
                                  int sec) {
-  /* Guard against my_time_t overflow(on system with 32 bit my_time_t) */
-  assert(!(year == TIMESTAMP_MAX_YEAR && mon == 1 && mday > 17));
   /*
     It turns out that only whenever month is normalized or unnormalized
     plays role.
   */
-  assert(mon > 0 && mon < 13);
-  long days = year * DAYS_PER_NYEAR - EPOCH_YEAR * DAYS_PER_NYEAR +
-              LEAPS_THRU_END_OF(year - 1) - LEAPS_THRU_END_OF(EPOCH_YEAR - 1);
+  assert(mon > 0 && mon < 13 && year <= 9999);
+  my_time_t days = year * DAYS_PER_NYEAR - EPOCH_YEAR * DAYS_PER_NYEAR +
+                   LEAPS_THRU_END_OF(year - 1) -
+                   LEAPS_THRU_END_OF(EPOCH_YEAR - 1);
   days += mon_starts[isleap(year)][mon - 1];
   days += mday - 1;
 
-  return ((days * HOURS_PER_DAY + hour) * MINS_PER_HOUR + min) * SECS_PER_MIN +
-         sec;
+  const my_time_t result =
+      ((days * HOURS_PER_DAY + hour) * MINS_PER_HOUR + min) * SECS_PER_MIN +
+      sec;
+  return result;
+}
+
+/**
+ Converts time from a MYSQL_TIME struct to a unix timestamp-like 64 bit
+ integer. The function is guaranteed to use 64 bits on any platform.
+
+ @todo Make sec_since_epoch() call this function instead of duplicating the
+ code.
+
+ @param mt The time to convert.
+ @return A value compatible with a 64 bit Unix timestamp.
+*/
+static int64_t sec_since_epoch64(const MYSQL_TIME &mt) {
+  assert(mt.month > 0 && mt.month < 13);
+  // The year can be negative wrt to the epoch, hence the cast to signed.
+  auto year = static_cast<int64_t>(mt.year);
+  return sec_since_epoch(year, mt.month, mt.day, mt.hour, mt.minute, mt.second);
 }
 
 static my_time_t sec_since_epoch(const MYSQL_TIME &mt) {
@@ -404,7 +398,7 @@ static my_time_t sec_since_epoch(const MYSQL_TIME &mt) {
 
   SYNOPSIS
     TIME_to_gmt_sec()
-      t               - pointer to structure for broken down represenatation
+      t               - pointer to structure for broken down representation
       sp              - pointer to struct with time zone description
       in_dst_time_gap - pointer to bool which is set to true if datetime
                         value passed doesn't really exist (i.e. falls into
@@ -420,11 +414,12 @@ static my_time_t sec_since_epoch(const MYSQL_TIME &mt) {
     - It can give wrong results near the ends of my_time_t due to
       overflows, but we are safe since in MySQL we will never
       call this function for such dates (its restriction for year
-      between 1970 and 2038 gives us several days of reserve).
+      between 1970 and 2038 gives us several days of reserve for 32 bit
+      time platforms).
     - By default it doesn't support un-normalized input. But if
       sec_since_epoch() function supports un-normalized dates
       then this function should handle un-normalized input right,
-      altough it won't normalize structure TIME.
+      although it won't normalize structure TIME.
 
     Traditional approach to problem of conversion from broken down
     representation to time_t is iterative. Both elsie's and glibc
@@ -440,7 +435,7 @@ static my_time_t sec_since_epoch(const MYSQL_TIME &mt) {
     and we have two possible answers).
 
     We use completely different approach. It is better since it is both
-    faster than iterative implementations and fully determenistic. If you
+    faster than iterative implementations and fully deterministic. If you
     look at my_time_t to MYSQL_TIME conversion then you'll find that it consist
     of two steps:
     The first is calculating shifted my_time_t value and the second - TIME
@@ -449,7 +444,7 @@ static my_time_t sec_since_epoch(const MYSQL_TIME &mt) {
     my_time_t conversion. It is piecewise linear function which is defined
     by combination of transition times as break points and times offset
     as changing function parameter. The possible inverse function for this
-    converison would be ambiguos but with MySQL's restrictions we can use
+    conversion would be ambiguous but with MySQL's restrictions we can use
     some function which is the same as inverse function on unambigiuos
     ranges and coincides with one of branches of inverse function in
     other ranges. Thus we just need to build table which will determine
@@ -472,14 +467,14 @@ static my_time_t sec_since_epoch(const MYSQL_TIME &mt) {
 */
 static my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp,
                                  bool *in_dst_time_gap) {
-  longlong local_t;
+  my_time_t local_t;
   uint saved_seconds;
   uint i;
   int shift = 0;
 
   DBUG_TRACE;
 
-  if (!validate_timestamp_range(*t)) return 0;
+  if (!validate_my_time(*t)) return 0;
 
   /* We need this for correct leap seconds handling */
   if (t->second < SECS_PER_MIN)
@@ -499,11 +494,11 @@ static my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp,
     => we negative values of local_t are ok.
   */
 
-  if ((t->year == TIMESTAMP_MAX_YEAR) && (t->month == 1) && t->day > 4) {
+  if ((t->year == MYTIME_MAX_YEAR) && (t->month == 1) && t->day > 4) {
     /*
       We will pass (t->day - shift) to sec_since_epoch(), and
       want this value to be a positive number, so we shift
-      only dates > 4.01.2038 (to avoid owerflow).
+      only dates > 2038-01-04 (to avoid overflow).
     */
     shift = 2;
   }
@@ -526,13 +521,13 @@ static my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp,
   i = find_time_range(local_t, sp->revts, sp->revcnt);
 
   /*
-    As there are no offset switches at the end of TIMESTAMP range,
+    As there are no offset switches at the end of my_time_t range,
     we could simply check for overflow here (and don't need to bother
     about DST gaps etc)
   */
   if (shift) {
-    if (local_t > (my_time_t)(TIMESTAMP_MAX_VALUE - shift * SECS_PER_DAY +
-                              sp->revtis[i].rt_offset - saved_seconds)) {
+    if (local_t > MYTIME_MAX_VALUE - shift * SECS_PER_DAY +
+                      sp->revtis[i].rt_offset - saved_seconds) {
       return 0; /* my_time_t overflow */
     }
     local_t += shift * SECS_PER_DAY;
@@ -550,10 +545,10 @@ static my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, const TIME_ZONE_INFO *sp,
   } else
     local_t = local_t + saved_seconds - sp->revtis[i].rt_offset;
 
-  /* check for TIMESTAMP_MAX_VALUE was already done above */
-  if (local_t < TIMESTAMP_MIN_VALUE) local_t = 0;
-
-  return static_cast<my_time_t>(local_t);
+  if (is_time_t_valid_for_timestamp(local_t))
+    return static_cast<my_time_t>(local_t);
+  else
+    return 0;
 }
 
 /*
@@ -621,10 +616,10 @@ bool convert_time_zone_displacement(const Time_zone *tz, MYSQL_TIME *mt) {
   if (check_time_zone_convertibility(*mt)) return true;
 
   MYSQL_TIME out;
-  std::int64_t epoch_secs_in_utc =
+  const std::int64_t epoch_secs_in_utc =
       sec_since_epoch64(*mt) - mt->time_zone_displacement;
 
-  ulong microseconds = mt->second_part;
+  const ulong microseconds = mt->second_part;
 
   tz->gmt_sec_to_TIME(&out, epoch_secs_in_utc);
   out.second_part = microseconds;
@@ -647,7 +642,7 @@ bool convert_time_zone_displacement(const Time_zone *tz, MYSQL_TIME *mt) {
   were no explicit time zone specified. On the other hand because of this
   conversion methods provided by this class is significantly slower and
   possibly less multi-threaded-friendly than corresponding Time_zone_db
-  methods so the latter should be preffered there it is possible.
+  methods so the latter should be preferred there it is possible.
 */
 class Time_zone_system : public Time_zone {
  public:
@@ -692,7 +687,7 @@ my_time_t Time_zone_system::TIME_to_gmt_sec(const MYSQL_TIME *mt,
   if (mt->time_type == MYSQL_TIMESTAMP_DATETIME_TZ)
     return sec_since_epoch(*mt) - mt->time_zone_displacement;
 
-  long not_used;
+  my_time_t not_used;
   return my_system_gmt_sec(*mt, &not_used, in_dst_time_gap);
 }
 
@@ -714,9 +709,20 @@ my_time_t Time_zone_system::TIME_to_gmt_sec(const MYSQL_TIME *mt,
 */
 void Time_zone_system::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const {
   struct tm tmp_tm;
-  time_t tmp_t = (time_t)t;
+  const time_t tmp_t = (time_t)t;
 
   localtime_r(&tmp_t, &tmp_tm);
+  if (tmp_tm.tm_year <= 0) {  // Windows sets -1 if timestamp is too high.
+    tmp->year = 0;
+    tmp->month = 0;
+    tmp->day = 0;
+    tmp->hour = 0;
+    tmp->minute = 0;
+    tmp->second = 0;
+    tmp->second_part = 0;
+    tmp->time_type = MYSQL_TIMESTAMP_DATETIME;
+    return;
+  }
   localtime_to_TIME(tmp, &tmp_tm);
   tmp->time_type = MYSQL_TIMESTAMP_DATETIME;
   adjust_leap_second(tmp);
@@ -769,8 +775,9 @@ class Time_zone_utc : public Time_zone {
   RETURN VALUE
     Corresponding my_time_t value, or 0 in case of error.
 */
-my_time_t Time_zone_utc::TIME_to_gmt_sec(
-    const MYSQL_TIME *mt, bool *in_dst_time_gap MY_ATTRIBUTE((unused))) const {
+my_time_t Time_zone_utc::TIME_to_gmt_sec(const MYSQL_TIME *mt,
+                                         bool *in_dst_time_gap
+                                         [[maybe_unused]]) const {
   return sec_since_epoch(*mt);
 }
 
@@ -784,12 +791,24 @@ my_time_t Time_zone_utc::TIME_to_gmt_sec(
       t   - my_time_t value to be converted
 
   NOTE
-    See note for apropriate Time_zone_system method.
+    See note for appropriate Time_zone_system method.
 */
 void Time_zone_utc::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const {
   struct tm tmp_tm;
-  time_t tmp_t = (time_t)t;
+  const time_t tmp_t = (time_t)t;
   gmtime_r(&tmp_t, &tmp_tm);
+
+  if (tmp_tm.tm_year <= 0) {  // Windows sets -1 if timestamp is too high.
+    tmp->year = 0;
+    tmp->month = 0;
+    tmp->day = 0;
+    tmp->hour = 0;
+    tmp->minute = 0;
+    tmp->second = 0;
+    tmp->second_part = 0;
+    tmp->time_type = MYSQL_TIMESTAMP_DATETIME;
+    return;
+  }
   localtime_to_TIME(tmp, &tmp_tm);
   tmp->time_type = MYSQL_TIMESTAMP_DATETIME;
   adjust_leap_second(tmp);
@@ -934,10 +953,10 @@ class Time_zone_offset : public Time_zone {
                       Positive for direction to east.
 */
 Time_zone_offset::Time_zone_offset(long tz_offset_arg) : offset(tz_offset_arg) {
-  uint hours = abs((int)(offset / SECS_PER_HOUR));
-  uint minutes = abs((int)(offset % SECS_PER_HOUR / SECS_PER_MIN));
-  size_t length = snprintf(name_buff, sizeof(name_buff), "%s%02d:%02d",
-                           (offset >= 0) ? "+" : "-", hours, minutes);
+  const uint hours = abs((int)(offset / SECS_PER_HOUR));
+  const uint minutes = abs((int)(offset % SECS_PER_HOUR / SECS_PER_MIN));
+  const size_t length = snprintf(name_buff, sizeof(name_buff), "%s%02d:%02d",
+                                 (offset >= 0) ? "+" : "-", hours, minutes);
   name.set(name_buff, length, &my_charset_latin1);
 }
 
@@ -953,25 +972,21 @@ Time_zone_offset::Time_zone_offset(long tz_offset_arg) : offset(tz_offset_arg) {
 
   @return Corresponding my_time_t value or 0 for invalid datetime values.
 */
-my_time_t Time_zone_offset::TIME_to_gmt_sec(
-    const MYSQL_TIME *t, bool *in_dst_time_gap MY_ATTRIBUTE((unused))) const {
-  /*
-    Check timestamp range. We have to do this as the caller relies on
-    us to make all validation checks here.
-  */
-  if (!validate_timestamp_range(*t)) return 0;
+my_time_t Time_zone_offset::TIME_to_gmt_sec(const MYSQL_TIME *t,
+                                            bool *in_dst_time_gap
+                                            [[maybe_unused]]) const {
+  if (!validate_my_time(*t)) return 0;
 
   /*
     Do a temporary shift of the boundary dates to avoid
     overflow of my_time_t if the time value is near its
     maximum range
   */
-  int shift = ((t->year == TIMESTAMP_MAX_YEAR) && (t->month == 1) && t->day > 4)
-                  ? 2
-                  : 0;
+  const int shift =
+      ((t->year == MYTIME_MAX_YEAR) && (t->month == 1) && t->day > 4) ? 2 : 0;
 
-  longlong local_t = sec_since_epoch(t->year, t->month, (t->day - shift),
-                                     t->hour, t->minute, t->second);
+  my_time_t local_t = sec_since_epoch(t->year, t->month, (t->day - shift),
+                                      t->hour, t->minute, t->second);
 
   if (t->time_type == MYSQL_TIMESTAMP_DATETIME_TZ)
     local_t -= t->time_zone_displacement;
@@ -983,8 +998,8 @@ my_time_t Time_zone_offset::TIME_to_gmt_sec(
     local_t += shift * SECS_PER_DAY;
   }
 
-  if (local_t >= TIMESTAMP_MIN_VALUE && local_t <= TIMESTAMP_MAX_VALUE)
-    return static_cast<my_time_t>(local_t);
+  if (local_t >= MYTIME_MIN_VALUE && local_t <= MYTIME_MAX_VALUE)
+    return local_t;
 
   /* range error*/
   return 0;
@@ -1049,21 +1064,20 @@ static MEM_ROOT tz_storage;
   above to take.
 */
 static mysql_mutex_t tz_LOCK;
-static bool tz_inited = false;
 
 /*
-  This two static variables are inteded for holding info about leap seconds
+  These two static variables are intended for holding info about leap seconds
   shared by all time zones.
 */
 static uint tz_leapcnt = 0;
-static LS_INFO *tz_lsis = nullptr;
+static LS_INFO tz_lsis[TZ_MAX_LEAPS];
 
 /*
   Shows whenever we have found time zone tables during start-up.
   Used for avoiding of putting those tables to global table list
   for queries that use time zone info.
 */
-static bool time_zone_tables_exist = true;
+static bool time_zone_tables_exist = false;
 
 /*
   Names of tables (with their lengths) that are needed
@@ -1083,25 +1097,28 @@ static const LEX_CSTRING tz_tables_db_name = {STRING_WITH_LEN("mysql")};
 class Tz_names_entry {
  public:
   String name;
-  Time_zone *tz;
+  Time_zone *tz = nullptr;
 };
 
+static Tz_names_entry tz_ne_system = {
+    {STRING_WITH_LEN("SYSTEM"), &my_charset_latin1}, my_tz_SYSTEM};
+
 /*
   Prepare table list with time zone related tables from preallocated array.
 
   SYNOPSIS
     tz_init_table_list()
       tz_tabs         - pointer to preallocated array of MY_TZ_TABLES_COUNT
-                        TABLE_LIST objects
+                        Table_ref objects
 
   DESCRIPTION
-    This function prepares list of TABLE_LIST objects which can be used
+    This function prepares list of Table_ref objects which can be used
     for opening of time zone tables from preallocated array.
 */
 
-static void tz_init_table_list(TABLE_LIST *tz_tabs) {
+static void tz_init_table_list(Table_ref *tz_tabs) {
   for (int i = 0; i < MY_TZ_TABLES_COUNT; i++) {
-    new (&tz_tabs[i]) TABLE_LIST;
+    new (&tz_tabs[i]) Table_ref;
     tz_tabs[i].alias = tz_tabs[i].table_name = tz_tables_names[i].str;
     tz_tabs[i].table_name_length = tz_tables_names[i].length;
     tz_tabs[i].db = tz_tables_db_name.str;
@@ -1118,6 +1135,7 @@ static PSI_memory_key key_memory_tz_storage;
 
 #ifdef HAVE_PSI_INTERFACE
 static PSI_mutex_key key_tz_LOCK;
+static bool tz_inited = false;
 
 static PSI_mutex_info all_tz_mutexes[] = {
     {&key_tz_LOCK, "tz_LOCK", PSI_FLAG_SINGLETON, 0, PSI_DOCUMENT_ME}};
@@ -1126,10 +1144,13 @@ static PSI_memory_info all_tz_memory[] = {{&key_memory_tz_storage, "tz_storage",
                                            PSI_FLAG_ONLY_GLOBAL_STAT, 0,
                                            "Shared time zone data."}};
 
-class Tz_names_entry;
-
 static collation_unordered_map<std::string, Tz_names_entry *> tz_names{
     &my_charset_latin1, key_memory_tz_storage};
+[[maybe_unused]] static auto unused1 = [] {
+  tz_names.emplace(std::string{STRING_WITH_LEN("SYSTEM")}, &tz_ne_system);
+  return 0;
+}();
+
 static malloc_unordered_map<long, Time_zone_offset *> offset_tzs{
     key_memory_tz_storage};
 
@@ -1145,14 +1166,34 @@ static void init_tz_psi_keys(void) {
 }
 #endif /* HAVE_PSI_INTERFACE */
 
+/**
+  Minimal initalization which allows tz_find() to be called.
+  Only initializes psi keys, tz mem_root, and the tz_LOCK mutex.
+  Only system tz and offsets tz can actually be used.
+  Timezone table is not loaded (requires SE) and default
+  time zone variable is not assigned (requires options to be processed).
+ */
+bool my_tz_minimal_init() {
+#ifdef HAVE_PSI_INTERFACE
+  init_tz_psi_keys();
+#endif
+  /* Init all memory structures that require explicit destruction, only
+     the first time the function is called. The steps below are not idempotent
+     so redoing them will cause memory leaks. */
+  init_sql_alloc(key_memory_tz_storage, &tz_storage, 32 * 1024);
+  if (mysql_mutex_init(key_tz_LOCK, &tz_LOCK, MY_MUTEX_INIT_FAST)) return true;
+  tz_inited = true;
+  return false;
+}
+
 /*
-  Initialize time zone support infrastructure.
+  Fully initialize time zone support infrastructure.
 
   SYNOPSIS
-    my_tz_init()
-      thd            - current thread object
+    my_tz_full_init()
       default_tzname - default time zone or 0 if none.
-      bootstrap      - indicates whenever we are in bootstrap mode
+      opt_init       - indicates that server is running with
+  --initialize(-insecure)
 
   DESCRIPTION
     This function will init memory structures needed for time zone support,
@@ -1170,44 +1211,28 @@ static void init_tz_psi_keys(void) {
     0 - ok
     1 - Error
 */
-bool my_tz_init(THD *org_thd, const char *default_tzname, bool bootstrap) {
-  THD *thd;
-  TABLE_LIST tz_tables[1 + MY_TZ_TABLES_COUNT];
-  TABLE *table;
-  Tz_names_entry *tmp_tzname;
+bool my_tz_full_init(const char *default_tzname, bool opt_init) {
+  Table_ref tz_tables[1 + MY_TZ_TABLES_COUNT];
+  TABLE *table = nullptr;
   bool return_val = true;
-  LEX_CSTRING db = {STRING_WITH_LEN("mysql")};
-  int res;
+  const LEX_CSTRING db = {STRING_WITH_LEN("mysql")};
+  int res = 0;
   DBUG_TRACE;
 
-#ifdef HAVE_PSI_INTERFACE
-  init_tz_psi_keys();
-#endif
-
   /*
     To be able to run this from boot, we allocate a temporary THD
   */
-  if (!(thd = new THD)) return true;
-  thd->thread_stack = (char *)&thd;
+  THD thd_object;
+  THD *thd = &thd_object;
+
+  thd->thread_stack = pointer_cast<char *>(&thd);
   thd->store_globals();
 
-  /* Init all memory structures that require explicit destruction */
-  init_sql_alloc(key_memory_tz_storage, &tz_storage, 32 * 1024, 0);
-  mysql_mutex_init(key_tz_LOCK, &tz_LOCK, MY_MUTEX_INIT_FAST);
-  tz_inited = true;
+  assert(tz_names.contains("SYSTEM"));
 
-  /* Add 'SYSTEM' time zone to tz_names hash */
-  if (!(tmp_tzname = new (&tz_storage) Tz_names_entry())) {
-    LogErr(ERROR_LEVEL, ER_TZ_OOM_INITIALIZING_TIME_ZONES);
-    goto end_with_cleanup;
-  }
-  tmp_tzname->name.set(STRING_WITH_LEN("SYSTEM"), &my_charset_latin1);
-  tmp_tzname->tz = my_tz_SYSTEM;
-  tz_names.emplace("SYSTEM", tmp_tzname);
-
-  if (bootstrap) {
-    /* If we are in bootstrap mode we should not load time zone tables */
-    return_val = time_zone_tables_exist = false;
+  if (opt_init) {
+    /* If we are doing --initialize we should not load time zone tables */
+    return_val = false;
     goto end_with_setting_default_tz;
   }
 
@@ -1236,45 +1261,34 @@ bool my_tz_init(THD *org_thd, const char *default_tzname, bool bootstrap) {
     LogErr(WARNING_LEVEL, ER_TZ_CANT_OPEN_AND_LOCK_TIME_ZONE_TABLE,
            thd->get_stmt_da()->message_text());
     /* We will try emulate that everything is ok */
-    return_val = time_zone_tables_exist = false;
+    return_val = false;
     goto end_with_setting_default_tz;
   }
 
-  for (TABLE_LIST *tl = tz_tables; tl; tl = tl->next_global) {
+  for (Table_ref *tl = tz_tables; tl != nullptr; tl = tl->next_global) {
     /* Force close at the end of the function to free memory. */
     tl->table->invalidate_dict();
   }
 
-  /*
-    Now we are going to load leap seconds descriptions that are shared
-    between all time zones that use them. We are using index for getting
-    records in proper order. Since we share the same MEM_ROOT between
-    all time zones we just allocate enough memory for it first.
-  */
-  if (!(tz_lsis =
-            (LS_INFO *)tz_storage.Alloc(sizeof(LS_INFO) * TZ_MAX_LEAPS))) {
-    LogErr(ERROR_LEVEL, ER_TZ_OOM_LOADING_LEAP_SECOND_TABLE);
-    goto end_with_close;
-  }
-
   table = tz_tables[0].table;
 
-  if (table->file->ha_index_init(0, true)) goto end_with_close;
+  if (table->file->ha_index_init(0, true) != 0) goto end_with_close;
   table->use_all_columns();
 
   tz_leapcnt = 0;
 
   res = table->file->ha_index_first(table->record[0]);
 
-  while (!res) {
+  while (res == 0) {
     if (tz_leapcnt + 1 > TZ_MAX_LEAPS) {
       LogErr(ERROR_LEVEL, ER_TZ_TOO_MANY_LEAPS_IN_LEAP_SECOND_TABLE);
       table->file->ha_index_end();
       goto end_with_close;
     }
 
-    tz_lsis[tz_leapcnt].ls_trans = (my_time_t)table->field[0]->val_int();
-    tz_lsis[tz_leapcnt].ls_corr = (long)table->field[1]->val_int();
+    tz_lsis[tz_leapcnt].ls_trans =
+        static_cast<my_time_t>(table->field[0]->val_int());
+    tz_lsis[tz_leapcnt].ls_corr = static_cast<long>(table->field[1]->val_int());
 
     tz_leapcnt++;
 
@@ -1296,7 +1310,7 @@ bool my_tz_init(THD *org_thd, const char *default_tzname, bool bootstrap) {
   /*
     Loading of info about leap seconds succeeded
   */
-
+  time_zone_tables_exist = true;
   return_val = false;
 
 end_with_close:
@@ -1304,29 +1318,26 @@ end_with_close:
 
 end_with_setting_default_tz:
   /* If we have default time zone try to load it */
-  if (!return_val && default_tzname) {
-    String tmp_tzname2(default_tzname, &my_charset_latin1);
+  if (!return_val && default_tzname != nullptr) {
+    const String tmp_tzname2(default_tzname, &my_charset_latin1);
     /*
       Time zone tables may be open here, and my_tz_find() may open
       most of them once more, but this is OK for system tables open
       for READ.
     */
-    if (!(global_system_variables.time_zone = my_tz_find(thd, &tmp_tzname2))) {
+    global_system_variables.time_zone = my_tz_find(thd, &tmp_tzname2);
+    if (global_system_variables.time_zone == nullptr) {
       LogErr(ERROR_LEVEL, ER_TZ_UNKNOWN_OR_ILLEGAL_DEFAULT_TIME_ZONE,
              default_tzname);
       return_val = true;
     }
   }
 
-end_with_cleanup:
   /* if there were error free time zone describing structs */
   if (return_val) my_tz_free();
 
-  delete thd;
-  if (org_thd) org_thd->store_globals(); /* purecov: inspected */
-
-  default_tz =
-      default_tz_name ? global_system_variables.time_zone : my_tz_SYSTEM;
+  default_tz = default_tz_name != nullptr ? global_system_variables.time_zone
+                                          : my_tz_SYSTEM;
 
   return return_val;
 }
@@ -1339,15 +1350,15 @@ end_with_cleanup:
 */
 
 void my_tz_free() {
+  default_tz = nullptr;
+  global_system_variables.time_zone = my_tz_SYSTEM;
   if (tz_inited) {
-    default_tz = nullptr;
-    global_system_variables.time_zone = my_tz_SYSTEM;
-    tz_inited = false;
     mysql_mutex_destroy(&tz_LOCK);
-    offset_tzs.clear();
-    tz_names.clear();
-    free_root(&tz_storage, MYF(0));
   }
+  tz_inited = true;
+  offset_tzs.clear();
+  tz_names.clear();
+  tz_storage.Clear();
 }
 
 /*
@@ -1373,7 +1384,7 @@ void my_tz_free() {
 */
 
 static Time_zone *tz_load_from_open_tables(const String *tz_name,
-                                           TABLE_LIST *tz_tables) {
+                                           Table_ref *tz_tables) {
   TABLE *table = nullptr;
   TIME_ZONE_INFO *tz_info = nullptr;
   Tz_names_entry *tmp_tzname;
@@ -1578,7 +1589,7 @@ static Time_zone *tz_load_from_open_tables(const String *tz_name,
 
   /*
     We have to allow HA_ERR_KEY_NOT_FOUND because some time zones
-    for example UTC have no transitons.
+    for example UTC have no transitions.
   */
   if (res != HA_ERR_END_OF_FILE && res != HA_ERR_KEY_NOT_FOUND) {
     assert(res != HA_ERR_LOCK_WAIT_TIMEOUT && res != HA_ERR_LOCK_DEADLOCK);
@@ -1793,7 +1804,7 @@ Time_zone *my_tz_find(THD *thd, const String *name) {
 
   if (!name || name->is_empty()) return nullptr;
 
-  Mutex_guard guard(&tz_LOCK);
+  MUTEX_LOCK(guard, &tz_LOCK);
 
   int displacement;
   if (!str_to_offset(name->ptr(), name->length(), &displacement)) {
@@ -1802,8 +1813,6 @@ Time_zone *my_tz_find(THD *thd, const String *name) {
     if (it != offset_tzs.end())
       return it->second;
     else {
-      DBUG_PRINT("info", ("Creating new Time_zone_offset object"));
-
       auto new_tz = new (&tz_storage) Time_zone_offset(displacement);
       if (new_tz != nullptr) {
         offset_tzs.emplace(displacement, new_tz);
@@ -1814,6 +1823,8 @@ Time_zone *my_tz_find(THD *thd, const String *name) {
       }
     }
   } else {
+    DBUG_LOG("tz", "No displacement found. time_zone_tables_exist:"
+                       << time_zone_tables_exist);
     /*
       The time zone information is not a valid numeric displacement, so we
       assume it's a time zone *name*.
@@ -1822,7 +1833,7 @@ Time_zone *my_tz_find(THD *thd, const String *name) {
     if (it != tz_names.end())
       return it->second->tz;
     else if (time_zone_tables_exist) {
-      TABLE_LIST tz_tables[MY_TZ_TABLES_COUNT];
+      Table_ref tz_tables[MY_TZ_TABLES_COUNT];
 
       tz_init_table_list(tz_tables);
       init_mdl_requests(tz_tables);
