@@ -1,18 +1,19 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2023, Oracle and/or its affiliates.
+Copyright (c) 1996, 2025, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -871,6 +872,28 @@ bool dict_foreign_set_validate(const dict_table_t &table) {
   return (dict_foreign_set_validate(table.foreign_set) &&
           dict_foreign_set_validate(table.referenced_set));
 }
+
+bool dict_foreign_t::is_fts_col_affected() const {
+  /* The check is skipped:
+     - if the table has no full text index defined.
+     - if it is a self referential foreign constaint. This is because
+       in the context of cascading DML operation, only the referenced
+       table is relevant for the validation even if the current table
+       has FTS index.*/
+  if (!foreign_table->fts || foreign_table == referenced_table) {
+    return false;
+  }
+
+  for (ulint i = 0; i < n_fields; i++) {
+    const dict_col_t *col = foreign_index->get_col(i);
+    if (dict_table_is_fts_column(foreign_table->fts->indexes,
+                                 dict_col_get_no(col),
+                                 col->is_virtual()) != ULINT_UNDEFINED) {
+      return true;
+    }
+  }
+  return false;
+}
 #endif /* !UNIV_HOTBACKUP */
 
 std::ostream &operator<<(std::ostream &out, const dict_foreign_t &foreign) {
@@ -895,9 +918,44 @@ page_size_t dict_index_t::get_page_size() const {
   return (dict_table_page_size(table));
 }
 
+#ifndef UNIV_HOTBACKUP
+
+void dict_index_t::assert_stats_initialized() const {
+  UNIV_MEM_ASSERT_RW_ABORT(stat_n_diff_key_vals,
+                           n_uniq * sizeof(stat_n_diff_key_vals[0]));
+
+  UNIV_MEM_ASSERT_RW_ABORT(stat_n_sample_sizes,
+                           n_uniq * sizeof(stat_n_sample_sizes[0]));
+
+  UNIV_MEM_ASSERT_RW_ABORT(stat_n_non_null_key_vals,
+                           n_uniq * sizeof(stat_n_non_null_key_vals[0]));
+
+  UNIV_MEM_ASSERT_RW_ABORT(&stat_index_size, sizeof(stat_index_size));
+
+  UNIV_MEM_ASSERT_RW_ABORT(&stat_n_leaf_pages, sizeof(stat_n_leaf_pages));
+}
+
+#endif /* !UNIV_HOTBACKUP */
+
 bool dict_table_t::has_pk() const {
   const dict_index_t *first = first_index();
   const size_t len = strlen(innobase_index_reserve_name);
   const int cmp = strncmp(innobase_index_reserve_name, first->name(), len);
   return cmp != 0;
 }
+
+#ifndef UNIV_HOTBACKUP
+void dict_index_stats_t::assert_initialized() const {
+  UNIV_MEM_ASSERT_RW_ABORT(n_diff_key_vals,
+                           n_uniq * sizeof(n_diff_key_vals[0]));
+
+  UNIV_MEM_ASSERT_RW_ABORT(n_sample_sizes, n_uniq * sizeof(n_sample_sizes[0]));
+
+  UNIV_MEM_ASSERT_RW_ABORT(n_non_null_key_vals,
+                           n_uniq * sizeof(n_non_null_key_vals[0]));
+
+  UNIV_MEM_ASSERT_RW_ABORT(&index_size, sizeof(index_size));
+
+  UNIV_MEM_ASSERT_RW_ABORT(&n_leaf_pages, sizeof(n_leaf_pages));
+}
+#endif /* !UNIV_HOTBACKUP */
