@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -926,6 +927,7 @@ rec_t *rtr_page_split_and_insert(
   }
 
 func_start:
+  ut_ad(*heap != nullptr);
   ut_ad(tuple->m_heap != *heap);
   mem_heap_empty(*heap);
   *offsets = nullptr;
@@ -1159,10 +1161,9 @@ after_insert:
   page_zip = buf_block_get_page_zip(root_block);
   page_set_ssn_id(root_block, page_zip, next_ssn, mtr);
 
-  /* Insert fit on the page: update the free bits for the
-  left and right pages in the same mtr */
-
-  if (page_is_leaf(page)) {
+  /* Insert fit on the page: update the free bits (for non-temporary
+  tablespaces) for the left and right pages in the same mtr */
+  if (page_is_leaf(page) && !cursor->index->table->is_temporary()) {
     ibuf_update_free_bits_for_two_pages_low(block, new_block, mtr);
   }
 
