@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2013, 2021, Oracle and/or its affiliates.
+Copyright (c) 2013, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -55,9 +56,9 @@ struct OSTrackMutex {
   ~OSTrackMutex() UNIV_NOTHROW { ut_ad(!m_destroy_at_exit || !m_locked); }
 
   /** Initialise the mutex.
-  @param[in]	id              Mutex ID
-  @param[in]	filename	File where mutex was created
-  @param[in]	line		Line in filename */
+  @param[in]    id              Mutex ID
+  @param[in]    filename        File where mutex was created
+  @param[in]    line            Line in filename */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
     ut_ad(m_freed);
     ut_ad(!m_locked);
@@ -91,12 +92,14 @@ struct OSTrackMutex {
   }
 
   /** Acquire the mutex.
-  @param[in]	max_spins	max number of spins
-  @param[in]	max_delay	max delay per spin
-  @param[in]	filename	from where called
-  @param[in]	line		within filename */
-  void enter(uint32_t max_spins, uint32_t max_delay, const char *filename,
-             uint32_t line) UNIV_NOTHROW {
+  @param[in]    max_spins       max number of spins
+  @param[in]    max_delay       max delay per spin
+  @param[in]    filename        from where called
+  @param[in]    line            within filename */
+  void enter(uint32_t max_spins [[maybe_unused]],
+             uint32_t max_delay [[maybe_unused]],
+             const char *filename [[maybe_unused]],
+             uint32_t line [[maybe_unused]]) UNIV_NOTHROW {
     ut_ad(innodb_calling_exit || !m_freed);
 
     m_mutex.enter();
@@ -105,6 +108,9 @@ struct OSTrackMutex {
     ut_d(m_locked = true);
   }
 
+  void lock() { enter(); }
+  void unlock() { exit(); }
+
   /** @return true if locking succeeded */
   bool try_lock() UNIV_NOTHROW {
     ut_ad(innodb_calling_exit || !m_freed);
@@ -161,35 +167,60 @@ template <template <typename> class Policy = NoPolicy>
 struct TTASFutexMutex {
   typedef Policy<TTASFutexMutex> MutexPolicy;
 
-  TTASFutexMutex() UNIV_NOTHROW : m_lock_word(MUTEX_STATE_UNLOCKED) {
-    /* Check that lock_word is aligned. */
-    ut_ad(!((ulint)&m_lock_word % sizeof(ulint)));
+  /** The type of second argument to syscall(SYS_futex, uint32_t *uaddr,...)*/
+  using futex_word_t = uint32_t;
+  /** Mutex states. */
+  enum class mutex_state_t : futex_word_t {
+    /** Mutex is free */
+    UNLOCKED = 0,
+
+    /** Mutex is acquired by some thread. */
+    LOCKED = 1,
+
+    /** Mutex is contended and there are threads waiting on the lock. */
+    LOCKED_WITH_WAITERS = 2
+  };
+  using lock_word_t = mutex_state_t;
+
+  TTASFutexMutex() UNIV_NOTHROW : m_lock_word(mutex_state_t::UNLOCKED) {
+    /* The futex API operates on uint32_t futex words, aligned to 4 byte
+    boundaries. OTOH we want the luxury of accessing it via std::atomic<>.
+    Thus we need to verify that std::atomic doesn't add any extra fluff,
+    and is properly aligned */
+    using m_lock_word_t = decltype(m_lock_word);
+    static_assert(m_lock_word_t::is_always_lock_free);
+    static_assert(sizeof(m_lock_word_t) == sizeof(futex_word_t));
+#ifdef UNIV_DEBUG
+    const auto addr = reinterpret_cast<std::uintptr_t>(&m_lock_word);
+    ut_a(addr % alignof(m_lock_word_t) == 0);
+    ut_a(addr % 4 == 0);
+#endif
   }
 
-  ~TTASFutexMutex() { ut_a(m_lock_word == MUTEX_STATE_UNLOCKED); }
+  ~TTASFutexMutex() { ut_a(m_lock_word == mutex_state_t::UNLOCKED); }
 
   /** Called when the mutex is "created". Note: Not from the constructor
   but when the mutex is initialised.
-  @param[in]	id		Mutex ID
-  @param[in]	filename	File where mutex was created
-  @param[in]	line		Line in filename */
+  @param[in]    id              Mutex ID
+  @param[in]    filename        File where mutex was created
+  @param[in]    line            Line in filename */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
-    ut_a(m_lock_word == MUTEX_STATE_UNLOCKED);
+    ut_a(m_lock_word == mutex_state_t::UNLOCKED);
     m_policy.init(*this, id, filename, line);
   }
 
   /** Destroy the mutex. */
   void destroy() UNIV_NOTHROW {
     /* The destructor can be called at shutdown. */
-    ut_a(m_lock_word == MUTEX_STATE_UNLOCKED);
+    ut_a(m_lock_word == mutex_state_t::UNLOCKED);
     m_policy.destroy();
   }
 
   /** Acquire the mutex.
-  @param[in]	max_spins	max number of spins
-  @param[in]	max_delay	max delay per spin
-  @param[in]	filename	from where called
-  @param[in]	line		within filename */
+  @param[in]    max_spins       max number of spins
+  @param[in]    max_delay       max delay per spin
+  @param[in]    filename        from where called
+  @param[in]    line            within filename */
   void enter(uint32_t max_spins, uint32_t max_delay, const char *filename,
              uint32_t line) UNIV_NOTHROW {
     uint32_t n_spins;
@@ -202,18 +233,10 @@ struct TTASFutexMutex {
     by then. In this case the thread can assume it
     was granted the mutex. */
 
-    uint32_t n_waits;
-
-    if (lock != MUTEX_STATE_UNLOCKED) {
-      if (lock != MUTEX_STATE_LOCKED || !set_waiters()) {
-        n_waits = wait();
-      } else {
-        n_waits = 0;
-      }
-
-    } else {
-      n_waits = 0;
-    }
+    const uint32_t n_waits = (lock == mutex_state_t::LOCKED_WITH_WAITERS ||
+                              (lock == mutex_state_t::LOCKED && !set_waiters()))
+                                 ? wait()
+                                 : 0;
 
     m_policy.add(n_spins, n_waits);
   }
@@ -226,10 +249,10 @@ struct TTASFutexMutex {
 
     std::atomic_thread_fence(std::memory_order_acquire);
 
-    if (state() == MUTEX_STATE_WAITERS) {
-      m_lock_word = MUTEX_STATE_UNLOCKED;
+    if (state() == mutex_state_t::LOCKED_WITH_WAITERS) {
+      m_lock_word = mutex_state_t::UNLOCKED;
 
-    } else if (unlock() == MUTEX_STATE_LOCKED) {
+    } else if (unlock() == mutex_state_t::LOCKED) {
       /* No threads waiting, no need to signal a wakeup. */
       return;
     }
@@ -240,21 +263,21 @@ struct TTASFutexMutex {
   /** Try and lock the mutex.
   @return the old state of the mutex */
   lock_word_t trylock() UNIV_NOTHROW {
-    lock_word_t unlocked = MUTEX_STATE_UNLOCKED;
-    m_lock_word.compare_exchange_strong(unlocked, MUTEX_STATE_LOCKED);
+    lock_word_t unlocked = mutex_state_t::UNLOCKED;
+    m_lock_word.compare_exchange_strong(unlocked, mutex_state_t::LOCKED);
     return unlocked;
   }
 
   /** Try and lock the mutex.
   @return true if successful */
   bool try_lock() UNIV_NOTHROW {
-    lock_word_t unlocked = MUTEX_STATE_UNLOCKED;
-    return m_lock_word.compare_exchange_strong(unlocked, MUTEX_STATE_LOCKED);
+    lock_word_t unlocked = mutex_state_t::UNLOCKED;
+    return m_lock_word.compare_exchange_strong(unlocked, mutex_state_t::LOCKED);
   }
 
   /** @return true if mutex is unlocked */
   bool is_locked() const UNIV_NOTHROW {
-    return (state() != MUTEX_STATE_UNLOCKED);
+    return (state() != mutex_state_t::UNLOCKED);
   }
 
 #ifdef UNIV_DEBUG
@@ -277,13 +300,14 @@ struct TTASFutexMutex {
   /** Release the mutex.
   @return the old state of the mutex */
   lock_word_t unlock() UNIV_NOTHROW {
-    return m_lock_word.exchange(MUTEX_STATE_UNLOCKED);
+    return m_lock_word.exchange(mutex_state_t::UNLOCKED);
   }
 
   /** Note that there are threads waiting and need to be woken up.
-  @return true if state was MUTEX_STATE_UNLOCKED (ie. granted) */
+  @return true if state was mutex_state_t::UNLOCKED (ie. granted) */
   bool set_waiters() UNIV_NOTHROW {
-    return m_lock_word.exchange(MUTEX_STATE_WAITERS) == MUTEX_STATE_UNLOCKED;
+    return m_lock_word.exchange(mutex_state_t::LOCKED_WITH_WAITERS) ==
+           mutex_state_t::UNLOCKED;
   }
 
   /** Wait if the lock is contended.
@@ -297,8 +321,8 @@ struct TTASFutexMutex {
     do {
       ++n_waits;
 
-      syscall(SYS_futex, &m_lock_word, FUTEX_WAIT_PRIVATE, MUTEX_STATE_WAITERS,
-              0, 0, 0);
+      syscall(SYS_futex, &m_lock_word, FUTEX_WAIT_PRIVATE,
+              mutex_state_t::LOCKED_WITH_WAITERS, 0, 0, 0);
 
       // Since we are retrying the operation the return
       // value doesn't matter.
@@ -314,9 +338,9 @@ struct TTASFutexMutex {
   }
 
   /** Poll waiting for mutex to be unlocked.
-  @param[in]	max_spins	max spins
-  @param[in]	max_delay	max delay per spin
-  @param[out]	n_spins		retries before acquire
+  @param[in]    max_spins       max spins
+  @param[in]    max_delay       max delay per spin
+  @param[out]   n_spins         retries before acquire
   @return value of lock word before locking. */
   lock_word_t ttas(uint32_t max_spins, uint32_t max_delay,
                    uint32_t &n_spins) UNIV_NOTHROW {
@@ -326,13 +350,13 @@ struct TTASFutexMutex {
       if (!is_locked()) {
         lock_word_t lock = trylock();
 
-        if (lock == MUTEX_STATE_UNLOCKED) {
+        if (lock == mutex_state_t::UNLOCKED) {
           /* Lock successful */
           return (lock);
         }
       }
 
-      ut_delay(ut_rnd_interval(0, max_delay));
+      ut_delay(ut::random_from_interval_fast(0, max_delay));
     }
 
     return (trylock());
@@ -342,7 +366,7 @@ struct TTASFutexMutex {
   /** Policy data */
   MutexPolicy m_policy;
 
-  std::atomic<lock_word_t> m_lock_word;
+  alignas(4) std::atomic<lock_word_t> m_lock_word;
 };
 
 #endif /* HAVE_IB_LINUX_FUTEX */
@@ -351,34 +375,38 @@ template <template <typename> class Policy = NoPolicy>
 struct TTASEventMutex {
   typedef Policy<TTASEventMutex> MutexPolicy;
 
-  TTASEventMutex() UNIV_NOTHROW : m_lock_word(MUTEX_STATE_UNLOCKED),
-                                  m_waiters(),
-                                  m_event() {
-    /* Check that lock_word is aligned. */
-    ut_ad(!((ulint)&m_lock_word % sizeof(ulint)));
+  TTASEventMutex() UNIV_NOTHROW {
+    /* Check that m_owner is aligned. */
+    using m_owner_t = decltype(m_owner);
+    ut_ad(reinterpret_cast<std::uintptr_t>(&m_owner) % alignof(m_owner_t) == 0);
+    static_assert(m_owner_t::is_always_lock_free);
   }
 
-  ~TTASEventMutex() UNIV_NOTHROW { ut_ad(!m_lock_word.load()); }
+  ~TTASEventMutex() UNIV_NOTHROW { ut_ad(!is_locked()); }
+
+  /** If the lock is locked, returns the current owner of the lock, otherwise
+  returns the default std::thread::id{} */
+  std::thread::id peek_owner() const UNIV_NOTHROW { return m_owner.load(); }
 
   /** Called when the mutex is "created". Note: Not from the constructor
   but when the mutex is initialised.
-  @param[in]	id		Mutex ID
-  @param[in]	filename	File where mutex was created
-  @param[in]	line		Line in filename */
+  @param[in]    id              Mutex ID
+  @param[in]    filename        File where mutex was created
+  @param[in]    line            Line in filename */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
     ut_a(m_event == nullptr);
-    ut_a(!m_lock_word.load(std::memory_order_relaxed));
+    ut_a(!is_locked());
 
     m_event = os_event_create();
 
     m_policy.init(*this, id, filename, line);
   }
 
-  /** This is the real desctructor. This mutex can be created in BSS and
-  its desctructor will be called on exit(). We can't call
+  /** This is the real destructor. This mutex can be created in BSS and
+  its destructor will be called on exit(). We can't call
   os_event_destroy() at that stage. */
   void destroy() UNIV_NOTHROW {
-    ut_ad(!m_lock_word.load(std::memory_order_relaxed));
+    ut_ad(!is_locked());
 
     /* We have to free the event before InnoDB shuts down. */
     os_event_destroy(m_event);
@@ -390,13 +418,14 @@ struct TTASEventMutex {
   /** Try and lock the mutex. Note: POSIX returns 0 on success.
   @return true on success */
   bool try_lock() UNIV_NOTHROW {
-    bool expected = false;
-    return (m_lock_word.compare_exchange_strong(expected, true));
+    auto expected = std::thread::id{};
+    return m_owner.compare_exchange_strong(expected,
+                                           std::this_thread::get_id());
   }
 
   /** Release the mutex. */
   void exit() UNIV_NOTHROW {
-    m_lock_word.store(false);
+    m_owner.store(std::thread::id{});
 
     if (m_waiters.load()) {
       signal();
@@ -404,10 +433,10 @@ struct TTASEventMutex {
   }
 
   /** Acquire the mutex.
-  @param[in]	max_spins	max number of spins
-  @param[in]	max_delay	max delay per spin
-  @param[in]	filename	from where called
-  @param[in]	line		within filename */
+  @param[in]    max_spins       max number of spins
+  @param[in]    max_delay       max delay per spin
+  @param[in]    filename        from where called
+  @param[in]    line            within filename */
   void enter(uint32_t max_spins, uint32_t max_delay, const char *filename,
              uint32_t line) UNIV_NOTHROW {
     if (!try_lock()) {
@@ -415,26 +444,19 @@ struct TTASEventMutex {
     }
   }
 
-  /** @return true if locked. */
-  bool state() const UNIV_NOTHROW {
-    return (m_lock_word.load(std::memory_order_relaxed));
-  }
-
   /** The event that the mutex will wait in sync0arr.cc
   @return even instance */
   os_event_t event() UNIV_NOTHROW { return (m_event); }
 
   /** @return true if locked by some thread */
   bool is_locked() const UNIV_NOTHROW {
-    return (m_lock_word.load(std::memory_order_relaxed));
+    return peek_owner() != std::thread::id{};
   }
 
-#ifdef UNIV_DEBUG
   /** @return true if the calling thread owns the mutex. */
   bool is_owned() const UNIV_NOTHROW {
-    return (is_locked() && m_policy.is_owned());
+    return peek_owner() == std::this_thread::get_id();
   }
-#endif /* UNIV_DEBUG */
 
   /** @return non-const version of the policy */
   MutexPolicy &policy() UNIV_NOTHROW { return (m_policy); }
@@ -444,16 +466,16 @@ struct TTASEventMutex {
 
  private:
   /** Wait in the sync array.
-  @param[in]	filename	from where it was called
-  @param[in]	line		line number in file
-  @param[in]	spin		retry this many times again
+  @param[in]    filename        from where it was called
+  @param[in]    line            line number in file
+  @param[in]    spin            retry this many times again
   @return true if the mutex acquisition was successful. */
   bool wait(const char *filename, uint32_t line, uint32_t spin) UNIV_NOTHROW;
 
   /** Spin and wait for the mutex to become free.
-  @param[in]	max_spins	max spins
-  @param[in]	max_delay	max delay per spin
-  @param[in,out]	n_spins		spin start index
+  @param[in]    max_spins       max spins
+  @param[in]    max_delay       max delay per spin
+  @param[in,out]        n_spins         spin start index
   @return true if unlocked */
   bool is_free(uint32_t max_spins, uint32_t max_delay,
                uint32_t &n_spins) const UNIV_NOTHROW {
@@ -470,7 +492,7 @@ struct TTASEventMutex {
         return (true);
       }
 
-      ut_delay(ut_rnd_interval(0, max_delay));
+      ut_delay(ut::random_from_interval_fast(0, max_delay));
 
       ++n_spins;
 
@@ -480,10 +502,10 @@ struct TTASEventMutex {
   }
 
   /** Spin while trying to acquire the mutex
-  @param[in]	max_spins	max number of spins
-  @param[in]	max_delay	max delay per spin
-  @param[in]	filename	from where called
-  @param[in]	line		within filename */
+  @param[in]    max_spins       max number of spins
+  @param[in]    max_delay       max delay per spin
+  @param[in]    filename        from where called
+  @param[in]    line            within filename */
   void spin_and_try_lock(uint32_t max_spins, uint32_t max_delay,
                          const char *filename, uint32_t line) UNIV_NOTHROW {
     uint32_t n_spins = 0;
@@ -546,19 +568,19 @@ struct TTASEventMutex {
   TTASEventMutex &operator=(TTASEventMutex &&) = delete;
   TTASEventMutex &operator=(const TTASEventMutex &) = delete;
 
-  /** lock_word is the target of the atomic test-and-set instruction
-  when atomic operations are enabled. */
-  std::atomic_bool m_lock_word;
-
-  /** true if there are (or may be) threads waiting
-  in the global wait array for this mutex to be released. */
-  std::atomic_bool m_waiters;
+  /** Set to owner's thread's id when locked, and reset to the default
+  std::thread::id{} when unlocked. */
+  std::atomic<std::thread::id> m_owner{std::thread::id{}};
 
   /** Used by sync0arr.cc for the wait queue */
-  os_event_t m_event;
+  os_event_t m_event{};
 
   /** Policy data */
   MutexPolicy m_policy;
+
+  /** true if there are (or may be) threads waiting
+  in the global wait array for this mutex to be released. */
+  std::atomic_bool m_waiters{false};
 };
 
 /** Mutex interface for all policy mutexes. This class handles the interfacing
@@ -574,7 +596,7 @@ struct PolicyMutex {
 #endif /* UNIV_PFS_MUTEX */
   }
 
-  ~PolicyMutex() {}
+  ~PolicyMutex() = default;
 
   /** @return non-const version of the policy */
   Policy &policy() UNIV_NOTHROW { return (m_impl.policy()); }
@@ -594,10 +616,10 @@ struct PolicyMutex {
   }
 
   /** Acquire the mutex.
-  @param n_spins	max number of spins
-  @param n_delay	max delay per spin
-  @param name	filename where locked
-  @param line	line number where locked */
+  @param n_spins        max number of spins
+  @param n_delay        max delay per spin
+  @param name   filename where locked
+  @param line   line number where locked */
   void enter(uint32_t n_spins, uint32_t n_delay, const char *name,
              uint32_t line) UNIV_NOTHROW {
 #ifdef UNIV_PFS_MUTEX
@@ -621,8 +643,8 @@ struct PolicyMutex {
   }
 
   /** Try and lock the mutex, return 0 on SUCCESS and 1 otherwise.
-  @param name	filename where locked
-  @param line	line number where locked */
+  @param name   filename where locked
+  @param line   line number where locked */
   int trylock(const char *name, uint32_t line) UNIV_NOTHROW {
 #ifdef UNIV_PFS_MUTEX
     /* Note: locker is really an alias for state. That's why
@@ -636,7 +658,7 @@ struct PolicyMutex {
 
     /* There is a subtlety here, we check the mutex ordering
     after locking here. This is only done to avoid add and
-    then remove if the trylock was unsuccesful. */
+    then remove if the trylock was unsuccessful. */
 
     int ret = m_impl.try_lock() ? 0 : 1;
 
@@ -661,9 +683,9 @@ struct PolicyMutex {
   /**
   Initialise the mutex.
 
-  @param[in]	id              Mutex ID
-  @param[in]	filename	file where created
-  @param[in]	line		line number in file where created */
+  @param[in]    id              Mutex ID
+  @param[in]    filename        file where created
+  @param[in]    line            line number in file where created */
   void init(latch_id_t id, const char *filename, uint32_t line) UNIV_NOTHROW {
 #ifdef UNIV_PFS_MUTEX
     pfs_add(sync_latch_get_pfs_key(id));
