@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
 Portions of this file contain modifications contributed and copyrighted by
 Google, Inc. Those modifications are gratefully acknowledged and are described
@@ -12,12 +12,13 @@ This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -69,8 +70,11 @@ enum class Log_Type : uint32_t {
   /** Alter Unencrypt a tablespace */
   ALTER_UNENCRYPT_TABLESPACE_LOG,
 
+  /** For Atomic DROP/CREATE SCHEMA implementation */
+  DELETE_SCHEMA_DIRECTORY_LOG,
+
   /** Biggest log type */
-  BIGGEST_LOG = ALTER_UNENCRYPT_TABLESPACE_LOG
+  BIGGEST_LOG = DELETE_SCHEMA_DIRECTORY_LOG
 };
 
 /** DDL log record */
@@ -483,6 +487,15 @@ class Log_DDL {
   @return DB_SUCCESS or error */
   dberr_t write_remove_cache_log(trx_t *trx, dict_table_t *table);
 
+  /** Write a DELETE_SCHEMA_DIRECTORY_LOG
+  @param[in,out]    trx                   transaction
+  @param[in]        schema_directory_path path to the database directory
+  @param[in]        is_drop_schema        is it DROP SCHEMA query
+  @return           DB_SUCCESS or error */
+  dberr_t write_delete_schema_directory_log(trx_t *trx,
+                                            const char *schema_directory_path,
+                                            const bool is_drop_schema);
+
   /** Replay DDL log record
   @param[in,out]        record  DDL log record
   return DB_SUCCESS or error */
@@ -618,6 +631,21 @@ class Log_DDL {
   @param[in]    table_name      table name */
   void replay_remove_cache_log(table_id_t table_id, const char *table_name);
 
+  /** Insert a DELETE_SCHEMA_DIRECTORY_LOG
+  @param[in,out]  trx                   transaction
+  @param[in]      id                    record id
+  @param[in]      thread_id             thread id
+  @param[in]      schema_directory_path path to the schema directory
+  @return         DB_SUCCESS or not */
+  dberr_t insert_delete_schema_directory_log(trx_t *trx, uint64_t id,
+                                             ulint thread_id,
+                                             const char *schema_directory_path);
+
+  /** Replay a DELETE_SCHEMA_DIRECTORY_LOG
+  @param  schema_directory_path path to the schema directory
+  @return DB_SUCCESS or not */
+  dberr_t replay_delete_schema_directory_log(const char *schema_directory_path);
+
   /** Delete log record by id
   @param[in]    trx             transaction instance
   @param[in]    id              log id
