@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -41,6 +42,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #ifndef os0numa_h
 #define os0numa_h
 
+#include "mysql/components/library_mysys/my_system.h"  // my_num_vcpus
 #include "univ.i"
 
 #ifdef HAVE_LIBNUMA
@@ -147,11 +149,7 @@ inline int os_numa_num_configured_cpus() {
 
   return (n_cpus);
 #else
-  /* Consider
-  boost::thread::hardware_concurrency() or
-  std::thread::hardware_concurrency() (C++11) */
-  ut_error;
-  return (-1);
+  return my_num_vcpus();
 #endif
 }
 
