@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2021, Oracle and/or its affiliates.
+Copyright (c) 1997, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -56,16 +57,16 @@ class ReadView {
 
     /**
     Destructor */
-    ~ids_t() { UT_DELETE_ARRAY(m_ptr); }
+    ~ids_t() { ut::delete_arr(m_ptr); }
 
     /** Try and increase the size of the array. Old elements are copied across.
     It is a no-op if n is < current size.
-    @param n 		Make space for n elements */
+    @param n            Make space for n elements */
     void reserve(ulint n);
 
     /**
     Resize the array, sets the current element count.
-    @param n		new size of the array, in elements */
+    @param n            new size of the array, in elements */
     void resize(ulint n) {
       ut_ad(n <= capacity());
 
@@ -83,8 +84,8 @@ class ReadView {
     /**
     Copy and overwrite the current array contents
 
-    @param start		Source array
-    @param end		Pointer to end of array */
+    @param start                Source array
+    @param end          Pointer to end of array */
     void assign(const value_type *start, const value_type *end);
 
     /**
@@ -110,7 +111,7 @@ class ReadView {
 
     /**
     Append a value to the array.
-    @param value		the value to append */
+    @param value                the value to append */
     void push_back(value_type value);
 
     /**
@@ -151,16 +152,16 @@ class ReadView {
   ReadView();
   ~ReadView();
   /** Check whether transaction id is valid.
-  @param[in]	id		transaction id to check
-  @param[in]	name		table name */
+  @param[in]    id              transaction id to check
+  @param[in]    name            table name */
   static void check_trx_id_sanity(trx_id_t id, const table_name_t &name);
 
   /** Check whether the changes by id are visible.
-  @param[in]	id	transaction id to check against the view
-  @param[in]	name	table name
+  @param[in]    id      transaction id to check against the view
+  @param[in]    name    table name
   @return whether the view sees the modifications of id. */
-  bool changes_visible(trx_id_t id, const table_name_t &name) const
-      MY_ATTRIBUTE((warn_unused_result)) {
+  [[nodiscard]] bool changes_visible(trx_id_t id,
+                                     const table_name_t &name) const {
     ut_ad(id > 0);
 
     if (id < m_up_limit_id || id == m_creator_trx_id) {
@@ -182,7 +183,7 @@ class ReadView {
   }
 
   /**
-  @param id		transaction to check
+  @param id             transaction to check
   @return true if view sees transaction id */
   bool sees(trx_id_t id) const { return (id < m_up_limit_id); }
 
@@ -195,11 +196,11 @@ class ReadView {
 
   /**
   @return true if the view is closed */
-  bool is_closed() const { return (m_closed); }
+  bool is_closed() const { return m_closed.load(); }
 
   /**
   Write the limits to the file.
-  @param file		file to write to */
+  @param file           file to write to */
   void print_limits(FILE *file) const {
     fprintf(file,
             "Trx read view will not see trx with"
@@ -207,17 +208,6 @@ class ReadView {
             m_low_limit_id, m_up_limit_id);
   }
 
-  /** Check and reduce low limit number for read view. Used to
-  block purge till GTID is persisted on disk table.
-  @param[in]	trx_no	transaction number to check with */
-  void reduce_low_limit(trx_id_t trx_no) {
-    if (trx_no < m_low_limit_no) {
-      /* Save low limit number set for Read View for MVCC. */
-      ut_d(m_view_low_limit_no = m_low_limit_no);
-      m_low_limit_no = trx_no;
-    }
-  }
-
   /**
   @return the low limit no */
   trx_id_t low_limit_no() const { return (m_low_limit_no); }
@@ -232,11 +222,7 @@ class ReadView {
 
 #ifdef UNIV_DEBUG
   /**
-  @return the view low limit number */
-  trx_id_t view_low_limit_no() const { return (m_view_low_limit_no); }
-
-  /**
-  @param rhs		view to compare with
+  @param rhs            view to compare with
   @return truen if this view is less than or equal rhs */
   bool le(const ReadView *rhs) const {
     return (m_low_limit_no <= rhs->m_low_limit_no);
@@ -250,12 +236,12 @@ class ReadView {
   /**
   Opens a read view where exactly the transactions serialized before this
   point in time are seen in the view.
-  @param id		Creator transaction id */
+  @param id             Creator transaction id */
   inline void prepare(trx_id_t id);
 
   /**
   Copy state from another view. Must call copy_complete() to finish.
-  @param other		view to copy from */
+  @param other          view to copy from */
   inline void copy_prepare(const ReadView &other);
 
   /**
@@ -300,16 +286,8 @@ class ReadView {
   they can be removed in purge if not needed by other views */
   trx_id_t m_low_limit_no;
 
-#ifdef UNIV_DEBUG
-  /** The low limit number up to which read views don't need to access
-  undo log records for MVCC. This could be higher than m_low_limit_no
-  if purge is blocked for GTID persistence. Currently used for debug
-  variable INNODB_PURGE_VIEW_TRX_ID_AGE. */
-  trx_id_t m_view_low_limit_no;
-#endif /* UNIV_DEBUG */
-
   /** AC-NL-RO transaction view that has been "closed". */
-  bool m_closed;
+  std::atomic_bool m_closed;
 
   typedef UT_LIST_NODE_T(ReadView) node_t;
 
