@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2024, Oracle and/or its affiliates.
+Copyright (c) 1994, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -176,24 +176,12 @@ static inline void rec_old_set_versioned(rec_t *rec, bool flag);
 [[nodiscard]] static inline bool rec_get_node_ptr_flag(
     const rec_t *rec); /*!< in: physical record */
 
-/** The following function is used to get the order number of an old-style
-record in the heap of the index page.
-@param[in]      rec     physical record
-@return heap order number */
-[[nodiscard]] static inline ulint rec_get_heap_no_old(const rec_t *rec);
-
 /** The following function is used to set the heap number field in an old-style
 record.
 @param[in]      rec     physical record
 @param[in]      heap_no the heap number */
 static inline void rec_set_heap_no_old(rec_t *rec, ulint heap_no);
 
-/** The following function is used to get the order number of a new-style
-record in the heap of the index page.
-@param[in]      rec     physical record
-@return heap order number */
-[[nodiscard]] static inline ulint rec_get_heap_no_new(const rec_t *rec);
-
 /** The following function is used to set the heap number field in a new-style
 record.
 @param[in,out]  rec     physical record
@@ -409,11 +397,9 @@ static inline rec_t *rec_copy(void *buf, const rec_t *rec,
 @param[out]     extra           extra size
 @param[in]      rec_version     row version of record
 @return total size */
-[[nodiscard]] ulint rec_get_serialize_size(const dict_index_t *index,
-                                           const dfield_t *fields,
-                                           ulint n_fields,
-                                           const dtuple_t *v_entry,
-                                           ulint *extra, uint8_t rec_version);
+[[nodiscard]] ulint rec_get_serialize_size(
+    const dict_index_t *index, const dfield_t *fields, ulint n_fields,
+    const dtuple_t *v_entry, ulint *extra, row_version_t rec_version);
 
 /** Determine the offset to each field in temporary file.
 @param[in]      rec     temporary file record
@@ -433,7 +419,7 @@ void rec_deserialize_init_offsets(const rec_t *rec, const dict_index_t *index,
 @see rec_deserialize_init_offsets() */
 void rec_serialize_dtuple(rec_t *rec, const dict_index_t *index,
                           const dfield_t *fields, ulint n_fields,
-                          const dtuple_t *v_entry, uint8_t rec_version);
+                          const dtuple_t *v_entry, row_version_t rec_version);
 
 /** Copies the first n fields of a physical record to a new physical record in
 a buffer.
@@ -532,13 +518,15 @@ static inline uint8_t rec_get_n_fields_length(ulint n_fields);
 This is only needed for table after instant ADD/DROP COLUMN.
 @param[in,out]  rec             leaf page record
 @param[in]      version         row version */
-static inline void rec_set_instant_row_version_new(rec_t *rec, uint8_t version);
+static inline void rec_set_instant_row_version_new(rec_t *rec,
+                                                   row_version_t version);
 
 /** Set the row version on one old style leaf page record.
 This is only needed for table after instant ADD/DROP COLUMN.
 @param[in,out]  rec             leaf page record
 @param[in]      version         row version */
-static inline void rec_set_instant_row_version_old(rec_t *rec, uint8_t version);
+static inline void rec_set_instant_row_version_old(rec_t *rec,
+                                                   row_version_t version);
 
 /** Validates the consistency of a physical record.
 @param[in]      rec     physical record
