@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -77,7 +77,8 @@ static dberr_t replace_inline(InsertContext &ctx, trx_t *trx,
 that the LOB partial update feature code is hit.
 @param[in]      uf      the update field information
 @param[in]      index   index where partial update happens.*/
-static void print_partial_update_hit(upd_field_t *uf, dict_index_t *index) {
+static void print_partial_update_hit(const upd_field_t *uf,
+                                     const dict_index_t *index) {
   ib::info(ER_IB_MSG_632) << "LOB partial update of field=("
                           << uf->mysql_field->field_name << ") on index=("
                           << index->name << ") in table=(" << index->table_name
@@ -99,6 +100,7 @@ dberr_t update(InsertContext &ctx, trx_t *trx, dict_index_t *index,
   dberr_t err = DB_SUCCESS;
   mtr_t *mtr = ctx.get_mtr();
   const undo_no_t undo_no = (trx == nullptr ? 0 : trx->undo_no - 1);
+  const trx_id_t trxid = (trx == nullptr ? 0 : trx->id);
 
   const Binary_diff_vector *bdiff_vector =
       upd->get_binary_diff_by_field_no(field_no);
@@ -109,7 +111,7 @@ dberr_t update(InsertContext &ctx, trx_t *trx, dict_index_t *index,
   const bool small_change =
       (bytes_changed <= ref_t::LOB_SMALL_CHANGE_THRESHOLD);
 
-  upd_field_t *uf = upd->get_field_by_field_no(field_no, index);
+  const upd_field_t *const uf = upd->get_field_by_field_no(field_no, index);
 
 #ifdef UNIV_DEBUG
   /* Print information on server error log file, which can be
@@ -126,7 +128,7 @@ dberr_t update(InsertContext &ctx, trx_t *trx, dict_index_t *index,
 
   first_page_t first_page(mtr, index);
   first_page.load_x(first_page_id, page_size);
-  first_page.set_last_trx_id(trx->id);
+  first_page.set_last_trx_id(trxid);
   first_page.set_last_trx_undo_no(undo_no);
   uint32_t lob_version = 0;
 
