@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2024, Oracle and/or its affiliates.
+Copyright (c) 1995, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -909,8 +909,6 @@ Log_handle log_buffer_reserve(log_t &log, size_t len) {
 #ifdef UNIV_DEBUG
   if (!recv_recovery_is_on()) {
     log_background_threads_active_validate(log);
-  } else {
-    ut_a(!recv_no_ibuf_operations);
   }
 #endif
 
@@ -1191,10 +1189,6 @@ void log_buffer_flush_to_disk(log_t &log, bool sync) {
   log_write_up_to(log, lsn, sync);
 }
 
-void log_buffer_flush_to_disk(bool sync) {
-  log_buffer_flush_to_disk(*log_sys, sync);
-}
-
 void log_buffer_sync_in_background() {
   log_t &log = *log_sys;
 
