@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2024, Oracle and/or its affiliates.
+Copyright (c) 1994, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -35,6 +35,32 @@ this program; if not, write to the Free Software Foundation, Inc.,
 other files in library. The code in this file is used to make a library for
 external tools. */
 
+/**
+Extra Bytes in Redudant Row Format
+
+The extra bytes of the redundant row format (old row format) is explained here.
+It contains a total of 6 bytes that occurs before the record origin.  These
+bits and bytes are accessed with reference to the record origin.  So when we
+say 3rd byte, it means that it is 3rd byte from the record origin.
+
+  byte 6    byte 5    byte 4    byte 3    byte 2    byte 1
+[iiiioooo][hhhhhhhh][hhhhhfff][fffffffs][pppppppp][pppppppp]+
+
+1. The + is the record origin.
+2. The next record pointer is given by the bits marked as 'p'.  This takes
+   2 bytes - 1st and 2nd byte.
+3. One bit is used to indicate whether the field offsets array uses 1 byte or
+   2 bytes each.  This is given by the bit 's' in 3rd byte.
+4. The total number of fields is given by the bits marked as 'f'.  It spans
+   the 4th and 3rd bytes.  It uses a total of 10 bits.
+5. The heap number of the record is given by the bits marked as 'h'.  It spans
+   the 5th and 4th bytes.  It uses a total of 13 bits.
+6. The record owned (by dir slot) information is given by bits marked as 'o'.
+   It uses a total of 4 bits. It is available in the 6th byte.
+7. The info bits are given by the bits marked as 'i'.  It uses a total of 4
+   bits. It is available in the 6th byte.
+*/
+
 #ifndef rem_rec_h
 #define rem_rec_h
 
@@ -357,7 +383,7 @@ current index.
       return n;
     }
 
-    uint16_t n_uniq = dict_index_get_n_unique_in_tree_nonleaf(index);
+    auto n_uniq = dict_index_get_n_unique_in_tree_nonleaf(index);
 
     ut_ad(index->is_clustered());
     ut_ad(n <= dict_index_get_n_fields(index));
@@ -507,7 +533,7 @@ static inline uint64_t rec_get_instant_offset(const dict_index_t *index,
   index->get_nth_default(n, &length);
 
   if (length == UNIV_SQL_NULL) {
-    return (offs | REC_OFFS_SQL_NULL);
+    return (offs | REC_OFFS_DEFAULT | REC_OFFS_SQL_NULL);
   } else {
     return (offs | REC_OFFS_DEFAULT);
   }
@@ -534,13 +560,14 @@ void rec_init_offsets(const rec_t *rec, const dict_index_t *index,
 
 #ifdef UNIV_DEBUG
 /** Validates offsets returned by rec_get_offsets().
- @return true if valid */
+@param[in]  rec       record whose offsets are being validated or nullptr.
+@param[in]  index     index to which record belongs or nullptr.
+@param[in]  offsets   the record offsets array returned by rec_get_offsets()
+@param[in]  check_status  if true, check status bits of the record.
+@return true if valid */
 [[nodiscard]] static inline bool rec_offs_validate(
-    const rec_t *rec,          /*!< in: record or NULL */
-    const dict_index_t *index, /*!< in: record descriptor or NULL */
-    const ulint *offsets)      /*!< in: array returned by
-                               rec_get_offsets() */
-{
+    const rec_t *rec, const dict_index_t *index, const ulint *offsets,
+    const bool check_status = true) {
   ulint i = rec_offs_n_fields(offsets);
   ulint last = ULINT_MAX;
   ulint comp = *rec_offs_base(offsets) & REC_OFFS_COMPACT;
@@ -551,8 +578,9 @@ void rec_init_offsets(const rec_t *rec, const dict_index_t *index,
     - cached and shared by many records, in which case we've passed rec=nullptr
       when preparing the offsets array.
     We use caching only for the ROW_FORMAT=COMPACT format. */
-    ut_ad((ulint)rec == offsets[2] || ((ulint) nullptr == offsets[2] &&
-                                       offsets == index->rec_cache.offsets));
+    ut_ad((ulint)rec == offsets[2] ||
+          ((ulint) nullptr == offsets[2] &&
+           (index != nullptr && offsets == index->rec_cache.offsets)));
     if (!comp && index != nullptr) {
       ut_a(rec_get_n_fields_old(rec, index) >= i);
     }
@@ -568,21 +596,25 @@ void rec_init_offsets(const rec_t *rec, const dict_index_t *index,
       ut_a(index->has_instant_cols_or_row_versions());
     }
 
-    if (comp && rec) {
-      switch (rec_get_status(rec)) {
-        case REC_STATUS_ORDINARY:
-          break;
-        case REC_STATUS_NODE_PTR:
-          max_n_fields = dict_index_get_n_unique_in_tree(index) + 1;
-          break;
-        case REC_STATUS_INFIMUM:
-        case REC_STATUS_SUPREMUM:
-          max_n_fields = 1;
-          break;
-        default:
-          ut_error;
+    /* In the case of mrec_t the status will not be there.  */
+    if (check_status) {
+      if (comp && rec) {
+        switch (rec_get_status(rec)) {
+          case REC_STATUS_ORDINARY:
+            break;
+          case REC_STATUS_NODE_PTR:
+            max_n_fields = dict_index_get_n_unique_in_tree(index) + 1;
+            break;
+          case REC_STATUS_INFIMUM:
+          case REC_STATUS_SUPREMUM:
+            max_n_fields = 1;
+            break;
+          default:
+            ut_error;
+        }
       }
     }
+
     /* index->n_def == 0 for dummy indexes if !comp */
     ut_a(!comp || index->n_def);
     ut_a(!index->n_def || i <= max_n_fields);
@@ -773,7 +805,8 @@ static inline enum REC_INSERT_STATE get_rec_insert_state(
       ((temp ? REC_N_TMP_EXTRA_BYTES : REC_N_NEW_EXTRA_BYTES) + 1);
   const bool is_versioned =
       (temp) ? rec_new_temp_is_versioned(rec) : rec_new_is_versioned(rec);
-  const uint8_t version = (is_versioned) ? (uint8_t)(*v_ptr) : UINT8_UNDEFINED;
+  const row_version_t version =
+      (is_versioned) ? static_cast<row_version_t>(*v_ptr) : INVALID_ROW_VERSION;
 
   const bool is_instant = (temp) ? rec_get_instant_flag_new_temp(rec)
                                  : rec_get_instant_flag_new(rec);
@@ -829,12 +862,12 @@ record generated for a record from REDUNDANT FORAMT
 static inline enum REC_INSERT_STATE init_nulls_lens_for_temp_redundant(
     const dict_index_t *index, const rec_t *rec, uint16_t *n_null,
     const byte **nulls, const byte **lens, uint16_t &non_default_fields,
-    uint8_t &row_version) {
+    row_version_t &row_version) {
   ut_ad(!dict_table_is_comp(index->table));
 
   non_default_fields = static_cast<uint16_t>(dict_index_get_n_fields(index));
 
-  row_version = UINT8_UNDEFINED;
+  row_version = INVALID_ROW_VERSION;
 
   /* Set nulls just before the record */
   *nulls = rec - 1;
@@ -892,7 +925,7 @@ for a new-style temporary record
 static inline enum REC_INSERT_STATE rec_init_null_and_len_temp(
     const rec_t *rec, const dict_index_t *index, const byte **nulls,
     const byte **lens, uint16_t *n_null, uint16_t &non_default_fields,
-    uint8_t &row_version) {
+    row_version_t &row_version) {
   /* Following is the format for TEMP record.
   +----+----+-------------------+--------------------+
   | OP | ES |<-- Extra info --> | F1 | F2 | ...  | Fn|
@@ -917,7 +950,7 @@ static inline enum REC_INSERT_STATE rec_init_null_and_len_temp(
 
   non_default_fields = static_cast<uint16_t>(dict_index_get_n_fields(index));
 
-  row_version = UINT8_UNDEFINED;
+  row_version = INVALID_ROW_VERSION;
 
   /* Set nulls just before the record */
   *nulls = rec - 1;
@@ -994,10 +1027,10 @@ for a new style record
 static inline enum REC_INSERT_STATE rec_init_null_and_len_comp(
     const rec_t *rec, const dict_index_t *index, const byte **nulls,
     const byte **lens, uint16_t *n_null, uint16_t &non_default_fields,
-    uint8_t &row_version) {
+    row_version_t &row_version) {
   non_default_fields = static_cast<uint16_t>(dict_index_get_n_fields(index));
 
-  row_version = UINT8_UNDEFINED;
+  row_version = INVALID_ROW_VERSION;
 
   /* Position nulls */
   *nulls = rec - (REC_N_NEW_EXTRA_BYTES + 1);
@@ -1076,9 +1109,189 @@ rec_get_offsets().
 @param[in]      temp    whether to use the format for temporary files in index
                         creation
 @param[in]      index   record descriptor
-@param[in,out]  offsets array of offsets */
-void rec_init_offsets_comp_ordinary(const rec_t *rec, bool temp,
-                                    const dict_index_t *index, ulint *offsets);
+@param[in,out]  offsets array of offsets
+@note This long method is made inline because it is on performance sensitive hot
+path. One must run performance tests if they intend to improve this method. */
+inline void rec_init_offsets_comp_ordinary(const rec_t *rec, bool temp,
+                                           const dict_index_t *index,
+                                           ulint *offsets) {
+#ifdef UNIV_DEBUG
+  /* We cannot invoke rec_offs_make_valid() here if temp=true.
+  Similarly, rec_offs_validate() will fail in that case, because
+  it invokes rec_get_status(). */
+  offsets[2] = (ulint)rec;
+  offsets[3] = (ulint)index;
+#endif /* UNIV_DEBUG */
+
+  const byte *nulls = nullptr;
+  const byte *lens = nullptr;
+  uint16_t n_null = 0;
+  enum REC_INSERT_STATE rec_insert_state = REC_INSERT_STATE::NONE;
+  row_version_t row_version = INVALID_ROW_VERSION;
+  uint16_t non_default_fields = 0;
+
+  if (temp) {
+    rec_insert_state = rec_init_null_and_len_temp(
+        rec, index, &nulls, &lens, &n_null, non_default_fields, row_version);
+  } else {
+    rec_insert_state = rec_init_null_and_len_comp(
+        rec, index, &nulls, &lens, &n_null, non_default_fields, row_version);
+  }
+
+  ut_ad(temp || dict_table_is_comp(index->table));
+
+  if (temp) {
+    if (dict_table_is_comp(index->table)) {
+      /* No need to do adjust fixed_len=0. We only need to
+      adjust it for ROW_FORMAT=REDUNDANT. */
+      temp = false;
+    } else {
+      /* Redundant temp row. Old instant record is logged as version 0. */
+      if (rec_insert_state == INSERTED_BEFORE_INSTANT_ADD_OLD_IMPLEMENTATION ||
+          rec_insert_state == INSERTED_AFTER_INSTANT_ADD_OLD_IMPLEMENTATION) {
+        rec_insert_state = INSERTED_BEFORE_INSTANT_ADD_NEW_IMPLEMENTATION;
+        ut_ad(row_version == INVALID_ROW_VERSION);
+      }
+    }
+  }
+
+  /* read the lengths of fields 0..n */
+  ulint offs = 0;
+  ulint any_ext = 0;
+  ulint null_mask = 1;
+  uint16_t i = 0;
+  do {
+    /* Fields are stored on disk in version they are added in and are
+    maintained in fields_array in the same order. Get the right field. */
+    const dict_field_t *field = index->get_physical_field(i);
+    const dict_col_t *col = field->col;
+    uint64_t len;
+
+    switch (rec_insert_state) {
+      case INSERTED_INTO_TABLE_WITH_NO_INSTANT_NO_VERSION:
+        ut_ad(!index->has_instant_cols_or_row_versions());
+        break;
+
+      case INSERTED_BEFORE_INSTANT_ADD_NEW_IMPLEMENTATION: {
+        ut_ad(row_version == INVALID_ROW_VERSION || row_version == 0);
+        ut_ad(index->has_row_versions() || temp);
+        /* Record has to be interpreted in v0. */
+        row_version = 0;
+      }
+        [[fallthrough]];
+      case INSERTED_AFTER_UPGRADE_BEFORE_INSTANT_ADD_NEW_IMPLEMENTATION:
+      case INSERTED_AFTER_INSTANT_ADD_NEW_IMPLEMENTATION: {
+        ut_ad(is_valid_row_version(row_version));
+        /* A record may have version=0 if it's from upgrade table */
+        ut_ad(index->has_row_versions() ||
+              (index->table->is_upgraded_instant() && row_version == 0));
+
+        /* Based on the record version and column information, see if this
+        column is there in this record or not. */
+        if (col->is_dropped_in_or_before(row_version)) {
+          /* This columns is dropped before or on this row version so its data
+          won't be there on row. So no need to store the length. Instead, store
+          offs ORed with REC_OFFS_DROP to indicate the same. */
+          len = offs | REC_OFFS_DROP;
+          goto resolved;
+
+          /* NOTE : Existing rows, which have data for this column, would still
+          need to process this column, so don't skip and store the correct
+          length there. Though it will be skipped while fetching row. */
+        } else if (col->is_added_after(row_version)) {
+          /* This columns is added after this row version. In this case no need
+          to store the length. Instead store only if it is NULL or DEFAULT
+          value. */
+          len = rec_get_instant_offset(index, i, offs);
+
+          goto resolved;
+        }
+      } break;
+
+      case INSERTED_BEFORE_INSTANT_ADD_OLD_IMPLEMENTATION:
+      case INSERTED_AFTER_INSTANT_ADD_OLD_IMPLEMENTATION: {
+        ut_ad(non_default_fields > 0);
+        ut_ad(index->has_instant_cols());
+        ut_ad(!is_valid_row_version(row_version));
+
+        if (i >= non_default_fields) {
+          /* This would be the case when column doesn't exists in the row. In
+          this case we need not to store the length. Instead we store only if
+          the column is NULL or DEFAULT value. */
+          len = rec_get_instant_offset(index, i, offs);
+
+          goto resolved;
+        }
+
+        /* Note : Even if the column has been dropped, this row in V1 would
+        definitely have the value of this column. */
+      } break;
+
+      default:
+        ut_ad(false);
+    }
+
+    if (!(col->prtype & DATA_NOT_NULL)) {
+      /* nullable field => read the null flag */
+      ut_ad(n_null--);
+
+      if (UNIV_UNLIKELY(!(byte)null_mask)) {
+        nulls--;
+        null_mask = 1;
+      }
+
+      if (*nulls & null_mask) {
+        null_mask <<= 1;
+        /* No length is stored for NULL fields.
+        We do not advance offs, and we set
+        the length to zero and enable the
+        SQL NULL flag in offsets[]. */
+        len = offs | REC_OFFS_SQL_NULL;
+        goto resolved;
+      }
+      null_mask <<= 1;
+    }
+
+    if (!field->fixed_len || (temp && !col->get_fixed_size(temp))) {
+      ut_ad(col->mtype != DATA_POINT);
+      /* Variable-length field: read the length */
+      len = *lens--;
+      /* If the maximum length of the field is up
+      to 255 bytes, the actual length is always
+      stored in one byte. If the maximum length is
+      more than 255 bytes, the actual length is
+      stored in one byte for 0..127.  The length
+      will be encoded in two bytes when it is 128 or
+      more, or when the field is stored externally. */
+      if (DATA_BIG_COL(col)) {
+        if (len & 0x80) {
+          /* 1exxxxxxx xxxxxxxx */
+          len <<= 8;
+          len |= *lens--;
+
+          offs += len & 0x3fff;
+          if (UNIV_UNLIKELY(len & 0x4000)) {
+            ut_ad(index->is_clustered());
+            any_ext = REC_OFFS_EXTERNAL;
+            len = offs | REC_OFFS_EXTERNAL;
+          } else {
+            len = offs;
+          }
+
+          goto resolved;
+        }
+      }
+
+      len = offs += len;
+    } else {
+      len = offs += field->fixed_len;
+    }
+  resolved:
+    rec_offs_base(offsets)[i + 1] = len;
+  } while (++i < rec_offs_n_fields(offsets));
+
+  *rec_offs_base(offsets) = (rec - (lens + 1)) | REC_OFFS_COMPACT | any_ext;
+}
 
 /** The following function is used to test whether the data offsets in the
  record are stored in one-byte or two-byte format.
