@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2021, Oracle and/or its affiliates.
+Copyright (c) 1997, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -37,13 +38,14 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "current_thd.h"
 #include "debug_sync.h"
 #include "mysqld.h"
+#include "string_with_len.h"
 
 #include "dict0dd.h"
 #include "fsp0fsp.h"
 #include "ha_innodb.h"
 #include "handler.h"
 #include "lob0lob.h"
-#include "log0log.h"
+#include "log0chkp.h"
 #include "mach0data.h"
 #include "que0que.h"
 #include "row0log.h"
@@ -54,6 +56,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "sql_base.h"
 #include "srv0mon.h"
 #include "srv0start.h"
+#include "sync0types.h"
 #include "table.h"
 #include "trx0purge.h"
 #include "trx0rec.h"
@@ -67,14 +70,14 @@ IMPORTANT NOTE: Any operation that generates redo MUST check that there
 is enough space in the redo log before for that operation. This is
 done by calling log_free_check(). The reason for checking the
 availability of the redo log space before the start of the operation is
-that we MUST not hold any synchonization objects when performing the
+that we MUST not hold any synchronization objects when performing the
 check.
 If you make a change in this module make sure that no codepath is
 introduced where a call to log_free_check() is bypassed. */
 
 /** Create a purge node to a query graph.
-@param[in]	parent	parent node, i.e., a thr node
-@param[in]	heap	memory heap where created
+@param[in]      parent  parent node, i.e., a thr node
+@param[in]      heap    memory heap where created
 @return own: purge node */
 purge_node_t *row_purge_node_create(que_thr_t *parent, mem_heap_t *heap) {
   ut_ad(parent != nullptr);
@@ -88,9 +91,10 @@ purge_node_t *row_purge_node_create(que_thr_t *parent, mem_heap_t *heap) {
   node->common.parent = parent;
   node->done = true;
 
-  node->heap = mem_heap_create(256);
+  node->heap = mem_heap_create(256, UT_LOCATION_HERE);
 
   node->recs = nullptr;
+  node->init();
 
   return (node);
 }
@@ -98,7 +102,7 @@ purge_node_t *row_purge_node_create(que_thr_t *parent, mem_heap_t *heap) {
 /** Repositions the pcur in the purge node on the clustered index record,
  if found. If the record is not found, close pcur.
  @return true if the record was found */
-static ibool row_purge_reposition_pcur(
+static bool row_purge_reposition_pcur(
     ulint mode,         /*!< in: latching mode */
     purge_node_t *node, /*!< in: row purge node */
     mtr_t *mtr)         /*!< in: mtr */
@@ -106,20 +110,21 @@ static ibool row_purge_reposition_pcur(
   if (node->found_clust) {
     ut_ad(node->validate_pcur());
 
-    node->found_clust = btr_pcur_restore_position(mode, &node->pcur, mtr);
+    node->found_clust =
+        node->pcur.restore_position(mode, mtr, UT_LOCATION_HERE);
 
   } else {
     node->found_clust =
         row_search_on_row_ref(&node->pcur, mode, node->table, node->ref, mtr);
 
     if (node->found_clust) {
-      btr_pcur_store_position(&node->pcur, mtr);
+      node->pcur.store_position(mtr);
     }
   }
 
   /* Close the current cursor if we fail to position it correctly. */
   if (!node->found_clust) {
-    btr_pcur_close(&node->pcur);
+    node->pcur.close();
   }
 
   return (node->found_clust);
@@ -128,7 +133,7 @@ static ibool row_purge_reposition_pcur(
 /** Removes a delete marked clustered index record if possible.
  @retval true if the row was not found, or it was successfully removed
  @retval false if the row was modified after the delete marking */
-static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_clust_if_poss_low(
+[[nodiscard]] static bool row_purge_remove_clust_if_poss_low(
     purge_node_t *node, /*!<
                            in/out:
                            row
@@ -171,9 +176,10 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_clust_if_poss_lo
     goto func_exit;
   }
 
-  rec = btr_pcur_get_rec(&node->pcur);
+  rec = node->pcur.get_rec();
 
-  offsets = rec_get_offsets(rec, index, offsets_, ULINT_UNDEFINED, &heap);
+  offsets = rec_get_offsets(rec, index, offsets_, ULINT_UNDEFINED,
+                            UT_LOCATION_HERE, &heap);
 
   if (node->roll_ptr != row_get_rec_roll_ptr(rec, index, offsets)) {
     /* Someone else has modified the record later: do not remove */
@@ -183,8 +189,7 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_clust_if_poss_lo
   ut_ad(rec_get_deleted_flag(rec, rec_offs_comp(offsets)));
 
   if (mode == BTR_MODIFY_LEAF) {
-    success =
-        btr_cur_optimistic_delete(btr_pcur_get_btr_cur(&node->pcur), 0, &mtr);
+    success = btr_cur_optimistic_delete(node->pcur.get_btr_cur(), 0, &mtr);
   } else {
     dberr_t err;
     ut_ad(mode == (BTR_MODIFY_TREE | BTR_LATCH_FOR_DELETE));
@@ -199,9 +204,9 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_clust_if_poss_lo
       }
     });
 
-    btr_cur_pessimistic_delete(&err, FALSE, btr_pcur_get_btr_cur(&node->pcur),
-                               0, false, node->trx_id, node->undo_no,
-                               node->rec_type, &mtr, &node->pcur);
+    btr_cur_pessimistic_delete(&err, false, node->pcur.get_btr_cur(), 0, false,
+                               node->trx_id, node->undo_no, node->rec_type,
+                               &mtr, &node->pcur, node);
 
     switch (err) {
       case DB_SUCCESS:
@@ -221,7 +226,7 @@ func_exit:
 
   /* Persistent cursor is closed if reposition fails. */
   if (node->found_clust) {
-    btr_pcur_commit_specify_mtr(&node->pcur, &mtr);
+    node->pcur.commit_specify_mtr(&mtr);
   } else {
     mtr_commit(&mtr);
   }
@@ -234,7 +239,7 @@ func_exit:
  @retval true if the row was not found, or it was successfully removed
  @retval false the purge needs to be suspended because of running out
  of file space. */
-static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_clust_if_poss(
+[[nodiscard]] static bool row_purge_remove_clust_if_poss(
     purge_node_t *node) /*!< in/out: row purge node */
 {
   if (row_purge_remove_clust_if_poss_low(node, BTR_MODIFY_LEAF)) {
@@ -280,12 +285,12 @@ bool row_purge_poss_sec(purge_node_t *node,    /*!< in/out: row purge node */
 
   can_delete =
       !row_purge_reposition_pcur(BTR_SEARCH_LEAF, node, &mtr) ||
-      !row_vers_old_has_index_entry(TRUE, btr_pcur_get_rec(&node->pcur), &mtr,
-                                    index, entry, node->roll_ptr, node->trx_id);
+      !row_vers_old_has_index_entry(true, node->pcur.get_rec(), &mtr, index,
+                                    entry, node->roll_ptr, node->trx_id);
 
   /* Persistent cursor is closed if reposition fails. */
   if (node->found_clust) {
-    btr_pcur_commit_specify_mtr(&node->pcur, &mtr);
+    node->pcur.commit_specify_mtr(&mtr);
   } else {
     mtr_commit(&mtr);
   }
@@ -297,15 +302,14 @@ bool row_purge_poss_sec(purge_node_t *node,    /*!< in/out: row purge node */
 Removes a secondary index entry if possible, by modifying the
 index tree.  Does not try to buffer the delete.
 @return true if success or if not found */
-static MY_ATTRIBUTE((warn_unused_result)) ibool
-    row_purge_remove_sec_if_poss_tree(
-        purge_node_t *node,    /*!< in: row purge node */
-        dict_index_t *index,   /*!< in: index */
-        const dtuple_t *entry) /*!< in: index entry */
+[[nodiscard]] static bool row_purge_remove_sec_if_poss_tree(
+    purge_node_t *node,    /*!< in: row purge node */
+    dict_index_t *index,   /*!< in: index */
+    const dtuple_t *entry) /*!< in: index entry */
 {
   btr_pcur_t pcur;
   btr_cur_t *btr_cur;
-  ibool success = TRUE;
+  bool success = true;
   dberr_t err;
   mtr_t mtr;
   enum row_search_result search_result;
@@ -317,7 +321,7 @@ static MY_ATTRIBUTE((warn_unused_result)) ibool
     /* The index->online_status may change if the index is
     or was being created online, but not committed yet. It
     is protected by index->lock. */
-    mtr_sx_lock(dict_index_get_lock(index), &mtr);
+    mtr_sx_lock(dict_index_get_lock(index), &mtr, UT_LOCATION_HERE);
 
     if (dict_index_is_online_ddl(index)) {
       /* Online secondary index creation will not
@@ -362,7 +366,7 @@ static MY_ATTRIBUTE((warn_unused_result)) ibool
       ut_error;
   }
 
-  btr_cur = btr_pcur_get_btr_cur(&pcur);
+  btr_cur = pcur.get_btr_cur();
 
   /* We should remove the index record if no later version of the row,
   which cannot be purged yet, requires its existence. If some requires,
@@ -380,18 +384,18 @@ static MY_ATTRIBUTE((warn_unused_result)) ibool
           << ": tuple: " << *entry
           << ", record: " << rec_index_print(btr_cur_get_rec(btr_cur), index);
 
-      ut_ad(0);
+      ut_d(ut_error);
 
-      goto func_exit;
+      ut_o(goto func_exit);
     }
 
-    btr_cur_pessimistic_delete(&err, FALSE, btr_cur, 0, false, 0, node->undo_no,
-                               node->rec_type, &mtr);
+    btr_cur_pessimistic_delete(&err, false, btr_cur, 0, false, 0, node->undo_no,
+                               node->rec_type, &mtr, &pcur, node);
     switch (UNIV_EXPECT(err, DB_SUCCESS)) {
       case DB_SUCCESS:
         break;
       case DB_OUT_OF_FILE_SPACE:
-        success = FALSE;
+        success = false;
         break;
       default:
         ut_error;
@@ -399,7 +403,7 @@ static MY_ATTRIBUTE((warn_unused_result)) ibool
   }
 
 func_exit:
-  btr_pcur_close(&pcur);
+  pcur.close();
 func_exit_no_pcur:
   mtr_commit(&mtr);
 
@@ -411,7 +415,7 @@ Removes a secondary index entry without modifying the index tree,
 if possible.
 @retval true if success or if not found
 @retval false if row_purge_remove_sec_if_poss_tree() should be invoked */
-static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf(
+[[nodiscard]] static bool row_purge_remove_sec_if_poss_leaf(
     purge_node_t *node,    /*!<
                               in:
                               row
@@ -447,7 +451,7 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
     /* The index->online_status may change if the the
     index is or was being created online, but not
     committed yet. It is protected by index->lock. */
-    mtr_s_lock(dict_index_get_lock(index), &mtr);
+    mtr_s_lock(dict_index_get_lock(index), &mtr, UT_LOCATION_HERE);
 
     if (dict_index_is_online_ddl(index)) {
       /* Online secondary index creation will not
@@ -478,7 +482,7 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
   /* Set the purge node for the call to row_purge_poss_sec(). */
   pcur.m_btr_cur.purge_node = node;
   if (dict_index_is_spatial(index)) {
-    rw_lock_sx_lock(dict_index_get_lock(index));
+    rw_lock_sx_lock(dict_index_get_lock(index), UT_LOCATION_HERE);
     pcur.m_btr_cur.thr = nullptr;
   } else {
     /* Set the query thread, so that ibuf_insert_low() will be
@@ -497,7 +501,7 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
       /* Before attempting to purge a record, check
       if it is safe to do so. */
       if (row_purge_poss_sec(node, index, entry)) {
-        btr_cur_t *btr_cur = btr_pcur_get_btr_cur(&pcur);
+        btr_cur_t *btr_cur = pcur.get_btr_cur();
 
         /* Only delete-marked records should be purged. */
         if (!rec_get_deleted_flag(btr_cur_get_rec(btr_cur),
@@ -509,11 +513,12 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
               << index->name << " of table " << index->table->name
               << ": tuple: " << *entry << ", record: "
               << rec_index_print(btr_cur_get_rec(btr_cur), index);
-          ut_ad(0);
 
-          btr_pcur_close(&pcur);
+          pcur.close();
 
-          goto func_exit_no_pcur;
+          ut_d(ut_error);
+
+          ut_o(goto func_exit_no_pcur);
         }
 
         if (dict_index_is_spatial(index)) {
@@ -527,7 +532,7 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
 
           page = btr_cur_get_page(btr_cur);
 
-          if (!lock_test_prdt_page_lock(trx, page_get_page_id(page)) &&
+          if (lock_other_has_prdt_page_lock(trx, page_get_page_id(page)) &&
               page_get_n_recs(page) < 2 &&
               page_get_page_no(page) != dict_index_get_page(index)) {
             /* this is the last record on page,
@@ -540,7 +545,7 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
                                      << page_get_page_no(page) << ".";
 #endif /* UNIV_DEBUG */
 
-            btr_pcur_close(&pcur);
+            pcur.close();
             mtr_commit(&mtr);
             return (success);
           }
@@ -553,13 +558,14 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
       }
       /* fall through (the index entry is still needed,
       or the deletion succeeded) */
+      [[fallthrough]];
     case ROW_NOT_DELETED_REF:
       /* The index entry is still needed. */
     case ROW_BUFFERED:
       /* The deletion was buffered. */
     case ROW_NOT_FOUND:
       /* The index entry does not exist, nothing to do. */
-      btr_pcur_close(&pcur);
+      pcur.close();
     func_exit_no_pcur:
       mtr_commit(&mtr);
       return (success);
@@ -570,15 +576,14 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_remove_sec_if_poss_leaf
 }
 
 /** Removes a secondary index entry if possible. */
-UNIV_INLINE
-void row_purge_remove_sec_if_poss(purge_node_t *node, /*!< in: row purge node */
-                                  dict_index_t *index,   /*!< in: index */
-                                  const dtuple_t *entry) /*!< in: index entry */
+static inline void row_purge_remove_sec_if_poss(
+    purge_node_t *node,    /*!< in: row purge node */
+    dict_index_t *index,   /*!< in: index */
+    const dtuple_t *entry) /*!< in: index entry */
 {
-  ibool success;
   ulint n_tries = 0;
 
-  /*	fputs("Purge: Removing secondary record\n", stderr); */
+  /*    fputs("Purge: Removing secondary record\n", stderr); */
 
   if (!entry) {
     /* The node->row must have lacked some fields of this
@@ -591,7 +596,7 @@ void row_purge_remove_sec_if_poss(purge_node_t *node, /*!< in: row purge node */
     return;
   }
 retry:
-  success = row_purge_remove_sec_if_poss_tree(node, index, entry);
+  auto success = row_purge_remove_sec_if_poss_tree(node, index, entry);
   /* The delete operation may fail if we have little
   file space left: TODO: easiest to crash the database
   and restart with more file space */
@@ -609,7 +614,7 @@ retry:
 }
 
 /** Skip uncommitted virtual indexes on newly added virtual column.
-@param[in,out]	index	dict index object */
+@param[in,out]  index   dict index object */
 static inline void row_purge_skip_uncommitted_virtual_index(
     dict_index_t *&index) {
   /* We need to skip virtual indexes which is not
@@ -624,9 +629,9 @@ static inline void row_purge_skip_uncommitted_virtual_index(
 }
 
 /** Remove multi-value index entries if possible.
-@param[in,out]	node		row purge node
-@param[in,out]	heap		memory heap
-@param[in]	selected	true if only selected multi-value data should
+@param[in,out]  node            row purge node
+@param[in,out]  heap            memory heap
+@param[in]      selected        true if only selected multi-value data should
                                 be purged */
 static inline void row_purge_remove_multi_sec_if_poss(purge_node_t *node,
                                                       mem_heap_t *heap,
@@ -648,12 +653,12 @@ static inline void row_purge_remove_multi_sec_if_poss(purge_node_t *node,
  @retval true if the row was not found, or it was successfully removed
  @retval false the purge needs to be suspended because of
  running out of file space */
-static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_del_mark(
+[[nodiscard]] static bool row_purge_del_mark(
     purge_node_t *node) /*!< in/out: row purge node */
 {
   mem_heap_t *heap;
 
-  heap = mem_heap_create(1024);
+  heap = mem_heap_create(1024, UT_LOCATION_HERE);
 
   while (node->index != nullptr) {
     /* skip corrupted secondary index */
@@ -686,14 +691,13 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_del_mark(
 }
 
 /** Purges an update of an existing record. Also purges an update of a delete
- marked record if that record contained an externally stored field. */
-static void row_purge_upd_exist_or_extern_func(
-#ifdef UNIV_DEBUG
-    const que_thr_t *thr,     /*!< in: query thread */
-#endif                        /* UNIV_DEBUG */
-    purge_node_t *node,       /*!< in: row purge node */
-    trx_undo_rec_t *undo_rec) /*!< in: record to purge */
-{
+ marked record if that record contained an externally stored field.
+@param[in] thr query thread
+@param[in] node row purge node
+@param[in] undo_rec record to purge */
+static void row_purge_upd_exist_or_extern_func(IF_DEBUG(const que_thr_t *thr, )
+                                                   purge_node_t *node,
+                                               trx_undo_rec_t *undo_rec) {
   mem_heap_t *heap;
 
   ut_ad(!node->table->skip_alter_undo);
@@ -703,7 +707,7 @@ static void row_purge_upd_exist_or_extern_func(
     goto skip_secondaries;
   }
 
-  heap = mem_heap_create(1024);
+  heap = mem_heap_create(1024, UT_LOCATION_HERE);
 
   while (node->index != nullptr) {
     bool non_mv_upd = false;
@@ -716,6 +720,10 @@ static void row_purge_upd_exist_or_extern_func(
       break;
     }
 
+#ifndef UNIV_DEBUG
+    que_thr_t *thr = nullptr;
+#endif
+
     if (row_upd_changes_ord_field_binary(
             node->index, node->update, thr, nullptr, nullptr,
             (node->index->is_multi_value() ? &non_mv_upd : nullptr))) {
@@ -746,8 +754,8 @@ skip_secondaries:
       ulint internal_offset;
       byte *data_field;
       dict_index_t *index;
-      ibool is_insert;
-      ulint rseg_id;
+      bool is_insert;
+      ulint undo_num;
       page_no_t page_no;
       space_id_t undo_space_id;
       ulint offset;
@@ -764,14 +772,14 @@ skip_secondaries:
 
       ut_a(internal_offset < UNIV_PAGE_SIZE);
 
-      trx_undo_decode_roll_ptr(node->roll_ptr, &is_insert, &rseg_id, &page_no,
+      trx_undo_decode_roll_ptr(node->roll_ptr, &is_insert, &undo_num, &page_no,
                                &offset);
 
       /* If table is temp then it can't have its undo log
       residing in rollback segment with REDO log enabled. */
       bool is_temp = node->table->is_temporary();
 
-      undo_space_id = trx_rseg_id_to_space_id(rseg_id, is_temp);
+      undo_space_id = trx_undo_num_to_space_id(undo_num, is_temp);
 
       mtr_start(&mtr);
 
@@ -780,7 +788,7 @@ skip_secondaries:
 
       index = node->table->first_index();
 
-      mtr_sx_lock(dict_index_get_lock(index), &mtr);
+      mtr_sx_lock(dict_index_get_lock(index), &mtr, UT_LOCATION_HERE);
 
       /* NOTE: we must also acquire an X-latch to the
       root page of the tree. We will need it when we
@@ -794,7 +802,7 @@ skip_secondaries:
       btr_root_get(index, &mtr);
 
       block = buf_page_get(page_id_t(undo_space_id, page_no), univ_page_size,
-                           RW_X_LATCH, &mtr);
+                           RW_X_LATCH, UT_LOCATION_HERE, &mtr);
 
       buf_block_dbg_add_level(block, SYNC_TRX_UNDO_PAGE);
 
@@ -805,41 +813,39 @@ skip_secondaries:
       byte *field_ref = data_field + dfield_get_len(&ufield->new_val) -
                         BTR_EXTERN_FIELD_REF_SIZE;
 
-      lob::BtrContext btr_ctx(&mtr, nullptr, index, nullptr, nullptr, block);
+      lob::BtrContext btr_ctx(&mtr, index, block);
 
       lob::DeleteContext ctx(btr_ctx, field_ref, ufield->field_no, false);
 
       lob::purge(&ctx, index, node->modifier_trx_id,
-                 trx_undo_rec_get_undo_no(undo_rec), node->rec_type, ufield);
+                 trx_undo_rec_get_undo_no(undo_rec), node->rec_type, ufield,
+                 node);
 
       mtr_commit(&mtr);
     }
   }
 }
 
-#ifdef UNIV_DEBUG
-#define row_purge_upd_exist_or_extern(thr, node, undo_rec) \
-  row_purge_upd_exist_or_extern_func(thr, node, undo_rec)
-#else /* UNIV_DEBUG */
-#define row_purge_upd_exist_or_extern(thr, node, undo_rec) \
-  row_purge_upd_exist_or_extern_func(node, undo_rec)
-#endif /* UNIV_DEBUG */
+inline void row_purge_upd_exist_or_extern(const que_thr_t *thr [[maybe_unused]],
+                                          purge_node_t *node,
+                                          trx_undo_rec_t *undo_rec) {
+  row_purge_upd_exist_or_extern_func(IF_DEBUG(thr, ) node, undo_rec);
+}
 
 /** Parses the row reference and other info in a modify undo log record.
- @param[in,out]	node			row undo node
- @param[in]	undo_rec		undo record to purge
- @param[out]	updated_extern		whether an externally stored
+ @param[in,out] node                    row undo node
+ @param[in]     undo_rec                undo record to purge
+ @param[out]    updated_extern          whether an externally stored
  field was updated
- @param[in,out]	thd			current thread
- @param[in,out]	thr			execution thread
+ @param[in,out] thd                     current thread
+ @param[in,out] thr                     execution thread
  @return true if purge operation required */
 static bool row_purge_parse_undo_rec(purge_node_t *node,
                                      trx_undo_rec_t *undo_rec,
                                      bool *updated_extern, THD *thd,
                                      que_thr_t *thr) {
   dict_index_t *clust_index;
-  byte *ptr;
-  trx_t *trx;
+  const byte *ptr;
   undo_no_t undo_no;
   table_id_t table_id;
   trx_id_t trx_id;
@@ -871,7 +877,7 @@ try_again:
 #endif /* INNODB_DD_VC_SUPPORT */
 
   /* Cannot call dd_table_open_on_id() before server is fully up */
-  if (!srv_upgrade_old_undo_found && !dict_table_is_system(table_id)) {
+  if (!dict_table_is_system(table_id)) {
     while (!mysqld_server_started) {
       if (srv_shutdown_state.load() >= SRV_SHUTDOWN_PURGE) {
         return (false);
@@ -886,7 +892,7 @@ try_again:
   exclusive MDL on SDI tables */
   ut_ad(!dict_table_is_system(table_id));
 
-  if (dict_table_is_sdi(table_id) || srv_upgrade_old_undo_found) {
+  if (dict_table_is_sdi(table_id)) {
     if (dict_table_is_sdi(table_id)) {
       space_id_t space_id = dict_sdi_get_space_id(table_id);
 
@@ -910,18 +916,18 @@ try_again:
       const auto no_mdl = nullptr;
       node->mdl = no_mdl;
 
-      mutex_enter(&dict_sys->mutex);
+      dict_sys_mutex_enter();
       node->table = dd_table_open_on_id(table_id, thd, &node->mdl, true, true);
 
       if (node->table && node->table->is_temporary()) {
         /* Temp table does not do purge */
         ut_ad(node->mdl == nullptr);
         dd_table_close(node->table, nullptr, nullptr, true);
-        mutex_exit(&dict_sys->mutex);
+        dict_sys_mutex_exit();
         goto err_exit;
       }
 
-      mutex_exit(&dict_sys->mutex);
+      dict_sys_mutex_exit();
 
       if (node->table != nullptr) {
         if (node->table->is_fts_aux()) {
@@ -1021,11 +1027,11 @@ try_again:
     we do not have an index to call it with. */
   close_exit:
     /* Purge requires no changes to indexes: we may return */
-    if (dict_table_is_sdi(node->table->id) || srv_upgrade_old_undo_found) {
+    if (dict_table_is_sdi(node->table->id)) {
       if (dict_table_is_sdi(node->table->id)) {
         dd_table_close(node->table, thd, &node->mdl, false);
       } else {
-        dict_table_close(node->table, FALSE, FALSE);
+        dict_table_close(node->table, false, false);
       }
       node->table = nullptr;
     } else {
@@ -1047,11 +1053,9 @@ try_again:
 
   ptr = trx_undo_rec_get_row_ref(ptr, clust_index, &(node->ref), node->heap);
 
-  trx = thr_get_trx(thr);
-
   ptr = trx_undo_update_rec_get_update(ptr, clust_index, type, trx_id, roll_ptr,
-                                       info_bits, trx, node->heap,
-                                       &(node->update), nullptr, type_cmpl);
+                                       info_bits, node->heap, &(node->update),
+                                       nullptr, type_cmpl);
 
   /* Read to the partial row the fields that occur in indexes */
 
@@ -1064,22 +1068,15 @@ try_again:
 }
 
 /** Purges the parsed record.
-@param[in,out]	node		row purge node
-@param[in]	undo_rec	undo record to purge */
-#ifdef UNIV_DEBUG
-/**
-@param[in,out]	thr		query thread */
-#endif /* UNIV_DEBUG */
-/**
-@param[in]	updated_extern	whether external columns were updated
-@param[in,out]	thd		current thread
+@param[in,out]  node            row purge node
+@param[in]      undo_rec        undo record to purge
+@param[in,out]  thr             query thread
+@param[in]      updated_extern  whether external columns were updated
+@param[in,out]  thd             current thread
 @return true if purged, false if skipped */
-static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_record_func(
+[[nodiscard]] static bool row_purge_record_func(
     purge_node_t *node, trx_undo_rec_t *undo_rec,
-#ifdef UNIV_DEBUG
-    const que_thr_t *thr,
-#endif /* UNIV_DEBUG */
-    bool updated_extern, THD *thd) {
+    IF_DEBUG(const que_thr_t *thr, ) bool updated_extern, THD *thd) {
   dict_index_t *clust_index;
   bool purged = true;
 
@@ -1103,9 +1100,11 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_record_func(
       if (!updated_extern) {
         break;
       }
-      /* fall through */
+      [[fallthrough]];
     case TRX_UNDO_UPD_EXIST_REC:
-      row_purge_upd_exist_or_extern(thr, node, undo_rec);
+      DBUG_EXECUTE_IF("innodb_purge_sleep_12",
+                      std::this_thread::sleep_for(std::chrono::seconds(5)););
+      row_purge_upd_exist_or_extern_func(IF_DEBUG(thr, ) node, undo_rec);
       MONITOR_INC(MONITOR_N_UPD_EXIST_EXTERN);
       break;
   }
@@ -1115,8 +1114,8 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_record_func(
   }
 
   if (node->found_clust) {
-    btr_pcur_close(&node->pcur);
-    node->found_clust = FALSE;
+    node->pcur.close();
+    node->found_clust = false;
   }
 
   if (node->table != nullptr) {
@@ -1140,13 +1139,12 @@ static MY_ATTRIBUTE((warn_unused_result)) bool row_purge_record_func(
   return (purged);
 }
 
-#ifdef UNIV_DEBUG
-#define row_purge_record(node, undo_rec, thr, updated_extern, thd) \
-  row_purge_record_func(node, undo_rec, thr, updated_extern, thd)
-#else /* UNIV_DEBUG */
-#define row_purge_record(node, undo_rec, thr, updated_extern, thd) \
-  row_purge_record_func(node, undo_rec, updated_extern, thd)
-#endif /* UNIV_DEBUG */
+inline bool row_purge_record(purge_node_t *node, trx_undo_rec_t *undo_rec,
+                             const que_thr_t *thr [[maybe_unused]],
+                             bool updated_extern, THD *thd) {
+  return row_purge_record_func(node, undo_rec, IF_DEBUG(thr, ) updated_extern,
+                               thd);
+}
 
 /** Fetches an undo log record and does the purge for the recorded operation.
  If none left, or the current purge completed, returns the control to the
@@ -1179,7 +1177,7 @@ static void row_purge(purge_node_t *node,       /*!< in: row purge node */
 }
 
 /** Reset the purge query thread.
-@param[in,out]	thr		The query thread to execute */
+@param[in,out]  thr             The query thread to execute */
 static void row_purge_end(que_thr_t *thr) {
   purge_node_t *node;
 
@@ -1210,7 +1208,7 @@ static void row_purge_end(que_thr_t *thr) {
 
 /** Does the purge operation for a single undo log record. This is a high-level
 function used in an SQL execution graph.
-@param[in,out]	thr		The query thread to execute
+@param[in,out]  thr             The query thread to execute
 @return query thread to run next or nullptr */
 que_thr_t *row_purge_step(que_thr_t *thr) {
   purge_node_t *node;
@@ -1222,7 +1220,7 @@ que_thr_t *row_purge_step(que_thr_t *thr) {
   node->ref = nullptr;
   node->index = nullptr;
   node->update = nullptr;
-  node->found_clust = FALSE;
+  node->found_clust = false;
   node->rec_type = ULINT_UNDEFINED;
   node->cmpl_info = ULINT_UNDEFINED;
 
@@ -1280,8 +1278,9 @@ bool purge_node_t::validate_pcur() {
 
   dict_index_t *clust_index = pcur.m_btr_cur.index;
 
-  ulint *offsets = rec_get_offsets(pcur.m_old_rec, clust_index, nullptr,
-                                   pcur.m_old_n_fields, &heap);
+  ulint *offsets =
+      rec_get_offsets(pcur.m_old_rec, clust_index, nullptr, pcur.m_old_n_fields,
+                      UT_LOCATION_HERE, &heap);
 
   /* Here we are comparing the purge ref record and the stored initial
   part in persistent cursor. Both cases we store n_uniq fields of the
@@ -1338,3 +1337,73 @@ bool purge_node_t::check_duplicate_undo_no() const {
   return (true);
 }
 #endif /* UNIV_DEBUG */
+
+void purge_node_t::add_lob_page(dict_index_t *index, const page_id_t &page_id) {
+  const index_id_t id(page_id.space(), index->id);
+  const auto tup = std::make_tuple(id, page_id, index->table->id);
+  ut_ad(m_lob_pages.find(tup) == m_lob_pages.end());
+  m_lob_pages.insert(tup);
+}
+
+void purge_node_t::free_lob_pages() {
+#ifdef UNIV_DEBUG
+  {
+    /** Ensure that the caller does not hold any latches. */
+    sync_allowed_latches check;
+    ut_ad(!sync_check_iterate(check));
+    mtr_t::check_my_thread_mtrs_are_not_latching();
+  }
+#endif
+
+  mtr_t local_mtr;
+
+  THD *thd = current_thd;
+
+  for (const auto &tup : m_lob_pages) {
+    const index_id_t index_id = std::get<0>(tup);
+    const page_id_t &page_id = std::get<1>(tup);
+    const table_id_t table_id = std::get<2>(tup);
+    const space_id_t space_id = page_id.space();
+
+    MDL_ticket *mdl{};
+    dict_table_t *table = dd_table_open_on_id(table_id, thd, &mdl, false, true);
+
+    if (table == nullptr) {
+      continue;
+    }
+
+    const dict_index_t *idx = table->first_index();
+
+    if (idx == nullptr || idx->id != index_id.m_index_id ||
+        idx->space != space_id || idx->page == FIL_NULL ||
+        idx->table->id != table_id) {
+      dd_table_close(table, thd, &mdl, false);
+      continue;
+    }
+
+    dict_index_t *index = const_cast<dict_index_t *>(idx);
+    const page_size_t page_size = dict_table_page_size(index->table);
+
+    fil_space_t *space = fil_space_acquire_silent(space_id);
+
+    if (space != nullptr) {
+      mtr_start(&local_mtr);
+      buf_block_t *block = buf_page_get(page_id, page_size, RW_X_LATCH,
+                                        UT_LOCATION_HERE, &local_mtr);
+
+#ifdef UNIV_DEBUG
+      const page_type_t page_type = block->get_page_type();
+      ut_ad(page_type == FIL_PAGE_TYPE_LOB_FIRST ||
+            page_type == FIL_PAGE_TYPE_ZLOB_FIRST);
+#endif /* UNIV_DEBUG */
+
+      btr_page_free_low(index, block, ULINT_UNDEFINED, &local_mtr);
+      mtr_commit(&local_mtr);
+
+      fil_space_release(space);
+    }
+    dd_table_close(table, thd, &mdl, false);
+  }
+
+  m_lob_pages.clear();
+}
