@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2024, Oracle and/or its affiliates.
+Copyright (c) 1995, 2025, Oracle and/or its affiliates.
 Copyright (c) 2008, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -49,6 +49,7 @@ unsigned int mysql_pfs_key_t::s_count = 0;
 #endif /* HAVE_PSI_INTERFACE */
 
 #ifdef UNIV_PFS_MUTEX
+mysql_pfs_key_t alter_stage_mutex_key;
 /* Key to register autoinc_mutex with performance schema */
 mysql_pfs_key_t autoinc_mutex_key;
 mysql_pfs_key_t autoinc_persisted_mutex_key;
@@ -88,6 +89,7 @@ mysql_pfs_key_t log_write_notifier_mutex_key;
 mysql_pfs_key_t log_flush_notifier_mutex_key;
 mysql_pfs_key_t log_limits_mutex_key;
 mysql_pfs_key_t log_files_mutex_key;
+mysql_pfs_key_t log_governor_mutex_key;
 mysql_pfs_key_t log_cmdq_mutex_key;
 mysql_pfs_key_t log_sn_lock_key;
 mysql_pfs_key_t log_sn_mutex_key;
@@ -103,7 +105,6 @@ mysql_pfs_key_t recv_sys_mutex_key;
 mysql_pfs_key_t recv_writer_mutex_key;
 mysql_pfs_key_t temp_space_rseg_mutex_key;
 mysql_pfs_key_t undo_space_rseg_mutex_key;
-mysql_pfs_key_t trx_sys_rseg_mutex_key;
 mysql_pfs_key_t page_zip_stat_per_index_mutex_key;
 #ifdef UNIV_DEBUG
 mysql_pfs_key_t rw_lock_debug_mutex_key;
@@ -148,6 +149,7 @@ mysql_pfs_key_t clone_snapshot_mutex_key;
 mysql_pfs_key_t parallel_read_mutex_key;
 mysql_pfs_key_t dblwr_mutex_key;
 mysql_pfs_key_t ahi_enabled_mutex_key;
+mysql_pfs_key_t dict_table_stats_compute_mutex_key;
 
 #endif /* UNIV_PFS_MUTEX */
 
@@ -245,10 +247,10 @@ void MutexMonitor::enable() {
   /** Note: We don't add any latch meta-data after startup. Therefore
   there is no need to use a mutex here. */
 
-  LatchMetaData::iterator end = latch_meta.end();
+  auto end = latch_meta.end();
 
-  for (LatchMetaData::iterator it = latch_meta.begin(); it != end; ++it) {
-    if (*it != NULL) {
+  for (auto it = latch_meta.begin(); it != end; ++it) {
+    if (*it != nullptr) {
       (*it)->get_counter()->enable();
     }
   }
@@ -259,10 +261,10 @@ void MutexMonitor::disable() {
   /** Note: We don't add any latch meta-data after startup. Therefore
   there is no need to use a mutex here. */
 
-  LatchMetaData::iterator end = latch_meta.end();
+  auto end = latch_meta.end();
 
-  for (LatchMetaData::iterator it = latch_meta.begin(); it != end; ++it) {
-    if (*it != NULL) {
+  for (auto it = latch_meta.begin(); it != end; ++it) {
+    if (*it != nullptr) {
       (*it)->get_counter()->disable();
     }
   }
@@ -273,17 +275,17 @@ void MutexMonitor::reset() {
   /** Note: We don't add any latch meta-data after startup. Therefore
   there is no need to use a mutex here. */
 
-  LatchMetaData::iterator end = latch_meta.end();
+  auto end = latch_meta.end();
 
-  for (LatchMetaData::iterator it = latch_meta.begin(); it != end; ++it) {
-    if (*it != NULL) {
+  for (auto it = latch_meta.begin(); it != end; ++it) {
+    if (*it != nullptr) {
       (*it)->get_counter()->reset();
     }
   }
 
   mutex_enter(&rw_lock_list_mutex);
 
-  for (auto rw_lock : rw_lock_list) {
+  for (auto *rw_lock : rw_lock_list) {
     rw_lock->count_os_wait = 0;
   }
 
