@@ -1,15 +1,16 @@
-/* Copyright (c) 2001, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2001, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -72,6 +73,8 @@
 #include "storage/myisam/myisamdef.h"
 #include "template_utils.h"
 
+struct CHARSET_INFO;
+
 /* search with boolean queries */
 
 static double _wghts[11] = {
@@ -143,9 +146,9 @@ struct FTB : public FT_INFO {
 };
 
 static int FTB_WORD_cmp(void *v_v, uchar *u_a, uchar *u_b) {
-  my_off_t *v = static_cast<my_off_t *>(v_v);
-  FTB_WORD *a = pointer_cast<FTB_WORD *>(u_a);
-  FTB_WORD *b = pointer_cast<FTB_WORD *>(u_b);
+  auto *v = static_cast<my_off_t *>(v_v);
+  auto *a = pointer_cast<FTB_WORD *>(u_a);
+  auto *b = pointer_cast<FTB_WORD *>(u_b);
   int i;
 
   /* if a==curdoc, take it as  a < b */
@@ -166,13 +169,13 @@ struct MY_FTB_PARAM {
 
 static int ftb_query_add_word(MYSQL_FTPARSER_PARAM *param, char *word,
                               int word_len, MYSQL_FTPARSER_BOOLEAN_INFO *info) {
-  MY_FTB_PARAM *ftb_param = (MY_FTB_PARAM *)param->mysql_ftparam;
+  auto *ftb_param = (MY_FTB_PARAM *)param->mysql_ftparam;
   FTB_WORD *ftbw;
   FTB_EXPR *ftbe, *tmp_expr;
   FT_WORD *phrase_word;
   LIST *tmp_element;
-  int r = info->weight_adjust;
-  float weight =
+  int const r = info->weight_adjust;
+  auto weight =
       (float)(info->wasign ? nwghts : wghts)[(r > 5) ? 5 : ((r < -5) ? -5 : r)];
 
   switch (info->type) {
@@ -204,7 +207,7 @@ static int ftb_query_add_word(MYSQL_FTPARSER_PARAM *param, char *word,
       for (tmp_expr = ftb_param->ftbe; tmp_expr->up; tmp_expr = tmp_expr->up)
         if (!(tmp_expr->flags & FTB_FLAG_YES)) break;
       ftbw->max_docid_expr = tmp_expr;
-      /* fall through */
+      [[fallthrough]];
     case FT_TOKEN_STOPWORD:
       if (!ftb_param->up_quot) break;
       phrase_word = (FT_WORD *)ftb_param->ftb->mem_root.Alloc(sizeof(FT_WORD));
@@ -263,10 +266,10 @@ static int ftb_query_add_word(MYSQL_FTPARSER_PARAM *param, char *word,
 
 static int ftb_parse_query_internal(MYSQL_FTPARSER_PARAM *param, char *query,
                                     int len) {
-  MY_FTB_PARAM *ftb_param = (MY_FTB_PARAM *)param->mysql_ftparam;
+  auto *ftb_param = (MY_FTB_PARAM *)param->mysql_ftparam;
   MYSQL_FTPARSER_BOOLEAN_INFO info;
   const CHARSET_INFO *cs = ftb_param->ftb->charset;
-  uchar **start = (uchar **)&query;
+  auto **start = (uchar **)&query;
   uchar *end = (uchar *)query + len;
   FT_WORD w;
 
@@ -311,7 +314,7 @@ static int _ftb_no_dupes_cmp(const void *, const void *a, const void *b) {
   When performing prefix search (a word with truncation operator), we
   must preserve original prefix to ensure that characters which may be
   expanded/contracted do not break the prefix. This is done by storing
-  newly found key immediatly after the original word in ftbw->word
+  newly found key immediately after the original word in ftbw->word
   buffer.
 
   ftbw->word= LENGTH WORD [ LENGTH1 WORD1 ] WEIGHT REFERENCE
@@ -331,9 +334,10 @@ static int _ft2_search_no_lock(FTB *ftb, FTB_WORD *ftbw, bool init_search) {
   MI_INFO *info = ftb->info;
   uint off = 0, extra = HA_FT_WLEN + info->s->rec_reflength;
   uchar *lastkey_buf = ftbw->word + ftbw->off;
-  uint max_word_length = (ftbw->flags & FTB_FLAG_TRUNC)
-                             ? MI_MAX_KEY_BUFF
-                             : ((ftbw->len) * ftb->charset->mbmaxlen) + extra;
+  uint const max_word_length =
+      (ftbw->flags & FTB_FLAG_TRUNC)
+          ? MI_MAX_KEY_BUFF
+          : ((ftbw->len) * ftb->charset->mbmaxlen) + extra;
 
   if (ftbw->flags & FTB_FLAG_TRUNC) lastkey_buf += ftbw->len;
 
@@ -393,8 +397,8 @@ static int _ft2_search_no_lock(FTB *ftb, FTB_WORD *ftbw, bool init_search) {
         */
         ftb->state = FTB::INDEX_DONE;
         return 1; /* search is done */
-      } else
-        return 0;
+      }
+      return 0;
     }
 
     /*
@@ -484,7 +488,7 @@ static void _ftb_init_index_search(FT_INFO *ftb_base) {
            ftbe->up->flags |= FTB_FLAG_TRUNC, ftbe = ftbe->up) {
         if (ftbe->flags & FTB_FLAG_NO || /* 2 */
             ftbe->up->ythresh - ftbe->up->yweaks >
-                ((ftbe->flags & FTB_FLAG_YES) ? 1 : 0)) /* 1 */
+                ((ftbe->flags & FTB_FLAG_YES) ? 1U : 0)) /* 1 */
         {
           FTB_EXPR *top_ftbe = ftbe->up;
           ftbw->docid[0] = HA_OFFSET_ERROR;
@@ -527,10 +531,10 @@ FT_INFO *ft_init_boolean_search(MI_INFO *info, uint keynr, uchar *query,
   assert(keynr == NO_SUCH_KEY || cs == info->s->keyinfo[keynr].seg->charset);
   ftb->with_scan = 0;
   ftb->lastpos = HA_OFFSET_ERROR;
-  memset(&ftb->no_dupes, 0, sizeof(TREE));
+  memset((void *)&ftb->no_dupes, 0, sizeof(TREE));
   ftb->last_word = nullptr;
 
-  init_alloc_root(PSI_INSTRUMENT_ME, &ftb->mem_root, 1024, 1024);
+  ::new ((void *)&ftb->mem_root) MEM_ROOT(PSI_INSTRUMENT_ME, 1024);
   ftb->queue.max_elements = 0;
   if (!(ftbe = (FTB_EXPR *)ftb->mem_root.Alloc(sizeof(FTB_EXPR)))) goto err;
   ftbe->weight = 1;
@@ -565,9 +569,9 @@ FT_INFO *ft_init_boolean_search(MI_INFO *info, uint keynr, uchar *query,
   std::sort(ftb->list, ftb->list + ftb->queue.elements,
             [ftb](FTB_WORD *a, FTB_WORD *b) {
               /* ORDER BY word, ndepth */
-              int i = ha_compare_text(ftb->charset, (uchar *)a->word + 1,
-                                      a->len - 1, (uchar *)b->word + 1,
-                                      b->len - 1, false);
+              int const i = ha_compare_text(ftb->charset, (uchar *)a->word + 1,
+                                            a->len - 1, (uchar *)b->word + 1,
+                                            b->len - 1, false);
               if (i != 0) return i < 0;
               return a->ndepth < b->ndepth;
             });
@@ -575,7 +579,7 @@ FT_INFO *ft_init_boolean_search(MI_INFO *info, uint keynr, uchar *query,
   ftb->state = FTB::READY;
   return (FT_INFO *)ftb;
 err:
-  free_root(&ftb->mem_root, MYF(0));
+  ftb->mem_root.Clear();
   my_free(ftb);
   return nullptr;
 }
@@ -589,12 +593,12 @@ struct MY_FTB_PHRASE_PARAM {
   uint match;
 };
 
-static int ftb_phrase_add_word(
-    MYSQL_FTPARSER_PARAM *param, char *word, int word_len,
-    MYSQL_FTPARSER_BOOLEAN_INFO *boolean_info MY_ATTRIBUTE((unused))) {
-  MY_FTB_PHRASE_PARAM *phrase_param =
-      (MY_FTB_PHRASE_PARAM *)param->mysql_ftparam;
-  FT_WORD *w = (FT_WORD *)phrase_param->document->data;
+static int ftb_phrase_add_word(MYSQL_FTPARSER_PARAM *param, char *word,
+                               int word_len,
+                               MYSQL_FTPARSER_BOOLEAN_INFO *boolean_info
+                               [[maybe_unused]]) {
+  auto *phrase_param = (MY_FTB_PHRASE_PARAM *)param->mysql_ftparam;
+  auto *w = (FT_WORD *)phrase_param->document->data;
   LIST *phrase, *document;
   w->pos = (uchar *)word;
   w->len = word_len;
@@ -607,8 +611,8 @@ static int ftb_phrase_add_word(
      comparing the same word twice. */
   for (phrase = phrase_param->phrase, document = phrase_param->document->next;
        phrase; phrase = phrase->next, document = document->next) {
-    FT_WORD *phrase_word = (FT_WORD *)phrase->data;
-    FT_WORD *document_word = (FT_WORD *)document->data;
+    auto *phrase_word = (FT_WORD *)phrase->data;
+    auto *document_word = (FT_WORD *)document->data;
     if (my_strnncoll(phrase_param->cs, (uchar *)phrase_word->pos,
                      phrase_word->len, (uchar *)document_word->pos,
                      document_word->len))
@@ -621,8 +625,7 @@ static int ftb_phrase_add_word(
 static int ftb_check_phrase_internal(MYSQL_FTPARSER_PARAM *param,
                                      char *document, int len) {
   FT_WORD word;
-  MY_FTB_PHRASE_PARAM *phrase_param =
-      (MY_FTB_PHRASE_PARAM *)param->mysql_ftparam;
+  auto *phrase_param = (MY_FTB_PHRASE_PARAM *)param->mysql_ftparam;
   const uchar *docend = (uchar *)document + len;
   while (ft_simple_get_word(phrase_param->cs, (uchar **)&document, docend,
                             &word, false)) {
@@ -681,7 +684,7 @@ static int _ftb_climb_the_tree(FTB *ftb, FTB_WORD *ftbw,
   FTB_EXPR *ftbe;
   float weight = ftbw->weight;
   int yn_flag = ftbw->flags, ythresh, mode = (ftsi_orig != nullptr);
-  my_off_t curdoc = ftbw->docid[mode];
+  my_off_t const curdoc = ftbw->docid[mode];
   struct st_mysql_ftparser *parser =
       ftb->keynr == NO_SUCH_KEY ? &ft_default_parser
                                 : ftb->info->s->keyinfo[ftb->keynr].parser;
@@ -811,10 +814,11 @@ struct MY_FTB_FIND_PARAM {
   FT_SEG_ITERATOR *ftsi;
 };
 
-static int ftb_find_relevance_add_word(
-    MYSQL_FTPARSER_PARAM *param, char *word, int len,
-    MYSQL_FTPARSER_BOOLEAN_INFO *boolean_info MY_ATTRIBUTE((unused))) {
-  MY_FTB_FIND_PARAM *ftb_param = (MY_FTB_FIND_PARAM *)param->mysql_ftparam;
+static int ftb_find_relevance_add_word(MYSQL_FTPARSER_PARAM *param, char *word,
+                                       int len,
+                                       MYSQL_FTPARSER_BOOLEAN_INFO *boolean_info
+                                       [[maybe_unused]]) {
+  auto *ftb_param = (MY_FTB_FIND_PARAM *)param->mysql_ftparam;
   FTB *ftb = ftb_param->ftb;
   FTB_WORD *ftbw;
   int a, b, c;
@@ -854,10 +858,8 @@ static int ftb_find_relevance_add_word(
     if (ha_compare_text(ftb->charset, (uchar *)word, len,
                         (uchar *)ftbw->word + 1, ftbw->len - 1,
                         (bool)(ftbw->flags & FTB_FLAG_TRUNC))) {
-      if (ftb->with_scan & FTB_FLAG_TRUNC)
-        continue;
-      else
-        break;
+      if (ftb->with_scan & FTB_FLAG_TRUNC) continue;
+      break;
     }
     if (ftbw->docid[1] == ftb->info->lastpos) continue;
     ftbw->docid[1] = ftb->info->lastpos;
@@ -868,7 +870,7 @@ static int ftb_find_relevance_add_word(
 
 static int ftb_find_relevance_parse(MYSQL_FTPARSER_PARAM *param, char *doc,
                                     int len) {
-  MY_FTB_FIND_PARAM *ftb_param = (MY_FTB_FIND_PARAM *)param->mysql_ftparam;
+  auto *ftb_param = (MY_FTB_FIND_PARAM *)param->mysql_ftparam;
   FTB *ftb = ftb_param->ftb;
   uchar *end = (uchar *)doc + len;
   FT_WORD w;
@@ -882,7 +884,7 @@ extern "C" float ft_boolean_find_relevance(FT_INFO *ftb_base, uchar *record,
   FTB *ftb = (FTB *)ftb_base;
   FTB_EXPR *ftbe;
   FT_SEG_ITERATOR ftsi, ftsi2;
-  my_off_t docid = ftb->info->lastpos;
+  my_off_t const docid = ftb->info->lastpos;
   MY_FTB_FIND_PARAM ftb_param;
   MYSQL_FTPARSER_PARAM *param;
   struct st_mysql_ftparser *parser =
@@ -929,9 +931,9 @@ extern "C" float ft_boolean_find_relevance(FT_INFO *ftb_base, uchar *record,
   if (ftbe->docid[1] == docid && ftbe->cur_weight > 0 &&
       ftbe->yesses >= ftbe->ythresh && !ftbe->nos) { /* row matched ! */
     return ftbe->cur_weight;
-  } else { /* match failed ! */
-    return 0.0;
   }
+  /* match failed ! */
+  return 0.0;
 }
 
 extern "C" void ft_boolean_close_search(FT_INFO *ftb_base) {
@@ -939,7 +941,7 @@ extern "C" void ft_boolean_close_search(FT_INFO *ftb_base) {
   if (is_tree_inited(&ftb->no_dupes)) {
     delete_tree(&ftb->no_dupes);
   }
-  free_root(&ftb->mem_root, MYF(0));
+  ftb->mem_root.Clear();
   my_free(ftb);
 }
 
