@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2010, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2010, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,9 +36,7 @@ public interface Db {
 
     public Dictionary getDictionary();
 
-    public ClusterTransaction startTransaction(String joinTransactionId);
-
-    public boolean isRetriable(ClusterJDatastoreException ex);
+    public ClusterTransaction startTransaction();
 
     public void assertNotClosed(String where);
 }
