@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2012, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2012, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,6 +34,7 @@ import java.sql.Date;
 import java.sql.Time;
 import java.sql.Timestamp;
 
+import com.mysql.clusterj.ClusterJDatastoreException;
 import com.mysql.clusterj.ClusterJFatalInternalException;
 import com.mysql.clusterj.ClusterJUserException;
 import com.mysql.clusterj.ColumnMetadata;
@@ -80,19 +82,6 @@ public class NdbRecordSmartValueHandlerImpl implements SmartValueHandler {
     /** My logger */
     static final Logger logger = LoggerFactoryService.getFactory().getInstance(InvocationHandlerImpl.class);
 
-    /** Finalize this object. This method is called by the garbage collector
-     * when the proxy that delegates to this object is no longer reachable.
-     */
-    @SuppressWarnings("deprecation")
-    protected void finalize() throws Throwable {
-        if (logger.isDetailEnabled()) logger.detail("NdbRecordSmartValueHandler.finalize");
-        try {
-            release();
-        } finally {
-            super.finalize();
-        }
-    }
-
     /** Release any resources associated with this object.
      * This method is called by the owner of this object.
      */
@@ -108,7 +97,6 @@ public class NdbRecordSmartValueHandlerImpl implements SmartValueHandler {
         domainFieldHandlers = null;
         fieldNumberToColumnNumberMap = null;
         transientValues = null;
-        proxy = null;
     }
 
     /** Was this value handler released? */
@@ -141,8 +129,6 @@ public class NdbRecordSmartValueHandlerImpl implements SmartValueHandler {
 
     private boolean[] transientModified; 
 
-    private Object proxy;
-
     public NdbRecordSmartValueHandlerImpl(DomainTypeHandlerImpl<?> domainTypeHandler) {
         this.domainTypeHandler = domainTypeHandler;
         this.domainFieldHandlers = domainTypeHandler.getFieldHandlers();
@@ -727,6 +713,15 @@ public class NdbRecordSmartValueHandlerImpl implements SmartValueHandler {
         this.found = found;
     }
 
+    private DomainFieldHandler getDomainFieldHandler(int index) {
+        try {
+            return domainFieldHandlers[index];
+        } catch (Throwable t) {
+            throw ClusterJDatastoreException.forSchemaChange(
+                "Obsolete schema in ValueHandler", -3, t);
+        }
+    }
+
     /** Return the value of a dynamic field stored in the NdbRecord buffer.
      * @param fieldNumber the field number
      * @return the value from data storage
@@ -737,7 +732,7 @@ public class NdbRecordSmartValueHandlerImpl implements SmartValueHandler {
         if (columnId < 0) {
             return transientValues[-1 - columnId];
         }
-        return domainFieldHandlers[fieldNumber].objectGetValue(this);
+        return getDomainFieldHandler(fieldNumber).objectGetValue(this);
     }
 
     public void set(int fieldNumber, Object value) {
@@ -747,7 +742,7 @@ public class NdbRecordSmartValueHandlerImpl implements SmartValueHandler {
             transientValues[-1 - columnId] = value;
             transientModified[-1 - columnId] = true;
         } else {
-            domainFieldHandlers[fieldNumber].objectSetValue(value, this);
+            getDomainFieldHandler(fieldNumber).objectSetValue(value, this);
         }
     }
 
@@ -789,13 +784,4 @@ public class NdbRecordSmartValueHandlerImpl implements SmartValueHandler {
 
     public void setCacheManager(CacheManager cm) {
     }
-
-    public void setProxy(Object proxy) {
-        this.proxy = proxy;
-    }
-
-    public Object getProxy() {
-        return this.proxy;
-    }
-
 }
