@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -1366,7 +1366,7 @@ class Ndb {
    * pollEvents2 will also return >0 when there is an event data
    * representing empty or error epoch available on the head of the event queue.
    */
-  int pollEvents2(int aMillisecondNumber, Uint64 *highestQueuedEpoch = 0);
+  int pollEvents2(int aMillisecondNumber, Uint64 *highestQueuedEpoch = nullptr);
 
   /**
    * Check if higher queued epochs have been seen by the last
@@ -1416,7 +1416,7 @@ class Ndb {
    * - removes empty epochs from the event queue head until a regular
    *   event data is found or the whole queue is processed.
    */
-  int pollEvents(int aMillisecondNumber, Uint64 *latestGCI = 0);
+  int pollEvents(int aMillisecondNumber, Uint64 *latestGCI = nullptr);
 
   /**
    * Returns the event operation associated with the dequeued
@@ -1549,7 +1549,7 @@ class Ndb {
 
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
   int flushIncompleteEvents(Uint64 gci);
-  NdbEventOperation *getEventOperation(NdbEventOperation *eventOp = 0);
+  NdbEventOperation *getEventOperation(NdbEventOperation *eventOp = nullptr);
   Uint64 getLatestGCI();
   void forceGCP();
   void setReportThreshEventGCISlip(unsigned thresh);
@@ -1584,8 +1584,9 @@ class Ndb {
    *
    * @return NdbTransaction object, or NULL on failure.
    */
-  NdbTransaction *startTransaction(const NdbDictionary::Table *table = 0,
-                                   const char *keyData = 0, Uint32 keyLen = 0);
+  NdbTransaction *startTransaction(const NdbDictionary::Table *table = nullptr,
+                                   const char *keyData = nullptr,
+                                   Uint32 keyLen = 0);
 
   /**
    * Structure for passing in pointers to distribution key values
@@ -1708,7 +1709,8 @@ class Ndb {
    */
   NdbTransaction *startTransaction(const NdbDictionary::Table *table,
                                    const struct Key_part_ptr *keyData,
-                                   void *xfrmbuf = 0, Uint32 xfrmbuflen = 0);
+                                   void *xfrmbuf = nullptr,
+                                   Uint32 xfrmbuflen = 0);
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
   /**
    * Unlike startTransaction function above, this function will not try to
@@ -1760,8 +1762,8 @@ class Ndb {
    */
   static int computeHash(Uint32 *hashvalueptr,
                          const NdbDictionary::Table *table,
-                         const struct Key_part_ptr *keyData, void *xfrmbuf = 0,
-                         Uint32 xfrmbuflen = 0);
+                         const struct Key_part_ptr *keyData,
+                         void *xfrmbuf = nullptr, Uint32 xfrmbuflen = 0);
 #ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
   /**
    * Unlike computeHash function above, this function will not try to allocate
@@ -2146,7 +2148,7 @@ class Ndb {
   Uint32 insert_sent_list(NdbTransaction *);
 
   int sendRecSignal(Uint16 aNodeId, Uint32 aWaitState, NdbApiSignal *aSignal,
-                    Uint32 nodeSequence, Uint32 *ret_conn_seq = 0);
+                    Uint32 nodeSequence, Uint32 *ret_conn_seq = nullptr);
 
   // Get block number of this NDBAPI object
   int getBlockNumber();
@@ -2220,13 +2222,13 @@ class Ndb {
   static BaseString internalize_table_name(const char *db_name,
                                            const char *schema,
                                            const char *table_name);
-  const BaseString internalize_table_name(const char *external_name) const;
+  BaseString internalize_table_name(const char *external_name) const;
 
   static const char *externalizeIndexName(const char *internalIndexName,
                                           bool fullyQualifiedNames);
 
-  static const BaseString getDatabaseFromInternalName(const char *internalName);
-  static const BaseString getSchemaFromInternalName(const char *internalName);
+  static BaseString getDatabaseFromInternalName(const char *internalName);
+  static BaseString getSchemaFromInternalName(const char *internalName);
 
   Uint64 allocate_transaction_id();
 
