@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -116,7 +117,7 @@ class NdbReceiver {
    * At setup
    */
   class NdbRecAttr *getValue(const class NdbColumnImpl *, char *user_dst_ptr);
-  void getValues(const NdbRecord *, char *);
+  void getValues(const NdbRecord *, char *, char *, Uint32);
   void prepareSend();
 
   static void calculate_batch_size(const NdbImpl &, Uint32 parallelism,
@@ -148,6 +149,7 @@ class NdbReceiver {
     during a scan using NdbRecord.
   */
   void do_setup_ndbrecord(const NdbRecord *ndb_record, char *row_buffer,
+                          char *row_side_buffer, Uint32 row_side_buffer_size,
                           bool read_range_no, bool read_key_info);
 
   /**
@@ -181,6 +183,8 @@ class NdbReceiver {
 
   /* The (single) current row in 'unpacked' NdbRecord format */
   char *m_row_buffer;
+  char *m_row_side_buffer;
+  Uint32 m_row_side_buffer_size;
 
   /* Block of memory used to buffer all rows in a batch during scan. */
   NdbReceiverBuffer *m_recv_buffer;
@@ -241,7 +245,9 @@ class NdbReceiver {
    * Return the number of words consumed.
    */
   static Uint32 unpackNdbRecord(const NdbRecord *record, Uint32 bmlen,
-                                const Uint32 *aDataPtr, char *row);
+                                const Uint32 *aDataPtr, char *row,
+                                char *row_side_buffer,
+                                Uint32 row_side_buffer_size);
 
   /**
    * Handle a stream of field values, both 'READ_PACKED' and plain
