@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -467,8 +467,9 @@ class NdbScanOperation : public NdbOperation {
    */
   const NdbOperation *lockCurrentTuple(
       NdbTransaction *takeOverTrans, const NdbRecord *result_rec,
-      char *result_row = 0, const unsigned char *result_mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      char *result_row = nullptr, const unsigned char *result_mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
 
   /*
    * Update the current tuple, NdbRecord version.
@@ -478,8 +479,9 @@ class NdbScanOperation : public NdbOperation {
    */
   const NdbOperation *updateCurrentTuple(
       NdbTransaction *takeOverTrans, const NdbRecord *attr_rec,
-      const char *attr_row, const unsigned char *mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      const char *attr_row, const unsigned char *mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
 
   /* Delete the current tuple. NdbRecord version.
    * The tuple can be read before being deleted.  Specify the columns to read
@@ -489,8 +491,9 @@ class NdbScanOperation : public NdbOperation {
    */
   const NdbOperation *deleteCurrentTuple(
       NdbTransaction *takeOverTrans, const NdbRecord *result_rec,
-      char *result_row = 0, const unsigned char *result_mask = 0,
-      const NdbOperation::OperationOptions *opts = 0, Uint32 sizeOfOptions = 0);
+      char *result_row = nullptr, const unsigned char *result_mask = nullptr,
+      const NdbOperation::OperationOptions *opts = nullptr,
+      Uint32 sizeOfOptions = 0);
 
   /**
    * Get NdbTransaction object for this scan operation
@@ -509,7 +512,8 @@ class NdbScanOperation : public NdbOperation {
                    NdbOperation::Type aType = NdbOperation::TableScan);
   ~NdbScanOperation() override;
 
-  NdbRecAttr *getValue_impl(const NdbColumnImpl *, char *aValue = 0) override;
+  NdbRecAttr *getValue_impl(const NdbColumnImpl *,
+                            char *aValue = nullptr) override;
   NdbRecAttr *getValue_NdbRecord_scan(const NdbColumnImpl *, char *aValue);
   NdbRecAttr *getValue_NdbRecAttr_scan(const NdbColumnImpl *, char *aValue);
 
@@ -522,8 +526,8 @@ class NdbScanOperation : public NdbOperation {
                           Uint32 sizeOfPartInfo, Ndb::PartitionSpec &partValue);
   int getPartValueFromInfo(const Ndb::PartitionSpec *partInfo,
                            const NdbTableImpl *table, Uint32 *partValue);
-  int generatePackedReadAIs(const NdbRecord *reseult_record, bool &haveBlob,
-                            const Uint32 *readMask);
+  int generatePackedReadAIs(const NdbRecord *result_record,
+                            bool &usesBlobHandles, const Uint32 *readMask);
   int scanImpl(const NdbScanOperation::ScanOptions *options,
                const Uint32 *readMask);
   int scanTableImpl(const NdbRecord *result_record,
@@ -637,7 +641,7 @@ class NdbScanOperation : public NdbOperation {
   int send_next_scan(Uint32 cnt, bool close);
   void receiver_delivered(NdbReceiver *);
   void receiver_completed(NdbReceiver *);
-  void execCLOSE_SCAN_REP();
+  void execCLOSE_SCAN_REP(Uint32 errorCode, bool needClose);
 
   int getKeyFromKEYINFO20(Uint32 *data, Uint32 &size);
   NdbOperation *takeOverScanOp(OperationType opType, NdbTransaction *);
@@ -719,6 +723,13 @@ class NdbScanOperation : public NdbOperation {
    */
   NdbBlob *getBlobHandle(const char *anAttrName) const override;
   NdbBlob *getBlobHandle(Uint32 anAttrId) const override;
+
+ protected:
+  /*
+    Owned by the receiver thread
+    Read by API thread under mutex protection
+  */
+  Uint32 m_kernel_error_code;
 };
 
 inline NdbOperation *NdbScanOperation::lockCurrentTuple() {
@@ -745,7 +756,7 @@ inline int NdbScanOperation::deleteCurrentTuple() {
 
 inline int NdbScanOperation::deleteCurrentTuple(NdbTransaction *takeOverTrans) {
   void *res = takeOverScanOp(NdbOperation::DeleteRequest, takeOverTrans);
-  if (res == 0) return -1;
+  if (res == nullptr) return -1;
   return 0;
 }
 
