@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,28 +29,28 @@
 //      TransporterRegistry
 //
 //  DESCRIPTION
-//      TransporterRegistry (singleton) is the interface to the 
-//      transporter layer. It handles transporter states and 
+//      TransporterRegistry (singleton) is the interface to the
+//      transporter layer. It handles transporter states and
 //      holds the transporter arrays.
 //
 //***************************************************************************/
 #ifndef TransporterRegistry_H
 #define TransporterRegistry_H
 
-#include <assert.h>
+#include <cassert>
 #include "ndb_config.h"
 
 #if defined(HAVE_EPOLL_CREATE)
 #include <sys/epoll.h>
 #endif
-#include "TransporterDefinitions.hpp"
-#include <SocketServer.hpp>
 #include <SocketClient.hpp>
+#include <SocketServer.hpp>
+#include "TransporterDefinitions.hpp"
 
 #include <mgmapi/mgmapi.h>
 
-#include <NodeBitmask.hpp>
 #include <NdbMutex.h>
+#include <NodeBitmask.hpp>
 
 #include "portlib/NdbTick.h"
 #include "portlib/ndb_sockaddr.h"
@@ -76,19 +77,18 @@
 // A transporter is always in an IOState.
 // NoHalt is used initially and as long as it is no restrictions on
 // sending or receiving.
+// Note that the Halt states are a bitmask.
+// Also note that only NoHalt and HaltIO seems to be used.
 enum IOState {
-  NoHalt     = 0,
-  HaltInput  = 1,
+  NoHalt = 0,
+  HaltInput = 1,
   HaltOutput = 2,
-  HaltIO     = 3
+  HaltIO = HaltInput | HaltOutput
 };
 
-
-static const char *performStateString[] = 
-  { "is connected",
-    "is trying to connect",
-    "does nothing",
-    "is trying to disconnect" };
+static const char *performStateString[] = {
+    "is connected", "is trying to connect", "does nothing",
+    "is trying to disconnect"};
 
 class Transporter;
 class TCP_Transporter;
@@ -99,19 +99,18 @@ class TransporterRegistry;
 class SocketAuthenticator;
 
 class TransporterService : public SocketServer::Service {
-  SocketAuthenticator * m_auth;
-  TransporterRegistry * m_transporter_registry;
-public:
-  TransporterService(SocketAuthenticator *auth= nullptr)
-  {
-    m_auth= auth;
-    m_transporter_registry= nullptr;
+  SocketAuthenticator *m_auth;
+  TransporterRegistry *m_transporter_registry;
+
+ public:
+  TransporterService(SocketAuthenticator *auth = nullptr) {
+    m_auth = auth;
+    m_transporter_registry = nullptr;
   }
-  void setTransporterRegistry(TransporterRegistry *t)
-  {
-    m_transporter_registry= t;
+  void setTransporterRegistry(TransporterRegistry *t) {
+    m_transporter_registry = t;
   }
-  SocketServer::Session * newSession(ndb_socket_t socket) override;
+  SocketServer::Session *newSession(NdbSocket &&socket) override;
 };
 
 /**
@@ -120,18 +119,17 @@ public:
  *   State for pollReceive/performReceive
  *   Moved into own class to enable multiple receive threads
  */
-struct TransporterReceiveData
-{
+struct TransporterReceiveData {
   TransporterReceiveData();
   ~TransporterReceiveData();
 
-  bool init (unsigned maxTransporters);
+  bool init(unsigned maxTransporters);
 
   /**
    * Add a transporter to epoll_set
    *   does nothing if epoll not active
    */
-  bool epoll_add(Transporter*);
+  bool epoll_add(Transporter *);
 
   /**
    * Bitmask of transporters currently handled by this instance
@@ -146,7 +144,7 @@ struct TransporterReceiveData
 
   /**
    * Bitmask of transporters that has already received data buffered
-   * inside its transporter. Possibly "carried over" from last 
+   * inside its transporter. Possibly "carried over" from last
    * performReceive
    */
   TrpBitmask m_has_data_transporters;
@@ -154,7 +152,7 @@ struct TransporterReceiveData
   /**
    * Subset of m_has_data_transporters which we completed handling
    * of in previous ::performReceive before we was interrupted due
-   * to lack of job buffers. Will skip these when we later retry 
+   * to lack of job buffers. Will skip these when we later retry
    * ::performReceive in order to avoid starvation of non-handled
    * transporters.
    */
@@ -163,14 +161,8 @@ struct TransporterReceiveData
   /**
    * Bitmask of transporters having received corrupted or unsupported
    * message. No more unpacking and delivery of messages allowed.
-   *
-   * OJA FIXME:
-   *    Documented as 'Bitmask of transporters' (TrpBitmask)
-   *    Declared and used(!) as a NodeBitMask!
-   *
-   * Could it possibly be the root cause of the multiTransporter checksum bug?
    */
-  NodeBitmask m_bad_data_transporters;
+  TrpBitmask m_bad_data_transporters;
 
   /**
    * Last transporter received from if unable to complete all transporters
@@ -210,19 +202,19 @@ class TransporterSendBufferHandle;
  * @class TransporterRegistry
  * @brief ...
  */
-class TransporterRegistry 
-{
+class TransporterRegistry {
   friend class SHM_Transporter;
   friend class SHM_Writer;
   friend class Transporter;
   friend class TransporterService;
-public:
- /**
-  * Constructor
-  */
+
+ public:
+  /**
+   * Constructor
+   */
   TransporterRegistry(TransporterCallback *callback,
-                      TransporterReceiveHandle * receiveHandle,
-		      unsigned maxTransporters = MAX_NTRANSPORTERS);
+                      TransporterReceiveHandle *receiveHandle,
+                      unsigned maxTransporters = MAX_NTRANSPORTERS);
 
   /**
    * this handle will be used in the client connect thread
@@ -230,7 +222,7 @@ public:
    * (if set) is destroyed, and this is destroyed by the destructor
    */
   void set_mgm_handle(NdbMgmHandle h);
-  NdbMgmHandle get_mgm_handle(void) { return m_mgm_handle; }
+  NdbMgmHandle get_mgm_handle() { return m_mgm_handle; }
 
   bool init(NodeId localNodeId);
 
@@ -238,14 +230,14 @@ public:
    * Iff using non-default TransporterReceiveHandle's
    *   they need to get initialized
    */
-  bool init(TransporterReceiveHandle&);
+  bool init(TransporterReceiveHandle &);
 
   /**
    * Initialize TLS context. Cannot be called prior to init(NodeId).
    * Returns true on success.
    */
-  bool init_tls(const char * search_path, int node_type,
-                bool is_primary, int mgm_tls_requirement_level);
+  bool init_tls(const char *search_path, int node_type,
+                int mgm_tls_requirement_level);
 
   /**
      Perform handshaking of a client connection to accept it
@@ -257,17 +249,11 @@ public:
      @param sockfd           the socket to handshake
      @param msg              error message describing why handshake failed,
                              to be filled in when function return
-     @param close_with_reset allows the function to indicate to the caller
-                             how the socket should be closed when function
-                             returns false
      @param log_failure      whether a failure to connect is log-worthy
 
      @returns false on failure and true on success
   */
-  bool connect_server(NdbSocket & sockfd,
-                      BaseString& msg,
-                      bool& close_with_reset,
-                      bool& log_failure);
+  bool connect_server(NdbSocket &&sockfd, BaseString &msg, bool &log_failure);
 
   bool connect_client(NdbMgmHandle *h);
 
@@ -275,7 +261,7 @@ public:
    * Given a hostname and port, creates a NdbMgmHandle, turns it into
    * a transporter, and returns the socket.
    */
-  NdbSocket connect_ndb_mgmd(const char* server_name,
+  NdbSocket connect_ndb_mgmd(const char *server_name,
                              unsigned short server_port);
 
   /**
@@ -289,16 +275,21 @@ public:
    * Multi_Transporter changes. There is a mutex protecting changes
    * to those data structures.
    */
-  void lockMultiTransporters();
-  void unlockMultiTransporters();
-  void insert_allTransporters(Transporter*);
-  void remove_allTransporters(Transporter*);
-  void insert_node_transporter(NodeId, Transporter*);
-  bool isMultiTransporter(Transporter*);
-  void switch_active_trp(Multi_Transporter*);
-  Uint32 get_num_active_transporters(Multi_Transporter*);
-private:
+  void lockMultiTransporters() const;
+  void unlockMultiTransporters() const;
+  void insert_allTransporters(Transporter *);
+  void remove_allTransporters(Transporter *);
+  static void switch_active_trp(Multi_Transporter *);
+  static Uint32 get_num_active_transporters(Multi_Transporter *);
 
+  /**
+   * An inactive transporter is part of a Multi_transporter.
+   * It is currently not in use, until activated by switch_active_trp().
+   * It is always in state DISCONNECTED.
+   */
+  bool is_inactive_trp(TrpId trpId) const;
+
+ private:
   NdbMutex *theMultiTransporterMutex;
   /**
    * Report the dynamically allocated ports to ndb_mgmd so that clients
@@ -310,7 +301,7 @@ private:
    * Remove all transporters
    */
   void removeAll();
-  
+
   /**
    * Disconnect all transporters
    */
@@ -319,24 +310,23 @@ private:
   /**
    * Reset awake state on shared memory transporters before sleep.
    */
-  int reset_shm_awake_state(TransporterReceiveHandle& recvdata,
-                            bool& sleep_state_set);
+  int reset_shm_awake_state(TransporterReceiveHandle &recvdata,
+                            bool &sleep_state_set);
 
   /**
    * Set awake state on shared memory transporters after sleep.
    */
-  void set_shm_awake_state(TransporterReceiveHandle& recvdata);
-
-public:
+  void set_shm_awake_state(TransporterReceiveHandle &recvdata);
 
+ public:
   /**
-   * Stops the server, disconnects all the transporter 
+   * Stops the server, disconnects all the transporter
    * and deletes them and remove it from the transporter arrays
    */
   virtual ~TransporterRegistry();
 
-  bool start_service(SocketServer& server);
-  struct NdbThread* start_clients();
+  bool start_service(SocketServer &server);
+  struct NdbThread *start_clients();
   bool stop_clients();
   void start_clients_thread();
 
@@ -345,7 +335,7 @@ public:
    */
   void startReceiving();
   void stopReceiving();
-  
+
   /**
    * Start/Stop sending
    */
@@ -353,70 +343,74 @@ public:
   void stopSending();
 
   // A transporter is always in a PerformState.
-  // PerformIO is used initially and as long as any of the events 
-  // PerformConnect, ... 
+  // PerformIO is used initially and as long as any of the events
+  // PerformConnect, ...
   enum PerformState {
-    CONNECTED         = 0,
-    CONNECTING        = 1,
-    DISCONNECTED      = 2,
-    DISCONNECTING     = 3
+    CONNECTED = 0,
+    CONNECTING = 1,
+    DISCONNECTED = 2,
+    DISCONNECTING = 3
   };
-  const char *getPerformStateString(NodeId nodeId) const
-  { return performStateString[(unsigned)performStates[nodeId]]; }
-
-  PerformState getPerformState(NodeId nodeId) const { return performStates[nodeId]; }
-
   /**
    * Get and set methods for PerformState
    */
-  void do_connect(NodeId node_id);
-  /**
-   * do_disconnect can be issued both from send and recv, it is possible to
-   * specify from where it is called in send_source parameter, this enables
-   * us to provide more detailed information for disconnects.
-   */
-  bool do_disconnect(NodeId node_id, int errnum = 0, bool send_source = true);
-  bool is_connected(NodeId node_id) const {
-    return performStates[node_id] == CONNECTED;
+  const char *getPerformStateString(TrpId trpId) const {
+    return performStateString[(unsigned)performStates[trpId]];
   }
-private:
-  void report_connect(TransporterReceiveHandle&, NodeId node_id);
-  void report_disconnect(TransporterReceiveHandle&, NodeId node_id, int errnum);
-  void report_error(NodeId nodeId, TransporterError errorCode,
+  PerformState getPerformState(TrpId trpId) const {
+    return performStates[trpId];
+  }
+  /**
+   * Initiate asynch connecting 'protocol' for transporters
+   */
+  void start_connecting(TrpId trpId);
+  /**
+   * start_disconnecting can be issued both from send and recv.
+   * It is possible to specify from where it is called
+   * in send_source parameter, this enables us to provide more
+   * detailed information for disconnects.
+   */
+  bool start_disconnecting(TrpId trpId, int errnum = 0,
+                           bool send_source = true);
+  bool is_connected(TrpId trpId) const {
+    return performStates[trpId] == CONNECTED;
+  }
+
+ private:
+  void report_connect(TransporterReceiveHandle &, TrpId trpId);
+  void report_disconnect(TransporterReceiveHandle &, TrpId trpId, int errnum);
+  void report_error(TrpId trpId, TransporterError errorCode,
                     const char *errorInfo = nullptr);
   void dump_and_report_bad_message(const char file[], unsigned line,
-                    TransporterReceiveHandle & recvHandle,
-                    Uint32 * readPtr,
-                    size_t sizeOfData,
-                    NodeId remoteNodeId,
-                    IOState state,
-                    TransporterError errorCode);
-public:
-  
+                                   TransporterReceiveHandle &recvHandle,
+                                   Uint32 *readPtr, size_t sizeOfData,
+                                   NodeId remoteNodeId, TrpId trpId,
+                                   IOState state, TransporterError errorCode);
+
+ public:
   /**
-   * Get and set methods for IOState
+   * Set IOState on the Transporter
    */
-  IOState ioState(NodeId nodeId) const;
-  void setIOState(NodeId nodeId, IOState state);
+  void setIOState(TrpId trpId, IOState state);
 
   /**
    * Methods to handle backoff of connection attempts when attempt fails
    */
-public:
+ public:
   void indicate_node_up(NodeId nodeId);
   void set_connect_backoff_max_time_in_ms(Uint32 max_time_in_ms);
-private:
+
+ private:
   Uint32 get_connect_backoff_max_time_in_laps() const;
   bool get_and_clear_node_up_indicator(NodeId nodeId);
   void backoff_reset_connecting_time(NodeId nodeId);
   bool backoff_update_and_check_time_for_connect(NodeId nodeId);
 
-private:
+ private:
+  bool createTCPTransporter(TransporterConfiguration *config);
+  bool createSHMTransporter(TransporterConfiguration *config);
 
-  bool createTCPTransporter(TransporterConfiguration * config);
-  bool createSHMTransporter(TransporterConfiguration * config);
-
-public:
+ public:
   bool createMultiTransporter(NodeId node_id, Uint32 num_trps);
   /**
    *   configureTransporter
@@ -425,7 +419,7 @@ public:
    *   does not exist otherwise try to reconfigure it
    *
    */
-  bool configureTransporter(TransporterConfiguration * config);
+  bool configureTransporter(TransporterConfiguration *config);
 
   /**
    * Get sum of max send buffer over all transporters, to be used as a default
@@ -434,112 +428,109 @@ public:
    * Must be called after creating all transporters for returned value to be
    * correct.
    */
-  Uint64 get_total_max_send_buffer() {
+  Uint64 get_total_max_send_buffer() const {
     assert(m_total_max_send_buffer > 0);
     return m_total_max_send_buffer;
-  } 
+  }
 
   /**
    * Get transporter's connect count
    */
-  Uint32 get_connect_count(NodeId nodeId);
+  Uint32 get_connect_count(TrpId trpId) const;
+
+  /**
+   * Update send buffer allocated and used bytes for given transporter
+   */
+  void update_send_buffer_usage(TrpId trpId, Uint64 allocBytes,
+                                Uint64 usedBytes);
 
   /**
    * Set or clear overloaded bit.
    * Query if any overloaded bit is set.
    */
   void set_status_overloaded(NodeId nodeId, bool val);
-  const NodeBitmask& get_status_overloaded() const;
-  
+  const NodeBitmask &get_status_overloaded() const;
+
   /**
    * Get transporter's overload count since connect
    */
-  Uint32 get_overload_count(NodeId nodeId);
+  Uint32 get_overload_count(NodeId nodeId) const;
 
   /**
    * Set or clear slowdown bit.
    * Query if any slowdown bit is set.
    */
   void set_status_slowdown(NodeId nodeId, bool val);
-  const NodeBitmask& get_status_slowdown() const;
- 
-  /** 
+  const NodeBitmask &get_status_slowdown() const;
+
+  /**
    * Get transporter's slowdown count since connect
    */
-  Uint32 get_slowdown_count(NodeId nodeId);
+  Uint32 get_slowdown_count(NodeId nodeId) const;
+
+  /**
+   * Get SendBuffer alloc + usage info
+   */
+  Uint64 get_send_buffer_alloc_bytes(TrpId trpId) const;
+  Uint64 get_send_buffer_used_bytes(TrpId trpId) const;
+  Uint64 get_send_buffer_max_alloc_bytes(TrpId trpId) const;
+  Uint64 get_send_buffer_max_used_bytes(TrpId trpId) const;
 
   /**
    * prepareSend
    *
-   * When IOState is HaltOutput or HaltIO do not send or insert any 
-   * signals in the SendBuffer, unless it is intended for the remote 
+   * When IOState has the HaltOutput bit set, do not send or insert any
+   * signals in the SendBuffer, unless it is intended for the remote
    * QMGR block (blockno 252)
-   * Perform prepareSend on the transporter. 
+   * Perform prepareSend on the transporter.
    *
-   * NOTE signalHeader->xxxBlockRef should contain block numbers and 
+   * NOTE signalHeader->xxxBlockRef should contain block numbers and
    *                                not references
    */
 
-private:
+ private:
   template <typename AnySectionArg>
-  SendStatus prepareSendTemplate(
-                         TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader *signalHeader,
-                         Uint8 prio,
-                         const Uint32 *signalData,
-                         NodeId nodeId,
-                         Transporter* t,
-                         AnySectionArg section);
+  SendStatus prepareSendTemplate(TransporterSendBufferHandle *sendHandle,
+                                 const SignalHeader *signalHeader, Uint8 prio,
+                                 const Uint32 *signalData, Transporter *t,
+                                 AnySectionArg section);
 
-  Transporter* prepareSend_getTransporter(const SignalHeader *signalHeader,
-                                          NodeId nodeId,
-                                          TrpId &trp_id,
-                                          SendStatus& status);
+  Transporter *prepareSend_getTransporter(const SignalHeader *signalHeader,
+                                          NodeId nodeId, TrpId &trp_id,
+                                          SendStatus &status);
 
-public:
+ public:
   SendStatus prepareSend(TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader *signalHeader,
-                         Uint8 prio,
-                         const Uint32 *signalData,
-                         NodeId nodeId,
-                         TrpId &trp_id,
+                         const SignalHeader *signalHeader, Uint8 prio,
+                         const Uint32 *signalData, NodeId nodeId, TrpId &trp_id,
                          const LinearSectionPtr ptr[3]);
 
   SendStatus prepareSend(TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader *signalHeader,
-                         Uint8 prio,
-                         const Uint32 *signalData,
-                         NodeId nodeId,
-                         TrpId &trp_id,
-                         class SectionSegmentPool & pool,
+                         const SignalHeader *signalHeader, Uint8 prio,
+                         const Uint32 *signalData, NodeId nodeId, TrpId &trp_id,
+                         class SectionSegmentPool &pool,
                          const SegmentedSectionPtr ptr[3]);
 
   SendStatus prepareSend(TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader *signalHeader,
-                         Uint8 prio,
-                         const Uint32 *signalData,
-                         NodeId nodeId,
-                         TrpId &trp_id,
+                         const SignalHeader *signalHeader, Uint8 prio,
+                         const Uint32 *signalData, NodeId nodeId, TrpId &trp_id,
                          const GenericSectionPtr ptr[3]);
 
-  SendStatus prepareSendOverAllLinks(
-                         TransporterSendBufferHandle *sendHandle,
-                         const SignalHeader *signalHeader,
-                         Uint8 prio,
-                         const Uint32 *signalData,
-                         NodeId nodeId,
-                         TrpBitmask &trp_mask);
+  SendStatus prepareSendOverAllLinks(TransporterSendBufferHandle *sendHandle,
+                                     const SignalHeader *signalHeader,
+                                     Uint8 prio, const Uint32 *signalData,
+                                     NodeId nodeId, TrpBitmask &trp_mask);
 
   /* Send on a specific transporter */
   bool performSend(TrpId id, bool need_wakeup = true);
   void performSend();
-  
+
   void printState();
 
   class Transporter_interface {
-  public:
+   public:
     NodeId m_remote_nodeId;
-    int m_s_service_port;			// signed port number
+    int m_s_service_port;  // signed port number
     const char *m_interface;
     bool m_require_tls;
   };
@@ -548,45 +539,49 @@ public:
                                  int s_port, bool requireTls);
 
   int get_transporter_count() const;
-  Transporter* get_transporter(TrpId id) const;
-  Transporter* get_node_transporter(NodeId nodeId) const;
-  bool is_shm_transporter(NodeId nodeId);
-  ndb_sockaddr get_connect_address(NodeId node_id) const;
-  bool is_encrypted_link(NodeId) const;
+  NodeId get_transporter_node_id(TrpId id) const;
+  Transporter *get_transporter(TrpId id) const;
+  Transporter *get_node_transporter(NodeId nodeId) const;
+  Transporter *get_node_base_transporter(NodeId nodeId) const;
+  Transporter *get_node_transporter_instance(NodeId nodeId, int inst) const;
+  bool is_shm_transporter(TrpId trp_id);
+  TransporterType get_transporter_type(TrpId id) const;
 
-  Uint64 get_bytes_sent(NodeId nodeId) const;
-  Uint64 get_bytes_received(NodeId nodeId) const;
+  ndb_sockaddr get_connect_address_node(NodeId nodeId) const;
+  ndb_sockaddr get_connect_address(TrpId trpId) const;
+  bool is_encrypted_link(TrpId trpId) const;
 
-  Uint32 get_num_multi_transporters();
-  Multi_Transporter* get_multi_transporter(Uint32 index);
-  Multi_Transporter* get_node_multi_transporter(NodeId node_id);
+  Uint64 get_bytes_sent(TrpId trpId) const;
+  Uint64 get_bytes_received(TrpId trpId) const;
 
-private:
+  Multi_Transporter *get_node_multi_transporter(NodeId node_id) const;
+
+  static bool is_permitted_halt_signal(const SignalHeader *signalHeader);
+
+ private:
   TransporterCallback *const callbackObj;
   TransporterReceiveHandle *const receiveHandle;
 
   NdbMgmHandle m_mgm_handle;
 
-  struct NdbThread   *m_start_clients_thread;
-  bool                m_run_start_clients_thread;
+  struct NdbThread *m_start_clients_thread;
+  bool m_run_start_clients_thread;
 
   int sendCounter;
   NodeId localNodeId;
   unsigned maxTransporters;
   Uint32 nTransporters;
-  Uint32 nMultiTransporters;
   Uint32 nTCPTransporters;
   Uint32 nSHMTransporters;
   TlsKeyManager m_tls_keys;
   int m_mgm_tls_req;
 
 #ifdef ERROR_INSERT
-  NodeBitmask m_blocked;
-  TrpBitmask m_blocked_trp;
-  NodeBitmask m_blocked_disconnected;
+  TrpBitmask m_blocked;
+  TrpBitmask m_blocked_disconnected;
   int m_disconnect_errors[MAX_NTRANSPORTERS];
 
-  NodeBitmask m_sendBlocked;
+  TrpBitmask m_sendBlocked;
 
   Uint32 m_mixology_level;
 #endif
@@ -594,26 +589,30 @@ private:
   /**
    * Arrays holding all transporters in the order they are created
    */
-  Transporter**     allTransporters;
-  Multi_Transporter** theMultiTransporters;
-  TCP_Transporter** theTCPTransporters;
+  Transporter **allTransporters;
+  TCP_Transporter **theTCPTransporters;
 #ifdef NDB_SHM_TRANSPORTER_SUPPORTED
-  SHM_Transporter** theSHMTransporters;
+  SHM_Transporter **theSHMTransporters;
 #endif
 
   /**
-   * Array, indexed by nodeId, holding all transporters
+   * Array, indexed by nodeId, holding all base transporters
    */
-  TransporterType* theTransporterTypes;
-  Transporter**    theNodeIdTransporters;
+  TransporterType *theTransporterTypes;
+  Transporter **theNodeIdTransporters;
 
-  /** 
-   * State arrays, index by host id
+  /**
+   * Array, indexed by nodeId for those having a MultiTransporter
    */
-  PerformState* performStates;
-  int*          m_disconnect_errnum;
-  Uint32*       m_disconnect_enomem_error;
-  IOState*      ioStates;
+  Multi_Transporter **theNodeIdMultiTransporters;
+
+  /**
+   * State arrays, index by Transporter id (TrpId)
+   */
+  PerformState *performStates;
+  int *m_disconnect_errnum;
+  Uint32 *m_disconnect_enomem_error;
+  IOState *ioStates;
   struct ErrorState {
     TransporterError m_code;
     const char *m_info;
@@ -625,13 +624,13 @@ private:
    * to indicate that node is probable up.
    * It is read and cleared by start clients thread.
    */
-  volatile bool* peerUpIndicators;
+  volatile bool *peerUpIndicators;
 
   /**
    * Count of how long time one have been attempting to
    * connect to node nodeId, in units of 100ms.
    */
-  Uint32*       connectingTime;
+  Uint32 *connectingTime;
 
   /**
    * The current maximal time between connection attempts to a
@@ -643,6 +642,7 @@ private:
   /**
    * Overloaded bits, for fast check.
    * Similarly slowdown bits for fast check.
+   * TODO: Should be TrpBitmask's
    */
   NodeBitmask m_status_overloaded;
   NodeBitmask m_status_slowdown;
@@ -652,124 +652,106 @@ private:
    *
    * Defined in Packer.cpp.
    */
-  Uint32 unpack(TransporterReceiveHandle&,
-                Uint32 * readPtr,
-                Uint32 bufferSize,
-                NodeId remoteNodeId,
-                IOState state,
-		bool & stopReceiving);
 
-  Uint32 * unpack(TransporterReceiveHandle&,
-                  Uint32 * readPtr,
-                  Uint32 * eodPtr,
-                  Uint32 * endPtr,
-                  NodeId remoteNodeId,
-                  IOState state,
-		  bool & stopReceiving);
+  Uint32 unpack(TransporterReceiveHandle &, Uint32 *readPtr, Uint32 bufferSize,
+                NodeId remoteNodeId, TrpId trpId, bool &stopReceiving);
 
-  static Uint32 unpack_length_words(const Uint32 *readPtr,
-                                    Uint32 maxWords,
+  Uint32 *unpack(TransporterReceiveHandle &, Uint32 *readPtr, Uint32 *eodPtr,
+                 Uint32 *endPtr, NodeId remoteNodeId, TrpId trpId,
+                 bool &stopReceiving);
+
+  static Uint32 unpack_length_words(const Uint32 *readPtr, Uint32 maxWords,
                                     bool extra_signal);
 
-  Uint32 poll_TCP(Uint32 timeOutMillis, TransporterReceiveHandle&);
-  Uint32 poll_SHM(TransporterReceiveHandle&, bool &any_connected);
-  Uint32 poll_SHM(TransporterReceiveHandle&,
-                  NDB_TICKS start_time,
+  Uint32 poll_TCP(Uint32 timeOutMillis, TransporterReceiveHandle &);
+  Uint32 poll_SHM(TransporterReceiveHandle &, bool &any_connected);
+  Uint32 poll_SHM(TransporterReceiveHandle &, NDB_TICKS start_time,
                   Uint32 micros_to_poll);
-  Uint32 check_TCP(TransporterReceiveHandle&, Uint32 timeoutMillis);
-  Uint32 spin_check_transporters(TransporterReceiveHandle&);
+  Uint32 check_TCP(TransporterReceiveHandle &, Uint32 timeoutMillis);
+  Uint32 spin_check_transporters(TransporterReceiveHandle &);
 
 #ifdef NDB_SHM_TRANSPORTER_SUPPORTED
   int m_shm_own_pid;
 #endif
   Uint32 m_transp_count;
 
-public:
-  bool setup_wakeup_socket(TransporterReceiveHandle&);
+ public:
+  bool setup_wakeup_socket(TransporterReceiveHandle &);
   void wakeup();
 
   inline bool setup_wakeup_socket() {
     assert(receiveHandle != nullptr);
-    return setup_wakeup_socket(* receiveHandle);
+    return setup_wakeup_socket(*receiveHandle);
   }
-private:
+
+ private:
   bool m_has_extra_wakeup_socket;
   ndb_socket_t m_extra_wakeup_sockets[2];
   void consume_extra_sockets();
 
-  Uint32 *getWritePtr(TransporterSendBufferHandle *handle,
-                      Transporter*,
-                      TrpId trp_id,
-                      Uint32 lenBytes,
-                      Uint32 prio,
+  Uint32 *getWritePtr(TransporterSendBufferHandle *handle, Transporter *,
+                      TrpId trp_id, Uint32 lenBytes, Uint32 prio,
                       SendStatus *error);
-  void updateWritePtr(TransporterSendBufferHandle *handle,
-                      Transporter*,
-                      TrpId trp_id,
-                      Uint32 lenBytes,
-                      Uint32 prio);
+  void updateWritePtr(TransporterSendBufferHandle *handle, Transporter *,
+                      TrpId trp_id, Uint32 lenBytes, Uint32 prio);
 
-public:
+ public:
   /* Various internal */
   void inc_overload_count(NodeId nodeId);
   void inc_slowdown_count(NodeId nodeId);
 
-  void get_trps_for_node(NodeId nodeId,
-                         TrpId *trp_ids,
-                         Uint32 &num_trp_ids,
-                         Uint32 max_trp_ids);
+  void get_trps_for_node(NodeId nodeId, TrpId *trp_ids, Uint32 &num_trp_ids,
+                         Uint32 max_trp_ids) const;
+  TrpId get_the_only_base_trp(NodeId nodeId) const;
 
   Uint32 get_num_trps();
-  TlsKeyManager * getTlsKeyManager()  { return & m_tls_keys; }
-  bool hasTlsCert() const             { return (bool) m_tls_keys.ctx(); }
+  TlsKeyManager *getTlsKeyManager() { return &m_tls_keys; }
+  bool hasTlsCert() const { return (bool)m_tls_keys.ctx(); }
 
-private:
+ private:
   /**
    * Sum of max transporter memory for each transporter.
    * Used to compute default send buffer size.
    */
   Uint64 m_total_max_send_buffer;
 
-public:
+ public:
   /**
    * Receiving
    */
-  Uint32 pollReceive(Uint32 timeOutMillis, TransporterReceiveHandle& mask);
-  Uint32 performReceive(TransporterReceiveHandle&, Uint32 receive_thread_idx);
-  Uint32 update_connections(TransporterReceiveHandle&,
-                          Uint32 max_spintime = UINT32_MAX);
+  Uint32 pollReceive(Uint32 timeOutMillis, TransporterReceiveHandle &mask);
+  Uint32 performReceive(TransporterReceiveHandle &, Uint32 receive_thread_idx);
+  Uint32 update_connections(TransporterReceiveHandle &,
+                            Uint32 max_spintime = UINT32_MAX);
 
   inline Uint32 pollReceive(Uint32 timeOutMillis) {
     assert(receiveHandle != nullptr);
-    return pollReceive(timeOutMillis, * receiveHandle);
+    return pollReceive(timeOutMillis, *receiveHandle);
   }
 
   inline Uint32 performReceive() {
     assert(receiveHandle != nullptr);
-    return performReceive(* receiveHandle, 0);
+    return performReceive(*receiveHandle, 0);
   }
 
   inline void update_connections() {
     assert(receiveHandle != nullptr);
-    update_connections(* receiveHandle);
+    update_connections(*receiveHandle);
   }
 
   Uint32 get_total_spintime() const;
   void reset_total_spintime() const;
 
-  TrpId getTransporterIndex(Transporter* t);
-  void set_recv_thread_idx(Transporter* t, Uint32 recv_thread_idx);
+  static void set_recv_thread_idx(Transporter *t, Uint32 recv_thread_idx);
 
 #ifdef ERROR_INSERT
   /* Utils for testing latency issues */
-  bool isBlocked(NodeId nodeId);
-  void blockReceive(TransporterReceiveHandle&, NodeId nodeId);
-  void unblockReceive(TransporterReceiveHandle&, NodeId nodeId);
-  bool isSendBlocked(NodeId nodeId) const;
-  void blockSend(TransporterReceiveHandle& recvdata,
-                 NodeId nodeId);
-  void unblockSend(TransporterReceiveHandle& recvdata,
-                   NodeId nodeId);
+  bool isBlocked(TrpId trpId) const;
+  void blockReceive(TransporterReceiveHandle &, TrpId trpId);
+  void unblockReceive(TransporterReceiveHandle &, TrpId trpId);
+  bool isSendBlocked(TrpId trpId) const;
+  void blockSend(TransporterReceiveHandle &recvdata, TrpId trpId);
+  void unblockSend(TransporterReceiveHandle &recvdata, TrpId trpId);
 
   /* Testing interleaving of signal processing */
   Uint32 getMixologyLevel() const;
@@ -777,83 +759,63 @@ public:
 #endif
 };
 
-inline Uint32
-TransporterRegistry::get_num_trps()
-{
-  return nTransporters;
-}
+inline Uint32 TransporterRegistry::get_num_trps() { return nTransporters; }
 
-inline void
-TransporterRegistry::set_status_overloaded(NodeId nodeId, bool val)
-{
+inline void TransporterRegistry::set_status_overloaded(NodeId nodeId,
+                                                       bool val) {
   assert(nodeId < MAX_NODES);
-  if (val != m_status_overloaded.get(nodeId))
-  {
+  if (val != m_status_overloaded.get(nodeId)) {
     m_status_overloaded.set(nodeId, val);
-    if (val)
-      inc_overload_count(nodeId);
+    if (val) inc_overload_count(nodeId);
   }
-  if (val)
-    set_status_slowdown(nodeId, val);
+  if (val) set_status_slowdown(nodeId, val);
 }
 
-inline const NodeBitmask&
-TransporterRegistry::get_status_overloaded() const
-{
+inline const NodeBitmask &TransporterRegistry::get_status_overloaded() const {
   return m_status_overloaded;
 }
 
-inline void
-TransporterRegistry::set_status_slowdown(NodeId nodeId, bool val)
-{
+inline void TransporterRegistry::set_status_slowdown(NodeId nodeId, bool val) {
   assert(nodeId < MAX_NODES);
-  if (val != m_status_slowdown.get(nodeId))
-  {
+  if (val != m_status_slowdown.get(nodeId)) {
     m_status_slowdown.set(nodeId, val);
-    if (val)
-      inc_slowdown_count(nodeId);
+    if (val) inc_slowdown_count(nodeId);
   }
 }
 
-inline const NodeBitmask&
-TransporterRegistry::get_status_slowdown() const
-{
+inline const NodeBitmask &TransporterRegistry::get_status_slowdown() const {
   return m_status_slowdown;
 }
 
-inline void
-TransporterRegistry::indicate_node_up(NodeId nodeId) // Called from receive thread
+inline void TransporterRegistry::indicate_node_up(
+    NodeId nodeId)  // Called from receive thread
 {
   assert(nodeId < MAX_NODES);
 
-  if (!peerUpIndicators[nodeId])
-  {
+  if (!peerUpIndicators[nodeId]) {
     peerUpIndicators[nodeId] = true;
   }
 }
 
-inline bool
-TransporterRegistry::get_and_clear_node_up_indicator(NodeId nodeId) // Called from start client thread
+inline bool TransporterRegistry::get_and_clear_node_up_indicator(
+    NodeId nodeId)  // Called from start client thread
 {
   assert(nodeId < MAX_NODES);
 
   bool indicator = peerUpIndicators[nodeId];
-  if (indicator)
-  {
+  if (indicator) {
     peerUpIndicators[nodeId] = false;
   }
   return indicator;
 }
 
-inline Uint32
-TransporterRegistry::get_connect_backoff_max_time_in_laps() const
-{ /* one lap, 100 ms */
+inline Uint32 TransporterRegistry::get_connect_backoff_max_time_in_laps()
+    const { /* one lap, 100 ms */
   return connectBackoffMaxTime;
 }
 
-inline void
-TransporterRegistry::set_connect_backoff_max_time_in_ms(Uint32 backoff_max_time_in_ms)
-{
+inline void TransporterRegistry::set_connect_backoff_max_time_in_ms(
+    Uint32 backoff_max_time_in_ms) {
   /**
    * Round up backoff_max_time to nearest higher 100ms, since that is lap time
    * in start_client_threads using this function.
@@ -861,31 +823,26 @@ TransporterRegistry::set_connect_backoff_max_time_in_ms(Uint32 backoff_max_time_
   connectBackoffMaxTime = (backoff_max_time_in_ms + 99) / 100;
 }
 
-inline void
-TransporterRegistry::backoff_reset_connecting_time(NodeId nodeId)
-{
+inline void TransporterRegistry::backoff_reset_connecting_time(NodeId nodeId) {
   assert(nodeId < MAX_NODES);
 
   connectingTime[nodeId] = 0;
 }
 
-inline bool
-TransporterRegistry::backoff_update_and_check_time_for_connect(NodeId nodeId)
-{
+inline bool TransporterRegistry::backoff_update_and_check_time_for_connect(
+    NodeId nodeId) {
   assert(nodeId < MAX_NODES);
 
   Uint32 backoff_max_time = get_connect_backoff_max_time_in_laps();
 
-  if (backoff_max_time == 0)
-  {
+  if (backoff_max_time == 0) {
     // Backoff disabled
     return true;
   }
 
-  connectingTime[nodeId] ++;
+  connectingTime[nodeId]++;
 
-  if (connectingTime[nodeId] >= backoff_max_time)
-  {
+  if (connectingTime[nodeId] >= backoff_max_time) {
     return (connectingTime[nodeId] % backoff_max_time == 0);
   }
 
@@ -895,20 +852,20 @@ TransporterRegistry::backoff_update_and_check_time_for_connect(NodeId nodeId)
    * roughly every 100ms for each node it is connecting
    * to.
    */
-  static const Uint16 attempt_moments[] = {1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024};
-  static const int attempt_moments_count = sizeof(attempt_moments) / sizeof(attempt_moments[0]);
-  for(int i = 0; i < attempt_moments_count; i ++)
-  {
-    if (connectingTime[nodeId] == attempt_moments[i])
-    {
+  static const Uint16 attempt_moments[] = {1,  2,   4,   8,   16,  32,
+                                           64, 128, 256, 512, 1024};
+  static const int attempt_moments_count =
+      sizeof(attempt_moments) / sizeof(attempt_moments[0]);
+  for (int i = 0; i < attempt_moments_count; i++) {
+    if (connectingTime[nodeId] == attempt_moments[i]) {
       return true;
     }
-    else if (connectingTime[nodeId] < attempt_moments[i])
-    {
+    if (connectingTime[nodeId] < attempt_moments[i]) {
       return false;
     }
   }
-  return (connectingTime[nodeId] % attempt_moments[attempt_moments_count - 1] == 0);
+  return (connectingTime[nodeId] % attempt_moments[attempt_moments_count - 1] ==
+          0);
 }
 
 /**
@@ -920,6 +877,5 @@ TransporterRegistry::backoff_update_and_check_time_for_connect(NodeId nodeId)
 void calculate_send_buffer_level(Uint64 node_send_buffer_size,
                                  Uint64 total_send_buffer_size,
                                  Uint64 total_used_send_buffer_size,
-                                 Uint32 num_threads,
-                                 SB_LevelType &level);
-#endif // Define of TransporterRegistry_H
+                                 Uint32 num_threads, SB_LevelType &level);
+#endif  // Define of TransporterRegistry_H
