@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -33,6 +33,7 @@
 
 #include <array>
 
+#include "my_inttypes.h"
 #include "sql/partitioning/partition_handler.h"
 #include "storage/ndb/include/kernel/ndb_limits.h"
 #include "storage/ndb/include/ndbapi/NdbApi.hpp"
@@ -186,6 +187,7 @@ class ha_ndbcluster : public handler, public Partition_handler {
   ha_rows multi_range_read_info_const(uint keyno, RANGE_SEQ_IF *seq,
                                       void *seq_init_param, uint n_ranges,
                                       uint *bufsz, uint *flags,
+                                      bool *force_default_mrr,
                                       Cost_estimate *cost) override;
   ha_rows multi_range_read_info(uint keyno, uint n_ranges, uint keys,
                                 uint *bufsz, uint *flags,
@@ -559,7 +561,8 @@ class ha_ndbcluster : public handler, public Partition_handler {
   const NdbOperation *pk_unique_index_read_key(uint idx, const uchar *key,
                                                uchar *buf,
                                                NdbOperation::LockMode lm,
-                                               Uint32 *ppartition_id);
+                                               Uint32 *ppartition_id,
+                                               uchar *row_side_buffer);
   int pk_unique_index_read_key_pushed(uint idx, const uchar *key);
 
   int read_multi_range_fetch_next();
@@ -706,6 +709,12 @@ class ha_ndbcluster : public handler, public Partition_handler {
                                  const NdbDictionary::Table *ndbtab) const;
   } copying_alter;
 
+  MY_BITMAP m_in_row_side_buffer;
+
+  uint m_row_side_buffer_size;
+  uint m_mrr_reclength;
+  uchar *m_row_side_buffer;
+
   /* State for setActiveHook() callback for reading blob data. */
   uint m_blob_counter;
   uint m_blob_expected_count_per_row;
@@ -738,5 +747,6 @@ class ha_ndbcluster : public handler, public Partition_handler {
 };
 
 int ndb_to_mysql_error(const NdbError *ndberr);
+int fail_index_offline(TABLE *t, int index);
 
 #endif
