@@ -90,6 +90,10 @@ static struct view {
     {"ndbinfo", "blocks",
      "SELECT block_number, block_name "
      "FROM `ndbinfo`.`ndb$blocks`"},
+    {"ndbinfo", "certificates",
+     "SELECT distinct node_id as Node_id, name as Name, "
+     "from_unixtime(expires, '%d-%b-%Y') as Expires, serial as Serial "
+     "FROM `ndbinfo`.ndb$certificates"},
     {"ndbinfo", "cluster_locks",
      "SELECT "
      "`ndbinfo`.`ndb$acc_operations`.`node_id` AS `node_id`,"
@@ -680,7 +684,7 @@ static struct view {
      " END AS status, "
      " remote_address, bytes_sent, bytes_received, "
      " connect_count, "
-     " overloaded, overload_count, slowdown, slowdown_count "
+     " overloaded, overload_count, slowdown, slowdown_count, encrypted "
      "FROM `ndbinfo`.`ndb$transporters`"},
 };
 
@@ -747,9 +751,7 @@ static struct lookup {
         "single_user_mode enum('locked','read_only','read_write') NOT NULL, "
         "force_var_part INT UNSIGNED NOT NULL, "
         "GCI_bits INT UNSIGNED NOT NULL, "
-        "author_bits INT UNSIGNED NOT NULL, "
-        "extra_metadata_version enum('FRM', 'SDI'), "
-        "extra_metadata LONGBLOB",
+        "author_bits INT UNSIGNED NOT NULL",
     },
     {"ndbinfo", "events",
      "event_id INT UNSIGNED NOT NULL PRIMARY KEY, "
@@ -925,7 +927,7 @@ bool ndbinfo_define_dd_tables(List<const Plugin_table> *plugin_tables) {
                 return (strcmp(x->m.name, y->m.name) < 0);
               });
 
-    for (auto *table : tables) {
+    for (const auto *table : tables) {
       plugin_tables->push_back(ndbinfo_define_table(*table));
     }
   }
