@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+   Copyright (c) 2017, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -70,7 +71,7 @@ dd::String_type Ndb_metadata::partition_expression() const {
   dd::String_type expr;
   if (m_ndbtab->getFragmentType() == NdbDictionary::Table::HashMapPartition &&
       m_ndbtab->getDefaultNoPartitionsFlag() &&
-      m_ndbtab->getFragmentCount() == 0 && m_ndbtab->getLinearFlag() == false) {
+      m_ndbtab->getFragmentCount() == 0 && !m_ndbtab->getLinearFlag()) {
     // Default partitioning
     return expr;
   }
@@ -88,7 +89,7 @@ dd::String_type Ndb_metadata::partition_expression() const {
   return expr;
 }
 
-void Ndb_metadata::create_columns(dd::Table *table_def) const {
+bool Ndb_metadata::create_columns(dd::Table *table_def) const {
   const bool hidden_pk = ndb_table_has_hidden_pk(m_ndbtab);
 
   // Virtual generated columns are a problem since they aren't stored in NDB
@@ -211,8 +212,10 @@ void Ndb_metadata::create_columns(dd::Table *table_def) const {
         dd_column->set_type(dd::enum_column_types::VARCHAR);
         break;
       case NdbDictionary::Column::Datetime:
-        dd_column->set_type(dd::enum_column_types::DATETIME);
-        break;
+        ndb_log_error(
+            "Can not map NDB Datetime column %s.%s to a MySQL column type.",
+            m_ndbtab->getName(), ndb_column->getName());
+        return false;
       case NdbDictionary::Column::Date:
         dd_column->set_type(dd::enum_column_types::NEWDATE);
         break;
@@ -247,16 +250,20 @@ void Ndb_metadata::create_columns(dd::Table *table_def) const {
         dd_column->set_type(dd::enum_column_types::VARCHAR);
         break;
       case NdbDictionary::Column::Time:
-        dd_column->set_type(dd::enum_column_types::TIME);
-        break;
+        ndb_log_error(
+            "Can not map NDB Time column %s.%s to a MySQL column type.",
+            m_ndbtab->getName(), ndb_column->getName());
+        return false;
       case NdbDictionary::Column::Year:
         dd_column->set_type(dd::enum_column_types::YEAR);
         dd_column->set_unsigned(true);
         dd_column->set_zerofill(true);
         break;
       case NdbDictionary::Column::Timestamp:
-        dd_column->set_type(dd::enum_column_types::TIMESTAMP);
-        break;
+        ndb_log_error(
+            "Can not map NDB Timestamp column %s.%s to a MySQL column type.",
+            m_ndbtab->getName(), ndb_column->getName());
+        return false;
       case NdbDictionary::Column::Time2:
         dd_column->set_type(dd::enum_column_types::TIME2);
         dd_column->set_datetime_precision(ndb_column->getPrecision());
@@ -322,6 +329,7 @@ void Ndb_metadata::create_columns(dd::Table *table_def) const {
       dd_column->options().set(key_column_format,
                                static_cast<uint32>(COLUMN_FORMAT_TYPE_DYNAMIC));
   }
+  return true;
 }
 
 bool Ndb_metadata::create_indexes(const NdbDictionary::Dictionary *dict,
@@ -552,6 +560,14 @@ bool Ndb_metadata::create_foreign_keys(Ndb *ndb, dd::Table *table_def) const {
 
     Ndb_table_guard ndb_table_guard(ndb, parent_db, parent_name);
     const NdbDictionary::Table *parent_table = ndb_table_guard.get_table();
+    if (!parent_table) {
+      // Failed to open the table from NDB
+      ndb_log_error("Got error '%d: %s' from NDB",
+                    ndb_table_guard.getNdbError().code,
+                    ndb_table_guard.getNdbError().message);
+      ndb_log_error("Failed to open table '%s.%s'", parent_db, parent_name);
+      return false;
+    }
     for (unsigned int j = 0; j < ndb_fk.getChildColumnCount(); j++) {
       // Create FK element(s) for child columns
       dd::Foreign_key_element *fk_element = dd_fk->add_element();
@@ -589,7 +605,7 @@ bool Ndb_metadata::create_table_def(Ndb *ndb, dd::Table *table_def) const {
   table_def->set_engine("ndbcluster");
 
   // row_format
-  if (m_ndbtab->getForceVarPart() == false) {
+  if (!m_ndbtab->getForceVarPart()) {
     table_def->set_row_format(dd::Table::RF_FIXED);
   } else {
     table_def->set_row_format(dd::Table::RF_DYNAMIC);
@@ -605,8 +621,8 @@ bool Ndb_metadata::create_table_def(Ndb *ndb, dd::Table *table_def) const {
   // table_def->set_comment(some_comment);
 
   // se_private_id, se_private_data
-  ndb_dd_table_set_object_id_and_version(table_def, m_ndbtab->getObjectId(),
-                                         m_ndbtab->getObjectVersion());
+  ndb_dd_table_set_spi_and_version(table_def, m_ndbtab->getObjectId(),
+                                   m_ndbtab->getObjectVersion());
 
   // storage
   // no DD API setters or types available -> hardcode
@@ -663,7 +679,10 @@ bool Ndb_metadata::create_table_def(Ndb *ndb, dd::Table *table_def) const {
     // table_def->set_subpartition_expression_utf8();
   }
 
-  create_columns(table_def);
+  if (!create_columns(table_def)) {
+    ndb_log_error("Failed to create columns");
+    return false;
+  }
 
   if (!create_indexes(ndb->getDictionary(), table_def)) {
     ndb_log_error("Failed to create indexes");
@@ -736,7 +755,8 @@ class Compare_context {
 
  private:
   std::vector<std::string> diffs;
-  void add_diff(const char *property, std::string a, std::string b) {
+  void add_diff(const char *property, const std::string &a,
+                const std::string &b) {
     std::string diff;
     diff.append("Diff in '")
         .append(property)
@@ -749,7 +769,7 @@ class Compare_context {
   }
 
   void add_diff(object_type type, const char *name, const char *property,
-                std::string a, std::string b) {
+                const std::string &a, const std::string &b) {
     std::string object_type_string;
     switch (type) {
       case COLUMN:
@@ -780,7 +800,8 @@ class Compare_context {
   }
 
  public:
-  void compare(const char *property, dd::String_type a, dd::String_type b) {
+  void compare(const char *property, const dd::String_type &a,
+               const dd::String_type &b) {
     if (a == b) return;
     add_diff(property, a.c_str(), b.c_str());
   }
@@ -792,7 +813,7 @@ class Compare_context {
   }
 
   void compare(object_type type, const char *name, const char *property,
-               dd::String_type a, dd::String_type b) {
+               const dd::String_type &a, const dd::String_type &b) {
     if (a == b) return;
     add_diff(type, name, property, a.c_str(), b.c_str());
   }
@@ -804,7 +825,7 @@ class Compare_context {
   }
 
   bool equal() {
-    if (diffs.size() == 0) return true;
+    if (diffs.empty()) return true;
 
     // Print the list of diffs
     ndb_log_error("Metadata check has failed");
@@ -864,12 +885,10 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
 
   // se_private_id and se_private_data.object_version (local)
   {
-    int t1_id, t1_version;
-    ndb_dd_table_get_object_id_and_version(t1, t1_id, t1_version);
-    int t2_id, t2_version;
-    ndb_dd_table_get_object_id_and_version(t2, t2_id, t2_version);
-    ctx.compare("se_private_id", t1_id, t2_id);
-    ctx.compare("object_version", t1_version, t2_version);
+    Ndb_dd_handle t1_handle = ndb_dd_table_get_spi_and_version(t1);
+    Ndb_dd_handle t2_handle = ndb_dd_table_get_spi_and_version(t2);
+    ctx.compare("se_private_id", t1_handle.spi, t2_handle.spi);
+    ctx.compare("object_version", t1_handle.version, t2_handle.version);
   }
 
   // storage
@@ -937,15 +956,8 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
     const dd::Column *column2 = t2->get_column(column1->name());
     if (column2 == nullptr) continue;
 
-    /*
-      Diff in 'column_name' detected, 'D' != 'd'
-
-      ALTER TABLE t8 CHANGE D d INT UNSIGNED causes the above. Only an issue
-      with the same character with different case. For example,
-      ALTER TABLE t8 CHANGE D c INT UNSIGNED works perfectly well.
-      Bug#31958327 has been filed
-    */
-    // ctx.compare("column_name", column1->name(), column2->name());
+    // column name
+    ctx.compare("column_name", column1->name(), column2->name());
 
     const char *column_name = column1->name().c_str();
 
@@ -971,6 +983,7 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
     const dd::enum_column_types column2_type = column2->type();
     if (column1_type != dd::enum_column_types::BIT &&
         column1_type != dd::enum_column_types::LONG_BLOB &&
+        column1_type != dd::enum_column_types::VARCHAR &&
         column2_type != dd::enum_column_types::ENUM &&
         column2_type != dd::enum_column_types::SET)
       ctx.compare(Compare_context::COLUMN, column_name, "type",
@@ -1178,6 +1191,7 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
 
     /*
       Diff in 'column_comment' detected, '' != 'NDB_COLUMN=MAX_BLOB_PART_SIZE'
+                                         '' != 'NDB_COLUMN=BLOB_INLINE_SIZE'
 
       Column comments aren't stored in NDB Dictionary
     */
@@ -1418,7 +1432,7 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
           break;
         }
       }
-      if (index_it2 == t2->indexes().end()) {
+      if (index2 == nullptr) {
         // Index not found in the DD table. Continue to the next index
         // comparison
         ctx.compare("index_name", index1->name(), "");
@@ -1490,9 +1504,17 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
     //            static_cast<unsigned long long>(index2->type()));
 
     // Algorithm
-    ctx.compare(Compare_context::INDEX, index_name, "algorithm",
-                static_cast<unsigned long long>(index1->algorithm()),
-                static_cast<unsigned long long>(index2->algorithm()));
+
+    /*
+      CREATE TABLE t1 (a int primary key, b int, unique(b)) engine=ndb;
+      This creates two indexes "b" and "b$unique". If someone uses ndb_restore
+      or ndb_drop_index to drop one or the other of "b" and "b$unique" (but not
+      both), this test will fail.  In general, post-bug#28584066, DD and NDB
+      are not required to agree about indexes.
+    */
+    // ctx.compare(Compare_context::INDEX, index_name, "algorithm",
+    //             static_cast<unsigned long long>(index1->algorithm()),
+    //             static_cast<unsigned long long>(index2->algorithm()));
 
     // Explicit algorithm
     /*
@@ -1559,7 +1581,7 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
             break;
           }
         }
-        if (idx_elem_it2 == index2->elements().end()) {
+        if (idx_element2 == nullptr) {
           // Index element not found. Continue to the next index element
           // comparison
           ctx.compare(Compare_context::INDEX, index_name, "element.column",
@@ -1676,7 +1698,8 @@ bool Ndb_metadata::compare_table_def(const dd::Table *t1,
           break;
         }
       }
-      if (fk_it3 == t2->foreign_keys().end()) {
+      if (fk2 == nullptr) {
+        // FK not found. Continue to the next FK comparison
         ctx.compare("fk_name", fk1->name(), "");
         continue;
       }
