@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2009, 2024, Oracle and/or its affiliates.
-    Use is subject to license terms.
+   Copyright (c) 2009, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,7 +29,7 @@
 
 bool printLOCAL_ROUTE_ORD(FILE *output, const Uint32 *theData, Uint32 len,
                           Uint16 /*rbn*/) {
-  const LocalRouteOrd *sig = (const LocalRouteOrd *)theData;
+  const auto *sig = (const LocalRouteOrd *)theData;
   Uint32 pathcnt = sig->cnt >> 16;
   Uint32 dstcnt = sig->cnt & 0xFFFF;
 
