@@ -1,15 +1,16 @@
-/* Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,7 +31,7 @@ bool printSUB_CREATE_REQ(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubCreateReq *const sig = (const SubCreateReq *)theData;
+  const auto *const sig = (const SubCreateReq *)theData;
   fprintf(output, " senderRef: %x\n", sig->senderRef);
   fprintf(output, " senderData: %x\n", sig->senderData);
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
@@ -48,7 +49,7 @@ bool printSUB_CREATE_CONF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubCreateConf *const sig = (const SubCreateConf *)theData;
+  const auto *const sig = (const SubCreateConf *)theData;
   fprintf(output, " senderData: %x\n", sig->senderData);
   return false;
 }
@@ -60,7 +61,7 @@ bool printSUB_CREATE_REF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubCreateRef *const sig = (const SubCreateRef *)theData;
+  const auto *const sig = (const SubCreateRef *)theData;
   fprintf(output, " senderData: %x\n", sig->senderData);
   return false;
 }
@@ -72,7 +73,7 @@ bool printSUB_REMOVE_REQ(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubRemoveReq *const sig = (const SubRemoveReq *)theData;
+  const auto *const sig = (const SubRemoveReq *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   return false;
@@ -85,7 +86,7 @@ bool printSUB_REMOVE_CONF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubRemoveConf *const sig = (const SubRemoveConf *)theData;
+  const auto *const sig = (const SubRemoveConf *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " senderData: %x\n", sig->senderData);
@@ -99,7 +100,7 @@ bool printSUB_REMOVE_REF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubRemoveRef *const sig = (const SubRemoveRef *)theData;
+  const auto *const sig = (const SubRemoveRef *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " senderData: %x\n", sig->senderData);
@@ -114,10 +115,13 @@ bool printSUB_START_REQ(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubStartReq *const sig = (const SubStartReq *)theData;
+  const auto *const sig = (const SubStartReq *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " senderData: %x\n", sig->senderData);
+  if (len > SubStartReq::SignalLengthWithoutRequestInfo)
+    fprintf(output, " requestInfo: %x\n", sig->requestInfo);
+
   return false;
 }
 
@@ -128,7 +132,7 @@ bool printSUB_START_REF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubStartRef *const sig = (const SubStartRef *)theData;
+  const auto *const sig = (const SubStartRef *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " startPart: %x\n", sig->part);
@@ -144,7 +148,7 @@ bool printSUB_START_CONF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubStartConf *const sig = (const SubStartConf *)theData;
+  const auto *const sig = (const SubStartConf *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " startPart: %x\n", sig->part);
@@ -159,7 +163,7 @@ bool printSUB_STOP_REQ(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubStopReq *const sig = (const SubStopReq *)theData;
+  const auto *const sig = (const SubStopReq *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " senderData: %x\n", sig->senderData);
@@ -173,7 +177,7 @@ bool printSUB_STOP_REF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubStopRef *const sig = (const SubStopRef *)theData;
+  const auto *const sig = (const SubStopRef *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " senderData: %x\n", sig->senderData);
@@ -188,7 +192,7 @@ bool printSUB_STOP_CONF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubStopConf *const sig = (const SubStopConf *)theData;
+  const auto *const sig = (const SubStopConf *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " senderData: %x\n", sig->senderData);
@@ -202,7 +206,7 @@ bool printSUB_SYNC_REQ(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubSyncReq *const sig = (const SubSyncReq *)theData;
+  const auto *const sig = (const SubSyncReq *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   fprintf(output, " syncPart: %x\n", sig->part);
@@ -220,7 +224,7 @@ bool printSUB_SYNC_REF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubSyncRef *const sig = (const SubSyncRef *)theData;
+  const auto *const sig = (const SubSyncRef *)theData;
   fprintf(output, " senderData: %x\n", sig->senderData);
   fprintf(output, " errorCode: %x\n", sig->errorCode);
   return false;
@@ -233,7 +237,7 @@ bool printSUB_SYNC_CONF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubSyncConf *const sig = (const SubSyncConf *)theData;
+  const auto *const sig = (const SubSyncConf *)theData;
   fprintf(output, " senderData: %x\n", sig->senderData);
   return false;
 }
@@ -245,7 +249,7 @@ bool printSUB_TABLE_DATA(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubTableData *const sig = (const SubTableData *)theData;
+  const auto *const sig = (const SubTableData *)theData;
   fprintf(output, " senderData: %x\n", sig->senderData);
   fprintf(output, " gci_hi: %x\n", sig->gci_hi);
   fprintf(output, " gci_lo: %x\n", sig->gci_lo);
@@ -265,7 +269,7 @@ bool printSUB_SYNC_CONTINUE_REQ(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubSyncContinueReq *const sig = (const SubSyncContinueReq *)theData;
+  const auto *const sig = (const SubSyncContinueReq *)theData;
   fprintf(output, " subscriberData: %x\n", sig->subscriberData);
   fprintf(output, " noOfRowsSent: %x\n", sig->noOfRowsSent);
   return false;
@@ -278,7 +282,7 @@ bool printSUB_SYNC_CONTINUE_REF(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubSyncContinueRef *const sig = (const SubSyncContinueRef *)theData;
+  const auto *const sig = (const SubSyncContinueRef *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   return false;
@@ -291,7 +295,7 @@ bool printSUB_SYNC_CONTINUE_CONF(FILE *output, const Uint32 *theData,
     return false;
   }
 
-  const SubSyncContinueConf *const sig = (const SubSyncContinueConf *)theData;
+  const auto *const sig = (const SubSyncContinueConf *)theData;
   fprintf(output, " subscriptionId: %x\n", sig->subscriptionId);
   fprintf(output, " subscriptionKey: %x\n", sig->subscriptionKey);
   return false;
@@ -304,7 +308,7 @@ bool printSUB_GCP_COMPLETE_REP(FILE *output, const Uint32 *theData, Uint32 len,
     return false;
   }
 
-  const SubGcpCompleteRep *const sig = (const SubGcpCompleteRep *)theData;
+  const auto *const sig = (const SubGcpCompleteRep *)theData;
   fprintf(output, " gci_hi: %x gci_lo: %x\n", sig->gci_hi, sig->gci_lo);
   return false;
 }
