@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,8 +22,8 @@
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "util/ndb_ndbxfrm1.h"
-#include <stdio.h>
 #include <algorithm>
+#include <cstdio>
 #include "my_byteorder.h"  // WORDS_BIGENDIAN
 #include "my_compiler.h"   // unlikely
 #include "util/require.h"
@@ -37,7 +38,7 @@
     }                                                                    \
   } while (0)
 #endif
-//#define RETURN(rv) REQUIRE(rv)
+// #define RETURN(rv) REQUIRE(rv)
 #define RETURN(rv) return (rv)
 
 const char ndb_ndbxfrm1::magic[8] = {'N', 'D', 'B', 'X', 'F', 'R', 'M', '1'};
@@ -76,8 +77,7 @@ int ndb_ndbxfrm1::header::detect_header(const ndbxfrm_input_iterator *in,
   if (memcmp(buf, magic, 8) != 0) return -2;
 
   if (len < sizeof(fixed_header::magic)) return need_more_input;
-  const fixed_header::magic *mh =
-      reinterpret_cast<const fixed_header::magic *>(buf);
+  const auto *mh = reinterpret_cast<const fixed_header::magic *>(buf);
 
   bool toggle_endian = (mh->m_endian != native_endian_marker);
   if (toggle_endian && (mh->m_endian != reverse_endian_marker))
@@ -107,8 +107,7 @@ int ndb_ndbxfrm1::header::read_header(ndbxfrm_input_iterator *in) {
   const byte *p = buf;
 
   if (len < sizeof(fixed_header::magic)) RETURN(-1);
-  const fixed_header::magic *magicp =
-      reinterpret_cast<const fixed_header::magic *>(p);
+  const auto *magicp = reinterpret_cast<const fixed_header::magic *>(p);
 
   bool detect_toggle_endian = (magicp->m_endian == reverse_endian_marker);
   if (!detect_toggle_endian && magicp->m_endian != native_endian_marker) {
@@ -942,9 +941,8 @@ int ndb_ndbxfrm1::trailer::read_trailer(ndbxfrm_input_reverse_iterator *in) {
   const byte *endp = p + len;
 
   if (len < sizeof(fixed_trailer::magic)) RETURN(-1);
-  const fixed_trailer::magic *magicp =
-      reinterpret_cast<const fixed_trailer::magic *>(
-          endp - sizeof(fixed_trailer::magic));
+  const auto *magicp = reinterpret_cast<const fixed_trailer::magic *>(
+      endp - sizeof(fixed_trailer::magic));
 
   bool detect_toggle_endian = (magicp->m_endian != native_endian_marker);
   Uint32 trailer_size = magicp->m_trailer_size;
@@ -956,7 +954,7 @@ int ndb_ndbxfrm1::trailer::read_trailer(ndbxfrm_input_reverse_iterator *in) {
   }
   if (fixed_trailer_size > len) RETURN(-1);
   if (fixed_trailer_size < sizeof(fixed_trailer::magic)) RETURN(-1);
-  const fixed_trailer *trailerp =
+  const auto *trailerp =
       reinterpret_cast<const fixed_trailer *>(endp - fixed_trailer_size);
 
   Uint32 copy_size = fixed_trailer_size - sizeof(fixed_trailer::magic);
