@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -23,10 +23,10 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-void ndbPrintFullyCompatibleTable(void);
-void ndbPrintUpgradeCompatibleTable(void);
+void ndbPrintFullyCompatibleTable();
+void ndbPrintUpgradeCompatibleTable();
 
-int main(void) {
+int main() {
   ndbPrintFullyCompatibleTable();
   ndbPrintUpgradeCompatibleTable();
 }
