@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2009, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2009, 2025, Oracle and/or its affiliates.
 
 
    This program is free software; you can redistribute it and/or modify
@@ -336,12 +336,12 @@ class Ndbd : public Mgmd {
 
   NdbProcess::Args &args() { return m_args; }
 
-  void set_connect_string(BaseString connect_string) {
+  void set_connect_string(const BaseString &connect_string) {
     m_args.add("-c");
     m_args.add(connect_string.c_str());
   }
 
-  bool start(const char *working_dir, BaseString connect_string) {
+  bool start(const char *working_dir, const BaseString &connect_string) {
     set_connect_string(connect_string);
     return Mgmd::start(working_dir, m_args);
   }
@@ -1615,6 +1615,37 @@ int runTestMultiMGMDDisconnection(NDBT_Context *ctx, NDBT_Step *step) {
   return NDBT_OK;
 }
 
+int runTestMyCnf(NDBT_Context *ctx, NDBT_Step *step) {
+  /* Construct path name to my.cnf */
+  NDBT_Workingdir wd("test_mgmd");
+  BaseString my_cnf_file = path(wd.path(), "my.cnf", NULL);
+
+  /* Create args to start mgmd using a my.cnf file */
+  NdbProcess::Args args;
+  args.add("--defaults-file=", my_cnf_file.c_str());
+  args.add("--configdir=", wd.path());
+  args.add("--ndb-nodeid=", 1);
+  args.add("--mycnf");
+
+  /* Write a my.cnf file containing an illegal config parameter */
+  FILE *fp = fopen(my_cnf_file.c_str(), "w");
+  CHECK(fp != nullptr);
+  fprintf(fp, "[cluster_config]\n");
+  fprintf(fp, "ndbcluster=on\n");
+  fclose(fp);
+
+  /* Start mgmd */
+  Mgmd mgmd;
+  CHECK(mgmd.start(wd.path(), args));
+
+  /* Mgmd exits with status 1 due to syntax error in my.cnf */
+  int ret;
+  mgmd.wait(ret);
+  CHECK(ret == 1);
+
+  return NDBT_OK;
+}
+
 int runTestSshKeySigning(NDBT_Context *ctx, NDBT_Step *step) {
   /* Skip this test in PB2 environments, where "ssh localhost"
      does not necessarily work.
@@ -2062,6 +2093,9 @@ TESTCASE("MultiMGMDDisconnection",
          "Test multi mgmd robustness against other mgmd disconnections") {
   INITIALIZER(runTestMultiMGMDDisconnection);
 }
+TESTCASE("MyCnf", "Test reading config from my.cnf") {
+  INITIALIZER(runTestMyCnf);
+}
 
 #if OPENSSL_VERSION_NUMBER >= NDB_TLS_MINIMUM_OPENSSL
 
