@@ -1,16 +1,17 @@
 /*
-   Copyright (c) 2003, 2023, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -61,7 +62,8 @@ extern RestoreLogger restoreLogger;
 static Uint32 g_tableCompabilityMask = 0;
 static int ga_nodeId = 0;
 static int ga_nParallelism = 128;
-static int ga_backupId = 0;
+static int64 ga_inputBackupId = -1;
+static Uint32 ga_backupId = 0;
 bool ga_dont_ignore_systab_0 = false;
 static bool ga_no_upgrade = false;
 static bool ga_promote_attributes = false;
@@ -132,7 +134,7 @@ static bool ga_exclude_missing_columns = false;
 static bool ga_exclude_missing_tables = false;
 static bool opt_exclude_intermediate_sql_tables = true;
 static bool ga_with_apply_status = false;
-bool opt_include_stored_grants = false;
+static bool opt_include_stored_grants = false;
 #ifdef ERROR_INSERT
 static unsigned int _error_insert = 0;
 #endif
@@ -185,7 +187,6 @@ static const char *opt_exclude_databases = NULL;
 static const char *opt_include_databases = NULL;
 static const char *opt_rewrite_database = NULL;
 static const char *opt_one_remap_col_arg = NULL;
-static bool opt_restore_privilege_tables = false;
 
 /**
  * ExtraTableInfo
@@ -288,8 +289,8 @@ static struct my_option my_long_options[] = {
      "Read encryption password for backup file from stdin",
      &opt_backup_password_from_stdin.opt_value, nullptr, 0, GET_BOOL, NO_ARG, 0,
      0, 0, nullptr, 0, &opt_backup_password_from_stdin},
-    {"backupid", 'b', "Backup id", &ga_backupId, nullptr, nullptr, GET_INT,
-     REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+    {"backupid", 'b', "Backup id", &ga_inputBackupId, nullptr, nullptr, GET_LL,
+     REQUIRED_ARG, -1, -1, 0, 0, 0, 0},
     {"decrypt", NDB_OPT_NOSHORT, "Decrypt file", &opt_decrypt, nullptr, nullptr,
      GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
     {"restore_data", 'r',
@@ -408,10 +409,6 @@ static struct my_option my_long_options[] = {
      "Example: db1.t1,db3.t1",
      &opt_exclude_tables, nullptr, nullptr, GET_STR, REQUIRED_ARG, 0, 0, 0, 0,
      0, 0},
-    {"restore-privilege-tables", NDB_OPT_NOSHORT,
-     "Restore privilege tables (after they have been moved to ndb)",
-     &opt_restore_privilege_tables, nullptr, nullptr, GET_BOOL, NO_ARG, 0, 0, 0,
-     0, 0, 0},
     {"include-stored-grants", NDB_OPT_NOSHORT,
      "Restore users and grants to ndb_sql_metadata table",
      &opt_include_stored_grants, nullptr, nullptr, GET_BOOL, OPT_ARG, false, 0,
@@ -467,7 +464,7 @@ static struct my_option my_long_options[] = {
      0, 0, 0},
     NdbStdOpt::end_of_options};
 
-static bool parse_remap_option(const BaseString option, BaseString &db_name,
+static bool parse_remap_option(const BaseString &option, BaseString &db_name,
                                BaseString &tab_name, BaseString &col_name,
                                BaseString &func_name, BaseString &func_args,
                                BaseString &error_msg) {
@@ -545,17 +542,18 @@ static bool get_one_option(int optid, const struct my_option *opt,
       break;
     case 'n':
       if (ga_nodeId == 0) {
-        err << "Error in --nodeid,-n setting, see --help";
+        err << "Error in --nodeid,-n setting, see --help" << endl;
         exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
       }
       info.setLevel(254);
       info << "Nodeid = " << ga_nodeId << endl;
       break;
     case 'b':
-      if (ga_backupId == 0) {
-        err << "Error in --backupid,-b setting, see --help";
+      if (ga_inputBackupId == -1 || (ga_inputBackupId >= (int64(1) << 32))) {
+        err << "Error in --backupid,-b setting, see --help" << endl;
         exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
       }
+      ga_backupId = Uint32(ga_inputBackupId);
       info.setLevel(254);
       info << "Backup Id = " << ga_backupId << endl;
       break;
@@ -631,19 +629,6 @@ BaseString makeExternalTableName(const BaseString &internalName) {
   return externalName;
 }
 
-// Exclude the legacy privilege tables from Cluster 7.x
-void exclude_privilege_tables() {
-  static const char *priv_tables[] = {
-      "mysql.user",         "mysql.db",         "mysql.tables_priv",
-      "mysql.columns_priv", "mysql.procs_priv", "mysql.proxies_priv"};
-
-  for (size_t i = 0; i < array_elements(priv_tables); i++) {
-    g_exclude_tables.push_back(priv_tables[i]);
-    save_include_exclude(OPT_EXCLUDE_TABLES,
-                         const_cast<char *>(priv_tables[i]));
-  }
-}
-
 bool readArguments(Ndb_opts &opts, char ***pargv) {
   Uint32 i;
   BaseString tmp;
@@ -689,7 +674,7 @@ bool readArguments(Ndb_opts &opts, char ***pargv) {
     err << "Backup file node ID not specified, please provide --nodeid" << endl;
     exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
   }
-  if (ga_backupId == 0) {
+  if (ga_inputBackupId == -1) {
     err << "Backup ID not specified, please provide --backupid" << endl;
     exitHandler(NdbToolsProgramExitCode::WRONG_ARGS);
   }
@@ -725,9 +710,6 @@ bool readArguments(Ndb_opts &opts, char ***pargv) {
   }
 
   if (ga_restore) {
-    // Exclude privilege tables unless explicitly included
-    if (!opt_restore_privilege_tables) exclude_privilege_tables();
-
     // Move over old style arguments to include/exclude lists
     if (g_databases.size() > 0) {
       BaseString tab_prefix, tab;
@@ -951,6 +933,10 @@ bool create_consumers(RestoreThreadData *data) {
     if (data->m_part_id == 1) restore->m_delete_epoch_tuple = true;
   }
 
+  if (opt_include_stored_grants) {
+    restore->m_with_sql_metadata = true;
+  }
+
   {
     BackupConsumer *c = printer;
     data->m_consumers.push_back(c);
@@ -986,10 +972,6 @@ static inline bool isIndex(const TableS *table) {
   return (int)tmptab.m_indexType != (int)NdbDictionary::Index::Undefined;
 }
 
-static inline bool isSYSTAB_0(const TableS *table) {
-  return table->isSYSTAB_0();
-}
-
 const char *getTableName(const TableS *table) {
   const char *table_name;
   if (isBlobTable(table))
@@ -1046,7 +1028,8 @@ static void save_include_exclude(int optid, char *argument) {
     g_include_exclude.push_back(option);
   }
 }
-static bool check_include_exclude(BaseString database, BaseString table) {
+static bool check_include_exclude(const BaseString &database,
+                                  const BaseString &table) {
   const char *db = database.c_str();
   const char *tbl = table.c_str();
   bool do_include = true;
@@ -1184,8 +1167,19 @@ static inline bool checkDbAndTableName(const TableS *table) {
   return false;
 }
 
-static void exclude_missing_tables(const RestoreMetaData &metaData,
-                                   const Vector<BackupConsumer *> g_consumers) {
+static inline bool rebuildSysTableIdx(const TableS *table) {
+  const char *table_name = table->getTableName();
+  bool res = false;
+
+  res |= opt_include_stored_grants &&
+         strcmp(table_name, NDB_REP_DB "/def/" NDB_SQL_METADATA_TABLE) == 0;
+  res |= checkSysTable(table) && checkDbAndTableName(table);
+  return res;
+}
+
+static void exclude_missing_tables(
+    const RestoreMetaData &metaData,
+    const Vector<BackupConsumer *> &g_consumers) {
   Uint32 i, j;
   bool isMissing;
   Vector<BaseString> missingTables;
@@ -1756,6 +1750,18 @@ static bool check_slice_skip_fragment(const TableS *table, Uint32 fragmentId) {
   return table->getSliceSkipFlag(fragmentId);
 }
 
+static bool is_included_sys_table(const TableS *table) {
+  if (table->isSYSTAB_0()) return true;
+  if (ga_with_apply_status &&
+      strcmp(table->getTableName(), NDB_REP_DB "/def/" NDB_APPLY_TABLE) == 0)
+    return true;
+  if (opt_include_stored_grants &&
+      strcmp(table->getTableName(), "mysql/def/ndb_sql_metadata") == 0)
+    return true;
+
+  return false;
+}
+
 int do_restore(RestoreThreadData *thrdata) {
   init_progress();
 
@@ -1968,6 +1974,9 @@ int do_restore(RestoreThreadData *thrdata) {
       restoreLogger.log_info(" Object create progress: %u objects out of %u",
                              i + 1, metaData.getNoOfObjects());
     }
+    if (ga_error_thread > 0) {
+      return NdbToolsProgramExitCode::FAILED;
+    }
   }
 
   restoreLogger.log_debug("Handling index stat tables");
@@ -1977,6 +1986,9 @@ int do_restore(RestoreThreadData *thrdata) {
           "Restore: Failed to handle index stat tables ... Exiting ");
       return NdbToolsProgramExitCode::FAILED;
     }
+    if (ga_error_thread > 0) {
+      return NdbToolsProgramExitCode::FAILED;
+    }
   }
 
   Vector<OutputStream *> table_output(metaData.getNoOfTables());
@@ -1987,9 +1999,7 @@ int do_restore(RestoreThreadData *thrdata) {
     const TableS *table = metaData[i];
     table_output.push_back(NULL);
     if (!checkDbAndTableName(table)) continue;
-    if (isSYSTAB_0(table) || (strcmp(table->getTableName(),
-                                     NDB_REP_DB "/def/" NDB_APPLY_TABLE) == 0 &&
-                              ga_with_apply_status)) {
+    if (is_included_sys_table(table)) {
       table_output[i] = ndbout.m_out;
     }
     if (checkSysTable(table)) {
@@ -2038,6 +2048,9 @@ int do_restore(RestoreThreadData *thrdata) {
       restoreLogger.log_info("Table create progress: %u tables out of %u",
                              i + 1, metaData.getNoOfTables());
     }
+    if (ga_error_thread > 0) {
+      return NdbToolsProgramExitCode::FAILED;
+    }
   }
 
   restoreLogger.log_debug("Save foreign key info");
@@ -2047,6 +2060,9 @@ int do_restore(RestoreThreadData *thrdata) {
       if (!g_consumers[j]->fk(metaData.getObjType(i), metaData.getObjPtr(i))) {
         return NdbToolsProgramExitCode::FAILED;
       }
+      if (ga_error_thread > 0) {
+        return NdbToolsProgramExitCode::FAILED;
+      }
     }
   }
 
@@ -2212,6 +2228,9 @@ int do_restore(RestoreThreadData *thrdata) {
           ndbout.m_out = tmp;
           if (check_progress())
             report_progress("Data file progress: ", dataIter);
+          if (ga_error_thread > 0) {
+            return NdbToolsProgramExitCode::FAILED;
+          }
         }  // while (tuple != NULL);
 
         if (res < 0) {
@@ -2236,7 +2255,19 @@ int do_restore(RestoreThreadData *thrdata) {
 
       dataIter.validateFooter();  // not implemented
 
-      for (i = 0; i < g_consumers.size(); i++) g_consumers[i]->endOfTuples();
+      {
+        bool consumersOk = true;
+        for (i = 0; i < g_consumers.size(); i++) {
+          consumersOk &= g_consumers[i]->endOfTuples();
+        }
+
+        if (!consumersOk) {
+          restoreLogger.log_error(
+              "Restore: An error occurred while restoring data."
+              "Exiting");
+          return NdbToolsProgramExitCode::FAILED;
+        }
+      }
 
       /* report to clusterlog if applicable */
       for (i = 0; i < g_consumers.size(); i++) {
@@ -2273,6 +2304,9 @@ int do_restore(RestoreThreadData *thrdata) {
         }
 
         if (check_progress()) report_progress("Log file progress: ", logIter);
+        if (ga_error_thread > 0) {
+          return NdbToolsProgramExitCode::FAILED;
+        }
       }
       if (res < 0) {
         restoreLogger.log_error(
@@ -2280,7 +2314,18 @@ int do_restore(RestoreThreadData *thrdata) {
         return NdbToolsProgramExitCode::FAILED;
       }
       logIter.validateFooter();  // not implemented
-      for (i = 0; i < g_consumers.size(); i++) g_consumers[i]->endOfLogEntrys();
+      {
+        bool consumersOk = true;
+        for (i = 0; i < g_consumers.size(); i++) {
+          consumersOk &= g_consumers[i]->endOfLogEntrys();
+        }
+
+        if (!consumersOk) {
+          restoreLogger.log_error(
+              "Restore: Error reading the data log. Exiting");
+          return NdbToolsProgramExitCode::FAILED;
+        }
+      }
 
       /* report to clusterlog if applicable */
       for (i = 0; i < g_consumers.size(); i++) {
@@ -2302,6 +2347,9 @@ int do_restore(RestoreThreadData *thrdata) {
             }
           }
         }
+        if (ga_error_thread > 0) {
+          return NdbToolsProgramExitCode::FAILED;
+        }
       }
     }
 
@@ -2321,6 +2369,9 @@ int do_restore(RestoreThreadData *thrdata) {
             return NdbToolsProgramExitCode::FAILED;
           }
         }
+        if (ga_error_thread > 0) {
+          return NdbToolsProgramExitCode::FAILED;
+        }
       }
       if (ga_num_slices != 1) {
         restoreLogger.log_info(
@@ -2384,11 +2435,7 @@ int do_restore(RestoreThreadData *thrdata) {
 
   unsigned j;
   for (j = 0; j < g_consumers.size(); j++) {
-    if (g_consumers[j]->has_temp_error()) {
-      ndbout_c(
-          "\nRestore successful, but encountered temporary error, "
-          "please look at configuration.");
-    }
+    g_consumers[j]->log_temp_errors();
   }
 
   if (ga_rebuild_indexes) {
@@ -2408,13 +2455,16 @@ int do_restore(RestoreThreadData *thrdata) {
 
     for (i = 0; i < metaData.getNoOfTables(); i++) {
       const TableS *table = metaData[i];
-      if (!(checkSysTable(table) && checkDbAndTableName(table))) continue;
+      if (!rebuildSysTableIdx(table)) continue;
       if (isBlobTable(table) || isIndex(table)) continue;
       for (Uint32 j = 0; j < g_consumers.size(); j++) {
         if (!g_consumers[j]->rebuild_indexes(*table)) {
           return NdbToolsProgramExitCode::FAILED;
         }
       }
+      if (ga_error_thread > 0) {
+        return NdbToolsProgramExitCode::FAILED;
+      }
     }
     for (Uint32 j = 0; j < g_consumers.size(); j++) {
       if (!g_consumers[j]->endOfTablesFK()) {
@@ -2481,7 +2531,7 @@ int detect_backup_format() {
       //      BACKUP-1-PART-1-OF-3 : not found, continue
       //      BACKUP-1-PART-1-OF-4 : FOUND, set ga_part_count and break
       BaseString::snprintf(
-          name, sz, "%s%sBACKUP-%d-PART-1-OF-%u%sBACKUP-%u.%d.ctl",
+          name, sz, "%s%sBACKUP-%u-PART-1-OF-%u%sBACKUP-%u.%d.ctl",
           ga_backupPath, DIR_SEPARATOR, ga_backupId, ga_part_count,
           DIR_SEPARATOR, ga_backupId, ga_nodeId);
       if (my_stat(name, &buf, 0)) {
