@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -261,14 +261,40 @@ class PFS_index_data_locks : public PFS_engine_index {
   }
 };
 
-class PFS_index_data_locks_by_lock_id : public PFS_index_data_locks {
+class PFS_pk_data_locks : public PFS_index_data_locks {
  public:
-  PFS_index_data_locks_by_lock_id()
+  PFS_pk_data_locks()
       : PFS_index_data_locks(&m_key_1, &m_key_2),
         m_key_1("ENGINE_LOCK_ID"),
         m_key_2("ENGINE") {}
 
-  ~PFS_index_data_locks_by_lock_id() override = default;
+  ~PFS_pk_data_locks() override = default;
+
+  pk_pos_data_lock *get_pk() {
+    if (m_fields >= 1) {
+      const char *key_value;
+      size_t key_value_length;
+      bool is_null;
+
+      /* Read the value of ENGINE_LOCK_ID. */
+      m_key_1.get_exact_key_value(is_null, key_value, key_value_length);
+      if (is_null) {
+        return nullptr;
+      }
+
+      /* Build a primary key with it. */
+      m_pk_pos.set(key_value, key_value_length);
+
+      /*
+       * IMPORTANT NOTE:
+       * We do not read the second field, ENGINE.
+       * See comments in table_data_locks::index_next()
+       */
+      return &m_pk_pos;
+    }
+
+    return nullptr;
+  }
 
   bool match_lock_id(const char *engine_lock_id,
                      size_t engine_lock_id_length) override {
@@ -294,6 +320,7 @@ class PFS_index_data_locks_by_lock_id : public PFS_index_data_locks {
  private:
   PFS_key_engine_lock_id m_key_1;
   PFS_key_engine_name m_key_2;
+  pk_pos_data_lock m_pk_pos;
 };
 
 class PFS_index_data_locks_by_transaction_id : public PFS_index_data_locks {
@@ -416,6 +443,10 @@ class PFS_index_data_lock_waits : public PFS_engine_index {
   PFS_index_data_lock_waits(PFS_engine_key *key_1, PFS_engine_key *key_2)
       : PFS_engine_index(key_1, key_2) {}
 
+  PFS_index_data_lock_waits(PFS_engine_key *key_1, PFS_engine_key *key_2,
+                            PFS_engine_key *key_3)
+      : PFS_engine_index(key_1, key_2, key_3) {}
+
   ~PFS_index_data_lock_waits() override = default;
 
   virtual bool match_engine(const char *engine [[maybe_unused]],
@@ -462,6 +493,91 @@ class PFS_index_data_lock_waits : public PFS_engine_index {
   }
 };
 
+class PFS_pk_data_lock_waits : public PFS_index_data_lock_waits {
+ public:
+  PFS_pk_data_lock_waits()
+      : PFS_index_data_lock_waits(&m_key_1, &m_key_2, &m_key_3),
+        m_key_1("REQUESTING_ENGINE_LOCK_ID"),
+        m_key_2("BLOCKING_ENGINE_LOCK_ID"),
+        m_key_3("ENGINE") {}
+
+  ~PFS_pk_data_lock_waits() override = default;
+
+  pk_pos_data_lock_wait *get_pk() {
+    if (m_fields >= 2) {
+      const char *key_value_1;
+      size_t key_value_length_1;
+      bool is_null_1;
+      const char *key_value_2;
+      size_t key_value_length_2;
+      bool is_null_2;
+
+      /* Read the value of REQUESTING_ENGINE_LOCK_ID. */
+      m_key_1.get_exact_key_value(is_null_1, key_value_1, key_value_length_1);
+      if (is_null_1) {
+        return nullptr;
+      }
+
+      /* Read the value of BLOCKING_ENGINE_LOCK_ID. */
+      m_key_2.get_exact_key_value(is_null_2, key_value_2, key_value_length_2);
+      if (is_null_2) {
+        return nullptr;
+      }
+
+      /* Build a primary key with it. */
+      m_pk_pos.set(key_value_1, key_value_length_1, key_value_2,
+                   key_value_length_2);
+
+      /*
+       * IMPORTANT NOTE:
+       * We do not read the third field, ENGINE.
+       * See comments in table_data_lock_waits::index_next()
+       */
+      return &m_pk_pos;
+    }
+
+    return nullptr;
+  }
+
+  bool match_requesting_lock_id(const char *engine_lock_id,
+                                size_t engine_lock_id_length) override {
+    if (m_fields >= 1) {
+      if (!m_key_1.match(engine_lock_id, engine_lock_id_length)) {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  bool match_blocking_lock_id(const char *engine_lock_id,
+                              size_t engine_lock_id_length) override {
+    if (m_fields >= 2) {
+      if (!m_key_2.match(engine_lock_id, engine_lock_id_length)) {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  bool match_engine(const char *engine, size_t engine_length) override {
+    if (m_fields >= 3) {
+      if (!m_key_3.match(engine, engine_length)) {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+ private:
+  PFS_key_engine_lock_id m_key_1;
+  PFS_key_engine_lock_id m_key_2;
+  PFS_key_engine_name m_key_3;
+  pk_pos_data_lock_wait m_pk_pos;
+};
+
 class PFS_index_data_lock_waits_by_requesting_lock_id
     : public PFS_index_data_lock_waits {
  public:
