@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -135,6 +135,7 @@
 #include "storage/perfschema/table_setup_actors.h"
 #include "storage/perfschema/table_setup_consumers.h"
 #include "storage/perfschema/table_setup_instruments.h"
+#include "storage/perfschema/table_setup_loggers.h"
 #include "storage/perfschema/table_setup_meters.h"
 #include "storage/perfschema/table_setup_metrics.h"
 #include "storage/perfschema/table_setup_objects.h"
@@ -507,6 +508,7 @@ static PFS_engine_table_share *all_shares[] = {
     &table_setup_actors::m_share,
     &table_setup_consumers::m_share,
     &table_setup_instruments::m_share,
+    &table_setup_loggers::m_share,
     &table_setup_meters::m_share,
     &table_setup_metrics::m_share,
     &table_setup_objects::m_share,
@@ -1516,6 +1518,7 @@ void PFS_engine_index::read_key(const uchar *key, uint key_len,
   }
 
   m_fields = reader.m_parts_found;
+  m_key_fetch_count = 0;
 }
 
 /** @} */
