@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -383,7 +383,7 @@ int table_events_waits_common::make_metadata_lock_object_columns(
 
   if (safe_metadata_lock->get_version() == wait->m_weak_version) {
     // TODO: remove code duplication with PFS_column_row::make_row()
-    static_assert(MDL_key::NAMESPACE_END == 18,
+    static_assert(MDL_key::NAMESPACE_END == 19,
                   "Adjust performance schema when changing enum_mdl_namespace");
 
     const MDL_key *mdl = &safe_metadata_lock->m_mdl_key;
@@ -431,6 +431,13 @@ int table_events_waits_common::make_metadata_lock_object_columns(
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
+      case MDL_key::LIBRARY:
+        m_row.m_object_type = "LIBRARY";
+        m_row.m_object_type_length = 7;
+        set_schema_name(&m_row.m_object_schema, mdl);
+        m_row.m_object_name_length = mdl->name_length();
+        m_row.m_index_name_length = 0;
+        break;
       case MDL_key::TRIGGER:
         m_row.m_object_type = "TRIGGER";
         m_row.m_object_type_length = 7;
