@@ -1,4 +1,4 @@
-/* Copyright (c) 2023, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2023, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -40,8 +40,8 @@ constexpr bool CanTypeFitValue(const U value) {
 template <typename T, typename U>
 T Clamp(U x) {
   return CanTypeFitValue<T>(x) ? T(x)
-                               : x < 0 ? std::numeric_limits<T>::min()
-                                       : std::numeric_limits<T>::max();
+         : x < 0               ? std::numeric_limits<T>::min()
+                               : std::numeric_limits<T>::max();
 }
 
 // simple (no measurement attributes supported) metric callback
@@ -57,6 +57,7 @@ static void get_metric_simple_integer(void *measurement_context,
   delivery->value_int64(delivery_context, value);
 }
 
+#ifdef HAVE_PSI_METRICS_INTERFACE
 static void get_metric_mutex_instances_lost(
     void * /* measurement_context */, measurement_delivery_callback_t delivery,
     void *delivery_context) {
@@ -117,8 +118,8 @@ static PSI_metric_info_v1 ps_metrics[] = {
      "The number of indexes for which statistics were lost "
      "(Performance_schema_index_stat_lost)",
      MetricOTELType::ASYNC_COUNTER, MetricNumType::METRIC_INTEGER, 0, 0,
-     get_metric_simple_integer<decltype(
-         global_table_share_index_container.m_lost)>,
+     get_metric_simple_integer<
+         decltype(global_table_share_index_container.m_lost)>,
      &global_table_share_index_container.m_lost},
     {"locker_lost", "",
      "How many events are 'lost' or not recorded "
@@ -148,6 +149,12 @@ static PSI_metric_info_v1 ps_metrics[] = {
      MetricOTELType::ASYNC_COUNTER, MetricNumType::METRIC_INTEGER, 0, 0,
      get_metric_simple_integer<decltype(metric_class_lost)>,
      &metric_class_lost},
+    {"logger_lost", "",
+     "How many logger instruments could not be loaded "
+     "(Performance_schema_logger_lost)",
+     MetricOTELType::ASYNC_COUNTER, MetricNumType::METRIC_INTEGER, 0, 0,
+     get_metric_simple_integer<decltype(logger_class_lost)>,
+     &logger_class_lost},
     {"mutex_classes_lost", "",
      "How many mutex instruments could not be loaded "
      "(Performance_schema_mutex_classes_lost)",
@@ -240,8 +247,8 @@ static PSI_metric_info_v1 ps_metrics[] = {
      "The number of tables for which lock statistics were lost "
      "(Performance_schema_table_lock_stat_lost)",
      MetricOTELType::ASYNC_COUNTER, MetricNumType::METRIC_INTEGER, 0, 0,
-     get_metric_simple_integer<decltype(
-         global_table_share_lock_container.m_lost)>,
+     get_metric_simple_integer<
+         decltype(global_table_share_lock_container.m_lost)>,
      &global_table_share_lock_container.m_lost},
     {"thread_classes_lost", "",
      "How many thread instruments could not be loaded "
@@ -265,11 +272,16 @@ static PSI_metric_info_v1 ps_metrics[] = {
 static PSI_meter_info_v1 ps_meters[] = {
     {"mysql.perf_schema", "MySql performance_schema lost instruments", 10, 0, 0,
      ps_metrics, std::size(ps_metrics)}};
+#endif /* HAVE_PSI_METRICS_INTERFACE */
 
 void register_pfs_metric_sources() {
+#ifdef HAVE_PSI_METRICS_INTERFACE
   mysql_meter_register(ps_meters, std::size(ps_meters));
+#endif /* HAVE_PSI_METRICS_INTERFACE */
 }
 
 void unregister_pfs_metric_sources() {
+#ifdef HAVE_PSI_METRICS_INTERFACE
   mysql_meter_unregister(ps_meters, std::size(ps_meters));
+#endif /* HAVE_PSI_METRICS_INTERFACE */
 }
