@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -24,6 +24,7 @@
 #include <assert.h>
 #include <string.h>
 
+#include "my_inttypes.h"
 #include "my_sys.h"
 #include "storage/perfschema/pfs_global.h"
 
@@ -32,6 +33,7 @@ size_t pfs_allocated_memory_size = 0;
 size_t pfs_allocated_memory_count = 0;
 
 bool stub_alloc_always_fails = true;
+bool stub_alloc_maybe_fails = true;
 int stub_alloc_fails_after_count = 0;
 
 void *pfs_malloc(PFS_builtin_memory_class *, size_t size, myf) {
@@ -42,15 +44,64 @@ void *pfs_malloc(PFS_builtin_memory_class *, size_t size, myf) {
 
   if (stub_alloc_always_fails) return nullptr;
 
-  if (--stub_alloc_fails_after_count <= 0) return nullptr;
+  if (stub_alloc_maybe_fails) {
+    if (--stub_alloc_fails_after_count <= 0) {
+      return nullptr;
+    }
+  }
+
+  void *ptr = nullptr;
+
+#ifdef PFS_ALIGNMENT
+#ifdef HAVE_POSIX_MEMALIGN
+  /* Linux */
+  if (unlikely(posix_memalign(&ptr, PFS_ALIGNMENT, size))) {
+    return nullptr;
+  }
+#else
+#ifdef HAVE_MEMALIGN
+  /* Solaris */
+  ptr = memalign(PFS_ALIGNMENT, size);
+#else
+#ifdef HAVE_ALIGNED_MALLOC
+  /* Windows */
+  ptr = _aligned_malloc(size, PFS_ALIGNMENT);
+#else
+#error "Missing implementation for PFS_ALIGNMENT"
+#endif /* HAVE_ALIGNED_MALLOC */
+#endif /* HAVE_MEMALIGN */
+#endif /* HAVE_POSIX_MEMALIGN */
+#else  /* PFS_ALIGNMENT */
+  /* Everything else */
+  ptr = malloc(size);
+#endif
 
-  void *ptr = malloc(size);
   if (ptr != nullptr) memset(ptr, 0, size);
   return ptr;
 }
 
 void pfs_free(PFS_builtin_memory_class *, size_t, void *ptr) {
-  if (ptr != nullptr) free(ptr);
+  if (ptr == nullptr) {
+    return;
+  }
+
+#ifdef HAVE_POSIX_MEMALIGN
+  /* Allocated with posix_memalign() */
+  free(ptr);
+#else
+#ifdef HAVE_MEMALIGN
+  /* Allocated with memalign() */
+  free(ptr);
+#else
+#ifdef HAVE_ALIGNED_MALLOC
+  /* Allocated with _aligned_malloc() */
+  _aligned_free(ptr);
+#else
+  /* Allocated with malloc() */
+  free(ptr);
+#endif /* HAVE_ALIGNED_MALLOC */
+#endif /* HAVE_MEMALIGN */
+#endif /* HAVE_POSIX_MEMALIGN */
 }
 
 void *pfs_malloc_array(PFS_builtin_memory_class *klass, size_t n, size_t size,
