@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -27,6 +28,7 @@ TempTable custom allocator. */
 #define TEMPTABLE_ALLOCATOR_H
 
 #include <algorithm>  // std::max
+#include <atomic>     // std::atomic
 #include <cstddef>    // size_t
 #include <limits>     // std::numeric_limits
 #include <memory>     // std::shared_ptr
@@ -35,13 +37,16 @@ TempTable custom allocator. */
 
 #include "my_dbug.h"
 #include "my_sys.h"
-#include "sql/mysqld.h"  // temptable_max_ram, temptable_max_mmap
+#include "sql/current_thd.h"  // current_thd
+#include "sql/mysqld.h"       // temptable_max_ram, temptable_max_mmap
+#include "sql/sql_class.h"    // current_thd
 #include "storage/temptable/include/temptable/block.h"
 #include "storage/temptable/include/temptable/chunk.h"
 #include "storage/temptable/include/temptable/constants.h"
 #include "storage/temptable/include/temptable/memutils.h"
 
 namespace temptable {
+extern std::atomic_uint64_t count_hit_max_ram;
 
 /* Thin abstraction which enables logging of memory operations.
  *
@@ -99,13 +104,7 @@ struct MemoryMonitor {
      * Allocator.
      *
      * @return MMAP-memory threshold. */
-    static size_t threshold() {
-      if (temptable_use_mmap) {
-        return temptable_max_mmap;
-      } else {
-        return 0;
-      }
-    }
+    static size_t threshold() { return temptable_max_mmap; }
     /** Get current level of MMAP-backed memory consumption.
      *
      * @return Current level of MMAP-backed memory consumption (in bytes). */
@@ -195,6 +194,9 @@ struct Allocation_scheme {
   static size_t block_size(size_t number_of_blocks, size_t n_bytes_requested) {
     return Block_size_policy::block_size(number_of_blocks, n_bytes_requested);
   }
+  static void block_freed(uint32_t block_size, Source block_source) {
+    Block_source_policy::block_freed(block_size, block_source);
+  }
 };
 
 /* Concrete implementation of Block_source_policy, a type which controls where
@@ -219,6 +221,12 @@ struct Prefer_RAM_over_MMAP_policy {
         MemoryMonitor::RAM::decrease(block_size);
       }
     }
+
+    /* Track through the global status variable,
+     * if table exceeded the specified memory limit
+     */
+    ++count_hit_max_ram;
+
     if (MemoryMonitor::MMAP::consumption() < MemoryMonitor::MMAP::threshold()) {
       if (MemoryMonitor::MMAP::increase(block_size) <=
           MemoryMonitor::MMAP::threshold()) {
@@ -229,6 +237,14 @@ struct Prefer_RAM_over_MMAP_policy {
     }
     throw Result::RECORD_FILE_FULL;
   }
+
+  static void block_freed(uint32_t block_size, Source block_source) {
+    if (block_source == Source::RAM) {
+      MemoryMonitor::RAM::decrease(block_size);
+    } else {
+      MemoryMonitor::MMAP::decrease(block_size);
+    }
+  }
 };
 
 /* Concrete implementation of Block_size_policy, a type which controls how big
@@ -299,7 +315,77 @@ using Exponential_growth_preferring_RAM_over_MMAP =
   guide's recommendation to have clear ownership of objects, but at least it
   avoids the use-after-free.
  */
-struct AllocatorState {
+template <class AllocationScheme>
+class AllocatorState {
+ public:
+  /**
+   * Destroys the state, deallocate the current_block if it was left empty.
+   */
+  ~AllocatorState() noexcept {
+    if (!current_block.is_empty()) {
+      free_block(current_block);
+    }
+    /* User must deallocate all data from all blocks, otherwise the memory will
+     * be leaked.
+     */
+    assert(number_of_blocks == 0);
+  }
+
+  /**
+   * Gets a Block from which a new allocation of the specified size should be
+   * performed. It will use the current Block or create a new one if it is too
+   * small.
+   * [in] Number of bytes that will be allocated from the returned block.
+   */
+  Block *get_block_for_new_allocation(size_t n_bytes_requested) {
+    if (current_block.is_empty() ||
+        !current_block.can_accommodate(n_bytes_requested)) {
+      /* The current_block may have been left empty during some deallocate()
+       * call. It is the last opportunity to free it before we lose reference to
+       * it.
+       */
+      if (!current_block.is_empty() &&
+          current_block.number_of_used_chunks() == 0) {
+        free_block(current_block);
+      }
+
+      const size_t block_size =
+          AllocationScheme::block_size(number_of_blocks, n_bytes_requested);
+      current_block =
+          Block(block_size, AllocationScheme::block_source(block_size));
+      ++number_of_blocks;
+    }
+    return &current_block;
+  }
+
+  /**
+   * Informs the state object of a block that has no data allocated inside of it
+   * anymore for garbage collection.
+   * [in] The empty block to manage and possibly free.
+   */
+  void block_is_not_used_anymore(Block block) noexcept {
+    if (block == current_block) {
+      /* Do nothing. Keep the last block alive. Some queries are repeatedly
+       * allocating one Row and freeing it, leading to constant allocation and
+       * deallocation of 1MB of memory for the current_block. Let's keep this
+       * block empty ready for a future use.
+       */
+    } else {
+      free_block(block);
+    }
+  }
+
+ private:
+  /**
+   * Frees the specified block and takes care of all accounting.
+   * [in] The empty block to free.
+   */
+  void free_block(Block &block) noexcept {
+    AllocationScheme::block_freed(block.size(), block.type());
+    block.destroy();
+    --number_of_blocks;
+  }
+
   /** Current not-yet-full block to feed allocations from. */
   Block current_block;
 
@@ -432,7 +518,7 @@ class Allocator {
       /** [in] Memory where to create the object. */
       U *mem,
       /** Arguments to pass to U's constructor. */
-      Args &&... args);
+      Args &&...args);
 
   /** Destroy an object of type `U`. The memory is not returned to the OS, this
    * is the counterpart of `construct()`. */
@@ -449,7 +535,7 @@ class Allocator {
     Shared state between all the copies and rebinds of this allocator.
     See AllocatorState for details.
    */
-  std::shared_ptr<AllocatorState> m_state;
+  std::shared_ptr<AllocatorState<AllocationScheme>> m_state;
 
   /** A block of memory which is a state external to this allocator and can be
    * shared among different instances of the allocator (not simultaneously). In
@@ -468,7 +554,7 @@ class Allocator {
 template <class T, class AllocationScheme>
 inline Allocator<T, AllocationScheme>::Allocator(
     Block *shared_block, TableResourceMonitor &table_resource_monitor)
-    : m_state(std::make_shared<AllocatorState>()),
+    : m_state(std::make_shared<AllocatorState<AllocationScheme>>()),
       m_shared_block(shared_block),
       m_table_resource_monitor(table_resource_monitor) {}
 
@@ -526,16 +612,8 @@ inline T *Allocator<T, AllocationScheme>::allocate(size_t n_elements) {
   } else if (m_shared_block &&
              m_shared_block->can_accommodate(n_bytes_requested)) {
     block = m_shared_block;
-  } else if (m_state->current_block.is_empty() ||
-             !m_state->current_block.can_accommodate(n_bytes_requested)) {
-    const size_t block_size = AllocationScheme::block_size(
-        m_state->number_of_blocks, n_bytes_requested);
-    m_state->current_block =
-        Block(block_size, AllocationScheme::block_source(block_size));
-    block = &m_state->current_block;
-    ++m_state->number_of_blocks;
   } else {
-    block = &m_state->current_block;
+    block = m_state->get_block_for_new_allocation(n_bytes_requested);
   }
 
   /* temptable::Table is allowed to fit no more data than the given threshold
@@ -552,6 +630,12 @@ inline T *Allocator<T, AllocationScheme>::allocate(size_t n_elements) {
    */
   if (m_table_resource_monitor.consumption() + n_bytes_requested >
       m_table_resource_monitor.threshold()) {
+    if (current_thd) {
+      /* Track through the status variable, if table
+       * size exceeded the tmp_table_size in user session
+       */
+      current_thd->inc_status_count_hit_tmp_table_size();
+    }
     throw Result::RECORD_FILE_FULL;
   }
   m_table_resource_monitor.increase(n_bytes_requested);
@@ -580,18 +664,7 @@ inline void Allocator<T, AllocationScheme>::deallocate(T *chunk_data,
     if (m_shared_block && (block == *m_shared_block)) {
       // Do nothing. Keep the last block alive.
     } else {
-      assert(m_state->number_of_blocks > 0);
-      if (block.type() == Source::RAM) {
-        MemoryMonitor::RAM::decrease(block.size());
-      } else {
-        MemoryMonitor::MMAP::decrease(block.size());
-      }
-      if (block == m_state->current_block) {
-        m_state->current_block.destroy();
-      } else {
-        block.destroy();
-      }
-      --m_state->number_of_blocks;
+      m_state->block_is_not_used_anymore(block);
     }
   }
   m_table_resource_monitor.decrease(n_bytes_requested);
@@ -599,7 +672,7 @@ inline void Allocator<T, AllocationScheme>::deallocate(T *chunk_data,
 
 template <class T, class AllocationScheme>
 template <class U, class... Args>
-inline void Allocator<T, AllocationScheme>::construct(U *mem, Args &&... args) {
+inline void Allocator<T, AllocationScheme>::construct(U *mem, Args &&...args) {
   new (mem) U(std::forward<Args>(args)...);
 }
 
