@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -29,7 +29,7 @@ TempTable Indexed Column. */
 
 #include <cstddef>
 
-#include "m_ctype.h"
+#include "mysql/strings/m_ctype.h"
 #include "sql/field.h"
 #include "sql/key.h"
 #include "storage/temptable/include/temptable/cell_calculator.h"
