@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -24,10 +25,15 @@ this program; if not, write to the Free Software Foundation, Inc.,
 TempTable public handler API implementation. */
 
 #include "storage/temptable/include/temptable/handler.h"
+
+#include <string.h>
+
 #include "my_base.h"
 #include "my_dbug.h"
+#include "mysql/components/services/log_builtins.h"
 #include "mysql/plugin.h"
 #include "sql/mysqld.h"
+#include "sql/sql_class.h"
 #include "sql/sql_thd_internal_api.h"
 #include "sql/system_variables.h"
 #include "storage/temptable/include/temptable/row.h"
@@ -37,27 +43,66 @@ TempTable public handler API implementation. */
 namespace temptable {
 
 /** Key-value store containing all tables for all existing connections.
- * See `Sharded_key_value_store` documentation for more details.
- * */
+ * See `Sharded_key_value_store` documentation for more details. */
 static Sharded_key_value_store<KV_STORE_SHARDS_COUNT> kv_store_shard;
 
 /* Pool of shared-blocks, an external state to the custom `TempTable` memory
  * allocator. See `Lock_free_shared_block_pool` documentation for more details.
- * */
+ */
 static Lock_free_shared_block_pool<SHARED_BLOCK_POOL_SIZE> shared_block_pool;
 
-/** Small helper function which debug-prints the miscelaneous statistics which
- * key-value store has collected.
- * */
-void kv_store_shards_debug_dump() { kv_store_shard.dbug_print(); }
+/** Small helper function which debug-prints the miscellaneous statistics which
+ * key-value store has collected. */
+static void kv_store_shards_debug_dump() { kv_store_shard.dbug_print(); }
+
+/** Helper function that calls the logging API to log errors. */
+static void log_event(const std::string &msg, loglevel lvl) {
+  LogEvent()
+      .type(LOG_TYPE_ERROR)
+      .prio(lvl)
+      .errcode(ER_TEMPTABLE_ENGINE_ERROR)
+      .subsys("Temptable")
+      .verbatim(msg.c_str());
+}
+
+/** Function that erases leaked temptables by accessing the appropriate shard
+ * for the THD involved and iterating through the shard to find all temptables
+ * owned by this THD. This should only be called during close_connection if
+ * there is a leaked temptable in the optimizer layer, so any time this is
+ * called it indicates a bug. It logs an error to the server logs. */
+static void erase_owned_temptables(THD *thd) {
+  const auto id = thd_thread_id(thd);
+  const std::vector<std::string> tables = kv_store_shard[id].find_all(id);
+
+  for (const auto &t : tables) {
+    kv_store_shard[id].erase(t);
+    thd->decrement_temptable_count();
+    std::stringstream msg;
+    msg << "Detected leaked temptable used internally for query execution: "
+        << t
+        << ". The leak has been cleaned up but we shouldn't end up here "
+           "during normal operation. Please report a bug.";
+    log_event(msg.str(), ERROR_LEVEL);
+  }
+}
 
 /** Small helper function which releases the slot (and memory occupied by the
- * Block) in shared-block pool.
- * */
-void shared_block_pool_release(THD *thd) {
+ * Block) in shared-block pool. */
+static void shared_block_pool_release(THD *thd) {
   shared_block_pool.try_release(thd_thread_id(thd));
 }
 
+/** Helper function that performs the temptable related cleanup. Called from
+ * plugin->close_connection. */
+void close_connection(THD *thd) {
+  if (thd->get_temptable_count() > 0) {
+    temptable::erase_owned_temptables(thd);
+  }
+  assert(thd->get_temptable_count() == 0);
+  temptable::kv_store_shards_debug_dump();
+  temptable::shared_block_pool_release(thd);
+}
+
 #if defined(HAVE_WINNUMA)
 /** Page size used in memory allocation. */
 DWORD win_page_size;
@@ -102,8 +147,6 @@ Handler::Handler(handlerton *hton, TABLE_SHARE *table_share_arg)
 #endif /* HAVE_WINNUMA */
 }
 
-Handler::~Handler() {}
-
 int Handler::create(const char *table_name, TABLE *mysql_table,
                     HA_CREATE_INFO *, dd::Table *) {
   DBUG_TRACE;
@@ -131,14 +174,25 @@ int Handler::create(const char *table_name, TABLE *mysql_table,
     DBUG_EXECUTE_IF("temptable_create_return_non_result_type_exception",
                     throw 42;);
 
+    // Calculate m_number_of_elements_per_page, see Table::Table():
+    if (all_columns_are_fixed_size) {
+      Storage rows_of_the_table = Storage(nullptr);
+      rows_of_the_table.element_size(mysql_table->s->rec_buff_length);
+      if (rows_of_the_table.number_of_elements_per_page() == 0)
+        DBUG_RET(Result::TOO_BIG_ROW);
+    }
+
+    size_t per_table_limit = thd_get_tmp_table_size(ha_thd());
     auto &kv_store = kv_store_shard[thd_thread_id(ha_thd())];
     const auto insert_result = kv_store.emplace(
         std::piecewise_construct, std::forward_as_tuple(table_name),
         std::forward_as_tuple(mysql_table, m_shared_block,
-                              all_columns_are_fixed_size));
+                              all_columns_are_fixed_size, per_table_limit));
 
     ret = insert_result.second ? Result::OK : Result::TABLE_EXIST;
-
+    if (insert_result.second) {
+      ha_thd()->increment_temptable_count();
+    }
   } catch (Result ex) {
     ret = ex;
   } catch (...) {
@@ -149,6 +203,7 @@ int Handler::create(const char *table_name, TABLE *mysql_table,
 }
 
 int Handler::delete_table(const char *table_name, const dd::Table *) {
+  DBUG_EXECUTE_IF("temptable_leak_all", { DBUG_RET(Result::OK); });
   DBUG_TRACE;
 
   assert(table_name != nullptr);
@@ -162,6 +217,7 @@ int Handler::delete_table(const char *table_name, const dd::Table *) {
       if (m_opened_table != table) {
         kv_store.erase(table_name);
         ret = Result::OK;
+        ha_thd()->decrement_temptable_count();
       } else {
         /* Attempt to delete the currently opened table. */
         ret = Result::UNSUPPORTED;
@@ -307,7 +363,8 @@ int Handler::rnd_pos(uchar *mysql_row, uchar *position) {
 
   handler::ha_statistic_increment(&System_status_var::ha_read_rnd_count);
 
-  Storage::Element *row = *reinterpret_cast<Storage::Element **>(position);
+  Storage::Element *row;
+  memcpy(&row, position, sizeof(row));
 
   m_rnd_iterator = Storage::Iterator(&m_opened_table->rows(), row);
 
@@ -446,6 +503,7 @@ int Handler::index_read(uchar *mysql_row, const uchar *mysql_search_cells,
       case HA_READ_MBR_WITHIN:
       case HA_READ_MBR_DISJOINT:
       case HA_READ_MBR_EQUAL:
+      case HA_READ_NEAREST_NEIGHBOR:
       case HA_READ_INVALID:
         ret = Result::UNSUPPORTED;
         break;
@@ -842,8 +900,8 @@ uint Handler::max_supported_key_length() const {
   return length;
 }
 
-uint Handler::max_supported_key_part_length(
-    HA_CREATE_INFO *create_info MY_ATTRIBUTE((unused))) const {
+uint Handler::max_supported_key_part_length(HA_CREATE_INFO *create_info
+                                            [[maybe_unused]]) const {
   DBUG_TRACE;
 
   const uint length = std::numeric_limits<uint>::max();
