@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2007, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -203,6 +203,13 @@ size_t my_fcvt(double x, int precision, char *to, bool *error) {
   return my_fcvt_internal(x, precision, false, to, error);
 }
 
+/// Identical to my_fcvt, except that trailing zeros after the decimal point
+/// will be removed.
+size_t my_fcvt_no_trailing_zero(double x, int precision, char *to,
+                                bool *error) {
+  return my_fcvt_internal(x, precision, true, to, error);
+}
+
 /**
    @brief
    Converts a given floating point number to a zero-terminated string
@@ -341,9 +348,9 @@ size_t my_gcvt(double x, my_gcvt_arg_type type, int width, char *to,
      2. 0 < decpt < len, i.e. we have "NNN.NNN" => F = len + 1
      3. len <= decpt, i.e. we have "NNN00" => F = decpt
   */
-  have_space =
-      (decpt <= 0 ? len - decpt + 2
-                  : decpt > 0 && decpt < len ? len + 1 : decpt) <= width;
+  have_space = (decpt <= 0                 ? len - decpt + 2
+                : decpt > 0 && decpt < len ? len + 1
+                                           : decpt) <= width;
   /*
     The following is true when no significant digits can be placed with the
     specified field width using the 'f' format, and the 'e' format
@@ -746,7 +753,7 @@ static void Bfree(Bigint *v, Stack_alloc *alloc) {
 
 static char *dtoa_alloc(int i, Stack_alloc *alloc) {
   char *rv;
-  int aligned_size = MY_ALIGN(i, SIZEOF_CHARP);
+  int const aligned_size = MY_ALIGN(i, SIZEOF_CHARP);
   if (alloc->free + aligned_size <= alloc->end) {
     rv = alloc->free;
     alloc->free += aligned_size;
@@ -814,7 +821,7 @@ static Bigint *s2b(const char *s, int nd0, int nd, uint32_t y9,
   int i, k;
   int32_t y = 0;
 
-  int32_t x = (nd + 8) / 9;
+  int32_t const x = (nd + 8) / 9;
   for (k = 0, y = 1; x > y; y <<= 1, k++)
     ;
   b = Balloc(k, alloc);
@@ -824,8 +831,7 @@ static Bigint *s2b(const char *s, int nd0, int nd, uint32_t y9,
   i = 9;
   if (9 < nd0) {
     s += 9;
-    do
-      b = multadd(b, 10, *s++ - '0', alloc);
+    do b = multadd(b, 10, *s++ - '0', alloc);
     while (++i < nd0);
     s++; /* skip '.' */
   } else
@@ -995,7 +1001,7 @@ static Bigint p5_a[] = {
 static Bigint *pow5mult(Bigint *b, int k, Stack_alloc *alloc) {
   Bigint *b1, *p5, *p51 = nullptr;
   int i;
-  static int p05[3] = {5, 25, 125};
+  static int const p05[3] = {5, 25, 125};
   bool overflow = false;
 
   if ((i = k & 3)) b = multadd(b, p05[i - 1], 0, alloc);
@@ -1048,8 +1054,7 @@ static Bigint *lshift(Bigint *b, int k, Stack_alloc *alloc) {
     } while (x < xe);
     if ((*x1 = z)) ++n1;
   } else
-    do
-      *x1++ = *x++;
+    do *x1++ = *x++;
     while (x < xe);
   b1->wds = n1 - 1;
   Bfree(b, alloc);
@@ -1121,7 +1126,7 @@ static Bigint *diff(Bigint *a, Bigint *b, Stack_alloc *alloc) {
 static double ulp(U *x) {
   U u;
 
-  int32_t L = (word0(x) & Exp_mask) - (P - 1) * Exp_msk1;
+  int32_t const L = (word0(x) & Exp_mask) - (P - 1) * Exp_msk1;
   word0(&u) = L;
   word1(&u) = 0;
   return dval(&u);
@@ -1141,7 +1146,7 @@ static double b2d(Bigint *a, int *e) {
   *e = 32 - k;
   if (k < Ebits) {
     d0 = Exp_1 | y >> (Ebits - k);
-    uint32_t w = xa > xa0 ? *--xa : 0;
+    uint32_t const w = xa > xa0 ? *--xa : 0;
     d1 = y << ((32 - Ebits) + k) | w >> (Ebits - k);
     goto ret_d;
   }
