@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -77,8 +78,8 @@ TEST_F(dynamic_loader, try_unload_the_same_component_in_group) {
   static const char *urns[] = {"file://component_example_component1"};
   std::string path;
   const char *absolute_urns[2];
-  for (int i = 0; i < 2; i++)
-    absolute_urns[i] = (char *)malloc(2046 * sizeof(char));
+  for (auto &absolute_urn : absolute_urns)
+    absolute_urn = (char *)malloc(2046 * sizeof(char));
   make_absolute_urn(*urns, &path);
   strcpy(const_cast<char *>(absolute_urns[0]), path.c_str());
   ASSERT_FALSE(loader->load(absolute_urns, 1));
@@ -88,7 +89,8 @@ TEST_F(dynamic_loader, try_unload_the_same_component_in_group) {
   strcpy(const_cast<char *>(absolute_urns[1]), path.c_str());
   ASSERT_TRUE(loader->unload((const char **)absolute_urns, 2));
   ASSERT_FALSE(loader->unload((const char **)&absolute_urns[0], 1));
-  for (int i = 0; i < 2; i++) free(const_cast<char *>(absolute_urns[i]));
+  for (auto &absolute_urn : absolute_urns)
+    free(const_cast<char *>(absolute_urn));
 }
 
 TEST_F(dynamic_loader, try_load_twice) {
@@ -100,7 +102,7 @@ TEST_F(dynamic_loader, try_load_twice) {
   ASSERT_FALSE(loader->load(&absolute_urns, 1));
   ASSERT_TRUE(loader->load(&absolute_urns, 1));
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_FALSE((bool)service);
   }
 
@@ -150,17 +152,17 @@ TEST_F(dynamic_loader, load_different_components) {
   std::string path1, path2, path3;
   const char *absolute_urn;
   const char *absolute_urns[2];
-  for (int i = 0; i < 2; i++)
-    absolute_urns[i] = (char *)malloc(2046 * sizeof(char));
+  for (auto &loop_absolute_urn : absolute_urns)
+    loop_absolute_urn = (char *)malloc(2046 * sizeof(char));
   make_absolute_urn(*urns1, &path1);
   absolute_urn = path1.c_str();
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_TRUE((bool)service);
   }
   ASSERT_FALSE(loader->load(&absolute_urn, 1));
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_FALSE((bool)service);
   }
   ASSERT_FALSE(loader->unload(&absolute_urn, 1));
@@ -170,15 +172,16 @@ TEST_F(dynamic_loader, load_different_components) {
   strcpy(const_cast<char *>(absolute_urns[1]), path3.c_str());
   ASSERT_FALSE(loader->load(absolute_urns, 2));
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_FALSE((bool)service);
   }
   ASSERT_FALSE(loader->unload(absolute_urns, 2));
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_TRUE((bool)service);
   }
-  for (int i = 0; i < 2; i++) free(const_cast<char *>(absolute_urns[i]));
+  for (auto &loop_absolute_urn : absolute_urns)
+    free(const_cast<char *>(loop_absolute_urn));
 }
 
 TEST_F(dynamic_loader, dependencies) {
@@ -188,17 +191,17 @@ TEST_F(dynamic_loader, dependencies) {
   std::string path1, path2;
   const char *absolute_urn;
   const char *absolute_urns[2];
-  for (int i = 0; i < 2; i++)
-    absolute_urns[i] = (char *)malloc(2046 * sizeof(char));
+  for (auto &loop_absolute_urn : absolute_urns)
+    loop_absolute_urn = (char *)malloc(2046 * sizeof(char));
   make_absolute_urn(*urns1, &path1);
   absolute_urn = path1.c_str();
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_TRUE((bool)service);
   }
   ASSERT_TRUE(loader->load(&absolute_urn, 1));
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_TRUE((bool)service);
   }
   make_absolute_urn(urns2[0], &path2);
@@ -208,10 +211,11 @@ TEST_F(dynamic_loader, dependencies) {
   ASSERT_FALSE(loader->load(absolute_urns, 2));
   ASSERT_FALSE(loader->unload(absolute_urns, 2));
   {
-    my_service<SERVICE_TYPE(example_math)> service("example_math", reg);
+    my_service<SERVICE_TYPE(example_math)> const service("example_math", reg);
     ASSERT_TRUE((bool)service);
   }
-  for (int i = 0; i < 2; i++) free(const_cast<char *>(absolute_urns[i]));
+  for (auto &loop_absolute_urn : absolute_urns)
+    free(const_cast<char *>(loop_absolute_urn));
 }
 
 TEST_F(dynamic_loader, cyclic_dependencies) {
@@ -246,16 +250,16 @@ TEST_F(dynamic_loader, cyclic_dependencies) {
   ASSERT_TRUE(loader->load(&absolute_cyclic_depends_broken2, 1));
 
   /* Correct cyclic dependency.*/
-  for (int i = 0; i < 2; i++)
-    absolute_cyclic_depends[i] = (char *)malloc(2046 * sizeof(char));
+  for (auto &absolute_cyclic_depend : absolute_cyclic_depends)
+    absolute_cyclic_depend = (char *)malloc(2046 * sizeof(char));
   make_absolute_urn(urns_cyclic_depends[0], &path2);
   strcpy(const_cast<char *>(absolute_cyclic_depends[0]), path2.c_str());
   make_absolute_urn(urns_cyclic_depends[1], &path3);
   strcpy(const_cast<char *>(absolute_cyclic_depends[1]), path3.c_str());
   ASSERT_FALSE(loader->load(absolute_cyclic_depends, 2));
   ASSERT_FALSE(loader->unload(absolute_cyclic_depends, 2));
-  for (int i = 0; i < 2; i++)
-    free(const_cast<char *>(absolute_cyclic_depends[i]));
+  for (auto &absolute_cyclic_depend : absolute_cyclic_depends)
+    free(const_cast<char *>(absolute_cyclic_depend));
 }
 
 TEST_F(dynamic_loader, first_dependency) {
@@ -302,8 +306,8 @@ TEST_F(dynamic_loader, iteration) {
                                "file://component_example_component3"};
   std::string path;
   const char *absolute_urns[3];
-  for (int i = 0; i < 3; i++)
-    absolute_urns[i] = (char *)malloc(2046 * sizeof(char));
+  for (auto &absolute_urn : absolute_urns)
+    absolute_urn = (char *)malloc(2046 * sizeof(char));
   make_absolute_urn(urns[0], &path);
   strcpy(const_cast<char *>(absolute_urns[0]), path.c_str());
   make_absolute_urn(urns[1], &path);
@@ -335,7 +339,8 @@ TEST_F(dynamic_loader, iteration) {
   /* there should be at least 3 test components loaded. */
   ASSERT_GE(count, 3);
   ASSERT_TRUE(test_library_found);
-  for (int i = 0; i < 3; i++) free(const_cast<char *>(absolute_urns[i]));
+  for (auto &absolute_urn : absolute_urns)
+    free(const_cast<char *>(absolute_urn));
 }
 
 TEST_F(dynamic_loader, metadata) {
@@ -347,7 +352,7 @@ TEST_F(dynamic_loader, metadata) {
       "dynamic_loader_metadata_enumerate", reg);
   ASSERT_FALSE(metadata_service);
 
-  my_service<SERVICE_TYPE(dynamic_loader_metadata_query)>
+  my_service<SERVICE_TYPE(dynamic_loader_metadata_query)> const
       metadata_query_service("dynamic_loader_metadata_query", reg);
   ASSERT_FALSE(metadata_query_service);
 
@@ -357,8 +362,8 @@ TEST_F(dynamic_loader, metadata) {
 
   std::string path;
   const char *absolute_urns[3];
-  for (int i = 0; i < 3; i++)
-    absolute_urns[i] = (char *)malloc(2046 * sizeof(char));
+  for (auto &absolute_urn : absolute_urns)
+    absolute_urn = (char *)malloc(2046 * sizeof(char));
   make_absolute_urn(urns[0], &path);
   strcpy(const_cast<char *>(absolute_urns[0]), path.c_str());
   make_absolute_urn(urns[1], &path);
@@ -425,7 +430,8 @@ TEST_F(dynamic_loader, metadata) {
       ASSERT_TRUE(property_found);
     }
   }
-  for (int i = 0; i < 3; i++) free(const_cast<char *>(absolute_urns[i]));
+  for (auto &absolute_urn : absolute_urns)
+    free(const_cast<char *>(absolute_urn));
 }
 
 /* mandatory main function */
@@ -442,7 +448,7 @@ int main(int argc, char **argv) {
   if (res_length > 0) basedir_buf[res_length - 1] = '\0';
   my_setwd(basedir_buf, 0);
 
-  int retval = RUN_ALL_TESTS();
+  int const retval = RUN_ALL_TESTS();
   my_end(0);
   return retval;
 }
