@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,9 +28,7 @@
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
 
-namespace containers {
-namespace lf {
-namespace unittests {
+namespace containers::lf::unittests {
 
 static std::atomic<int> pushed{0};
 static std::atomic<int> popped{0};
@@ -99,7 +98,7 @@ class Integrals_lockfree_queue_test : public ::testing::Test {
     popped = 0;
     removed = 0;
 
-    size_t total_threads{Threads * 4};
+    size_t const total_threads{Threads * 4};
     std::vector<std::thread> threads;
 
     for (size_t idx = 0; idx != total_threads; ++idx) {
@@ -108,7 +107,7 @@ class Integrals_lockfree_queue_test : public ::testing::Test {
             switch (n_thread % 4) {
               case 0: {  // Producer threads
                 for (size_t k = 0; k != Workload;) {
-                  value_type value = n_thread * Workload + k;
+                  value_type const value = n_thread * Workload + k;
                   queue << value;  // Using `push` or `<<` is the same
                   if (queue.get_state() == Queue::enum_queue_state::SUCCESS) {
                     ++pushed;
@@ -120,7 +119,7 @@ class Integrals_lockfree_queue_test : public ::testing::Test {
               }
               case 1: {  // Remover thread
                 for (; popped.load() + removed.load() != Workload * Threads;) {
-                  int n_success =
+                  int const n_success =
                       queue.erase_if([=](value_type item) mutable -> bool {
                         return item >= ((n_thread - 1) * Workload) &&
                                item < (n_thread * Workload);
@@ -174,12 +173,12 @@ class Integrals_lockfree_queue_test : public ::testing::Test {
 
  protected:
   Integrals_lockfree_queue_test() = default;
-  virtual void SetUp() {}
-  virtual void TearDown() {}
+  void SetUp() override {}
+  void TearDown() override {}
 };
 
 TEST_F(Integrals_lockfree_queue_test, Padding_indexing_test) {
-  size_t size = Workload;
+  size_t const size = Workload;
   container::Integrals_lockfree_queue<Integrals_lockfree_queue_test::value_type,
                                       Null, Erased>
       queue{size};
@@ -190,7 +189,7 @@ TEST_F(Integrals_lockfree_queue_test, Padding_indexing_test) {
 }
 
 TEST_F(Integrals_lockfree_queue_test, Interleaved_indexing_test) {
-  size_t size = Workload;
+  size_t const size = Workload;
   container::Integrals_lockfree_queue<
       Integrals_lockfree_queue_test::value_type, Null, Erased,
       container::Interleaved_indexing<
@@ -203,6 +202,4 @@ TEST_F(Integrals_lockfree_queue_test, Interleaved_indexing_test) {
   Integrals_lockfree_queue_test::test_queue(queue);
 }
 
-}  // namespace unittests
-}  // namespace lf
-}  // namespace containers
+}  // namespace containers::lf::unittests
