@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -31,11 +31,13 @@
 #include <string>
 
 #include "gtest/gtest-message.h"
-#include "m_ctype.h"
 #include "m_string.h"
 // assert
 #include "my_inttypes.h"
+#include "my_rnd.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
+#include "sql-common/my_decimal.h"
 #include "sql/binlog.h"
 #include "sql/client_settings.h"
 #include "sql/conn_handler/connection_handler_manager.h"
@@ -47,8 +49,7 @@
 #include "sql/derror.h"
 #include "sql/item_func.h"
 #include "sql/keycaches.h"
-#include "sql/log.h"  // query_logger
-#include "sql/my_decimal.h"
+#include "sql/log.h"     // query_logger
 #include "sql/mysqld.h"  // set_remaining_args
 #include "sql/mysqld_thd_manager.h"
 #include "sql/opt_costconstantcache.h"  // optimizer cost constant cache
@@ -210,4 +211,21 @@ void DD_initializer::TearDown() {
   delete dd::Dictionary_impl::s_instance;
 }
 
+size_t get_number_of_occurrences(std::string_view hay,
+                                 std::string_view needle) {
+  size_t pos = hay.find(needle);
+  size_t count = 0;
+  if (pos != std::string::npos) {
+    for (;;) {
+      count++;
+      const auto new_pos = hay.substr(pos + 1).find(needle);
+      if (new_pos == std::string::npos) {
+        break;
+      }
+      pos += 1 + new_pos;
+    }
+  }
+  return count;
+}
+
 }  // namespace my_testing
